pub type AnimationNodeIndex = NodeIndex;Available on crate feature
bevy_animation only.Expand description
The index of either an animation or blend node in the animation graph.
These indices are the way that animation players identify each animation.
Aliased Type
pub struct AnimationNodeIndex(/* private fields */);