pub struct AnimatedBy(pub Entity);Expand description
A component that links an animated entity to an entity containing an
AnimationPlayer. Typically used alongside the AnimationTargetId
component - the linked AnimationPlayer plays AnimationClip assets, and
the AnimationTargetId identifies which curves in the AnimationClip will
affect the target entity.
By convention, asset loaders add AnimationTargetId components to the
descendants of an AnimationPlayer, as well as to the AnimationPlayer
entity itself, but Bevy doesn’t require this in any way. So, for example,
it’s entirely possible for an AnimationPlayer to animate a target that
it isn’t an ancestor of. If you add a new bone to or delete a bone from an
armature at runtime, you may want to update the AnimationTargetId
component as appropriate, as Bevy won’t do this automatically.
Note that each entity can only be animated by one animation player at a
time. However, you can change AnimatedBy components at runtime and
link them to a different player.
Tuple Fields
0: EntityTrait Implementations
Sourceimpl Clone for AnimatedBy
impl Clone for AnimatedBy
Sourcefn clone(&self) -> AnimatedBy
fn clone(&self) -> AnimatedBy
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Component for AnimatedBy
impl Component for AnimatedBy
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn map_entities<M>(this: &mut AnimatedBy, mapper: &mut M)where
M: EntityMapper,
fn map_entities<M>(this: &mut AnimatedBy, mapper: &mut M)where
M: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<AnimatedBy>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<AnimatedBy>>
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_replace() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_replace() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourceimpl Debug for AnimatedBy
impl Debug for AnimatedBy
Sourceimpl FromArg for AnimatedBy
impl FromArg for AnimatedBy
Sourceimpl FromReflect for AnimatedBy
impl FromReflect for AnimatedBy
Sourcefn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<AnimatedBy>
fn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<AnimatedBy>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for AnimatedBy
impl GetOwnership for AnimatedBy
Sourceimpl GetTypeRegistration for AnimatedBy
impl GetTypeRegistration for AnimatedBy
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for AnimatedBy
impl IntoReturn for AnimatedBy
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
AnimatedBy: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
AnimatedBy: 'into_return,
Sourceimpl PartialReflect for AnimatedBy
impl PartialReflect for AnimatedBy
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<AnimatedBy>) -> ReflectOwned
fn reflect_owned(self: Box<AnimatedBy>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<AnimatedBy>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<AnimatedBy>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<AnimatedBy>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<AnimatedBy>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails. Read moreSourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for AnimatedBy
impl Reflect for AnimatedBy
Sourcefn into_any(self: Box<AnimatedBy>) -> Box<dyn Any>
fn into_any(self: Box<AnimatedBy>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<AnimatedBy>) -> Box<dyn Reflect>
fn into_reflect(self: Box<AnimatedBy>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl TupleStruct for AnimatedBy
impl TupleStruct for AnimatedBy
Sourcefn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn Reflect.Sourcefn field_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn Reflect.Sourcefn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
fn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
Sourcefn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
fn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
DynamicTupleStruct from this tuple struct.Sourcefn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
fn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
None if TypeInfo is not available.Sourceimpl TypePath for AnimatedBy
impl TypePath for AnimatedBy
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
Sourceimpl Typed for AnimatedBy
impl Typed for AnimatedBy
impl Copy for AnimatedBy