AnimationClip

Struct AnimationClip 

pub struct AnimationClip { /* private fields */ }
A list of VariableCurves and the AnimationTargetIds to which they apply.

Because animation clips refer to targets by UUID, they can target any entity with that ID.

Implementations

Source

impl AnimationClip

pub fn curves( &self, ) -> &HashMap<AnimationTargetId, Vec<VariableCurve>, NoOpHash>

VariableCurves for each animation target. Indexed by the AnimationTargetId.

pub fn curves_mut( &mut self, ) -> &mut HashMap<AnimationTargetId, Vec<VariableCurve>, NoOpHash>

Get mutable references of VariableCurves for each animation target. Indexed by the AnimationTargetId.

pub fn curves_for_target( &self, target_id: AnimationTargetId, ) -> Option<&Vec<VariableCurve>>

Gets the curves for a single animation target.

Returns None if this clip doesn’t animate the target.

pub fn curves_for_target_mut( &mut self, target_id: AnimationTargetId, ) -> Option<&mut Vec<VariableCurve>>

Gets mutable references of the curves for a single animation target.

Returns None if this clip doesn’t animate the target.

pub fn duration(&self) -> f32

Duration of the clip, represented in seconds.

pub fn set_duration(&mut self, duration_sec: f32)

Set the duration of the clip in seconds.

pub fn add_curve_to_target( &mut self, target_id: AnimationTargetId, curve: impl AnimationCurve, )

Adds an AnimationCurve that can target an entity with the given AnimationTargetId component.

If the curve extends beyond the current duration of this clip, this method lengthens this clip to include the entire time span that the curve covers.

More specifically:

  • This clip will be sampled on the interval [0, duration].
  • Each curve in the clip is sampled by first clamping the sample time to its domain.
  • Curves that extend forever never contribute to the duration.

For example, a curve with domain [2, 5] will extend the clip to cover [0, 5] when added and will produce the same output on the entire interval [0, 2] because these time values all get clamped to 2.

By contrast, a curve with domain [-10, ∞] will never extend the clip duration when added and will be sampled only on [0, duration], ignoring all negative time values.

Examples found in repository?
examples/animation/animated_transform.rs (lines 54-69)
22fn setup(
23    mut commands: Commands,
24    mut meshes: ResMut<Assets<Mesh>>,
25    mut materials: ResMut<Assets<StandardMaterial>>,
26    mut animations: ResMut<Assets<AnimationClip>>,
27    mut graphs: ResMut<Assets<AnimationGraph>>,
28) {
29    // Camera
30    commands.spawn((
31        Camera3d::default(),
32        Transform::from_xyz(-2.0, 2.5, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
33    ));
34
35    // Light
36    commands.spawn((
37        PointLight {
38            intensity: 500_000.0,
39            ..default()
40        },
41        Transform::from_xyz(0.0, 2.5, 0.0),
42    ));
43
44    // Let's use the `Name` component to target entities. We can use anything we
45    // like, but names are convenient.
46    let planet = Name::new("planet");
47    let orbit_controller = Name::new("orbit_controller");
48    let satellite = Name::new("satellite");
49
50    // Creating the animation
51    let mut animation = AnimationClip::default();
52    // A curve can modify a single part of a transform: here, the translation.
53    let planet_animation_target_id = AnimationTargetId::from_name(&planet);
54    animation.add_curve_to_target(
55        planet_animation_target_id,
56        AnimatableCurve::new(
57            animated_field!(Transform::translation),
58            UnevenSampleAutoCurve::new([0.0, 1.0, 2.0, 3.0, 4.0].into_iter().zip([
59                Vec3::new(1.0, 0.0, 1.0),
60                Vec3::new(-1.0, 0.0, 1.0),
61                Vec3::new(-1.0, 0.0, -1.0),
62                Vec3::new(1.0, 0.0, -1.0),
63                // in case seamless looping is wanted, the last keyframe should
64                // be the same as the first one
65                Vec3::new(1.0, 0.0, 1.0),
66            ]))
67            .expect("should be able to build translation curve because we pass in valid samples"),
68        ),
69    );
70    // Or it can modify the rotation of the transform.
71    // To find the entity to modify, the hierarchy will be traversed looking for
72    // an entity with the right name at each level.
73    let orbit_controller_animation_target_id =
74        AnimationTargetId::from_names([planet.clone(), orbit_controller.clone()].iter());
75    animation.add_curve_to_target(
76        orbit_controller_animation_target_id,
77        AnimatableCurve::new(
78            animated_field!(Transform::rotation),
79            UnevenSampleAutoCurve::new([0.0, 1.0, 2.0, 3.0, 4.0].into_iter().zip([
80                Quat::IDENTITY,
81                Quat::from_axis_angle(Vec3::Y, PI / 2.),
82                Quat::from_axis_angle(Vec3::Y, PI / 2. * 2.),
83                Quat::from_axis_angle(Vec3::Y, PI / 2. * 3.),
84                Quat::IDENTITY,
85            ]))
86            .expect("Failed to build rotation curve"),
87        ),
88    );
89    // If a curve in an animation is shorter than the other, it will not repeat
90    // until all other curves are finished. In that case, another animation should
91    // be created for each part that would have a different duration / period.
92    let satellite_animation_target_id = AnimationTargetId::from_names(
93        [planet.clone(), orbit_controller.clone(), satellite.clone()].iter(),
94    );
95    animation.add_curve_to_target(
96        satellite_animation_target_id,
97        AnimatableCurve::new(
98            animated_field!(Transform::scale),
99            UnevenSampleAutoCurve::new(
100                [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0]
101                    .into_iter()
102                    .zip([
103                        Vec3::splat(0.8),
104                        Vec3::splat(1.2),
105                        Vec3::splat(0.8),
106                        Vec3::splat(1.2),
107                        Vec3::splat(0.8),
108                        Vec3::splat(1.2),
109                        Vec3::splat(0.8),
110                        Vec3::splat(1.2),
111                        Vec3::splat(0.8),
112                    ]),
113            )
114            .expect("Failed to build scale curve"),
115        ),
116    );
117    // There can be more than one curve targeting the same entity path.
118    animation.add_curve_to_target(
119        AnimationTargetId::from_names(
120            [planet.clone(), orbit_controller.clone(), satellite.clone()].iter(),
121        ),
122        AnimatableCurve::new(
123            animated_field!(Transform::rotation),
124            UnevenSampleAutoCurve::new([0.0, 1.0, 2.0, 3.0, 4.0].into_iter().zip([
125                Quat::IDENTITY,
126                Quat::from_axis_angle(Vec3::Y, PI / 2.),
127                Quat::from_axis_angle(Vec3::Y, PI / 2. * 2.),
128                Quat::from_axis_angle(Vec3::Y, PI / 2. * 3.),
129                Quat::IDENTITY,
130            ]))
131            .expect("should be able to build translation curve because we pass in valid samples"),
132        ),
133    );
134
135    // Create the animation graph
136    let (graph, animation_index) = AnimationGraph::from_clip(animations.add(animation));
137
138    // Create the animation player, and set it to repeat
139    let mut player = AnimationPlayer::default();
140    player.play(animation_index).repeat();
141
142    // Create the scene that will be animated
143    // First entity is the planet
144    let planet_entity = commands
145        .spawn((
146            Mesh3d(meshes.add(Sphere::default())),
147            MeshMaterial3d(materials.add(Color::srgb(0.8, 0.7, 0.6))),
148            // Add the animation graph and player
149            planet,
150            AnimationGraphHandle(graphs.add(graph)),
151            player,
152        ))
153        .id();
154    commands.entity(planet_entity).insert((
155        planet_animation_target_id,
156        AnimatedBy(planet_entity),
157        children![(
158            Transform::default(),
159            Visibility::default(),
160            orbit_controller,
161            orbit_controller_animation_target_id,
162            AnimatedBy(planet_entity),
163            children![(
164                Mesh3d(meshes.add(Cuboid::new(0.5, 0.5, 0.5))),
165                MeshMaterial3d(materials.add(Color::srgb(0.3, 0.9, 0.3))),
166                Transform::from_xyz(1.5, 0.0, 0.0),
167                satellite_animation_target_id,
168                AnimatedBy(planet_entity),
169                satellite,
170            )],
171        )],
172    ));
173}
pub fn add_variable_curve_to_target( &mut self, target_id: AnimationTargetId, variable_curve: VariableCurve, )

Like add_curve_to_target, but adding a VariableCurve directly.

Under normal circumstances, that method is generally more convenient.

pub fn add_event(&mut self, time: f32, event: impl AnimationEvent)

Add an EntityEvent with no AnimationTargetId.

The event will be cloned and triggered on the AnimationPlayer entity once the time (in seconds) is reached in the animation.

See also add_event_to_target.

pub fn add_event_to_target( &mut self, target_id: AnimationTargetId, time: f32, event: impl AnimationEvent, )

Add an EntityEvent with an AnimationTargetId.

The event will be cloned and triggered on the entity matching the target once the time (in seconds) is reached in the animation.

Use add_event instead if you don’t have a specific target.

pub fn add_event_fn( &mut self, time: f32, func: impl Fn(&mut Commands<'_, '_>, Entity, f32, f32) + Send + Sync + 'static, )

Add an event function with no AnimationTargetId to this AnimationClip.

The func will trigger on the AnimationPlayer entity once the time (in seconds) is reached in the animation.

For a simpler EntityEvent-based alternative, see AnimationClip::add_event. See also add_event_to_target.

clip.add_event_fn(1.0, |commands, entity, time, weight| {
  println!("Animation event triggered {entity:#?} at time {time} with weight {weight}");
})
pub fn add_event_fn_to_target( &mut self, target_id: AnimationTargetId, time: f32, func: impl Fn(&mut Commands<'_, '_>, Entity, f32, f32) + Send + Sync + 'static, )

Add an event function with an AnimationTargetId.

The func will trigger on the entity matching the target once the time (in seconds) is reached in the animation.

For a simpler EntityEvent-based alternative, see AnimationClip::add_event_to_target. Use add_event instead if you don’t have a specific target.

clip.add_event_fn_to_target(AnimationTargetId::from_iter(["Arm", "Hand"]), 1.0, |commands, entity, time, weight| {
  println!("Animation event triggered {entity:#?} at time {time} with weight {weight}");
})