pub enum CoreStage {
First,
PreUpdate,
Update,
PostUpdate,
Last,
}Expand description
The names of the default App stages.
The relative Stages are added by App::add_default_stages.
Variants
First
The Stage that runs before all other app stages.
PreUpdate
The Stage that runs before CoreStage::Update.
Update
The Stage responsible for doing most app logic. Systems should be registered here by default.
PostUpdate
The Stage that runs after CoreStage::Update.
Last
The Stage that runs after all other app stages.