pub enum Handle<A>where
A: Asset,{
Strong(Arc<StrongHandle>),
Weak(AssetId<A>),
}Expand description
A handle to a specific Asset of type A. Handles act as abstract “references” to
assets, whose data are stored in the Assets<A> resource,
avoiding the need to store multiple copies of the same data.
If a Handle is Handle::Strong, the Asset will be kept
alive until the Handle is dropped. If a Handle is Handle::Weak, it does not necessarily reference a live Asset,
nor will it keep assets alive.
Modifying a handle will change which existing asset is referenced, but modifying the asset
(by mutating the Assets resource) will change the asset for all handles referencing it.
Handle can be cloned. If a Handle::Strong is cloned, the referenced Asset will not be freed until all instances
of the Handle are dropped.
Handle::Strong, via StrongHandle also provides access to useful Asset metadata, such as the AssetPath (if it exists).
Variants
Strong(Arc<StrongHandle>)
A “strong” reference to a live (or loading) Asset. If a Handle is Handle::Strong, the Asset will be kept
alive until the Handle is dropped. Strong handles also provide access to additional asset metadata.
Weak(AssetId<A>)
A “weak” reference to an Asset. If a Handle is Handle::Weak, it does not necessarily reference a live Asset,
nor will it keep assets alive.
Implementations
Sourceimpl<A> Handle<A>where
A: Asset,
impl<A> Handle<A>where
A: Asset,
Sourcepub const fn weak_from_u128(value: u128) -> Handle<A>
👎Deprecated since 0.16.0: use the weak_handle! macro with a UUID string instead
pub const fn weak_from_u128(value: u128) -> Handle<A>
weak_handle! macro with a UUID string insteadCreate a new Handle::Weak with the given u128 encoding of a Uuid.
Sourcepub fn id(&self) -> AssetId<A>
pub fn id(&self) -> AssetId<A>
Examples found in repository?
167fn print_counts(
168 time: Res<Time>,
169 mut timer: Local<PrintingTimer>,
170 texts: Query<&ViewVisibility, With<Text2d>>,
171 atlases: Res<FontAtlasSets>,
172 font: Res<FontHandle>,
173) {
174 timer.tick(time.delta());
175 if !timer.just_finished() {
176 return;
177 }
178
179 let num_atlases = atlases
180 .get(font.0.id())
181 .map(|set| set.iter().map(|atlas| atlas.1.len()).sum())
182 .unwrap_or(0);
183
184 let visible_texts = texts.iter().filter(|visibility| visibility.get()).count();
185
186 info!(
187 "Texts: {} Visible: {} Atlases: {}",
188 texts.iter().count(),
189 visible_texts,
190 num_atlases
191 );
192}More examples
222fn resize_image(
223 image_mesh: Query<(&MeshMaterial3d<StandardMaterial>, &Mesh3d), With<HDRViewer>>,
224 materials: Res<Assets<StandardMaterial>>,
225 mut meshes: ResMut<Assets<Mesh>>,
226 images: Res<Assets<Image>>,
227 mut image_events: EventReader<AssetEvent<Image>>,
228) {
229 for event in image_events.read() {
230 let (AssetEvent::Added { id } | AssetEvent::Modified { id }) = event else {
231 continue;
232 };
233
234 for (mat_h, mesh_h) in &image_mesh {
235 let Some(mat) = materials.get(mat_h) else {
236 continue;
237 };
238
239 let Some(ref base_color_texture) = mat.base_color_texture else {
240 continue;
241 };
242
243 if *id != base_color_texture.id() {
244 continue;
245 };
246
247 let Some(image_changed) = images.get(*id) else {
248 continue;
249 };
250
251 let size = image_changed.size_f32().normalize_or_zero() * 1.4;
252 // Resize Mesh
253 let quad = Mesh::from(Rectangle::from_size(size));
254 meshes.insert(mesh_h, quad);
255 }
256 }
257}50fn create_array_texture(
51 mut commands: Commands,
52 asset_server: Res<AssetServer>,
53 mut loading_texture: ResMut<LoadingTexture>,
54 mut images: ResMut<Assets<Image>>,
55 mut meshes: ResMut<Assets<Mesh>>,
56 mut materials: ResMut<Assets<ArrayTextureMaterial>>,
57) {
58 if loading_texture.is_loaded
59 || !asset_server
60 .load_state(loading_texture.handle.id())
61 .is_loaded()
62 {
63 return;
64 }
65 loading_texture.is_loaded = true;
66 let image = images.get_mut(&loading_texture.handle).unwrap();
67
68 // Create a new array texture asset from the loaded texture.
69 let array_layers = 4;
70 image.reinterpret_stacked_2d_as_array(array_layers);
71
72 // Spawn some cubes using the array texture
73 let mesh_handle = meshes.add(Cuboid::default());
74 let material_handle = materials.add(ArrayTextureMaterial {
75 array_texture: loading_texture.handle.clone(),
76 });
77 for x in -5..=5 {
78 commands.spawn((
79 Mesh3d(mesh_handle.clone()),
80 MeshMaterial3d(material_handle.clone()),
81 Transform::from_xyz(x as f32 + 0.5, 0.0, 0.0),
82 ));
83 }
84}328pub fn extract_colored_mesh2d(
329 mut commands: Commands,
330 mut previous_len: Local<usize>,
331 // When extracting, you must use `Extract` to mark the `SystemParam`s
332 // which should be taken from the main world.
333 query: Extract<
334 Query<
335 (
336 Entity,
337 RenderEntity,
338 &ViewVisibility,
339 &GlobalTransform,
340 &Mesh2d,
341 ),
342 With<ColoredMesh2d>,
343 >,
344 >,
345 mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
346) {
347 let mut values = Vec::with_capacity(*previous_len);
348 for (entity, render_entity, view_visibility, transform, handle) in &query {
349 if !view_visibility.get() {
350 continue;
351 }
352
353 let transforms = Mesh2dTransforms {
354 world_from_local: (&transform.affine()).into(),
355 flags: MeshFlags::empty().bits(),
356 };
357
358 values.push((render_entity, ColoredMesh2d));
359 render_mesh_instances.insert(
360 entity.into(),
361 RenderMesh2dInstance {
362 mesh_asset_id: handle.0.id(),
363 transforms,
364 material_bind_group_id: Material2dBindGroupId::default(),
365 automatic_batching: false,
366 tag: 0,
367 },
368 );
369 }
370 *previous_len = values.len();
371 commands.try_insert_batch(values);
372}59fn change_material(
60 trigger: Trigger<SceneInstanceReady>,
61 mut commands: Commands,
62 children: Query<&Children>,
63 color_override: Query<&ColorOverride>,
64 mesh_materials: Query<&MeshMaterial3d<StandardMaterial>>,
65 mut asset_materials: ResMut<Assets<StandardMaterial>>,
66) {
67 // Get the `ColorOverride` of the entity, if it does not have a color override, skip
68 let Ok(color_override) = color_override.get(trigger.target()) else {
69 return;
70 };
71
72 // Iterate over all children recursively
73 for descendants in children.iter_descendants(trigger.target()) {
74 // Get the material of the descendant
75 if let Some(material) = mesh_materials
76 .get(descendants)
77 .ok()
78 .and_then(|id| asset_materials.get_mut(id.id()))
79 {
80 // Create a copy of the material and override base color
81 // If you intend on creating multiple models with the same tint, it
82 // is best to cache the handle somewhere, as having multiple materials
83 // that are identical is expensive
84 let mut new_material = material.clone();
85 new_material.base_color = color_override.0;
86
87 // Override `MeshMaterial3d` with new material
88 commands
89 .entity(descendants)
90 .insert(MeshMaterial3d(asset_materials.add(new_material)));
91 }
92 }
93}472fn update(
473 images_to_save: Query<&ImageToSave>,
474 receiver: Res<MainWorldReceiver>,
475 mut images: ResMut<Assets<Image>>,
476 mut scene_controller: ResMut<SceneController>,
477 mut app_exit_writer: EventWriter<AppExit>,
478 mut file_number: Local<u32>,
479) {
480 if let SceneState::Render(n) = scene_controller.state {
481 if n < 1 {
482 // We don't want to block the main world on this,
483 // so we use try_recv which attempts to receive without blocking
484 let mut image_data = Vec::new();
485 while let Ok(data) = receiver.try_recv() {
486 // image generation could be faster than saving to fs,
487 // that's why use only last of them
488 image_data = data;
489 }
490 if !image_data.is_empty() {
491 for image in images_to_save.iter() {
492 // Fill correct data from channel to image
493 let img_bytes = images.get_mut(image.id()).unwrap();
494
495 // We need to ensure that this works regardless of the image dimensions
496 // If the image became wider when copying from the texture to the buffer,
497 // then the data is reduced to its original size when copying from the buffer to the image.
498 let row_bytes = img_bytes.width() as usize
499 * img_bytes.texture_descriptor.format.pixel_size();
500 let aligned_row_bytes = RenderDevice::align_copy_bytes_per_row(row_bytes);
501 if row_bytes == aligned_row_bytes {
502 img_bytes.data.as_mut().unwrap().clone_from(&image_data);
503 } else {
504 // shrink data to original image size
505 img_bytes.data = Some(
506 image_data
507 .chunks(aligned_row_bytes)
508 .take(img_bytes.height() as usize)
509 .flat_map(|row| &row[..row_bytes.min(row.len())])
510 .cloned()
511 .collect(),
512 );
513 }
514
515 // Create RGBA Image Buffer
516 let img = match img_bytes.clone().try_into_dynamic() {
517 Ok(img) => img.to_rgba8(),
518 Err(e) => panic!("Failed to create image buffer {e:?}"),
519 };
520
521 // Prepare directory for images, test_images in bevy folder is used here for example
522 // You should choose the path depending on your needs
523 let images_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_images");
524 info!("Saving image to: {images_dir:?}");
525 std::fs::create_dir_all(&images_dir).unwrap();
526
527 // Choose filename starting from 000.png
528 let image_path = images_dir.join(format!("{:03}.png", file_number.deref()));
529 *file_number.deref_mut() += 1;
530
531 // Finally saving image to file, this heavy blocking operation is kept here
532 // for example simplicity, but in real app you should move it to a separate task
533 if let Err(e) = img.save(image_path) {
534 panic!("Failed to save image: {e}");
535 };
536 }
537 if scene_controller.single_image {
538 app_exit_writer.write(AppExit::Success);
539 }
540 }
541 } else {
542 // clears channel for skipped frames
543 while receiver.try_recv().is_ok() {}
544 scene_controller.state = SceneState::Render(n - 1);
545 }
546 }
547}Sourcepub fn path(&self) -> Option<&AssetPath<'static>>
pub fn path(&self) -> Option<&AssetPath<'static>>
Returns the path if this is (1) a strong handle and (2) the asset has a path
Sourcepub fn clone_weak(&self) -> Handle<A>
pub fn clone_weak(&self) -> Handle<A>
Creates a Handle::Weak clone of this Handle, which will not keep the referenced Asset alive.
Sourcepub fn untyped(self) -> UntypedHandle
pub fn untyped(self) -> UntypedHandle
Converts this Handle to an “untyped” / “generic-less” UntypedHandle, which stores the Asset type information
inside UntypedHandle. This will return UntypedHandle::Strong for Handle::Strong and UntypedHandle::Weak for
Handle::Weak.