pub struct LoadTransformAndSave<L, T, S>where
L: AssetLoader,
T: AssetTransformer<AssetInput = <L as AssetLoader>::Asset>,
S: AssetSaver<Asset = <T as AssetTransformer>::AssetOutput>,{ /* private fields */ }Expand description
A flexible Process implementation that loads the source Asset using the L AssetLoader, then transforms
the L asset into an S AssetSaver asset using the T AssetTransformer, and lastly saves the asset using the S AssetSaver.
When creating custom processors, it is generally recommended to use the LoadTransformAndSave Process implementation,
as it encourages you to separate your code into an AssetLoader capable of loading assets without processing enabled,
an AssetTransformer capable of converting from an L asset to an S asset, and
an AssetSaver that allows you save any S asset. However you can
also implement Process directly if LoadTransformAndSave feels limiting or unnecessary.
If your Process does not need to transform the Asset, you can use IdentityAssetTransformer as T.
This will directly return the input Asset, allowing your Process to directly load and then save an Asset.
However, this pattern should only be used for cases such as file format conversion.
Otherwise, consider refactoring your AssetLoader and AssetSaver to isolate the transformation step into an explicit AssetTransformer.
This uses LoadTransformAndSaveSettings to configure the processor.
Implementations
Sourceimpl<L, T, S> LoadTransformAndSave<L, T, S>where
L: AssetLoader,
T: AssetTransformer<AssetInput = <L as AssetLoader>::Asset>,
S: AssetSaver<Asset = <T as AssetTransformer>::AssetOutput>,
impl<L, T, S> LoadTransformAndSave<L, T, S>where
L: AssetLoader,
T: AssetTransformer<AssetInput = <L as AssetLoader>::Asset>,
S: AssetSaver<Asset = <T as AssetTransformer>::AssetOutput>,
pub fn new(transformer: T, saver: S) -> LoadTransformAndSave<L, T, S>