pub struct AudioPlayer<Source = AudioSource>(pub Handle<Source>)
where
Source: Asset + Decodable;bevy_audio only.Expand description
A component for playing a sound.
Insert this component onto an entity to trigger an audio source to begin playing.
If the handle refers to an unavailable asset (such as if it has not finished loading yet), the audio will not begin playing immediately. The audio will play when the asset is ready.
When Bevy begins the audio playback, an AudioSink component will be
added to the entity. You can use that component to control the audio settings during playback.
Playback can be configured using the PlaybackSettings component. Note that changes to the
PlaybackSettings component will not affect already-playing audio.
Tuple Fields
0: Handle<Source>Implementations
Sourceimpl AudioPlayer
impl AudioPlayer
Sourcepub fn new(source: Handle<AudioSource>) -> AudioPlayer
pub fn new(source: Handle<AudioSource>) -> AudioPlayer
Creates a new AudioPlayer with the given Handle<AudioSource>.
For convenience reasons, this hard-codes the AudioSource type. If you want to
initialize an AudioPlayer with a different type, just initialize it directly using normal
tuple struct syntax.
Examples found in repository?
More examples
45fn keyboard_event(
46 keyboard_input: Res<ButtonInput<KeyCode>>,
47 sound_effect: Res<SoundEffect>,
48 mut commands: Commands,
49) {
50 if keyboard_input.just_pressed(KeyCode::Space) {
51 commands.spawn((
52 AudioPlayer::new(sound_effect.clone()),
53 PlaybackSettings::DESPAWN,
54 ));
55 }
56}16fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
17 commands.spawn((
18 AudioPlayer::new(asset_server.load("sounds/Windless Slopes.ogg")),
19 MyMusic,
20 ));
21
22 commands.spawn((
23 Text::new(""),
24 Node {
25 position_type: PositionType::Absolute,
26 top: px(12),
27 left: px(12),
28 ..default()
29 },
30 ProgressText,
31 ));
32
33 // example instructions
34 commands.spawn((
35 Text::new("-/=: Volume Down/Up\nSpace: Toggle Playback\nM: Toggle Mute"),
36 Node {
37 position_type: PositionType::Absolute,
38 bottom: px(12),
39 left: px(12),
40 ..default()
41 },
42 ));
43
44 // camera
45 commands.spawn(Camera3d::default());
46}26fn setup(
27 mut commands: Commands,
28 mut meshes: ResMut<Assets<Mesh>>,
29 mut materials: ResMut<Assets<ColorMaterial>>,
30 asset_server: Res<AssetServer>,
31) {
32 // Space between the two ears
33 let gap = 400.0;
34
35 // sound emitter
36 commands.spawn((
37 Mesh2d(meshes.add(Circle::new(15.0))),
38 MeshMaterial2d(materials.add(Color::from(BLUE))),
39 Transform::from_translation(Vec3::new(0.0, 50.0, 0.0)),
40 Emitter::default(),
41 AudioPlayer::new(asset_server.load("sounds/Windless Slopes.ogg")),
42 PlaybackSettings::LOOP.with_spatial(true),
43 ));
44
45 let listener = SpatialListener::new(gap);
46 commands.spawn((
47 Transform::default(),
48 Visibility::default(),
49 listener.clone(),
50 children![
51 // left ear
52 (
53 Sprite::from_color(RED, Vec2::splat(20.0)),
54 Transform::from_xyz(-gap / 2.0, 0.0, 0.0),
55 ),
56 // right ear
57 (
58 Sprite::from_color(LIME, Vec2::splat(20.0)),
59 Transform::from_xyz(gap / 2.0, 0.0, 0.0),
60 )
61 ],
62 ));
63
64 // example instructions
65 commands.spawn((
66 Text::new("Up/Down/Left/Right: Move Listener\nSpace: Toggle Emitter Movement"),
67 Node {
68 position_type: PositionType::Absolute,
69 bottom: px(12),
70 left: px(12),
71 ..default()
72 },
73 ));
74
75 // camera
76 commands.spawn(Camera2d);
77}18fn setup(
19 mut commands: Commands,
20 asset_server: Res<AssetServer>,
21 mut meshes: ResMut<Assets<Mesh>>,
22 mut materials: ResMut<Assets<StandardMaterial>>,
23) {
24 // Space between the two ears
25 let gap = 4.0;
26
27 // sound emitter
28 commands.spawn((
29 Mesh3d(meshes.add(Sphere::new(0.2).mesh().uv(32, 18))),
30 MeshMaterial3d(materials.add(Color::from(BLUE))),
31 Transform::from_xyz(0.0, 0.0, 0.0),
32 Emitter::default(),
33 AudioPlayer::new(asset_server.load("sounds/Windless Slopes.ogg")),
34 PlaybackSettings::LOOP.with_spatial(true),
35 ));
36
37 let listener = SpatialListener::new(gap);
38 commands.spawn((
39 Transform::default(),
40 Visibility::default(),
41 listener.clone(),
42 children![
43 // left ear indicator
44 (
45 Mesh3d(meshes.add(Cuboid::new(0.2, 0.2, 0.2))),
46 MeshMaterial3d(materials.add(Color::from(RED))),
47 Transform::from_translation(listener.left_ear_offset),
48 ),
49 // right ear indicator
50 (
51 Mesh3d(meshes.add(Cuboid::new(0.2, 0.2, 0.2))),
52 MeshMaterial3d(materials.add(Color::from(LIME))),
53 Transform::from_translation(listener.right_ear_offset),
54 )
55 ],
56 ));
57
58 // light
59 commands.spawn((
60 DirectionalLight::default(),
61 Transform::from_xyz(4.0, 8.0, 4.0).looking_at(Vec3::ZERO, Vec3::Y),
62 ));
63
64 // example instructions
65 commands.spawn((
66 Text::new(
67 "Up/Down/Left/Right: Move Listener\nSpace: Toggle Emitter Movement\nM: Toggle Mute",
68 ),
69 Node {
70 position_type: PositionType::Absolute,
71 bottom: px(12),
72 left: px(12),
73 ..default()
74 },
75 ));
76
77 // camera
78 commands.spawn((
79 Camera3d::default(),
80 Transform::from_xyz(0.0, 5.0, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
81 ));
82}Trait Implementations
Sourceimpl<Source> Clone for AudioPlayer<Source>
impl<Source> Clone for AudioPlayer<Source>
Sourcefn clone(&self) -> AudioPlayer<Source>
fn clone(&self) -> AudioPlayer<Source>
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<Source> Component for AudioPlayer<Source>
Required Components: PlaybackSettings.
impl<Source> Component for AudioPlayer<Source>
Required Components: PlaybackSettings.
A component’s Required Components are inserted whenever it is inserted. Note that this will also insert the required components of the required components, recursively, in depth-first order.
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<AudioPlayer<Source>>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<AudioPlayer<Source>>>
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceconst HAS_SUMMARY_TICK: bool = false
const HAS_SUMMARY_TICK: bool = false
Sourceimpl<Source> FromArg for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> FromArg for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl<Source> FromReflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> FromReflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<AudioPlayer<Source>>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<AudioPlayer<Source>>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl<Source> FromTemplate for AudioPlayer<Source>
impl<Source> FromTemplate for AudioPlayer<Source>
Sourcetype Template = AudioPlayerTemplate<Source>
type Template = AudioPlayerTemplate<Source>
Template for this type.Sourceimpl<Source> GetOwnership for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> GetOwnership for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl<Source> GetTypeRegistration for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> GetTypeRegistration for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl<Source> IntoReturn for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> IntoReturn for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
AudioPlayer<Source>: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
AudioPlayer<Source>: 'into_return,
Sourceimpl<Source> PartialReflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> PartialReflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<AudioPlayer<Source>>) -> ReflectOwned
fn reflect_owned(self: Box<AudioPlayer<Source>>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<AudioPlayer<Source>>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<AudioPlayer<Source>>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(
self: Box<AudioPlayer<Source>>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<AudioPlayer<Source>>, ) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
fn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl<Source> Reflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> Reflect for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_any(self: Box<AudioPlayer<Source>>) -> Box<dyn Any>
fn into_any(self: Box<AudioPlayer<Source>>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<AudioPlayer<Source>>) -> Box<dyn Reflect>
fn into_reflect(self: Box<AudioPlayer<Source>>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl<Source> TupleStruct for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> TupleStruct for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn Reflect.Sourcefn field_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn Reflect.Sourcefn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
fn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
Sourcefn to_dynamic_tuple_struct(
&self,
) -> Result<DynamicTupleStruct, ReflectCloneError>
fn to_dynamic_tuple_struct( &self, ) -> Result<DynamicTupleStruct, ReflectCloneError>
DynamicTupleStruct from this tuple struct. Read moreSourcefn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
fn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
None if TypeInfo is not available.Sourceimpl<Source> TypePath for AudioPlayer<Source>
impl<Source> TypePath for AudioPlayer<Source>
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
Sourceimpl<Source> Typed for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> Typed for AudioPlayer<Source>where
AudioPlayer<Source>: Any + Send + Sync,
Source: Asset + Decodable + TypePath,
Handle<Source>: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl<Source> Unpin for AudioPlayer<Source>