pub enum ViewportConversionError {
NoViewportSize,
PastNearPlane,
PastFarPlane,
InvalidData,
}Expand description
Error returned when a conversion between world-space and viewport-space coordinates fails.
See world_to_viewport and viewport_to_world.
Variants
NoViewportSize
The pre-computed size of the viewport was not available.
This may be because the Camera was just created and camera_system has not been executed
yet, or because the RenderTarget is misconfigured in one of the following ways:
- it references the
PrimaryWindowwhen there is none, - it references a
Windowentity that doesn’t exist or doesn’t actually have aWindowcomponent, - it references an
Imagethat doesn’t exist (invalid handle), - it references a
TextureViewthat doesn’t exist (invalid handle).
PastNearPlane
The computed coordinate was beyond the Camera’s near plane.
Only applicable when converting from world-space to viewport-space.
PastFarPlane
The computed coordinate was beyond the Camera’s far plane.
Only applicable when converting from world-space to viewport-space.
InvalidData
The Normalized Device Coordinates could not be computed because the camera_transform, the
world_position, or the projection matrix defined by Projection
contained NAN (see world_to_ndc and ndc_to_world).