pub struct Camera {
pub viewport: Option<Viewport>,
pub order: isize,
pub is_active: bool,
pub computed: ComputedCameraValues,
pub output_mode: CameraOutputMode,
pub msaa_writeback: MsaaWriteback,
pub clear_color: ClearColorConfig,
pub invert_culling: bool,
pub sub_camera_view: Option<SubCameraView>,
}Expand description
The defining Component for camera entities,
storing information about how and what to render through this camera.
The Camera component is added to an entity to define the properties of the viewpoint from
which rendering occurs. It defines the position of the view to render, the projection method
to transform the 3D objects into a 2D image, as well as the render target into which that image
is produced.
Note that a Camera needs a CameraRenderGraph to render anything.
This is typically provided by adding a Camera2d or Camera3d component,
but custom render graphs can also be defined. Inserting a Camera with no render
graph will emit an error at runtime.
Fields
viewport: Option<Viewport>If set, this camera will render to the given Viewport rectangle within the configured RenderTarget.
order: isizeCameras with a higher order are rendered later, and thus on top of lower order cameras.
is_active: boolIf this is set to true, this camera will be rendered to its specified RenderTarget. If false, this
camera will not be rendered.
computed: ComputedCameraValuesComputed values for this camera, such as the projection matrix and the render target size.
output_mode: CameraOutputModeThe CameraOutputMode for this camera.
msaa_writeback: MsaaWritebackControls when MSAA writeback occurs for this camera.
See MsaaWriteback for available options.
clear_color: ClearColorConfigThe clear color operation to perform on the render target.
invert_culling: boolWhether to switch culling mode so that materials that request backface culling cull front faces, and vice versa.
This is typically used for cameras that mirror the world that they render across a plane, because doing that flips the winding of each polygon.
This setting doesn’t affect materials that disable backface culling.
sub_camera_view: Option<SubCameraView>If set, this camera will be a sub camera of a large view, defined by a SubCameraView.
Implementations
Sourceimpl Camera
impl Camera
Sourcepub fn to_logical(&self, physical_size: UVec2) -> Option<Vec2>
pub fn to_logical(&self, physical_size: UVec2) -> Option<Vec2>
Converts a physical size in this Camera to a logical size.
Sourcepub fn physical_viewport_rect(&self) -> Option<URect>
pub fn physical_viewport_rect(&self) -> Option<URect>
The rendered physical bounds URect of the camera. If the viewport field is
set to Some, this will be the rect of that custom viewport. Otherwise it will default to
the full physical rect of the current RenderTarget.
Sourcepub fn logical_viewport_rect(&self) -> Option<Rect>
pub fn logical_viewport_rect(&self) -> Option<Rect>
The rendered logical bounds Rect of the camera. If the viewport field is set to
Some, this will be the rect of that custom viewport. Otherwise it will default to the
full logical rect of the current RenderTarget.
Sourcepub fn logical_viewport_size(&self) -> Option<Vec2>
pub fn logical_viewport_size(&self) -> Option<Vec2>
The logical size of this camera’s viewport. If the viewport field is set to Some, this
will be the size of that custom viewport. Otherwise it will default to the full logical size
of the current RenderTarget.
For logic that requires the full logical size of the
RenderTarget, prefer Camera::logical_target_size.
Returns None if either:
- the function is called just after the
Camerais created, beforecamera_systemis executed, - the
RenderTargetisn’t correctly set:- it references the
PrimaryWindowwhen there is none, - it references a
Windowentity that doesn’t exist or doesn’t actually have aWindowcomponent, - it references an
Imagethat doesn’t exist (invalid handle), - it references a
TextureViewthat doesn’t exist (invalid handle).
- it references the
Sourcepub fn physical_viewport_size(&self) -> Option<UVec2>
pub fn physical_viewport_size(&self) -> Option<UVec2>
The physical size of this camera’s viewport (in physical pixels).
If the viewport field is set to Some, this
will be the size of that custom viewport. Otherwise it will default to the full physical size of
the current RenderTarget.
For logic that requires the full physical size of the RenderTarget, prefer Camera::physical_target_size.
Sourcepub fn logical_target_size(&self) -> Option<Vec2>
pub fn logical_target_size(&self) -> Option<Vec2>
The full logical size of this camera’s RenderTarget, ignoring custom viewport configuration.
Note that if the viewport field is Some, this will not represent the size of the rendered area.
For logic that requires the size of the actually rendered area, prefer Camera::logical_viewport_size.
Sourcepub fn physical_target_size(&self) -> Option<UVec2>
pub fn physical_target_size(&self) -> Option<UVec2>
The full physical size of this camera’s RenderTarget (in physical pixels),
ignoring custom viewport configuration.
Note that if the viewport field is Some, this will not represent the size of the rendered area.
For logic that requires the size of the actually rendered area, prefer Camera::physical_viewport_size.
pub fn target_scaling_factor(&self) -> Option<f32>
Sourcepub fn clip_from_view(&self) -> Mat4
pub fn clip_from_view(&self) -> Mat4
The projection matrix computed using this camera’s Projection.
Sourcepub fn world_to_viewport(
&self,
camera_transform: &GlobalTransform,
world_position: Vec3,
) -> Result<Vec2, ViewportConversionError>
pub fn world_to_viewport( &self, camera_transform: &GlobalTransform, world_position: Vec3, ) -> Result<Vec2, ViewportConversionError>
Given a position in world space, use the camera to compute the viewport-space coordinates.
To get the coordinates in Normalized Device Coordinates, you should use
world_to_ndc.
Panics
Will panic if glam_assert is enabled and the camera_transform contains NAN
(see world_to_ndc).
Examples found in repository?
22fn draw_cursor(
23 camera_query: Single<(&Camera, &GlobalTransform)>,
24 window: Single<&Window>,
25 mut gizmos: Gizmos,
26) {
27 let (camera, camera_transform) = *camera_query;
28
29 if let Some(cursor_position) = window.cursor_position()
30 // Calculate a world position based on the cursor's position.
31 && let Ok(world_pos) = camera.viewport_to_world_2d(camera_transform, cursor_position)
32 // To test Camera::world_to_viewport, convert result back to viewport space and then back to world space.
33 && let Ok(viewport_check) = camera.world_to_viewport(camera_transform, world_pos.extend(0.0))
34 && let Ok(world_check) = camera.viewport_to_world_2d(camera_transform, viewport_check.xy())
35 {
36 gizmos.circle_2d(world_pos, 10., WHITE);
37 // Should be the same as world_pos
38 gizmos.circle_2d(world_check, 8., RED);
39 }
40}More examples
247fn example_control_system(
248 mut materials: ResMut<Assets<StandardMaterial>>,
249 controllable: Query<(&MeshMaterial3d<StandardMaterial>, &ExampleControls)>,
250 camera: Single<
251 (
252 Entity,
253 &mut Camera,
254 &mut Transform,
255 &GlobalTransform,
256 Has<Hdr>,
257 ),
258 With<Camera3d>,
259 >,
260 mut labels: Query<(&mut Node, &ExampleLabel)>,
261 mut display: Single<&mut Text, With<ExampleDisplay>>,
262 labeled: Query<&GlobalTransform>,
263 mut state: Local<ExampleState>,
264 time: Res<Time>,
265 input: Res<ButtonInput<KeyCode>>,
266 mut commands: Commands,
267) {
268 if input.pressed(KeyCode::ArrowUp) {
269 state.alpha = (state.alpha + time.delta_secs()).min(1.0);
270 } else if input.pressed(KeyCode::ArrowDown) {
271 state.alpha = (state.alpha - time.delta_secs()).max(0.0);
272 }
273
274 if input.just_pressed(KeyCode::Space) {
275 state.unlit = !state.unlit;
276 }
277
278 let randomize_colors = input.just_pressed(KeyCode::KeyC);
279
280 for (material_handle, controls) in &controllable {
281 let mut material = materials.get_mut(material_handle).unwrap();
282
283 if controls.color && randomize_colors {
284 material.base_color = Srgba {
285 red: random(),
286 green: random(),
287 blue: random(),
288 alpha: state.alpha,
289 }
290 .into();
291 } else {
292 material.base_color.set_alpha(state.alpha);
293 }
294
295 if controls.unlit {
296 material.unlit = state.unlit;
297 }
298 }
299
300 let (entity, camera, mut camera_transform, camera_global_transform, hdr) = camera.into_inner();
301
302 if input.just_pressed(KeyCode::KeyH) {
303 if hdr {
304 commands.entity(entity).remove::<Hdr>();
305 } else {
306 commands.entity(entity).insert(Hdr);
307 }
308 }
309
310 let rotation = if input.pressed(KeyCode::ArrowLeft) {
311 time.delta_secs()
312 } else if input.pressed(KeyCode::ArrowRight) {
313 -time.delta_secs()
314 } else {
315 0.0
316 };
317
318 camera_transform.rotate_around(Vec3::ZERO, Quat::from_rotation_y(rotation));
319
320 for (mut node, label) in &mut labels {
321 let world_position = labeled.get(label.entity).unwrap().translation() + Vec3::Y;
322
323 let viewport_position = camera
324 .world_to_viewport(camera_global_transform, world_position)
325 .unwrap();
326
327 node.top = px(viewport_position.y);
328 node.left = px(viewport_position.x);
329 }
330
331 display.0 = format!(
332 " HDR: {}\nAlpha: {:.2}",
333 if hdr { "ON " } else { "OFF" },
334 state.alpha
335 );
336}Sourcepub fn world_to_viewport_with_depth(
&self,
camera_transform: &GlobalTransform,
world_position: Vec3,
) -> Result<Vec3, ViewportConversionError>
pub fn world_to_viewport_with_depth( &self, camera_transform: &GlobalTransform, world_position: Vec3, ) -> Result<Vec3, ViewportConversionError>
Given a position in world space, use the camera to compute the viewport-space coordinates and depth.
To get the coordinates in Normalized Device Coordinates, you should use
world_to_ndc.
Panics
Will panic if glam_assert is enabled and the camera_transform contains NAN
(see world_to_ndc).
Sourcepub fn viewport_to_world(
&self,
camera_transform: &GlobalTransform,
viewport_position: Vec2,
) -> Result<Ray3d, ViewportConversionError>
pub fn viewport_to_world( &self, camera_transform: &GlobalTransform, viewport_position: Vec2, ) -> Result<Ray3d, ViewportConversionError>
Returns a ray originating from the camera, that passes through everything beyond viewport_position.
The resulting ray starts on the near plane of the camera.
If the camera’s projection is orthographic the direction of the ray is always equal to camera_transform.forward().
To get the world space coordinates with Normalized Device Coordinates, you should use
ndc_to_world.
Example
fn system(camera_query: Single<(&Camera, &GlobalTransform)>, window: Single<&Window>) {
let (camera, camera_transform) = *camera_query;
if let Some(cursor_position) = window.cursor_position()
// Calculate a ray pointing from the camera into the world based on the cursor's position.
&& let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
{
println!("{ray:?}");
}
}
// Run the system after transform propagation so the camera's global transform is up-to-date.
app.add_systems(PostUpdate, system.after(TransformSystems::Propagate));Panics
Will panic if the camera’s projection matrix is invalid (has a determinant of 0) and
glam_assert is enabled (see ndc_to_world.
Examples found in repository?
190fn handle_click(
191 mouse_button_input: Res<ButtonInput<MouseButton>>,
192 camera: Single<(&Camera, &GlobalTransform)>,
193 windows: Query<&Window>,
194 mut commands: Commands,
195) {
196 let Ok(windows) = windows.single() else {
197 return;
198 };
199
200 let (camera, camera_transform) = *camera;
201 if let Some(pos) = windows
202 .cursor_position()
203 .and_then(|cursor| camera.viewport_to_world(camera_transform, cursor).ok())
204 .map(|ray| ray.origin.truncate())
205 && mouse_button_input.just_pressed(MouseButton::Left)
206 {
207 commands.trigger(ExplodeMines { pos, radius: 1.0 });
208 }
209}More examples
13fn draw_cursor(
14 camera_query: Single<(&Camera, &GlobalTransform)>,
15 ground: Single<&GlobalTransform, With<Ground>>,
16 window: Single<&Window>,
17 mut gizmos: Gizmos,
18) {
19 let (camera, camera_transform) = *camera_query;
20
21 if let Some(cursor_position) = window.cursor_position()
22 // Calculate a ray pointing from the camera into the world based on the cursor's position.
23 && let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
24 // Calculate if and where the ray is hitting the ground plane.
25 && let Some(point) = ray.plane_intersection_point(ground.translation(), InfinitePlane3d::new(ground.up()))
26 {
27 // Draw a circle just above the ground plane at that position.
28 gizmos.circle(
29 Isometry3d::new(
30 point + ground.up() * 0.01,
31 Quat::from_rotation_arc(Vec3::Z, ground.up().as_vec3()),
32 ),
33 0.2,
34 Color::WHITE,
35 );
36 }
37}452fn handle_mouse_clicks(
453 buttons: Res<ButtonInput<MouseButton>>,
454 windows: Query<&Window, With<PrimaryWindow>>,
455 cameras: Query<(&Camera, &GlobalTransform)>,
456 mut main_objects: Query<&mut Transform, With<MainObject>>,
457) {
458 if !buttons.pressed(MouseButton::Left) {
459 return;
460 }
461 let Some(mouse_position) = windows.iter().next().and_then(Window::cursor_position) else {
462 return;
463 };
464 let Some((camera, camera_transform)) = cameras.iter().next() else {
465 return;
466 };
467
468 // Figure out where the user clicked on the plane.
469 let Ok(ray) = camera.viewport_to_world(camera_transform, mouse_position) else {
470 return;
471 };
472 let Some(plane_intersection) =
473 ray.plane_intersection_point(Vec3::ZERO, InfinitePlane3d::new(Vec3::Y))
474 else {
475 return;
476 };
477 // Move all the main objects.
478 for mut transform in main_objects.iter_mut() {
479 transform.translation = vec3(
480 plane_intersection.x,
481 transform.translation.y,
482 plane_intersection.z,
483 );
484 }
485}443fn move_fox_on_mouse_down(
444 mut scene_roots_query: Query<&mut Transform, With<WorldAssetRoot>>,
445 windows_query: Query<&Window, With<PrimaryWindow>>,
446 cameras_query: Query<(&Camera, &GlobalTransform)>,
447 interactions_query: Query<&Interaction, With<RadioButton>>,
448 buttons: Res<ButtonInput<MouseButton>>,
449 app_status: Res<AppStatus>,
450) {
451 // Only process the mouse motion if the left mouse button is pressed, the
452 // mouse action is set to move the fox, and the pointer isn't over a UI
453 // widget.
454 if app_status.drag_action != DragAction::MoveFox
455 || !buttons.pressed(MouseButton::Left)
456 || interactions_query
457 .iter()
458 .any(|interaction| *interaction != Interaction::None)
459 {
460 return;
461 }
462
463 // Find out where the user clicked the mouse.
464 let Some(mouse_position) = windows_query
465 .iter()
466 .next()
467 .and_then(Window::cursor_position)
468 else {
469 return;
470 };
471
472 // Grab the camera.
473 let Some((camera, camera_transform)) = cameras_query.iter().next() else {
474 return;
475 };
476
477 // Figure out where the user clicked on the plane.
478 let Ok(ray) = camera.viewport_to_world(camera_transform, mouse_position) else {
479 return;
480 };
481 let Some(ray_distance) = ray.intersect_plane(Vec3::ZERO, InfinitePlane3d::new(Vec3::Y)) else {
482 return;
483 };
484 let plane_intersection = ray.origin + ray.direction.normalize() * ray_distance;
485
486 // Move the fox.
487 for mut transform in scene_roots_query.iter_mut() {
488 transform.translation = transform.translation.with_xz(plane_intersection.xz());
489 }
490}Sourcepub fn viewport_to_world_2d(
&self,
camera_transform: &GlobalTransform,
viewport_position: Vec2,
) -> Result<Vec2, ViewportConversionError>
pub fn viewport_to_world_2d( &self, camera_transform: &GlobalTransform, viewport_position: Vec2, ) -> Result<Vec2, ViewportConversionError>
Returns a 2D world position computed from a position on this Camera’s viewport.
Useful for 2D cameras and other cameras with an orthographic projection pointing along the Z axis.
To get the world space coordinates with Normalized Device Coordinates, you should use
ndc_to_world.
Example
fn system(camera_query: Single<(&Camera, &GlobalTransform)>, window: Single<&Window>) {
let (camera, camera_transform) = *camera_query;
if let Some(cursor_position) = window.cursor_position()
// Calculate a world position based on the cursor's position.
&& let Ok(world_pos) = camera.viewport_to_world_2d(camera_transform, cursor_position)
{
println!("World position: {world_pos:.2}");
}
}
// Run the system after transform propagation so the camera's global transform is up-to-date.
app.add_systems(PostUpdate, system.after(TransformSystems::Propagate));Panics
Will panic if the camera’s projection matrix is invalid (has a determinant of 0) and
glam_assert is enabled (see ndc_to_world.
Examples found in repository?
205fn get_cursor_world_pos(
206 mut cursor_world_pos: ResMut<CursorWorldPos>,
207 primary_window: Single<&Window, With<PrimaryWindow>>,
208 q_camera: Single<(&Camera, &GlobalTransform)>,
209) {
210 let (main_camera, main_camera_transform) = *q_camera;
211 // Get the cursor position in the world
212 cursor_world_pos.0 = primary_window.cursor_position().and_then(|cursor_pos| {
213 main_camera
214 .viewport_to_world_2d(main_camera_transform, cursor_pos)
215 .ok()
216 });
217}More examples
265fn spawn_box(
266 event: On<Pointer<Press>>,
267 window: Query<(), With<Window>>,
268 camera: Single<(&Camera, &GlobalTransform)>,
269 mut commands: Commands,
270) {
271 if event.button != PointerButton::Primary {
272 return;
273 }
274 if !window.contains(event.entity) {
275 return;
276 }
277
278 let (camera, camera_transform) = camera.into_inner();
279 let Ok(click_point) =
280 camera.viewport_to_world_2d(camera_transform, event.pointer_location.position)
281 else {
282 return;
283 };
284 commands.spawn((
285 Sprite::from_color(tailwind::RED_500, Vec2::new(100.0, 100.0)),
286 Transform::from_translation(click_point.extend(0.0)),
287 Pickable::default(),
288 Box,
289 ));
290}22fn draw_cursor(
23 camera_query: Single<(&Camera, &GlobalTransform)>,
24 window: Single<&Window>,
25 mut gizmos: Gizmos,
26) {
27 let (camera, camera_transform) = *camera_query;
28
29 if let Some(cursor_position) = window.cursor_position()
30 // Calculate a world position based on the cursor's position.
31 && let Ok(world_pos) = camera.viewport_to_world_2d(camera_transform, cursor_position)
32 // To test Camera::world_to_viewport, convert result back to viewport space and then back to world space.
33 && let Ok(viewport_check) = camera.world_to_viewport(camera_transform, world_pos.extend(0.0))
34 && let Ok(world_check) = camera.viewport_to_world_2d(camera_transform, viewport_check.xy())
35 {
36 gizmos.circle_2d(world_pos, 10., WHITE);
37 // Should be the same as world_pos
38 gizmos.circle_2d(world_check, 8., RED);
39 }
40}38fn player_movement_system(
39 mut player: Single<&mut Transform, With<Player>>,
40 camera_query: Single<(&Camera, &GlobalTransform)>,
41 window: Single<&Window>,
42) {
43 let (camera, camera_transform) = *camera_query;
44
45 if let Some(cursor_position) = window.cursor_position()
46 // Calculate a world position based on the cursor's position.
47 && let Ok(cursor_world_pos) = camera.viewport_to_world_2d(camera_transform, cursor_position)
48 {
49 // The angle an entity needs to rotate to face a point is defined
50 // by the vector between the two points (Vec2 - Vec2), which we can then
51 // turn into radians using to_angle.
52 //
53 // FRAC_PI_2 is because our sprite's ship is facing "up" so we rotate it an additional 90 degrees
54 // so that it faces the cursor.
55 player.rotation = Quat::from_rotation_z(
56 (cursor_world_pos - player.translation.xy()).to_angle() - FRAC_PI_2,
57 );
58 }
59}34fn click(
35 click: On<Pointer<Click>>,
36 mut commands: Commands,
37 squares: Query<(Entity, &Transform), With<BlinkySquare>>,
38 cameras: Query<(&Camera, &GlobalTransform)>,
39) {
40 let (camera, camera_transform) = cameras.single().unwrap();
41 let mut delayed = commands.delayed();
42 for (entity, transform) in squares.iter() {
43 // convert the pointer position to world position
44 let mouse_world_pos = camera
45 .viewport_to_world_2d(camera_transform, click.pointer_location.position)
46 .unwrap();
47
48 // delay the blinkiness by distance to cursor
49 let dist = mouse_world_pos.distance(transform.translation.truncate());
50 let delay = dist / 1000.0;
51 delayed
52 .secs(delay)
53 .entity(entity)
54 .insert(Sprite::from_color(Color::WHITE, SQUARE_SIZE));
55 delayed
56 .secs(delay + 0.1)
57 .entity(entity)
58 .insert(Sprite::from_color(Color::BLACK, SQUARE_SIZE));
59 }
60}208fn try_plot(
209 event: On<TryPlot>,
210 sprite: Query<(&Sprite, &Anchor, &GlobalTransform), With<SpriteToSave>>,
211 camera: Single<(&Camera, &GlobalTransform)>,
212 texture_atlases: Res<Assets<TextureAtlasLayout>>,
213 draw_color: Res<DrawColor>,
214 mut images: ResMut<Assets<Image>>,
215) {
216 let Ok((sprite, anchor, sprite_transform)) = sprite.get(event.entity) else {
217 return;
218 };
219 let (camera, camera_transform) = camera.into_inner();
220 let Ok(world_position) = camera.viewport_to_world_2d(camera_transform, event.location.position)
221 else {
222 return;
223 };
224 let relative_to_sprite = sprite_transform
225 .affine()
226 .inverse()
227 .transform_point3(world_position.extend(0.0));
228 let Ok(pixel_space) = sprite.compute_pixel_space_point(
229 relative_to_sprite.xy(),
230 *anchor,
231 &images,
232 &texture_atlases,
233 ) else {
234 return;
235 };
236 let pixel_coordinates = pixel_space.floor().as_uvec2();
237 let mut image = images.get_mut(&sprite.image).unwrap();
238 // For an actual drawing app, you'd at least draw a line from the last point, but this is
239 // simpler.
240 image
241 .set_color_at(pixel_coordinates.x, pixel_coordinates.y, draw_color.0)
242 .unwrap();
243}Sourcepub fn world_to_ndc<V>(
&self,
camera_transform: &GlobalTransform,
world_point: V,
) -> Option<V>
pub fn world_to_ndc<V>( &self, camera_transform: &GlobalTransform, world_point: V, ) -> Option<V>
Given a point in world space, use the camera’s viewport to compute the Normalized Device Coordinates of the point.
When the point is within the viewport the values returned will be between -1.0 (bottom left) and 1.0 (top right)
on the X and Y axes, and between 0.0 (far) and 1.0 (near) on the Z axis.
To get the coordinates in the render target’s viewport dimensions, you should use
world_to_viewport.
Returns None if the camera_transform, the world_position, or the projection matrix defined by
Projection contain NAN.
Panics
Will panic if the camera_transform contains NAN and the glam_assert feature is enabled.
Sourcepub fn ndc_to_world<V>(
&self,
camera_transform: &GlobalTransform,
ndc_point: V,
) -> Option<V>
pub fn ndc_to_world<V>( &self, camera_transform: &GlobalTransform, ndc_point: V, ) -> Option<V>
Given a position in Normalized Device Coordinates, use the camera’s viewport to compute the world space position.
The input is expected to be in NDC: x and y in the range [-1.0, 1.0], and z in [0.0, 1.0]
(with z = 0.0 at the far plane and z = 1.0 at the near plane).
The returned value is a position in world space (your game’s world units) and is not limited to [-1.0, 1.0].
To convert from a viewport position to world space, you should use
viewport_to_world.
Returns None if the camera_transform, the ndc_point, or the projection matrix defined by
Projection contain NAN.
Panics
Will panic if the projection matrix is invalid (has a determinant of 0) and glam_assert is enabled.
Sourcepub fn depth_ndc_to_view_z(&self, ndc_depth: f32) -> f32
pub fn depth_ndc_to_view_z(&self, ndc_depth: f32) -> f32
Converts the depth in Normalized Device Coordinates to linear view z for perspective projections.
Note: Depth values in front of the camera will be negative as -z is forward
Sourcepub fn depth_ndc_to_view_z_2d(&self, ndc_depth: f32) -> f32
pub fn depth_ndc_to_view_z_2d(&self, ndc_depth: f32) -> f32
Converts the depth in Normalized Device Coordinates to linear view z for orthographic projections.
Note: Depth values in front of the camera will be negative as -z is forward
Sourcepub fn viewport_to_ndc(
&self,
viewport_position: Vec2,
) -> Result<Vec2, ViewportConversionError>
pub fn viewport_to_ndc( &self, viewport_position: Vec2, ) -> Result<Vec2, ViewportConversionError>
Converts a position in viewport coordinates to NDC.