pub struct CameraMainTextureUsages(pub TextureUsages);Expand description
This component lets you control the TextureUsages field of the main texture generated for the camera
Tuple Fields
0: TextureUsagesImplementations
Sourceimpl CameraMainTextureUsages
impl CameraMainTextureUsages
Sourcepub fn with(self, usages: TextureUsages) -> CameraMainTextureUsages
pub fn with(self, usages: TextureUsages) -> CameraMainTextureUsages
Examples found in repository?
examples/3d/solari.rs (line 149)
77fn setup_pica_pica(
78 mut commands: Commands,
79 asset_server: Res<AssetServer>,
80 args: Res<Args>,
81 #[cfg(all(feature = "dlss", not(feature = "force_disable_dlss")))] dlss_rr_supported: Option<
82 Res<DlssRayReconstructionSupported>,
83 >,
84) {
85 commands
86 .spawn((
87 WorldAssetRoot(
88 asset_server.load(
89 GltfAssetLabel::Scene(0)
90 .from_asset("https://github.com/bevyengine/bevy_asset_files/raw/2a5950295a8b6d9d051d59c0df69e87abcda58c3/pica_pica/mini_diorama_01.glb")
91 ),
92 ),
93 Transform::from_scale(Vec3::splat(10.0)),
94 ))
95 .observe(add_raytracing_meshes_on_scene_load);
96
97 commands
98 .spawn((
99 WorldAssetRoot(asset_server.load(
100 GltfAssetLabel::Scene(0).from_asset("https://github.com/bevyengine/bevy_asset_files/raw/2a5950295a8b6d9d051d59c0df69e87abcda58c3/pica_pica/robot_01.glb")
101 )),
102 Transform::from_scale(Vec3::splat(2.0))
103 .with_translation(Vec3::new(-2.0, 0.05, -2.1))
104 .with_rotation(Quat::from_rotation_y(PI / 2.0)),
105 PatrolPath {
106 path: vec![
107 (Vec3::new(-2.0, 0.05, -2.1), Quat::from_rotation_y(PI / 2.0)),
108 (Vec3::new(2.2, 0.05, -2.1), Quat::from_rotation_y(0.0)),
109 (
110 Vec3::new(2.2, 0.05, 2.1),
111 Quat::from_rotation_y(3.0 * PI / 2.0),
112 ),
113 (Vec3::new(-2.0, 0.05, 2.1), Quat::from_rotation_y(PI)),
114 ],
115 i: 0,
116 },
117 ))
118 .observe(add_raytracing_meshes_on_scene_load);
119
120 commands.spawn((
121 DirectionalLight {
122 illuminance: light_consts::lux::FULL_DAYLIGHT,
123 shadow_maps_enabled: false, // Solari replaces shadow mapping
124 ..default()
125 },
126 Transform::from_rotation(Quat::from_xyzw(
127 -0.13334629,
128 -0.86597735,
129 -0.3586996,
130 0.3219264,
131 )),
132 ));
133
134 let mut camera = commands.spawn((
135 Camera3d::default(),
136 Camera {
137 clear_color: ClearColorConfig::Custom(Color::BLACK),
138 ..default()
139 },
140 FreeCamera {
141 walk_speed: 3.0,
142 run_speed: 10.0,
143 ..Default::default()
144 },
145 Transform::from_translation(Vec3::new(0.219417, 2.5764852, 6.9718704)).with_rotation(
146 Quat::from_xyzw(-0.1466768, 0.013738206, 0.002037309, 0.989087),
147 ),
148 // Msaa::Off and CameraMainTextureUsages with STORAGE_BINDING are required for Solari
149 CameraMainTextureUsages::default().with(TextureUsages::STORAGE_BINDING),
150 Msaa::Off,
151 ));
152
153 if args.pathtracer == Some(true) {
154 camera.insert(Pathtracer::default());
155 } else {
156 camera.insert(SolariLighting::default());
157 }
158
159 // Using DLSS Ray Reconstruction for denoising (and cheaper rendering via upscaling) is _highly_ recommended when using Solari
160 #[cfg(all(feature = "dlss", not(feature = "force_disable_dlss")))]
161 if dlss_rr_supported.is_some() {
162 camera.insert(Dlss::<DlssRayReconstructionFeature> {
163 perf_quality_mode: Default::default(),
164 reset: Default::default(),
165 _phantom_data: Default::default(),
166 });
167 }
168
169 commands.spawn((
170 ControlText,
171 Text::default(),
172 Node {
173 position_type: PositionType::Absolute,
174 bottom: px(12.0),
175 left: px(12.0),
176 ..default()
177 },
178 ));
179
180 commands.spawn((
181 Node {
182 position_type: PositionType::Absolute,
183 right: px(0.0),
184 padding: px(4.0).all(),
185 border_radius: BorderRadius::bottom_left(px(4.0)),
186 ..default()
187 },
188 BackgroundColor(Color::srgba(0.10, 0.10, 0.10, 0.8)),
189 children![(
190 PerformanceText,
191 Text::default(),
192 TextFont {
193 font_size: FontSize::Px(8.0),
194 ..default()
195 },
196 )],
197 ));
198}
199
200fn setup_many_lights(
201 mut commands: Commands,
202 asset_server: Res<AssetServer>,
203 mut meshes: ResMut<Assets<Mesh>>,
204 mut materials: ResMut<Assets<StandardMaterial>>,
205 args: Res<Args>,
206 #[cfg(all(feature = "dlss", not(feature = "force_disable_dlss")))] dlss_rr_supported: Option<
207 Res<DlssRayReconstructionSupported>,
208 >,
209) {
210 let mut rng = ChaCha8Rng::seed_from_u64(42);
211
212 let mut plane_mesh = Plane3d::default()
213 .mesh()
214 .size(400.0, 400.0)
215 .build()
216 .with_generated_tangents()
217 .unwrap();
218 match plane_mesh.attribute_mut(Mesh::ATTRIBUTE_UV_0).unwrap() {
219 VertexAttributeValues::Float32x2(items) => {
220 items.iter_mut().flatten().for_each(|x| *x *= 3.0);
221 }
222 _ => unreachable!(),
223 }
224 let plane_mesh = meshes.add(plane_mesh);
225 let cube_mesh = meshes.add(
226 Cuboid::default()
227 .mesh()
228 .build()
229 .with_generated_tangents()
230 .unwrap(),
231 );
232 let sphere_mesh = meshes.add(
233 Sphere::new(1.0)
234 .mesh()
235 .build()
236 .with_generated_tangents()
237 .unwrap(),
238 );
239
240 commands
241 .spawn((
242 RaytracingMesh3d(plane_mesh.clone()),
243 MeshMaterial3d(
244 materials.add(StandardMaterial {
245 base_color_texture: Some(
246 asset_server
247 .load_builder()
248 .with_settings::<ImageLoaderSettings>(|settings| {
249 settings
250 .sampler
251 .get_or_init_descriptor()
252 .set_address_mode(ImageAddressMode::Repeat);
253 })
254 .load("textures/uv_checker_bw.png"),
255 ),
256 perceptual_roughness: 0.0,
257 ..default()
258 }),
259 ),
260 ))
261 .insert_if(Mesh3d(plane_mesh), || args.pathtracer != Some(true));
262
263 for _ in 0..8000 {
264 commands
265 .spawn((
266 RaytracingMesh3d(cube_mesh.clone()),
267 MeshMaterial3d(materials.add(StandardMaterial {
268 base_color: Color::srgb(rng.random(), rng.random(), rng.random()),
269 perceptual_roughness: rng.random(),
270 ..default()
271 })),
272 Transform::default()
273 .with_scale(Vec3 {
274 x: rng.random_range(0.2..=2.0),
275 y: rng.random_range(0.2..=2.0),
276 z: rng.random_range(0.2..=2.0),
277 })
278 .with_translation(Vec3::new(
279 rng.random_range(-180.0..=180.0),
280 0.2,
281 rng.random_range(-180.0..=180.0),
282 )),
283 ))
284 .insert_if(Mesh3d(cube_mesh.clone()), || args.pathtracer != Some(true));
285 }
286
287 for x in -10..=10 {
288 for y in -10..=10 {
289 commands
290 .spawn((
291 RaytracingMesh3d(sphere_mesh.clone()),
292 MeshMaterial3d(
293 materials.add(StandardMaterial {
294 emissive: Color::linear_rgb(
295 rng.random::<f32>() * 60000.0,
296 rng.random::<f32>() * 60000.0,
297 rng.random::<f32>() * 60000.0,
298 )
299 .into(),
300 ..default()
301 }),
302 ),
303 Transform::default().with_translation(Vec3::new(
304 (x * 20) as f32,
305 7.0,
306 (y * 20) as f32,
307 )),
308 ))
309 .insert_if(Mesh3d(sphere_mesh.clone()), || {
310 args.pathtracer != Some(true)
311 });
312 }
313 }
314
315 let mut camera = commands.spawn((
316 Camera3d::default(),
317 Camera {
318 clear_color: ClearColorConfig::Custom(Color::BLACK),
319 ..default()
320 },
321 FreeCamera {
322 walk_speed: 3.0,
323 run_speed: 10.0,
324 ..Default::default()
325 },
326 Transform::from_translation(Vec3::new(6.11329, 166.74896, 451.8226)).with_rotation(
327 Quat::from_xyzw(-0.183938, 0.009093744, 0.0017017953, 0.9828943),
328 ),
329 // Msaa::Off and CameraMainTextureUsages with STORAGE_BINDING are required for Solari
330 CameraMainTextureUsages::default().with(TextureUsages::STORAGE_BINDING),
331 Msaa::Off,
332 Bloom {
333 intensity: 0.1,
334 ..Bloom::NATURAL
335 },
336 ));
337
338 if args.pathtracer == Some(true) {
339 camera.insert(Pathtracer::default());
340 } else {
341 camera.insert(SolariLighting::default());
342 }
343
344 // Using DLSS Ray Reconstruction for denoising (and cheaper rendering via upscaling) is _highly_ recommended when using Solari
345 #[cfg(all(feature = "dlss", not(feature = "force_disable_dlss")))]
346 if dlss_rr_supported.is_some() {
347 camera.insert(Dlss::<DlssRayReconstructionFeature> {
348 perf_quality_mode: Default::default(),
349 reset: Default::default(),
350 _phantom_data: Default::default(),
351 });
352 }
353
354 commands.spawn((
355 Node {
356 position_type: PositionType::Absolute,
357 right: px(0.0),
358 padding: px(4.0).all(),
359 border_radius: BorderRadius::bottom_left(px(4.0)),
360 ..default()
361 },
362 BackgroundColor(Color::srgba(0.10, 0.10, 0.10, 0.8)),
363 children![(
364 PerformanceText,
365 Text::default(),
366 TextFont {
367 font_size: FontSize::Px(8.0),
368 ..default()
369 },
370 )],
371 ));
372}Trait Implementations
Sourceimpl Clone for CameraMainTextureUsages
impl Clone for CameraMainTextureUsages
Sourcefn clone(&self) -> CameraMainTextureUsages
fn clone(&self) -> CameraMainTextureUsages
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Component for CameraMainTextureUsages
impl Component for CameraMainTextureUsages
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
A constant indicating the storage type used for this component.
Sourcetype Mutability = Mutable
type Mutability = Mutable
A marker type to assist Bevy with determining if this component is
mutable, or immutable. Mutable components will have
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Registers required components. Read more
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Called when registering this component, allowing to override clone function (or disable cloning altogether) for this component. Read more
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<CameraMainTextureUsages>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<CameraMainTextureUsages>>
Returns
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
Maps the entities on this component using the given
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreimpl Copy for CameraMainTextureUsages
Sourceimpl Default for CameraMainTextureUsages
impl Default for CameraMainTextureUsages
Sourcefn default() -> CameraMainTextureUsages
fn default() -> CameraMainTextureUsages
Returns the “default value” for a type. Read more
Sourceimpl ExtractComponent for CameraMainTextureUsages
impl ExtractComponent for CameraMainTextureUsages
Sourcetype QueryData = &'static CameraMainTextureUsages
type QueryData = &'static CameraMainTextureUsages
ECS
ReadOnlyQueryData to fetch the components to extract.Sourcetype QueryFilter = ()
type QueryFilter = ()
Filters the entities with additional constraints.
Sourcetype Out = CameraMainTextureUsages
type Out = CameraMainTextureUsages
The output from extraction, i.e.
ExtractComponent::extract_component. Read moreSourcefn extract_component(
item: <<CameraMainTextureUsages as ExtractComponent>::QueryData as QueryData>::Item<'_, '_>,
) -> Option<<CameraMainTextureUsages as ExtractComponent>::Out>
fn extract_component( item: <<CameraMainTextureUsages as ExtractComponent>::QueryData as QueryData>::Item<'_, '_>, ) -> Option<<CameraMainTextureUsages as ExtractComponent>::Out>
Defines how the component is transferred into the “render world”. Read more
Sourceimpl FromArg for CameraMainTextureUsages
impl FromArg for CameraMainTextureUsages
Sourceimpl FromReflect for CameraMainTextureUsages
impl FromReflect for CameraMainTextureUsages
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<CameraMainTextureUsages>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<CameraMainTextureUsages>
Constructs a concrete instance of
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Attempts to downcast the given value to
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetTypeRegistration for CameraMainTextureUsages
impl GetTypeRegistration for CameraMainTextureUsages
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(_registry: &mut TypeRegistry)
fn register_type_dependencies(_registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl IntoReturn for CameraMainTextureUsages
impl IntoReturn for CameraMainTextureUsages
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
CameraMainTextureUsages: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
CameraMainTextureUsages: 'into_return,
Sourceimpl PartialReflect for CameraMainTextureUsages
impl PartialReflect for CameraMainTextureUsages
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<CameraMainTextureUsages>) -> ReflectOwned
fn reflect_owned(self: Box<CameraMainTextureUsages>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<CameraMainTextureUsages>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<CameraMainTextureUsages>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(
self: Box<CameraMainTextureUsages>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<CameraMainTextureUsages>, ) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn reflect_partial_eq(
&self,
_value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, _value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
_value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, _value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl Reflect for CameraMainTextureUsages
impl Reflect for CameraMainTextureUsages
Sourcefn into_any(self: Box<CameraMainTextureUsages>) -> Box<dyn Any>
fn into_any(self: Box<CameraMainTextureUsages>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<CameraMainTextureUsages>) -> Box<dyn Reflect>
fn into_reflect(self: Box<CameraMainTextureUsages>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
Sourceimpl SyncComponent for CameraMainTextureUsages
impl SyncComponent for CameraMainTextureUsages
Sourcetype Target = CameraMainTextureUsages
type Target = CameraMainTextureUsages
Describes what components should be removed from the render world if the
implementing component is removed.
Sourceimpl TypePath for CameraMainTextureUsages
impl TypePath for CameraMainTextureUsages
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>