pub enum VisibilitySystems {
CalculateBounds,
UpdateFrusta,
VisibilityPropagate,
CheckVisibility,
MarkNewlyHiddenEntitiesInvisible,
}bevy_camera only.Variants
CalculateBounds
Label for the calculate_bounds, calculate_bounds_2d and calculate_bounds_text2d systems,
calculating and inserting an Aabb to relevant entities.
UpdateFrusta
Label for update_frusta in CameraProjectionPlugin.
VisibilityPropagate
Label for the system propagating the InheritedVisibility in a
ChildOf / Children hierarchy.
CheckVisibility
Label for the check_visibility_cpu_culling and
check_visibility_gpu_culling systems updating ViewVisibility of
each entity and the VisibleEntities of each view.
System order ambiguities between systems in this set are ignored: the
order of systems within this set is irrelevant, as
check_visibility_cpu_culling assumes that its operations are
irreversible during the frame.
MarkNewlyHiddenEntitiesInvisible
Label for the mark_newly_hidden_entities_invisible system, which sets
ViewVisibility to ViewVisibility::HIDDEN for entities that no
view has marked as visible.