pub struct VisibleEntityRanges { /* private fields */ }Expand description
Stores which entities are in within the VisibilityRanges of views.
This doesn’t store the results of frustum or occlusion culling; use
ViewVisibility for that. Thus entities in this list may not
actually be visible.
For efficiency, these tables only store entities that have
VisibilityRange components. Entities without such a component won’t be
in these tables at all.
The table is indexed by entity and stores a 32-bit bitmask with one bit for each camera, where a 0 bit corresponds to “out of range” and a 1 bit corresponds to “in range”. Hence it’s limited to storing information for 32 views.
Implementations
Sourceimpl VisibleEntityRanges
impl VisibleEntityRanges
Sourcepub fn entity_is_in_range_of_view(&self, entity: Entity, view: Entity) -> bool
pub fn entity_is_in_range_of_view(&self, entity: Entity, view: Entity) -> bool
Returns true if the entity is in range of the given camera.
This only checks VisibilityRanges and doesn’t perform any frustum or
occlusion culling. Thus the entity might not actually be visible.
The entity is assumed to have a VisibilityRange component. If the
entity doesn’t have that component, this method will return false.
Sourcepub fn entity_is_in_range_of_any_view(&self, entity: Entity) -> bool
pub fn entity_is_in_range_of_any_view(&self, entity: Entity) -> bool
Returns true if the entity is in range of any view.
This only checks VisibilityRanges and doesn’t perform any frustum or
occlusion culling. Thus the entity might not actually be visible.
The entity is assumed to have a VisibilityRange component. If the
entity doesn’t have that component, this method will return false.