FreeCamera

Struct FreeCamera 

pub struct FreeCamera {
Show 20 fields pub sensitivity: f32, pub key_forward: KeyCode, pub key_back: KeyCode, pub key_left: KeyCode, pub key_right: KeyCode, pub key_up: KeyCode, pub key_down: KeyCode, pub key_run: KeyCode, pub mouse_key_cursor_grab: MouseButton, pub keyboard_key_toggle_cursor_grab: KeyCode, pub key_snap_reverse: KeyCode, pub axis_top: KeyCode, pub axis_right: KeyCode, pub axis_front: KeyCode, pub walk_speed: f32, pub run_speed: f32, pub scroll_factor: f32, pub friction: f32, pub rotation_speed: f32, pub vertical_movement_axis: VerticalMovementAxis,
}
Available on crate features bevy_camera_controller and free_camera only.
Stores the settings for the FreeCamera controller.

This component defines static configuration for camera controls, including movement speed, sensitivity, and input bindings.

From the controller’s perspective, this data is treated as immutable, but it may be modified externally (e.g., by a settings UI) at runtime.

Add this component to a Camera entity to enable FreeCamera controls. The associated dynamic state is automatically handled by FreeCameraState, which is added to the entity as a required component.

To activate the controller, add the FreeCameraPlugin to your App.

Fields

sensitivity: f32

Multiplier for pitch and yaw rotation speed.

key_forward: KeyCode

KeyCode for forward translation.

key_back: KeyCode

KeyCode for backward translation.

key_left: KeyCode

KeyCode for left translation.

key_right: KeyCode

KeyCode for right translation.

key_up: KeyCode

KeyCode for up translation.

key_down: KeyCode

KeyCode for down translation.

key_run: KeyCode

KeyCode to use run_speed instead of walk_speed for translation.

mouse_key_cursor_grab: MouseButton

MouseButton for grabbing the mouse focus.

keyboard_key_toggle_cursor_grab: KeyCode

KeyCode for grabbing the keyboard focus.

key_snap_reverse: KeyCode

Modifier KeyCode for making pressed axis alignment buttons go in opposite direction

axis_top: KeyCode

KeyCode for snapping camera to top/bottom (+Y/-Y).

axis_right: KeyCode

KeyCode for snapping camera to right/left (+X/-X).

axis_front: KeyCode

KeyCode for snapping camera to front/back (-Z/+Z).

walk_speed: f32

Base multiplier for unmodified translation speed.

run_speed: f32

Base multiplier for running translation speed.

scroll_factor: f32

Multiplier for how much the mouse scroll wheel affects walk_speed and run_speed.

Mouse scroll affects speed exponentially. This is to ensure that scrolling the same amount always has the same effect on speed, regardless of how the scroll amount is reported by the hardware (i.e. as one big event vs many smaller events). This also allows the free camera to navigate very large scenes easier.

For every unit of scroll, the speed of the camera is multiplied by a factor of e^(scroll_factor).

A reasonable value to start with is a scroll_factor between 0.04879016 (~ln(1.05)) and 0.0953102 (~ln(1.1)). They represent an increase by a factor between 1.05 and 1.1 per positive unit scroll and a reduction between ~0.952 (~e^-0.04879016) and ~0.909 (~e^-0.0953102) times its value per negative unit scroll

A scroll_factor closer to 0.0 means that speed will be less sensitive to scroll. A scroll_factor equal to 0.0 means that speed is unaffected by scroll (it will be multiplied by a factor of 1.0 per positive and negative unit scroll).

friction: f32

Friction factor used to exponentially decay velocity over time.

rotation_speed: f32

Speed of camera rotation to snapped axis in radians/second

vertical_movement_axis: VerticalMovementAxis

Whether the vertical inputs translate the camera in world or local space axes.