CurrentMotion

Enum CurrentMotion 

pub enum CurrentMotion {
    Stationary,
    Momentum {
        velocity: Velocity,
        momentum_start: Instant,
    },
    UserControlled {
        anchor: DVec3,
        motion_inputs: MotionInputs,
    },
}
Available on crate features bevy_camera_controller and pan_orbit_camera only.
The current motion state of the camera.

Variants



Stationary

The camera is not moving.



Momentum

The camera is in motion, but not being directly controlled by the user. This happens while the camera has momentum.

Fields

velocity: Velocity

Contains inherited velocity, if any. This will decay based on momentum settings.

momentum_start: Instant

Used to compute how long the camera has been in the momentum state. Useful for debouncing user inputs.



UserControlled

The camera is being directly controlled by the user.

Fields

anchor: DVec3

The point the camera is rotating about, zooming into, or panning with, in view space (relative to the camera).

  • Rotation: the direction of the anchor does not change, it is fixed in screenspace.
  • Panning: the depth of the anchor does not change, the camera only moves in x and y.
  • Zoom: the direction of the anchor does not change, but the length does.
motion_inputs: MotionInputs

Pan and orbit are mutually exclusive, however both can be used with zoom.

Implementations

Source

impl CurrentMotion

pub fn is_moving(&self) -> bool

Returns true if the camera is moving due to inputs or momentum.

pub fn is_user_controlled(&self) -> bool

Returns true if the camera is moving due to user inputs.

pub fn inputs(&self) -> Option<&MotionInputs>

Get the user motion inputs if they exist.

Examples found in repository?
examples/camera/pan_orbit_camera_custom_input_plugin.rs (line 91)
69pub fn custom_camera_inputs(
70    pointers: Query<(&PointerId, &PointerLocation)>,
71    pointer_map: Res<CameraPointerMap>,
72    mut controller: MessageWriter<PanOrbitCameraInputMessage>,
73    mut mouse_wheel: MessageReader<MouseWheel>,
74    mouse_input: Res<ButtonInput<MouseButton>>,
75    cameras: Query<(Entity, &Camera, &RenderTarget, &PanOrbitCamera)>,
76    primary_window: Query<Entity, With<PrimaryWindow>>,
77) {
78    let orbit_start = MouseButton::Right;
79    let pan_start = MouseButton::Left;
80    let zoom_stop = 0.0;
81
82    if let Some(&camera) = pointer_map.get(&PointerId::Mouse) {
83        let camera_query = cameras.get(camera).ok();
84        let is_in_zoom_mode = camera_query
85            .map(|(.., editor_cam)| editor_cam.current_motion.is_zooming_only())
86            .unwrap_or_default();
87        let zoom_amount_abs = camera_query
88            .and_then(|(.., editor_cam)| {
89                editor_cam
90                    .current_motion
91                    .inputs()
92                    .map(|inputs| inputs.zoom_velocity_abs(editor_cam.smoothing.zoom.mul_f32(2.0)))
93            })
94            .unwrap_or(0.0);
95        let should_zoom_end = is_in_zoom_mode && zoom_amount_abs <= zoom_stop;
96
97        if mouse_input.any_just_released([orbit_start, pan_start]) || should_zoom_end {
98            controller.write(PanOrbitCameraInputMessage::End { camera });
99        }
100    }
101
102    for (&pointer, pointer_location) in pointers
103        .iter()
104        .filter_map(|(id, loc)| loc.location().map(|loc| (id, loc)))
105    {
106        match pointer {
107            PointerId::Mouse => {
108                let Some((camera, ..)) = cameras.iter().find(|(_, camera, render_target, _)| {
109                    pointer_location.is_in_viewport(camera, render_target, &primary_window)
110                }) else {
111                    continue; // Pointer must be in viewport to start a motion.
112                };
113
114                if mouse_input.just_pressed(orbit_start) {
115                    controller.write(PanOrbitCameraInputMessage::Start {
116                        kind: MotionKind::OrbitZoom,
117                        camera,
118                        pointer,
119                    });
120                } else if mouse_input.just_pressed(pan_start) {
121                    controller.write(PanOrbitCameraInputMessage::Start {
122                        kind: MotionKind::PanZoom,
123                        camera,
124                        pointer,
125                    });
126                } else if mouse_wheel.read().map(|mw| mw.y.abs()).sum::<f32>() > 0.0 {
127                    // Note we can't just check if the mouse wheel inputs are empty, we need to
128                    // check if the y value abs greater than zero, otherwise we get a bunch of false
129                    // positives, which can cause issues with figuring out what the user is trying
130                    // to do.
131                    controller.write(PanOrbitCameraInputMessage::Start {
132                        kind: MotionKind::Zoom,
133                        camera,
134                        pointer,
135                    });
136                }
137            }
138            PointerId::Touch(_) | PointerId::Custom(_) => continue,
139        }
140    }
141
142    // This must be cleared manually because reading these inputs is conditional - we are not
143    // guaranteed to be flushing the events every frame.
144    mouse_wheel.clear();
145}
pub fn is_orbiting(&self) -> bool

Returns true if the camera is user controlled and orbiting.

pub fn is_panning(&self) -> bool

Returns true if the camera is user controlled and panning.

pub fn is_zooming_only(&self) -> bool

Returns true if the camera is user controlled and only zooming.

Examples found in repository?
examples/camera/pan_orbit_camera_custom_input_plugin.rs (line 85)
69pub fn custom_camera_inputs(
70    pointers: Query<(&PointerId, &PointerLocation)>,
71    pointer_map: Res<CameraPointerMap>,
72    mut controller: MessageWriter<PanOrbitCameraInputMessage>,
73    mut mouse_wheel: MessageReader<MouseWheel>,
74    mouse_input: Res<ButtonInput<MouseButton>>,
75    cameras: Query<(Entity, &Camera, &RenderTarget, &PanOrbitCamera)>,
76    primary_window: Query<Entity, With<PrimaryWindow>>,
77) {
78    let orbit_start = MouseButton::Right;
79    let pan_start = MouseButton::Left;
80    let zoom_stop = 0.0;
81
82    if let Some(&camera) = pointer_map.get(&PointerId::Mouse) {
83        let camera_query = cameras.get(camera).ok();
84        let is_in_zoom_mode = camera_query
85            .map(|(.., editor_cam)| editor_cam.current_motion.is_zooming_only())
86            .unwrap_or_default();
87        let zoom_amount_abs = camera_query
88            .and_then(|(.., editor_cam)| {
89                editor_cam
90                    .current_motion
91                    .inputs()
92                    .map(|inputs| inputs.zoom_velocity_abs(editor_cam.smoothing.zoom.mul_f32(2.0)))
93            })
94            .unwrap_or(0.0);
95        let should_zoom_end = is_in_zoom_mode && zoom_amount_abs <= zoom_stop;
96
97        if mouse_input.any_just_released([orbit_start, pan_start]) || should_zoom_end {
98            controller.write(PanOrbitCameraInputMessage::End { camera });
99        }
100    }
101
102    for (&pointer, pointer_location) in pointers
103        .iter()
104        .filter_map(|(id, loc)| loc.location().map(|loc| (id, loc)))
105    {
106        match pointer {
107            PointerId::Mouse => {
108                let Some((camera, ..)) = cameras.iter().find(|(_, camera, render_target, _)| {
109                    pointer_location.is_in_viewport(camera, render_target, &primary_window)
110                }) else {
111                    continue; // Pointer must be in viewport to start a motion.
112                };
113
114                if mouse_input.just_pressed(orbit_start) {
115                    controller.write(PanOrbitCameraInputMessage::Start {
116                        kind: MotionKind::OrbitZoom,
117                        camera,
118                        pointer,
119                    });
120                } else if mouse_input.just_pressed(pan_start) {
121                    controller.write(PanOrbitCameraInputMessage::Start {
122                        kind: MotionKind::PanZoom,
123                        camera,
124                        pointer,
125                    });
126                } else if mouse_wheel.read().map(|mw| mw.y.abs()).sum::<f32>() > 0.0 {
127                    // Note we can't just check if the mouse wheel inputs are empty, we need to
128                    // check if the y value abs greater than zero, otherwise we get a bunch of false
129                    // positives, which can cause issues with figuring out what the user is trying
130                    // to do.
131                    controller.write(PanOrbitCameraInputMessage::Start {
132                        kind: MotionKind::Zoom,
133                        camera,
134                        pointer,
135                    });
136                }
137            }
138            PointerId::Touch(_) | PointerId::Custom(_) => continue,
139        }
140    }
141
142    // This must be cleared manually because reading these inputs is conditional - we are not
143    // guaranteed to be flushing the events every frame.
144    mouse_wheel.clear();
145}
pub fn momentum_duration(&self) -> Option<Duration>

How long has the camera been moving with momentum, without user input? This is equivalent to the amount of time since the last input event ended.