pub struct FunctionCurve<T, F> { /* private fields */ }Available on crate feature
bevy_curve only.Expand description
A curve defined by a function together with a fixed domain.
This is a curve that holds an inner function f which takes numbers (f32) as input and produces
output of type T. The value of this curve when sampled at time t is just f(t).
Implementations
Sourceimpl<T, F> FunctionCurve<T, F>
impl<T, F> FunctionCurve<T, F>
Sourcepub fn new(domain: Interval, function: F) -> FunctionCurve<T, F>
pub fn new(domain: Interval, function: F) -> FunctionCurve<T, F>
Create a new curve with the given domain from the given function. When sampled, the
function is evaluated at the sample time to compute the output.
Examples found in repository?
examples/gizmos/2d_gizmos.rs (line 81)
49fn draw_example_collection(
50 mut gizmos: Gizmos,
51 mut my_gizmos: Gizmos<MyRoundGizmos>,
52 time: Res<Time>,
53) {
54 let sin_t_scaled = ops::sin(time.elapsed_secs()) * 50.;
55 gizmos.line_2d(Vec2::Y * -sin_t_scaled, Vec2::splat(-80.), RED);
56 gizmos.ray_2d(Vec2::Y * sin_t_scaled, Vec2::splat(80.), LIME);
57
58 gizmos
59 .grid_2d(
60 Isometry2d::IDENTITY,
61 UVec2::new(16, 9),
62 Vec2::new(80., 80.),
63 // Dark gray
64 LinearRgba::gray(0.05),
65 )
66 .outer_edges();
67
68 // Triangle
69 gizmos.linestrip_gradient_2d([
70 (Vec2::Y * 300., BLUE),
71 (Vec2::new(-255., -155.), RED),
72 (Vec2::new(255., -155.), LIME),
73 (Vec2::Y * 300., BLUE),
74 ]);
75
76 gizmos.rect_2d(Isometry2d::IDENTITY, Vec2::splat(650.), BLACK);
77
78 gizmos.cross_2d(Vec2::new(-160., 120.), 12., FUCHSIA);
79
80 let domain = Interval::EVERYWHERE;
81 let curve = FunctionCurve::new(domain, |t| Vec2::new(t, ops::sin(t / 25.0) * 100.0));
82 let resolution = ((ops::sin(time.elapsed_secs()) + 1.0) * 50.0) as usize;
83 let times_and_colors = (0..=resolution)
84 .map(|n| n as f32 / resolution as f32)
85 .map(|t| (t - 0.5) * 600.0)
86 .map(|t| (t, TEAL.mix(&HOT_PINK, (t + 300.0) / 600.0)));
87 gizmos.curve_gradient_2d(curve, times_and_colors);
88
89 my_gizmos
90 .rounded_rect_2d(Isometry2d::IDENTITY, Vec2::splat(630.), BLACK)
91 .corner_radius(ops::cos(time.elapsed_secs() / 3.) * 100.);
92
93 // Circles have 32 line-segments by default.
94 // You may want to increase this for larger circles.
95 my_gizmos
96 .circle_2d(Isometry2d::IDENTITY, 300., NAVY)
97 .resolution(64);
98
99 my_gizmos.ellipse_2d(
100 Rot2::radians(time.elapsed_secs() % TAU),
101 Vec2::new(100., 200.),
102 YELLOW_GREEN,
103 );
104
105 // Arcs default resolution is linearly interpolated between
106 // 1 and 32, using the arc length as scalar.
107 my_gizmos.arc_2d(
108 Rot2::radians(sin_t_scaled / 10.),
109 FRAC_PI_2,
110 310.,
111 ORANGE_RED,
112 );
113 my_gizmos.arc_2d(Isometry2d::IDENTITY, FRAC_PI_2, 80.0, ORANGE_RED);
114 my_gizmos.long_arc_2d_between(Vec2::ZERO, Vec2::X * 20.0, Vec2::Y * 20.0, ORANGE_RED);
115 my_gizmos.short_arc_2d_between(Vec2::ZERO, Vec2::X * 40.0, Vec2::Y * 40.0, ORANGE_RED);
116
117 gizmos.arrow_2d(
118 Vec2::ZERO,
119 Vec2::from_angle(sin_t_scaled / -10. + PI / 2.) * 50.,
120 YELLOW,
121 );
122
123 // You can create more complex arrows using the arrow builder.
124 gizmos
125 .arrow_2d(
126 Vec2::ZERO,
127 Vec2::from_angle(sin_t_scaled / -10.) * 50.,
128 GREEN,
129 )
130 .with_double_end()
131 .with_tip_length(10.);
132
133 my_gizmos.arc_2d(Isometry2d::default(), FRAC_PI_2, 210., OLD_LACE);
134}More examples
examples/gizmos/3d_gizmos.rs (lines 158-160)
107fn draw_example_collection(
108 mut gizmos: Gizmos,
109 mut my_gizmos: Gizmos<MyRoundGizmos>,
110 time: Res<Time>,
111) {
112 gizmos.grid(
113 Quat::from_rotation_x(PI / 2.),
114 UVec2::splat(20),
115 Vec2::new(2., 2.),
116 // Light gray
117 LinearRgba::gray(0.65),
118 );
119 gizmos.grid(
120 Isometry3d::new(Vec3::splat(10.0), Quat::from_rotation_x(PI / 3. * 2.)),
121 UVec2::splat(20),
122 Vec2::new(2., 2.),
123 PURPLE,
124 );
125 gizmos.sphere(Vec3::splat(10.0), 1.0, PURPLE);
126
127 gizmos
128 .primitive_3d(
129 &Plane3d {
130 normal: Dir3::Y,
131 half_size: Vec2::splat(1.0),
132 },
133 Isometry3d::new(
134 Vec3::splat(4.0) + Vec2::from(ops::sin_cos(time.elapsed_secs())).extend(0.0),
135 Quat::from_rotation_x(PI / 2. + time.elapsed_secs()),
136 ),
137 GREEN,
138 )
139 .cell_count(UVec2::new(5, 10))
140 .spacing(Vec2::new(0.2, 0.1));
141
142 gizmos.cube(
143 Transform::from_translation(Vec3::Y * 0.5).with_scale(Vec3::splat(1.25)),
144 BLACK,
145 );
146 gizmos.rect(
147 Isometry3d::new(
148 Vec3::new(ops::cos(time.elapsed_secs()) * 2.5, 1., 0.),
149 Quat::from_rotation_y(PI / 2.),
150 ),
151 Vec2::splat(2.),
152 LIME,
153 );
154
155 gizmos.cross(Vec3::new(-1., 1., 1.), 0.5, FUCHSIA);
156
157 let domain = Interval::EVERYWHERE;
158 let curve = FunctionCurve::new(domain, |t| {
159 (Vec2::from(ops::sin_cos(t * 10.0))).extend(t - 6.0)
160 });
161 let resolution = ((ops::sin(time.elapsed_secs()) + 1.0) * 100.0) as usize;
162 let times_and_colors = (0..=resolution)
163 .map(|n| n as f32 / resolution as f32)
164 .map(|t| t * 5.0)
165 .map(|t| (t, TEAL.mix(&HOT_PINK, t / 5.0)));
166 gizmos.curve_gradient_3d(curve, times_and_colors);
167
168 gizmos.primitive_3d(
169 &Capsule3d::new(0.35, 0.5),
170 Isometry3d::new(
171 Vec3::new(-1.75, 0.75, 0.75),
172 Quat::from_rotation_y(ops::cos(time.elapsed_secs() / 2.0) * 10.0)
173 * Quat::from_rotation_z(PI / 3.0),
174 ),
175 YELLOW_GREEN,
176 );
177
178 my_gizmos.sphere(Vec3::new(1., 0.5, 0.), 0.5, RED);
179
180 my_gizmos
181 .rounded_cuboid(Vec3::new(-2.0, 0.75, -0.75), Vec3::splat(0.9), TURQUOISE)
182 .edge_radius(0.1)
183 .arc_resolution(4);
184
185 for y in [0., 0.5, 1.] {
186 gizmos.ray(
187 Vec3::new(1., y, 0.),
188 Vec3::new(-3., ops::sin(time.elapsed_secs() * 3.), 0.),
189 BLUE,
190 );
191 }
192
193 my_gizmos
194 .arc_3d(
195 180.0_f32.to_radians(),
196 0.2,
197 Isometry3d::new(
198 Vec3::ONE,
199 Quat::from_rotation_arc(Vec3::Y, Vec3::ONE.normalize()),
200 ),
201 ORANGE,
202 )
203 .resolution(10);
204
205 // Circles have 32 line-segments by default.
206 my_gizmos.circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3., BLACK);
207
208 // You may want to increase this for larger circles or spheres.
209 my_gizmos
210 .circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3.1, NAVY)
211 .resolution(64);
212 my_gizmos
213 .sphere(Isometry3d::IDENTITY, 3.2, BLACK)
214 .resolution(64);
215
216 gizmos.arrow(Vec3::ZERO, Vec3::splat(1.5), YELLOW);
217
218 // You can create more complex arrows using the arrow builder.
219 gizmos
220 .arrow(Vec3::new(2., 0., 2.), Vec3::new(2., 2., 2.), ORANGE_RED)
221 .with_double_end()
222 .with_tip_length(0.5);
223
224 let from = Vec3::new(1.0, 2.0, 3.0);
225 let to = Vec3::new(3.0, 2.5, 4.0);
226 gizmos.rect(from, Vec2::ONE, RED);
227 my_gizmos.short_arc_3d_between((from + to) / 2.0, from, to, YELLOW_GREEN);
228 gizmos.rect(to, Vec2::ONE, RED);
229}Trait Implementations
Sourceimpl<T, F> Clone for FunctionCurve<T, F>
impl<T, F> Clone for FunctionCurve<T, F>
Sourcefn clone(&self) -> FunctionCurve<T, F>
fn clone(&self) -> FunctionCurve<T, F>
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<T, F> Curve<T> for FunctionCurve<T, F>
impl<T, F> Curve<T> for FunctionCurve<T, F>
Sourcefn sample_unchecked(&self, t: f32) -> T
fn sample_unchecked(&self, t: f32) -> T
Sample a point on this curve at the parameter value
t, extracting the associated value.
This is the unchecked version of sampling, which should only be used if the sample time t
is already known to lie within the curve’s domain. Read moreSourcefn sample(&self, t: f32) -> Option<T>
fn sample(&self, t: f32) -> Option<T>
Sample a point on this curve at the parameter value
t, returning None if the point is
outside of the curve’s domain.Sourcefn sample_clamped(&self, t: f32) -> T
fn sample_clamped(&self, t: f32) -> T
Sample a point on this curve at the parameter value
t, clamping t to lie inside the
domain of the curve.Sourceimpl<T, F> Debug for FunctionCurve<T, F>
impl<T, F> Debug for FunctionCurve<T, F>
Sourceimpl<'de, T, F> Deserialize<'de> for FunctionCurve<T, F>where
F: Deserialize<'de>,
impl<'de, T, F> Deserialize<'de> for FunctionCurve<T, F>where
F: Deserialize<'de>,
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<FunctionCurve<T, F>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<FunctionCurve<T, F>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl<T, F> FromArg for FunctionCurve<T, F>
impl<T, F> FromArg for FunctionCurve<T, F>
Sourceimpl<T, F> GetOwnership for FunctionCurve<T, F>
impl<T, F> GetOwnership for FunctionCurve<T, F>
Sourceimpl<T, F> GetTypeRegistration for FunctionCurve<T, F>
impl<T, F> GetTypeRegistration for FunctionCurve<T, F>
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl<T, F> IntoReturn for FunctionCurve<T, F>
impl<T, F> IntoReturn for FunctionCurve<T, F>
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
FunctionCurve<T, F>: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
FunctionCurve<T, F>: 'into_return,
Sourceimpl<T, F> PartialReflect for FunctionCurve<T, F>
impl<T, F> PartialReflect for FunctionCurve<T, F>
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<FunctionCurve<T, F>>) -> ReflectOwned
fn reflect_owned(self: Box<FunctionCurve<T, F>>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<FunctionCurve<T, F>>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<FunctionCurve<T, F>>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(
self: Box<FunctionCurve<T, F>>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<FunctionCurve<T, F>>, ) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
fn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl<T, F> Reflect for FunctionCurve<T, F>
impl<T, F> Reflect for FunctionCurve<T, F>
Sourcefn into_any(self: Box<FunctionCurve<T, F>>) -> Box<dyn Any>
fn into_any(self: Box<FunctionCurve<T, F>>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<FunctionCurve<T, F>>) -> Box<dyn Reflect>
fn into_reflect(self: Box<FunctionCurve<T, F>>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
Sourceimpl<T, F> Serialize for FunctionCurve<T, F>where
F: Serialize,
impl<T, F> Serialize for FunctionCurve<T, F>where
F: Serialize,
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Serialize this value into the given Serde serializer. Read more
Sourceimpl<T, F> Struct for FunctionCurve<T, F>
impl<T, F> Struct for FunctionCurve<T, F>
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field named
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field named
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field with index
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field with index
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Gets the index of the field with the given name.
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Returns an iterator over the values of the reflectable fields for this struct.
Sourcefn to_dynamic_struct(&self) -> Result<DynamicStruct, ReflectCloneError>
fn to_dynamic_struct(&self) -> Result<DynamicStruct, ReflectCloneError>
Creates a new
DynamicStruct from this struct. Read moreSourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
Will return
None if TypeInfo is not available.Sourceimpl<T, F> TypePath for FunctionCurve<T, F>where
T: TypePath,
F: 'static,
Available on crate feature bevy_reflect only.Note: This is not a fully stable implementation of TypePath due to usage of type_name
for function members.
impl<T, F> TypePath for FunctionCurve<T, F>where
T: TypePath,
F: 'static,
Available on crate feature
bevy_reflect only.Note: This is not a fully stable implementation of TypePath due to usage of type_name
for function members.
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>