Module bevy::dev_tools::ci_testing
source · Available on crate feature
bevy_ci_testing only.Expand description
Utilities for testing in CI environments.
Structs
- A configuration struct for automated CI testing.
- A custom event that can be configured from a configuration file for CI testing.
- An event to send at a given frame, used for CI testing.
- A plugin that instruments continuous integration testing by automatically executing user-defined actions.
- Setup for a test.
Enums
- An event to send, used for CI testing.