CiTestingConfig

Struct CiTestingConfig 

pub struct CiTestingConfig {
    pub setup: CiTestingSetup,
    pub events: Vec<CiTestingEventOnFrame>,
}
Available on crate feature bevy_ci_testing only.
A configuration struct for automated CI testing.

It gets used when the bevy_ci_testing feature is enabled to automatically exit a Bevy app when run through the CI. This is needed because otherwise Bevy apps would be stuck in the game loop and wouldn’t allow the CI to progress.

Fields

setup: CiTestingSetup

The setup for this test.

events: Vec<CiTestingEventOnFrame>

Events to send, with their associated frame.