EasyScreenshotPlugin

Struct EasyScreenshotPlugin 

pub struct EasyScreenshotPlugin {
    pub trigger: KeyCode,
    pub format: ScreenshotFormat,
}
Available on crate feature bevy_dev_tools only.
Add this plugin to your app to enable easy screenshotting.

Add this plugin, press the key, and you have a screenshot ๐ŸŽ‰

Fields

trigger: KeyCode

Key that will trigger a screenshot

format: ScreenshotFormat

Format of the screenshot

The corresponding image format must be supported by bevy renderer