LogDiagnosticsState

Struct LogDiagnosticsState 

pub struct LogDiagnosticsState { /* private fields */ }
State used by the LogDiagnosticsPlugin

Implementations

Source

impl LogDiagnosticsState

pub fn set_timer_duration(&mut self, duration: Duration)

Sets a new duration for the log timer

pub fn add_filter(&mut self, diagnostic_path: DiagnosticPath) -> bool

Add a filter to the log state, returning true if the DiagnosticPath was not present

pub fn extend_filter(&mut self, iter: impl IntoIterator<Item = DiagnosticPath>)

Extends the filter of the log state with multiple DiagnosticPaths

Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 168)
164fn enable_filters(
165    log_state: &mut LogDiagnosticsState,
166    diagnostics: impl IntoIterator<Item = DiagnosticPath>,
167) {
168    log_state.extend_filter(diagnostics);
169}
pub fn remove_filter(&mut self, diagnostic_path: &DiagnosticPath) -> bool

Removes a filter from the log state, returning true if it was present

Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 176)
171fn disable_filters(
172    log_state: &mut LogDiagnosticsState,
173    diagnostics: impl IntoIterator<Item = DiagnosticPath>,
174) {
175    for diagnostic in diagnostics {
176        log_state.remove_filter(&diagnostic);
177    }
178}
pub fn clear_filter(&mut self)

Clears the filters of the log state

pub fn enable_filtering(&mut self)

Enables filtering with empty filters

Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 116)
103fn filters_inputs(
104    keys: Res<ButtonInput<KeyCode>>,
105    mut status: ResMut<LogDiagnosticsStatus>,
106    mut filters: ResMut<LogDiagnosticsFilters>,
107    mut log_state: ResMut<LogDiagnosticsState>,
108) {
109    if keys.just_pressed(KeyCode::KeyQ) {
110        *status = match *status {
111            LogDiagnosticsStatus::Enabled => {
112                log_state.disable_filtering();
113                LogDiagnosticsStatus::Disabled
114            }
115            LogDiagnosticsStatus::Disabled => {
116                log_state.enable_filtering();
117                if filters.frame_time {
118                    enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
119                }
120                if filters.entity_count {
121                    enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
122                }
123                if filters.system_info {
124                    enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
125                }
126                LogDiagnosticsStatus::Enabled
127            }
128        };
129    }
130
131    let enabled = *status == LogDiagnosticsStatus::Enabled;
132    if keys.just_pressed(KeyCode::Digit1) {
133        filters.frame_time = !filters.frame_time;
134        if enabled {
135            if filters.frame_time {
136                enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
137            } else {
138                disable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
139            }
140        }
141    }
142    if keys.just_pressed(KeyCode::Digit2) {
143        filters.entity_count = !filters.entity_count;
144        if enabled {
145            if filters.entity_count {
146                enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
147            } else {
148                disable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
149            }
150        }
151    }
152    if keys.just_pressed(KeyCode::Digit3) {
153        filters.system_info = !filters.system_info;
154        if enabled {
155            if filters.system_info {
156                enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
157            } else {
158                disable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
159            }
160        }
161    }
162}
pub fn disable_filtering(&mut self)

Disables filtering

Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 112)
103fn filters_inputs(
104    keys: Res<ButtonInput<KeyCode>>,
105    mut status: ResMut<LogDiagnosticsStatus>,
106    mut filters: ResMut<LogDiagnosticsFilters>,
107    mut log_state: ResMut<LogDiagnosticsState>,
108) {
109    if keys.just_pressed(KeyCode::KeyQ) {
110        *status = match *status {
111            LogDiagnosticsStatus::Enabled => {
112                log_state.disable_filtering();
113                LogDiagnosticsStatus::Disabled
114            }
115            LogDiagnosticsStatus::Disabled => {
116                log_state.enable_filtering();
117                if filters.frame_time {
118                    enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
119                }
120                if filters.entity_count {
121                    enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
122                }
123                if filters.system_info {
124                    enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
125                }
126                LogDiagnosticsStatus::Enabled
127            }
128        };
129    }
130
131    let enabled = *status == LogDiagnosticsStatus::Enabled;
132    if keys.just_pressed(KeyCode::Digit1) {
133        filters.frame_time = !filters.frame_time;
134        if enabled {
135            if filters.frame_time {
136                enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
137            } else {
138                disable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
139            }
140        }
141    }
142    if keys.just_pressed(KeyCode::Digit2) {
143        filters.entity_count = !filters.entity_count;
144        if enabled {
145            if filters.entity_count {
146                enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
147            } else {
148                disable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
149            }
150        }
151    }
152    if keys.just_pressed(KeyCode::Digit3) {
153        filters.system_info = !filters.system_info;
154        if enabled {
155            if filters.system_info {
156                enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
157            } else {
158                disable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
159            }
160        }
161    }
162}