Edges

Struct Edges 

pub struct Edges { /* private fields */ }
Archetypes and bundles form a graph. Adding or removing a bundle moves an Entity to a new Archetype.

Edges caches the results of these moves. Each archetype caches the result of a structural alteration. This can be used to monitor the state of the archetype graph.

Note: This type only contains edges the World has already traversed. If any of functions return None, it doesn’t mean there is guaranteed not to be a result of adding or removing that bundle, but rather that operation that has moved an entity along that edge has not been performed yet.

Implementations

Source

impl Edges

pub fn get_archetype_after_bundle_insert( &self, bundle_id: BundleId, ) -> Option<ArchetypeId>

Checks the cache for the target archetype when inserting a bundle into the source archetype.

If this returns None, it means there has not been a transition from the source archetype via the provided bundle.

pub fn get_archetype_after_bundle_remove( &self, bundle_id: BundleId, ) -> Option<Option<ArchetypeId>>

Checks the cache for the target archetype when removing a bundle from the source archetype.

If this returns None, it means there has not been a transition from the source archetype via the provided bundle.

If this returns Some(None), it means that the bundle cannot be removed from the source archetype.

pub fn get_archetype_after_bundle_take( &self, bundle_id: BundleId, ) -> Option<Option<ArchetypeId>>

Checks the cache for the target archetype when taking a bundle from the source archetype.

Unlike remove, take will only succeed if the source archetype contains all of the components in the bundle.

If this returns None, it means there has not been a transition from the source archetype via the provided bundle.

If this returns Some(None), it means that the bundle cannot be taken from the source archetype.