pub struct ComponentId(/* private fields */);Expand description
A value which uniquely identifies the type of a Component or Resource within a
World.
Each time a new Component type is registered within a World using
e.g. World::register_component or World::register_component_with_descriptor
or a Resource with e.g. World::init_resource,
a corresponding ComponentId is created to track it.
While the distinction between ComponentId and TypeId may seem superficial, breaking them
into two separate but related concepts allows components to exist outside of Rust’s type system.
Each Rust type registered as a Component will have a corresponding ComponentId, but additional
ComponentIds may exist in a World to track components which cannot be
represented as Rust types for scripting or other advanced use-cases.
A ComponentId is tightly coupled to its parent World. Attempting to use a ComponentId from
one World to access the metadata of a Component in a different World is undefined behavior
and must not be attempted.
Given a type T which implements Component, the ComponentId for T can be retrieved
from a World using World::component_id() or via Components::component_id(). Access
to the ComponentId for a Resource is available via Components::resource_id().
Implementations
Sourceimpl ComponentId
impl ComponentId
Sourcepub const fn new(index: usize) -> ComponentId
pub const fn new(index: usize) -> ComponentId
Creates a new ComponentId.
The index is a unique value associated with each type of component in a given world.
Usually, this value is taken from a counter incremented for each type of component registered with the world.
Sourcepub fn index(self) -> usize
pub fn index(self) -> usize
Returns the index of the current component.
Examples found in repository?
51fn main() {
52 let mut world = World::new();
53 let mut lines = std::io::stdin().lines();
54 let mut component_names = HashMap::<String, ComponentId>::new();
55 let mut component_info = HashMap::<ComponentId, ComponentInfo>::new();
56
57 println!("{PROMPT}");
58 loop {
59 print!("\n> ");
60 let _ = std::io::stdout().flush();
61 let Some(Ok(line)) = lines.next() else {
62 return;
63 };
64
65 if line.is_empty() {
66 return;
67 };
68
69 let Some((first, rest)) = line.trim().split_once(|c: char| c.is_whitespace()) else {
70 match &line.chars().next() {
71 Some('c') => println!("{COMPONENT_PROMPT}"),
72 Some('s') => println!("{ENTITY_PROMPT}"),
73 Some('q') => println!("{QUERY_PROMPT}"),
74 _ => println!("{PROMPT}"),
75 }
76 continue;
77 };
78
79 match &first[0..1] {
80 "c" => {
81 rest.split(',').for_each(|component| {
82 let mut component = component.split_whitespace();
83 let Some(name) = component.next() else {
84 return;
85 };
86 let size = match component.next().map(str::parse) {
87 Some(Ok(size)) => size,
88 _ => 0,
89 };
90 // Register our new component to the world with a layout specified by it's size
91 // SAFETY: [u64] is Send + Sync
92 let id = world.register_component_with_descriptor(unsafe {
93 ComponentDescriptor::new_with_layout(
94 name.to_string(),
95 StorageType::Table,
96 Layout::array::<u64>(size).unwrap(),
97 None,
98 true,
99 ComponentCloneBehavior::Default,
100 )
101 });
102 let Some(info) = world.components().get_info(id) else {
103 return;
104 };
105 component_names.insert(name.to_string(), id);
106 component_info.insert(id, info.clone());
107 println!("Component {} created with id: {}", name, id.index());
108 });
109 }
110 "s" => {
111 let mut to_insert_ids = Vec::new();
112 let mut to_insert_data = Vec::new();
113 rest.split(',').for_each(|component| {
114 let mut component = component.split_whitespace();
115 let Some(name) = component.next() else {
116 return;
117 };
118
119 // Get the id for the component with the given name
120 let Some(&id) = component_names.get(name) else {
121 println!("Component {name} does not exist");
122 return;
123 };
124
125 // Calculate the length for the array based on the layout created for this component id
126 let info = world.components().get_info(id).unwrap();
127 let len = info.layout().size() / size_of::<u64>();
128 let mut values: Vec<u64> = component
129 .take(len)
130 .filter_map(|value| value.parse::<u64>().ok())
131 .collect();
132 values.resize(len, 0);
133
134 // Collect the id and array to be inserted onto our entity
135 to_insert_ids.push(id);
136 to_insert_data.push(values);
137 });
138
139 let mut entity = world.spawn_empty();
140
141 // Construct an `OwningPtr` for each component in `to_insert_data`
142 let to_insert_ptr = to_owning_ptrs(&mut to_insert_data);
143
144 // SAFETY:
145 // - Component ids have been taken from the same world
146 // - Each array is created to the layout specified in the world
147 unsafe {
148 entity.insert_by_ids(&to_insert_ids, to_insert_ptr.into_iter());
149 }
150
151 println!("Entity spawned with id: {}", entity.id());
152 }
153 "q" => {
154 let mut builder = QueryBuilder::<FilteredEntityMut>::new(&mut world);
155 parse_query(rest, &mut builder, &component_names);
156 let mut query = builder.build();
157 query.iter_mut(&mut world).for_each(|filtered_entity| {
158 let terms = filtered_entity
159 .access()
160 .try_iter_component_access()
161 .unwrap()
162 .map(|component_access| {
163 let id = *component_access.index();
164 let ptr = filtered_entity.get_by_id(id).unwrap();
165 let info = component_info.get(&id).unwrap();
166 let len = info.layout().size() / size_of::<u64>();
167
168 // SAFETY:
169 // - All components are created with layout [u64]
170 // - len is calculated from the component descriptor
171 let data = unsafe {
172 std::slice::from_raw_parts_mut(
173 ptr.assert_unique().as_ptr().cast::<u64>(),
174 len,
175 )
176 };
177
178 // If we have write access, increment each value once
179 if matches!(component_access, ComponentAccessKind::Exclusive(_)) {
180 data.iter_mut().for_each(|data| {
181 *data += 1;
182 });
183 }
184
185 format!("{}: {:?}", info.name(), data[0..len].to_vec())
186 })
187 .collect::<Vec<_>>()
188 .join(", ");
189
190 println!("{}: {}", filtered_entity.id(), terms);
191 });
192 }
193 _ => continue,
194 }
195 }
196}Trait Implementations
Sourceimpl Clone for ComponentId
impl Clone for ComponentId
Sourcefn clone(&self) -> ComponentId
fn clone(&self) -> ComponentId
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for ComponentId
impl Debug for ComponentId
Sourceimpl DynamicComponentFetch for ComponentId
impl DynamicComponentFetch for ComponentId
Sourcetype Ref<'w> = Ptr<'w>
type Ref<'w> = Ptr<'w>
DynamicComponentFetch::fetch_ref.Sourcetype Mut<'w> = MutUntyped<'w>
type Mut<'w> = MutUntyped<'w>
DynamicComponentFetch::fetch_mut.Sourceunsafe fn fetch_ref(
self,
cell: UnsafeEntityCell<'_>,
) -> Result<<ComponentId as DynamicComponentFetch>::Ref<'_>, EntityComponentError>
unsafe fn fetch_ref( self, cell: UnsafeEntityCell<'_>, ) -> Result<<ComponentId as DynamicComponentFetch>::Ref<'_>, EntityComponentError>
ComponentIds, as determined by self. Read moreSourceunsafe fn fetch_mut(
self,
cell: UnsafeEntityCell<'_>,
) -> Result<<ComponentId as DynamicComponentFetch>::Mut<'_>, EntityComponentError>
unsafe fn fetch_mut( self, cell: UnsafeEntityCell<'_>, ) -> Result<<ComponentId as DynamicComponentFetch>::Mut<'_>, EntityComponentError>
ComponentIds, as determined by self. Read moreSourceunsafe fn fetch_mut_assume_mutable(
self,
cell: UnsafeEntityCell<'_>,
) -> Result<<ComponentId as DynamicComponentFetch>::Mut<'_>, EntityComponentError>
unsafe fn fetch_mut_assume_mutable( self, cell: UnsafeEntityCell<'_>, ) -> Result<<ComponentId as DynamicComponentFetch>::Mut<'_>, EntityComponentError>
ComponentIds, as determined by self.
Assumes all ComponentIds refer to mutable components. Read moreSourceimpl<T> From<ComponentIdFor<'_, T>> for ComponentIdwhere
T: Component,
impl<T> From<ComponentIdFor<'_, T>> for ComponentIdwhere
T: Component,
Sourcefn from(to_component_id: ComponentIdFor<'_, T>) -> ComponentId
fn from(to_component_id: ComponentIdFor<'_, T>) -> ComponentId
Sourceimpl FromArg for &'static ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl FromArg for &'static ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl FromArg for &'static mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl FromArg for &'static mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl FromArg for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl FromArg for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl FromReflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl FromReflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<ComponentId>
fn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<ComponentId>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for &ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl GetOwnership for &ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl GetOwnership for &mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl GetOwnership for &mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl GetOwnership for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl GetOwnership for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourceimpl GetTypeRegistration for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl GetTypeRegistration for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl Hash for ComponentId
impl Hash for ComponentId
Sourceimpl IntoReturn for &ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl IntoReturn for &ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
&ComponentId: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
&ComponentId: 'into_return,
Sourceimpl IntoReturn for &mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl IntoReturn for &mut ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
&mut ComponentId: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
&mut ComponentId: 'into_return,
Sourceimpl IntoReturn for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl IntoReturn for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
ComponentId: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
ComponentId: 'into_return,
Sourceimpl Ord for ComponentId
impl Ord for ComponentId
Sourcefn cmp(&self, other: &ComponentId) -> Ordering
fn cmp(&self, other: &ComponentId) -> Ordering
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Sourceimpl PartialEq for ComponentId
impl PartialEq for ComponentId
Sourceimpl PartialOrd for ComponentId
impl PartialOrd for ComponentId
Sourceimpl PartialReflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl PartialReflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<ComponentId>) -> ReflectOwned
fn reflect_owned(self: Box<ComponentId>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<ComponentId>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<ComponentId>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<ComponentId>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<ComponentId>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn clone_value(&self) -> Box<dyn PartialReflect>
fn clone_value(&self) -> Box<dyn PartialReflect>
reflect_clone. To convert reflected values to dynamic ones, use to_dynamic.Self into its dynamic representation. Read moreSourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl Reflect for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn into_any(self: Box<ComponentId>) -> Box<dyn Any>
fn into_any(self: Box<ComponentId>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<ComponentId>) -> Box<dyn Reflect>
fn into_reflect(self: Box<ComponentId>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl SparseSetIndex for ComponentId
impl SparseSetIndex for ComponentId
Sourcefn sparse_set_index(&self) -> usize
fn sparse_set_index(&self) -> usize
Sourcefn get_sparse_set_index(value: usize) -> ComponentId
fn get_sparse_set_index(value: usize) -> ComponentId
Sourceimpl TriggerTargets for ComponentId
impl TriggerTargets for ComponentId
Sourceimpl TupleStruct for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl TupleStruct for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
Sourcefn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn Reflect.Sourcefn field_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn Reflect.Sourcefn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
fn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
Sourcefn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
fn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
DynamicTupleStruct from this tuple struct.Sourcefn clone_dynamic(&self) -> DynamicTupleStruct
fn clone_dynamic(&self) -> DynamicTupleStruct
to_dynamic_tuple_struct insteadDynamicTupleStruct.Sourcefn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
fn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
None if TypeInfo is not available.Sourceimpl TypePath for ComponentId
impl TypePath for ComponentId
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
Sourceimpl Typed for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl Typed for ComponentIdwhere
ComponentId: Any + Send + Sync,
usize: FromReflect + TypePath + MaybeTyped + RegisterForReflection,
impl Copy for ComponentId
impl Eq for ComponentId
impl StructuralPartialEq for ComponentId