EntityEquivalentHashSet

Struct EntityEquivalentHashSet 

pub struct EntityEquivalentHashSet<K>(/* private fields */)
where
    K: EntityEquivalent + Hash;
A HashSet pre-configured to use EntityHash hashing.

Implementations

Source

impl<K> EntityEquivalentHashSet<K>

pub const fn new() -> EntityEquivalentHashSet<K>

Creates an empty EntityEquivalentHashSet.

Equivalent to HashSet::with_hasher(EntityHash).

Examples found in repository?
examples/ecs/relationships.rs (line 161)
152    fn check_for_cycles(
153        // We want to check every entity for cycles
154        query_to_check: Query<Entity, With<Targeting>>,
155        // Fetch the names for easier debugging.
156        name_query: Query<&Name>,
157        // The targeting_query allows us to traverse the relationship graph.
158        targeting_query: Query<&Targeting>,
159    ) -> Result<(), TargetingCycle> {
160        for initial_entity in query_to_check.iter() {
161            let mut visited = EntityHashSet::new();
162            let mut targeting_name = name_query.get(initial_entity).unwrap().clone();
163            println!("Checking for cycles starting at {targeting_name}",);
164
165            // There's all sorts of methods like this; check the `Query` docs for more!
166            // This would also be easy to do by just manually checking the `Targeting` component,
167            // and calling `query.get(targeted_entity)` on the entity that it targets in a loop.
168            for targeting in targeting_query.iter_ancestors(initial_entity) {
169                let target_name = name_query.get(targeting).unwrap();
170                println!("{targeting_name} is targeting {target_name}",);
171                targeting_name = target_name.clone();
172
173                if !visited.insert(targeting) {
174                    return Err(TargetingCycle {
175                        initial_entity,
176                        visited,
177                    });
178                }
179            }
180        }
181
182        // If we've checked all the entities and haven't found a cycle, we're good!
183        Ok(())
184    }
pub fn with_capacity(n: usize) -> EntityEquivalentHashSet<K>

Creates an empty EntityEquivalentHashSet with the specified capacity.

Equivalent to HashSet::with_capacity_and_hasher(n, EntityHash).

pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

pub const fn from_hash_set( set: HashSet<K, EntityHash>, ) -> EntityEquivalentHashSet<K>

Constructs an EntityHashSet from an HashSet.

pub fn into_inner(self) -> HashSet<K, EntityHash>

Returns the inner HashSet.

pub fn drain(&mut self) -> Drain<'_, K> ⓘ

Clears the set, returning all elements in an iterator.

Equivalent to HashSet::drain.

pub fn iter(&self) -> Iter<'_, K> ⓘ

An iterator visiting all elements in arbitrary order. The iterator element type is &'a Entity.

Equivalent to HashSet::iter.

Examples found in repository?
examples/ecs/observers.rs (line 230)
221    fn get_nearby(&self, pos: Vec2) -> Vec<Entity> {
222        let tile = (
223            (pos.x / CELL_SIZE).floor() as i32,
224            (pos.y / CELL_SIZE).floor() as i32,
225        );
226        let mut nearby = Vec::new();
227        for x in -1..2 {
228            for y in -1..2 {
229                if let Some(mines) = self.map.get(&(tile.0 + x, tile.1 + y)) {
230                    nearby.extend(mines.iter());
231                }
232            }
233        }
234        nearby
235    }
More examples
Hide additional examples
examples/gltf/gltf_extension_animation_graph.rs (line 278)
257    fn on_scene_completed(
258        &mut self,
259        load_context: &mut LoadContext<'_>,
260        _scene: &gltf::Scene,
261        _world_root_id: Entity,
262        world: &mut World,
263    ) {
264        // Create an AnimationGraph from the desired clip
265        let (graph, index) = AnimationGraph::from_clip(self.clip.clone().unwrap());
266        // Store the animation graph as an asset with an arbitrary label
267        // We only have one graph, so this label will be unique
268        let graph_handle =
269            load_context.add_labeled_asset("MyAnimationGraphLabel".to_string(), graph);
270
271        // Create a component that stores a reference to our animation
272        let animation_to_play = AnimationToPlay {
273            graph_handle,
274            index,
275        };
276
277        // Insert the `AnimationToPlay` component on the first animation root
278        let mut entity = world.entity_mut(*self.animation_root_entities.iter().next().unwrap());
279        entity.insert(animation_to_play);
280    }
pub fn extract_if<F>(&mut self, f: F) -> ExtractIf<'_, K, F> ⓘ
where F: FnMut(&K) -> bool,

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

Equivalent to HashSet::extract_if.

Methods from Deref<Target = HashSet<K, EntityHash>>

pub fn capacity(&self) -> usize

Returns the number of elements the set can hold without reallocating.

Refer to capacity for further details.

Examples
let map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);
pub fn iter(&self) -> Iter<'_, T> ⓘ

An iterator visiting all elements in arbitrary order. The iterator element type is &'a T.

Refer to iter for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

for value in map.iter() {
    // "foo", "bar", "baz"
    // Note that the above order is not guaranteed
}
pub fn len(&self) -> usize

Returns the number of elements in the set.

Refer to len for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.len(), 0);

map.insert("foo");

assert_eq!(map.len(), 1);
pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

Refer to is_empty for further details.

Examples
let mut map = HashSet::new();

assert!(map.is_empty());

map.insert("foo");

assert!(!map.is_empty());
pub fn drain(&mut self) -> Drain<'_, T> ⓘ

Clears the set, returning all elements in an iterator.

Refer to drain for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

for value in map.drain() {
    // "foo", "bar", "baz"
    // Note that the above order is not guaranteed
}

assert!(map.is_empty());
pub fn retain<F>(&mut self, f: F)
where F: FnMut(&T) -> bool,

Retains only the elements specified by the predicate.

Refer to retain for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

map.retain(|value| *value == "baz");

assert_eq!(map.len(), 1);
pub fn extract_if<F>(&mut self, f: F) -> ExtractIf<'_, T, F> ⓘ
where F: FnMut(&T) -> bool,

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

Refer to extract_if for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

let extracted = map
    .extract_if(|value| *value == "baz")
    .collect::<Vec<_>>();

assert_eq!(map.len(), 2);
assert_eq!(extracted.len(), 1);
pub fn clear(&mut self)

Clears the set, removing all values.

Refer to clear for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

map.clear();

assert!(map.is_empty());
Examples found in repository?
examples/ui/navigation/directional_navigation.rs (line 335)
330fn process_inputs(
331    mut action_state: ResMut<ActionState>,
332    keyboard_input: Res<ButtonInput<KeyCode>>,
333    gamepad_input: Query<&Gamepad>,
334) {
335    action_state.pressed_actions.clear();
336
337    for action in DirectionalNavigationAction::variants() {
338        if keyboard_input.just_pressed(action.keycode()) {
339            action_state.pressed_actions.insert(action);
340        }
341    }
342
343    for gamepad in gamepad_input.iter() {
344        for action in DirectionalNavigationAction::variants() {
345            if gamepad.just_pressed(action.gamepad_button()) {
346                action_state.pressed_actions.insert(action);
347            }
348        }
349    }
350}
More examples
Hide additional examples
examples/ui/navigation/directional_navigation_overrides.rs (line 657)
652fn process_inputs(
653    mut action_state: ResMut<ActionState>,
654    keyboard_input: Res<ButtonInput<KeyCode>>,
655    gamepad_input: Query<&Gamepad>,
656) {
657    action_state.pressed_actions.clear();
658
659    for action in DirectionalNavigationAction::variants() {
660        if keyboard_input.just_pressed(action.keycode()) {
661            action_state.pressed_actions.insert(action);
662        }
663    }
664
665    for gamepad in gamepad_input.iter() {
666        for action in DirectionalNavigationAction::variants() {
667            if gamepad.just_pressed(action.gamepad_button()) {
668                action_state.pressed_actions.insert(action);
669            }
670        }
671    }
672}
examples/shader_advanced/custom_render_phase.rs (line 533)
528fn extract_camera_phases(
529    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
530    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
531    mut live_entities: Local<HashSet<RetainedViewEntity>>,
532) {
533    live_entities.clear();
534    for (main_entity, camera) in &cameras {
535        if !camera.is_active {
536            continue;
537        }
538        // This is the main camera, so we use the first subview index (0)
539        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
540
541        stencil_phases.prepare_for_new_frame(retained_view_entity);
542        live_entities.insert(retained_view_entity);
543    }
544
545    // Clear out all dead views.
546    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
547}
examples/2d/mesh2d_manual.rs (line 497)
492fn extract_colored_mesh2d_camera_phases(
493    mut colored_mesh2d_render_phases: ResMut<ViewSortedRenderPhases<TransparentColoredMesh2d>>,
494    cameras_2d: Extract<Query<(Entity, &Camera), With<Camera2d>>>,
495    mut live_entities: Local<HashSet<RetainedViewEntity>>,
496) {
497    live_entities.clear();
498
499    for (main_entity, camera) in &cameras_2d {
500        if !camera.is_active {
501            continue;
502        }
503
504        // This is the main 2D camera, so we use the first subview index (0).
505        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
506
507        colored_mesh2d_render_phases.prepare_for_new_frame(retained_view_entity);
508
509        live_entities.insert(retained_view_entity);
510    }
511
512    // Clear out all dead views.
513    colored_mesh2d_render_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
514}
pub fn hasher(&self) -> &S

Returns a reference to the set’s BuildHasher.

Refer to hasher for further details.

pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the HashSet. The collection may reserve more space to avoid frequent reallocations.

Refer to reserve for further details.

Examples
let mut map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);

map.reserve(10);

assert!(map.capacity() - map.len() >= 10);
pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve capacity for at least additional more elements to be inserted in the given HashSet<K,V>. The collection may reserve more space to avoid frequent reallocations.

Refer to try_reserve for further details.

Examples
let mut map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);

map.try_reserve(10).expect("Out of Memory!");

assert!(map.capacity() - map.len() >= 10);
pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the set as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Refer to shrink_to_fit for further details.

Examples
let mut map = HashSet::with_capacity(5);

map.insert("foo");
map.insert("bar");
map.insert("baz");

assert!(map.capacity() >= 5);

map.shrink_to_fit();

assert_eq!(map.capacity(), 3);
pub fn shrink_to(&mut self, min_capacity: usize)

Shrinks the capacity of the set with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Refer to shrink_to for further details.

pub fn difference<'a>( &'a self, other: &'a HashSet<T, S>, ) -> Difference<'a, T, S> ⓘ

Visits the values representing the difference, i.e., the values that are in self but not in other.

Refer to difference for further details.

pub fn symmetric_difference<'a>( &'a self, other: &'a HashSet<T, S>, ) -> SymmetricDifference<'a, T, S> ⓘ

Visits the values representing the symmetric difference, i.e., the values that are in self or in other but not in both.

Refer to symmetric_difference for further details.

pub fn intersection<'a>( &'a self, other: &'a HashSet<T, S>, ) -> Intersection<'a, T, S> ⓘ

Visits the values representing the intersection, i.e., the values that are both in self and other.

Refer to intersection for further details.

pub fn union<'a>(&'a self, other: &'a HashSet<T, S>) -> Union<'a, T, S> ⓘ

Visits the values representing the union, i.e., all the values in self or other, without duplicates.

Refer to union for further details.

pub fn contains<Q>(&self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Returns true if the set contains a value.

Refer to contains for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.contains("foo"));
Examples found in repository?
examples/gltf/gltf_extension_animation_graph.rs (line 251)
245    fn on_gltf_node(
246        &mut self,
247        _load_context: &mut LoadContext<'_>,
248        gltf_node: &gltf::Node,
249        entity: &mut EntityWorldMut,
250    ) {
251        if self.animation_root_indices.contains(&gltf_node.index()) {
252            self.animation_root_entities.insert(entity.id());
253        }
254    }
More examples
Hide additional examples
examples/shader_advanced/custom_render_phase.rs (line 546)
528fn extract_camera_phases(
529    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
530    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
531    mut live_entities: Local<HashSet<RetainedViewEntity>>,
532) {
533    live_entities.clear();
534    for (main_entity, camera) in &cameras {
535        if !camera.is_active {
536            continue;
537        }
538        // This is the main camera, so we use the first subview index (0)
539        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
540
541        stencil_phases.prepare_for_new_frame(retained_view_entity);
542        live_entities.insert(retained_view_entity);
543    }
544
545    // Clear out all dead views.
546    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
547}
examples/2d/mesh2d_manual.rs (line 513)
492fn extract_colored_mesh2d_camera_phases(
493    mut colored_mesh2d_render_phases: ResMut<ViewSortedRenderPhases<TransparentColoredMesh2d>>,
494    cameras_2d: Extract<Query<(Entity, &Camera), With<Camera2d>>>,
495    mut live_entities: Local<HashSet<RetainedViewEntity>>,
496) {
497    live_entities.clear();
498
499    for (main_entity, camera) in &cameras_2d {
500        if !camera.is_active {
501            continue;
502        }
503
504        // This is the main 2D camera, so we use the first subview index (0).
505        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
506
507        colored_mesh2d_render_phases.prepare_for_new_frame(retained_view_entity);
508
509        live_entities.insert(retained_view_entity);
510    }
511
512    // Clear out all dead views.
513    colored_mesh2d_render_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
514}
examples/ui/images/ui_texture_atlas_slice.rs (line 46)
27fn button_system(
28    mut buttons: Query<
29        (
30            Entity,
31            Option<Ref<Pressed>>,
32            Ref<Hovered>,
33            &mut ImageNode,
34            &Children,
35        ),
36        With<Button>,
37    >,
38    mut removed_pressed: RemovedComponents<Pressed>,
39    mut text_query: Query<&mut Text>,
40) {
41    // Buttons that had `Pressed` removed this frame; change detection does not report removals.
42    let just_unpressed: HashSet<Entity> = removed_pressed.read().collect();
43    for (entity, pressed, hovered, mut image, children) in &mut buttons {
44        let changed = hovered.is_changed()
45            || pressed.as_ref().is_some_and(Ref::is_changed)
46            || just_unpressed.contains(&entity);
47        if changed {
48            set_button_style(
49                pressed.is_some(),
50                hovered.get(),
51                &mut image,
52                children,
53                &mut text_query,
54            );
55        }
56    }
57}
examples/ui/images/ui_texture_slice.rs (line 50)
31fn button_system(
32    mut buttons: Query<
33        (
34            Entity,
35            Option<Ref<Pressed>>,
36            Ref<Hovered>,
37            &mut ImageNode,
38            &Children,
39        ),
40        With<Button>,
41    >,
42    mut removed_pressed: RemovedComponents<Pressed>,
43    mut text_query: Query<&mut Text>,
44) {
45    // Buttons that had `Pressed` removed this frame; change detection does not report removals.
46    let just_unpressed: HashSet<Entity> = removed_pressed.read().collect();
47    for (entity, pressed, hovered, mut image, children) in &mut buttons {
48        let changed = hovered.is_changed()
49            || pressed.as_ref().is_some_and(Ref::is_changed)
50            || just_unpressed.contains(&entity);
51        if changed {
52            set_button_style(
53                pressed.is_some(),
54                hovered.get(),
55                &mut image,
56                children,
57                &mut text_query,
58            );
59        }
60    }
61}
examples/testbed/helpers.rs (line 26)
12pub fn switch_scene_in_ci<Scene: States + FreelyMutableState + Next>(
13    mut ci_config: ResMut<CiTestingConfig>,
14    scene: Res<State<Scene>>,
15    mut next_scene: ResMut<NextState<Scene>>,
16    mut scenes_visited: Local<HashSet<Scene>>,
17    frame_count: Res<FrameCount>,
18    captured: RemovedComponents<Captured>,
19) {
20    if scene.is_changed() {
21        // Changed scene! trigger a screenshot in 100 frames
22        ci_config.events.push(CiTestingEventOnFrame(
23            frame_count.0 + 100,
24            CiTestingEvent::NamedScreenshot(format!("{:?}", scene.get())),
25        ));
26        if scenes_visited.contains(scene.get()) {
27            // Exit once all scenes have been screenshotted
28            ci_config.events.push(CiTestingEventOnFrame(
29                frame_count.0 + 1,
30                CiTestingEvent::AppExit,
31            ));
32        }
33        return;
34    }
35
36    if !captured.is_empty() {
37        // Screenshot taken! Switch to the next scene
38        scenes_visited.insert(scene.get().clone());
39        next_scene.set(scene.get().next());
40    }
41}
pub fn get<Q>(&self, value: &Q) -> Option<&T>
where Q: Hash + Equivalent<T> + ?Sized,

Returns a reference to the value in the set, if any, that is equal to the given value.

Refer to get for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.get("foo"), Some(&"foo"));
pub fn get_or_insert(&mut self, value: T) -> &T

Inserts the given value into the set if it is not present, then returns a reference to the value in the set.

Refer to get_or_insert for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.get_or_insert("foo"), &"foo");
pub fn get_or_insert_with<Q, F>(&mut self, value: &Q, f: F) -> &T
where Q: Hash + Equivalent<T> + ?Sized, F: FnOnce(&Q) -> T,

Inserts a value computed from f into the set if the given value is not present, then returns a reference to the value in the set.

Refer to get_or_insert_with for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.get_or_insert_with(&"foo", |_| "foo"), &"foo");
pub fn entry(&mut self, value: T) -> Entry<'_, T, S>

Gets the given value’s corresponding entry in the set for in-place manipulation.

Refer to entry for further details.

Examples
let mut map = HashSet::new();

let value = map.entry("foo").or_insert();
pub fn is_disjoint(&self, other: &HashSet<T, S>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

Refer to is_disjoint for further details.

pub fn is_subset(&self, other: &HashSet<T, S>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

Refer to is_subset for further details.

pub fn is_superset(&self, other: &HashSet<T, S>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

Refer to is_superset for further details.

pub fn insert(&mut self, value: T) -> bool

Adds a value to the set.

Refer to insert for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.contains("foo"));
Examples found in repository?
examples/gltf/gltf_extension_animation_graph.rs (line 252)
245    fn on_gltf_node(
246        &mut self,
247        _load_context: &mut LoadContext<'_>,
248        gltf_node: &gltf::Node,
249        entity: &mut EntityWorldMut,
250    ) {
251        if self.animation_root_indices.contains(&gltf_node.index()) {
252            self.animation_root_entities.insert(entity.id());
253        }
254    }
More examples
Hide additional examples
examples/ecs/observers.rs (line 148)
142fn on_add_mine(add: On<Add<Mine>>, query: Query<&Mine>, mut index: ResMut<SpatialIndex>) {
143    let mine = query.get(add.entity).unwrap();
144    let tile = (
145        (mine.pos.x / CELL_SIZE).floor() as i32,
146        (mine.pos.y / CELL_SIZE).floor() as i32,
147    );
148    index.map.entry(tile).or_default().insert(add.entity);
149}
examples/shader_advanced/manual_material.rs (line 320)
311fn extract_image_materials_needing_specialization(
312    entities_needing_specialization: Extract<Res<EntitiesNeedingSpecialization<ImageMaterial>>>,
313    mut dirty_specializations: ResMut<DirtySpecializations>,
314) {
315    // Drain the list of entities needing specialization from the main world
316    // into the render-world `DirtySpecializations` table.
317    for entity in entities_needing_specialization.changed.iter() {
318        dirty_specializations
319            .changed_renderables
320            .insert(MainEntity::from(*entity));
321    }
322}
323
324/// A system that adds entities that were judged to need their specializations
325/// removed to the appropriate table in [`DirtySpecializations`].
326fn extract_image_materials_that_need_specializations_removed(
327    entities_needing_specialization: Extract<Res<EntitiesNeedingSpecialization<ImageMaterial>>>,
328    mut dirty_specializations: ResMut<DirtySpecializations>,
329) {
330    for entity in entities_needing_specialization.removed.iter() {
331        dirty_specializations
332            .removed_renderables
333            .insert(MainEntity::from(*entity));
334    }
335}
examples/ui/navigation/directional_navigation.rs (line 339)
330fn process_inputs(
331    mut action_state: ResMut<ActionState>,
332    keyboard_input: Res<ButtonInput<KeyCode>>,
333    gamepad_input: Query<&Gamepad>,
334) {
335    action_state.pressed_actions.clear();
336
337    for action in DirectionalNavigationAction::variants() {
338        if keyboard_input.just_pressed(action.keycode()) {
339            action_state.pressed_actions.insert(action);
340        }
341    }
342
343    for gamepad in gamepad_input.iter() {
344        for action in DirectionalNavigationAction::variants() {
345            if gamepad.just_pressed(action.gamepad_button()) {
346                action_state.pressed_actions.insert(action);
347            }
348        }
349    }
350}
examples/ui/navigation/directional_navigation_overrides.rs (line 661)
652fn process_inputs(
653    mut action_state: ResMut<ActionState>,
654    keyboard_input: Res<ButtonInput<KeyCode>>,
655    gamepad_input: Query<&Gamepad>,
656) {
657    action_state.pressed_actions.clear();
658
659    for action in DirectionalNavigationAction::variants() {
660        if keyboard_input.just_pressed(action.keycode()) {
661            action_state.pressed_actions.insert(action);
662        }
663    }
664
665    for gamepad in gamepad_input.iter() {
666        for action in DirectionalNavigationAction::variants() {
667            if gamepad.just_pressed(action.gamepad_button()) {
668                action_state.pressed_actions.insert(action);
669            }
670        }
671    }
672}
examples/shader_advanced/custom_render_phase.rs (line 542)
528fn extract_camera_phases(
529    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
530    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
531    mut live_entities: Local<HashSet<RetainedViewEntity>>,
532) {
533    live_entities.clear();
534    for (main_entity, camera) in &cameras {
535        if !camera.is_active {
536            continue;
537        }
538        // This is the main camera, so we use the first subview index (0)
539        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
540
541        stencil_phases.prepare_for_new_frame(retained_view_entity);
542        live_entities.insert(retained_view_entity);
543    }
544
545    // Clear out all dead views.
546    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
547}
548
549/// A resource that stores meshes that couldn't be specialized yet because their
550/// materials hadn't loaded.
551///
552/// See the documentation for [`PendingQueues`] for more information.
553#[derive(Default, Deref, DerefMut, Resource)]
554struct PendingCustomMeshQueues(pub PendingQueues);
555
556// This is a very important step when writing a custom phase.
557//
558// This system determines which meshes will be added to the phase.
559fn queue_custom_meshes(
560    custom_draw_functions: Res<DrawFunctions<Stencil3d>>,
561    mut pipelines: ResMut<SpecializedMeshPipelines<StencilPipeline>>,
562    pipeline_cache: Res<PipelineCache>,
563    custom_draw_pipeline: Res<StencilPipeline>,
564    render_meshes: Res<RenderAssets<RenderMesh>>,
565    render_mesh_instances: Res<RenderMeshInstances>,
566    maybe_batched_instance_buffers: Option<
567        Res<BatchedInstanceBuffers<MeshUniform, MeshInputUniform>>,
568    >,
569    mut custom_render_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
570    mut views: Query<(&ExtractedView, &RenderVisibleEntities)>,
571    view_key_cache: Res<ViewKeyCache>,
572    dirty_specializations: Res<DirtySpecializations>,
573    mut pending_custom_mesh_queues: ResMut<PendingCustomMeshQueues>,
574    has_marker: Query<(), With<DrawStencil>>,
575    mut mesh_instances_queued_this_iteration_scratch_space: Local<MainEntityHashSet>,
576) {
577    for (view, visible_entities) in &mut views {
578        let Some(custom_phase) = custom_render_phases.get_mut(&view.retained_view_entity) else {
579            continue;
580        };
581        let draw_custom = custom_draw_functions.read().id::<DrawMesh3dStencil>();
582
583        let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
584            continue;
585        };
586
587        // Since our phase can work on any 3d mesh we can reuse the default mesh 3d filter
588        let Some(render_visible_mesh_entities) = visible_entities.get::<Mesh3d>() else {
589            continue;
590        };
591
592        let view_pending_custom_mesh_queues =
593            pending_custom_mesh_queues.prepare_for_new_frame(view.retained_view_entity);
594
595        // First, remove meshes that need to be respecialized, and those that were removed, from the bins.
596        for &main_entity in dirty_specializations
597            .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
598        {
599            custom_phase.remove(Entity::PLACEHOLDER, main_entity);
600        }
601
602        for (render_entity, visible_entity) in dirty_specializations.iter_to_queue(
603            view.retained_view_entity,
604            render_visible_mesh_entities,
605            &view_pending_custom_mesh_queues.prev_frame,
606            &mut mesh_instances_queued_this_iteration_scratch_space,
607        ) {
608            // We only want meshes with the marker component to be queued to our phase.
609            if has_marker.get(*render_entity).is_err() {
610                continue;
611            }
612            let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*visible_entity)
613            else {
614                // We couldn't fetch the mesh, probably because it hasn't been
615                // loaded yet. Add the entity to the list of pending custom mesh
616                // queues and bail.
617                view_pending_custom_mesh_queues
618                    .current_frame
619                    .insert((*render_entity, *visible_entity));
620                continue;
621            };
622            let Some(mesh) = render_meshes.get(mesh_instance.mesh_asset_id()) else {
623                continue;
624            };
625
626            // Specialize the key for the current mesh entity
627            // For this example we only specialize based on the mesh topology
628            // but you could have more complex keys and that's where you'd need to create those keys
629            let mut mesh_key = view_key;
630            mesh_key |= MeshPipelineKey::from_primitive_topology_and_strip_index(
631                mesh.primitive_topology(),
632                mesh.index_format(),
633            );
634
635            let pipeline_id = pipelines.specialize(
636                &pipeline_cache,
637                &custom_draw_pipeline,
638                mesh_key,
639                &mesh.layout,
640            );
641            let pipeline_id = match pipeline_id {
642                Ok(id) => id,
643                Err(err) => {
644                    error!("{}", err);
645                    continue;
646                }
647            };
648            // At this point we have all the data we need to create a phase item and add it to our
649            // phase
650            custom_phase.add_retained(Stencil3d {
651                sorting_info: TransparentSortingInfo3d::Sorted {
652                    mesh_center: pbr::get_mesh_instance_world_from_local(
653                        *visible_entity,
654                        mesh_instance.current_uniform_index,
655                        &render_mesh_instances,
656                        maybe_batched_instance_buffers.as_deref(),
657                    )
658                    .transform_point3(
659                        render_meshes
660                            .get(mesh_instance.mesh_asset_id())
661                            .unwrap()
662                            .aabb_center,
663                    ),
664                    depth_bias: 0.0,
665                },
666                distance: FloatOrd(0.0),
667                entity: (Entity::PLACEHOLDER, *visible_entity),
668                pipeline: pipeline_id,
669                draw_function: draw_custom,
670                // Sorted phase items aren't batched
671                batch_range: 0..1,
672                extra_index: PhaseItemExtraIndex::None,
673                indexed: mesh.indexed(),
674            });
675        }
676    }
677}
pub fn replace(&mut self, value: T) -> Option<T>

Adds a value to the set, replacing the existing value, if any, that is equal to the given one. Returns the replaced value.

Refer to replace for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.replace("foo"), Some("foo"));
pub fn remove<Q>(&mut self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Removes a value from the set. Returns whether the value was present in the set.

Refer to remove for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.remove("foo"));

assert!(map.is_empty());
Examples found in repository?
examples/ecs/observers.rs (line 159)
152fn on_remove_mine(remove: On<Remove<Mine>>, query: Query<&Mine>, mut index: ResMut<SpatialIndex>) {
153    let mine = query.get(remove.entity).unwrap();
154    let tile = (
155        (mine.pos.x / CELL_SIZE).floor() as i32,
156        (mine.pos.y / CELL_SIZE).floor() as i32,
157    );
158    index.map.entry(tile).and_modify(|set| {
159        set.remove(&remove.entity);
160    });
161}
pub fn take<Q>(&mut self, value: &Q) -> Option<T>
where Q: Hash + Equivalent<T> + ?Sized,

Removes and returns the value in the set, if any, that is equal to the given one.

Refer to take for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.take("foo"), Some("foo"));

assert!(map.is_empty());
pub fn allocation_size(&self) -> usize

Returns the total amount of memory allocated internally by the hash set, in bytes.

Refer to allocation_size for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.allocation_size(), 0);

map.insert("foo");

assert!(map.allocation_size() >= size_of::<&'static str>());
pub unsafe fn insert_unique_unchecked(&mut self, value: T) -> &T

Insert a value the set without checking if the value already exists in the set.

Refer to insert_unique_unchecked for further details.

Safety

This operation is safe if a value does not exist in the set.

However, if a value exists in the set already, the behavior is unspecified: this operation may panic, loop forever, or any following operation with the set may panic, loop forever or return arbitrary result.

That said, this operation (and following operations) are guaranteed to not violate memory safety.

However this operation is still unsafe because the resulting HashSet may be passed to unsafe code which does expect the set to behave correctly, and would cause unsoundness as a result.

Methods from Deref<Target = HashSet<T, S>>

pub fn capacity(&self) -> usize

Returns the number of elements the set can hold without reallocating.

Examples
use hashbrown::HashSet;
let set: HashSet<i32> = HashSet::with_capacity(100);
assert!(set.capacity() >= 100);
pub fn iter(&self) -> Iter<'_, T> ⓘ

An iterator visiting all elements in arbitrary order. The iterator element type is &'a T.

Examples
use hashbrown::HashSet;
let mut set = HashSet::new();
set.insert("a");
set.insert("b");

// Will print in an arbitrary order.
for x in set.iter() {
    println!("{}", x);
}
pub fn len(&self) -> usize

Returns the number of elements in the set.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
assert_eq!(v.len(), 0);
v.insert(1);
assert_eq!(v.len(), 1);
pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
assert!(v.is_empty());
v.insert(1);
assert!(!v.is_empty());
pub fn drain(&mut self) -> Drain<'_, T, A> ⓘ

Clears the set, returning all elements in an iterator.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert!(!set.is_empty());

// print 1, 2, 3 in an arbitrary order
for i in set.drain() {
    println!("{}", i);
}

assert!(set.is_empty());
pub fn retain<F>(&mut self, f: F)
where F: FnMut(&T) -> bool,

Retains only the elements specified by the predicate.

In other words, remove all elements e such that f(&e) returns false.

Examples
use hashbrown::HashSet;

let xs = [1,2,3,4,5,6];
let mut set: HashSet<i32> = xs.into_iter().collect();
set.retain(|&k| k % 2 == 0);
assert_eq!(set.len(), 3);
pub fn extract_if<F>(&mut self, f: F) -> ExtractIf<'_, T, F, A> ⓘ
where F: FnMut(&T) -> bool,

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

In other words, move all elements e such that f(&e) returns true out into another iterator.

If the returned ExtractIf is not exhausted, e.g. because it is dropped without iterating or the iteration short-circuits, then the remaining elements will be retained. Use retain() with a negated predicate if you do not need the returned iterator.

Examples
use hashbrown::HashSet;

let mut set: HashSet<i32> = (0..8).collect();
let drained: HashSet<i32> = set.extract_if(|v| v % 2 == 0).collect();

let mut evens = drained.into_iter().collect::<Vec<_>>();
let mut odds = set.into_iter().collect::<Vec<_>>();
evens.sort();
odds.sort();

assert_eq!(evens, vec![0, 2, 4, 6]);
assert_eq!(odds, vec![1, 3, 5, 7]);
pub fn clear(&mut self)

Clears the set, removing all values.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
v.insert(1);
v.clear();
assert!(v.is_empty());
pub fn allocator(&self) -> &A

Returns a reference to the underlying allocator.

pub fn hasher(&self) -> &S

Returns a reference to the set’s BuildHasher.

Examples
use hashbrown::HashSet;
use hashbrown::DefaultHashBuilder;

let hasher = DefaultHashBuilder::default();
let set: HashSet<i32> = HashSet::with_hasher(hasher);
let hasher: &DefaultHashBuilder = set.hasher();
pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the HashSet. The collection may reserve more space to avoid frequent reallocations.

Panics

Panics if the new capacity exceeds isize::MAX bytes and abort the program in case of allocation error. Use try_reserve instead if you want to handle memory allocation failure.

Examples
use hashbrown::HashSet;
let mut set: HashSet<i32> = HashSet::new();
set.reserve(10);
assert!(set.capacity() >= 10);
pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve capacity for at least additional more elements to be inserted in the given HashSet<K,V>. The collection may reserve more space to avoid frequent reallocations.

Errors

If the capacity overflows, or the allocator reports a failure, then an error is returned.

Examples
use hashbrown::HashSet;
let mut set: HashSet<i32> = HashSet::new();
set.try_reserve(10).expect("why is the test harness OOMing on 10 bytes?");
pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the set as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Examples
use hashbrown::HashSet;

let mut set = HashSet::with_capacity(100);
set.insert(1);
set.insert(2);
assert!(set.capacity() >= 100);
set.shrink_to_fit();
assert!(set.capacity() >= 2);
pub fn shrink_to(&mut self, min_capacity: usize)

Shrinks the capacity of the set with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Panics if the current capacity is smaller than the supplied minimum capacity.

Examples
use hashbrown::HashSet;

let mut set = HashSet::with_capacity(100);
set.insert(1);
set.insert(2);
assert!(set.capacity() >= 100);
set.shrink_to(10);
assert!(set.capacity() >= 10);
set.shrink_to(0);
assert!(set.capacity() >= 2);
pub fn difference<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> Difference<'a, T, S, A> ⓘ

Visits the values representing the difference, i.e., the values that are in self but not in other.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Can be seen as `a - b`.
for x in a.difference(&b) {
    println!("{}", x); // Print 1
}

let diff: HashSet<_> = a.difference(&b).collect();
assert_eq!(diff, [1].iter().collect());

// Note that difference is not symmetric,
// and `b - a` means something else:
let diff: HashSet<_> = b.difference(&a).collect();
assert_eq!(diff, [4].iter().collect());
pub fn symmetric_difference<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> SymmetricDifference<'a, T, S, A> ⓘ

Visits the values representing the symmetric difference, i.e., the values that are in self or in other but not in both.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 1, 4 in arbitrary order.
for x in a.symmetric_difference(&b) {
    println!("{}", x);
}

let diff1: HashSet<_> = a.symmetric_difference(&b).collect();
let diff2: HashSet<_> = b.symmetric_difference(&a).collect();

assert_eq!(diff1, diff2);
assert_eq!(diff1, [1, 4].iter().collect());
pub fn intersection<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> Intersection<'a, T, S, A> ⓘ

Visits the values representing the intersection, i.e., the values that are both in self and other.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 2, 3 in arbitrary order.
for x in a.intersection(&b) {
    println!("{}", x);
}

let intersection: HashSet<_> = a.intersection(&b).collect();
assert_eq!(intersection, [2, 3].iter().collect());
pub fn union<'a>(&'a self, other: &'a HashSet<T, S, A>) -> Union<'a, T, S, A> ⓘ

Visits the values representing the union, i.e., all the values in self or other, without duplicates.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 1, 2, 3, 4 in arbitrary order.
for x in a.union(&b) {
    println!("{}", x);
}

let union: HashSet<_> = a.union(&b).collect();
assert_eq!(union, [1, 2, 3, 4].iter().collect());
pub fn contains<Q>(&self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Returns true if the set contains a value.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.contains(&1), true);
assert_eq!(set.contains(&4), false);
pub fn get<Q>(&self, value: &Q) -> Option<&T>
where Q: Hash + Equivalent<T> + ?Sized,

Returns a reference to the value in the set, if any, that is equal to the given value.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.get(&2), Some(&2));
assert_eq!(set.get(&4), None);
pub fn get_or_insert(&mut self, value: T) -> &T

Inserts the given value into the set if it is not present, then returns a reference to the value in the set.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.len(), 3);
assert_eq!(set.get_or_insert(2), &2);
assert_eq!(set.get_or_insert(100), &100);
assert_eq!(set.len(), 4); // 100 was inserted
pub fn get_or_insert_with<Q, F>(&mut self, value: &Q, f: F) -> &T
where Q: Hash + Equivalent<T> + ?Sized, F: FnOnce(&Q) -> T,

Inserts a value computed from f into the set if the given value is not present, then returns a reference to the value in the set.

Examples
use hashbrown::HashSet;

let mut set: HashSet<String> = ["cat", "dog", "horse"]
    .iter().map(|&pet| pet.to_owned()).collect();

assert_eq!(set.len(), 3);
for &pet in &["cat", "dog", "fish"] {
    let value = set.get_or_insert_with(pet, str::to_owned);
    assert_eq!(value, pet);
}
assert_eq!(set.len(), 4); // a new "fish" was inserted

The following example will panic because the new value doesn’t match.

ⓘ
let mut set = hashbrown::HashSet::new();
set.get_or_insert_with("rust", |_| String::new());
pub fn entry(&mut self, value: T) -> Entry<'_, T, S, A>

Gets the given value’s corresponding entry in the set for in-place manipulation.

Examples
use hashbrown::HashSet;
use hashbrown::hash_set::Entry::*;

let mut singles = HashSet::new();
let mut dupes = HashSet::new();

for ch in "a short treatise on fungi".chars() {
    if let Vacant(dupe_entry) = dupes.entry(ch) {
        // We haven't already seen a duplicate, so
        // check if we've at least seen it once.
        match singles.entry(ch) {
            Vacant(single_entry) => {
                // We found a new character for the first time.
                single_entry.insert();
            }
            Occupied(single_entry) => {
                // We've already seen this once, "move" it to dupes.
                single_entry.remove();
                dupe_entry.insert();
            }
        }
    }
}

assert!(!singles.contains(&'t') && dupes.contains(&'t'));
assert!(singles.contains(&'u') && !dupes.contains(&'u'));
assert!(!singles.contains(&'v') && !dupes.contains(&'v'));
pub fn is_disjoint(&self, other: &HashSet<T, S, A>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

Examples
use hashbrown::HashSet;

let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let mut b = HashSet::new();

assert_eq!(a.is_disjoint(&b), true);
b.insert(4);
assert_eq!(a.is_disjoint(&b), true);
b.insert(1);
assert_eq!(a.is_disjoint(&b), false);
pub fn is_subset(&self, other: &HashSet<T, S, A>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

Examples
use hashbrown::HashSet;

let sup: HashSet<_> = [1, 2, 3].into_iter().collect();
let mut set = HashSet::new();

assert_eq!(set.is_subset(&sup), true);
set.insert(2);
assert_eq!(set.is_subset(&sup), true);
set.insert(4);
assert_eq!(set.is_subset(&sup), false);
pub fn is_superset(&self, other: &HashSet<T, S, A>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

Examples
use hashbrown::HashSet;

let sub: HashSet<_> = [1, 2].into_iter().collect();
let mut set = HashSet::new();

assert_eq!(set.is_superset(&sub), false);

set.insert(0);
set.insert(1);
assert_eq!(set.is_superset(&sub), false);

set.insert(2);
assert_eq!(set.is_superset(&sub), true);
pub fn insert(&mut self, value: T) -> bool

Adds a value to the set.

If the set did not have this value present, true is returned.

If the set did have this value present, false is returned.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();

assert_eq!(set.insert(2), true);
assert_eq!(set.insert(2), false);
assert_eq!(set.len(), 1);
pub unsafe fn insert_unique_unchecked(&mut self, value: T) -> &T

Insert a value the set without checking if the value already exists in the set.

This operation is faster than regular insert, because it does not perform lookup before insertion.

This operation is useful during initial population of the set. For example, when constructing a set from another set, we know that values are unique.

Safety

This operation is safe if a value does not exist in the set.

However, if a value exists in the set already, the behavior is unspecified: this operation may panic, loop forever, or any following operation with the set may panic, loop forever or return arbitrary result.

That said, this operation (and following operations) are guaranteed to not violate memory safety.

However this operation is still unsafe because the resulting HashSet may be passed to unsafe code which does expect the set to behave correctly, and would cause unsoundness as a result.

pub fn replace(&mut self, value: T) -> Option<T>

Adds a value to the set, replacing the existing value, if any, that is equal to the given one. Returns the replaced value.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();
set.insert(Vec::<i32>::new());

assert_eq!(set.get(&[][..]).unwrap().capacity(), 0);
set.replace(Vec::with_capacity(10));
assert_eq!(set.get(&[][..]).unwrap().capacity(), 10);
pub fn remove<Q>(&mut self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Removes a value from the set. Returns whether the value was present in the set.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();

set.insert(2);
assert_eq!(set.remove(&2), true);
assert_eq!(set.remove(&2), false);
pub fn take<Q>(&mut self, value: &Q) -> Option<T>
where Q: Hash + Equivalent<T> + ?Sized,

Removes and returns the value in the set, if any, that is equal to the given one.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.take(&2), Some(2));
assert_eq!(set.take(&2), None);
pub fn allocation_size(&self) -> usize

Returns the total amount of memory allocated internally by the hash set, in bytes.

The returned number is informational only. It is intended to be primarily used for memory profiling.