pub struct Entities { /* private fields */ }Expand description
Entities tracks all known EntityIndexs and their metadata.
This is like a base table of information all entities have.
Implementations
Sourceimpl Entities
impl Entities
Sourcepub fn get_spawned(
&self,
entity: Entity,
) -> Result<EntityLocation, EntityNotSpawnedError>
pub fn get_spawned( &self, entity: Entity, ) -> Result<EntityLocation, EntityNotSpawnedError>
Returns the EntityLocation of an Entity if it is valid and spawned.
This will return an error if the EntityGeneration of this entity has passed or if the EntityIndex is not spawned.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn get(
&self,
entity: Entity,
) -> Result<Option<EntityLocation>, InvalidEntityError>
pub fn get( &self, entity: Entity, ) -> Result<Option<EntityLocation>, InvalidEntityError>
Returns the EntityLocation of an Entity if it is valid.
The location will be None if the entity is not spawned.
If you expect the entity to be spawned, use get_spawned.
This will fail if the Entity is not valid (ex: the generation is mismatched).
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn resolve_from_index(&self, index: EntityIndex) -> Entity
pub fn resolve_from_index(&self, index: EntityIndex) -> Entity
Get the Entity for the given EntityIndex.
Note that this entity may not be spawned yet.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn is_index_spawned(&self, index: EntityIndex) -> bool
pub fn is_index_spawned(&self, index: EntityIndex) -> bool
Returns whether the entity at this index is spawned or not.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn contains(&self, entity: Entity) -> bool
pub fn contains(&self, entity: Entity) -> bool
Returns true if the entity is valid. This will return true for entities that are valid but have not been spawned.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn contains_spawned(&self, entity: Entity) -> bool
pub fn contains_spawned(&self, entity: Entity) -> bool
Returns true if the entity is valid and is spawned.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn check_can_spawn_at(&self, entity: Entity) -> Result<(), SpawnError>
pub fn check_can_spawn_at(&self, entity: Entity) -> Result<(), SpawnError>
Provides information regarding if entity may be safely spawned.
This can error if the entity is invalid or is already spawned.
See the module docs for a full explanation of these ids, entity life cycles, and the meaning of this result.
Sourcepub fn entity_get_spawned_or_despawned_by(
&self,
entity: Entity,
) -> MaybeLocation<Option<&'static Location<'static>>>
pub fn entity_get_spawned_or_despawned_by( &self, entity: Entity, ) -> MaybeLocation<Option<&'static Location<'static>>>
Try to get the source code location from which this entity has last been spawned or despawned.
Returns None if the entity does not exist or has never been construced/despawned.
Sourcepub fn entity_get_spawn_or_despawn_tick(&self, entity: Entity) -> Option<Tick>
pub fn entity_get_spawn_or_despawn_tick(&self, entity: Entity) -> Option<Tick>
Try to get the Tick at which this entity has last been spawned or despawned.
Returns None if the entity does not exist or has never been construced/despawned.
Sourcepub fn len(&self) -> u32
pub fn len(&self) -> u32
The count of currently allocated entity indices.
For information on active entities, see Self::count_spawned.
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Checks if any entity has been declared.
For information on active entities, see Self::any_spawned.
Sourcepub fn count_spawned(&self) -> u32
pub fn count_spawned(&self) -> u32
Counts the number of entity indices currently spawned. See the module docs for a more precise explanation of what spawning means. Be aware that this is O(n) and is intended only to be used as a diagnostic for tests.
Sourcepub fn any_spawned(&self) -> bool
pub fn any_spawned(&self) -> bool
Returns true if there are any entity indices currently spawned. See the module docs for a more precise explanation of what spawning means.