pub struct EntityLocation {
pub archetype_id: ArchetypeId,
pub archetype_row: ArchetypeRow,
pub table_id: TableId,
pub table_row: TableRow,
}Expand description
A location of an entity in an archetype.
Fields
archetype_id: ArchetypeIdarchetype_row: ArchetypeRowtable_id: TableIdtable_row: TableRowTrait Implementations
Sourceimpl Clone for EntityLocation
impl Clone for EntityLocation
Sourcefn clone(&self) -> EntityLocation
fn clone(&self) -> EntityLocation
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for EntityLocation
impl Debug for EntityLocation
Sourceimpl PartialEq for EntityLocation
impl PartialEq for EntityLocation
Sourceimpl QueryData for EntityLocation
SAFETY: Self is the same as Self::ReadOnly
impl QueryData for EntityLocation
SAFETY: Self is the same as Self::ReadOnly
Sourceconst IS_READ_ONLY: bool = true
const IS_READ_ONLY: bool = true
True if this query is read-only and may not perform mutable access.
Sourceconst IS_ARCHETYPAL: bool = true
const IS_ARCHETYPAL: bool = true
Returns true if (and only if) this query data relies strictly on archetypes to limit which
entities are accessed by the Query. Read more
Sourcetype ReadOnly = EntityLocation
type ReadOnly = EntityLocation
The read-only variant of this
QueryData, which satisfies the ReadOnlyQueryData trait.Sourcetype Item<'w, 's> = EntityLocation
type Item<'w, 's> = EntityLocation
The item returned by this
WorldQuery
This will be the data retrieved by the query,
and is visible to the end user when calling e.g. Query<Self>::get.Sourcefn shrink<'wlong, 'wshort, 's>(
item: <EntityLocation as QueryData>::Item<'wlong, 's>,
) -> <EntityLocation as QueryData>::Item<'wshort, 's>where
'wlong: 'wshort,
fn shrink<'wlong, 'wshort, 's>(
item: <EntityLocation as QueryData>::Item<'wlong, 's>,
) -> <EntityLocation as QueryData>::Item<'wshort, 's>where
'wlong: 'wshort,
This function manually implements subtyping for the query items.
Sourceunsafe fn fetch<'w, 's>(
_state: &'s <EntityLocation as WorldQuery>::State,
fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>,
entity: Entity,
_table_row: TableRow,
) -> Option<<EntityLocation as QueryData>::Item<'w, 's>>
unsafe fn fetch<'w, 's>( _state: &'s <EntityLocation as WorldQuery>::State, fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>, entity: Entity, _table_row: TableRow, ) -> Option<<EntityLocation as QueryData>::Item<'w, 's>>
Fetch
Self::Item for either the given entity in the current Table,
or for the given entity in the current Archetype. This must always be called after
WorldQuery::set_table with a table_row in the range of the current Table or after
WorldQuery::set_archetype with an entity in the current archetype.
Accesses components registered in WorldQuery::update_component_access. Read moreSourcefn iter_access(
_state: &<EntityLocation as WorldQuery>::State,
) -> impl Iterator<Item = EcsAccessType<'_>>
fn iter_access( _state: &<EntityLocation as WorldQuery>::State, ) -> impl Iterator<Item = EcsAccessType<'_>>
Returns an iterator over the access needed by
QueryData::fetch. Access conflicts are usually
checked in WorldQuery::update_component_access, but in certain cases this method can be useful to implement
a way of checking for access conflicts in a non-allocating way.Sourcefn provide_extra_access(
_state: &mut Self::State,
_access: &mut Access,
_available_access: &Access,
)
fn provide_extra_access( _state: &mut Self::State, _access: &mut Access, _available_access: &Access, )
Offers additional access above what we requested in
update_component_access.
Implementations may add additional access that is a subset of available_access
and does not conflict with anything in access,
and must update access to include that access. Read moreSourceimpl ReleaseStateQueryData for EntityLocation
impl ReleaseStateQueryData for EntityLocation
Sourcefn release_state<'w>(
item: <EntityLocation as QueryData>::Item<'w, '_>,
) -> <EntityLocation as QueryData>::Item<'w, 'static>
fn release_state<'w>( item: <EntityLocation as QueryData>::Item<'w, '_>, ) -> <EntityLocation as QueryData>::Item<'w, 'static>
Releases the borrow from the query state by converting an item to have a
'static state lifetime.Sourceimpl WorldQuery for EntityLocation
SAFETY:
update_component_access does nothing.
This is sound because fetch does not access components.
impl WorldQuery for EntityLocation
SAFETY:
update_component_access does nothing.
This is sound because fetch does not access components.
Sourceconst IS_DENSE: bool = true
const IS_DENSE: bool = true
Returns true if (and only if) every table of every archetype matched by this fetch contains
all of the matched components. Read more
Sourcetype Fetch<'w> = &'w Entities
type Fetch<'w> = &'w Entities
Per archetype/table state retrieved by this
WorldQuery to compute Self::Item for each entity.Sourcetype State = ()
type State = ()
State used to construct a
Self::Fetch. This will be cached inside QueryState,
so it is best to move as much data / computation here as possible to reduce the cost of
constructing Self::Fetch.Sourcefn shrink_fetch<'wlong, 'wshort>(
fetch: <EntityLocation as WorldQuery>::Fetch<'wlong>,
) -> <EntityLocation as WorldQuery>::Fetch<'wshort>where
'wlong: 'wshort,
fn shrink_fetch<'wlong, 'wshort>(
fetch: <EntityLocation as WorldQuery>::Fetch<'wlong>,
) -> <EntityLocation as WorldQuery>::Fetch<'wshort>where
'wlong: 'wshort,
This function manually implements subtyping for the query fetches.
Sourceunsafe fn init_fetch<'w, 's>(
world: UnsafeWorldCell<'w>,
_state: &'s <EntityLocation as WorldQuery>::State,
_last_run: Tick,
_this_run: Tick,
) -> <EntityLocation as WorldQuery>::Fetch<'w>
unsafe fn init_fetch<'w, 's>( world: UnsafeWorldCell<'w>, _state: &'s <EntityLocation as WorldQuery>::State, _last_run: Tick, _this_run: Tick, ) -> <EntityLocation as WorldQuery>::Fetch<'w>
Creates a new instance of
Self::Fetch,
by combining data from the World with the cached Self::State.
Readonly accesses resources registered in WorldQuery::update_component_access. Read moreSourceunsafe fn set_archetype<'w, 's>(
_fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>,
_state: &'s <EntityLocation as WorldQuery>::State,
_archetype: &'w Archetype,
_table: &Table,
)
unsafe fn set_archetype<'w, 's>( _fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>, _state: &'s <EntityLocation as WorldQuery>::State, _archetype: &'w Archetype, _table: &Table, )
Adjusts internal state to account for the next
Archetype. This will always be called on
archetypes that match this WorldQuery. Read moreSourceunsafe fn set_table<'w, 's>(
_fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>,
_state: &'s <EntityLocation as WorldQuery>::State,
_table: &'w Table,
)
unsafe fn set_table<'w, 's>( _fetch: &mut <EntityLocation as WorldQuery>::Fetch<'w>, _state: &'s <EntityLocation as WorldQuery>::State, _table: &'w Table, )
Adjusts internal state to account for the next
Table. This will always be called on tables
that match this WorldQuery. Read moreSourcefn update_component_access(
_state: &<EntityLocation as WorldQuery>::State,
_access: &mut FilteredAccess,
)
fn update_component_access( _state: &<EntityLocation as WorldQuery>::State, _access: &mut FilteredAccess, )
Sourcefn init_state(_world: &mut World)
fn init_state(_world: &mut World)
Creates and initializes a
State for this WorldQuery type.Sourcefn matches_component_set(
_state: &<EntityLocation as WorldQuery>::State,
_set_contains_id: &impl Fn(ComponentId) -> bool,
) -> bool
fn matches_component_set( _state: &<EntityLocation as WorldQuery>::State, _set_contains_id: &impl Fn(ComponentId) -> bool, ) -> bool
impl ArchetypeQueryData for EntityLocation
impl Copy for EntityLocation
impl ReadOnlyQueryData for EntityLocation
SAFETY: access is read only
impl StructuralPartialEq for EntityLocation