pub struct UniqueEntityEquivalentSliceIterMut<'a, T, I>{ /* private fields */ }Expand description
An iterator that yields &mut UniqueEntityEquivalentSlice. Note that an entity may appear
in multiple slices, depending on the wrapped iterator.
Implementations
Sourceimpl<'a, T, I> UniqueEntityEquivalentSliceIterMut<'a, T, I>
impl<'a, T, I> UniqueEntityEquivalentSliceIterMut<'a, T, I>
Sourcepub const unsafe fn from_mut_slice_iter_unchecked(
iter: I,
) -> UniqueEntityEquivalentSliceIterMut<'a, T, I> ⓘ
pub const unsafe fn from_mut_slice_iter_unchecked( iter: I, ) -> UniqueEntityEquivalentSliceIterMut<'a, T, I> ⓘ
Constructs a UniqueEntityEquivalentSliceIterMut from a mutable slice iterator unsafely.
Safety
All elements in each of the slices must be unique.
Sourcepub fn into_inner(self) -> I
pub fn into_inner(self) -> I
Returns the inner I.
Sourcepub const unsafe fn as_mut_inner(&mut self) -> &mut I
pub const unsafe fn as_mut_inner(&mut self) -> &mut I
Returns a mutable reference to the inner I.
Safety
self must always contain an iterator that yields unique elements,
even while this reference is live.
Sourceimpl<'a, T> UniqueEntityEquivalentSliceIterMut<'a, T, ChunksExactMut<'a, T>>where
T: EntityEquivalent,
impl<'a, T> UniqueEntityEquivalentSliceIterMut<'a, T, ChunksExactMut<'a, T>>where
T: EntityEquivalent,
Sourcepub fn into_remainder(self) -> &'a mut UniqueEntityEquivalentSlice<T>
pub fn into_remainder(self) -> &'a mut UniqueEntityEquivalentSlice<T>
Returns the remainder of the original slice that is not going to be returned by the iterator.
Equivalent to slice::ChunksExactMut::into_remainder.
Sourceimpl<'a, T> UniqueEntityEquivalentSliceIterMut<'a, T, RChunksExactMut<'a, T>>where
T: EntityEquivalent,
impl<'a, T> UniqueEntityEquivalentSliceIterMut<'a, T, RChunksExactMut<'a, T>>where
T: EntityEquivalent,
Sourcepub fn into_remainder(self) -> &'a mut UniqueEntityEquivalentSlice<T>
pub fn into_remainder(self) -> &'a mut UniqueEntityEquivalentSlice<T>
Returns the remainder of the original slice that is not going to be returned by the iterator.
Equivalent to slice::RChunksExactMut::into_remainder.