pub struct MessageParIter<'a, M>where
M: Message,{ /* private fields */ }multi_threaded only.Expand description
A parallel iterator over Messages.
Implementations
Sourceimpl<'a, M> MessageParIter<'a, M>where
M: Message,
impl<'a, M> MessageParIter<'a, M>where
M: Message,
Sourcepub fn new(
reader: &'a mut MessageCursor<M>,
messages: &'a Messages<M>,
) -> MessageParIter<'a, M>
pub fn new( reader: &'a mut MessageCursor<M>, messages: &'a Messages<M>, ) -> MessageParIter<'a, M>
Creates a new parallel iterator over messages that have not yet been seen by reader.
Sourcepub fn batching_strategy(
self,
strategy: BatchingStrategy,
) -> MessageParIter<'a, M>
pub fn batching_strategy( self, strategy: BatchingStrategy, ) -> MessageParIter<'a, M>
Changes the batching strategy used when iterating.
For more information on how this affects the resultant iteration, see
BatchingStrategy.
Sourcepub fn for_each<FN>(self, func: FN)
pub fn for_each<FN>(self, func: FN)
Runs the provided closure for each unread message in parallel.
Unlike normal iteration, the message order is not guaranteed in any form.
Panics
If the ComputeTaskPool is not initialized. If using this from a message reader that is being
initialized and run from the ECS scheduler, this should never panic.
Sourcepub fn for_each_with_id<FN>(self, func: FN)
pub fn for_each_with_id<FN>(self, func: FN)
Runs the provided closure for each unread message in parallel, like for_each,
but additionally provides the MessageId to the closure.
Note that the order of iteration is not guaranteed, but MessageIds are ordered by send order.
Panics
If the ComputeTaskPool is not initialized. If using this from a message reader that is being
initialized and run from the ECS scheduler, this should never panic.