pub struct ParallelCommands<'w, 's> { /* private fields */ }Expand description
An alternative to Commands that can be used in parallel contexts, such as those
in Query::par_iter.
For cases where multiple non-computation-heavy (lightweight) bundles of the same
Bundle type need to be spawned, consider using
Commands::spawn_batch for better performance.
Note
Because command application order will depend on how many threads are ran, non-commutative commands may result in non-deterministic results.
Example
fn parallel_command_system(
mut query: Query<(Entity, &Velocity)>,
par_commands: ParallelCommands
) {
query.par_iter().for_each(|(entity, velocity)| {
if velocity.magnitude() > 10.0 {
par_commands.command_scope(|mut commands| {
commands.entity(entity).despawn();
});
}
});
}Implementations
Sourceimpl<'w, 's> ParallelCommands<'w, 's>
impl<'w, 's> ParallelCommands<'w, 's>
Sourcepub fn command_scope<R>(&self, f: impl FnOnce(Commands<'_, '_>) -> R) -> R
pub fn command_scope<R>(&self, f: impl FnOnce(Commands<'_, '_>) -> R) -> R
Temporarily provides access to the Commands for the current thread.
For an example, see the type-level documentation for ParallelCommands.