pub struct Added<T>(/* private fields */);Expand description
A filter on a component that only retains results the first time after they have been added.
A common use for this filter is one-time initialization.
To retain all results without filtering but still check whether they were added after the
system last ran, use Ref<T>.
Note that this includes changes that happened before the first time this Query was run.
Deferred
Note, that entity modifications issued with Commands
are visible only after deferred operations are applied, typically after the system
that queued them.
Time complexity
Added is not ArchetypeFilter, which practically means that
if the query (with T component filter) matches a million entities,
Added<T> filter will iterate over all of them even if none of them were just added.
For example, these two systems are roughly equivalent in terms of performance:
fn system1(q: Query<&MyComponent, Added<Transform>>) {
for item in &q { /* component added */ }
}
fn system2(q: Query<(&MyComponent, Ref<Transform>)>) {
for item in &q {
if item.1.is_added() { /* component added */ }
}
}Examples
fn print_add_name_component(query: Query<&Name, Added<Name>>) {
for name in &query {
println!("Named entity created: {:?}", name)
}
}