pub struct ReflectEventFns {
pub trigger: fn(&mut World, &(dyn PartialReflect + 'static), &TypeRegistry),
}bevy_reflect only.Expand description
The raw function pointers needed to make up a ReflectEvent.
This is used when creating custom implementations of ReflectEvent with
ReflectEvent::new().
Note: Creating custom implementations of
ReflectEventis an advanced feature that most users will not need. Usually aReflectEventis created for a type by derivingReflectand adding the#[reflect(Event)]attribute. After adding the event to theTypeRegistry, itsReflectEventcan then be retrieved when needed.
Creating a custom ReflectEvent may be useful if you need to create new
event types at runtime, for example, for scripting implementations.
By creating a custom ReflectEvent and inserting it into a type’s
TypeRegistration, you can modify the way
that reflected event of that type will be triggered in the Bevy world.
Fields
trigger: fn(&mut World, &(dyn PartialReflect + 'static), &TypeRegistry)Function pointer implementing ReflectEvent::trigger.
Implementations
Sourceimpl ReflectEventFns
impl ReflectEventFns
Sourcepub fn new<'a, T>() -> ReflectEventFns
pub fn new<'a, T>() -> ReflectEventFns
Get the default set of ReflectEventFns for a specific event type
using its FromType implementation.
This is useful if you want to start with the default implementation before overriding some of the functions to create a custom implementation.