pub struct ReflectFromWorld(/* private fields */);Expand description
A struct used to operate on the reflected FromWorld trait of a type.
A ReflectFromWorld for type T can be obtained via
bevy_reflect::TypeRegistration::data.
Implementations
Sourceimpl ReflectFromWorld
impl ReflectFromWorld
Sourcepub fn from_world(&self, world: &mut World) -> Box<dyn Reflect>
pub fn from_world(&self, world: &mut World) -> Box<dyn Reflect>
Constructs default reflected FromWorld from world using from_world().
Sourcepub fn new(fns: ReflectFromWorldFns) -> ReflectFromWorld
pub fn new(fns: ReflectFromWorldFns) -> ReflectFromWorld
Create a custom implementation of ReflectFromWorld.
This is an advanced feature, useful for scripting implementations, that should not be used by most users unless you know what you are doing.
Usually you should derive Reflect and add the #[reflect(FromWorld)] bundle
to generate a ReflectFromWorld implementation automatically.
See ReflectFromWorldFns for more information.
Sourcepub fn fn_pointers(&self) -> &ReflectFromWorldFns
pub fn fn_pointers(&self) -> &ReflectFromWorldFns
The underlying function pointers implementing methods on ReflectFromWorld.
This is useful when you want to keep track locally of an individual function pointer.
Calling TypeRegistry::get followed by
TypeRegistration::data::<ReflectFromWorld> can be costly if done several
times per frame. Consider cloning ReflectFromWorld and keeping it
between frames, cloning a ReflectFromWorld is very cheap.
If you only need a subset of the methods on ReflectFromWorld,
use fn_pointers to get the underlying ReflectFromWorldFns
and copy the subset of function pointers you care about.