pub struct SimpleExecutor { /* private fields */ }๐Deprecated since 0.17.0: Use SingleThreadedExecutor instead. See https://github.com/bevyengine/bevy/issues/18453 for motivation.
A variant of SingleThreadedExecutor that calls
apply_deferred immediately after running each system.
Implementations
Sourceimpl SimpleExecutor
impl SimpleExecutor
Sourcepub const fn new() -> SimpleExecutor
pub const fn new() -> SimpleExecutor
Creates a new simple executor for use in a Schedule.
This calls each system in order and immediately calls System::apply_deferred.