pub struct Systems { /* private fields */ }Expand description
Container for systems in a schedule.
Implementations
Sourceimpl Systems
impl Systems
Sourcepub fn get(&self, key: SystemKey) -> Option<&SystemWithAccess>
pub fn get(&self, key: SystemKey) -> Option<&SystemWithAccess>
Returns a reference to the system with the given key, if it exists.
Sourcepub fn get_mut(&mut self, key: SystemKey) -> Option<&mut SystemWithAccess>
pub fn get_mut(&mut self, key: SystemKey) -> Option<&mut SystemWithAccess>
Returns a mutable reference to the system with the given key, if it exists.
Sourcepub fn has_conditions(&self, key: SystemKey) -> bool
pub fn has_conditions(&self, key: SystemKey) -> bool
Returns true if the system with the given key has conditions.
Sourcepub fn get_conditions(&self, key: SystemKey) -> Option<&[ConditionWithAccess]>
pub fn get_conditions(&self, key: SystemKey) -> Option<&[ConditionWithAccess]>
Returns a reference to the conditions for the system with the given key, if it exists.
Sourcepub fn get_conditions_mut(
&mut self,
key: SystemKey,
) -> Option<&mut Vec<ConditionWithAccess>>
pub fn get_conditions_mut( &mut self, key: SystemKey, ) -> Option<&mut Vec<ConditionWithAccess>>
Returns a mutable reference to the conditions for the system with the given key, if it exists.
Sourcepub fn iter(
&self,
) -> impl Iterator<Item = (SystemKey, &Box<dyn System<Out = (), In = ()>>, &[ConditionWithAccess])>
pub fn iter( &self, ) -> impl Iterator<Item = (SystemKey, &Box<dyn System<Out = (), In = ()>>, &[ConditionWithAccess])>
Returns an iterator over all systems and their conditions in this container.
Sourcepub fn insert(
&mut self,
system: Box<dyn System<Out = (), In = ()>>,
conditions: Vec<Box<dyn ReadOnlySystem<Out = bool, In = ()>>>,
) -> SystemKey
pub fn insert( &mut self, system: Box<dyn System<Out = (), In = ()>>, conditions: Vec<Box<dyn ReadOnlySystem<Out = bool, In = ()>>>, ) -> SystemKey
Inserts a new system into the container, along with its conditions,
and queues it to be initialized later in Systems::initialize.
We have to defer initialization of systems in the container until we have
&mut World access, so we store these in a list until
Systems::initialize is called. This is usually done upon the first
run of the schedule.
Sourcepub fn is_initialized(&self) -> bool
pub fn is_initialized(&self) -> bool
Returns true if all systems in this container have been initialized.
Sourcepub fn initialize(&mut self, world: &mut World)
pub fn initialize(&mut self, world: &mut World)
Initializes all systems and their conditions that have not been initialized yet.
Sourcepub fn get_conflicting_systems(
&self,
flat_dependency_analysis: &DagAnalysis<SystemKey>,
flat_ambiguous_with: &Graph<false, SystemKey>,
ambiguous_with_all: &HashSet<NodeId>,
ignored_ambiguities: &BTreeSet<ComponentId>,
) -> ConflictingSystems
pub fn get_conflicting_systems( &self, flat_dependency_analysis: &DagAnalysis<SystemKey>, flat_ambiguous_with: &Graph<false, SystemKey>, ambiguous_with_all: &HashSet<NodeId>, ignored_ambiguities: &BTreeSet<ComponentId>, ) -> ConflictingSystems
Calculates the list of systems that conflict with each other based on their access patterns.
If the Box<[ComponentId]> is empty for a given pair of systems, then the
systems conflict on World access in general (e.g. one of them is
exclusive, or both systems have Query<EntityMut>).