pub struct CombinatorSystem<Func, A, B> { /* private fields */ }Expand description
Implementations
Sourceimpl<Func, A, B> CombinatorSystem<Func, A, B>
impl<Func, A, B> CombinatorSystem<Func, A, B>
Sourcepub fn new(a: A, b: B, name: DebugName) -> CombinatorSystem<Func, A, B>
pub fn new(a: A, b: B, name: DebugName) -> CombinatorSystem<Func, A, B>
Creates a new system that combines two inner systems.
The returned system will only be usable if Func implements Combine<A, B>.