MemorySize

Struct MemorySize 

pub struct MemorySize(pub u64);
The size of an object in memory, in bytes.

The helper methods on this type use powers of 2 for conversions between units, consistent with the standards for memory reporting.

While it would technically be more correct to use e.g. “kibibytes” instead of “kilobytes”, “mebibytes” instead of “megabytes”, etc., the more common terms are used here for familiarity and discoverability.

Tuple Fields

0: u64

Implementations

Source

impl MemorySize

pub fn new(bytes: usize) -> MemorySize

Creates a new MemorySize from the given number of bytes.

This method is provided for convenience, as many other APIs in Rust use usize for sizes and counts. To initialize this type with a u64, just call MemorySize(bytes) directly.

pub fn as_bytes(&self) -> usize

Returns the size in bytes, as a usize.

This method is provided for convenience, as many other APIs in Rust use usize for sizes and counts. To access the value as a u64, just use the .0 field directly.

1 byte = 8 bits.

pub fn as_kilobytes(&self) -> f64

Returns the size in kilobytes.

1 kilobyte = 1024 bytes.

pub fn as_megabytes(&self) -> f64

Returns the size in megabytes.

1 megabyte = 1024 kilobytes.

pub fn as_gigabytes(&self) -> f64

Returns the size in gigabytes.

1 gigabyte = 1024 megabytes.

pub fn as_terabytes(&self) -> f64

Returns the size in terabytes.

1 terabyte = 1024 gigabytes.

pub fn appropriate_unit(&self) -> MemoryUnit

Determine the appropriate unit for displaying the memory size.

Units are chosen such that the value is at least 1 in that unit.

This is used for formatting the memory size in a human-readable way, such as in the Display implementation for this type.