pub enum ComponentEntry<'w, 'a, T>where
T: Component,{
Occupied(OccupiedComponentEntry<'w, 'a, T>),
Vacant(VacantComponentEntry<'w, 'a, T>),
}Expand description
A view into a single entity and component in a world, which may either be vacant or occupied.
This enum can only be constructed from the entry method on EntityWorldMut.
Variants
Occupied(OccupiedComponentEntry<'w, 'a, T>)
An occupied entry.
Vacant(VacantComponentEntry<'w, 'a, T>)
A vacant entry.
Implementations
Sourceimpl<'w, 'a, T> ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> ComponentEntry<'w, 'a, T>
Sourcepub fn and_modify<F>(self, f: F) -> ComponentEntry<'w, 'a, T>
pub fn and_modify<F>(self, f: F) -> ComponentEntry<'w, 'a, T>
Provides in-place mutable access to an occupied entry.
Examples
#[derive(Component, Default, Clone, Copy, Debug, PartialEq)]
struct Comp(u32);
let mut entity = world.spawn(Comp(0));
entity.entry::<Comp>().and_modify(|mut c| c.0 += 1);
assert_eq!(world.query::<&Comp>().single(&world).unwrap().0, 1);Sourceimpl<'w, 'a, T> ComponentEntry<'w, 'a, T>where
T: Component,
impl<'w, 'a, T> ComponentEntry<'w, 'a, T>where
T: Component,
Sourcepub fn insert_entry(self, component: T) -> OccupiedComponentEntry<'w, 'a, T>
pub fn insert_entry(self, component: T) -> OccupiedComponentEntry<'w, 'a, T>
Replaces the component of the entry, and returns an OccupiedComponentEntry.
Examples
#[derive(Component, Default, Clone, Copy, Debug, PartialEq)]
struct Comp(u32);
let mut entity = world.spawn_empty();
let entry = entity.entry().insert_entry(Comp(4));
assert_eq!(entry.get(), &Comp(4));
let entry = entity.entry().insert_entry(Comp(2));
assert_eq!(entry.get(), &Comp(2));Sourcepub fn or_insert(self, default: T) -> OccupiedComponentEntry<'w, 'a, T>
pub fn or_insert(self, default: T) -> OccupiedComponentEntry<'w, 'a, T>
Ensures the entry has this component by inserting the given default if empty, and returns a mutable reference to this component in the entry.
Examples
#[derive(Component, Default, Clone, Copy, Debug, PartialEq)]
struct Comp(u32);
let mut entity = world.spawn_empty();
entity.entry().or_insert(Comp(4));
assert_eq!(world.query::<&Comp>().single(&world).unwrap().0, 4);
entity.entry().or_insert(Comp(15)).into_mut().0 *= 2;
assert_eq!(world.query::<&Comp>().single(&world).unwrap().0, 8);Sourcepub fn or_insert_with<F>(self, default: F) -> OccupiedComponentEntry<'w, 'a, T>where
F: FnOnce() -> T,
pub fn or_insert_with<F>(self, default: F) -> OccupiedComponentEntry<'w, 'a, T>where
F: FnOnce() -> T,
Ensures the entry has this component by inserting the result of the default function if empty, and returns a mutable reference to this component in the entry.
Examples
#[derive(Component, Default, Clone, Copy, Debug, PartialEq)]
struct Comp(u32);
let mut entity = world.spawn_empty();
entity.entry().or_insert_with(|| Comp(4));
assert_eq!(world.query::<&Comp>().single(&world).unwrap().0, 4);Sourceimpl<'w, 'a, T> ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> ComponentEntry<'w, 'a, T>
Sourcepub fn or_default(self) -> OccupiedComponentEntry<'w, 'a, T>
pub fn or_default(self) -> OccupiedComponentEntry<'w, 'a, T>
Ensures the entry has this component by inserting the default value if empty, and returns a mutable reference to this component in the entry.
Examples
#[derive(Component, Default, Clone, Copy, Debug, PartialEq)]
struct Comp(u32);
let mut entity = world.spawn_empty();
entity.entry::<Comp>().or_default();
assert_eq!(world.query::<&Comp>().single(&world).unwrap().0, 0);Auto Trait Implementations
impl<'w, 'a, T> Freeze for ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> !RefUnwindSafe for ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> Send for ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> Sync for ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> Unpin for ComponentEntry<'w, 'a, T>where
T: Unpin,
impl<'w, 'a, T> UnsafeUnpin for ComponentEntry<'w, 'a, T>
impl<'w, 'a, T> !UnwindSafe for ComponentEntry<'w, 'a, T>