GltfLoader

Struct GltfLoader 

pub struct GltfLoader {
    pub supported_compressed_formats: CompressedImageFormats,
    pub custom_vertex_attributes: HashMap<Box<str>, MeshVertexAttribute>,
    pub default_sampler: Arc<Mutex<ImageSamplerDescriptor>>,
    pub default_convert_coordinates: GltfConvertCoordinates,
    pub extensions: Arc<RwLock<Vec<Box<dyn GltfExtensionHandler>>>>,
}
Loads glTF files with all of their data as their corresponding bevy representations.

Fields

supported_compressed_formats: CompressedImageFormats

List of compressed image formats handled by the loader.

custom_vertex_attributes: HashMap<Box<str>, MeshVertexAttribute>

Custom vertex attributes that will be recognized when loading a glTF file.

Keys must be the attribute names as found in the glTF data, which must start with an underscore. See this section of the glTF specification for additional details on custom attributes.

default_sampler: Arc<Mutex<ImageSamplerDescriptor>>

Arc to default ImageSamplerDescriptor.

default_convert_coordinates: GltfConvertCoordinates

The default glTF coordinate conversion setting. This can be overridden per-load by GltfLoaderSettings::convert_coordinates.

extensions: Arc<RwLock<Vec<Box<dyn GltfExtensionHandler>>>>

glTF extension data processors. These are Bevy-side processors designed to access glTF extension data during the loading process.

Implementations

Source

impl GltfLoader

pub async fn load_gltf<'a, 'b, 'c>( loader: &GltfLoader, bytes: &'a [u8], load_context: &'b mut LoadContext<'c>, settings: &'b GltfLoaderSettings, ) -> Result<Gltf, GltfError>

Loads an entire glTF file.