pub struct GltfPlugin {
pub default_sampler: ImageSamplerDescriptor,
pub use_model_forward_direction: bool,
pub custom_vertex_attributes: HashMap<Box<str>, MeshVertexAttribute>,
}Expand description
Adds support for glTF file loading to the app.
Fields
default_sampler: ImageSamplerDescriptorThe default image sampler to lay glTF sampler data on top of.
Can be modified with the DefaultGltfImageSampler resource.
use_model_forward_direction: boolCAUTION: This is an experimental feature with known issues. Behavior may change in future versions.
How to convert glTF coordinates on import. Assuming glTF cameras, glTF lights, and glTF meshes had global identity transforms,
their Bevy Transform::forward will be pointing in the following global directions:
- When set to
false- glTF cameras and glTF lights: global -Z,
- glTF models: global +Z.
- When set to
true- glTF cameras and glTF lights: global +Z,
- glTF models: global -Z.
The default is false.
custom_vertex_attributes: HashMap<Box<str>, MeshVertexAttribute>Registry for custom vertex attributes.
To specify, use GltfPlugin::add_custom_vertex_attribute.
Implementations
Sourceimpl GltfPlugin
impl GltfPlugin
Sourcepub fn add_custom_vertex_attribute(
self,
name: &str,
attribute: MeshVertexAttribute,
) -> GltfPlugin
pub fn add_custom_vertex_attribute( self, name: &str, attribute: MeshVertexAttribute, ) -> GltfPlugin
Register a custom vertex attribute so that it is recognized when loading a glTF file with the GltfLoader.
name must be the attribute name as found in the glTF data, which must start with an underscore.
See this section of the glTF specification
for additional details on custom attributes.
Examples found in repository?
24fn main() {
25 App::new()
26 .insert_resource(AmbientLight {
27 color: Color::WHITE,
28 brightness: 1.0 / 5.0f32,
29 ..default()
30 })
31 .add_plugins((
32 DefaultPlugins.set(
33 GltfPlugin::default()
34 // Map a custom glTF attribute name to a `MeshVertexAttribute`.
35 // The glTF file used here has an attribute name with *two*
36 // underscores: __BARYCENTRIC
37 // One is stripped to do the comparison here.
38 .add_custom_vertex_attribute("_BARYCENTRIC", ATTRIBUTE_BARYCENTRIC),
39 ),
40 Material2dPlugin::<CustomMaterial>::default(),
41 ))
42 .add_systems(Startup, setup)
43 .run();
44}Trait Implementations
Sourceimpl Default for GltfPlugin
impl Default for GltfPlugin
Sourcefn default() -> GltfPlugin
fn default() -> GltfPlugin
Sourceimpl Plugin for GltfPlugin
impl Plugin for GltfPlugin
Sourcefn finish(&self, app: &mut App)
fn finish(&self, app: &mut App)
App, once all plugins registered are ready. This can
be useful for plugins that depends on another plugin asynchronous setup, like the renderer.Sourcefn ready(&self, _app: &App) -> bool
fn ready(&self, _app: &App) -> bool
finish should be called.Sourcefn cleanup(&self, _app: &mut App)
fn cleanup(&self, _app: &mut App)