pub struct GltfPrimitive {
pub index: usize,
pub parent_mesh_index: usize,
pub name: String,
pub mesh: Handle<Mesh>,
pub material: Option<Handle<StandardMaterial>>,
pub extras: Option<GltfExtras>,
pub material_extras: Option<GltfExtras>,
}Expand description
Part of a GltfMesh that consists of a Mesh, an optional StandardMaterial and GltfExtras.
Fields
index: usizeIndex of the primitive inside the mesh
parent_mesh_index: usizeIndex of the parent GltfMesh of this primitive
name: StringComputed name for a primitive - either a user defined primitive name from gLTF or a generated name from index
mesh: Handle<Mesh>Topology to be rendered.
material: Option<Handle<StandardMaterial>>Material to apply to the mesh.
extras: Option<GltfExtras>Additional data.
material_extras: Option<GltfExtras>Additional data of the material.
Implementations
Sourceimpl GltfPrimitive
impl GltfPrimitive
Sourcepub fn new(
gltf_mesh: &Mesh<'_>,
gltf_primitive: &Primitive<'_>,
mesh: Handle<Mesh>,
material: Option<Handle<StandardMaterial>>,
extras: Option<GltfExtras>,
material_extras: Option<GltfExtras>,
) -> GltfPrimitive
pub fn new( gltf_mesh: &Mesh<'_>, gltf_primitive: &Primitive<'_>, mesh: Handle<Mesh>, material: Option<Handle<StandardMaterial>>, extras: Option<GltfExtras>, material_extras: Option<GltfExtras>, ) -> GltfPrimitive
Create a primitive extracting name and index from glTF def
Sourcepub fn asset_label(&self) -> GltfAssetLabel
pub fn asset_label(&self) -> GltfAssetLabel
Subasset label for this primitive within its parent GltfMesh within the gLTF parent asset.