pub enum ImageAddressMode {
ClampToEdge,
Repeat,
MirrorRepeat,
ClampToBorder,
}Expand description
How edges should be handled in texture addressing.
See ImageSamplerDescriptor for information how to configure this.
This type mirrors AddressMode.
Variants
ClampToEdge
Clamp the value to the edge of the texture.
-0.25 -> 0.0 1.25 -> 1.0
Repeat
Repeat the texture in a tiling fashion.
-0.25 -> 0.75 1.25 -> 0.25
MirrorRepeat
Repeat the texture, mirroring it every repeat.
-0.25 -> 0.25 1.25 -> 0.75
ClampToBorder
Clamp the value to the border of the texture
Requires the wgpu feature Features::ADDRESS_MODE_CLAMP_TO_BORDER.
-0.25 -> border 1.25 -> border