pub enum ImageFormat {
Show 16 variants
Basis,
Bmp,
Dds,
Farbfeld,
Gif,
OpenExr,
Hdr,
Ico,
Jpeg,
Ktx2,
Png,
Pnm,
Qoi,
Tga,
Tiff,
WebP,
}Expand description
The format of an on-disk image asset.
Variants
Basis
basis-universal only.An image in basis universal format.
Bmp
bmp only.An image in BMP format.
Dds
dds only.An image in DDS format.
Farbfeld
ff only.An image in Farbfeld format.
Gif
gif only.An image in GIF format.
OpenExr
exr only.An image in Open EXR format.
Hdr
hdr only.An image in Radiance HDR format.
Ico
ico only.An image in ICO format.
Jpeg
jpeg only.An image in JPEG format.
Ktx2
ktx2 only.An image in Khronos KTX2 format.
Png
png only.An image in PNG format.
Pnm
pnm only.An image in general PNM format
Qoi
qoi only.An image in QOI format.
Tga
tga only.An image in TGA format.
Tiff
tiff only.An image in TIFF format.
WebP
webp only.An image in WEBP format.
Implementations
Sourceimpl ImageFormat
impl ImageFormat
Sourcepub const fn to_file_extensions(&self) -> &'static [&'static str]
pub const fn to_file_extensions(&self) -> &'static [&'static str]
Gets the file extensions for a given format.
Sourcepub const fn to_mime_types(&self) -> &'static [&'static str]
pub const fn to_mime_types(&self) -> &'static [&'static str]
Gets the MIME types for a given format.
If a format doesn’t have any dedicated MIME types, this list will be empty.
Sourcepub fn from_mime_type(mime_type: &str) -> Option<ImageFormat>
pub fn from_mime_type(mime_type: &str) -> Option<ImageFormat>
Returns the image format associated with the given MIME type.
None is returned if the MIME type is unknown, or its format’s feature is not enabled.
Sourcepub fn from_extension(extension: &str) -> Option<ImageFormat>
pub fn from_extension(extension: &str) -> Option<ImageFormat>
Returns the image format associated with the given file extension.
None is returned if the extension is unknown, or its format’s feature is not enabled.
Sourcepub fn as_image_crate_format(&self) -> Option<ImageFormat>
pub fn as_image_crate_format(&self) -> Option<ImageFormat>
Returns the equivalent image::ImageFormat if available.
Sourcepub fn from_image_crate_format(format: ImageFormat) -> Option<ImageFormat>
pub fn from_image_crate_format(format: ImageFormat) -> Option<ImageFormat>
Returns the equivalent bevy ImageFormat of an image::ImageFormat.
Returns None if the format is unsupported, or its feature is disabled.