pub enum SaveImageError {
MissingExtension(AssetPath<'static>),
UnknownExtension(String),
ImageMissingData,
UnsupportedFormat(ImageFormat),
UnsupportedSaveColorTypeForFormat(ImageFormat, TextureFormat),
ImageError(ImageError),
IoError(Error),
}Available on crate feature
bevy_image only.Expand description
An error while saving an image.
Variants
MissingExtension(AssetPath<'static>)
Cannot deduce file format from extension because there is no extension.
UnknownExtension(String)
Cannot deduce file format from extension since this extension is unknown. Holds the extension that could not be matched.
ImageMissingData
Image::data is None, so there is no data to save. See
RenderAssetUsages for more.
UnsupportedFormat(ImageFormat)
The image saver doesn’t support the file format being requested.
UnsupportedSaveColorTypeForFormat(ImageFormat, TextureFormat)
The image saver doesn’t support the texture format of the image data for the image format.
ImageError(ImageError)
The image crate returned an error.
IoError(Error)
Writing the bytes returned an error.