CompressedImageFormats

Struct CompressedImageFormats 

pub struct CompressedImageFormats(/* private fields */);
A set of flags describing the compressed formats supported by a render device.

Implementations

Source

impl CompressedImageFormats

pub const NONE: CompressedImageFormats

No support for compressed textures.

pub const ASTC_LDR: CompressedImageFormats

Support for Adaptive Scalable Texture Compression.

For more information see:

pub const ASTC_HDR: CompressedImageFormats

Support for ASTC HDR textures.

For more information see:

pub const BC: CompressedImageFormats

Support for Block Compressed textures.

For more information see:

pub const ETC2: CompressedImageFormats

Support for Ericsson Texture Compression.

For more information see:

Source

impl CompressedImageFormats

pub const fn empty() -> CompressedImageFormats

Get a flags value with all bits unset.

pub const fn all() -> CompressedImageFormats

Get a flags value with all known bits set.

pub const fn bits(&self) -> u32

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

pub const fn from_bits(bits: u32) -> Option<CompressedImageFormats>

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: u32) -> CompressedImageFormats

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: u32) -> CompressedImageFormats

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<CompressedImageFormats>

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Whether all bits in self are unset.

pub const fn is_all(&self) -> bool

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: CompressedImageFormats) -> bool

Whether any set bits in other are also set in self.

pub const fn contains(&self, other: CompressedImageFormats) -> bool

Whether all set bits in other are also set in self.

Examples found in repository?
examples/3d/skybox.rs (line 128)
106fn cycle_cubemap_asset(
107    time: Res<Time>,
108    mut next_swap: Local<f32>,
109    mut cubemap: ResMut<Cubemap>,
110    asset_server: Res<AssetServer>,
111    render_device: Res<RenderDevice>,
112) {
113    let now = time.elapsed_secs();
114    if *next_swap == 0.0 {
115        *next_swap = now + CUBEMAP_SWAP_DELAY;
116        return;
117    } else if now < *next_swap {
118        return;
119    }
120    *next_swap += CUBEMAP_SWAP_DELAY;
121
122    let supported_compressed_formats =
123        CompressedImageFormats::from_features(render_device.features());
124
125    let mut new_index = cubemap.index;
126    for _ in 0..CUBEMAPS.len() {
127        new_index = (new_index + 1) % CUBEMAPS.len();
128        if supported_compressed_formats.contains(CUBEMAPS[new_index].1) {
129            break;
130        }
131        info!(
132            "Skipping format which is not supported by current hardware: {:?}",
133            CUBEMAPS[new_index]
134        );
135    }
136
137    // Skip swapping to the same texture. Useful for when ktx2, zstd, or compressed texture support
138    // is missing
139    if new_index == cubemap.index {
140        return;
141    }
142
143    cubemap.index = new_index;
144    cubemap.image_handle = asset_server.load(CUBEMAPS[cubemap.index].0);
145    cubemap.is_loaded = false;
146}
pub fn insert(&mut self, other: CompressedImageFormats)

The bitwise or (|) of the bits in self and other.

pub fn remove(&mut self, other: CompressedImageFormats)

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: CompressedImageFormats)

The bitwise exclusive-or (^) of the bits in self and other.

pub fn set(&mut self, other: CompressedImageFormats, value: bool)

Call insert when value is true or remove when value is false.

pub const fn intersection( self, other: CompressedImageFormats, ) -> CompressedImageFormats

The bitwise and (&) of the bits in self and other.

pub const fn union( self, other: CompressedImageFormats, ) -> CompressedImageFormats

The bitwise or (|) of the bits in self and other.

pub const fn difference( self, other: CompressedImageFormats, ) -> CompressedImageFormats

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference( self, other: CompressedImageFormats, ) -> CompressedImageFormats

The bitwise exclusive-or (^) of the bits in self and other.

pub const fn complement(self) -> CompressedImageFormats

The bitwise negation (!) of the bits in self, truncating the result.

Source

impl CompressedImageFormats

pub const fn iter(&self) -> Iter<CompressedImageFormats>

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<CompressedImageFormats>

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.

Source

impl CompressedImageFormats

pub fn from_features(features: Features) -> CompressedImageFormats

Get the supported texture compression formats from a wgpu device’s features.

Examples found in repository?
examples/3d/skybox.rs (line 123)
106fn cycle_cubemap_asset(
107    time: Res<Time>,
108    mut next_swap: Local<f32>,
109    mut cubemap: ResMut<Cubemap>,
110    asset_server: Res<AssetServer>,
111    render_device: Res<RenderDevice>,
112) {
113    let now = time.elapsed_secs();
114    if *next_swap == 0.0 {
115        *next_swap = now + CUBEMAP_SWAP_DELAY;
116        return;
117    } else if now < *next_swap {
118        return;
119    }
120    *next_swap += CUBEMAP_SWAP_DELAY;
121
122    let supported_compressed_formats =
123        CompressedImageFormats::from_features(render_device.features());
124
125    let mut new_index = cubemap.index;
126    for _ in 0..CUBEMAPS.len() {
127        new_index = (new_index + 1) % CUBEMAPS.len();
128        if supported_compressed_formats.contains(CUBEMAPS[new_index].1) {
129            break;
130        }
131        info!(
132            "Skipping format which is not supported by current hardware: {:?}",
133            CUBEMAPS[new_index]
134        );
135    }
136
137    // Skip swapping to the same texture. Useful for when ktx2, zstd, or compressed texture support
138    // is missing
139    if new_index == cubemap.index {
140        return;
141    }
142
143    cubemap.index = new_index;
144    cubemap.image_handle = asset_server.load(CUBEMAPS[cubemap.index].0);
145    cubemap.is_loaded = false;
146}
pub fn supports(&self, format: TextureFormat) -> bool

Returns true if the given TextureFormat is supported by the available compression features.

Always returns true for uncompressed formats.