pub struct ImageLoader { /* private fields */ }Expand description
Loader for images that can be read by the image crate.
Implementations
Sourceimpl ImageLoader
impl ImageLoader
Sourcepub const SUPPORTED_FORMATS: &'static [ImageFormat]
pub const SUPPORTED_FORMATS: &'static [ImageFormat]
Full list of supported formats.
Sourcepub const SUPPORTED_FILE_EXTENSIONS: &'static [&'static str]
pub const SUPPORTED_FILE_EXTENSIONS: &'static [&'static str]
Gets the list of file extensions for all formats.
Sourcepub fn new(supported_compressed_formats: CompressedImageFormats) -> ImageLoader
pub fn new(supported_compressed_formats: CompressedImageFormats) -> ImageLoader
Creates a new image loader that supports the provided formats.