pub enum GamepadButton {
Show 20 variants
South,
East,
North,
West,
C,
Z,
LeftTrigger,
LeftTrigger2,
RightTrigger,
RightTrigger2,
Select,
Start,
Mode,
LeftThumb,
RightThumb,
DPadUp,
DPadDown,
DPadLeft,
DPadRight,
Other(u8),
}Expand description
Variants
South
The bottom action button of the action pad (i.e. PS: Cross, Xbox: A).
East
The right action button of the action pad (i.e. PS: Circle, Xbox: B).
North
The upper action button of the action pad (i.e. PS: Triangle, Xbox: Y).
West
The left action button of the action pad (i.e. PS: Square, Xbox: X).
C
The C button.
Z
The Z button.
LeftTrigger
The first left trigger.
LeftTrigger2
The second left trigger.
RightTrigger
The first right trigger.
RightTrigger2
The second right trigger.
Select
The select button.
Start
The start button.
Mode
The mode button.
LeftThumb
The left thumb stick button.
RightThumb
The right thumb stick button.
DPadUp
The up button of the D-Pad.
DPadDown
The down button of the D-Pad.
DPadLeft
The left button of the D-Pad.
DPadRight
The right button of the D-Pad.
Other(u8)
Miscellaneous buttons, considered non-standard (i.e. Extra buttons on a flight stick that do not have a gamepad equivalent).
Implementations
Sourceimpl GamepadButton
impl GamepadButton
Sourcepub const fn all() -> [GamepadButton; 19]
pub const fn all() -> [GamepadButton; 19]
Returns an array of all the standard GamepadButton.