pub struct GamepadRumbleIntensity {
pub strong_motor: f32,
pub weak_motor: f32,
}gamepad only.Expand description
The intensity at which a gamepad’s force-feedback motors may rumble.
Fields
strong_motor: f32The rumble intensity of the strong gamepad motor.
Ranges from 0.0 to 1.0.
By convention, this is usually a low-frequency motor on the left-hand side of the gamepad, though it may vary across platforms and hardware.
weak_motor: f32The rumble intensity of the weak gamepad motor.
Ranges from 0.0 to 1.0.
By convention, this is usually a high-frequency motor on the right-hand side of the gamepad, though it may vary across platforms and hardware.
Implementations
Sourceimpl GamepadRumbleIntensity
impl GamepadRumbleIntensity
Sourcepub const MAX: GamepadRumbleIntensity
pub const MAX: GamepadRumbleIntensity
Rumble both gamepad motors at maximum intensity.
Sourcepub const WEAK_MAX: GamepadRumbleIntensity
pub const WEAK_MAX: GamepadRumbleIntensity
Rumble the weak motor at maximum intensity.
Sourcepub const STRONG_MAX: GamepadRumbleIntensity
pub const STRONG_MAX: GamepadRumbleIntensity
Rumble the strong motor at maximum intensity.
Sourcepub const fn weak_motor(intensity: f32) -> GamepadRumbleIntensity
pub const fn weak_motor(intensity: f32) -> GamepadRumbleIntensity
Creates a new rumble intensity with weak motor intensity set to the given value.
Clamped within the 0.0 to 1.0 range.
Examples found in repository?
17fn gamepad_system(
18 gamepads: Query<(Entity, &Gamepad)>,
19 mut rumble_requests: MessageWriter<GamepadRumbleRequest>,
20) {
21 for (entity, gamepad) in &gamepads {
22 if gamepad.just_pressed(GamepadButton::North) {
23 info!(
24 "North face button: strong (low-frequency) with low intensity for rumble for 5 seconds. Press multiple times to increase intensity."
25 );
26 rumble_requests.write(GamepadRumbleRequest::Add {
27 gamepad: entity,
28 intensity: GamepadRumbleIntensity::strong_motor(0.1),
29 duration: Duration::from_secs(5),
30 });
31 }
32
33 if gamepad.just_pressed(GamepadButton::East) {
34 info!("East face button: maximum rumble on both motors for 5 seconds");
35 rumble_requests.write(GamepadRumbleRequest::Add {
36 gamepad: entity,
37 duration: Duration::from_secs(5),
38 intensity: GamepadRumbleIntensity::MAX,
39 });
40 }
41
42 if gamepad.just_pressed(GamepadButton::South) {
43 info!("South face button: low-intensity rumble on the weak motor for 0.5 seconds");
44 rumble_requests.write(GamepadRumbleRequest::Add {
45 gamepad: entity,
46 duration: Duration::from_secs_f32(0.5),
47 intensity: GamepadRumbleIntensity::weak_motor(0.25),
48 });
49 }
50
51 if gamepad.just_pressed(GamepadButton::West) {
52 info!("West face button: custom rumble intensity for 5 second");
53 rumble_requests.write(GamepadRumbleRequest::Add {
54 gamepad: entity,
55 intensity: GamepadRumbleIntensity {
56 // intensity low-frequency motor, usually on the left-hand side
57 strong_motor: 0.5,
58 // intensity of high-frequency motor, usually on the right-hand side
59 weak_motor: 0.25,
60 },
61 duration: Duration::from_secs(5),
62 });
63 }
64
65 if gamepad.just_pressed(GamepadButton::Start) {
66 info!("Start button: Interrupt the current rumble");
67 rumble_requests.write(GamepadRumbleRequest::Stop { gamepad: entity });
68 }
69 }
70}Sourcepub const fn strong_motor(intensity: f32) -> GamepadRumbleIntensity
pub const fn strong_motor(intensity: f32) -> GamepadRumbleIntensity
Creates a new rumble intensity with strong motor intensity set to the given value.
Clamped within the 0.0 to 1.0 range.
Examples found in repository?
17fn gamepad_system(
18 gamepads: Query<(Entity, &Gamepad)>,
19 mut rumble_requests: MessageWriter<GamepadRumbleRequest>,
20) {
21 for (entity, gamepad) in &gamepads {
22 if gamepad.just_pressed(GamepadButton::North) {
23 info!(
24 "North face button: strong (low-frequency) with low intensity for rumble for 5 seconds. Press multiple times to increase intensity."
25 );
26 rumble_requests.write(GamepadRumbleRequest::Add {
27 gamepad: entity,
28 intensity: GamepadRumbleIntensity::strong_motor(0.1),
29 duration: Duration::from_secs(5),
30 });
31 }
32
33 if gamepad.just_pressed(GamepadButton::East) {
34 info!("East face button: maximum rumble on both motors for 5 seconds");
35 rumble_requests.write(GamepadRumbleRequest::Add {
36 gamepad: entity,
37 duration: Duration::from_secs(5),
38 intensity: GamepadRumbleIntensity::MAX,
39 });
40 }
41
42 if gamepad.just_pressed(GamepadButton::South) {
43 info!("South face button: low-intensity rumble on the weak motor for 0.5 seconds");
44 rumble_requests.write(GamepadRumbleRequest::Add {
45 gamepad: entity,
46 duration: Duration::from_secs_f32(0.5),
47 intensity: GamepadRumbleIntensity::weak_motor(0.25),
48 });
49 }
50
51 if gamepad.just_pressed(GamepadButton::West) {
52 info!("West face button: custom rumble intensity for 5 second");
53 rumble_requests.write(GamepadRumbleRequest::Add {
54 gamepad: entity,
55 intensity: GamepadRumbleIntensity {
56 // intensity low-frequency motor, usually on the left-hand side
57 strong_motor: 0.5,
58 // intensity of high-frequency motor, usually on the right-hand side
59 weak_motor: 0.25,
60 },
61 duration: Duration::from_secs(5),
62 });
63 }
64
65 if gamepad.just_pressed(GamepadButton::Start) {
66 info!("Start button: Interrupt the current rumble");
67 rumble_requests.write(GamepadRumbleRequest::Stop { gamepad: entity });
68 }
69 }
70}Trait Implementations
Sourceimpl Clone for GamepadRumbleIntensity
impl Clone for GamepadRumbleIntensity
Sourcefn clone(&self) -> GamepadRumbleIntensity
fn clone(&self) -> GamepadRumbleIntensity
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreimpl Copy for GamepadRumbleIntensity
Sourceimpl Debug for GamepadRumbleIntensity
impl Debug for GamepadRumbleIntensity
Sourceimpl FromArg for GamepadRumbleIntensity
impl FromArg for GamepadRumbleIntensity
Sourceimpl FromReflect for GamepadRumbleIntensity
impl FromReflect for GamepadRumbleIntensity
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<GamepadRumbleIntensity>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<GamepadRumbleIntensity>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetTypeRegistration for GamepadRumbleIntensity
impl GetTypeRegistration for GamepadRumbleIntensity
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for GamepadRumbleIntensity
impl IntoReturn for GamepadRumbleIntensity
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
GamepadRumbleIntensity: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
GamepadRumbleIntensity: 'into_return,
Sourceimpl PartialEq for GamepadRumbleIntensity
impl PartialEq for GamepadRumbleIntensity
Sourcefn eq(&self, other: &GamepadRumbleIntensity) -> bool
fn eq(&self, other: &GamepadRumbleIntensity) -> bool
self and other values to be equal, and is used by ==.Sourceimpl PartialReflect for GamepadRumbleIntensity
impl PartialReflect for GamepadRumbleIntensity
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<GamepadRumbleIntensity>) -> ReflectOwned
fn reflect_owned(self: Box<GamepadRumbleIntensity>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<GamepadRumbleIntensity>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<GamepadRumbleIntensity>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(
self: Box<GamepadRumbleIntensity>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<GamepadRumbleIntensity>, ) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for GamepadRumbleIntensity
impl Reflect for GamepadRumbleIntensity
Sourcefn into_any(self: Box<GamepadRumbleIntensity>) -> Box<dyn Any>
fn into_any(self: Box<GamepadRumbleIntensity>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<GamepadRumbleIntensity>) -> Box<dyn Reflect>
fn into_reflect(self: Box<GamepadRumbleIntensity>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl Struct for GamepadRumbleIntensity
impl Struct for GamepadRumbleIntensity
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.impl StructuralPartialEq for GamepadRumbleIntensity
Sourceimpl TypePath for GamepadRumbleIntensity
impl TypePath for GamepadRumbleIntensity
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>