Touches

Struct Touches 

pub struct Touches { /* private fields */ }
Available on crate features mouse or touch only.
A collection of Touches.

Usage

It is used to create a bevy resource that stores the data of the touches on a touchscreen and can be accessed inside of a system.

Updating

The resource is updated inside of the touch_screen_input_system.

Implementations

Source

impl Touches

pub fn iter(&self) -> impl Iterator<Item = &Touch>

An iterator visiting every pressed Touch input in arbitrary order.

Examples found in repository?
examples/input/touch_input.rs (line 34)
12fn touch_system(touches: Res<Touches>) {
13    for touch in touches.iter_just_pressed() {
14        info!(
15            "just pressed touch with id: {}, at: {}",
16            touch.id(),
17            touch.position()
18        );
19    }
20
21    for touch in touches.iter_just_released() {
22        info!(
23            "just released touch with id: {}, at: {}",
24            touch.id(),
25            touch.position()
26        );
27    }
28
29    for touch in touches.iter_just_canceled() {
30        info!("canceled touch with id: {}", touch.id());
31    }
32
33    // you can also iterate all current touches and retrieve their state like this:
34    for touch in touches.iter() {
35        info!("active touch: {touch:?}");
36        info!("  just_pressed: {}", touches.just_pressed(touch.id()));
37    }
38}
pub fn get_pressed(&self, id: u64) -> Option<&Touch>

Returns the Touch input corresponding to the id if it is being pressed.

pub fn any_just_pressed(&self) -> bool

Checks if any touch input was just pressed.

Examples found in repository?
examples/ecs/run_conditions.rs (line 75)
66fn has_user_input(
67    keyboard_input: Res<ButtonInput<KeyCode>>,
68    mouse_button_input: Res<ButtonInput<MouseButton>>,
69    touch_input: Res<Touches>,
70) -> bool {
71    keyboard_input.just_pressed(KeyCode::Space)
72        || keyboard_input.just_pressed(KeyCode::Enter)
73        || mouse_button_input.just_pressed(MouseButton::Left)
74        || mouse_button_input.just_pressed(MouseButton::Right)
75        || touch_input.any_just_pressed()
76}
pub fn release(&mut self, id: u64)

Register a release for a given touch input.

pub fn release_all(&mut self)

Registers a release for all currently pressed touch inputs.

pub fn just_pressed(&self, id: u64) -> bool

Returns true if the input corresponding to the id has just been pressed.

Examples found in repository?
examples/input/touch_input.rs (line 36)
12fn touch_system(touches: Res<Touches>) {
13    for touch in touches.iter_just_pressed() {
14        info!(
15            "just pressed touch with id: {}, at: {}",
16            touch.id(),
17            touch.position()
18        );
19    }
20
21    for touch in touches.iter_just_released() {
22        info!(
23            "just released touch with id: {}, at: {}",
24            touch.id(),
25            touch.position()
26        );
27    }
28
29    for touch in touches.iter_just_canceled() {
30        info!("canceled touch with id: {}", touch.id());
31    }
32
33    // you can also iterate all current touches and retrieve their state like this:
34    for touch in touches.iter() {
35        info!("active touch: {touch:?}");
36        info!("  just_pressed: {}", touches.just_pressed(touch.id()));
37    }
38}
pub fn clear_just_pressed(&mut self, id: u64) -> bool

Clears the just_pressed state of the touch input and returns true if the touch input has just been pressed.

Future calls to Touches::just_pressed for the given touch input will return false until a new press event occurs.

pub fn iter_just_pressed(&self) -> impl Iterator<Item = &Touch>

An iterator visiting every just pressed Touch input in arbitrary order.

Examples found in repository?
examples/input/touch_input.rs (line 13)
12fn touch_system(touches: Res<Touches>) {
13    for touch in touches.iter_just_pressed() {
14        info!(
15            "just pressed touch with id: {}, at: {}",
16            touch.id(),
17            touch.position()
18        );
19    }
20
21    for touch in touches.iter_just_released() {
22        info!(
23            "just released touch with id: {}, at: {}",
24            touch.id(),
25            touch.position()
26        );
27    }
28
29    for touch in touches.iter_just_canceled() {
30        info!("canceled touch with id: {}", touch.id());
31    }
32
33    // you can also iterate all current touches and retrieve their state like this:
34    for touch in touches.iter() {
35        info!("active touch: {touch:?}");
36        info!("  just_pressed: {}", touches.just_pressed(touch.id()));
37    }
38}
pub fn get_released(&self, id: u64) -> Option<&Touch>

Returns the Touch input corresponding to the id if it has just been released.

pub fn any_just_released(&self) -> bool

Checks if any touch input was just released.

pub fn just_released(&self, id: u64) -> bool

Returns true if the input corresponding to the id has just been released.

pub fn clear_just_released(&mut self, id: u64) -> bool

Clears the just_released state of the touch input and returns true if the touch input has just been released.

Future calls to Touches::just_released for the given touch input will return false until a new release event occurs.

pub fn iter_just_released(&self) -> impl Iterator<Item = &Touch>

An iterator visiting every just released Touch input in arbitrary order.

Examples found in repository?
examples/input/touch_input.rs (line 21)
12fn touch_system(touches: Res<Touches>) {
13    for touch in touches.iter_just_pressed() {
14        info!(
15            "just pressed touch with id: {}, at: {}",
16            touch.id(),
17            touch.position()
18        );
19    }
20
21    for touch in touches.iter_just_released() {
22        info!(
23            "just released touch with id: {}, at: {}",
24            touch.id(),
25            touch.position()
26        );
27    }
28
29    for touch in touches.iter_just_canceled() {
30        info!("canceled touch with id: {}", touch.id());
31    }
32
33    // you can also iterate all current touches and retrieve their state like this:
34    for touch in touches.iter() {
35        info!("active touch: {touch:?}");
36        info!("  just_pressed: {}", touches.just_pressed(touch.id()));
37    }
38}
pub fn any_just_canceled(&self) -> bool

Checks if any touch input was just canceled.

pub fn just_canceled(&self, id: u64) -> bool

Returns true if the input corresponding to the id has just been canceled.

pub fn clear_just_canceled(&mut self, id: u64) -> bool

Clears the just_canceled state of the touch input and returns true if the touch input has just been canceled.

Future calls to Touches::just_canceled for the given touch input will return false until a new cancel event occurs.

pub fn iter_just_canceled(&self) -> impl Iterator<Item = &Touch>

An iterator visiting every just canceled Touch input in arbitrary order.

Examples found in repository?
examples/input/touch_input.rs (line 29)
12fn touch_system(touches: Res<Touches>) {
13    for touch in touches.iter_just_pressed() {
14        info!(
15            "just pressed touch with id: {}, at: {}",
16            touch.id(),
17            touch.position()
18        );
19    }
20
21    for touch in touches.iter_just_released() {
22        info!(
23            "just released touch with id: {}, at: {}",
24            touch.id(),
25            touch.position()
26        );
27    }
28
29    for touch in touches.iter_just_canceled() {
30        info!("canceled touch with id: {}", touch.id());
31    }
32
33    // you can also iterate all current touches and retrieve their state like this:
34    for touch in touches.iter() {
35        info!("active touch: {touch:?}");
36        info!("  just_pressed: {}", touches.just_pressed(touch.id()));
37    }
38}
pub fn first_pressed_position(&self) -> Option<Vec2>

Retrieves the position of the first currently pressed touch, if any

pub fn clear(&mut self)

Clears just_pressed, just_released, and just_canceled data for every touch input.

See also Touches::reset_all for a full reset.

pub fn reset_all(&mut self)

Clears pressed, just_pressed, just_released, and just_canceled data for every touch input.

See also Touches::clear for clearing only touches that have just been pressed, released or canceled.