pub struct CascadeShadowConfigBuilder {
pub num_cascades: usize,
pub minimum_distance: f32,
pub maximum_distance: f32,
pub first_cascade_far_bound: f32,
pub overlap_proportion: f32,
}Expand description
Builder for CascadeShadowConfig.
Fields
num_cascades: usizeThe number of shadow cascades. More cascades increases shadow quality by mitigating perspective aliasing - a phenomenon where areas nearer the camera are covered by fewer shadow map texels than areas further from the camera, causing blocky looking shadows.
This does come at the cost increased rendering overhead, however this overhead is still less than if you were to use fewer cascades and much larger shadow map textures to achieve the same quality level.
In case rendered geometry covers a relatively narrow and static depth relative to camera, it may
make more sense to use fewer cascades and a higher resolution shadow map texture as perspective aliasing
is not as much an issue. Be sure to adjust minimum_distance and maximum_distance appropriately.
minimum_distance: f32The minimum shadow distance, which can help improve the texel resolution of the first cascade. Areas nearer to the camera than this will likely receive no shadows.
NOTE: Due to implementation details, this usually does not impact shadow quality as much as
first_cascade_far_bound and maximum_distance. At many view frustum field-of-views, the
texel resolution of the first cascade is dominated by the width / height of the view frustum plane
at first_cascade_far_bound rather than the depth of the frustum from minimum_distance to
first_cascade_far_bound.
maximum_distance: f32The maximum shadow distance. Areas further from the camera than this will likely receive no shadows.
first_cascade_far_bound: f32Sets the far bound of the first cascade, relative to the view origin. In-between cascades will be exponentially spaced relative to the maximum shadow distance. NOTE: This is ignored if there is only one cascade, the maximum distance takes precedence.
overlap_proportion: f32Sets the overlap proportion between cascades. The overlap is used to make the transition from one cascade’s shadow map to the next less abrupt by blending between both shadow maps.
Implementations
Sourceimpl CascadeShadowConfigBuilder
impl CascadeShadowConfigBuilder
Sourcepub fn build(&self) -> CascadeShadowConfig
pub fn build(&self) -> CascadeShadowConfig
Returns the cascade config as specified by this builder.
Examples found in repository?
18fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
19 commands.spawn((
20 Camera3d::default(),
21 Transform::from_xyz(0.7, 0.7, 1.0).looking_at(Vec3::new(0.0, 0.3, 0.0), Vec3::Y),
22 EnvironmentMapLight {
23 diffuse_map: asset_server.load("environment_maps/pisa_diffuse_rgb9e5_zstd.ktx2"),
24 specular_map: asset_server.load("environment_maps/pisa_specular_rgb9e5_zstd.ktx2"),
25 intensity: 250.0,
26 ..default()
27 },
28 ));
29
30 commands.spawn((
31 DirectionalLight {
32 shadows_enabled: true,
33 ..default()
34 },
35 // This is a relatively small scene, so use tighter shadow
36 // cascade bounds than the default for better quality.
37 // We also adjusted the shadow map to be larger since we're
38 // only using a single cascade.
39 CascadeShadowConfigBuilder {
40 num_cascades: 1,
41 maximum_distance: 1.6,
42 ..default()
43 }
44 .build(),
45 ));
46 commands.spawn(SceneRoot(asset_server.load(
47 GltfAssetLabel::Scene(0).from_asset("models/FlightHelmet/FlightHelmet.gltf"),
48 )));
49}Trait Implementations
Sourceimpl Default for CascadeShadowConfigBuilder
impl Default for CascadeShadowConfigBuilder
Sourcefn default() -> CascadeShadowConfigBuilder
fn default() -> CascadeShadowConfigBuilder
Sourceimpl From<CascadeShadowConfigBuilder> for CascadeShadowConfig
impl From<CascadeShadowConfigBuilder> for CascadeShadowConfig
Sourcefn from(builder: CascadeShadowConfigBuilder) -> CascadeShadowConfig
fn from(builder: CascadeShadowConfigBuilder) -> CascadeShadowConfig