pub struct Clusters {
pub tile_size: UVec2,
pub dimensions: UVec3,
pub near: f32,
pub far: f32,
pub last_frame_farthest_z: Option<f32>,
pub last_frame_total_cluster_index_count: Option<usize>,
pub clusterable_objects: ClusterableObjects,
}bevy_light only.Expand description
The cluster geometry generated by ClusterConfig.
Fields
tile_size: UVec2The dimensions of the rectangle the cluster occupies in screen-space, rounded up to the nearest pixel.
dimensions: UVec3Number of clusters in X / Y / Z in the view frustum
near: f32Distance to the far plane of the first depth slice. The first depth slice is special and explicitly-configured to avoid having unnecessarily many slices close to the camera.
far: f32Distance to the far plane of the last depth slice. This may change depending on ClusterZConfig used.
last_frame_farthest_z: Option<f32>The farthest Z value of any bounding sphere of any clusterable object on the previous frame.
This is used for the ClusterFarZMode::MaxClusterableObjectRange
feature.
last_frame_total_cluster_index_count: Option<usize>The sum of the number of objects that all clusters contained last frame.
This is used for the dynamic_resizing feature, which automatically
grows the number of clusters if clusters have likely become too large.
clusterable_objects: ClusterableObjectsAll objects within the cluster.