Event

Struct Event 

pub struct Event<'a> { /* private fields */ }
Events represent single points in time where something occurred during the execution of a program.

An Event can be compared to a log record in unstructured logging, but with two key differences:

  • Events exist within the context of a span. Unlike log lines, they may be located within the trace tree, allowing visibility into the temporal context in which the event occurred, as well as the source code location.
  • Like spans, Events have structured key-value data known as fields, which may include textual message. In general, a majority of the data associated with an event should be in the event’s fields rather than in the textual message, as the fields are more structured.

Implementations

Source

impl<'a> Event<'a>

pub fn dispatch(metadata: &'static Metadata<'static>, fields: &'a ValueSet<'_>)

Constructs a new Event with the specified metadata and set of values, and observes it with the current subscriber.

pub fn new( metadata: &'static Metadata<'static>, fields: &'a ValueSet<'a>, ) -> Event<'a>

Returns a new Event in the current span, with the specified metadata and set of values.

pub fn new_child_of( parent: impl Into<Option<Id>>, metadata: &'static Metadata<'static>, fields: &'a ValueSet<'a>, ) -> Event<'a>

Returns a new Event as a child of the specified span, with the provided metadata and set of values.

pub fn child_of( parent: impl Into<Option<Id>>, metadata: &'static Metadata<'static>, fields: &'a ValueSet<'_>, )

Constructs a new Event with the specified metadata and set of values, and observes it with the current subscriber and an explicit parent.

pub fn record(&self, visitor: &mut dyn Visit)

Visits all the fields on this Event with the specified visitor.

Examples found in repository?
examples/app/log_layers_ecs.rs (line 77)
68    fn on_event(
69        &self,
70        event: &tracing::Event<'_>,
71        _ctx: tracing_subscriber::layer::Context<'_, S>,
72    ) {
73        // In order to obtain the log message, we have to create a struct that implements
74        // Visit and holds a reference to our string. Then we use the `record` method and
75        // the struct to modify the reference to hold the message string.
76        let mut message = None;
77        event.record(&mut CaptureLayerVisitor(&mut message));
78        if let Some(message) = message {
79            let metadata = event.metadata();
80
81            self.sender
82                .send(LogMessage {
83                    message,
84                    level: *metadata.level(),
85                })
86                .expect("LogEvents resource no longer exists!");
87        }
88    }
pub fn fields(&self) -> Iter

Returns an iterator over the set of values on this Event.

pub fn metadata(&self) -> &'static Metadata<'static>

Returns metadata describing this Event.

Examples found in repository?
examples/app/log_layers.rs (line 21)
15    fn on_event(
16        &self,
17        event: &tracing::Event<'_>,
18        _ctx: bevy::log::tracing_subscriber::layer::Context<'_, S>,
19    ) {
20        println!("Got event!");
21        println!("  level={}", event.metadata().level());
22        println!("  target={}", event.metadata().target());
23        println!("  name={}", event.metadata().name());
24    }
More examples
Hide additional examples
examples/app/log_layers_ecs.rs (line 79)
68    fn on_event(
69        &self,
70        event: &tracing::Event<'_>,
71        _ctx: tracing_subscriber::layer::Context<'_, S>,
72    ) {
73        // In order to obtain the log message, we have to create a struct that implements
74        // Visit and holds a reference to our string. Then we use the `record` method and
75        // the struct to modify the reference to hold the message string.
76        let mut message = None;
77        event.record(&mut CaptureLayerVisitor(&mut message));
78        if let Some(message) = message {
79            let metadata = event.metadata();
80
81            self.sender
82                .send(LogMessage {
83                    message,
84                    level: *metadata.level(),
85                })
86                .expect("LogEvents resource no longer exists!");
87        }
88    }
pub fn is_root(&self) -> bool

Returns true if the new event should be a root.

pub fn is_contextual(&self) -> bool

Returns true if the new event’s parent should be determined based on the current context.

If this is true and the current thread is currently inside a span, then that span should be the new event’s parent. Otherwise, if the current thread is not inside a span, then the new event will be the root of its own trace tree.

pub fn parent(&self) -> Option<&Id>

Returns the new event’s explicitly-specified parent, if there is one.

Otherwise (if the new event is a root or is a child of the current span), returns None.