pub struct ComputePipelineDescriptor {
pub label: Option<Cow<'static, str>>,
pub layout: Vec<BindGroupLayoutDescriptor>,
pub immediate_size: u32,
pub shader: Handle<Shader>,
pub shader_defs: Vec<ShaderDefVal>,
pub entry_point: Option<Cow<'static, str>>,
pub zero_initialize_workgroup_memory: bool,
}Expand description
Describes a compute pipeline.
Fields
label: Option<Cow<'static, str>>layout: Vec<BindGroupLayoutDescriptor>immediate_size: u32shader: Handle<Shader>The compiled shader module for this stage.
shader_defs: Vec<ShaderDefVal>entry_point: Option<Cow<'static, str>>The name of the entry point in the compiled shader, or None if the default entry point
is used.
zero_initialize_workgroup_memory: boolWhether to zero-initialize workgroup memory by default. If you’re not sure, set this to true. If this is false, reading from workgroup variables before writing to them will result in garbage values.