pub struct FunctionCurve<T, F> { /* private fields */ }Available on crate feature
curve only.Expand description
A curve defined by a function together with a fixed domain.
This is a curve that holds an inner function f which takes numbers (f32) as input and produces
output of type T. The value of this curve when sampled at time t is just f(t).
Implementations
Sourceimpl<T, F> FunctionCurve<T, F>
impl<T, F> FunctionCurve<T, F>
Sourcepub fn new(domain: Interval, function: F) -> FunctionCurve<T, F>
pub fn new(domain: Interval, function: F) -> FunctionCurve<T, F>
Create a new curve with the given domain from the given function. When sampled, the
function is evaluated at the sample time to compute the output.
Examples found in repository?
examples/gizmos/2d_gizmos.rs (line 73)
41fn draw_example_collection(
42 mut gizmos: Gizmos,
43 mut my_gizmos: Gizmos<MyRoundGizmos>,
44 time: Res<Time>,
45) {
46 let sin_t_scaled = ops::sin(time.elapsed_secs()) * 50.;
47 gizmos.line_2d(Vec2::Y * -sin_t_scaled, Vec2::splat(-80.), RED);
48 gizmos.ray_2d(Vec2::Y * sin_t_scaled, Vec2::splat(80.), LIME);
49
50 gizmos
51 .grid_2d(
52 Isometry2d::IDENTITY,
53 UVec2::new(16, 9),
54 Vec2::new(80., 80.),
55 // Dark gray
56 LinearRgba::gray(0.05),
57 )
58 .outer_edges();
59
60 // Triangle
61 gizmos.linestrip_gradient_2d([
62 (Vec2::Y * 300., BLUE),
63 (Vec2::new(-255., -155.), RED),
64 (Vec2::new(255., -155.), LIME),
65 (Vec2::Y * 300., BLUE),
66 ]);
67
68 gizmos.rect_2d(Isometry2d::IDENTITY, Vec2::splat(650.), BLACK);
69
70 gizmos.cross_2d(Vec2::new(-160., 120.), 12., FUCHSIA);
71
72 let domain = Interval::EVERYWHERE;
73 let curve = FunctionCurve::new(domain, |t| Vec2::new(t, ops::sin(t / 25.0) * 100.0));
74 let resolution = ((ops::sin(time.elapsed_secs()) + 1.0) * 50.0) as usize;
75 let times_and_colors = (0..=resolution)
76 .map(|n| n as f32 / resolution as f32)
77 .map(|t| (t - 0.5) * 600.0)
78 .map(|t| (t, TEAL.mix(&HOT_PINK, (t + 300.0) / 600.0)));
79 gizmos.curve_gradient_2d(curve, times_and_colors);
80
81 my_gizmos
82 .rounded_rect_2d(Isometry2d::IDENTITY, Vec2::splat(630.), BLACK)
83 .corner_radius(ops::cos(time.elapsed_secs() / 3.) * 100.);
84
85 // Circles have 32 line-segments by default.
86 // You may want to increase this for larger circles.
87 my_gizmos
88 .circle_2d(Isometry2d::IDENTITY, 300., NAVY)
89 .resolution(64);
90
91 my_gizmos.ellipse_2d(
92 Rot2::radians(time.elapsed_secs() % TAU),
93 Vec2::new(100., 200.),
94 YELLOW_GREEN,
95 );
96
97 // Arcs default resolution is linearly interpolated between
98 // 1 and 32, using the arc length as scalar.
99 my_gizmos.arc_2d(
100 Rot2::radians(sin_t_scaled / 10.),
101 FRAC_PI_2,
102 310.,
103 ORANGE_RED,
104 );
105 my_gizmos.arc_2d(Isometry2d::IDENTITY, FRAC_PI_2, 80.0, ORANGE_RED);
106 my_gizmos.long_arc_2d_between(Vec2::ZERO, Vec2::X * 20.0, Vec2::Y * 20.0, ORANGE_RED);
107 my_gizmos.short_arc_2d_between(Vec2::ZERO, Vec2::X * 40.0, Vec2::Y * 40.0, ORANGE_RED);
108
109 gizmos.arrow_2d(
110 Vec2::ZERO,
111 Vec2::from_angle(sin_t_scaled / -10. + PI / 2.) * 50.,
112 YELLOW,
113 );
114
115 // You can create more complex arrows using the arrow builder.
116 gizmos
117 .arrow_2d(
118 Vec2::ZERO,
119 Vec2::from_angle(sin_t_scaled / -10.) * 50.,
120 GREEN,
121 )
122 .with_double_end()
123 .with_tip_length(10.);
124}More examples
examples/gizmos/3d_gizmos.rs (lines 150-152)
99fn draw_example_collection(
100 mut gizmos: Gizmos,
101 mut my_gizmos: Gizmos<MyRoundGizmos>,
102 time: Res<Time>,
103) {
104 gizmos.grid(
105 Quat::from_rotation_x(PI / 2.),
106 UVec2::splat(20),
107 Vec2::new(2., 2.),
108 // Light gray
109 LinearRgba::gray(0.65),
110 );
111 gizmos.grid(
112 Isometry3d::new(Vec3::splat(10.0), Quat::from_rotation_x(PI / 3. * 2.)),
113 UVec2::splat(20),
114 Vec2::new(2., 2.),
115 PURPLE,
116 );
117 gizmos.sphere(Vec3::splat(10.0), 1.0, PURPLE);
118
119 gizmos
120 .primitive_3d(
121 &Plane3d {
122 normal: Dir3::Y,
123 half_size: Vec2::splat(1.0),
124 },
125 Isometry3d::new(
126 Vec3::splat(4.0) + Vec2::from(ops::sin_cos(time.elapsed_secs())).extend(0.0),
127 Quat::from_rotation_x(PI / 2. + time.elapsed_secs()),
128 ),
129 GREEN,
130 )
131 .cell_count(UVec2::new(5, 10))
132 .spacing(Vec2::new(0.2, 0.1));
133
134 gizmos.cube(
135 Transform::from_translation(Vec3::Y * 0.5).with_scale(Vec3::splat(1.25)),
136 BLACK,
137 );
138 gizmos.rect(
139 Isometry3d::new(
140 Vec3::new(ops::cos(time.elapsed_secs()) * 2.5, 1., 0.),
141 Quat::from_rotation_y(PI / 2.),
142 ),
143 Vec2::splat(2.),
144 LIME,
145 );
146
147 gizmos.cross(Vec3::new(-1., 1., 1.), 0.5, FUCHSIA);
148
149 let domain = Interval::EVERYWHERE;
150 let curve = FunctionCurve::new(domain, |t| {
151 (Vec2::from(ops::sin_cos(t * 10.0))).extend(t - 6.0)
152 });
153 let resolution = ((ops::sin(time.elapsed_secs()) + 1.0) * 100.0) as usize;
154 let times_and_colors = (0..=resolution)
155 .map(|n| n as f32 / resolution as f32)
156 .map(|t| t * 5.0)
157 .map(|t| (t, TEAL.mix(&HOT_PINK, t / 5.0)));
158 gizmos.curve_gradient_3d(curve, times_and_colors);
159
160 my_gizmos.sphere(Vec3::new(1., 0.5, 0.), 0.5, RED);
161
162 my_gizmos
163 .rounded_cuboid(Vec3::new(-2.0, 0.75, -0.75), Vec3::splat(0.9), TURQUOISE)
164 .edge_radius(0.1)
165 .arc_resolution(4);
166
167 for y in [0., 0.5, 1.] {
168 gizmos.ray(
169 Vec3::new(1., y, 0.),
170 Vec3::new(-3., ops::sin(time.elapsed_secs() * 3.), 0.),
171 BLUE,
172 );
173 }
174
175 my_gizmos
176 .arc_3d(
177 180.0_f32.to_radians(),
178 0.2,
179 Isometry3d::new(
180 Vec3::ONE,
181 Quat::from_rotation_arc(Vec3::Y, Vec3::ONE.normalize()),
182 ),
183 ORANGE,
184 )
185 .resolution(10);
186
187 // Circles have 32 line-segments by default.
188 my_gizmos.circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3., BLACK);
189
190 // You may want to increase this for larger circles or spheres.
191 my_gizmos
192 .circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3.1, NAVY)
193 .resolution(64);
194 my_gizmos
195 .sphere(Isometry3d::IDENTITY, 3.2, BLACK)
196 .resolution(64);
197
198 gizmos.arrow(Vec3::ZERO, Vec3::splat(1.5), YELLOW);
199
200 // You can create more complex arrows using the arrow builder.
201 gizmos
202 .arrow(Vec3::new(2., 0., 2.), Vec3::new(2., 2., 2.), ORANGE_RED)
203 .with_double_end()
204 .with_tip_length(0.5);
205}Trait Implementations
Sourceimpl<T, F> Clone for FunctionCurve<T, F>
impl<T, F> Clone for FunctionCurve<T, F>
Sourcefn clone(&self) -> FunctionCurve<T, F>
fn clone(&self) -> FunctionCurve<T, F>
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<T, F> Curve<T> for FunctionCurve<T, F>
impl<T, F> Curve<T> for FunctionCurve<T, F>
Sourcefn sample_unchecked(&self, t: f32) -> T
fn sample_unchecked(&self, t: f32) -> T
Sample a point on this curve at the parameter value
t, extracting the associated value.
This is the unchecked version of sampling, which should only be used if the sample time t
is already known to lie within the curve’s domain. Read moreSourcefn sample(&self, t: f32) -> Option<T>
fn sample(&self, t: f32) -> Option<T>
Sample a point on this curve at the parameter value
t, returning None if the point is
outside of the curve’s domain.Sourcefn sample_clamped(&self, t: f32) -> T
fn sample_clamped(&self, t: f32) -> T
Sample a point on this curve at the parameter value
t, clamping t to lie inside the
domain of the curve.Sourceimpl<T, F> Debug for FunctionCurve<T, F>
impl<T, F> Debug for FunctionCurve<T, F>
Sourceimpl<'de, T, F> Deserialize<'de> for FunctionCurve<T, F>where
F: Deserialize<'de>,
impl<'de, T, F> Deserialize<'de> for FunctionCurve<T, F>where
F: Deserialize<'de>,
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<FunctionCurve<T, F>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<FunctionCurve<T, F>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl<T, F> FromArg for FunctionCurve<T, F>
impl<T, F> FromArg for FunctionCurve<T, F>
Sourceimpl<T, F> GetOwnership for FunctionCurve<T, F>
impl<T, F> GetOwnership for FunctionCurve<T, F>
Sourceimpl<T, F> GetTypeRegistration for FunctionCurve<T, F>
impl<T, F> GetTypeRegistration for FunctionCurve<T, F>
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl<T, F> IntoReturn for FunctionCurve<T, F>
impl<T, F> IntoReturn for FunctionCurve<T, F>
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
FunctionCurve<T, F>: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
FunctionCurve<T, F>: 'into_return,
Sourceimpl<T, F> PartialReflect for FunctionCurve<T, F>
impl<T, F> PartialReflect for FunctionCurve<T, F>
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<FunctionCurve<T, F>>) -> ReflectOwned
fn reflect_owned(self: Box<FunctionCurve<T, F>>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<FunctionCurve<T, F>>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<FunctionCurve<T, F>>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(
self: Box<FunctionCurve<T, F>>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<FunctionCurve<T, F>>, ) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl<T, F> Reflect for FunctionCurve<T, F>
impl<T, F> Reflect for FunctionCurve<T, F>
Sourcefn into_any(self: Box<FunctionCurve<T, F>>) -> Box<dyn Any>
fn into_any(self: Box<FunctionCurve<T, F>>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<FunctionCurve<T, F>>) -> Box<dyn Reflect>
fn into_reflect(self: Box<FunctionCurve<T, F>>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
Sourceimpl<T, F> Serialize for FunctionCurve<T, F>where
F: Serialize,
impl<T, F> Serialize for FunctionCurve<T, F>where
F: Serialize,
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Serialize this value into the given Serde serializer. Read more
Sourceimpl<T, F> Struct for FunctionCurve<T, F>
impl<T, F> Struct for FunctionCurve<T, F>
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field named
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field named
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field with index
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field with index
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Gets the index of the field with the given name.
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Returns an iterator over the values of the reflectable fields for this struct.
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
Creates a new
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
Will return
None if TypeInfo is not available.Sourceimpl<T, F> TypePath for FunctionCurve<T, F>where
T: TypePath,
F: 'static,
Available on crate feature bevy_reflect only.Note: This is not a fully stable implementation of TypePath due to usage of type_name
for function members.
impl<T, F> TypePath for FunctionCurve<T, F>where
T: TypePath,
F: 'static,
Available on crate feature
bevy_reflect only.Note: This is not a fully stable implementation of TypePath due to usage of type_name
for function members.
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>