#[repr(C)]pub struct Affine2 {
pub matrix2: Mat2,
pub translation: Vec2,
}Expand description
A 2D affine transform, which can represent translation, rotation, scaling and shear.
Fields
matrix2: Mat2translation: Vec2Implementations
Sourceimpl Affine2
impl Affine2
Sourcepub const ZERO: Affine2
pub const ZERO: Affine2
The degenerate zero transform.
This transforms any finite vector and point to zero. The zero transform is non-invertible.
Sourcepub const IDENTITY: Affine2
pub const IDENTITY: Affine2
The identity transform.
Multiplying a vector with this returns the same vector.
Sourcepub const fn from_cols(x_axis: Vec2, y_axis: Vec2, z_axis: Vec2) -> Affine2
pub const fn from_cols(x_axis: Vec2, y_axis: Vec2, z_axis: Vec2) -> Affine2
Creates an affine transform from three column vectors.
Sourcepub fn from_cols_array(m: &[f32; 6]) -> Affine2
pub fn from_cols_array(m: &[f32; 6]) -> Affine2
Creates an affine transform from a [f32; 6] array stored in column major order.
Sourcepub fn to_cols_array(&self) -> [f32; 6]
pub fn to_cols_array(&self) -> [f32; 6]
Creates a [f32; 6] array storing data in column major order.
Sourcepub fn from_cols_array_2d(m: &[[f32; 2]; 3]) -> Affine2
pub fn from_cols_array_2d(m: &[[f32; 2]; 3]) -> Affine2
Creates an affine transform from a [[f32; 2]; 3]
2D array stored in column major order.
If your data is in row major order you will need to transpose the returned
matrix.
Sourcepub fn to_cols_array_2d(&self) -> [[f32; 2]; 3]
pub fn to_cols_array_2d(&self) -> [[f32; 2]; 3]
Creates a [[f32; 2]; 3] 2D array storing data in
column major order.
If you require data in row major order transpose the matrix first.
Sourcepub fn from_cols_slice(slice: &[f32]) -> Affine2
pub fn from_cols_slice(slice: &[f32]) -> Affine2
Creates an affine transform from the first 6 values in slice.
Panics
Panics if slice is less than 6 elements long.
Sourcepub fn write_cols_to_slice(self, slice: &mut [f32])
pub fn write_cols_to_slice(self, slice: &mut [f32])
Writes the columns of self to the first 6 elements in slice.
Panics
Panics if slice is less than 6 elements long.
Sourcepub fn from_scale(scale: Vec2) -> Affine2
pub fn from_scale(scale: Vec2) -> Affine2
Creates an affine transform that changes scale. Note that if any scale is zero the transform will be non-invertible.
Examples found in repository?
17fn setup(
18 mut commands: Commands,
19 asset_server: Res<AssetServer>,
20 mut meshes: ResMut<Assets<Mesh>>,
21 mut materials: ResMut<Assets<StandardMaterial>>,
22) {
23 let image_with_default_sampler =
24 asset_server.load("textures/fantasy_ui_borders/panel-border-010.png");
25
26 // central cube with not repeated texture
27 commands.spawn((
28 Mesh3d(meshes.add(Cuboid::new(1.0, 1.0, 1.0))),
29 MeshMaterial3d(materials.add(StandardMaterial {
30 base_color_texture: Some(image_with_default_sampler.clone()),
31 ..default()
32 })),
33 Transform::from_translation(Vec3::ZERO),
34 ));
35
36 // left cube with repeated texture
37 commands.spawn((
38 Mesh3d(meshes.add(Cuboid::new(1.0, 1.0, 1.0))),
39 MeshMaterial3d(materials.add(StandardMaterial {
40 base_color_texture: Some(asset_server.load_with_settings(
41 "textures/fantasy_ui_borders/panel-border-010-repeated.png",
42 |s: &mut _| {
43 *s = ImageLoaderSettings {
44 sampler: ImageSampler::Descriptor(ImageSamplerDescriptor {
45 // rewriting mode to repeat image,
46 address_mode_u: ImageAddressMode::Repeat,
47 address_mode_v: ImageAddressMode::Repeat,
48 ..default()
49 }),
50 ..default()
51 }
52 },
53 )),
54
55 // uv_transform used here for proportions only, but it is full Affine2
56 // that's why you can use rotation and shift also
57 uv_transform: Affine2::from_scale(Vec2::new(2., 3.)),
58 ..default()
59 })),
60 Transform::from_xyz(-1.5, 0.0, 0.0),
61 ));
62
63 // right cube with scaled texture, because with default sampler
64 commands.spawn((
65 Mesh3d(meshes.add(Cuboid::new(1.0, 1.0, 1.0))),
66 MeshMaterial3d(materials.add(StandardMaterial {
67 // there is no sampler set, that's why
68 // by default you see only one small image in a row/column
69 // and other space is filled by image edge
70 base_color_texture: Some(image_with_default_sampler),
71
72 // uv_transform used here for proportions only, but it is full Affine2
73 // that's why you can use rotation and shift also
74 uv_transform: Affine2::from_scale(Vec2::new(2., 3.)),
75 ..default()
76 })),
77 Transform::from_xyz(1.5, 0.0, 0.0),
78 ));
79
80 // light
81 commands.spawn((
82 PointLight {
83 shadows_enabled: true,
84 ..default()
85 },
86 Transform::from_xyz(4.0, 8.0, 4.0),
87 ));
88 // camera
89 commands.spawn((
90 Camera3d::default(),
91 Transform::from_xyz(0.0, 1.5, 4.0).looking_at(Vec3::ZERO, Vec3::Y),
92 ));
93}More examples
25fn setup(
26 mut commands: Commands,
27 asset_server: Res<AssetServer>,
28 mut meshes: ResMut<Assets<Mesh>>,
29 mut materials: ResMut<Assets<ColorMaterial>>,
30) {
31 // #11111: We use a duplicated image so that it can be load with and without
32 // settings
33 let image_with_default_sampler =
34 asset_server.load("textures/fantasy_ui_borders/panel-border-010.png");
35 let image_with_repeated_sampler = asset_server.load_with_settings(
36 "textures/fantasy_ui_borders/panel-border-010-repeated.png",
37 |s: &mut _| {
38 *s = ImageLoaderSettings {
39 sampler: ImageSampler::Descriptor(ImageSamplerDescriptor {
40 // rewriting mode to repeat image,
41 address_mode_u: ImageAddressMode::Repeat,
42 address_mode_v: ImageAddressMode::Repeat,
43 ..default()
44 }),
45 ..default()
46 }
47 },
48 );
49
50 // central rectangle with not repeated texture
51 commands.spawn((
52 Mesh2d(meshes.add(Rectangle::new(RECTANGLE_SIDE, RECTANGLE_SIDE))),
53 MeshMaterial2d(materials.add(ColorMaterial {
54 texture: Some(image_with_default_sampler.clone()),
55 ..default()
56 })),
57 Transform::from_translation(Vec3::ZERO),
58 children![(
59 Text2d::new("Control"),
60 Transform::from_xyz(0., LABEL_OFFSET, 0.),
61 )],
62 ));
63
64 // left rectangle with repeated texture
65 commands.spawn((
66 Mesh2d(meshes.add(Rectangle::new(RECTANGLE_SIDE, RECTANGLE_SIDE))),
67 MeshMaterial2d(materials.add(ColorMaterial {
68 texture: Some(image_with_repeated_sampler),
69 // uv_transform used here for proportions only, but it is full Affine2
70 // that's why you can use rotation and shift also
71 uv_transform: Affine2::from_scale(Vec2::new(2., 3.)),
72 ..default()
73 })),
74 Transform::from_xyz(-RECTANGLE_OFFSET, 0.0, 0.0),
75 children![(
76 Text2d::new("Repeat On"),
77 Transform::from_xyz(0., LABEL_OFFSET, 0.),
78 )],
79 ));
80
81 // right rectangle with scaled texture, but with default sampler.
82 commands.spawn((
83 Mesh2d(meshes.add(Rectangle::new(RECTANGLE_SIDE, RECTANGLE_SIDE))),
84 MeshMaterial2d(materials.add(ColorMaterial {
85 // there is no sampler set, that's why
86 // by default you see only one small image in a row/column
87 // and other space is filled by image edge
88 texture: Some(image_with_default_sampler),
89
90 // uv_transform used here for proportions only, but it is full Affine2
91 // that's why you can use rotation and shift also
92 uv_transform: Affine2::from_scale(Vec2::new(2., 3.)),
93 ..default()
94 })),
95 Transform::from_xyz(RECTANGLE_OFFSET, 0.0, 0.0),
96 children![(
97 Text2d::new("Repeat Off"),
98 Transform::from_xyz(0., LABEL_OFFSET, 0.),
99 )],
100 ));
101
102 // camera
103 commands.spawn((
104 Camera2d,
105 Transform::default().looking_at(Vec3::ZERO, Vec3::Y),
106 ));
107}Sourcepub fn from_angle(angle: f32) -> Affine2
pub fn from_angle(angle: f32) -> Affine2
Creates an affine transform from the given rotation angle.
Sourcepub fn from_translation(translation: Vec2) -> Affine2
pub fn from_translation(translation: Vec2) -> Affine2
Creates an affine transformation from the given 2D translation.
Sourcepub fn from_mat2(matrix2: Mat2) -> Affine2
pub fn from_mat2(matrix2: Mat2) -> Affine2
Creates an affine transform from a 2x2 matrix (expressing scale, shear and rotation)
Sourcepub fn from_mat2_translation(matrix2: Mat2, translation: Vec2) -> Affine2
pub fn from_mat2_translation(matrix2: Mat2, translation: Vec2) -> Affine2
Creates an affine transform from a 2x2 matrix (expressing scale, shear and rotation) and a translation vector.
Equivalent to
Affine2::from_translation(translation) * Affine2::from_mat2(mat2)
Sourcepub fn from_scale_angle_translation(
scale: Vec2,
angle: f32,
translation: Vec2,
) -> Affine2
pub fn from_scale_angle_translation( scale: Vec2, angle: f32, translation: Vec2, ) -> Affine2
Creates an affine transform from the given 2D scale, rotation angle (in radians) and
translation.
Equivalent to Affine2::from_translation(translation) * Affine2::from_angle(angle) * Affine2::from_scale(scale)
Sourcepub fn from_angle_translation(angle: f32, translation: Vec2) -> Affine2
pub fn from_angle_translation(angle: f32, translation: Vec2) -> Affine2
Creates an affine transform from the given 2D rotation angle (in radians) and
translation.
Equivalent to Affine2::from_translation(translation) * Affine2::from_angle(angle)
Sourcepub fn from_mat3a(m: Mat3A) -> Affine2
pub fn from_mat3a(m: Mat3A) -> Affine2
The given Mat3A must be an affine transform,
Sourcepub fn to_scale_angle_translation(self) -> (Vec2, f32, Vec2)
pub fn to_scale_angle_translation(self) -> (Vec2, f32, Vec2)
Extracts scale, angle and translation from self.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Panics
Will panic if the determinant self.matrix2 is zero or if the resulting scale
vector contains any zero elements when glam_assert is enabled.
Sourcepub fn transform_point2(&self, rhs: Vec2) -> Vec2
pub fn transform_point2(&self, rhs: Vec2) -> Vec2
Transforms the given 2D point, applying shear, scale, rotation and translation.
Sourcepub fn transform_vector2(&self, rhs: Vec2) -> Vec2
pub fn transform_vector2(&self, rhs: Vec2) -> Vec2
Transforms the given 2D vector, applying shear, scale and rotation (but NOT translation).
To also apply translation, use Self::transform_point2() instead.
Sourcepub fn is_finite(&self) -> bool
pub fn is_finite(&self) -> bool
Returns true if, and only if, all elements are finite.
If any element is either NaN, positive or negative infinity, this will return
false.
Sourcepub fn abs_diff_eq(&self, rhs: Affine2, max_abs_diff: f32) -> bool
pub fn abs_diff_eq(&self, rhs: Affine2, max_abs_diff: f32) -> bool
Returns true if the absolute difference of all elements between self and rhs
is less than or equal to max_abs_diff.
This can be used to compare if two 3x4 matrices contain similar elements. It works
best when comparing with a known value. The max_abs_diff that should be used used
depends on the values being compared against.
For more see comparing floating point numbers.
Sourcepub fn inverse(&self) -> Affine2
pub fn inverse(&self) -> Affine2
Return the inverse of this transform.
Note that if the transform is not invertible the result will be invalid.
Sourcepub fn as_daffine2(&self) -> DAffine2
pub fn as_daffine2(&self) -> DAffine2
Casts all elements of self to f64.