pub struct Dir3(/* private fields */);Expand description
A normalized vector pointing in a direction in 3D space
Implementations
Sourceimpl Dir3
impl Dir3
Sourcepub const ALL_VERTICES: [Dir3; 8]
pub const ALL_VERTICES: [Dir3; 8]
The directions pointing towards the vertices of a cube centered at the origin.
Sourcepub const ALL_NEIGHBORS: [Dir3; 26]
pub const ALL_NEIGHBORS: [Dir3; 26]
All neighbors of a tile on a cube grid a 3x3x3 neighborhood. A combination of Self::CARDINALS, Self::ALL_EDGES and Self::ALL_VERTICES
Sourcepub fn new(value: Vec3) -> Result<Dir3, InvalidDirectionError>
pub fn new(value: Vec3) -> Result<Dir3, InvalidDirectionError>
Create a direction from a finite, nonzero Vec3, normalizing it.
Returns Err(InvalidDirectionError) if the length
of the given vector is zero (or very close to zero), infinite, or NaN.
Examples found in repository?
23fn bouncing_raycast(
24 mut ray_cast: MeshRayCast,
25 mut gizmos: Gizmos,
26 time: Res<Time>,
27 // The ray map stores rays cast by the cursor
28 ray_map: Res<RayMap>,
29) {
30 // Cast an automatically moving ray and bounce it off of surfaces
31 let t = ops::cos((time.elapsed_secs() - 4.0).max(0.0) * LASER_SPEED) * PI;
32 let ray_pos = Vec3::new(ops::sin(t), ops::cos(3.0 * t) * 0.5, ops::cos(t)) * 0.5;
33 let ray_dir = Dir3::new(-ray_pos).unwrap();
34 let ray = Ray3d::new(ray_pos, ray_dir);
35 gizmos.sphere(ray_pos, 0.1, Color::WHITE);
36 bounce_ray(ray, &mut ray_cast, &mut gizmos, Color::from(css::RED));
37
38 // Cast a ray from the cursor and bounce it off of surfaces
39 for (_, ray) in ray_map.iter() {
40 bounce_ray(*ray, &mut ray_cast, &mut gizmos, Color::from(css::GREEN));
41 }
42}
43
44// Bounces a ray off of surfaces `MAX_BOUNCES` times.
45fn bounce_ray(mut ray: Ray3d, ray_cast: &mut MeshRayCast, gizmos: &mut Gizmos, color: Color) {
46 let mut intersections = Vec::with_capacity(MAX_BOUNCES + 1);
47 intersections.push((ray.origin, Color::srgb(30.0, 0.0, 0.0)));
48
49 for i in 0..MAX_BOUNCES {
50 // Cast the ray and get the first hit
51 let Some((_, hit)) = ray_cast
52 .cast_ray(ray, &MeshRayCastSettings::default())
53 .first()
54 else {
55 break;
56 };
57
58 // Draw the point of intersection and add it to the list
59 let brightness = 1.0 + 10.0 * (1.0 - i as f32 / MAX_BOUNCES as f32);
60 intersections.push((hit.point, Color::BLACK.mix(&color, brightness)));
61 gizmos.sphere(hit.point, 0.005, Color::BLACK.mix(&color, brightness * 2.0));
62
63 // Reflect the ray off of the surface
64 ray.direction = Dir3::new(ray.direction.reflect(hit.normal)).unwrap();
65 ray.origin = hit.point + ray.direction * 1e-6;
66 }
67 gizmos.linestrip_gradient(intersections);
68}More examples
92fn move_target(
93 mut target: Single<&mut Transform, With<TargetSphere>>,
94 target_speed: Res<TargetSphereSpeed>,
95 mut target_pos: ResMut<TargetPosition>,
96 time: Res<Time>,
97 mut rng: ResMut<RandomSource>,
98) {
99 match Dir3::new(target_pos.0 - target.translation) {
100 // The target and the present position of the target sphere are far enough to have a well-
101 // defined direction between them, so let's move closer:
102 Ok(dir) => {
103 let delta_time = time.delta_secs();
104 let abs_delta = (target_pos.0 - target.translation).norm();
105
106 // Avoid overshooting in case of high values of `delta_time`:
107 let magnitude = f32::min(abs_delta, delta_time * target_speed.0);
108 target.translation += dir * magnitude;
109 }
110
111 // The two are really close, so let's generate a new target position:
112 Err(_) => {
113 let legal_region = Cuboid::from_size(Vec3::splat(4.0));
114 *target_pos = TargetPosition(legal_region.sample_interior(&mut rng.0));
115 }
116 }
117}13fn setup(
14 mut commands: Commands,
15 asset_server: Res<AssetServer>,
16 mut materials: ResMut<Assets<StandardMaterial>>,
17 meshes: Res<Assets<Mesh>>,
18) {
19 commands.spawn((Camera3d::default(), Transform::from_xyz(0.0, 0.0, 5.0)));
20
21 commands.spawn((
22 DirectionalLight::default(),
23 Transform::default().looking_to(Dir3::new(Vec3::new(-1.0, -1.0, -1.0)).unwrap(), Dir3::Y),
24 ));
25
26 // The simplest way to generate an asset is to add it directly to the `Assets`.
27 let material_handle = materials.add(StandardMaterial::default());
28
29 commands.spawn((
30 Transform::from_xyz(-2.0, 0.0, 0.0),
31 MeshMaterial3d(material_handle.clone()),
32 // Alternatively, `add_async` creates a task that runs your async function. Once it
33 // completes, the asset is added to the `Assets`. This is "deferred" meaning that the asset
34 // may take a frame to be added after the task completes.
35 Mesh3d(asset_server.add_async(generate_mesh_async())),
36 ));
37
38 // The last way to generate assets is to reserve a handle, and then use `Assets::insert` to
39 // populate the asset later. In this example, the `generate_mesh_system` system runs to populate
40 // the mesh.
41 let mesh_handle = meshes.reserve_handle();
42 commands.insert_resource(HandleToGenerate(mesh_handle.clone()));
43 commands.spawn((
44 Transform::from_xyz(2.0, 0.0, 0.0)
45 .with_rotation(Quat::from_rotation_x(50.0f32.to_radians())),
46 Mesh3d(mesh_handle),
47 MeshMaterial3d(material_handle),
48 ));
49}Sourcepub fn new_unchecked(value: Vec3) -> Dir3
pub fn new_unchecked(value: Vec3) -> Dir3
Sourcepub fn new_and_length(value: Vec3) -> Result<(Dir3, f32), InvalidDirectionError>
pub fn new_and_length(value: Vec3) -> Result<(Dir3, f32), InvalidDirectionError>
Create a direction from a finite, nonzero Vec3, normalizing it and
also returning its original length.
Returns Err(InvalidDirectionError) if the length
of the given vector is zero (or very close to zero), infinite, or NaN.
Sourcepub fn from_xyz(x: f32, y: f32, z: f32) -> Result<Dir3, InvalidDirectionError>
pub fn from_xyz(x: f32, y: f32, z: f32) -> Result<Dir3, InvalidDirectionError>
Create a direction from its x, y, and z components.
Returns Err(InvalidDirectionError) if the length
of the vector formed by the components is zero (or very close to zero), infinite, or NaN.
Sourcepub fn from_xyz_unchecked(x: f32, y: f32, z: f32) -> Dir3
pub fn from_xyz_unchecked(x: f32, y: f32, z: f32) -> Dir3
Create a direction from its x, y, and z components, assuming the resulting vector is normalized.
Warning
The vector produced from x, y, and z must be normalized, i.e its length must be 1.0.
Sourcepub const fn as_vec3(&self) -> Vec3
pub const fn as_vec3(&self) -> Vec3
Returns the inner Vec3
Examples found in repository?
13fn draw_cursor(
14 camera_query: Single<(&Camera, &GlobalTransform)>,
15 ground: Single<&GlobalTransform, With<Ground>>,
16 window: Single<&Window>,
17 mut gizmos: Gizmos,
18) {
19 let (camera, camera_transform) = *camera_query;
20
21 if let Some(cursor_position) = window.cursor_position()
22 // Calculate a ray pointing from the camera into the world based on the cursor's position.
23 && let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
24 // Calculate if and where the ray is hitting the ground plane.
25 && let Some(point) = ray.plane_intersection_point(ground.translation(), InfinitePlane3d::new(ground.up()))
26 {
27 // Draw a circle just above the ground plane at that position.
28 gizmos.circle(
29 Isometry3d::new(
30 point + ground.up() * 0.01,
31 Quat::from_rotation_arc(Vec3::Z, ground.up().as_vec3()),
32 ),
33 0.2,
34 Color::WHITE,
35 );
36 }
37}More examples
634fn move_camera(
635 keyboard_input: Res<ButtonInput<KeyCode>>,
636 mut mouse_wheel_reader: MessageReader<MouseWheel>,
637 mut cameras: Query<&mut Transform, With<Camera>>,
638) {
639 let (mut distance_delta, mut theta_delta) = (0.0, 0.0);
640
641 // Handle keyboard events.
642 if keyboard_input.pressed(KeyCode::KeyW) {
643 distance_delta -= CAMERA_KEYBOARD_ZOOM_SPEED;
644 }
645 if keyboard_input.pressed(KeyCode::KeyS) {
646 distance_delta += CAMERA_KEYBOARD_ZOOM_SPEED;
647 }
648 if keyboard_input.pressed(KeyCode::KeyA) {
649 theta_delta += CAMERA_KEYBOARD_ORBIT_SPEED;
650 }
651 if keyboard_input.pressed(KeyCode::KeyD) {
652 theta_delta -= CAMERA_KEYBOARD_ORBIT_SPEED;
653 }
654
655 // Handle mouse events.
656 for mouse_wheel in mouse_wheel_reader.read() {
657 distance_delta -= mouse_wheel.y * CAMERA_MOUSE_WHEEL_ZOOM_SPEED;
658 }
659
660 // Update transforms.
661 for mut camera_transform in cameras.iter_mut() {
662 let local_z = camera_transform.local_z().as_vec3().normalize_or_zero();
663 if distance_delta != 0.0 {
664 camera_transform.translation = (camera_transform.translation.length() + distance_delta)
665 .clamp(CAMERA_ZOOM_RANGE.start, CAMERA_ZOOM_RANGE.end)
666 * local_z;
667 }
668 if theta_delta != 0.0 {
669 camera_transform
670 .translate_around(Vec3::ZERO, Quat::from_axis_angle(Vec3::Y, theta_delta));
671 camera_transform.look_at(Vec3::ZERO, Vec3::Y);
672 }
673 }
674}Sourcepub fn slerp(self, rhs: Dir3, s: f32) -> Dir3
pub fn slerp(self, rhs: Dir3, s: f32) -> Dir3
Performs a spherical linear interpolation between self and rhs
based on the value s.
This corresponds to interpolating between the two directions at a constant angular velocity.
When s == 0.0, the result will be equal to self.
When s == 1.0, the result will be equal to rhs.
Example
let dir1 = Dir3::X;
let dir2 = Dir3::Y;
let result1 = dir1.slerp(dir2, 1.0 / 3.0);
#[cfg(feature = "approx")]
assert_relative_eq!(
result1,
Dir3::from_xyz(0.75_f32.sqrt(), 0.5, 0.0).unwrap(),
epsilon = 0.000001
);
let result2 = dir1.slerp(dir2, 0.5);
#[cfg(feature = "approx")]
assert_relative_eq!(result2, Dir3::from_xyz(0.5_f32.sqrt(), 0.5_f32.sqrt(), 0.0).unwrap());Sourcepub fn fast_renormalize(self) -> Dir3
pub fn fast_renormalize(self) -> Dir3
Returns self after an approximate normalization, assuming the value is already nearly normalized.
Useful for preventing numerical error accumulation.
Example
The following seemingly benign code would start accumulating errors over time,
leading to dir eventually not being normalized anymore.
let mut dir = Dir3::X;
let quaternion = Quat::from_euler(EulerRot::XYZ, 1.0, 2.0, 3.0);
for i in 0..N {
dir = quaternion * dir;
}Instead, do the following.
let mut dir = Dir3::X;
let quaternion = Quat::from_euler(EulerRot::XYZ, 1.0, 2.0, 3.0);
for i in 0..N {
dir = quaternion * dir;
dir = dir.fast_renormalize();
}Methods from Deref<Target = Vec3>
pub const ZERO: Vec3
pub const ONE: Vec3
pub const NEG_ONE: Vec3
pub const MIN: Vec3
pub const MAX: Vec3
pub const NAN: Vec3
pub const INFINITY: Vec3
pub const NEG_INFINITY: Vec3
pub const X: Vec3
pub const Y: Vec3
pub const Z: Vec3
pub const NEG_X: Vec3
pub const NEG_Y: Vec3
pub const NEG_Z: Vec3
pub const AXES: [Vec3; 3]
pub const USES_CORE_SIMD: bool = false
pub const USES_NEON: bool = false
pub const USES_SCALAR_MATH: bool = true
pub const USES_SSE2: bool = false
pub const USES_WASM_SIMD: bool = false
pub const USES_WASM32_SIMD: bool = false