Capsule2d

Struct Capsule2d 

pub struct Capsule2d {
    pub radius: f32,
    pub half_length: f32,
}
A 2D capsule primitive, also known as a stadium or pill shape.

A two-dimensional capsule is defined as a neighborhood of points at a distance (radius) from a line

Fields

radius: f32

The radius of the capsule

half_length: f32

Half the height of the capsule, excluding the semicircles

Implementations

Source

impl Capsule2d

pub const fn new(radius: f32, length: f32) -> Capsule2d

Create a new Capsule2d from a radius and length

Examples found in repository?
examples/testbed/2d.rs (line 125)
112    pub fn setup(
113        mut commands: Commands,
114        mut meshes: ResMut<Assets<Mesh>>,
115        mut materials: ResMut<Assets<ColorMaterial>>,
116    ) {
117        commands.spawn((Camera2d, DespawnOnExit(super::Scene::Shapes)));
118
119        let shapes = [
120            meshes.add(Circle::new(50.0)),
121            meshes.add(CircularSector::new(50.0, 1.0)),
122            meshes.add(CircularSegment::new(50.0, 1.25)),
123            meshes.add(Ellipse::new(25.0, 50.0)),
124            meshes.add(Annulus::new(25.0, 50.0)),
125            meshes.add(Capsule2d::new(25.0, 50.0)),
126            meshes.add(Rhombus::new(75.0, 100.0)),
127            meshes.add(Rectangle::new(50.0, 100.0)),
128            meshes.add(RegularPolygon::new(50.0, 6)),
129            meshes.add(Triangle2d::new(
130                Vec2::Y * 50.0,
131                Vec2::new(-50.0, -50.0),
132                Vec2::new(50.0, -50.0),
133            )),
134        ];
135        let num_shapes = shapes.len();
136
137        for (i, shape) in shapes.into_iter().enumerate() {
138            // Distribute colors evenly across the rainbow.
139            let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
140
141            commands.spawn((
142                Mesh2d(shape),
143                MeshMaterial2d(materials.add(color)),
144                Transform::from_xyz(
145                    // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
146                    -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
147                    0.0,
148                    0.0,
149                ),
150                DespawnOnExit(super::Scene::Shapes),
151            ));
152        }
153    }
More examples
Hide additional examples
examples/math/bounding_2d.rs (line 245)
202fn setup(mut commands: Commands) {
203    commands.spawn(Camera2d);
204
205    commands.spawn((
206        Transform::from_xyz(-OFFSET_X, OFFSET_Y, 0.),
207        Shape::Circle(Circle::new(45.)),
208        DesiredVolume::Aabb,
209        Intersects::default(),
210    ));
211
212    commands.spawn((
213        Transform::from_xyz(0., OFFSET_Y, 0.),
214        Shape::Rectangle(Rectangle::new(80., 80.)),
215        Spin,
216        DesiredVolume::Circle,
217        Intersects::default(),
218    ));
219
220    commands.spawn((
221        Transform::from_xyz(OFFSET_X, OFFSET_Y, 0.),
222        Shape::Triangle(Triangle2d::new(
223            Vec2::new(-40., -40.),
224            Vec2::new(-20., 40.),
225            Vec2::new(40., 50.),
226        )),
227        Spin,
228        DesiredVolume::Aabb,
229        Intersects::default(),
230    ));
231
232    commands.spawn((
233        Transform::from_xyz(-OFFSET_X, -OFFSET_Y, 0.),
234        Shape::Line(Segment2d::from_direction_and_length(
235            Dir2::from_xy(1., 0.3).unwrap(),
236            90.,
237        )),
238        Spin,
239        DesiredVolume::Circle,
240        Intersects::default(),
241    ));
242
243    commands.spawn((
244        Transform::from_xyz(0., -OFFSET_Y, 0.),
245        Shape::Capsule(Capsule2d::new(25., 50.)),
246        Spin,
247        DesiredVolume::Aabb,
248        Intersects::default(),
249    ));
250
251    commands.spawn((
252        Transform::from_xyz(OFFSET_X, -OFFSET_Y, 0.),
253        Shape::Polygon(RegularPolygon::new(50., 6)),
254        Spin,
255        DesiredVolume::Circle,
256        Intersects::default(),
257    ));
258
259    commands.spawn((
260        Text::default(),
261        Node {
262            position_type: PositionType::Absolute,
263            top: px(12),
264            left: px(12),
265            ..default()
266        },
267    ));
268}
examples/2d/2d_shapes.rs (line 60)
47fn setup(
48    mut commands: Commands,
49    mut meshes: ResMut<Assets<Mesh>>,
50    mut materials: ResMut<Assets<ColorMaterial>>,
51) {
52    commands.spawn(Camera2d);
53
54    let shapes = [
55        meshes.add(Circle::new(50.0)),
56        meshes.add(CircularSector::new(50.0, 1.0)),
57        meshes.add(CircularSegment::new(50.0, 1.25)),
58        meshes.add(Ellipse::new(25.0, 50.0)),
59        meshes.add(Annulus::new(25.0, 50.0)),
60        meshes.add(Capsule2d::new(25.0, 50.0)),
61        meshes.add(Rhombus::new(75.0, 100.0)),
62        meshes.add(Rectangle::new(50.0, 100.0)),
63        meshes.add(RegularPolygon::new(50.0, 6)),
64        meshes.add(Triangle2d::new(
65            Vec2::Y * 50.0,
66            Vec2::new(-50.0, -50.0),
67            Vec2::new(50.0, -50.0),
68        )),
69        meshes.add(Segment2d::new(
70            Vec2::new(-50.0, 50.0),
71            Vec2::new(50.0, -50.0),
72        )),
73        meshes.add(Polyline2d::new(vec![
74            Vec2::new(-50.0, 50.0),
75            Vec2::new(0.0, -50.0),
76            Vec2::new(50.0, 50.0),
77        ])),
78    ];
79    let num_shapes = shapes.len();
80
81    for (i, shape) in shapes.into_iter().enumerate() {
82        // Distribute colors evenly across the rainbow.
83        let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
84
85        commands.spawn((
86            Mesh2d(shape),
87            MeshMaterial2d(materials.add(color)),
88            Transform::from_xyz(
89                // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
90                -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
91                Y_EXTENT / 2.,
92                0.0,
93            ),
94        ));
95    }
96
97    let rings = [
98        meshes.add(Circle::new(50.0).to_ring(THICKNESS)),
99        // this visually produces an arc segment but this is not technically accurate
100        meshes.add(Ring::new(
101            CircularSector::new(50.0, 1.0),
102            CircularSector::new(45.0, 1.0),
103        )),
104        meshes.add(CircularSegment::new(50.0, 1.25).to_ring(THICKNESS)),
105        meshes.add({
106            // This is an approximation; Ellipse does not implement Inset as concentric ellipses do not have parallel curves
107            let outer = Ellipse::new(25.0, 50.0);
108            let mut inner = outer;
109            inner.half_size -= Vec2::splat(THICKNESS);
110            Ring::new(outer, inner)
111        }),
112        // this is equivalent to the Annulus::new(25.0, 50.0) above
113        meshes.add(Ring::new(Circle::new(50.0), Circle::new(25.0))),
114        meshes.add(Capsule2d::new(25.0, 50.0).to_ring(THICKNESS)),
115        meshes.add(Rhombus::new(75.0, 100.0).to_ring(THICKNESS)),
116        meshes.add(Rectangle::new(50.0, 100.0).to_ring(THICKNESS)),
117        meshes.add(RegularPolygon::new(50.0, 6).to_ring(THICKNESS)),
118        meshes.add(
119            Triangle2d::new(
120                Vec2::Y * 50.0,
121                Vec2::new(-50.0, -50.0),
122                Vec2::new(50.0, -50.0),
123            )
124            .to_ring(THICKNESS),
125        ),
126    ];
127    // Allow for 2 empty spaces
128    let num_rings = rings.len() + 2;
129
130    for (i, shape) in rings.into_iter().enumerate() {
131        // Distribute colors evenly across the rainbow.
132        let color = Color::hsl(360. * i as f32 / num_rings as f32, 0.95, 0.7);
133
134        commands.spawn((
135            Mesh2d(shape),
136            MeshMaterial2d(materials.add(color)),
137            Transform::from_xyz(
138                // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
139                -X_EXTENT / 2. + i as f32 / (num_rings - 1) as f32 * X_EXTENT,
140                -Y_EXTENT / 2.,
141                0.0,
142            ),
143        ));
144    }
145
146    let mut text = "Press 'R' to pause/resume rotation".to_string();
147    #[cfg(not(target_arch = "wasm32"))]
148    text.push_str("\nPress 'Space' to toggle wireframes");
149
150    commands.spawn((
151        Text::new(text),
152        Node {
153            position_type: PositionType::Absolute,
154            top: px(12),
155            left: px(12),
156            ..default()
157        },
158    ));
159}
pub const fn to_inner_rectangle(&self) -> Rectangle

Get the part connecting the semicircular ends of the capsule as a Rectangle