ConvexPolygon

Struct ConvexPolygon 

pub struct ConvexPolygon { /* private fields */ }
Available on crate feature alloc only.
A convex polygon with N vertices.

Implementations

Source

impl ConvexPolygon

pub fn new( vertices: impl IntoIterator<Item = Vec2>, ) -> Result<ConvexPolygon, ConvexPolygonError>

Create a ConvexPolygon from its vertices.

Errors

Returns ConvexPolygonError::Concave if the vertices do not form a convex polygon.

Examples found in repository?
examples/math/render_primitives.rs (line 462)
430fn draw_gizmos_2d(mut gizmos: Gizmos, state: Res<State<PrimitiveSelected>>, time: Res<Time>) {
431    const POSITION: Vec2 = Vec2::new(-LEFT_RIGHT_OFFSET_2D, 0.0);
432    let angle = time.elapsed_secs();
433    let isometry = Isometry2d::new(POSITION, Rot2::radians(angle));
434    let color = Color::WHITE;
435
436    #[expect(
437        clippy::match_same_arms,
438        reason = "Certain primitives don't have any 2D rendering support yet."
439    )]
440    match state.get() {
441        PrimitiveSelected::RectangleAndCuboid => {
442            gizmos.primitive_2d(&RECTANGLE, isometry, color);
443        }
444        PrimitiveSelected::CircleAndSphere => {
445            gizmos.primitive_2d(&CIRCLE, isometry, color);
446        }
447        PrimitiveSelected::Ellipse => drop(gizmos.primitive_2d(&ELLIPSE, isometry, color)),
448        PrimitiveSelected::Triangle => gizmos.primitive_2d(&TRIANGLE_2D, isometry, color),
449        PrimitiveSelected::Plane => gizmos.primitive_2d(&PLANE_2D, isometry, color),
450        PrimitiveSelected::Line => drop(gizmos.primitive_2d(&LINE_2D, isometry, color)),
451        PrimitiveSelected::Segment => {
452            drop(gizmos.primitive_2d(&SEGMENT_2D, isometry, color));
453        }
454        PrimitiveSelected::Polyline => gizmos.primitive_2d(
455            &Polyline2d {
456                vertices: POLYLINE_2D_VERTICES.to_vec(),
457            },
458            isometry,
459            color,
460        ),
461        PrimitiveSelected::ConvexPolygon => gizmos.primitive_2d(
462            &Polygon::from(ConvexPolygon::new(CONVEX_POLYGON_VERTICES).unwrap()),
463            isometry,
464            color,
465        ),
466        PrimitiveSelected::Polygon => gizmos.primitive_2d(
467            &Polygon {
468                vertices: vec![
469                    Vec2::new(-BIG_2D, -SMALL_2D),
470                    Vec2::new(BIG_2D, -SMALL_2D),
471                    Vec2::new(BIG_2D, SMALL_2D),
472                    Vec2::new(0.0, 0.0),
473                    Vec2::new(-BIG_2D, SMALL_2D),
474                ],
475            },
476            isometry,
477            color,
478        ),
479        PrimitiveSelected::RegularPolygon => {
480            gizmos.primitive_2d(&REGULAR_POLYGON, isometry, color);
481        }
482        PrimitiveSelected::Capsule => gizmos.primitive_2d(&CAPSULE_2D, isometry, color),
483        PrimitiveSelected::Cylinder => {}
484        PrimitiveSelected::Cone => {}
485        PrimitiveSelected::ConicalFrustum => {}
486        PrimitiveSelected::Torus => drop(gizmos.primitive_2d(&ANNULUS, isometry, color)),
487        PrimitiveSelected::Tetrahedron => {}
488        PrimitiveSelected::Arc => gizmos.primitive_2d(&ARC, isometry, color),
489        PrimitiveSelected::CircularSector => {
490            gizmos.primitive_2d(&CIRCULAR_SECTOR, isometry, color);
491        }
492        PrimitiveSelected::CircularSegment => {
493            gizmos.primitive_2d(&CIRCULAR_SEGMENT, isometry, color);
494        }
495    }
496}
497
498/// Marker for primitive meshes to record in which state they should be visible in
499#[derive(Debug, Clone, Component, Default, Reflect)]
500pub struct PrimitiveData {
501    camera_mode: CameraActive,
502    primitive_state: PrimitiveSelected,
503}
504
505/// Marker for meshes of 2D primitives
506#[derive(Debug, Clone, Component, Default)]
507pub struct MeshDim2;
508
509/// Marker for meshes of 3D primitives
510#[derive(Debug, Clone, Component, Default)]
511pub struct MeshDim3;
512
513fn spawn_primitive_2d(
514    mut commands: Commands,
515    mut materials: ResMut<Assets<ColorMaterial>>,
516    mut meshes: ResMut<Assets<Mesh>>,
517) {
518    const POSITION: Vec3 = Vec3::new(LEFT_RIGHT_OFFSET_2D, 0.0, 0.0);
519    let material: Handle<ColorMaterial> = materials.add(Color::WHITE);
520    let camera_mode = CameraActive::Dim2;
521    let polyline_2d = Polyline2d {
522        vertices: POLYLINE_2D_VERTICES.to_vec(),
523    };
524    let convex_polygon = ConvexPolygon::new(CONVEX_POLYGON_VERTICES).unwrap();
525    [
526        Some(RECTANGLE.mesh().build()),
527        Some(CIRCLE.mesh().build()),
528        Some(ELLIPSE.mesh().build()),
529        Some(TRIANGLE_2D.mesh().build()),
530        None, // plane
531        None, // line
532        Some(SEGMENT_2D.mesh().build()),
533        Some(polyline_2d.mesh().build()),
534        None, // polygon
535        Some(convex_polygon.mesh().build()),
536        Some(REGULAR_POLYGON.mesh().build()),
537        Some(CAPSULE_2D.mesh().build()),
538        None, // cylinder
539        None, // cone
540        None, // conical frustum
541        Some(ANNULUS.mesh().build()),
542        None, // tetrahedron
543        None, // arc
544        Some(CIRCULAR_SECTOR.mesh().build()),
545        Some(CIRCULAR_SEGMENT.mesh().build()),
546    ]
547    .into_iter()
548    .zip(PrimitiveSelected::ALL)
549    .for_each(|(maybe_mesh, state)| {
550        if let Some(mesh) = maybe_mesh {
551            commands.spawn((
552                MeshDim2,
553                PrimitiveData {
554                    camera_mode,
555                    primitive_state: state,
556                },
557                Mesh2d(meshes.add(mesh)),
558                MeshMaterial2d(material.clone()),
559                Transform::from_translation(POSITION),
560            ));
561        }
562    });
563}
More examples
Hide additional examples
examples/3d/3d_shapes.rs (lines 95-101)
57fn setup(
58    mut commands: Commands,
59    mut meshes: ResMut<Assets<Mesh>>,
60    mut images: ResMut<Assets<Image>>,
61    mut materials: ResMut<Assets<StandardMaterial>>,
62) {
63    let debug_material = materials.add(StandardMaterial {
64        base_color_texture: Some(images.add(uv_debug_texture())),
65        ..default()
66    });
67
68    let shapes = [
69        meshes.add(Cuboid::default()),
70        meshes.add(Tetrahedron::default()),
71        meshes.add(Capsule3d::default()),
72        meshes.add(Torus::default()),
73        meshes.add(Cylinder::default()),
74        meshes.add(Cone::default()),
75        meshes.add(ConicalFrustum::default()),
76        meshes.add(Sphere::default().mesh().ico(5).unwrap()),
77        meshes.add(Sphere::default().mesh().uv(32, 18)),
78        meshes.add(Segment3d::default()),
79        meshes.add(Polyline3d::new(vec![
80            Vec3::new(-0.5, 0.0, 0.0),
81            Vec3::new(0.5, 0.0, 0.0),
82            Vec3::new(0.0, 0.5, 0.0),
83        ])),
84    ];
85
86    let extrusions = [
87        meshes.add(Extrusion::new(Rectangle::default(), 1.)),
88        meshes.add(Extrusion::new(Capsule2d::default(), 1.)),
89        meshes.add(Extrusion::new(Annulus::default(), 1.)),
90        meshes.add(Extrusion::new(Circle::default(), 1.)),
91        meshes.add(Extrusion::new(Ellipse::default(), 1.)),
92        meshes.add(Extrusion::new(RegularPolygon::default(), 1.)),
93        meshes.add(Extrusion::new(Triangle2d::default(), 1.)),
94        meshes.add(Extrusion::new(
95            ConvexPolygon::new(vec![
96                Vec2::new(0.0, 0.8),
97                Vec2::new(-0.47, 0.25),
98                Vec2::new(-0.47, -0.65),
99                Vec2::new(0.47, -0.65),
100                Vec2::new(0.47, 0.25),
101            ])
102            .unwrap(),
103            1.0,
104        )),
105    ];
106
107    let ring_extrusions = [
108        meshes.add(Extrusion::new(Rectangle::default().to_ring(THICKNESS), 1.)),
109        meshes.add(Extrusion::new(Capsule2d::default().to_ring(THICKNESS), 1.)),
110        meshes.add(Extrusion::new(
111            Ring::new(Circle::new(1.0), Circle::new(0.5)),
112            1.,
113        )),
114        meshes.add(Extrusion::new(Circle::default().to_ring(THICKNESS), 1.)),
115        meshes.add(Extrusion::new(
116            {
117                // This is an approximation; Ellipse does not implement Inset as concentric ellipses do not have parallel curves
118                let outer = Ellipse::default();
119                let mut inner = outer;
120                inner.half_size -= Vec2::splat(THICKNESS);
121                Ring::new(outer, inner)
122            },
123            1.,
124        )),
125        meshes.add(Extrusion::new(
126            RegularPolygon::default().to_ring(THICKNESS),
127            1.,
128        )),
129        meshes.add(Extrusion::new(Triangle2d::default().to_ring(THICKNESS), 1.)),
130    ];
131
132    let num_shapes = shapes.len();
133
134    for (i, shape) in shapes.into_iter().enumerate() {
135        commands.spawn((
136            Mesh3d(shape),
137            MeshMaterial3d(debug_material.clone()),
138            Transform::from_xyz(
139                -SHAPES_X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * SHAPES_X_EXTENT,
140                2.0,
141                Row::Front.z(),
142            )
143            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
144            Shape,
145            Row::Front,
146        ));
147    }
148
149    let num_extrusions = extrusions.len();
150
151    for (i, shape) in extrusions.into_iter().enumerate() {
152        commands.spawn((
153            Mesh3d(shape),
154            MeshMaterial3d(debug_material.clone()),
155            Transform::from_xyz(
156                -EXTRUSION_X_EXTENT / 2.
157                    + i as f32 / (num_extrusions - 1) as f32 * EXTRUSION_X_EXTENT,
158                2.0,
159                Row::Middle.z(),
160            )
161            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
162            Shape,
163            Row::Middle,
164        ));
165    }
166
167    let num_ring_extrusions = ring_extrusions.len();
168
169    for (i, shape) in ring_extrusions.into_iter().enumerate() {
170        commands.spawn((
171            Mesh3d(shape),
172            MeshMaterial3d(debug_material.clone()),
173            Transform::from_xyz(
174                -EXTRUSION_X_EXTENT / 2.
175                    + i as f32 / (num_ring_extrusions - 1) as f32 * EXTRUSION_X_EXTENT,
176                2.0,
177                Row::Rear.z(),
178            )
179            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
180            Shape,
181            Row::Rear,
182        ));
183    }
184
185    commands.spawn((
186        PointLight {
187            shadow_maps_enabled: true,
188            intensity: 10_000_000.,
189            range: 100.0,
190            shadow_depth_bias: 0.2,
191            ..default()
192        },
193        Transform::from_xyz(8.0, 16.0, 8.0),
194    ));
195
196    // ground plane
197    commands.spawn((
198        Mesh3d(meshes.add(Plane3d::default().mesh().size(50.0, 50.0).subdivisions(10))),
199        MeshMaterial3d(materials.add(Color::from(SILVER))),
200    ));
201
202    commands.spawn((
203        Camera3d::default(),
204        Transform::from_xyz(0.0, 7., 14.0).looking_at(Vec3::new(0., 1., 0.), Vec3::Y),
205    ));
206
207    let mut text = "\
208        Press 'R' to pause/resume rotation\n\
209        Press 'Tab' to cycle through rows"
210        .to_string();
211    #[cfg(not(target_arch = "wasm32"))]
212    text.push_str("\nPress 'Space' to toggle wireframes");
213
214    commands.spawn((
215        Text::new(text),
216        Node {
217            position_type: PositionType::Absolute,
218            top: px(12),
219            left: px(12),
220            ..default()
221        },
222    ));
223}
pub fn new_unchecked(vertices: impl IntoIterator<Item = Vec2>) -> ConvexPolygon

Create a ConvexPolygon from its vertices, without checks. Use this version only if you know that the vertices make up a convex polygon.

pub fn vertices(&self) -> &[Vec2]

Get the vertices of this polygon