#[repr(C)]pub struct Affine3 {
pub matrix3: Mat3,
pub translation: Vec3,
}Expand description
A 3D affine transform, which can represent translation, rotation, scaling and shear.
Fields
matrix3: Mat3translation: Vec3Implementations
Sourceimpl Affine3
impl Affine3
Sourcepub const ZERO: Affine3
pub const ZERO: Affine3
The degenerate zero transform.
This transforms any finite vector and point to zero. The zero transform is non-invertible.
Sourcepub const IDENTITY: Affine3
pub const IDENTITY: Affine3
The identity transform.
Multiplying a vector with this returns the same vector.
Sourcepub const fn from_cols(
x_axis: Vec3,
y_axis: Vec3,
z_axis: Vec3,
w_axis: Vec3,
) -> Affine3
pub const fn from_cols( x_axis: Vec3, y_axis: Vec3, z_axis: Vec3, w_axis: Vec3, ) -> Affine3
Creates an affine transform from three column vectors.
Sourcepub fn from_cols_array(m: &[f32; 12]) -> Affine3
pub fn from_cols_array(m: &[f32; 12]) -> Affine3
Creates an affine transform from a [f32; 12] array stored in column major order.
Sourcepub fn to_cols_array(&self) -> [f32; 12]
pub fn to_cols_array(&self) -> [f32; 12]
Creates a [f32; 12] array storing data in column major order.
Sourcepub fn from_cols_array_2d(m: &[[f32; 3]; 4]) -> Affine3
pub fn from_cols_array_2d(m: &[[f32; 3]; 4]) -> Affine3
Creates an affine transform from a [[f32; 3]; 4]
3D array stored in column major order.
If your data is in row major order you will need to transpose the returned
matrix.
Sourcepub fn to_cols_array_2d(&self) -> [[f32; 3]; 4]
pub fn to_cols_array_2d(&self) -> [[f32; 3]; 4]
Creates a [[f32; 3]; 4] 3D array storing data in
column major order.
If you require data in row major order transpose the matrix first.
Sourcepub fn from_cols_slice(slice: &[f32]) -> Affine3
pub fn from_cols_slice(slice: &[f32]) -> Affine3
Creates an affine transform from the first 12 values in slice.
Panics
Panics if slice is less than 12 elements long.
Sourcepub fn write_cols_to_slice(&self, slice: &mut [f32])
pub fn write_cols_to_slice(&self, slice: &mut [f32])
Writes the columns of self to the first 12 elements in slice.
Panics
Panics if slice is less than 12 elements long.
Sourcepub fn from_scale(scale: Vec3) -> Affine3
pub fn from_scale(scale: Vec3) -> Affine3
Creates an affine transform that changes scale. Note that if any scale is zero the transform will be non-invertible.
Sourcepub fn from_quat(rotation: Quat) -> Affine3
pub fn from_quat(rotation: Quat) -> Affine3
Creates an affine transform from the given rotation quaternion.
Panics
Will panic if rotation is not normalized when glam_assert is enabled.
Sourcepub fn from_axis_angle(axis: Vec3, angle: f32) -> Affine3
pub fn from_axis_angle(axis: Vec3, angle: f32) -> Affine3
Creates an affine transform containing a 3D rotation around a normalized
rotation axis of angle (in radians).
Panics
Will panic if axis is not normalized when glam_assert is enabled.
Sourcepub fn from_rotation_x(angle: f32) -> Affine3
pub fn from_rotation_x(angle: f32) -> Affine3
Creates an affine transform containing a 3D rotation around the x axis of
angle (in radians).
Sourcepub fn from_rotation_y(angle: f32) -> Affine3
pub fn from_rotation_y(angle: f32) -> Affine3
Creates an affine transform containing a 3D rotation around the y axis of
angle (in radians).
Sourcepub fn from_rotation_z(angle: f32) -> Affine3
pub fn from_rotation_z(angle: f32) -> Affine3
Creates an affine transform containing a 3D rotation around the z axis of
angle (in radians).
Sourcepub fn from_translation(translation: Vec3) -> Affine3
pub fn from_translation(translation: Vec3) -> Affine3
Creates an affine transformation from the given 3D translation.
Sourcepub fn from_mat3(mat3: Mat3) -> Affine3
pub fn from_mat3(mat3: Mat3) -> Affine3
Creates an affine transform from a 3x3 matrix (expressing scale, shear and rotation)
Sourcepub fn from_mat3_translation(mat3: Mat3, translation: Vec3) -> Affine3
pub fn from_mat3_translation(mat3: Mat3, translation: Vec3) -> Affine3
Creates an affine transform from a 3x3 matrix (expressing scale, shear and rotation) and a translation vector.
Equivalent to Affine3::from_translation(translation) * Affine3::from_mat3(mat3)
Sourcepub fn from_scale_rotation_translation(
scale: Vec3,
rotation: Quat,
translation: Vec3,
) -> Affine3
pub fn from_scale_rotation_translation( scale: Vec3, rotation: Quat, translation: Vec3, ) -> Affine3
Creates an affine transform from the given 3D scale, rotation and
translation.
Equivalent to Affine3::from_translation(translation) * Affine3::from_quat(rotation) * Affine3::from_scale(scale)
Panics
Will panic if rotation is not normalized when glam_assert is enabled.
Sourcepub fn from_rotation_translation(rotation: Quat, translation: Vec3) -> Affine3
pub fn from_rotation_translation(rotation: Quat, translation: Vec3) -> Affine3
Creates an affine transform from the given 3D rotation and translation.
Equivalent to Affine3::from_translation(translation) * Affine3::from_quat(rotation)
Panics
Will panic if rotation is not normalized when glam_assert is enabled.
Sourcepub fn from_mat4(m: Mat4) -> Affine3
pub fn from_mat4(m: Mat4) -> Affine3
The given Mat4 must be an affine transform,
i.e. contain no perspective transform.
Sourcepub fn to_scale_rotation_translation(&self) -> (Vec3, Quat, Vec3)
pub fn to_scale_rotation_translation(&self) -> (Vec3, Quat, Vec3)
Extracts scale, rotation and translation from self.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Panics
Will panic if the determinant self.matrix3 is zero or if the resulting scale
vector contains any zero elements when glam_assert is enabled.
Sourcepub fn look_to_lh(eye: Vec3, dir: Vec3, up: Vec3) -> Affine3
👎Deprecated since 0.33.1: use the glam::camera::lh::view::look_to_affine3 function instead
pub fn look_to_lh(eye: Vec3, dir: Vec3, up: Vec3) -> Affine3
use the glam::camera::lh::view::look_to_affine3 function instead
Creates a left-handed view transform using a camera position, an up direction, and a facing direction.
For a view coordinate system with +X=right, +Y=up and +Z=forward.
Panics
Will panic if dir or up is zero length or not finite, or if dir and up are parallel,
when glam_assert is enabled.
Sourcepub fn look_to_rh(eye: Vec3, dir: Vec3, up: Vec3) -> Affine3
👎Deprecated since 0.33.1: use the glam::camera::rh::view::look_to_affine3 function instead
pub fn look_to_rh(eye: Vec3, dir: Vec3, up: Vec3) -> Affine3
use the glam::camera::rh::view::look_to_affine3 function instead
Creates a right-handed view transform using a camera position, an up direction, and a facing direction.
For a view coordinate system with +X=right, +Y=up and +Z=back.
Panics
Will panic if dir or up is zero length or not finite, or if dir and up are parallel,
when glam_assert is enabled.
Sourcepub fn look_at_lh(eye: Vec3, center: Vec3, up: Vec3) -> Affine3
👎Deprecated since 0.33.1: use the glam::camera::lh::view::look_at_affine3 function instead
pub fn look_at_lh(eye: Vec3, center: Vec3, up: Vec3) -> Affine3
use the glam::camera::lh::view::look_at_affine3 function instead
Creates a left-handed view transform using a camera position, an up direction, and a focal
point.
For a view coordinate system with +X=right, +Y=up and +Z=forward.
Panics
Will panic if up is not normalized, if center is equal to eye, or if the view
direction is parallel to up, when glam_assert is enabled.
Sourcepub fn look_at_rh(eye: Vec3, center: Vec3, up: Vec3) -> Affine3
👎Deprecated since 0.33.1: use the glam::camera::rh::view::look_at_affine3 function instead
pub fn look_at_rh(eye: Vec3, center: Vec3, up: Vec3) -> Affine3
use the glam::camera::rh::view::look_at_affine3 function instead
Creates a right-handed view transform using a camera position, an up direction, and a focal
point.
For a view coordinate system with +X=right, +Y=up and +Z=back.
Panics
Will panic if up is not normalized, if center is equal to eye, or if the view
direction is parallel to up, when glam_assert is enabled.
Sourcepub fn transform_point3(&self, rhs: Vec3) -> Vec3
pub fn transform_point3(&self, rhs: Vec3) -> Vec3
Transforms the given 3D points, applying shear, scale, rotation and translation.
Examples found in repository?
137fn queue_custom(
138 transparent_3d_draw_functions: Res<DrawFunctions<Transparent3d>>,
139 custom_pipeline: Res<CustomPipeline>,
140 mut pipelines: ResMut<SpecializedMeshPipelines<CustomPipeline>>,
141 pipeline_cache: Res<PipelineCache>,
142 meshes: Res<RenderAssets<RenderMesh>>,
143 render_mesh_instances: Res<RenderMeshInstances>,
144 maybe_batched_instance_buffers: Option<
145 Res<BatchedInstanceBuffers<MeshUniform, MeshInputUniform>>,
146 >,
147 material_meshes: Query<(Entity, &MainEntity), With<InstanceMaterialData>>,
148 mut transparent_render_phases: ResMut<ViewSortedRenderPhases<Transparent3d>>,
149 views: Query<&ExtractedView>,
150 view_key_cache: Res<ViewKeyCache>,
151) {
152 let draw_custom = transparent_3d_draw_functions.read().id::<DrawCustom>();
153
154 for view in &views {
155 let Some(transparent_phase) = transparent_render_phases.get_mut(&view.retained_view_entity)
156 else {
157 continue;
158 };
159
160 let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
161 continue;
162 };
163
164 for (entity, main_entity) in &material_meshes {
165 let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*main_entity)
166 else {
167 continue;
168 };
169 let Some(mesh) = meshes.get(mesh_instance.mesh_asset_id()) else {
170 continue;
171 };
172 let key = view_key
173 | MeshPipelineKey::from_primitive_topology_and_strip_index(
174 mesh.primitive_topology(),
175 mesh.index_format(),
176 );
177 let pipeline = pipelines
178 .specialize(&pipeline_cache, &custom_pipeline, key, &mesh.layout)
179 .unwrap();
180 transparent_phase.add_retained(Transparent3d {
181 sorting_info: TransparentSortingInfo3d::Sorted {
182 mesh_center: pbr::get_mesh_instance_world_from_local(
183 *main_entity,
184 mesh_instance.current_uniform_index,
185 &render_mesh_instances,
186 maybe_batched_instance_buffers.as_deref(),
187 )
188 .transform_point3(
189 meshes
190 .get(mesh_instance.mesh_asset_id())
191 .unwrap()
192 .aabb_center,
193 ),
194 depth_bias: 0.0,
195 },
196 entity: (entity, *main_entity),
197 pipeline,
198 draw_function: draw_custom,
199 distance: 0.0,
200 batch_range: 0..1,
201 extra_index: PhaseItemExtraIndex::None,
202 indexed: true,
203 });
204 }
205 }
206}More examples
559fn queue_custom_meshes(
560 custom_draw_functions: Res<DrawFunctions<Stencil3d>>,
561 mut pipelines: ResMut<SpecializedMeshPipelines<StencilPipeline>>,
562 pipeline_cache: Res<PipelineCache>,
563 custom_draw_pipeline: Res<StencilPipeline>,
564 render_meshes: Res<RenderAssets<RenderMesh>>,
565 render_mesh_instances: Res<RenderMeshInstances>,
566 maybe_batched_instance_buffers: Option<
567 Res<BatchedInstanceBuffers<MeshUniform, MeshInputUniform>>,
568 >,
569 mut custom_render_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
570 mut views: Query<(&ExtractedView, &RenderVisibleEntities)>,
571 view_key_cache: Res<ViewKeyCache>,
572 dirty_specializations: Res<DirtySpecializations>,
573 mut pending_custom_mesh_queues: ResMut<PendingCustomMeshQueues>,
574 has_marker: Query<(), With<DrawStencil>>,
575 mut mesh_instances_queued_this_iteration_scratch_space: Local<MainEntityHashSet>,
576) {
577 for (view, visible_entities) in &mut views {
578 let Some(custom_phase) = custom_render_phases.get_mut(&view.retained_view_entity) else {
579 continue;
580 };
581 let draw_custom = custom_draw_functions.read().id::<DrawMesh3dStencil>();
582
583 let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
584 continue;
585 };
586
587 // Since our phase can work on any 3d mesh we can reuse the default mesh 3d filter
588 let Some(render_visible_mesh_entities) = visible_entities.get::<Mesh3d>() else {
589 continue;
590 };
591
592 let view_pending_custom_mesh_queues =
593 pending_custom_mesh_queues.prepare_for_new_frame(view.retained_view_entity);
594
595 // First, remove meshes that need to be respecialized, and those that were removed, from the bins.
596 for &main_entity in dirty_specializations
597 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
598 {
599 custom_phase.remove(Entity::PLACEHOLDER, main_entity);
600 }
601
602 for (render_entity, visible_entity) in dirty_specializations.iter_to_queue(
603 view.retained_view_entity,
604 render_visible_mesh_entities,
605 &view_pending_custom_mesh_queues.prev_frame,
606 &mut mesh_instances_queued_this_iteration_scratch_space,
607 ) {
608 // We only want meshes with the marker component to be queued to our phase.
609 if has_marker.get(*render_entity).is_err() {
610 continue;
611 }
612 let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*visible_entity)
613 else {
614 // We couldn't fetch the mesh, probably because it hasn't been
615 // loaded yet. Add the entity to the list of pending custom mesh
616 // queues and bail.
617 view_pending_custom_mesh_queues
618 .current_frame
619 .insert((*render_entity, *visible_entity));
620 continue;
621 };
622 let Some(mesh) = render_meshes.get(mesh_instance.mesh_asset_id()) else {
623 continue;
624 };
625
626 // Specialize the key for the current mesh entity
627 // For this example we only specialize based on the mesh topology
628 // but you could have more complex keys and that's where you'd need to create those keys
629 let mut mesh_key = view_key;
630 mesh_key |= MeshPipelineKey::from_primitive_topology_and_strip_index(
631 mesh.primitive_topology(),
632 mesh.index_format(),
633 );
634
635 let pipeline_id = pipelines.specialize(
636 &pipeline_cache,
637 &custom_draw_pipeline,
638 mesh_key,
639 &mesh.layout,
640 );
641 let pipeline_id = match pipeline_id {
642 Ok(id) => id,
643 Err(err) => {
644 error!("{}", err);
645 continue;
646 }
647 };
648 // At this point we have all the data we need to create a phase item and add it to our
649 // phase
650 custom_phase.add_retained(Stencil3d {
651 sorting_info: TransparentSortingInfo3d::Sorted {
652 mesh_center: pbr::get_mesh_instance_world_from_local(
653 *visible_entity,
654 mesh_instance.current_uniform_index,
655 &render_mesh_instances,
656 maybe_batched_instance_buffers.as_deref(),
657 )
658 .transform_point3(
659 render_meshes
660 .get(mesh_instance.mesh_asset_id())
661 .unwrap()
662 .aabb_center,
663 ),
664 depth_bias: 0.0,
665 },
666 distance: FloatOrd(0.0),
667 entity: (Entity::PLACEHOLDER, *visible_entity),
668 pipeline: pipeline_id,
669 draw_function: draw_custom,
670 // Sorted phase items aren't batched
671 batch_range: 0..1,
672 extra_index: PhaseItemExtraIndex::None,
673 indexed: mesh.indexed(),
674 });
675 }
676 }
677}Sourcepub fn transform_vector3(&self, rhs: Vec3) -> Vec3
pub fn transform_vector3(&self, rhs: Vec3) -> Vec3
Transforms the given 3D vector, applying shear, scale and rotation (but NOT translation).
To also apply translation, use Self::transform_point3() instead.
Sourcepub fn is_finite(&self) -> bool
pub fn is_finite(&self) -> bool
Returns true if, and only if, all elements are finite.
If any element is either NaN, positive or negative infinity, this will return
false.
Sourcepub fn abs_diff_eq(&self, rhs: Affine3, max_abs_diff: f32) -> bool
pub fn abs_diff_eq(&self, rhs: Affine3, max_abs_diff: f32) -> bool
Returns true if the absolute difference of all elements between self and rhs
is less than or equal to max_abs_diff.
This can be used to compare if two 3x4 matrices contain similar elements. It works
best when comparing with a known value. The max_abs_diff that should be used used
depends on the values being compared against.
For more see comparing floating point numbers.
Sourcepub fn inverse(&self) -> Affine3
pub fn inverse(&self) -> Affine3
Return the inverse of this transform.
Note that if the transform is not invertible the result will be invalid.
Panics
Will panic if the resulting inverted matrix is not finite when glam_assert is enabled.
Sourcepub fn as_daffine3(&self) -> DAffine3
Available on crate feature f64 only.
pub fn as_daffine3(&self) -> DAffine3
f64 only.Casts all elements of self to f64.