pub struct Isometry2d {
pub rotation: Rot2,
pub translation: Vec2,
}Expand description
An isometry in two dimensions, representing a rotation followed by a translation. This can often be useful for expressing relative positions and transformations from one position to another.
In particular, this type represents a distance-preserving transformation known as a rigid motion or a direct motion, and belongs to the special Euclidean group SE(2). This includes translation and rotation, but excludes reflection.
For the three-dimensional version, see Isometry3d.
Example
Isometries can be created from a given translation and rotation:
let iso = Isometry2d::new(Vec2::new(2.0, 1.0), Rot2::degrees(90.0));Or from separate parts:
let iso1 = Isometry2d::from_translation(Vec2::new(2.0, 1.0));
let iso2 = Isometry2d::from_rotation(Rot2::degrees(90.0));The isometries can be used to transform points:
let iso = Isometry2d::new(Vec2::new(2.0, 1.0), Rot2::degrees(90.0));
let point = Vec2::new(4.0, 4.0);
// These are equivalent
let result = iso.transform_point(point);
let result = iso * point;
assert_eq!(result, Vec2::new(-2.0, 5.0));Isometries can also be composed together:
assert_eq!(iso1 * iso2, iso);One common operation is to compute an isometry representing the relative positions of two objects for things like intersection tests. This can be done with an inverse transformation:
let circle_iso = Isometry2d::from_translation(Vec2::new(2.0, 1.0));
let rectangle_iso = Isometry2d::from_rotation(Rot2::degrees(90.0));
// Compute the relative position and orientation between the two shapes
let relative_iso = circle_iso.inverse() * rectangle_iso;
// Or alternatively, to skip an extra rotation operation:
let relative_iso = circle_iso.inverse_mul(rectangle_iso);Fields
rotation: Rot2The rotational part of a two-dimensional isometry.
translation: Vec2The translational part of a two-dimensional isometry.
Implementations
Sourceimpl Isometry2d
impl Isometry2d
Sourcepub const IDENTITY: Isometry2d
pub const IDENTITY: Isometry2d
The identity isometry which represents the rigid motion of not doing anything.
Sourcepub const fn new(translation: Vec2, rotation: Rot2) -> Isometry2d
pub const fn new(translation: Vec2, rotation: Rot2) -> Isometry2d
Create a two-dimensional isometry from a rotation and a translation.
Examples found in repository?
104fn draw_bounds<Shape: Bounded2d + Send + Sync + 'static>(
105 q: Query<(&DrawBounds<Shape>, &GlobalTransform)>,
106 mut gizmos: Gizmos,
107) {
108 for (shape, transform) in &q {
109 let (_, rotation, translation) = transform.to_scale_rotation_translation();
110 let translation = translation.truncate();
111 let rotation = rotation.to_euler(EulerRot::XYZ).2;
112 let isometry = Isometry2d::new(translation, Rot2::radians(rotation));
113
114 let aabb = shape.0.aabb_2d(isometry);
115 gizmos.rect_2d(aabb.center(), aabb.half_size() * 2.0, RED);
116
117 let bounding_circle = shape.0.bounding_circle(isometry);
118 gizmos.circle_2d(bounding_circle.center, bounding_circle.radius(), BLUE);
119 }
120}More examples
21fn anchors(mut text_gizmos: Gizmos, time: Res<Time>) {
22 let t = time.elapsed_secs();
23 for (label, anchor, color) in [
24 ("left", vec2(-0.5, 0.0), RED),
25 ("right", vec2(0.5, 0.0), ORANGE),
26 ("center", Vec2::ZERO, YELLOW),
27 ("top", vec2(0.0, 0.5), GREEN),
28 ("bottom", vec2(0.0, -0.5), BLUE),
29 ] {
30 let position = Vec2::splat(350.0) * anchor;
31 text_gizmos.text_2d(
32 Isometry2d::from_translation(position),
33 "+",
34 12.,
35 Vec2::ZERO,
36 Color::WHITE,
37 );
38 text_gizmos.text_2d(
39 Isometry2d::new(position, Rot2::radians(t)),
40 label,
41 25.,
42 anchor,
43 color,
44 );
45 }
46}101fn render_shapes(mut gizmos: Gizmos, query: Query<(&Shape, &Transform)>) {
102 let color = GRAY;
103 for (shape, transform) in query.iter() {
104 let translation = transform.translation.xy();
105 let rotation = transform.rotation.to_euler(EulerRot::YXZ).2;
106 let isometry = Isometry2d::new(translation, Rot2::radians(rotation));
107 match shape {
108 Shape::Rectangle(r) => {
109 gizmos.primitive_2d(r, isometry, color);
110 }
111 Shape::Circle(c) => {
112 gizmos.primitive_2d(c, isometry, color);
113 }
114 Shape::Triangle(t) => {
115 gizmos.primitive_2d(t, isometry, color);
116 }
117 Shape::Line(l) => {
118 gizmos.primitive_2d(l, isometry, color);
119 }
120 Shape::Capsule(c) => {
121 gizmos.primitive_2d(c, isometry, color);
122 }
123 Shape::Polygon(p) => {
124 gizmos.primitive_2d(p, isometry, color);
125 }
126 }
127 }
128}
129
130#[derive(Component)]
131enum DesiredVolume {
132 Aabb,
133 Circle,
134}
135
136#[derive(Component, Debug)]
137enum CurrentVolume {
138 Aabb(Aabb2d),
139 Circle(BoundingCircle),
140}
141
142fn update_volumes(
143 mut commands: Commands,
144 query: Query<
145 (Entity, &DesiredVolume, &Shape, &Transform),
146 Or<(Changed<DesiredVolume>, Changed<Shape>, Changed<Transform>)>,
147 >,
148) {
149 for (entity, desired_volume, shape, transform) in query.iter() {
150 let translation = transform.translation.xy();
151 let rotation = transform.rotation.to_euler(EulerRot::YXZ).2;
152 let isometry = Isometry2d::new(translation, Rot2::radians(rotation));
153 match desired_volume {
154 DesiredVolume::Aabb => {
155 let aabb = match shape {
156 Shape::Rectangle(r) => r.aabb_2d(isometry),
157 Shape::Circle(c) => c.aabb_2d(isometry),
158 Shape::Triangle(t) => t.aabb_2d(isometry),
159 Shape::Line(l) => l.aabb_2d(isometry),
160 Shape::Capsule(c) => c.aabb_2d(isometry),
161 Shape::Polygon(p) => p.aabb_2d(isometry),
162 };
163 commands.entity(entity).insert(CurrentVolume::Aabb(aabb));
164 }
165 DesiredVolume::Circle => {
166 let circle = match shape {
167 Shape::Rectangle(r) => r.bounding_circle(isometry),
168 Shape::Circle(c) => c.bounding_circle(isometry),
169 Shape::Triangle(t) => t.bounding_circle(isometry),
170 Shape::Line(l) => l.bounding_circle(isometry),
171 Shape::Capsule(c) => c.bounding_circle(isometry),
172 Shape::Polygon(p) => p.bounding_circle(isometry),
173 };
174 commands
175 .entity(entity)
176 .insert(CurrentVolume::Circle(circle));
177 }
178 }
179 }
180}283fn bounding_shapes_2d(
284 shapes: Query<&Transform, With<Shape2d>>,
285 mut gizmos: Gizmos,
286 bounding_shape: Res<State<BoundingShape>>,
287) {
288 for transform in shapes.iter() {
289 // Get the rotation angle from the 3D rotation.
290 let rotation = transform.rotation.to_scaled_axis().z;
291 let rotation = Rot2::radians(rotation);
292 let isometry = Isometry2d::new(transform.translation.xy(), rotation);
293
294 match bounding_shape.get() {
295 BoundingShape::None => (),
296 BoundingShape::BoundingBox => {
297 // Get the AABB of the primitive with the rotation and translation of the mesh.
298 let aabb = HEART.aabb_2d(isometry);
299 gizmos.rect_2d(aabb.center(), aabb.half_size() * 2., WHITE);
300 }
301 BoundingShape::BoundingSphere => {
302 // Get the bounding sphere of the primitive with the rotation and translation of the mesh.
303 let bounding_circle = HEART.bounding_circle(isometry);
304 gizmos
305 .circle_2d(bounding_circle.center(), bounding_circle.radius(), WHITE)
306 .resolution(64);
307 }
308 }
309 }
310}430fn draw_gizmos_2d(mut gizmos: Gizmos, state: Res<State<PrimitiveSelected>>, time: Res<Time>) {
431 const POSITION: Vec2 = Vec2::new(-LEFT_RIGHT_OFFSET_2D, 0.0);
432 let angle = time.elapsed_secs();
433 let isometry = Isometry2d::new(POSITION, Rot2::radians(angle));
434 let color = Color::WHITE;
435
436 #[expect(
437 clippy::match_same_arms,
438 reason = "Certain primitives don't have any 2D rendering support yet."
439 )]
440 match state.get() {
441 PrimitiveSelected::RectangleAndCuboid => {
442 gizmos.primitive_2d(&RECTANGLE, isometry, color);
443 }
444 PrimitiveSelected::CircleAndSphere => {
445 gizmos.primitive_2d(&CIRCLE, isometry, color);
446 }
447 PrimitiveSelected::Ellipse => drop(gizmos.primitive_2d(&ELLIPSE, isometry, color)),
448 PrimitiveSelected::Triangle => gizmos.primitive_2d(&TRIANGLE_2D, isometry, color),
449 PrimitiveSelected::Plane => gizmos.primitive_2d(&PLANE_2D, isometry, color),
450 PrimitiveSelected::Line => drop(gizmos.primitive_2d(&LINE_2D, isometry, color)),
451 PrimitiveSelected::Segment => {
452 drop(gizmos.primitive_2d(&SEGMENT_2D, isometry, color));
453 }
454 PrimitiveSelected::Polyline => gizmos.primitive_2d(
455 &Polyline2d {
456 vertices: POLYLINE_2D_VERTICES.to_vec(),
457 },
458 isometry,
459 color,
460 ),
461 PrimitiveSelected::ConvexPolygon => gizmos.primitive_2d(
462 &Polygon::from(ConvexPolygon::new(CONVEX_POLYGON_VERTICES).unwrap()),
463 isometry,
464 color,
465 ),
466 PrimitiveSelected::Polygon => gizmos.primitive_2d(
467 &Polygon {
468 vertices: vec![
469 Vec2::new(-BIG_2D, -SMALL_2D),
470 Vec2::new(BIG_2D, -SMALL_2D),
471 Vec2::new(BIG_2D, SMALL_2D),
472 Vec2::new(0.0, 0.0),
473 Vec2::new(-BIG_2D, SMALL_2D),
474 ],
475 },
476 isometry,
477 color,
478 ),
479 PrimitiveSelected::RegularPolygon => {
480 gizmos.primitive_2d(®ULAR_POLYGON, isometry, color);
481 }
482 PrimitiveSelected::Capsule => gizmos.primitive_2d(&CAPSULE_2D, isometry, color),
483 PrimitiveSelected::Cylinder => {}
484 PrimitiveSelected::Cone => {}
485 PrimitiveSelected::ConicalFrustum => {}
486 PrimitiveSelected::Torus => drop(gizmos.primitive_2d(&ANNULUS, isometry, color)),
487 PrimitiveSelected::Tetrahedron => {}
488 PrimitiveSelected::Arc => gizmos.primitive_2d(&ARC, isometry, color),
489 PrimitiveSelected::CircularSector => {
490 gizmos.primitive_2d(&CIRCULAR_SECTOR, isometry, color);
491 }
492 PrimitiveSelected::CircularSegment => {
493 gizmos.primitive_2d(&CIRCULAR_SEGMENT, isometry, color);
494 }
495 }
496}Sourcepub const fn from_rotation(rotation: Rot2) -> Isometry2d
pub const fn from_rotation(rotation: Rot2) -> Isometry2d
Create a two-dimensional isometry from a rotation.
Sourcepub const fn from_translation(translation: Vec2) -> Isometry2d
pub const fn from_translation(translation: Vec2) -> Isometry2d
Create a two-dimensional isometry from a translation.
Examples found in repository?
21fn anchors(mut text_gizmos: Gizmos, time: Res<Time>) {
22 let t = time.elapsed_secs();
23 for (label, anchor, color) in [
24 ("left", vec2(-0.5, 0.0), RED),
25 ("right", vec2(0.5, 0.0), ORANGE),
26 ("center", Vec2::ZERO, YELLOW),
27 ("top", vec2(0.0, 0.5), GREEN),
28 ("bottom", vec2(0.0, -0.5), BLUE),
29 ] {
30 let position = Vec2::splat(350.0) * anchor;
31 text_gizmos.text_2d(
32 Isometry2d::from_translation(position),
33 "+",
34 12.,
35 Vec2::ZERO,
36 Color::WHITE,
37 );
38 text_gizmos.text_2d(
39 Isometry2d::new(position, Rot2::radians(t)),
40 label,
41 25.,
42 anchor,
43 color,
44 );
45 }
46}More examples
44fn draw_labels(mut text_gizmos: Gizmos, diagnostic: Res<DiagnosticsStore>) {
45 let colors = [RED, GREEN, BLUE, YELLOW];
46 for i in 0..TEXT_COUNT {
47 let row = i / 5;
48 let col = i % 5;
49 let color = colors[i % 4];
50 text_gizmos.text_2d(
51 Isometry2d {
52 translation: Vec2::new(
53 START_X + col as f32 * X_STEP,
54 START_Y - row as f32 * Y_STEP,
55 ),
56 rotation: Rot2::degrees(2.),
57 },
58 &format!("label {i}"),
59 25.,
60 Vec2::ZERO,
61 color,
62 );
63 }
64
65 if let Some(fps) = diagnostic.get(&FrameTimeDiagnosticsPlugin::FPS)
66 && let Some(fps_smoothed) = fps.smoothed()
67 {
68 text_gizmos.text_2d(
69 Isometry2d::from_translation(Vec2::new(600., START_Y + 150.)),
70 &format!("fps: {:.1}", fps_smoothed),
71 25.,
72 Vec2::ZERO,
73 Color::WHITE,
74 );
75 }
76
77 text_gizmos.text_2d(
78 Isometry2d::from_translation(Vec2::new(-300., START_Y + 200.)),
79 "lxgh",
80 150.,
81 Vec2::ZERO,
82 Color::WHITE,
83 );
84}394 pub fn draw_gizmos(mut gizmos: Gizmos) {
395 gizmos.rect_2d(
396 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
397 Vec2::new(200.0, 200.0),
398 RED,
399 );
400 gizmos
401 .circle_2d(
402 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
403 200.0,
404 GREEN,
405 )
406 .resolution(64);
407
408 gizmos.text_2d(
409 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
410 "text_2d gizmo",
411 15.,
412 Vec2 { x: 0., y: 0. },
413 Color::WHITE,
414 );
415
416 // 2d grids with all variations of outer edges on or off
417 for i in 0..4 {
418 let x = 200.0 * (1.0 + (i % 2) as f32);
419 let y = 150.0 * (0.5 - (i / 2) as f32);
420 let mut grid = gizmos.grid(
421 Vec3::new(x, y, 0.0),
422 UVec2::new(5, 4),
423 Vec2::splat(30.),
424 Color::WHITE,
425 );
426 if i & 1 > 0 {
427 grid = grid.outer_edges_x();
428 }
429 if i & 2 > 0 {
430 grid.outer_edges_y();
431 }
432 }
433 }Sourcepub const fn from_xy(x: f32, y: f32) -> Isometry2d
pub const fn from_xy(x: f32, y: f32) -> Isometry2d
Create a two-dimensional isometry from a translation with the given x and y components.
Sourcepub fn inverse(&self) -> Isometry2d
pub fn inverse(&self) -> Isometry2d
The inverse isometry that undoes this one.
Sourcepub fn inverse_mul(&self, rhs: Isometry2d) -> Isometry2d
pub fn inverse_mul(&self, rhs: Isometry2d) -> Isometry2d
Compute iso1.inverse() * iso2 in a more efficient way for one-shot cases.
If the same isometry is used multiple times, it is more efficient to instead compute the inverse once and use that for each transformation.
Sourcepub fn transform_point(&self, point: Vec2) -> Vec2
pub fn transform_point(&self, point: Vec2) -> Vec2
Transform a point by rotating and translating it using this isometry.
Sourcepub fn inverse_transform_point(&self, point: Vec2) -> Vec2
pub fn inverse_transform_point(&self, point: Vec2) -> Vec2
Transform a point by rotating and translating it using the inverse of this isometry.
This is more efficient than iso.inverse().transform_point(point) for one-shot cases.
If the same isometry is used multiple times, it is more efficient to instead compute
the inverse once and use that for each transformation.