#[repr(C)]pub struct Mat3A {
pub x_axis: Vec3A,
pub y_axis: Vec3A,
pub z_axis: Vec3A,
}Expand description
A 3x3 column major matrix.
This 3x3 matrix type features convenience methods for creating and using linear and
affine transformations. If you are primarily dealing with 2D affine transformations the
Affine2 type is much faster and more space efficient than
using a 3x3 matrix.
Linear transformations including 3D rotation and scale can be created using methods
such as Self::from_diagonal(), Self::from_quat(), Self::from_axis_angle(),
Self::from_rotation_x(), Self::from_rotation_y(), or
Self::from_rotation_z().
The resulting matrices can be use to transform 3D vectors using regular vector multiplication.
Affine transformations including 2D translation, rotation and scale can be created
using methods such as Self::from_translation(), Self::from_angle(),
Self::from_scale() and Self::from_scale_angle_translation().
The Self::transform_point2() and Self::transform_vector2() convenience methods
are provided for performing affine transforms on 2D vectors and points. These multiply
2D inputs as 3D vectors with an implicit z value of 1 for points and 0 for
vectors respectively. These methods assume that Self contains a valid affine
transform.
SIMD vector types are used for storage on supported platforms.
This type is 16 byte aligned.
Fields
x_axis: Vec3Ay_axis: Vec3Az_axis: Vec3AImplementations
Sourceimpl Mat3A
impl Mat3A
Sourcepub const IDENTITY: Mat3A
pub const IDENTITY: Mat3A
A 3x3 identity matrix, where all diagonal elements are 1, and all off-diagonal elements are 0.
Sourcepub const fn from_cols(x_axis: Vec3A, y_axis: Vec3A, z_axis: Vec3A) -> Mat3A
pub const fn from_cols(x_axis: Vec3A, y_axis: Vec3A, z_axis: Vec3A) -> Mat3A
Creates a 3x3 matrix from three column vectors.
See also Self::from_rows when the data is in row major order.
Sourcepub const fn from_rows(row0: Vec3A, row1: Vec3A, row2: Vec3A) -> Mat3A
pub const fn from_rows(row0: Vec3A, row1: Vec3A, row2: Vec3A) -> Mat3A
Creates a 3x3 matrix from three row vectors.
Matrices are stored in column major order, so the given rows are permuted into
the matrix layout. Use Self::from_cols instead when the data is already in
column major order.
Sourcepub const fn from_cols_array(m: &[f32; 9]) -> Mat3A
pub const fn from_cols_array(m: &[f32; 9]) -> Mat3A
Creates a 3x3 matrix from a [f32; 9] array stored in column major order.
If the data is in row major order use Self::from_rows_array instead.
Sourcepub const fn to_cols_array(&self) -> [f32; 9]
pub const fn to_cols_array(&self) -> [f32; 9]
Creates a [f32; 9] array storing data in column major order.
If you require the data in row major order use Self::to_rows_array instead.
Sourcepub const fn from_cols_array_2d(m: &[[f32; 3]; 3]) -> Mat3A
pub const fn from_cols_array_2d(m: &[[f32; 3]; 3]) -> Mat3A
Creates a 3x3 matrix from a [[f32; 3]; 3] 3D array stored in column major order.
If the data is in row major order transpose the returned matrix.
Sourcepub const fn to_cols_array_2d(&self) -> [[f32; 3]; 3]
pub const fn to_cols_array_2d(&self) -> [[f32; 3]; 3]
Creates a [[f32; 3]; 3] 3D array storing data in column major order.
If you require row major order transpose the matrix first.
Sourcepub const fn from_rows_array(m: &[f32; 9]) -> Mat3A
pub const fn from_rows_array(m: &[f32; 9]) -> Mat3A
Creates a 3x3 matrix from a [f32; 9] array stored in row major order.
Matrices are stored in column major order, so the array is permuted into the
matrix layout. Use Self::from_cols_array instead when the data is already in
column major order.
Sourcepub const fn to_rows_array(&self) -> [f32; 9]
pub const fn to_rows_array(&self) -> [f32; 9]
Creates a [f32; 9] array storing data in row major order.
Matrices are stored in column major order, so the array is permuted out of the
column major storage. Use Self::to_cols_array instead when you want data in
column major order.
Sourcepub const fn from_diagonal(diagonal: Vec3) -> Mat3A
pub const fn from_diagonal(diagonal: Vec3) -> Mat3A
Creates a 3x3 matrix with its diagonal set to diagonal and all other entries set to 0.
Sourcepub fn from_mat4(m: Mat4) -> Mat3A
pub fn from_mat4(m: Mat4) -> Mat3A
Creates a 3x3 matrix from a 4x4 matrix, discarding the 4th row and column.
Sourcepub fn from_mat4_minor(m: Mat4, i: usize, j: usize) -> Mat3A
pub fn from_mat4_minor(m: Mat4, i: usize, j: usize) -> Mat3A
Creates a 3x3 matrix from the minor of the given 4x4 matrix, discarding the ith column
and jth row.
Panics
Panics if i or j is greater than 3.
Sourcepub fn from_quat(rotation: Quat) -> Mat3A
pub fn from_quat(rotation: Quat) -> Mat3A
Creates a 3D rotation matrix from the given quaternion.
Panics
Will panic if rotation is not normalized when glam_assert is enabled.
Sourcepub fn from_axis_angle(axis: Vec3, angle: f32) -> Mat3A
pub fn from_axis_angle(axis: Vec3, angle: f32) -> Mat3A
Creates a 3D rotation matrix from a normalized rotation axis and angle (in
radians).
Panics
Will panic if axis is not normalized when glam_assert is enabled.
Sourcepub fn from_euler(order: EulerRot, a: f32, b: f32, c: f32) -> Mat3A
pub fn from_euler(order: EulerRot, a: f32, b: f32, c: f32) -> Mat3A
Creates a 3D rotation matrix from the given euler rotation sequence and the angles (in radians).
Sourcepub fn to_euler(&self, order: EulerRot) -> (f32, f32, f32)
pub fn to_euler(&self, order: EulerRot) -> (f32, f32, f32)
Extract Euler angles with the given Euler rotation order.
Note if the input matrix contains scales, shears, or other non-rotation transformations then the resulting Euler angles will be ill-defined.
Panics
Will panic if any input matrix column is not normalized when glam_assert is enabled.
Sourcepub fn from_rotation_x(angle: f32) -> Mat3A
pub fn from_rotation_x(angle: f32) -> Mat3A
Creates a 3D rotation matrix from angle (in radians) around the x axis.
Sourcepub fn from_rotation_y(angle: f32) -> Mat3A
pub fn from_rotation_y(angle: f32) -> Mat3A
Creates a 3D rotation matrix from angle (in radians) around the y axis.
Sourcepub fn from_rotation_z(angle: f32) -> Mat3A
pub fn from_rotation_z(angle: f32) -> Mat3A
Creates a 3D rotation matrix from angle (in radians) around the z axis.
Sourcepub fn from_translation(translation: Vec2) -> Mat3A
pub fn from_translation(translation: Vec2) -> Mat3A
Creates an affine transformation matrix from the given 2D translation.
The resulting matrix can be used to transform 2D points and vectors. See
Self::transform_point2() and Self::transform_vector2().
Sourcepub fn from_angle(angle: f32) -> Mat3A
pub fn from_angle(angle: f32) -> Mat3A
Creates an affine transformation matrix from the given 2D rotation angle (in
radians).
The resulting matrix can be used to transform 2D points and vectors. See
Self::transform_point2() and Self::transform_vector2().
Sourcepub fn from_scale_angle_translation(
scale: Vec2,
angle: f32,
translation: Vec2,
) -> Mat3A
pub fn from_scale_angle_translation( scale: Vec2, angle: f32, translation: Vec2, ) -> Mat3A
Creates an affine transformation matrix from the given 2D scale, rotation angle (in
radians) and translation.
The resulting matrix can be used to transform 2D points and vectors. See
Self::transform_point2() and Self::transform_vector2().
Sourcepub fn from_scale(scale: Vec2) -> Mat3A
pub fn from_scale(scale: Vec2) -> Mat3A
Creates an affine transformation matrix from the given non-uniform 2D scale.
The resulting matrix can be used to transform 2D points and vectors. See
Self::transform_point2() and Self::transform_vector2().
Panics
Will panic if all elements of scale are zero when glam_assert is enabled.
Sourcepub fn from_mat2(m: Mat2) -> Mat3A
pub fn from_mat2(m: Mat2) -> Mat3A
Creates an affine transformation matrix from the given 2x2 matrix.
The resulting matrix can be used to transform 2D points and vectors. See
Self::transform_point2() and Self::transform_vector2().
Sourcepub const fn from_cols_slice(slice: &[f32]) -> Mat3A
pub const fn from_cols_slice(slice: &[f32]) -> Mat3A
Creates a 3x3 matrix from the first 9 values in slice.
See also Self::from_rows_slice when the slice is in row major order.
Panics
Panics if slice is less than 9 elements long.
Sourcepub fn write_cols_to_slice(&self, slice: &mut [f32])
pub fn write_cols_to_slice(&self, slice: &mut [f32])
Writes the columns of self to the first 9 elements in slice.
Panics
Panics if slice is less than 9 elements long.
Sourcepub const fn from_rows_slice(slice: &[f32]) -> Mat3A
pub const fn from_rows_slice(slice: &[f32]) -> Mat3A
Creates a 3x3 matrix from the first 9 values in slice, stored in row
major order.
Matrices are stored in column major order, so the slice is permuted into the
matrix layout. Use Self::from_cols_slice instead when the slice is already in
column major order.
Panics
Panics if slice is less than 9 elements long.
Sourcepub fn col_mut(&mut self, index: usize) -> &mut Vec3A
pub fn col_mut(&mut self, index: usize) -> &mut Vec3A
Returns a mutable reference to the matrix column for the given index.
Panics
Panics if index is greater than 2.
Sourcepub fn row(&self, index: usize) -> Vec3A
pub fn row(&self, index: usize) -> Vec3A
Returns the matrix row for the given index.
See also Self::set_row when you need to change the row.
Panics
Panics if index is greater than 2.
Sourcepub fn set_row(&mut self, index: usize, row: Vec3A)
pub fn set_row(&mut self, index: usize, row: Vec3A)
Sets the matrix row for the given index.
Matrices are stored in column major order, so the row is spread across all
3 columns and writing it touches every column. Use Self::col_mut
instead when you can work with columns. See also Self::row.
Panics
Panics if index is greater than 2.
Sourcepub fn is_finite(&self) -> bool
pub fn is_finite(&self) -> bool
Returns true if, and only if, all elements are finite.
If any element is either NaN, positive or negative infinity, this will return false.
Sourcepub fn determinant(&self) -> f32
pub fn determinant(&self) -> f32
Returns the determinant of self.
Sourcepub fn inverse(&self) -> Mat3A
pub fn inverse(&self) -> Mat3A
Returns the inverse of self.
If the matrix is not invertible the returned matrix will be invalid. The
returned matrix will also be invalid if the inverse is not finite, which can
happen when self contains very large or very small values. Use
Self::try_inverse or Self::inverse_or_zero to detect these cases.
Panics
Will panic if the resulting inverted matrix is not finite when glam_assert
is enabled.
Examples found in repository?
341fn calculate_mirror_camera_transform_and_projection(
342 main_camera_transform: &Transform,
343 main_camera_projection: &PerspectiveProjection,
344 mirror_transform: &Transform,
345) -> (Transform, PerspectiveProjection) {
346 // Calculate the reflection matrix (a.k.a. Householder matrix) that will
347 // reflect the scene across the mirror plane.
348 //
349 // Note that you must calculate this in *matrix* form and only *afterward*
350 // convert to a `Transform` instead of composing `Transform`s. This is
351 // because the reflection matrix has non-uniform scale, and composing
352 // transforms can't always handle composition of matrices with non-uniform
353 // scales.
354 let mirror_camera_transform = Transform::from_matrix(
355 Mat4::from_mat3a(reflection_matrix(Vec3::NEG_Z)) * main_camera_transform.to_matrix(),
356 );
357
358 // Compute the distance from the camera to the mirror plane. This will be
359 // used to calculate the distance to the near clip plane for the mirror
360 // world.
361 let distance_from_camera_to_mirror = InfinitePlane3d::new(mirror_transform.rotation * Vec3::Y)
362 .signed_distance(
363 Isometry3d::IDENTITY,
364 mirror_transform.translation - main_camera_transform.translation,
365 );
366
367 // Compute the normal of the mirror plane in view space.
368 let view_from_world = main_camera_transform.compute_affine().matrix3.inverse();
369 let mirror_projection_plane_normal =
370 (view_from_world * (mirror_transform.rotation * Vec3::NEG_Y)).normalize();
371
372 // Compute the final projection. It should match the main camera projection,
373 // except that `near` and `near_normal` should be set to the updated near
374 // plane and near normal plane as above.
375 let mirror_camera_projection = PerspectiveProjection {
376 near_clip_plane: mirror_projection_plane_normal.extend(distance_from_camera_to_mirror),
377 ..*main_camera_projection
378 };
379
380 (mirror_camera_transform, mirror_camera_projection)
381}Sourcepub fn try_inverse(&self) -> Option<Mat3A>
pub fn try_inverse(&self) -> Option<Mat3A>
Returns the inverse of self or None if the matrix is not invertible, or if
the inverse is not finite.
Sourcepub fn inverse_or_zero(&self) -> Mat3A
pub fn inverse_or_zero(&self) -> Mat3A
Returns the inverse of self or Mat3A::ZERO if the matrix is not
invertible, or if the inverse is not finite.
Sourcepub fn transform_point2(&self, rhs: Vec2) -> Vec2
pub fn transform_point2(&self, rhs: Vec2) -> Vec2
Transforms the given 2D vector as a point.
This is the equivalent of multiplying rhs as a 3D vector where z is 1.
This method assumes that self contains a valid affine transform.
Panics
Will panic if the 2nd row of self is not (0, 0, 1) when glam_assert is enabled.
Sourcepub fn transform_vector2(&self, rhs: Vec2) -> Vec2
pub fn transform_vector2(&self, rhs: Vec2) -> Vec2
Rotates the given 2D vector.
This is the equivalent of multiplying rhs as a 3D vector where z is 0.
This method assumes that self contains a valid affine transform.
Panics
Will panic if the 2nd row of self is not (0, 0, 1) when glam_assert is enabled.
Sourcepub fn look_to_lh(dir: Vec3, up: Vec3) -> Mat3A
👎Deprecated since 0.33.1: use the glam::camera::lh::view::look_to_mat3 function instead
pub fn look_to_lh(dir: Vec3, up: Vec3) -> Mat3A
use the glam::camera::lh::view::look_to_mat3 function instead
Creates a left-handed view matrix using a facing direction and an up direction.
For a view coordinate system with +X=right, +Y=up and +Z=forward.
Panics
Will panic if dir or up are not normalized, or if dir and up are parallel,
when glam_assert is enabled.
Sourcepub fn look_to_rh(dir: Vec3, up: Vec3) -> Mat3A
👎Deprecated since 0.33.1: use the glam::camera::rh::view::look_to_mat3 function instead
pub fn look_to_rh(dir: Vec3, up: Vec3) -> Mat3A
use the glam::camera::rh::view::look_to_mat3 function instead
Creates a right-handed view matrix using a facing direction and an up direction.
For a view coordinate system with +X=right, +Y=up and +Z=back.
Panics
Will panic if dir or up are not normalized, or if dir and up are parallel,
when glam_assert is enabled.
Sourcepub fn look_at_lh(eye: Vec3, center: Vec3, up: Vec3) -> Mat3A
👎Deprecated since 0.33.1: use the glam::camera::lh::view::look_at_mat3 function instead
pub fn look_at_lh(eye: Vec3, center: Vec3, up: Vec3) -> Mat3A
use the glam::camera::lh::view::look_at_mat3 function instead
Creates a left-handed view matrix using a camera position, a focal point and an up direction.
For a view coordinate system with +X=right, +Y=up and +Z=forward.
Panics
Will panic if up is not normalized, if center is equal to eye, or if the view
direction is parallel to up, when glam_assert is enabled.
Sourcepub fn look_at_rh(eye: Vec3, center: Vec3, up: Vec3) -> Mat3A
👎Deprecated since 0.33.1: use the glam::camera::rh::view::look_at_mat3 function instead
pub fn look_at_rh(eye: Vec3, center: Vec3, up: Vec3) -> Mat3A
use the glam::camera::rh::view::look_at_mat3 function instead
Creates a right-handed view matrix using a camera position, a focal point and an up direction.
For a view coordinate system with +X=right, +Y=up and +Z=back.
Panics
Will panic if up is not normalized, if center is equal to eye, or if the view
direction is parallel to up, when glam_assert is enabled.
Sourcepub fn mul_transpose_vec3(&self, rhs: Vec3) -> Vec3
pub fn mul_transpose_vec3(&self, rhs: Vec3) -> Vec3
Transforms a 3D vector by the transpose of self.
Sourcepub fn mul_transpose_vec3a(&self, rhs: Vec3A) -> Vec3A
pub fn mul_transpose_vec3a(&self, rhs: Vec3A) -> Vec3A
Transforms a Vec3A by the transpose of self.
Sourcepub fn mul_scalar(&self, rhs: f32) -> Mat3A
pub fn mul_scalar(&self, rhs: f32) -> Mat3A
Multiplies a 3x3 matrix by a scalar.
Sourcepub fn mul_diagonal_scale(&self, scale: Vec3) -> Mat3A
pub fn mul_diagonal_scale(&self, scale: Vec3) -> Mat3A
Multiply self by a scaling vector scale.
This is faster than creating a whole diagonal scaling matrix and then multiplying that.
This operation is commutative.
Sourcepub fn div_scalar(&self, rhs: f32) -> Mat3A
pub fn div_scalar(&self, rhs: f32) -> Mat3A
Divides a 3x3 matrix by a scalar.
Sourcepub fn recip(&self) -> Mat3A
pub fn recip(&self) -> Mat3A
Returns a matrix containing the reciprocal 1.0/n of each element of self.
Sourcepub fn abs_diff_eq(&self, rhs: Mat3A, max_abs_diff: f32) -> bool
pub fn abs_diff_eq(&self, rhs: Mat3A, max_abs_diff: f32) -> bool
Returns true if the absolute difference of all elements between self and rhs
is less than or equal to max_abs_diff.
This can be used to compare if two matrices contain similar elements. It works best
when comparing with a known value. The max_abs_diff that should be used used
depends on the values being compared against.
For more see comparing floating point numbers.
pub fn as_dmat3(&self) -> DMat3
f64 only.Trait Implementations
Sourceimpl AbsDiffEq for Mat3A
impl AbsDiffEq for Mat3A
Sourcefn default_epsilon() -> <Mat3A as AbsDiffEq>::Epsilon
fn default_epsilon() -> <Mat3A as AbsDiffEq>::Epsilon
Sourcefn abs_diff_eq(
&self,
other: &Mat3A,
epsilon: <Mat3A as AbsDiffEq>::Epsilon,
) -> bool
fn abs_diff_eq( &self, other: &Mat3A, epsilon: <Mat3A as AbsDiffEq>::Epsilon, ) -> bool
Sourcefn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
fn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
AbsDiffEq::abs_diff_eq.Sourceimpl AddAssign for Mat3A
impl AddAssign for Mat3A
Sourcefn add_assign(&mut self, rhs: Mat3A)
fn add_assign(&mut self, rhs: Mat3A)
+= operation. Read moreSourceimpl AddAssign<&Mat3A> for Mat3A
impl AddAssign<&Mat3A> for Mat3A
Sourcefn add_assign(&mut self, rhs: &Mat3A)
fn add_assign(&mut self, rhs: &Mat3A)
+= operation. Read moreimpl Copy for Mat3A
Sourceimpl<'de> Deserialize<'de> for Mat3A
Deserialize expects a sequence of 9 values.
impl<'de> Deserialize<'de> for Mat3A
Deserialize expects a sequence of 9 values.
Sourcefn deserialize<D>(
deserializer: D,
) -> Result<Mat3A, <D as Deserializer<'de>>::Error>where
D: Deserializer<'de>,
fn deserialize<D>(
deserializer: D,
) -> Result<Mat3A, <D as Deserializer<'de>>::Error>where
D: Deserializer<'de>,
Sourceimpl DivAssign<&f32> for Mat3A
impl DivAssign<&f32> for Mat3A
Sourcefn div_assign(&mut self, rhs: &f32)
fn div_assign(&mut self, rhs: &f32)
/= operation. Read moreSourceimpl DivAssign<f32> for Mat3A
impl DivAssign<f32> for Mat3A
Sourcefn div_assign(&mut self, rhs: f32)
fn div_assign(&mut self, rhs: f32)
/= operation. Read moreSourceimpl FromReflect for Mat3A
impl FromReflect for Mat3A
Sourcefn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Mat3A>
fn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Mat3A>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetTypeRegistration for Mat3A
impl GetTypeRegistration for Mat3A
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for Mat3A
impl IntoReturn for Mat3A
Sourceimpl MulAssign for Mat3A
impl MulAssign for Mat3A
Sourcefn mul_assign(&mut self, rhs: Mat3A)
fn mul_assign(&mut self, rhs: Mat3A)
*= operation. Read moreSourceimpl MulAssign<&Affine2> for Mat3A
impl MulAssign<&Affine2> for Mat3A
Sourcefn mul_assign(&mut self, rhs: &Affine2)
fn mul_assign(&mut self, rhs: &Affine2)
*= operation. Read moreSourceimpl MulAssign<&Mat3A> for Mat3A
impl MulAssign<&Mat3A> for Mat3A
Sourcefn mul_assign(&mut self, rhs: &Mat3A)
fn mul_assign(&mut self, rhs: &Mat3A)
*= operation. Read moreSourceimpl MulAssign<&f32> for Mat3A
impl MulAssign<&f32> for Mat3A
Sourcefn mul_assign(&mut self, rhs: &f32)
fn mul_assign(&mut self, rhs: &f32)
*= operation. Read moreSourceimpl MulAssign<Affine2> for Mat3A
impl MulAssign<Affine2> for Mat3A
Sourcefn mul_assign(&mut self, rhs: Affine2)
fn mul_assign(&mut self, rhs: Affine2)
*= operation. Read moreSourceimpl MulAssign<f32> for Mat3A
impl MulAssign<f32> for Mat3A
Sourcefn mul_assign(&mut self, rhs: f32)
fn mul_assign(&mut self, rhs: f32)
*= operation. Read moreSourceimpl PartialReflect for Mat3A
impl PartialReflect for Mat3A
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<Mat3A>) -> ReflectOwned
fn reflect_owned(self: Box<Mat3A>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<Mat3A>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<Mat3A>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<Mat3A>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<Mat3A>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
fn to_dynamic(&self) -> Result<Box<dyn PartialReflect>, ReflectCloneError>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
impl Pod for Mat3A
Sourceimpl Reflect for Mat3A
impl Reflect for Mat3A
Sourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<Mat3A>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Mat3A>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl RelativeEq for Mat3A
impl RelativeEq for Mat3A
Sourcefn default_max_relative() -> <Mat3A as AbsDiffEq>::Epsilon
fn default_max_relative() -> <Mat3A as AbsDiffEq>::Epsilon
Sourcefn relative_eq(
&self,
other: &Mat3A,
epsilon: <Mat3A as AbsDiffEq>::Epsilon,
max_relative: <Mat3A as AbsDiffEq>::Epsilon,
) -> bool
fn relative_eq( &self, other: &Mat3A, epsilon: <Mat3A as AbsDiffEq>::Epsilon, max_relative: <Mat3A as AbsDiffEq>::Epsilon, ) -> bool
Sourcefn relative_ne(
&self,
other: &Rhs,
epsilon: Self::Epsilon,
max_relative: Self::Epsilon,
) -> bool
fn relative_ne( &self, other: &Rhs, epsilon: Self::Epsilon, max_relative: Self::Epsilon, ) -> bool
RelativeEq::relative_eq.Sourceimpl Serialize for Mat3A
Serialize as a sequence of 9 values.
impl Serialize for Mat3A
Serialize as a sequence of 9 values.
Sourcefn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
fn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
Sourceimpl Struct for Mat3A
impl Struct for Mat3A
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> Result<DynamicStruct, ReflectCloneError>
fn to_dynamic_struct(&self) -> Result<DynamicStruct, ReflectCloneError>
DynamicStruct from this struct. Read moreSourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.Sourceimpl SubAssign for Mat3A
impl SubAssign for Mat3A
Sourcefn sub_assign(&mut self, rhs: Mat3A)
fn sub_assign(&mut self, rhs: Mat3A)
-= operation. Read moreSourceimpl SubAssign<&Mat3A> for Mat3A
impl SubAssign<&Mat3A> for Mat3A
Sourcefn sub_assign(&mut self, rhs: &Mat3A)
fn sub_assign(&mut self, rhs: &Mat3A)
-= operation. Read moreSourceimpl TypePath for Mat3A
impl TypePath for Mat3A
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
Sourceimpl UlpsEq for Mat3A
impl UlpsEq for Mat3A
Sourcefn default_max_ulps() -> u32
fn default_max_ulps() -> u32