PerimeterSegment

Enum PerimeterSegment 

pub enum PerimeterSegment {
    Smooth {
        first_normal: Vec2,
        last_normal: Vec2,
        indices: Vec<u32>,
    },
    Flat {
        indices: Vec<u32>,
    },
}
Available on crate feature bevy_mesh only.
A type representing a segment of the perimeter of an extrudable mesh.

Variants



Smooth

This segment of the perimeter will be shaded smooth.

This has the effect of rendering the segment’s faces with softened edges, so it is appropriate for curved shapes.

The normals for the vertices that are part of this segment will be calculated based on the positions of their neighbors. Each normal is interpolated between the normals of the two line segments connecting it with its neighbors. Closer vertices have a stronger effect on the normal than more distant ones.

Since the vertices corresponding to the first and last indices do not have two neighboring vertices, their normals must be provided manually.

Fields

first_normal: Vec2

The normal of the first vertex.

last_normal: Vec2

The normal of the last vertex.

indices: Vec<u32>

A list of indices representing this segment of the perimeter of the mesh.

The indices refer to the indices of the vertices generated by the MeshBuilder of the underlying 2D primitive. For example, a triangle has 3 vertices with indices 0, 1 and 2.

The indices must be ordered such that the outside of the mesh is to the right when walking along the vertices of the mesh in the order provided by the indices.

For geometry to be rendered, you must provide at least two indices.



Flat

This segment of the perimeter will be shaded flat.

This has the effect of rendering the segment’s faces with hard edges.

Fields

indices: Vec<u32>

A list of indices representing this segment of the perimeter of the mesh.

The indices refer to the indices of the vertices generated by the MeshBuilder of the underlying 2D primitive. For example, a triangle has 3 vertices with indices 0, 1 and 2.

The indices must be ordered such that the outside of the mesh is to the right when walking along the vertices of the mesh in the order provided by indices.

For geometry to be rendered, you must provide at least two indices.