Mesh2d

Struct Mesh2d 

pub struct Mesh2d(pub Handle<Mesh>);
A component for 2D meshes. Requires a MeshMaterial2d to be rendered, commonly using a ColorMaterial.

Example

// Spawn an entity with a mesh using `ColorMaterial`.
fn setup(
    mut commands: Commands,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<ColorMaterial>>,
) {
    commands.spawn((
        Mesh2d(meshes.add(Circle::new(50.0))),
        MeshMaterial2d(materials.add(ColorMaterial::from_color(RED))),
    ));
}

Tuple Fields

0: Handle<Mesh>

Methods from Deref<Target = Handle<Mesh>>

pub fn id(&self) -> AssetId<A>

Returns the AssetId of this Asset.

Examples found in repository?
examples/shader_advanced/manual_material.rs (line 123)
122    fn as_asset_id(&self) -> AssetId<Self::Asset> {
123        self.0.id()
124    }
125}
126
127#[derive(Asset, TypePath, AsBindGroup, Debug, Clone)]
128struct ImageMaterial {
129    image: Handle<Image>,
130}
131
132impl ErasedRenderAsset for ImageMaterial {
133    type SourceAsset = ImageMaterial;
134    type ErasedAsset = PreparedMaterial;
135    type Param = (
136        SRes<DrawFunctions<Opaque3d>>,
137        SRes<ImageMaterialBindGroupLayout>,
138        SRes<AssetServer>,
139        SResMut<MaterialBindGroupAllocators>,
140        SResMut<RenderMaterialBindings>,
141        SRes<RenderAssets<GpuImage>>,
142        SRes<ImageMaterialBindGroupSampler>,
143    );
144
145    fn prepare_asset(
146        source_asset: Self::SourceAsset,
147        asset_id: AssetId<Self::SourceAsset>,
148        (
149            opaque_draw_functions,
150            material_layout,
151            asset_server,
152            bind_group_allocators,
153            render_material_bindings,
154            gpu_images,
155            image_material_sampler,
156        ): &mut SystemParamItem<Self::Param>,
157    ) -> std::result::Result<Self::ErasedAsset, PrepareAssetError<Self::SourceAsset>> {
158        let material_layout = material_layout.0.clone();
159        let draw_function_id = opaque_draw_functions.read().id::<DrawMaterial>();
160        let bind_group_allocator = bind_group_allocators
161            .get_mut(&TypeId::of::<ImageMaterial>())
162            .unwrap();
163        let Some(image) = gpu_images.get(&source_asset.image) else {
164            return Err(PrepareAssetError::RetryNextUpdate(source_asset));
165        };
166        let unprepared = UnpreparedBindGroup {
167            bindings: BindingResources(vec![
168                (
169                    0,
170                    OwnedBindingResource::TextureView(
171                        TextureViewDimension::D2,
172                        image.texture_view.clone(),
173                    ),
174                ),
175                (
176                    1,
177                    OwnedBindingResource::Sampler(
178                        SamplerBindingType::NonFiltering,
179                        image_material_sampler.0.clone(),
180                    ),
181                ),
182            ]),
183        };
184        let binding = match render_material_bindings.entry(asset_id.into()) {
185            Entry::Occupied(mut occupied_entry) => {
186                bind_group_allocator.free(*occupied_entry.get());
187                let new_binding =
188                    bind_group_allocator.allocate_unprepared(unprepared, &material_layout);
189                *occupied_entry.get_mut() = new_binding;
190                new_binding
191            }
192            Entry::Vacant(vacant_entry) => *vacant_entry
193                .insert(bind_group_allocator.allocate_unprepared(unprepared, &material_layout)),
194        };
195
196        let mut properties = MaterialProperties {
197            material_layout: Some(material_layout),
198            mesh_pipeline_key_bits: ErasedMeshPipelineKey::new(MeshPipelineKey::empty()),
199            base_specialize: Some(base_specialize),
200            ..Default::default()
201        };
202        properties.add_draw_function(MainPassOpaqueDrawFunction, draw_function_id);
203        properties.add_shader(MaterialFragmentShader, asset_server.load(SHADER_ASSET_PATH));
204
205        Ok(PreparedMaterial {
206            binding,
207            properties: Arc::new(properties),
208        })
209    }
210}
211
212/// set up a simple 3D scene
213fn setup(
214    mut commands: Commands,
215    mut meshes: ResMut<Assets<Mesh>>,
216    mut materials: ResMut<Assets<ImageMaterial>>,
217    asset_server: Res<AssetServer>,
218) {
219    // cube
220    commands.spawn((
221        Mesh3d(meshes.add(Cuboid::new(2.0, 2.0, 2.0))),
222        ImageMaterial3d(materials.add(ImageMaterial {
223            image: asset_server.load("branding/icon.png"),
224        })),
225        Transform::from_xyz(0.0, 0.5, 0.0),
226    ));
227    // light
228    commands.spawn((
229        PointLight {
230            shadow_maps_enabled: true,
231            ..default()
232        },
233        Transform::from_xyz(4.0, 8.0, 4.0),
234    ));
235    // camera
236    commands.spawn((
237        Camera3d::default(),
238        Transform::from_xyz(-2.5, 4.5, 9.0).looking_at(Vec3::ZERO, Vec3::Y),
239    ));
240}
241
242fn extract_image_materials(
243    mut material_instances: ResMut<RenderMaterialInstances>,
244    changed_meshes_query: Extract<
245        Query<
246            (Entity, &ViewVisibility, &ImageMaterial3d),
247            Or<(Changed<ViewVisibility>, Changed<ImageMaterial3d>)>,
248        >,
249    >,
250) {
251    let last_change_tick = material_instances.current_change_tick;
252
253    for (entity, view_visibility, material) in &changed_meshes_query {
254        if view_visibility.get() {
255            material_instances.instances.insert(
256                entity.into(),
257                RenderMaterialInstance {
258                    asset_id: material.0.id().untyped(),
259                    last_change_tick,
260                },
261            );
262        } else {
263            material_instances
264                .instances
265                .remove(&MainEntity::from(entity));
266        }
267    }
268}
More examples
Hide additional examples
examples/2d/dynamic_mip_generation.rs (line 463)
457fn extract_mipmap_source_image(
458    mipmap_source_image: Extract<Res<MipmapSourceImage>>,
459    app_status: Extract<Res<AppStatus>>,
460    mut mip_generation_jobs: ResMut<MipGenerationJobs>,
461) {
462    if app_status.enable_mip_generation == EnableMipGeneration::On {
463        mip_generation_jobs.add(MIP_GENERATION_PHASE_ID, mipmap_source_image.id());
464    }
465}
examples/shader_advanced/compute_mesh.rs (line 193)
173fn prepare_chunks(
174    meshes_to_generate: Query<&GenerateMesh>,
175    mut chunks: ResMut<ChunksToProcess>,
176    pipeline_cache: Res<PipelineCache>,
177    pipeline: Res<ComputePipeline>,
178    mut processed: Local<HashSet<AssetId<Mesh>>>,
179) {
180    // If the pipeline isn't ready, then meshes
181    // won't be processed. So we want to wait until
182    // the pipeline is ready before considering any mesh processed.
183    if pipeline_cache
184        .get_compute_pipeline(pipeline.pipeline)
185        .is_some()
186    {
187        // get the AssetId for each Handle<Mesh>
188        // which we'll use later to get the relevant buffers
189        // from the mesh_allocator
190        let chunk_data: Vec<AssetId<Mesh>> = meshes_to_generate
191            .iter()
192            .filter_map(|gmesh| {
193                let id = gmesh.0.id();
194                processed.contains(&id).not().then_some(id)
195            })
196            .collect();
197
198        // Cache any meshes we're going to process this frame
199        for id in &chunk_data {
200            processed.insert(*id);
201        }
202
203        chunks.0 = chunk_data;
204    }
205}
examples/3d/tonemapping.rs (line 248)
227fn resize_image(
228    image_mesh: Query<(&MeshMaterial3d<StandardMaterial>, &Mesh3d), With<HDRViewer>>,
229    materials: Res<Assets<StandardMaterial>>,
230    mut meshes: ResMut<Assets<Mesh>>,
231    images: Res<Assets<Image>>,
232    mut image_event_reader: MessageReader<AssetEvent<Image>>,
233) {
234    for event in image_event_reader.read() {
235        let (AssetEvent::Added { id } | AssetEvent::Modified { id }) = event else {
236            continue;
237        };
238
239        for (mat_h, mesh_h) in &image_mesh {
240            let Some(mat) = materials.get(mat_h) else {
241                continue;
242            };
243
244            let Some(ref base_color_texture) = mat.base_color_texture else {
245                continue;
246            };
247
248            if *id != base_color_texture.id() {
249                continue;
250            };
251
252            let Some(image_changed) = images.get(*id) else {
253                continue;
254            };
255
256            let size = image_changed.size_f32().normalize_or_zero() * 1.4;
257            // Resize Mesh
258            let quad = Mesh::from(Rectangle::from_size(size));
259            meshes.insert(mesh_h, quad).unwrap();
260        }
261    }
262}
examples/3d/mirror.rs (line 409)
388fn handle_window_resize_messages(
389    windows_query: Query<&Window>,
390    mut mirror_cameras_query: Query<&mut RenderTarget, With<MirrorCamera>>,
391    mut images: ResMut<Assets<Image>>,
392    mut mirror_image: ResMut<MirrorImage>,
393    mut screen_space_texture_materials: ResMut<
394        Assets<ExtendedMaterial<StandardMaterial, ScreenSpaceTextureExtension>>,
395    >,
396    mut resize_messages: MessageReader<WindowResized>,
397) {
398    // We run at most once, regardless of the number of window resize messages
399    // there were this frame.
400    let Some(resize_message) = resize_messages.read().next() else {
401        return;
402    };
403    let Ok(window) = windows_query.get(resize_message.window) else {
404        return;
405    };
406
407    let window_size = uvec2(window.physical_width(), window.physical_height());
408    let image = create_mirror_texture_image(&mut images, window_size);
409    images.remove(mirror_image.0.id());
410
411    mirror_image.0 = image.clone();
412
413    for mut target in mirror_cameras_query.iter_mut() {
414        *target = image.clone().into();
415    }
416
417    for (_, material) in screen_space_texture_materials.iter_mut() {
418        material.base.emissive_texture = Some(image.clone());
419    }
420}
examples/2d/mesh2d_manual.rs (line 366)
332pub fn extract_colored_mesh2d(
333    mut commands: Commands,
334    mut previous_len: Local<usize>,
335    // When extracting, you must use `Extract` to mark the `SystemParam`s
336    // which should be taken from the main world.
337    query: Extract<
338        Query<
339            (
340                Entity,
341                RenderEntity,
342                &ViewVisibility,
343                &GlobalTransform,
344                &Mesh2d,
345            ),
346            With<ColoredMesh2d>,
347        >,
348    >,
349    mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
350) {
351    let mut values = Vec::with_capacity(*previous_len);
352    for (entity, render_entity, view_visibility, transform, handle) in &query {
353        if !view_visibility.get() {
354            continue;
355        }
356
357        let transforms = Mesh2dTransforms {
358            world_from_local: transform.affine().into(),
359            flags: MeshFlags::empty().bits(),
360        };
361
362        values.push((render_entity, ColoredMesh2d));
363        render_mesh_instances.insert(
364            entity.into(),
365            RenderMesh2dInstance {
366                mesh_asset_id: handle.0.id(),
367                transforms,
368                material_bind_group_id: Material2dBindGroupId::default(),
369                automatic_batching: false,
370                tag: 0,
371            },
372        );
373    }
374    *previous_len = values.len();
375    commands.try_insert_batch(values);
376}
pub fn path(&self) -> Option<&AssetPath<'static>>

Returns the path if this is (1) a strong handle and (2) the asset has a path

pub fn is_uuid(&self) -> bool

Returns true if this is a uuid handle.

pub fn is_strong(&self) -> bool

Returns true if this is a strong handle.