MeshAttributeCompressionFlags

Struct MeshAttributeCompressionFlags 

pub struct MeshAttributeCompressionFlags(/* private fields */);
Available on crate feature bevy_mesh only.
If the corresponding attribute compression is enabled:

  • Position will be Snorm16x4 relative to the mesh’s AABB. The w component is unused.
  • Normal and tangent will be Snorm16x2 with octahedral encoding, using octahedral_encode_signed and octahedral_encode_tangent.
  • UV0 and UV1 will be Unorm16x2. UVs are remapped based on their min/max values so they can go beyond [0, 1], though a larger range will reduce precision.

Implementations

Source

impl MeshAttributeCompressionFlags

Source

impl MeshAttributeCompressionFlags

pub const fn empty() -> MeshAttributeCompressionFlags

Get a flags value with all bits unset.

pub const fn all() -> MeshAttributeCompressionFlags

Get a flags value with all known bits set.

pub const fn bits(&self) -> u8

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

pub const fn from_bits(bits: u8) -> Option<MeshAttributeCompressionFlags>

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: u8) -> MeshAttributeCompressionFlags

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: u8) -> MeshAttributeCompressionFlags

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<MeshAttributeCompressionFlags>

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Whether all bits in self are unset.

pub const fn is_all(&self) -> bool

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: MeshAttributeCompressionFlags) -> bool

Whether any set bits in other are also set in self.

pub const fn contains(&self, other: MeshAttributeCompressionFlags) -> bool

Whether all set bits in other are also set in self.

Examples found in repository?
examples/shader_advanced/custom_render_phase.rs (line 215)
202    fn specialize(
203        &self,
204        key: Self::Key,
205        layout: &MeshVertexBufferLayoutRef,
206    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
207        let mut shader_defs = Vec::new();
208        // We will only use the position of the mesh in our shader so we only need to specify that
209        let mut vertex_attributes = Vec::new();
210        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
211            // Handle compressed vertex positions.
212            if layout
213                .0
214                .get_attribute_compression()
215                .contains(MeshAttributeCompressionFlags::COMPRESS_POSITION)
216            {
217                shader_defs.push("VERTEX_POSITIONS_COMPRESSED".into());
218            }
219            // Make sure this matches the shader location
220            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
221        }
222        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
223        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
224        let view_layout = self
225            .mesh_pipeline
226            .get_view_layout(MeshPipelineViewLayoutKey::from(key));
227        Ok(RenderPipelineDescriptor {
228            label: Some("Specialized Mesh Pipeline".into()),
229            // We want to reuse the data from bevy so we use the same bind groups as the default
230            // mesh pipeline
231            layout: vec![
232                // Bind group 0 is the view uniform
233                view_layout.main_layout,
234                // Bind group 1 is empty
235                view_layout.empty_layout,
236                // Bind group 2 is the mesh uniform
237                self.mesh_pipeline.mesh_layouts.model_only.clone(),
238            ],
239            vertex: VertexState {
240                shader: self.shader_handle.clone(),
241                shader_defs: shader_defs.clone(),
242                buffers: vec![vertex_buffer_layout],
243                ..default()
244            },
245            fragment: Some(FragmentState {
246                shader: self.shader_handle.clone(),
247                shader_defs,
248                targets: vec![Some(ColorTargetState {
249                    format: key.target_format(),
250                    blend: None,
251                    write_mask: ColorWrites::ALL,
252                })],
253                ..default()
254            }),
255            primitive: PrimitiveState {
256                topology: key.primitive_topology(),
257                strip_index_format: key.strip_index_format(),
258                cull_mode: Some(Face::Back),
259                ..default()
260            },
261            // It's generally recommended to specialize your pipeline for MSAA,
262            // but it's not always possible
263            ..default()
264        })
265    }
More examples
Hide additional examples
examples/shader_advanced/specialized_mesh_pipeline.rs (line 228)
215    fn specialize(
216        &self,
217        mesh_key: Self::Key,
218        layout: &MeshVertexBufferLayoutRef,
219    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
220        let mut shader_defs = Vec::new();
221        // Define the vertex attributes based on a standard bevy [`Mesh`]
222        let mut vertex_attributes = Vec::new();
223        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
224            // Handle compressed vertex positions.
225            if layout
226                .0
227                .get_attribute_compression()
228                .contains(MeshAttributeCompressionFlags::COMPRESS_POSITION)
229            {
230                shader_defs.push("VERTEX_POSITIONS_COMPRESSED".into());
231            }
232            // Make sure this matches the shader location
233            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
234        }
235        if layout.0.contains(Mesh::ATTRIBUTE_COLOR) {
236            shader_defs.push("VERTEX_COLORS".into());
237            // Make sure this matches the shader location
238            vertex_attributes.push(Mesh::ATTRIBUTE_COLOR.at_shader_location(1));
239        }
240        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
241        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
242
243        let view_layout = self
244            .mesh_pipeline
245            .get_view_layout(MeshPipelineViewLayoutKey::from(mesh_key));
246
247        Ok(RenderPipelineDescriptor {
248            label: Some("Specialized Mesh Pipeline".into()),
249            layout: vec![
250                view_layout.main_layout,
251                view_layout.empty_layout,
252                self.mesh_pipeline.mesh_layouts.model_only.clone(),
253            ],
254            vertex: VertexState {
255                shader: self.shader_handle.clone(),
256                shader_defs: shader_defs.clone(),
257                // Customize how to store the meshes' vertex attributes in the vertex buffer
258                buffers: vec![vertex_buffer_layout],
259                ..default()
260            },
261            fragment: Some(FragmentState {
262                shader: self.shader_handle.clone(),
263                shader_defs,
264                targets: vec![Some(ColorTargetState {
265                    // This isn't required, but bevy supports rendering different formats
266                    // so it's generally recommended to specialize the pipeline for that
267                    format: mesh_key.target_format(),
268                    // For this example we only use opaque meshes,
269                    // but if you wanted to use alpha blending you would need to set it here
270                    blend: None,
271                    write_mask: ColorWrites::ALL,
272                })],
273                ..default()
274            }),
275            primitive: PrimitiveState {
276                topology: mesh_key.primitive_topology(),
277                strip_index_format: mesh_key.strip_index_format(),
278                front_face: FrontFace::Ccw,
279                cull_mode: Some(Face::Back),
280                polygon_mode: PolygonMode::Fill,
281                ..default()
282            },
283            // Note that if your view has no depth buffer this will need to be
284            // changed.
285            depth_stencil: Some(DepthStencilState {
286                format: CORE_3D_DEPTH_FORMAT,
287                depth_write_enabled: Some(true),
288                depth_compare: Some(CompareFunction::GreaterEqual),
289                stencil: default(),
290                bias: default(),
291            }),
292            // It's generally recommended to specialize your pipeline for MSAA,
293            // but it's not always possible
294            multisample: MultisampleState {
295                count: mesh_key.msaa_samples(),
296                ..default()
297            },
298            ..default()
299        })
300    }
pub fn insert(&mut self, other: MeshAttributeCompressionFlags)

The bitwise or (|) of the bits in self and other.

pub fn remove(&mut self, other: MeshAttributeCompressionFlags)

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: MeshAttributeCompressionFlags)

The bitwise exclusive-or (^) of the bits in self and other.

pub fn set(&mut self, other: MeshAttributeCompressionFlags, value: bool)

Call insert when value is true or remove when value is false.

pub const fn intersection( self, other: MeshAttributeCompressionFlags, ) -> MeshAttributeCompressionFlags

The bitwise and (&) of the bits in self and other.

pub const fn union( self, other: MeshAttributeCompressionFlags, ) -> MeshAttributeCompressionFlags

The bitwise or (|) of the bits in self and other.

pub const fn difference( self, other: MeshAttributeCompressionFlags, ) -> MeshAttributeCompressionFlags

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference( self, other: MeshAttributeCompressionFlags, ) -> MeshAttributeCompressionFlags

The bitwise exclusive-or (^) of the bits in self and other.

pub const fn complement(self) -> MeshAttributeCompressionFlags

The bitwise negation (!) of the bits in self, truncating the result.

Source

impl MeshAttributeCompressionFlags

pub const fn iter(&self) -> Iter<MeshAttributeCompressionFlags> ⓘ

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<MeshAttributeCompressionFlags> ⓘ

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.