pub struct VertexBufferLayout {
pub array_stride: u64,
pub step_mode: VertexStepMode,
pub attributes: Vec<VertexAttribute>,
}Expand description
Describes how the vertex buffer is interpreted.
Fields
array_stride: u64The stride, in bytes, between elements of this buffer.
step_mode: VertexStepModeHow often this vertex buffer is “stepped” forward.
attributes: Vec<VertexAttribute>The list of attributes which comprise a single vertex.
Implementations
Sourceimpl VertexBufferLayout
impl VertexBufferLayout
Sourcepub fn from_vertex_formats<T>(
step_mode: VertexStepMode,
vertex_formats: T,
) -> VertexBufferLayoutwhere
T: IntoIterator<Item = VertexFormat>,
pub fn from_vertex_formats<T>(
step_mode: VertexStepMode,
vertex_formats: T,
) -> VertexBufferLayoutwhere
T: IntoIterator<Item = VertexFormat>,
Creates a new densely packed VertexBufferLayout from an iterator of vertex formats.
Iteration order determines the shader_location and offset of the VertexAttributes.
The first iterated item will have a shader_location and offset of zero.
The array_stride is the sum of the size of the iterated VertexFormats (in bytes).
Examples found in repository?
examples/2d/mesh2d_manual.rs (line 164)
153 fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
154 // Customize how to store the meshes' vertex attributes in the vertex buffer
155 // Our meshes only have position and color
156 let formats = vec![
157 // Position
158 VertexFormat::Float32x3,
159 // Color
160 VertexFormat::Uint32,
161 ];
162
163 let vertex_layout =
164 VertexBufferLayout::from_vertex_formats(VertexStepMode::Vertex, formats);
165
166 let format = match key.contains(Mesh2dPipelineKey::HDR) {
167 true => ViewTarget::TEXTURE_FORMAT_HDR,
168 false => TextureFormat::bevy_default(),
169 };
170
171 RenderPipelineDescriptor {
172 vertex: VertexState {
173 // Use our custom shader
174 shader: self.shader.clone(),
175 // Use our custom vertex buffer
176 buffers: vec![vertex_layout],
177 ..default()
178 },
179 fragment: Some(FragmentState {
180 // Use our custom shader
181 shader: self.shader.clone(),
182 targets: vec![Some(ColorTargetState {
183 format,
184 blend: Some(BlendState::ALPHA_BLENDING),
185 write_mask: ColorWrites::ALL,
186 })],
187 ..default()
188 }),
189 // Use the two standard uniforms for 2d meshes
190 layout: vec![
191 // Bind group 0 is the view uniform
192 self.mesh2d_pipeline.view_layout.clone(),
193 // Bind group 1 is the mesh uniform
194 self.mesh2d_pipeline.mesh_layout.clone(),
195 ],
196 primitive: PrimitiveState {
197 cull_mode: Some(Face::Back),
198 topology: key.primitive_topology(),
199 ..default()
200 },
201 depth_stencil: Some(DepthStencilState {
202 format: CORE_2D_DEPTH_FORMAT,
203 depth_write_enabled: false,
204 depth_compare: CompareFunction::GreaterEqual,
205 stencil: StencilState {
206 front: StencilFaceState::IGNORE,
207 back: StencilFaceState::IGNORE,
208 read_mask: 0,
209 write_mask: 0,
210 },
211 bias: DepthBiasState {
212 constant: 0,
213 slope_scale: 0.0,
214 clamp: 0.0,
215 },
216 }),
217 multisample: MultisampleState {
218 count: key.msaa_samples(),
219 mask: !0,
220 alpha_to_coverage_enabled: false,
221 },
222 label: Some("colored_mesh2d_pipeline".into()),
223 ..default()
224 }
225 }Sourcepub fn offset_locations_by(self, location: u32) -> VertexBufferLayout
pub fn offset_locations_by(self, location: u32) -> VertexBufferLayout
Returns a VertexBufferLayout with the shader location of every attribute offset by
location.