#[repr(u32)]pub enum AtmosphereMode {
LookupTexture = 0,
Raymarched = 1,
}Expand description
Selects how the atmosphere is rendered. Choose based on scene scale and volumetric shadow quality, and based on performance needs.
Variants
LookupTexture = 0
High-performance solution tailored to scenes that are mostly inside of the atmosphere. Uses a set of lookup textures to approximate scattering integration. Slightly less accurate for very long-distance/space views (lighting precision tapers as the camera moves far from the scene origin) and for sharp volumetric (cloud/fog) shadows.
Raymarched = 1
Slower, more accurate rendering method for any type of scene. Integrates the scattering numerically with raymarching and produces sharp volumetric (cloud/fog) shadows. Best for cinematic shots, planets seen from orbit, and scenes requiring accurate long-distance lighting.