pub enum RenderMeshInstanceGpuQueue {
None,
CpuCulling {
changed: Vec<(MainEntity, RenderMeshInstanceGpuBuilder)>,
removed: Vec<MainEntity>,
},
GpuCulling {
changed_cpu_culling: Vec<(MainEntity, RenderMeshInstanceGpuBuilder, MeshCullingData)>,
changed_gpu_culling: Vec<(MainEntity, RenderMeshInstanceGpuBuilder, MeshCullingData)>,
removed: Vec<MainEntity>,
},
}bevy_pbr only.Expand description
The per-thread queues used during extract_meshes_for_gpu_building.
There are two varieties of these: one for when culling happens on CPU and one for when culling happens on GPU. Having the two varieties avoids wasting space if GPU culling is disabled.
Variants
None
The default value.
This becomes RenderMeshInstanceGpuQueue::CpuCulling or
RenderMeshInstanceGpuQueue::GpuCulling once extraction starts.
CpuCulling
The version of RenderMeshInstanceGpuQueue that omits the
MeshCullingData, so that we don’t waste space when GPU
culling is disabled.
Fields
changed: Vec<(MainEntity, RenderMeshInstanceGpuBuilder)>Stores GPU data for each entity that became visible or changed in
such a way that necessitates updating the MeshInputUniform (e.g.
changed transform).
removed: Vec<MainEntity>Stores the IDs of entities that became invisible this frame.
GpuCulling
The version of RenderMeshInstanceGpuQueue that contains the
MeshCullingData, used when any view has GPU culling
enabled.
Fields
changed_cpu_culling: Vec<(MainEntity, RenderMeshInstanceGpuBuilder, MeshCullingData)>Stores GPU data for each mesh entity that became visible or changed
in such a way as to necessitate updating the MeshInputUniform
(e.g. changed transform).
This only stores information for meshes without NoCpuCulling.
changed_gpu_culling: Vec<(MainEntity, RenderMeshInstanceGpuBuilder, MeshCullingData)>Stores GPU data for each mesh entity that changed in such a way as
to necessitate updating the MeshInputUniform (e.g. changed
transform).
This only stores information for meshes with NoCpuCulling.
removed: Vec<MainEntity>Stores the IDs of entities that became invisible this frame.