pub struct FogVolumeBundle {
pub fog_volume: FogVolume,
pub visibility: Visibility,
pub inherited_visibility: InheritedVisibility,
pub view_visibility: ViewVisibility,
pub transform: Transform,
pub global_transform: GlobalTransform,
}๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.Expand description
A convenient Bundle that contains all components necessary to generate a
fog volume.
Fields
fog_volume: FogVolume๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.The actual fog volume.
visibility: Visibility๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.Visibility.
inherited_visibility: InheritedVisibility๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.Inherited visibility.
view_visibility: ViewVisibility๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.View visibility.
transform: Transform๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.The local transform. Set this to change the position, and scale of the fogโs axis-aligned bounding box (AABB).
global_transform: GlobalTransform๐Deprecated since 0.15.0: Use the
FogVolume component instead. Inserting it will now also insert the other components required by it automatically.The global transform.