MeshesToReextractNextFrame

Struct MeshesToReextractNextFrame 

pub struct MeshesToReextractNextFrame(/* private fields */);
Holds a list of meshes that couldn’t be extracted this frame because their materials weren’t prepared yet.

On subsequent frames, we try to reextract those meshes.

Methods from Deref<Target = HashSet<MainEntity, EntityHash>>

pub fn capacity(&self) -> usize

Returns the number of elements the set can hold without reallocating.

Refer to capacity for further details.

Examples
let map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);
pub fn iter(&self) -> Iter<'_, T>

An iterator visiting all elements in arbitrary order. The iterator element type is &'a T.

Refer to iter for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

for value in map.iter() {
    // "foo", "bar", "baz"
    // Note that the above order is not guaranteed
}
pub fn len(&self) -> usize

Returns the number of elements in the set.

Refer to len for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.len(), 0);

map.insert("foo");

assert_eq!(map.len(), 1);
pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

Refer to is_empty for further details.

Examples
let mut map = HashSet::new();

assert!(map.is_empty());

map.insert("foo");

assert!(!map.is_empty());
pub fn drain(&mut self) -> Drain<'_, T>

Clears the set, returning all elements in an iterator.

Refer to drain for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

for value in map.drain() {
    // "foo", "bar", "baz"
    // Note that the above order is not guaranteed
}

assert!(map.is_empty());
pub fn retain<F>(&mut self, f: F)
where F: FnMut(&T) -> bool,

Retains only the elements specified by the predicate.

Refer to retain for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

map.retain(|value| *value == "baz");

assert_eq!(map.len(), 1);
pub fn extract_if<F>(&mut self, f: F) -> ExtractIf<'_, T, F>
where F: FnMut(&T) -> bool,

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

Refer to extract_if for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

let extracted = map
    .extract_if(|value| *value == "baz")
    .collect::<Vec<_>>();

assert_eq!(map.len(), 2);
assert_eq!(extracted.len(), 1);
pub fn clear(&mut self)

Clears the set, removing all values.

Refer to clear for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");
map.insert("bar");
map.insert("baz");

map.clear();

assert!(map.is_empty());
Examples found in repository?
examples/ui/navigation/directional_navigation.rs (line 315)
310fn process_inputs(
311    mut action_state: ResMut<ActionState>,
312    keyboard_input: Res<ButtonInput<KeyCode>>,
313    gamepad_input: Query<&Gamepad>,
314) {
315    action_state.pressed_actions.clear();
316
317    for action in DirectionalNavigationAction::variants() {
318        if keyboard_input.just_pressed(action.keycode()) {
319            action_state.pressed_actions.insert(action);
320        }
321    }
322
323    for gamepad in gamepad_input.iter() {
324        for action in DirectionalNavigationAction::variants() {
325            if gamepad.just_pressed(action.gamepad_button()) {
326                action_state.pressed_actions.insert(action);
327            }
328        }
329    }
330}
More examples
Hide additional examples
examples/ui/navigation/directional_navigation_overrides.rs (line 687)
682fn process_inputs(
683    mut action_state: ResMut<ActionState>,
684    keyboard_input: Res<ButtonInput<KeyCode>>,
685    gamepad_input: Query<&Gamepad>,
686) {
687    action_state.pressed_actions.clear();
688
689    for action in DirectionalNavigationAction::variants() {
690        if keyboard_input.just_pressed(action.keycode()) {
691            action_state.pressed_actions.insert(action);
692        }
693    }
694
695    for gamepad in gamepad_input.iter() {
696        for action in DirectionalNavigationAction::variants() {
697            if gamepad.just_pressed(action.gamepad_button()) {
698                action_state.pressed_actions.insert(action);
699            }
700        }
701    }
702}
examples/shader_advanced/custom_render_phase.rs (line 505)
500fn extract_camera_phases(
501    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
502    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
503    mut live_entities: Local<HashSet<RetainedViewEntity>>,
504) {
505    live_entities.clear();
506    for (main_entity, camera) in &cameras {
507        if !camera.is_active {
508            continue;
509        }
510        // This is the main camera, so we use the first subview index (0)
511        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
512
513        stencil_phases.prepare_for_new_frame(retained_view_entity);
514        live_entities.insert(retained_view_entity);
515    }
516
517    // Clear out all dead views.
518    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
519}
pub fn hasher(&self) -> &S

Returns a reference to the set’s BuildHasher.

Refer to hasher for further details.

pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the HashSet. The collection may reserve more space to avoid frequent reallocations.

Refer to reserve for further details.

Examples
let mut map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);

map.reserve(10);

assert!(map.capacity() - map.len() >= 10);
pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve capacity for at least additional more elements to be inserted in the given HashSet<K,V>. The collection may reserve more space to avoid frequent reallocations.

Refer to try_reserve for further details.

Examples
let mut map = HashSet::with_capacity(5);

assert!(map.capacity() >= 5);

map.try_reserve(10).expect("Out of Memory!");

assert!(map.capacity() - map.len() >= 10);
pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the set as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Refer to shrink_to_fit for further details.

Examples
let mut map = HashSet::with_capacity(5);

map.insert("foo");
map.insert("bar");
map.insert("baz");

assert!(map.capacity() >= 5);

map.shrink_to_fit();

assert_eq!(map.capacity(), 3);
pub fn shrink_to(&mut self, min_capacity: usize)

Shrinks the capacity of the set with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Refer to shrink_to for further details.

pub fn difference<'a>( &'a self, other: &'a HashSet<T, S>, ) -> Difference<'a, T, S>

Visits the values representing the difference, i.e., the values that are in self but not in other.

Refer to difference for further details.

pub fn symmetric_difference<'a>( &'a self, other: &'a HashSet<T, S>, ) -> SymmetricDifference<'a, T, S>

Visits the values representing the symmetric difference, i.e., the values that are in self or in other but not in both.

Refer to symmetric_difference for further details.

pub fn intersection<'a>( &'a self, other: &'a HashSet<T, S>, ) -> Intersection<'a, T, S>

Visits the values representing the intersection, i.e., the values that are both in self and other.

Refer to intersection for further details.

pub fn union<'a>(&'a self, other: &'a HashSet<T, S>) -> Union<'a, T, S>

Visits the values representing the union, i.e., all the values in self or other, without duplicates.

Refer to union for further details.

pub fn contains<Q>(&self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Returns true if the set contains a value.

Refer to contains for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.contains("foo"));
Examples found in repository?
examples/gltf/gltf_extension_animation_graph.rs (line 251)
245    fn on_gltf_node(
246        &mut self,
247        _load_context: &mut LoadContext<'_>,
248        gltf_node: &gltf::Node,
249        entity: &mut EntityWorldMut,
250    ) {
251        if self.animation_root_indices.contains(&gltf_node.index()) {
252            self.animation_root_entities.insert(entity.id());
253        }
254    }
More examples
Hide additional examples
examples/shader_advanced/custom_render_phase.rs (line 518)
500fn extract_camera_phases(
501    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
502    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
503    mut live_entities: Local<HashSet<RetainedViewEntity>>,
504) {
505    live_entities.clear();
506    for (main_entity, camera) in &cameras {
507        if !camera.is_active {
508            continue;
509        }
510        // This is the main camera, so we use the first subview index (0)
511        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
512
513        stencil_phases.prepare_for_new_frame(retained_view_entity);
514        live_entities.insert(retained_view_entity);
515    }
516
517    // Clear out all dead views.
518    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
519}
examples/testbed/helpers.rs (line 26)
12pub fn switch_scene_in_ci<Scene: States + FreelyMutableState + Next>(
13    mut ci_config: ResMut<CiTestingConfig>,
14    scene: Res<State<Scene>>,
15    mut next_scene: ResMut<NextState<Scene>>,
16    mut scenes_visited: Local<HashSet<Scene>>,
17    frame_count: Res<FrameCount>,
18    captured: RemovedComponents<Captured>,
19) {
20    if scene.is_changed() {
21        // Changed scene! trigger a screenshot in 100 frames
22        ci_config.events.push(CiTestingEventOnFrame(
23            frame_count.0 + 100,
24            CiTestingEvent::NamedScreenshot(format!("{:?}", scene.get())),
25        ));
26        if scenes_visited.contains(scene.get()) {
27            // Exit once all scenes have been screenshotted
28            ci_config.events.push(CiTestingEventOnFrame(
29                frame_count.0 + 1,
30                CiTestingEvent::AppExit,
31            ));
32        }
33        return;
34    }
35
36    if !captured.is_empty() {
37        // Screenshot taken! Switch to the next scene
38        scenes_visited.insert(scene.get().clone());
39        next_scene.set(scene.get().next());
40    }
41}
examples/shader_advanced/compute_mesh.rs (line 194)
173fn prepare_chunks(
174    meshes_to_generate: Query<&GenerateMesh>,
175    mut chunks: ResMut<ChunksToProcess>,
176    pipeline_cache: Res<PipelineCache>,
177    pipeline: Res<ComputePipeline>,
178    mut processed: Local<HashSet<AssetId<Mesh>>>,
179) {
180    // If the pipeline isn't ready, then meshes
181    // won't be processed. So we want to wait until
182    // the pipeline is ready before considering any mesh processed.
183    if pipeline_cache
184        .get_compute_pipeline(pipeline.pipeline)
185        .is_some()
186    {
187        // get the AssetId for each Handle<Mesh>
188        // which we'll use later to get the relevant buffers
189        // from the mesh_allocator
190        let chunk_data: Vec<AssetId<Mesh>> = meshes_to_generate
191            .iter()
192            .filter_map(|gmesh| {
193                let id = gmesh.0.id();
194                processed.contains(&id).not().then_some(id)
195            })
196            .collect();
197
198        // Cache any meshes we're going to process this frame
199        for id in &chunk_data {
200            processed.insert(*id);
201        }
202
203        chunks.0 = chunk_data;
204    }
205}
examples/ui/navigation/directional_navigation.rs (line 338)
332fn navigate(
333    action_state: Res<ActionState>,
334    mut auto_directional_navigator: AutoDirectionalNavigator,
335) {
336    let net_east_west = action_state
337        .pressed_actions
338        .contains(&DirectionalNavigationAction::Right) as i8
339        - action_state
340            .pressed_actions
341            .contains(&DirectionalNavigationAction::Left) as i8;
342
343    let net_north_south = action_state
344        .pressed_actions
345        .contains(&DirectionalNavigationAction::Up) as i8
346        - action_state
347            .pressed_actions
348            .contains(&DirectionalNavigationAction::Down) as i8;
349
350    // Use Dir2::from_xy to convert input to direction, then convert to CompassOctant
351    let maybe_direction = Dir2::from_xy(net_east_west as f32, net_north_south as f32)
352        .ok()
353        .map(CompassOctant::from);
354
355    if let Some(direction) = maybe_direction {
356        match auto_directional_navigator.navigate(direction) {
357            Ok(_entity) => {
358                // Successfully navigated
359            }
360            Err(_e) => {
361                // Navigation failed (no neighbor in that direction)
362            }
363        }
364    }
365}
366
367fn update_focus_display(
368    input_focus: Res<InputFocus>,
369    button_query: Query<&Name, With<Button>>,
370    mut display_query: Query<&mut Text, With<FocusDisplay>>,
371) {
372    if let Ok(mut text) = display_query.single_mut() {
373        if let Some(focused_entity) = input_focus.get() {
374            if let Ok(name) = button_query.get(focused_entity) {
375                **text = format!("Focused: {}", name);
376            } else {
377                **text = "Focused: Unknown".to_string();
378            }
379        } else {
380            **text = "Focused: None".to_string();
381        }
382    }
383}
384
385fn update_key_display(
386    keyboard_input: Res<ButtonInput<KeyCode>>,
387    gamepad_input: Query<&Gamepad>,
388    mut display_query: Query<&mut Text, With<KeyDisplay>>,
389) {
390    if let Ok(mut text) = display_query.single_mut() {
391        // Check for keyboard inputs
392        for action in DirectionalNavigationAction::variants() {
393            if keyboard_input.just_pressed(action.keycode()) {
394                let key_name = match action {
395                    DirectionalNavigationAction::Up => "Up Arrow",
396                    DirectionalNavigationAction::Down => "Down Arrow",
397                    DirectionalNavigationAction::Left => "Left Arrow",
398                    DirectionalNavigationAction::Right => "Right Arrow",
399                    DirectionalNavigationAction::Select => "Enter",
400                };
401                **text = format!("Last Key: {}", key_name);
402                return;
403            }
404        }
405
406        // Check for gamepad inputs
407        for gamepad in gamepad_input.iter() {
408            for action in DirectionalNavigationAction::variants() {
409                if gamepad.just_pressed(action.gamepad_button()) {
410                    let button_name = match action {
411                        DirectionalNavigationAction::Up => "D-Pad Up",
412                        DirectionalNavigationAction::Down => "D-Pad Down",
413                        DirectionalNavigationAction::Left => "D-Pad Left",
414                        DirectionalNavigationAction::Right => "D-Pad Right",
415                        DirectionalNavigationAction::Select => "A Button",
416                    };
417                    **text = format!("Last Key: {}", button_name);
418                    return;
419                }
420            }
421        }
422    }
423}
424
425fn highlight_focused_element(
426    input_focus: Res<InputFocus>,
427    input_focus_visible: Res<InputFocusVisible>,
428    mut query: Query<(Entity, &mut BorderColor)>,
429) {
430    for (entity, mut border_color) in query.iter_mut() {
431        if input_focus.get() == Some(entity) && input_focus_visible.0 {
432            *border_color = BorderColor::all(FOCUSED_BORDER);
433        } else {
434            *border_color = BorderColor::DEFAULT;
435        }
436    }
437}
438
439fn interact_with_focused_button(
440    action_state: Res<ActionState>,
441    input_focus: Res<InputFocus>,
442    mut commands: Commands,
443) {
444    if action_state
445        .pressed_actions
446        .contains(&DirectionalNavigationAction::Select)
447        && let Some(focused_entity) = input_focus.get()
448    {
449        commands.trigger(Pointer::new(
450            PointerId::Mouse,
451            Location {
452                target: NormalizedRenderTarget::None {
453                    width: 0,
454                    height: 0,
455                },
456                position: Vec2::ZERO,
457            },
458            Click {
459                button: PointerButton::Primary,
460                hit: HitData {
461                    camera: Entity::PLACEHOLDER,
462                    depth: 0.0,
463                    position: None,
464                    normal: None,
465                    extra: None,
466                },
467                count: 1,
468                duration: Duration::from_secs_f32(0.1),
469            },
470            focused_entity,
471        ));
472    }
473}
examples/ui/navigation/directional_navigation_overrides.rs (line 712)
704fn navigate(
705    action_state: Res<ActionState>,
706    parent_query: Query<&ChildOf>,
707    mut visibility_query: Query<&mut Visibility>,
708    mut auto_directional_navigator: AutoDirectionalNavigator,
709) {
710    let net_east_west = action_state
711        .pressed_actions
712        .contains(&DirectionalNavigationAction::Right) as i8
713        - action_state
714            .pressed_actions
715            .contains(&DirectionalNavigationAction::Left) as i8;
716
717    let net_north_south = action_state
718        .pressed_actions
719        .contains(&DirectionalNavigationAction::Up) as i8
720        - action_state
721            .pressed_actions
722            .contains(&DirectionalNavigationAction::Down) as i8;
723
724    // Use Dir2::from_xy to convert input to direction, then convert to CompassOctant
725    let maybe_direction = Dir2::from_xy(net_east_west as f32, net_north_south as f32)
726        .ok()
727        .map(CompassOctant::from);
728
729    // Store the previous focus in case navigation switches pages.
730    let previous_focus = auto_directional_navigator.input_focus();
731    if let Some(direction) = maybe_direction {
732        match auto_directional_navigator.navigate(direction) {
733            Ok(new_focus) => {
734                // Successfully navigated!
735
736                // If navigation switches between pages, change the visibilities of pages
737                if let Ok(current_child_of) = parent_query.get(new_focus)
738                    && let Ok(mut current_page_visibility) =
739                        visibility_query.get_mut(current_child_of.parent())
740                {
741                    *current_page_visibility = Visibility::Visible;
742
743                    if let Some(previous_focus_entity) = previous_focus
744                        && let Ok(previous_child_of) = parent_query.get(previous_focus_entity)
745                        && previous_child_of.parent() != current_child_of.parent()
746                        && let Ok(mut previous_page_visibility) =
747                            visibility_query.get_mut(previous_child_of.parent())
748                    {
749                        *previous_page_visibility = Visibility::Hidden;
750                    }
751                }
752            }
753            Err(_e) => {
754                // Navigation failed (no neighbor in that direction)
755            }
756        }
757    }
758}
759
760fn update_focus_display(
761    input_focus: Res<InputFocus>,
762    button_query: Query<&Name, With<Button>>,
763    mut display_query: Query<&mut Text, With<FocusDisplay>>,
764) {
765    if let Ok(mut text) = display_query.single_mut() {
766        if let Some(focused_entity) = input_focus.get() {
767            if let Ok(name) = button_query.get(focused_entity) {
768                **text = format!("Focused: {}", name);
769            } else {
770                **text = "Focused: Unknown".to_string();
771            }
772        } else {
773            **text = "Focused: None".to_string();
774        }
775    }
776}
777
778fn update_key_display(
779    keyboard_input: Res<ButtonInput<KeyCode>>,
780    gamepad_input: Query<&Gamepad>,
781    mut display_query: Query<&mut Text, With<KeyDisplay>>,
782) {
783    if let Ok(mut text) = display_query.single_mut() {
784        // Check for keyboard inputs
785        for action in DirectionalNavigationAction::variants() {
786            if keyboard_input.just_pressed(action.keycode()) {
787                let key_name = match action {
788                    DirectionalNavigationAction::Up => "Up Arrow",
789                    DirectionalNavigationAction::Down => "Down Arrow",
790                    DirectionalNavigationAction::Left => "Left Arrow",
791                    DirectionalNavigationAction::Right => "Right Arrow",
792                    DirectionalNavigationAction::Select => "Enter",
793                };
794                **text = format!("Last Key: {}", key_name);
795                return;
796            }
797        }
798
799        // Check for gamepad inputs
800        for gamepad in gamepad_input.iter() {
801            for action in DirectionalNavigationAction::variants() {
802                if gamepad.just_pressed(action.gamepad_button()) {
803                    let button_name = match action {
804                        DirectionalNavigationAction::Up => "D-Pad Up",
805                        DirectionalNavigationAction::Down => "D-Pad Down",
806                        DirectionalNavigationAction::Left => "D-Pad Left",
807                        DirectionalNavigationAction::Right => "D-Pad Right",
808                        DirectionalNavigationAction::Select => "A Button",
809                    };
810                    **text = format!("Last Key: {}", button_name);
811                    return;
812                }
813            }
814        }
815    }
816}
817
818fn highlight_focused_element(
819    input_focus: Res<InputFocus>,
820    input_focus_visible: Res<InputFocusVisible>,
821    mut query: Query<(Entity, &mut BorderColor, &Page)>,
822) {
823    for (entity, mut border_color, page) in query.iter_mut() {
824        if input_focus.get() == Some(entity) && input_focus_visible.0 {
825            *border_color = BorderColor::all(FOCUSED_BORDER_COLORS[page.0]);
826        } else {
827            *border_color = BorderColor::DEFAULT;
828        }
829    }
830}
831
832fn interact_with_focused_button(
833    action_state: Res<ActionState>,
834    input_focus: Res<InputFocus>,
835    mut commands: Commands,
836) {
837    if action_state
838        .pressed_actions
839        .contains(&DirectionalNavigationAction::Select)
840        && let Some(focused_entity) = input_focus.get()
841    {
842        commands.trigger(Pointer::new(
843            PointerId::Mouse,
844            Location {
845                target: NormalizedRenderTarget::None {
846                    width: 0,
847                    height: 0,
848                },
849                position: Vec2::ZERO,
850            },
851            Click {
852                button: PointerButton::Primary,
853                hit: HitData {
854                    camera: Entity::PLACEHOLDER,
855                    depth: 0.0,
856                    position: None,
857                    normal: None,
858                    extra: None,
859                },
860                count: 1,
861                duration: Duration::from_secs_f32(0.1),
862            },
863            focused_entity,
864        ));
865    }
866}
pub fn get<Q>(&self, value: &Q) -> Option<&T>
where Q: Hash + Equivalent<T> + ?Sized,

Returns a reference to the value in the set, if any, that is equal to the given value.

Refer to get for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.get("foo"), Some(&"foo"));
pub fn get_or_insert(&mut self, value: T) -> &T

Inserts the given value into the set if it is not present, then returns a reference to the value in the set.

Refer to get_or_insert for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.get_or_insert("foo"), &"foo");
pub fn get_or_insert_with<Q, F>(&mut self, value: &Q, f: F) -> &T
where Q: Hash + Equivalent<T> + ?Sized, F: FnOnce(&Q) -> T,

Inserts a value computed from f into the set if the given value is not present, then returns a reference to the value in the set.

Refer to get_or_insert_with for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.get_or_insert_with(&"foo", |_| "foo"), &"foo");
pub fn entry(&mut self, value: T) -> Entry<'_, T, S>

Gets the given value’s corresponding entry in the set for in-place manipulation.

Refer to entry for further details.

Examples
let mut map = HashSet::new();

let value = map.entry("foo").or_insert();
pub fn is_disjoint(&self, other: &HashSet<T, S>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

Refer to is_disjoint for further details.

pub fn is_subset(&self, other: &HashSet<T, S>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

Refer to is_subset for further details.

pub fn is_superset(&self, other: &HashSet<T, S>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

Refer to is_superset for further details.

pub fn insert(&mut self, value: T) -> bool

Adds a value to the set.

Refer to insert for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.contains("foo"));
Examples found in repository?
examples/gltf/gltf_extension_animation_graph.rs (line 252)
245    fn on_gltf_node(
246        &mut self,
247        _load_context: &mut LoadContext<'_>,
248        gltf_node: &gltf::Node,
249        entity: &mut EntityWorldMut,
250    ) {
251        if self.animation_root_indices.contains(&gltf_node.index()) {
252            self.animation_root_entities.insert(entity.id());
253        }
254    }
More examples
Hide additional examples
examples/ecs/observers.rs (line 148)
142fn on_add_mine(add: On<Add, Mine>, query: Query<&Mine>, mut index: ResMut<SpatialIndex>) {
143    let mine = query.get(add.entity).unwrap();
144    let tile = (
145        (mine.pos.x / CELL_SIZE).floor() as i32,
146        (mine.pos.y / CELL_SIZE).floor() as i32,
147    );
148    index.map.entry(tile).or_default().insert(add.entity);
149}
examples/shader_advanced/manual_material.rs (line 316)
307fn extract_image_materials_needing_specialization(
308    entities_needing_specialization: Extract<Res<EntitiesNeedingSpecialization<ImageMaterial>>>,
309    mut dirty_specializations: ResMut<DirtySpecializations>,
310) {
311    // Drain the list of entities needing specialization from the main world
312    // into the render-world `DirtySpecializations` table.
313    for entity in entities_needing_specialization.changed.iter() {
314        dirty_specializations
315            .changed_renderables
316            .insert(MainEntity::from(*entity));
317    }
318}
319
320/// A system that adds entities that were judged to need their specializations
321/// removed to the appropriate table in [`DirtySpecializations`].
322fn extract_image_materials_that_need_specializations_removed(
323    entities_needing_specialization: Extract<Res<EntitiesNeedingSpecialization<ImageMaterial>>>,
324    mut dirty_specializations: ResMut<DirtySpecializations>,
325) {
326    for entity in entities_needing_specialization.removed.iter() {
327        dirty_specializations
328            .removed_renderables
329            .insert(MainEntity::from(*entity));
330    }
331}
examples/ui/navigation/directional_navigation.rs (line 319)
310fn process_inputs(
311    mut action_state: ResMut<ActionState>,
312    keyboard_input: Res<ButtonInput<KeyCode>>,
313    gamepad_input: Query<&Gamepad>,
314) {
315    action_state.pressed_actions.clear();
316
317    for action in DirectionalNavigationAction::variants() {
318        if keyboard_input.just_pressed(action.keycode()) {
319            action_state.pressed_actions.insert(action);
320        }
321    }
322
323    for gamepad in gamepad_input.iter() {
324        for action in DirectionalNavigationAction::variants() {
325            if gamepad.just_pressed(action.gamepad_button()) {
326                action_state.pressed_actions.insert(action);
327            }
328        }
329    }
330}
examples/ui/navigation/directional_navigation_overrides.rs (line 691)
682fn process_inputs(
683    mut action_state: ResMut<ActionState>,
684    keyboard_input: Res<ButtonInput<KeyCode>>,
685    gamepad_input: Query<&Gamepad>,
686) {
687    action_state.pressed_actions.clear();
688
689    for action in DirectionalNavigationAction::variants() {
690        if keyboard_input.just_pressed(action.keycode()) {
691            action_state.pressed_actions.insert(action);
692        }
693    }
694
695    for gamepad in gamepad_input.iter() {
696        for action in DirectionalNavigationAction::variants() {
697            if gamepad.just_pressed(action.gamepad_button()) {
698                action_state.pressed_actions.insert(action);
699            }
700        }
701    }
702}
examples/shader_advanced/custom_render_phase.rs (line 514)
500fn extract_camera_phases(
501    mut stencil_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
502    cameras: Extract<Query<(Entity, &Camera), With<Camera3d>>>,
503    mut live_entities: Local<HashSet<RetainedViewEntity>>,
504) {
505    live_entities.clear();
506    for (main_entity, camera) in &cameras {
507        if !camera.is_active {
508            continue;
509        }
510        // This is the main camera, so we use the first subview index (0)
511        let retained_view_entity = RetainedViewEntity::new(main_entity.into(), None, 0);
512
513        stencil_phases.prepare_for_new_frame(retained_view_entity);
514        live_entities.insert(retained_view_entity);
515    }
516
517    // Clear out all dead views.
518    stencil_phases.retain(|camera_entity, _| live_entities.contains(camera_entity));
519}
520
521/// A resource that stores meshes that couldn't be specialized yet because their
522/// materials hadn't loaded.
523///
524/// See the documentation for [`PendingQueues`] for more information.
525#[derive(Default, Deref, DerefMut, Resource)]
526struct PendingCustomMeshQueues(pub PendingQueues);
527
528// This is a very important step when writing a custom phase.
529//
530// This system determines which meshes will be added to the phase.
531fn queue_custom_meshes(
532    custom_draw_functions: Res<DrawFunctions<Stencil3d>>,
533    mut pipelines: ResMut<SpecializedMeshPipelines<StencilPipeline>>,
534    pipeline_cache: Res<PipelineCache>,
535    custom_draw_pipeline: Res<StencilPipeline>,
536    render_meshes: Res<RenderAssets<RenderMesh>>,
537    render_mesh_instances: Res<RenderMeshInstances>,
538    maybe_batched_instance_buffers: Option<
539        Res<BatchedInstanceBuffers<MeshUniform, MeshInputUniform>>,
540    >,
541    mut custom_render_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
542    mut views: Query<(&ExtractedView, &RenderVisibleEntities)>,
543    view_key_cache: Res<ViewKeyCache>,
544    dirty_specializations: Res<DirtySpecializations>,
545    mut pending_custom_mesh_queues: ResMut<PendingCustomMeshQueues>,
546    has_marker: Query<(), With<DrawStencil>>,
547) {
548    for (view, visible_entities) in &mut views {
549        let Some(custom_phase) = custom_render_phases.get_mut(&view.retained_view_entity) else {
550            continue;
551        };
552        let draw_custom = custom_draw_functions.read().id::<DrawMesh3dStencil>();
553
554        let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
555            continue;
556        };
557
558        // Since our phase can work on any 3d mesh we can reuse the default mesh 3d filter
559        let Some(render_visible_mesh_entities) = visible_entities.get::<Mesh3d>() else {
560            continue;
561        };
562
563        let view_pending_custom_mesh_queues =
564            pending_custom_mesh_queues.prepare_for_new_frame(view.retained_view_entity);
565
566        // First, remove meshes that need to be respecialized, and those that were removed, from the bins.
567        for &main_entity in dirty_specializations
568            .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
569        {
570            custom_phase.remove(Entity::PLACEHOLDER, main_entity);
571        }
572
573        for (render_entity, visible_entity) in dirty_specializations.iter_to_queue(
574            view.retained_view_entity,
575            render_visible_mesh_entities,
576            &view_pending_custom_mesh_queues.prev_frame,
577        ) {
578            // We only want meshes with the marker component to be queued to our phase.
579            if has_marker.get(*render_entity).is_err() {
580                continue;
581            }
582            let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*visible_entity)
583            else {
584                // We couldn't fetch the mesh, probably because it hasn't been
585                // loaded yet. Add the entity to the list of pending custom mesh
586                // queues and bail.
587                view_pending_custom_mesh_queues
588                    .current_frame
589                    .insert((*render_entity, *visible_entity));
590                continue;
591            };
592            let Some(mesh) = render_meshes.get(mesh_instance.mesh_asset_id()) else {
593                continue;
594            };
595
596            // Specialize the key for the current mesh entity
597            // For this example we only specialize based on the mesh topology
598            // but you could have more complex keys and that's where you'd need to create those keys
599            let mut mesh_key = view_key;
600            mesh_key |= MeshPipelineKey::from_primitive_topology_and_strip_index(
601                mesh.primitive_topology(),
602                mesh.index_format(),
603            );
604
605            let pipeline_id = pipelines.specialize(
606                &pipeline_cache,
607                &custom_draw_pipeline,
608                mesh_key,
609                &mesh.layout,
610            );
611            let pipeline_id = match pipeline_id {
612                Ok(id) => id,
613                Err(err) => {
614                    error!("{}", err);
615                    continue;
616                }
617            };
618            // At this point we have all the data we need to create a phase item and add it to our
619            // phase
620            custom_phase.add_retained(Stencil3d {
621                sorting_info: TransparentSortingInfo3d::Sorted {
622                    mesh_center: pbr::get_mesh_instance_world_from_local(
623                        *visible_entity,
624                        mesh_instance.current_uniform_index,
625                        &render_mesh_instances,
626                        maybe_batched_instance_buffers.as_deref(),
627                    )
628                    .transform_point3(
629                        render_meshes
630                            .get(mesh_instance.mesh_asset_id())
631                            .unwrap()
632                            .aabb_center,
633                    ),
634                    depth_bias: 0.0,
635                },
636                distance: FloatOrd(0.0),
637                entity: (Entity::PLACEHOLDER, *visible_entity),
638                pipeline: pipeline_id,
639                draw_function: draw_custom,
640                // Sorted phase items aren't batched
641                batch_range: 0..1,
642                extra_index: PhaseItemExtraIndex::None,
643                indexed: mesh.indexed(),
644            });
645        }
646    }
647}
pub fn replace(&mut self, value: T) -> Option<T>

Adds a value to the set, replacing the existing value, if any, that is equal to the given one. Returns the replaced value.

Refer to replace for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.replace("foo"), Some("foo"));
pub fn remove<Q>(&mut self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Removes a value from the set. Returns whether the value was present in the set.

Refer to remove for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert!(map.remove("foo"));

assert!(map.is_empty());
Examples found in repository?
examples/ecs/observers.rs (line 159)
152fn on_remove_mine(remove: On<Remove, Mine>, query: Query<&Mine>, mut index: ResMut<SpatialIndex>) {
153    let mine = query.get(remove.entity).unwrap();
154    let tile = (
155        (mine.pos.x / CELL_SIZE).floor() as i32,
156        (mine.pos.y / CELL_SIZE).floor() as i32,
157    );
158    index.map.entry(tile).and_modify(|set| {
159        set.remove(&remove.entity);
160    });
161}
pub fn take<Q>(&mut self, value: &Q) -> Option<T>
where Q: Hash + Equivalent<T> + ?Sized,

Removes and returns the value in the set, if any, that is equal to the given one.

Refer to take for further details.

Examples
let mut map = HashSet::new();

map.insert("foo");

assert_eq!(map.take("foo"), Some("foo"));

assert!(map.is_empty());
pub fn allocation_size(&self) -> usize

Returns the total amount of memory allocated internally by the hash set, in bytes.

Refer to allocation_size for further details.

Examples
let mut map = HashSet::new();

assert_eq!(map.allocation_size(), 0);

map.insert("foo");

assert!(map.allocation_size() >= size_of::<&'static str>());
pub unsafe fn insert_unique_unchecked(&mut self, value: T) -> &T

Insert a value the set without checking if the value already exists in the set.

Refer to insert_unique_unchecked for further details.

Safety

This operation is safe if a value does not exist in the set.

However, if a value exists in the set already, the behavior is unspecified: this operation may panic, loop forever, or any following operation with the set may panic, loop forever or return arbitrary result.

That said, this operation (and following operations) are guaranteed to not violate memory safety.

However this operation is still unsafe because the resulting HashSet may be passed to unsafe code which does expect the set to behave correctly, and would cause unsoundness as a result.

Methods from Deref<Target = HashSet<T, S>>

pub fn capacity(&self) -> usize

Returns the number of elements the set can hold without reallocating.

Examples
use hashbrown::HashSet;
let set: HashSet<i32> = HashSet::with_capacity(100);
assert!(set.capacity() >= 100);
pub fn iter(&self) -> Iter<'_, T>

An iterator visiting all elements in arbitrary order. The iterator element type is &'a T.

Examples
use hashbrown::HashSet;
let mut set = HashSet::new();
set.insert("a");
set.insert("b");

// Will print in an arbitrary order.
for x in set.iter() {
    println!("{}", x);
}
pub fn len(&self) -> usize

Returns the number of elements in the set.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
assert_eq!(v.len(), 0);
v.insert(1);
assert_eq!(v.len(), 1);
pub fn is_empty(&self) -> bool

Returns true if the set contains no elements.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
assert!(v.is_empty());
v.insert(1);
assert!(!v.is_empty());
pub fn drain(&mut self) -> Drain<'_, T, A>

Clears the set, returning all elements in an iterator.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert!(!set.is_empty());

// print 1, 2, 3 in an arbitrary order
for i in set.drain() {
    println!("{}", i);
}

assert!(set.is_empty());
pub fn retain<F>(&mut self, f: F)
where F: FnMut(&T) -> bool,

Retains only the elements specified by the predicate.

In other words, remove all elements e such that f(&e) returns false.

Examples
use hashbrown::HashSet;

let xs = [1,2,3,4,5,6];
let mut set: HashSet<i32> = xs.into_iter().collect();
set.retain(|&k| k % 2 == 0);
assert_eq!(set.len(), 3);
pub fn extract_if<F>(&mut self, f: F) -> ExtractIf<'_, T, F, A>
where F: FnMut(&T) -> bool,

Drains elements which are true under the given predicate, and returns an iterator over the removed items.

In other words, move all elements e such that f(&e) returns true out into another iterator.

If the returned ExtractIf is not exhausted, e.g. because it is dropped without iterating or the iteration short-circuits, then the remaining elements will be retained. Use retain() with a negated predicate if you do not need the returned iterator.

Examples
use hashbrown::HashSet;

let mut set: HashSet<i32> = (0..8).collect();
let drained: HashSet<i32> = set.extract_if(|v| v % 2 == 0).collect();

let mut evens = drained.into_iter().collect::<Vec<_>>();
let mut odds = set.into_iter().collect::<Vec<_>>();
evens.sort();
odds.sort();

assert_eq!(evens, vec![0, 2, 4, 6]);
assert_eq!(odds, vec![1, 3, 5, 7]);
pub fn clear(&mut self)

Clears the set, removing all values.

Examples
use hashbrown::HashSet;

let mut v = HashSet::new();
v.insert(1);
v.clear();
assert!(v.is_empty());
pub fn allocator(&self) -> &A

Returns a reference to the underlying allocator.

pub fn hasher(&self) -> &S

Returns a reference to the set’s BuildHasher.

Examples
use hashbrown::HashSet;
use hashbrown::DefaultHashBuilder;

let hasher = DefaultHashBuilder::default();
let set: HashSet<i32> = HashSet::with_hasher(hasher);
let hasher: &DefaultHashBuilder = set.hasher();
pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the HashSet. The collection may reserve more space to avoid frequent reallocations.

Panics

Panics if the new capacity exceeds isize::MAX bytes and abort the program in case of allocation error. Use try_reserve instead if you want to handle memory allocation failure.

Examples
use hashbrown::HashSet;
let mut set: HashSet<i32> = HashSet::new();
set.reserve(10);
assert!(set.capacity() >= 10);
pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError>

Tries to reserve capacity for at least additional more elements to be inserted in the given HashSet<K,V>. The collection may reserve more space to avoid frequent reallocations.

Errors

If the capacity overflows, or the allocator reports a failure, then an error is returned.

Examples
use hashbrown::HashSet;
let mut set: HashSet<i32> = HashSet::new();
set.try_reserve(10).expect("why is the test harness OOMing on 10 bytes?");
pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the set as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Examples
use hashbrown::HashSet;

let mut set = HashSet::with_capacity(100);
set.insert(1);
set.insert(2);
assert!(set.capacity() >= 100);
set.shrink_to_fit();
assert!(set.capacity() >= 2);
pub fn shrink_to(&mut self, min_capacity: usize)

Shrinks the capacity of the set with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

Panics if the current capacity is smaller than the supplied minimum capacity.

Examples
use hashbrown::HashSet;

let mut set = HashSet::with_capacity(100);
set.insert(1);
set.insert(2);
assert!(set.capacity() >= 100);
set.shrink_to(10);
assert!(set.capacity() >= 10);
set.shrink_to(0);
assert!(set.capacity() >= 2);
pub fn difference<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> Difference<'a, T, S, A>

Visits the values representing the difference, i.e., the values that are in self but not in other.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Can be seen as `a - b`.
for x in a.difference(&b) {
    println!("{}", x); // Print 1
}

let diff: HashSet<_> = a.difference(&b).collect();
assert_eq!(diff, [1].iter().collect());

// Note that difference is not symmetric,
// and `b - a` means something else:
let diff: HashSet<_> = b.difference(&a).collect();
assert_eq!(diff, [4].iter().collect());
pub fn symmetric_difference<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> SymmetricDifference<'a, T, S, A>

Visits the values representing the symmetric difference, i.e., the values that are in self or in other but not in both.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 1, 4 in arbitrary order.
for x in a.symmetric_difference(&b) {
    println!("{}", x);
}

let diff1: HashSet<_> = a.symmetric_difference(&b).collect();
let diff2: HashSet<_> = b.symmetric_difference(&a).collect();

assert_eq!(diff1, diff2);
assert_eq!(diff1, [1, 4].iter().collect());
pub fn intersection<'a>( &'a self, other: &'a HashSet<T, S, A>, ) -> Intersection<'a, T, S, A>

Visits the values representing the intersection, i.e., the values that are both in self and other.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 2, 3 in arbitrary order.
for x in a.intersection(&b) {
    println!("{}", x);
}

let intersection: HashSet<_> = a.intersection(&b).collect();
assert_eq!(intersection, [2, 3].iter().collect());
pub fn union<'a>(&'a self, other: &'a HashSet<T, S, A>) -> Union<'a, T, S, A>

Visits the values representing the union, i.e., all the values in self or other, without duplicates.

Examples
use hashbrown::HashSet;
let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let b: HashSet<_> = [4, 2, 3, 4].into_iter().collect();

// Print 1, 2, 3, 4 in arbitrary order.
for x in a.union(&b) {
    println!("{}", x);
}

let union: HashSet<_> = a.union(&b).collect();
assert_eq!(union, [1, 2, 3, 4].iter().collect());
pub fn contains<Q>(&self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Returns true if the set contains a value.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.contains(&1), true);
assert_eq!(set.contains(&4), false);
pub fn get<Q>(&self, value: &Q) -> Option<&T>
where Q: Hash + Equivalent<T> + ?Sized,

Returns a reference to the value in the set, if any, that is equal to the given value.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.get(&2), Some(&2));
assert_eq!(set.get(&4), None);
pub fn get_or_insert(&mut self, value: T) -> &T

Inserts the given value into the set if it is not present, then returns a reference to the value in the set.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.len(), 3);
assert_eq!(set.get_or_insert(2), &2);
assert_eq!(set.get_or_insert(100), &100);
assert_eq!(set.len(), 4); // 100 was inserted
pub fn get_or_insert_with<Q, F>(&mut self, value: &Q, f: F) -> &T
where Q: Hash + Equivalent<T> + ?Sized, F: FnOnce(&Q) -> T,

Inserts a value computed from f into the set if the given value is not present, then returns a reference to the value in the set.

Examples
use hashbrown::HashSet;

let mut set: HashSet<String> = ["cat", "dog", "horse"]
    .iter().map(|&pet| pet.to_owned()).collect();

assert_eq!(set.len(), 3);
for &pet in &["cat", "dog", "fish"] {
    let value = set.get_or_insert_with(pet, str::to_owned);
    assert_eq!(value, pet);
}
assert_eq!(set.len(), 4); // a new "fish" was inserted

The following example will panic because the new value doesn’t match.

let mut set = hashbrown::HashSet::new();
set.get_or_insert_with("rust", |_| String::new());
pub fn entry(&mut self, value: T) -> Entry<'_, T, S, A>

Gets the given value’s corresponding entry in the set for in-place manipulation.

Examples
use hashbrown::HashSet;
use hashbrown::hash_set::Entry::*;

let mut singles = HashSet::new();
let mut dupes = HashSet::new();

for ch in "a short treatise on fungi".chars() {
    if let Vacant(dupe_entry) = dupes.entry(ch) {
        // We haven't already seen a duplicate, so
        // check if we've at least seen it once.
        match singles.entry(ch) {
            Vacant(single_entry) => {
                // We found a new character for the first time.
                single_entry.insert();
            }
            Occupied(single_entry) => {
                // We've already seen this once, "move" it to dupes.
                single_entry.remove();
                dupe_entry.insert();
            }
        }
    }
}

assert!(!singles.contains(&'t') && dupes.contains(&'t'));
assert!(singles.contains(&'u') && !dupes.contains(&'u'));
assert!(!singles.contains(&'v') && !dupes.contains(&'v'));
pub fn is_disjoint(&self, other: &HashSet<T, S, A>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

Examples
use hashbrown::HashSet;

let a: HashSet<_> = [1, 2, 3].into_iter().collect();
let mut b = HashSet::new();

assert_eq!(a.is_disjoint(&b), true);
b.insert(4);
assert_eq!(a.is_disjoint(&b), true);
b.insert(1);
assert_eq!(a.is_disjoint(&b), false);
pub fn is_subset(&self, other: &HashSet<T, S, A>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

Examples
use hashbrown::HashSet;

let sup: HashSet<_> = [1, 2, 3].into_iter().collect();
let mut set = HashSet::new();

assert_eq!(set.is_subset(&sup), true);
set.insert(2);
assert_eq!(set.is_subset(&sup), true);
set.insert(4);
assert_eq!(set.is_subset(&sup), false);
pub fn is_superset(&self, other: &HashSet<T, S, A>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

Examples
use hashbrown::HashSet;

let sub: HashSet<_> = [1, 2].into_iter().collect();
let mut set = HashSet::new();

assert_eq!(set.is_superset(&sub), false);

set.insert(0);
set.insert(1);
assert_eq!(set.is_superset(&sub), false);

set.insert(2);
assert_eq!(set.is_superset(&sub), true);
pub fn insert(&mut self, value: T) -> bool

Adds a value to the set.

If the set did not have this value present, true is returned.

If the set did have this value present, false is returned.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();

assert_eq!(set.insert(2), true);
assert_eq!(set.insert(2), false);
assert_eq!(set.len(), 1);
pub unsafe fn insert_unique_unchecked(&mut self, value: T) -> &T

Insert a value the set without checking if the value already exists in the set.

This operation is faster than regular insert, because it does not perform lookup before insertion.

This operation is useful during initial population of the set. For example, when constructing a set from another set, we know that values are unique.

Safety

This operation is safe if a value does not exist in the set.

However, if a value exists in the set already, the behavior is unspecified: this operation may panic, loop forever, or any following operation with the set may panic, loop forever or return arbitrary result.

That said, this operation (and following operations) are guaranteed to not violate memory safety.

However this operation is still unsafe because the resulting HashSet may be passed to unsafe code which does expect the set to behave correctly, and would cause unsoundness as a result.

pub fn replace(&mut self, value: T) -> Option<T>

Adds a value to the set, replacing the existing value, if any, that is equal to the given one. Returns the replaced value.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();
set.insert(Vec::<i32>::new());

assert_eq!(set.get(&[][..]).unwrap().capacity(), 0);
set.replace(Vec::with_capacity(10));
assert_eq!(set.get(&[][..]).unwrap().capacity(), 10);
pub fn remove<Q>(&mut self, value: &Q) -> bool
where Q: Hash + Equivalent<T> + ?Sized,

Removes a value from the set. Returns whether the value was present in the set.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let mut set = HashSet::new();

set.insert(2);
assert_eq!(set.remove(&2), true);
assert_eq!(set.remove(&2), false);
pub fn take<Q>(&mut self, value: &Q) -> Option<T>
where Q: Hash + Equivalent<T> + ?Sized,

Removes and returns the value in the set, if any, that is equal to the given one.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

Examples
use hashbrown::HashSet;

let mut set: HashSet<_> = [1, 2, 3].into_iter().collect();
assert_eq!(set.take(&2), Some(2));
assert_eq!(set.take(&2), None);
pub fn allocation_size(&self) -> usize

Returns the total amount of memory allocated internally by the hash set, in bytes.

The returned number is informational only. It is intended to be primarily used for memory profiling.