pub struct RenderMeshInstanceFlags(/* private fields */);Expand description
Various useful flags for [RenderMeshInstance]s.
Implementations
Sourceimpl RenderMeshInstanceFlags
impl RenderMeshInstanceFlags
Sourcepub const SHADOW_CASTER: RenderMeshInstanceFlags
pub const SHADOW_CASTER: RenderMeshInstanceFlags
The mesh casts shadows.
Sourcepub const AUTOMATIC_BATCHING: RenderMeshInstanceFlags
pub const AUTOMATIC_BATCHING: RenderMeshInstanceFlags
The mesh can participate in automatic batching.
Sourcepub const HAS_PREVIOUS_TRANSFORM: RenderMeshInstanceFlags
pub const HAS_PREVIOUS_TRANSFORM: RenderMeshInstanceFlags
The mesh had a transform last frame and so is eligible for motion vector computation.
Sourcepub const HAS_PREVIOUS_SKIN: RenderMeshInstanceFlags
pub const HAS_PREVIOUS_SKIN: RenderMeshInstanceFlags
The mesh had a skin last frame and so that skin should be taken into account for motion vector computation.
Sourcepub const HAS_PREVIOUS_MORPH: RenderMeshInstanceFlags
pub const HAS_PREVIOUS_MORPH: RenderMeshInstanceFlags
The mesh had morph targets last frame and so they should be taken into account for motion vector computation.
Sourcepub const NO_CPU_CULLING: RenderMeshInstanceFlags
pub const NO_CPU_CULLING: RenderMeshInstanceFlags
CPU culling has been disabled because the NoCpuCulling component
is present on the mesh instance.
Sourceimpl RenderMeshInstanceFlags
impl RenderMeshInstanceFlags
Sourcepub const fn empty() -> RenderMeshInstanceFlags
pub const fn empty() -> RenderMeshInstanceFlags
Get a flags value with all bits unset.
Sourcepub const fn all() -> RenderMeshInstanceFlags
pub const fn all() -> RenderMeshInstanceFlags
Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> u32
pub const fn bits(&self) -> u32
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u32) -> Option<RenderMeshInstanceFlags>
pub const fn from_bits(bits: u32) -> Option<RenderMeshInstanceFlags>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u32) -> RenderMeshInstanceFlags
pub const fn from_bits_truncate(bits: u32) -> RenderMeshInstanceFlags
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u32) -> RenderMeshInstanceFlags
pub const fn from_bits_retain(bits: u32) -> RenderMeshInstanceFlags
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<RenderMeshInstanceFlags>
pub fn from_name(name: &str) -> Option<RenderMeshInstanceFlags>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: RenderMeshInstanceFlags) -> bool
pub const fn intersects(&self, other: RenderMeshInstanceFlags) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: RenderMeshInstanceFlags) -> bool
pub const fn contains(&self, other: RenderMeshInstanceFlags) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn insert(&mut self, other: RenderMeshInstanceFlags)
pub fn insert(&mut self, other: RenderMeshInstanceFlags)
The bitwise or (|) of the bits in self and other.
Sourcepub fn remove(&mut self, other: RenderMeshInstanceFlags)
pub fn remove(&mut self, other: RenderMeshInstanceFlags)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: RenderMeshInstanceFlags)
pub fn toggle(&mut self, other: RenderMeshInstanceFlags)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: RenderMeshInstanceFlags, value: bool)
pub fn set(&mut self, other: RenderMeshInstanceFlags, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(
self,
other: RenderMeshInstanceFlags,
) -> RenderMeshInstanceFlags
pub const fn intersection( self, other: RenderMeshInstanceFlags, ) -> RenderMeshInstanceFlags
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(
self,
other: RenderMeshInstanceFlags,
) -> RenderMeshInstanceFlags
pub const fn union( self, other: RenderMeshInstanceFlags, ) -> RenderMeshInstanceFlags
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(
self,
other: RenderMeshInstanceFlags,
) -> RenderMeshInstanceFlags
pub const fn difference( self, other: RenderMeshInstanceFlags, ) -> RenderMeshInstanceFlags
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(
self,
other: RenderMeshInstanceFlags,
) -> RenderMeshInstanceFlags
pub const fn symmetric_difference( self, other: RenderMeshInstanceFlags, ) -> RenderMeshInstanceFlags
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> RenderMeshInstanceFlags
pub const fn complement(self) -> RenderMeshInstanceFlags
The bitwise negation (!) of the bits in self, truncating the result.
Sourceimpl RenderMeshInstanceFlags
impl RenderMeshInstanceFlags
Sourcepub const fn iter(&self) -> Iter<RenderMeshInstanceFlags> ⓘ
pub const fn iter(&self) -> Iter<RenderMeshInstanceFlags> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<RenderMeshInstanceFlags> ⓘ
pub const fn iter_names(&self) -> IterNames<RenderMeshInstanceFlags> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.