pub struct StandardMaterialFlags(/* private fields */);Expand description
Bitflags info about the material a shader is currently rendering.
This is accessible in the shader in the StandardMaterialUniform
Implementations
Sourceimpl StandardMaterialFlags
impl StandardMaterialFlags
pub const BASE_COLOR_TEXTURE: StandardMaterialFlags
pub const EMISSIVE_TEXTURE: StandardMaterialFlags
pub const METALLIC_ROUGHNESS_TEXTURE: StandardMaterialFlags
pub const OCCLUSION_TEXTURE: StandardMaterialFlags
pub const DOUBLE_SIDED: StandardMaterialFlags
pub const UNLIT: StandardMaterialFlags
pub const TWO_COMPONENT_NORMAL_MAP: StandardMaterialFlags
pub const FLIP_NORMAL_MAP_Y: StandardMaterialFlags
pub const FOG_ENABLED: StandardMaterialFlags
pub const DEPTH_MAP: StandardMaterialFlags
pub const SPECULAR_TRANSMISSION_TEXTURE: StandardMaterialFlags
pub const THICKNESS_TEXTURE: StandardMaterialFlags
pub const DIFFUSE_TRANSMISSION_TEXTURE: StandardMaterialFlags
pub const ATTENUATION_ENABLED: StandardMaterialFlags
pub const CLEARCOAT_TEXTURE: StandardMaterialFlags
pub const CLEARCOAT_ROUGHNESS_TEXTURE: StandardMaterialFlags
pub const CLEARCOAT_NORMAL_TEXTURE: StandardMaterialFlags
pub const ANISOTROPY_TEXTURE: StandardMaterialFlags
pub const SPECULAR_TEXTURE: StandardMaterialFlags
pub const SPECULAR_TINT_TEXTURE: StandardMaterialFlags
pub const ALPHA_MODE_RESERVED_BITS: StandardMaterialFlags
pub const ALPHA_MODE_OPAQUE: StandardMaterialFlags
pub const ALPHA_MODE_MASK: StandardMaterialFlags
pub const ALPHA_MODE_BLEND: StandardMaterialFlags
pub const ALPHA_MODE_PREMULTIPLIED: StandardMaterialFlags
pub const ALPHA_MODE_ADD: StandardMaterialFlags
pub const ALPHA_MODE_MULTIPLY: StandardMaterialFlags
pub const ALPHA_MODE_ALPHA_TO_COVERAGE: StandardMaterialFlags
pub const NONE: StandardMaterialFlags
pub const UNINITIALIZED: StandardMaterialFlags
Sourceimpl StandardMaterialFlags
impl StandardMaterialFlags
Sourcepub const fn empty() -> StandardMaterialFlags
pub const fn empty() -> StandardMaterialFlags
Get a flags value with all bits unset.
Sourcepub const fn all() -> StandardMaterialFlags
pub const fn all() -> StandardMaterialFlags
Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> u32
pub const fn bits(&self) -> u32
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u32) -> Option<StandardMaterialFlags>
pub const fn from_bits(bits: u32) -> Option<StandardMaterialFlags>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u32) -> StandardMaterialFlags
pub const fn from_bits_truncate(bits: u32) -> StandardMaterialFlags
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u32) -> StandardMaterialFlags
pub const fn from_bits_retain(bits: u32) -> StandardMaterialFlags
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<StandardMaterialFlags>
pub fn from_name(name: &str) -> Option<StandardMaterialFlags>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: StandardMaterialFlags) -> bool
pub const fn intersects(&self, other: StandardMaterialFlags) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: StandardMaterialFlags) -> bool
pub const fn contains(&self, other: StandardMaterialFlags) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn insert(&mut self, other: StandardMaterialFlags)
pub fn insert(&mut self, other: StandardMaterialFlags)
The bitwise or (|) of the bits in self and other.
Sourcepub fn remove(&mut self, other: StandardMaterialFlags)
pub fn remove(&mut self, other: StandardMaterialFlags)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: StandardMaterialFlags)
pub fn toggle(&mut self, other: StandardMaterialFlags)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: StandardMaterialFlags, value: bool)
pub fn set(&mut self, other: StandardMaterialFlags, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(
self,
other: StandardMaterialFlags,
) -> StandardMaterialFlags
pub const fn intersection( self, other: StandardMaterialFlags, ) -> StandardMaterialFlags
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(self, other: StandardMaterialFlags) -> StandardMaterialFlags
pub const fn union(self, other: StandardMaterialFlags) -> StandardMaterialFlags
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(
self,
other: StandardMaterialFlags,
) -> StandardMaterialFlags
pub const fn difference( self, other: StandardMaterialFlags, ) -> StandardMaterialFlags
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(
self,
other: StandardMaterialFlags,
) -> StandardMaterialFlags
pub const fn symmetric_difference( self, other: StandardMaterialFlags, ) -> StandardMaterialFlags
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> StandardMaterialFlags
pub const fn complement(self) -> StandardMaterialFlags
The bitwise negation (!) of the bits in self, truncating the result.
Sourceimpl StandardMaterialFlags
impl StandardMaterialFlags
Sourcepub const fn iter(&self) -> Iter<StandardMaterialFlags> ⓘ
pub const fn iter(&self) -> Iter<StandardMaterialFlags> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<StandardMaterialFlags> ⓘ
pub const fn iter_names(&self) -> IterNames<StandardMaterialFlags> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.