pub struct RwLockWriteGuard<'rwlock, T>where
T: 'rwlock + ?Sized,{ /* private fields */ }Expand description
Implementations
Sourceimpl<'rwlock, T> RwLockWriteGuard<'rwlock, T>where
T: ?Sized,
impl<'rwlock, T> RwLockWriteGuard<'rwlock, T>where
T: ?Sized,
1.92.0 · Sourcepub fn downgrade(s: RwLockWriteGuard<'rwlock, T>) -> RwLockReadGuard<'rwlock, T>
pub fn downgrade(s: RwLockWriteGuard<'rwlock, T>) -> RwLockReadGuard<'rwlock, T>
Downgrades a write-locked RwLockWriteGuard into a read-locked RwLockReadGuard.
Since we have the RwLockWriteGuard, the RwLock must already be locked for writing, so
this method cannot fail.
After downgrading, other readers will be allowed to read the protected data.
Examples
downgrade takes ownership of the RwLockWriteGuard and returns a RwLockReadGuard.
use std::sync::{RwLock, RwLockWriteGuard};
let rw = RwLock::new(0);
let mut write_guard = rw.write().unwrap();
*write_guard = 42;
let read_guard = RwLockWriteGuard::downgrade(write_guard);
assert_eq!(42, *read_guard);downgrade will atomically change the state of the RwLock from exclusive mode into
shared mode. This means that it is impossible for another writing thread to get in between a
thread calling downgrade and any reads it performs after downgrading.
use std::sync::{Arc, RwLock, RwLockWriteGuard};
let rw = Arc::new(RwLock::new(1));
// Put the lock in write mode.
let mut main_write_guard = rw.write().unwrap();
let rw_clone = rw.clone();
let evil_handle = std::thread::spawn(move || {
// This will not return until the main thread drops the `main_read_guard`.
let mut evil_guard = rw_clone.write().unwrap();
assert_eq!(*evil_guard, 2);
*evil_guard = 3;
});
*main_write_guard = 2;
// Atomically downgrade the write guard into a read guard.
let main_read_guard = RwLockWriteGuard::downgrade(main_write_guard);
// Since `downgrade` is atomic, the writer thread cannot have changed the protected data.
assert_eq!(*main_read_guard, 2, "`downgrade` was not atomic");Sourcepub fn map<U, F>(
orig: RwLockWriteGuard<'rwlock, T>,
f: F,
) -> MappedRwLockWriteGuard<'rwlock, U>
🔬This is a nightly-only experimental API. (mapped_lock_guards)
pub fn map<U, F>( orig: RwLockWriteGuard<'rwlock, T>, f: F, ) -> MappedRwLockWriteGuard<'rwlock, U>
mapped_lock_guards)Makes a MappedRwLockWriteGuard for a component of the borrowed data, e.g.
an enum variant.
The RwLock is already locked for writing, so this cannot fail.
This is an associated function that needs to be used as
RwLockWriteGuard::map(...). A method would interfere with methods of
the same name on the contents of the RwLockWriteGuard used through
Deref.
Panics
If the closure panics, the guard will be dropped (unlocked) and the RwLock will be poisoned.
Sourcepub fn filter_map<U, F>(
orig: RwLockWriteGuard<'rwlock, T>,
f: F,
) -> Result<MappedRwLockWriteGuard<'rwlock, U>, RwLockWriteGuard<'rwlock, T>>
🔬This is a nightly-only experimental API. (mapped_lock_guards)
pub fn filter_map<U, F>( orig: RwLockWriteGuard<'rwlock, T>, f: F, ) -> Result<MappedRwLockWriteGuard<'rwlock, U>, RwLockWriteGuard<'rwlock, T>>
mapped_lock_guards)Makes a MappedRwLockWriteGuard for a component of the borrowed data. The
original guard is returned as an Err(...) if the closure returns
None.
The RwLock is already locked for writing, so this cannot fail.
This is an associated function that needs to be used as
RwLockWriteGuard::filter_map(...). A method would interfere with methods
of the same name on the contents of the RwLockWriteGuard used through
Deref.
Panics
If the closure panics, the guard will be dropped (unlocked) and the RwLock will be poisoned.