PostProcessingUniformBuffers

Struct PostProcessingUniformBuffers 

pub struct PostProcessingUniformBuffers { /* private fields */ }
A resource, part of the render world, that stores the ChromaticAberrationUniforms for each view.

Methods from Deref<Target = DynamicUniformBuffer<ChromaticAberrationUniform>>

pub fn buffer(&self) -> Option<&Buffer>

pub fn binding(&self) -> Option<BindingResource<'_>>

pub fn is_empty(&self) -> bool

pub fn push(&mut self, value: &T) -> u32

Push data into the DynamicUniformBuffer’s internal vector (residing on system RAM).

pub fn set_label(&mut self, label: Option<&str>)

pub fn get_label(&self) -> Option<&str>

pub fn add_usages(&mut self, usage: BufferUsages)

Add more BufferUsages to the buffer.

This method only allows addition of flags to the default usage flags.

The default values for buffer usage are BufferUsages::COPY_DST and BufferUsages::UNIFORM.

pub fn get_writer<'a>( &'a mut self, max_count: usize, device: &RenderDevice, queue: &'a RenderQueue, ) -> Option<DynamicUniformBufferWriter<T>>

Creates a writer that can be used to directly write elements into the target buffer.

This method uses less memory and performs fewer memory copies using over push and write_buffer.

max_count must be greater than or equal to the number of elements that are to be written to the buffer, or the writer will panic while writing. Dropping the writer will schedule the buffer write into the provided RenderQueue.

If there is no GPU-side buffer allocated to hold the data currently stored, or if a GPU-side buffer previously allocated does not have enough capacity to hold max_count elements, a new GPU-side buffer is created.

Returns None if there is no allocated GPU-side buffer, and max_count is 0.

pub fn write_buffer(&mut self, device: &RenderDevice, queue: &RenderQueue)

Queues writing of data from system RAM to VRAM using the RenderDevice and the provided RenderQueue.

If there is no GPU-side buffer allocated to hold the data currently stored, or if a GPU-side buffer previously allocated does not have enough capacity, a new GPU-side buffer is created.

pub fn clear(&mut self)