pub enum AppExit {
Success,
Error(NonZero<u8>),
}Expand description
A Message that indicates the App should exit. If one or more of these are present at the end of an update,
the runner will end and (maybe) return control to the caller.
This message can be used to detect when an exit is requested. Make sure that systems listening for this message run before the current update ends.
Portability
This type is roughly meant to map to a standard definition of a process exit code (0 means success, not 0 means error). Due to portability concerns
(see ExitCode and process::exit)
we only allow error codes between 1 and 255.
Variants
Success
App exited without any problems.
Error(NonZero<u8>)
The App experienced an unhandleable error.
Holds the exit code we expect our app to return.
Implementations
Sourceimpl AppExit
impl AppExit
Sourcepub const fn error() -> AppExit
pub const fn error() -> AppExit
Creates a AppExit::Error with an error code of 1.
Examples found in repository?
154fn setup(
155 mut commands: Commands,
156 asset_server: Res<AssetServer>,
157 app_status: Res<AppStatus>,
158 render_device: Res<RenderDevice>,
159 render_adapter: Res<RenderAdapter>,
160 mut meshes: ResMut<Assets<Mesh>>,
161 mut materials: ResMut<Assets<ExtendedMaterial<StandardMaterial, CustomDecalExtension>>>,
162) {
163 // Error out if clustered decals aren't supported on the current platform.
164 if !decal::clustered::clustered_decals_are_usable(&render_device, &render_adapter) {
165 error!("Clustered decals aren't usable on this platform.");
166 commands.write_message(AppExit::error());
167 }
168
169 spawn_cube(&mut commands, &mut meshes, &mut materials);
170 spawn_camera(&mut commands);
171 spawn_light(&mut commands);
172 spawn_decals(&mut commands, &asset_server);
173 spawn_buttons(&mut commands);
174 spawn_help_text(&mut commands, &app_status);
175}More examples
143fn setup(
144 mut commands: Commands,
145 asset_server: Res<AssetServer>,
146 app_status: Res<AppStatus>,
147 render_device: Res<RenderDevice>,
148 render_adapter: Res<RenderAdapter>,
149 mut meshes: ResMut<Assets<Mesh>>,
150 mut materials: ResMut<Assets<StandardMaterial>>,
151) {
152 // Error out if clustered decals (and so light textures) aren't supported on the current platform.
153 if !decal::clustered::clustered_decals_are_usable(&render_device, &render_adapter) {
154 error!("Light textures aren't usable on this platform.");
155 commands.write_message(AppExit::error());
156 }
157
158 spawn_cubes(&mut commands, &mut meshes, &mut materials);
159 spawn_camera(&mut commands);
160 spawn_light(&mut commands, &asset_server);
161 spawn_buttons(&mut commands);
162 spawn_help_text(&mut commands, &app_status);
163 spawn_light_textures(&mut commands, &asset_server, &mut meshes, &mut materials);
164}106fn input(
107 input: Res<ButtonInput<Key>>,
108 mut error: ResMut<RenderError>,
109 mut handler: ResMut<RenderErrorHandler>,
110) {
111 *error = RenderError::None;
112 if input.just_pressed(Key::Character("o".into())) {
113 *error = RenderError::OutOfMemory;
114 }
115 if input.just_pressed(Key::Character("v".into())) {
116 *error = RenderError::Validation;
117 }
118 if input.just_pressed(Key::Character("d".into())) {
119 *error = RenderError::DeviceLost;
120 }
121 if input.just_pressed(Key::Character("l".into())) {
122 *error = RenderError::Loop;
123 }
124
125 if input.just_pressed(Key::Character("1".into())) {
126 *handler = RenderErrorHandler(|_, _, _| RenderErrorPolicy::Ignore);
127 }
128 if input.just_pressed(Key::Character("2".into())) {
129 *handler = RenderErrorHandler(|error, _, _| panic!("Rendering error {error:?}"));
130 }
131 if input.just_pressed(Key::Character("3".into())) {
132 *handler = RenderErrorHandler(|_, main_world, _| {
133 main_world.write_message(AppExit::error());
134 RenderErrorPolicy::StopRendering
135 });
136 }
137 if input.just_pressed(Key::Character("4".into())) {
138 *handler = RenderErrorHandler(|_, _, _| RenderErrorPolicy::StopRendering);
139 }
140 if input.just_pressed(Key::Character("5".into())) {
141 *handler = RenderErrorHandler(|_, _, _| RenderErrorPolicy::Recover(default()));
142 }
143}Sourcepub const fn is_success(&self) -> bool
pub const fn is_success(&self) -> bool
Returns true if self is a AppExit::Success.
Sourcepub const fn is_error(&self) -> bool
pub const fn is_error(&self) -> bool
Returns true if self is a AppExit::Error.
Sourcepub const fn from_code(code: u8) -> AppExit
pub const fn from_code(code: u8) -> AppExit
Creates a AppExit from a code.
When code is 0 a AppExit::Success is constructed otherwise a
AppExit::Error is constructed.