pub enum RunFixedMainLoopSystems {
BeforeFixedMainLoop,
FixedMainLoop,
AfterFixedMainLoop,
}Expand description
Set enum for the systems that want to run inside RunFixedMainLoop,
but before or after the fixed update logic. Systems in this set
will run exactly once per frame, regardless of the number of fixed updates.
They will also run under a variable timestep.
This is useful for handling things that need to run every frame, but also need to be read by the fixed update logic. See the individual variants for examples of what kind of systems should be placed in each.
Note that in contrast to most other Bevy schedules, systems added directly to
RunFixedMainLoop will not be parallelized between each other.
Variants
BeforeFixedMainLoop
Runs before the fixed update logic.
A good example of a system that fits here
is camera movement, which needs to be updated in a variable timestep,
as you want the camera to move with as much precision and updates as
the frame rate allows. A physics system that needs to read the camera
position and orientation, however, should run in the fixed update logic,
as it needs to be deterministic and run at a fixed rate for better stability.
Note that we are not placing the camera movement system in Update, as that
would mean that the physics system already ran at that point.
Example
App::new()
.add_systems(
RunFixedMainLoop,
update_camera_rotation.in_set(RunFixedMainLoopSystems::BeforeFixedMainLoop))
.add_systems(FixedUpdate, update_physics);
FixedMainLoop
Contains the fixed update logic.
Runs FixedMain zero or more times based on delta of
Time<Virtual> and Time::overstep.
Don’t place systems here, use FixedUpdate and friends instead.
Use this system instead to order your systems to run specifically inbetween the fixed update logic and all
other systems that run in RunFixedMainLoopSystems::BeforeFixedMainLoop or RunFixedMainLoopSystems::AfterFixedMainLoop.
Example
App::new()
.add_systems(FixedUpdate, update_physics)
.add_systems(
RunFixedMainLoop,
(
// This system will be called before all interpolation systems
// that third-party plugins might add.
prepare_for_interpolation
.after(RunFixedMainLoopSystems::FixedMainLoop)
.before(RunFixedMainLoopSystems::AfterFixedMainLoop),
)
);
AfterFixedMainLoop
Runs after the fixed update logic.
A good example of a system that fits here is a system that interpolates the transform of an entity between the last and current fixed update. See the fixed timestep example for more details.
Example
App::new()
.add_systems(FixedUpdate, update_physics)
.add_systems(
RunFixedMainLoop,
interpolate_transforms.in_set(RunFixedMainLoopSystems::AfterFixedMainLoop));