pub struct BorderRect {
pub min_inset: Vec2,
pub max_inset: Vec2,
}Expand description
Defines border insets that shrink a rectangle from its minimum and maximum corners.
This struct is used to represent thickness or offsets from the four edges of a rectangle, with values increasing inwards.
Fields
min_inset: Vec2Inset applied to the rectangle’s minimum corner
max_inset: Vec2Inset applied to the rectangle’s maximum corner
Implementations
Sourceimpl BorderRect
impl BorderRect
Sourcepub const ZERO: BorderRect
pub const ZERO: BorderRect
An empty border with zero thickness along each edge
Sourcepub const fn all(inset: f32) -> BorderRect
pub const fn all(inset: f32) -> BorderRect
Creates a border with the same inset along each edge
Examples found in repository?
examples/testbed/2d.rs (line 352)
333 pub fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
334 commands.spawn((Camera2d, DespawnOnExit(super::Scene::SpriteSlicing)));
335
336 let texture = asset_server.load("textures/slice_square_2.png");
337 let font = asset_server.load("fonts/FiraSans-Bold.ttf");
338
339 commands.spawn((
340 Sprite {
341 image: texture.clone(),
342 ..default()
343 },
344 Transform::from_translation(Vec3::new(-150.0, 50.0, 0.0)).with_scale(Vec3::splat(2.0)),
345 DespawnOnExit(super::Scene::SpriteSlicing),
346 ));
347
348 commands.spawn((
349 Sprite {
350 image: texture,
351 image_mode: SpriteImageMode::Sliced(TextureSlicer {
352 border: BorderRect::all(20.0),
353 center_scale_mode: SliceScaleMode::Stretch,
354 ..default()
355 }),
356 custom_size: Some(Vec2::new(200.0, 200.0)),
357 ..default()
358 },
359 Transform::from_translation(Vec3::new(150.0, 50.0, 0.0)),
360 DespawnOnExit(super::Scene::SpriteSlicing),
361 ));
362
363 commands.spawn((
364 Text2d::new("Original"),
365 TextFont {
366 font: FontSource::from(font.clone()),
367 font_size: FontSize::Px(20.0),
368 ..default()
369 },
370 Transform::from_translation(Vec3::new(-150.0, -80.0, 0.0)),
371 DespawnOnExit(super::Scene::SpriteSlicing),
372 ));
373
374 commands.spawn((
375 Text2d::new("Sliced"),
376 TextFont {
377 font: FontSource::from(font.clone()),
378 font_size: FontSize::Px(20.0),
379 ..default()
380 },
381 Transform::from_translation(Vec3::new(150.0, -80.0, 0.0)),
382 DespawnOnExit(super::Scene::SpriteSlicing),
383 ));
384 }More examples
examples/ui/images/ui_texture_slice.rs (line 48)
44fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
45 let image = asset_server.load("textures/fantasy_ui_borders/panel-border-010.png");
46
47 let slicer = TextureSlicer {
48 border: BorderRect::all(22.0),
49 center_scale_mode: SliceScaleMode::Stretch,
50 sides_scale_mode: SliceScaleMode::Stretch,
51 max_corner_scale: 1.0,
52 };
53 // ui camera
54 commands.spawn(Camera2d);
55 commands
56 .spawn(Node {
57 width: percent(100),
58 height: percent(100),
59 align_items: AlignItems::Center,
60 justify_content: JustifyContent::Center,
61 ..default()
62 })
63 .with_children(|parent| {
64 for [w, h] in [[150.0, 150.0], [300.0, 150.0], [150.0, 300.0]] {
65 parent
66 .spawn((
67 Button,
68 ImageNode {
69 image: image.clone(),
70 image_mode: NodeImageMode::Sliced(slicer.clone()),
71 ..default()
72 },
73 Node {
74 width: px(w),
75 height: px(h),
76 // horizontally center child text
77 justify_content: JustifyContent::Center,
78 // vertically center child text
79 align_items: AlignItems::Center,
80 margin: UiRect::all(px(20)),
81 ..default()
82 },
83 ))
84 .with_child((
85 Text::new("Button"),
86 TextFont {
87 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
88 font_size: FontSize::Px(33.0),
89 ..default()
90 },
91 TextColor(Color::srgb(0.9, 0.9, 0.9)),
92 ));
93 }
94 });
95}examples/ui/images/ui_texture_slice_flip_and_tile.rs (line 28)
17fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
18 let image = asset_server
19 .load_builder()
20 .with_settings(|settings: &mut ImageLoaderSettings| {
21 // Need to use nearest filtering to avoid bleeding between the slices with tiling
22 settings.sampler = ImageSampler::nearest();
23 })
24 .load("textures/fantasy_ui_borders/numbered_slices.png");
25
26 let slicer = TextureSlicer {
27 // `numbered_slices.png` is 48 pixels square. `BorderRect::square(16.)` insets the slicing line from each edge by 16 pixels, resulting in nine slices that are each 16 pixels square.
28 border: BorderRect::all(16.),
29 // With `SliceScaleMode::Tile` the side and center slices are tiled to fill the side and center sections of the target.
30 // And with a `stretch_value` of `1.` the tiles will have the same size as the corresponding slices in the source image.
31 center_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
32 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
33 ..default()
34 };
35
36 // ui camera
37 commands.spawn(Camera2d);
38
39 commands
40 .spawn(Node {
41 width: percent(100),
42 height: percent(100),
43 justify_content: JustifyContent::Center,
44 align_content: AlignContent::Center,
45 flex_wrap: FlexWrap::Wrap,
46 column_gap: px(10),
47 row_gap: px(10),
48 ..default()
49 })
50 .with_children(|parent| {
51 for [columns, rows] in [[3, 3], [4, 4], [5, 4], [4, 5], [5, 5]] {
52 for (flip_x, flip_y) in [(false, false), (false, true), (true, false), (true, true)]
53 {
54 parent.spawn((
55 ImageNode {
56 image: image.clone(),
57 flip_x,
58 flip_y,
59 image_mode: NodeImageMode::Sliced(slicer.clone()),
60 ..default()
61 },
62 Node {
63 width: px(16 * columns),
64 height: px(16 * rows),
65 ..default()
66 },
67 ));
68 }
69 }
70 });
71}examples/ui/images/ui_texture_atlas_slice.rs (line 58)
47fn setup(
48 mut commands: Commands,
49 asset_server: Res<AssetServer>,
50 mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
51) {
52 let texture_handle = asset_server.load("textures/fantasy_ui_borders/border_sheet.png");
53 let atlas_layout =
54 TextureAtlasLayout::from_grid(UVec2::new(50, 50), 6, 6, Some(UVec2::splat(2)), None);
55 let atlas_layout_handle = texture_atlases.add(atlas_layout);
56
57 let slicer = TextureSlicer {
58 border: BorderRect::all(24.0),
59 center_scale_mode: SliceScaleMode::Stretch,
60 sides_scale_mode: SliceScaleMode::Stretch,
61 max_corner_scale: 1.0,
62 };
63 // ui camera
64 commands.spawn(Camera2d);
65 commands
66 .spawn(Node {
67 width: percent(100),
68 height: percent(100),
69 align_items: AlignItems::Center,
70 justify_content: JustifyContent::Center,
71 ..default()
72 })
73 .with_children(|parent| {
74 for (idx, [w, h]) in [
75 (0, [150.0, 150.0]),
76 (7, [300.0, 150.0]),
77 (13, [150.0, 300.0]),
78 ] {
79 parent
80 .spawn((
81 Button,
82 ImageNode::from_atlas_image(
83 texture_handle.clone(),
84 TextureAtlas {
85 index: idx,
86 layout: atlas_layout_handle.clone(),
87 },
88 )
89 .with_mode(NodeImageMode::Sliced(slicer.clone())),
90 Node {
91 width: px(w),
92 height: px(h),
93 // horizontally center child text
94 justify_content: JustifyContent::Center,
95 // vertically center child text
96 align_items: AlignItems::Center,
97 margin: px(20).all(),
98 ..default()
99 },
100 ))
101 .with_children(|parent| {
102 parent.spawn((
103 Text::new("Button"),
104 TextFont {
105 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
106 font_size: FontSize::Px(33.0),
107 ..default()
108 },
109 TextColor(Color::srgb(0.9, 0.9, 0.9)),
110 ));
111 });
112 }
113 });
114}examples/2d/sprite_slice.rs (line 41)
12fn spawn_sprites(
13 commands: &mut Commands,
14 texture_handle: Handle<Image>,
15 mut position: Vec3,
16 slice_border: f32,
17 style: TextFont,
18 gap: f32,
19) {
20 let cases = [
21 // Reference sprite
22 (
23 "Original",
24 style.clone(),
25 Vec2::splat(100.0),
26 SpriteImageMode::Auto,
27 ),
28 // Scaled regular sprite
29 (
30 "Stretched",
31 style.clone(),
32 Vec2::new(100.0, 200.0),
33 SpriteImageMode::Auto,
34 ),
35 // Stretched Scaled sliced sprite
36 (
37 "With Slicing",
38 style.clone(),
39 Vec2::new(100.0, 200.0),
40 SpriteImageMode::Sliced(TextureSlicer {
41 border: BorderRect::all(slice_border),
42 center_scale_mode: SliceScaleMode::Stretch,
43 ..default()
44 }),
45 ),
46 // Scaled sliced sprite
47 (
48 "With Tiling",
49 style.clone(),
50 Vec2::new(100.0, 200.0),
51 SpriteImageMode::Sliced(TextureSlicer {
52 border: BorderRect::all(slice_border),
53 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.5 },
54 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
55 ..default()
56 }),
57 ),
58 // Scaled sliced sprite horizontally
59 (
60 "With Tiling",
61 style.clone(),
62 Vec2::new(300.0, 200.0),
63 SpriteImageMode::Sliced(TextureSlicer {
64 border: BorderRect::all(slice_border),
65 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
66 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.3 },
67 ..default()
68 }),
69 ),
70 // Scaled sliced sprite horizontally with max scale
71 (
72 "With Corners Constrained",
73 style,
74 Vec2::new(300.0, 200.0),
75 SpriteImageMode::Sliced(TextureSlicer {
76 border: BorderRect::all(slice_border),
77 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.1 },
78 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
79 max_corner_scale: 0.2,
80 }),
81 ),
82 ];
83
84 for (label, text_style, size, scale_mode) in cases {
85 position.x += 0.5 * size.x;
86 commands.spawn((
87 Sprite {
88 image: texture_handle.clone(),
89 custom_size: Some(size),
90 image_mode: scale_mode,
91 ..default()
92 },
93 Transform::from_translation(position),
94 children![(
95 Text2d::new(label),
96 text_style,
97 TextLayout::justify(Justify::Center),
98 Transform::from_xyz(0., -0.5 * size.y - 10., 0.0),
99 bevy::sprite::Anchor::TOP_CENTER,
100 )],
101 ));
102 position.x += 0.5 * size.x + gap;
103 }
104}examples/testbed/ui.rs (line 1031)
1026 pub fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
1027 commands.spawn((Camera2d, DespawnOnExit(super::Scene::Slice)));
1028 let image = asset_server.load("textures/fantasy_ui_borders/numbered_slices.png");
1029
1030 let slicer = TextureSlicer {
1031 border: BorderRect::all(16.0),
1032 center_scale_mode: SliceScaleMode::Tile { stretch_value: 1.0 },
1033 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 1.0 },
1034 ..default()
1035 };
1036 commands
1037 .spawn((
1038 Node {
1039 width: percent(100),
1040 height: percent(100),
1041 flex_direction: FlexDirection::Column,
1042 justify_content: JustifyContent::SpaceAround,
1043 align_content: AlignContent::Center,
1044 ..default()
1045 },
1046 DespawnOnExit(super::Scene::Slice),
1047 ))
1048 .with_children(|parent| {
1049 for visual_box in [
1050 VisualBox::BorderBox,
1051 VisualBox::PaddingBox,
1052 VisualBox::ContentBox,
1053 ] {
1054 parent
1055 .spawn(Node {
1056 justify_content: JustifyContent::SpaceAround,
1057 ..default()
1058 })
1059 .with_children(|parent| {
1060 for [w, h] in [[200.0, 200.0], [300.0, 200.0], [150., 200.0]] {
1061 parent.spawn((
1062 Button,
1063 ImageNode {
1064 image: image.clone(),
1065 image_mode: NodeImageMode::Sliced(slicer.clone()),
1066 visual_box,
1067 ..default()
1068 },
1069 Node {
1070 width: px(w),
1071 height: px(h),
1072 border: px(20.).all(),
1073 padding: px(20.).all(),
1074 ..default()
1075 },
1076 Outline {
1077 width: px(2.),
1078 ..default()
1079 },
1080 ));
1081 }
1082
1083 parent.spawn((
1084 ImageNode {
1085 image: asset_server
1086 .load("textures/fantasy_ui_borders/panel-border-010.png"),
1087 image_mode: NodeImageMode::Sliced(TextureSlicer {
1088 border: BorderRect::all(22.0),
1089 center_scale_mode: SliceScaleMode::Stretch,
1090 sides_scale_mode: SliceScaleMode::Stretch,
1091 max_corner_scale: 1.0,
1092 }),
1093 visual_box,
1094 ..Default::default()
1095 },
1096 Node {
1097 width: px(200),
1098 height: px(200),
1099 border: px(20.).all(),
1100 padding: px(20.).all(),
1101 ..default()
1102 },
1103 Outline {
1104 color: bevy::color::palettes::css::DARK_CYAN.into(),
1105 width: px(2.),
1106 ..default()
1107 },
1108 BackgroundColor(bevy::color::palettes::css::NAVY.into()),
1109 ));
1110 });
1111 }
1112 });
1113 }Sourcepub const fn axes(horizontal: f32, vertical: f32) -> BorderRect
pub const fn axes(horizontal: f32, vertical: f32) -> BorderRect
Creates a new border with the min.x and max.x insets equal to horizontal, and the min.y and max.y insets equal to vertical.