pub struct Circle {
pub radius: f32,
}Expand description
A circle primitive, representing the set of points some distance from the origin
Fields
radius: f32The radius of the circle
Implementations
Sourceimpl Circle
impl Circle
Sourcepub const fn new(radius: f32) -> Circle
pub const fn new(radius: f32) -> Circle
Create a new Circle from a radius
Examples found in repository?
examples/3d/mirror.rs (line 226)
220fn spawn_ground_plane(
221 commands: &mut Commands,
222 meshes: &mut Assets<Mesh>,
223 standard_materials: &mut Assets<StandardMaterial>,
224) {
225 commands.spawn((
226 Mesh3d(meshes.add(Circle::new(200.0))),
227 MeshMaterial3d(standard_materials.add(Color::from(GREEN))),
228 Transform::from_rotation(Quat::from_rotation_x(-FRAC_PI_2))
229 .with_translation(vec3(-25.0, 0.0, 0.0)),
230 ));
231}More examples
examples/camera/2d_top_down_camera.rs (line 45)
31fn setup_scene(
32 mut commands: Commands,
33 mut meshes: ResMut<Assets<Mesh>>,
34 mut materials: ResMut<Assets<ColorMaterial>>,
35) {
36 // World where we move the player
37 commands.spawn((
38 Mesh2d(meshes.add(Rectangle::new(1000., 700.))),
39 MeshMaterial2d(materials.add(Color::srgb(0.2, 0.2, 0.3))),
40 ));
41
42 // Player
43 commands.spawn((
44 Player,
45 Mesh2d(meshes.add(Circle::new(25.))),
46 MeshMaterial2d(materials.add(Color::srgb(6.25, 9.4, 9.1))), // RGB values exceed 1 to achieve a bright color for the bloom effect
47 Transform::from_xyz(0., 0., 2.),
48 ));
49}examples/picking/dragdrop_picking.rs (line 115)
102fn on_drag_enter(
103 mut event: On<Pointer<DragEnter>>,
104 button: Single<Entity, With<DraggableButton>>,
105 mut commands: Commands,
106 mut meshes: ResMut<Assets<Mesh>>,
107 mut materials: ResMut<Assets<ColorMaterial>>,
108) {
109 if event.dragged == *button {
110 let Some(position) = event.hit.position else {
111 return;
112 };
113 commands.spawn((
114 GhostPreview,
115 Mesh2d(meshes.add(Circle::new(ELEMENT_SIZE))),
116 MeshMaterial2d(materials.add(Color::srgba(1.0, 1.0, 0.6, 0.5))),
117 Transform::from_translation(position + 2. * Vec3::Z),
118 Pickable::IGNORE,
119 ));
120 event.propagate(false);
121 }
122}
123
124fn on_drag_over(
125 mut event: On<Pointer<DragOver>>,
126 button: Single<Entity, With<DraggableButton>>,
127 mut ghost_transform: Single<&mut Transform, With<GhostPreview>>,
128) {
129 if event.dragged == *button {
130 let Some(position) = event.hit.position else {
131 return;
132 };
133 ghost_transform.translation = position;
134 event.propagate(false);
135 }
136}
137
138fn on_drag_drop(
139 mut event: On<Pointer<DragDrop>>,
140 button: Single<Entity, With<DraggableButton>>,
141 mut commands: Commands,
142 ghost: Single<Entity, With<GhostPreview>>,
143 mut meshes: ResMut<Assets<Mesh>>,
144 mut materials: ResMut<Assets<ColorMaterial>>,
145) {
146 if event.dropped == *button {
147 commands.entity(*ghost).despawn();
148 let Some(position) = event.hit.position else {
149 return;
150 };
151 commands.spawn((
152 DroppedElement,
153 Mesh2d(meshes.add(Circle::new(ELEMENT_SIZE))),
154 MeshMaterial2d(materials.add(Color::srgb(1.0, 1.0, 0.6))),
155 Transform::from_translation(position + 2. * Vec3::Z),
156 Pickable::IGNORE,
157 ));
158 event.propagate(false);
159 }
160}examples/app/externally_driven_headless_renderer.rs (line 136)
130fn spawn_test_scene(
131 mut commands: Commands,
132 mut meshes: ResMut<Assets<Mesh>>,
133 mut materials: ResMut<Assets<StandardMaterial>>,
134) {
135 commands.spawn((
136 Mesh3d(meshes.add(Circle::new(4.0))),
137 MeshMaterial3d(materials.add(Color::WHITE)),
138 Transform::from_rotation(Quat::from_rotation_x(-std::f32::consts::FRAC_PI_2)),
139 ));
140 commands.spawn((
141 Mesh3d(meshes.add(Cuboid::new(2.0, 2.0, 2.0))),
142 MeshMaterial3d(materials.add(Color::srgb_u8(124, 144, 255))),
143 Transform::from_xyz(0.0, 1.0, 0.0),
144 ));
145 commands.spawn((
146 PointLight {
147 shadow_maps_enabled: true,
148 ..default()
149 },
150 Transform::from_xyz(4.0, 8.0, 4.0),
151 ));
152}examples/async_tasks/async_channel_pattern.rs (line 138)
131fn setup_env(
132 mut commands: Commands,
133 mut meshes: ResMut<Assets<Mesh>>,
134 mut materials: ResMut<Assets<StandardMaterial>>,
135) {
136 // Spawn a circular ground plane
137 commands.spawn((
138 Mesh3d(meshes.add(Circle::new(1.618 * NUM_CUBES as f32))),
139 MeshMaterial3d(materials.add(Color::WHITE)),
140 Transform::from_rotation(Quat::from_rotation_x(-std::f32::consts::FRAC_PI_2)),
141 ));
142
143 // Spawn a point light with shadows enabled
144 commands.spawn((
145 PointLight {
146 shadow_maps_enabled: true,
147 ..default()
148 },
149 Transform::from_xyz(0.0, LIGHT_RADIUS, 4.0),
150 ));
151
152 // Spawn a camera looking at the origin
153 commands.spawn((
154 Camera3d::default(),
155 Transform::from_xyz(-6.5, 5.5, 12.0).looking_at(Vec3::ZERO, Vec3::Y),
156 ));
157}examples/3d/3d_scene.rs (line 17)
13fn scene() -> impl SceneList {
14 bsn_list! [
15 (
16 #CircularBase
17 Mesh3d(asset_value(Circle::new(4.0)))
18 MeshMaterial3d::<StandardMaterial>(asset_value(Color::WHITE))
19 Transform::from_rotation(Quat::from_rotation_x(-std::f32::consts::FRAC_PI_2))
20 ),
21 (
22 #Cube
23 Mesh3d(asset_value(Cuboid::new(1.0, 1.0, 1.0)))
24 MeshMaterial3d::<StandardMaterial>(asset_value(Color::srgb_u8(124, 144, 255)))
25 Transform::from_xyz(0.0, 0.5, 0.0)
26 ),
27 (
28 PointLight {
29 shadow_maps_enabled: true,
30 }
31 Transform::from_xyz(4.0, 8.0, 4.0)
32 ),
33 (
34 Camera3d
35 template_value(Transform::from_xyz(-2.5, 4.5, 9.0).looking_at(Vec3::ZERO, Vec3::Y))
36 )
37 ]
38}Additional examples can be found in:
- examples/animation/animation_graph.rs
- examples/remote/server.rs
- examples/animation/animation_masks.rs
- examples/camera/custom_projection.rs
- examples/diagnostics/log_diagnostics.rs
- examples/shader_advanced/custom_render_phase.rs
- examples/2d/wireframe_2d.rs
- examples/audio/spatial_audio_2d.rs
- examples/picking/simple_picking.rs
- examples/testbed/2d.rs
- examples/picking/debug_picking.rs
- examples/math/bounding_2d.rs
- examples/2d/bloom_2d.rs
- examples/app/headless_renderer.rs
- examples/app/render_recovery.rs
- examples/stress_tests/many_cameras_lights.rs
- examples/shader_advanced/compute_mesh.rs
- examples/testbed/3d.rs
- examples/gizmos/light_gizmos.rs
- examples/2d/2d_shapes.rs
- examples/3d/3d_shapes.rs
Sourcepub fn closest_point(&self, point: Vec2) -> Vec2
pub fn closest_point(&self, point: Vec2) -> Vec2
Finds the point on the circle that is closest to the given point.
If the point is outside the circle, the returned point will be on the perimeter of the circle. Otherwise, it will be inside the circle and returned as is.