pub struct ComputedNode {
pub size: Vec2,
pub content_size: Vec2,
pub scrollbar_size: Vec2,
pub scroll_position: Vec2,
pub outline_width: f32,
pub outline_offset: f32,
pub unrounded_size: Vec2,
pub border: BorderRect,
pub border_radius: ResolvedBorderRadius,
pub padding: BorderRect,
pub inverse_scale_factor: f32,
}Expand description
Provides the computed size and layout properties of the node.
Fields in this struct are public but should not be modified under most circumstances.
For example, in a scrollbar you may want to derive the handle’s size from the proportion of
scrollable content in-view. You can directly modify ComputedNode after layout to set the
handle size without any delays.
Fields
size: Vec2The size of the node as width and height in physical pixels.
Automatically calculated by ui_layout_system.
content_size: Vec2Size of this node’s content.
Automatically calculated by ui_layout_system.
scrollbar_size: Vec2Space allocated for scrollbars.
Automatically calculated by ui_layout_system.
scroll_position: Vec2Resolved offset of scrolled content
Automatically calculated by ui_layout_system.
outline_width: f32The width of this node’s outline.
If this value is Auto, negative or 0. then no outline will be rendered.
Outline updates bypass change detection.
Automatically calculated by ui_layout_system.
outline_offset: f32The amount of space between the outline and the edge of the node. Outline updates bypass change detection.
Automatically calculated by ui_layout_system.
unrounded_size: Vec2The unrounded size of the node as width and height in physical pixels.
Automatically calculated by ui_layout_system.
border: BorderRectResolved border values in physical pixels. Border updates bypass change detection.
Automatically calculated by ui_layout_system.
border_radius: ResolvedBorderRadiusResolved border radius values in physical pixels. Border radius updates bypass change detection.
Automatically calculated by ui_layout_system.
padding: BorderRectResolved padding values in physical pixels. Padding updates bypass change detection.
Automatically calculated by ui_layout_system.
inverse_scale_factor: f32Inverse scale factor for this Node. Multiply physical coordinates by the inverse scale factor to give logical coordinates.
Automatically calculated by ui_layout_system.
Implementations
Sourceimpl ComputedNode
impl ComputedNode
Sourcepub const fn size(&self) -> Vec2
pub const fn size(&self) -> Vec2
The calculated node size as width and height in physical pixels.
Automatically calculated by ui_layout_system.
Examples found in repository?
439pub fn update_scroll_position(
440 mut mouse_wheel_reader: MessageReader<MouseWheel>,
441 hover_map: Res<HoverMap>,
442 mut scrolled_node_query: Query<(&mut ScrollPosition, &ComputedNode), Without<Scrollbar>>,
443 keyboard_input: Res<ButtonInput<KeyCode>>,
444) {
445 for mouse_wheel in mouse_wheel_reader.read() {
446 let (mut dx, mut dy) = match mouse_wheel.unit {
447 MouseScrollUnit::Line => (mouse_wheel.x * 20., mouse_wheel.y * 20.),
448 MouseScrollUnit::Pixel => (mouse_wheel.x, mouse_wheel.y),
449 };
450
451 if keyboard_input.pressed(KeyCode::ShiftLeft) || keyboard_input.pressed(KeyCode::ShiftRight)
452 {
453 std::mem::swap(&mut dx, &mut dy);
454 }
455
456 for (_pointer, pointer_map) in hover_map.iter() {
457 for (entity, _hit) in pointer_map.iter() {
458 if let Ok((mut scroll_position, scroll_content)) =
459 scrolled_node_query.get_mut(*entity)
460 {
461 let visible_size = scroll_content.size();
462 let content_size = scroll_content.content_size();
463
464 let range = (content_size.y - visible_size.y).max(0.)
465 * scroll_content.inverse_scale_factor;
466
467 scroll_position.x -= dx;
468 scroll_position.y = (scroll_position.y - dy).clamp(0., range);
469 }
470 }
471 }
472 }
473}More examples
61fn on_scroll_handler(
62 mut scroll: On<Scroll>,
63 mut query: Query<(&mut ScrollPosition, &Node, &ComputedNode)>,
64) {
65 let Ok((mut scroll_position, node, computed)) = query.get_mut(scroll.entity) else {
66 return;
67 };
68
69 let max_offset = (computed.content_size() - computed.size()) * computed.inverse_scale_factor();
70
71 let delta = &mut scroll.delta;
72 if node.overflow.x == OverflowAxis::Scroll && delta.x != 0. {
73 // Is this node already scrolled all the way in the direction of the scroll?
74 let max = if delta.x > 0. {
75 scroll_position.x >= max_offset.x
76 } else {
77 scroll_position.x <= 0.
78 };
79
80 if !max {
81 scroll_position.x += delta.x;
82 // Consume the X portion of the scroll delta.
83 delta.x = 0.;
84 }
85 }
86
87 if node.overflow.y == OverflowAxis::Scroll && delta.y != 0. {
88 // Is this node already scrolled all the way in the direction of the scroll?
89 let max = if delta.y > 0. {
90 scroll_position.y >= max_offset.y
91 } else {
92 scroll_position.y <= 0.
93 };
94
95 if !max {
96 scroll_position.y += delta.y;
97 // Consume the Y portion of the scroll delta.
98 delta.y = 0.;
99 }
100 }
101
102 // Stop propagating when the delta is fully consumed.
103 if *delta == Vec2::ZERO {
104 scroll.propagate(false);
105 }
106}Sourcepub const fn content_size(&self) -> Vec2
pub const fn content_size(&self) -> Vec2
The calculated node content size as width and height in physical pixels.
Automatically calculated by ui_layout_system.
Examples found in repository?
439pub fn update_scroll_position(
440 mut mouse_wheel_reader: MessageReader<MouseWheel>,
441 hover_map: Res<HoverMap>,
442 mut scrolled_node_query: Query<(&mut ScrollPosition, &ComputedNode), Without<Scrollbar>>,
443 keyboard_input: Res<ButtonInput<KeyCode>>,
444) {
445 for mouse_wheel in mouse_wheel_reader.read() {
446 let (mut dx, mut dy) = match mouse_wheel.unit {
447 MouseScrollUnit::Line => (mouse_wheel.x * 20., mouse_wheel.y * 20.),
448 MouseScrollUnit::Pixel => (mouse_wheel.x, mouse_wheel.y),
449 };
450
451 if keyboard_input.pressed(KeyCode::ShiftLeft) || keyboard_input.pressed(KeyCode::ShiftRight)
452 {
453 std::mem::swap(&mut dx, &mut dy);
454 }
455
456 for (_pointer, pointer_map) in hover_map.iter() {
457 for (entity, _hit) in pointer_map.iter() {
458 if let Ok((mut scroll_position, scroll_content)) =
459 scrolled_node_query.get_mut(*entity)
460 {
461 let visible_size = scroll_content.size();
462 let content_size = scroll_content.content_size();
463
464 let range = (content_size.y - visible_size.y).max(0.)
465 * scroll_content.inverse_scale_factor;
466
467 scroll_position.x -= dx;
468 scroll_position.y = (scroll_position.y - dy).clamp(0., range);
469 }
470 }
471 }
472 }
473}More examples
61fn on_scroll_handler(
62 mut scroll: On<Scroll>,
63 mut query: Query<(&mut ScrollPosition, &Node, &ComputedNode)>,
64) {
65 let Ok((mut scroll_position, node, computed)) = query.get_mut(scroll.entity) else {
66 return;
67 };
68
69 let max_offset = (computed.content_size() - computed.size()) * computed.inverse_scale_factor();
70
71 let delta = &mut scroll.delta;
72 if node.overflow.x == OverflowAxis::Scroll && delta.x != 0. {
73 // Is this node already scrolled all the way in the direction of the scroll?
74 let max = if delta.x > 0. {
75 scroll_position.x >= max_offset.x
76 } else {
77 scroll_position.x <= 0.
78 };
79
80 if !max {
81 scroll_position.x += delta.x;
82 // Consume the X portion of the scroll delta.
83 delta.x = 0.;
84 }
85 }
86
87 if node.overflow.y == OverflowAxis::Scroll && delta.y != 0. {
88 // Is this node already scrolled all the way in the direction of the scroll?
89 let max = if delta.y > 0. {
90 scroll_position.y >= max_offset.y
91 } else {
92 scroll_position.y <= 0.
93 };
94
95 if !max {
96 scroll_position.y += delta.y;
97 // Consume the Y portion of the scroll delta.
98 delta.y = 0.;
99 }
100 }
101
102 // Stop propagating when the delta is fully consumed.
103 if *delta == Vec2::ZERO {
104 scroll.propagate(false);
105 }
106}Sourcepub const fn is_empty(&self) -> bool
pub const fn is_empty(&self) -> bool
Check if the node is empty. A node is considered empty if it has a zero or negative extent along either of its axes.
Sourcepub const fn unrounded_size(&self) -> Vec2
pub const fn unrounded_size(&self) -> Vec2
The calculated node size as width and height in physical pixels before rounding.
Automatically calculated by ui_layout_system.
Sourcepub const fn outline_width(&self) -> f32
pub const fn outline_width(&self) -> f32
Returns the thickness of the UI node’s outline in physical pixels.
If this value is negative or 0. then no outline will be rendered.
Automatically calculated by ui_layout_system.
Sourcepub const fn outline_offset(&self) -> f32
pub const fn outline_offset(&self) -> f32
Returns the amount of space between the outline and the edge of the node in physical pixels.
Automatically calculated by ui_layout_system.
Sourcepub const fn outlined_node_size(&self) -> Vec2
pub const fn outlined_node_size(&self) -> Vec2
Returns the size of the node when including its outline.
Automatically calculated by ui_layout_system.
Sourcepub const fn outline_radius(&self) -> ResolvedBorderRadius
pub const fn outline_radius(&self) -> ResolvedBorderRadius
Returns the border radius for each corner of the outline An outline’s border radius is derived from the node’s border-radius so that the outline wraps the border equally at all points.
Automatically calculated by ui_layout_system.
Sourcepub const fn border(&self) -> BorderRect
pub const fn border(&self) -> BorderRect
Returns the thickness of the node’s border on each edge in physical pixels.
Automatically calculated by ui_layout_system.
Sourcepub const fn border_radius(&self) -> ResolvedBorderRadius
pub const fn border_radius(&self) -> ResolvedBorderRadius
Returns the border radius for each of the node’s corners in physical pixels.
Automatically calculated by ui_layout_system.
Sourcepub fn inner_radius(&self) -> ResolvedBorderRadius
pub fn inner_radius(&self) -> ResolvedBorderRadius
Returns the inner border radius for each of the node’s corners in physical pixels.
Sourcepub const fn padding(&self) -> BorderRect
pub const fn padding(&self) -> BorderRect
Returns the thickness of the node’s padding on each edge in physical pixels.
Automatically calculated by ui_layout_system.
Sourcepub fn content_inset(&self) -> BorderRect
pub fn content_inset(&self) -> BorderRect
Returns the combined inset on each edge including both padding and border thickness in physical pixels.
Sourcepub const fn inverse_scale_factor(&self) -> f32
pub const fn inverse_scale_factor(&self) -> f32
Returns the inverse of the scale factor for this node. To convert from physical coordinates to logical coordinates multiply by this value.
Examples found in repository?
61fn on_scroll_handler(
62 mut scroll: On<Scroll>,
63 mut query: Query<(&mut ScrollPosition, &Node, &ComputedNode)>,
64) {
65 let Ok((mut scroll_position, node, computed)) = query.get_mut(scroll.entity) else {
66 return;
67 };
68
69 let max_offset = (computed.content_size() - computed.size()) * computed.inverse_scale_factor();
70
71 let delta = &mut scroll.delta;
72 if node.overflow.x == OverflowAxis::Scroll && delta.x != 0. {
73 // Is this node already scrolled all the way in the direction of the scroll?
74 let max = if delta.x > 0. {
75 scroll_position.x >= max_offset.x
76 } else {
77 scroll_position.x <= 0.
78 };
79
80 if !max {
81 scroll_position.x += delta.x;
82 // Consume the X portion of the scroll delta.
83 delta.x = 0.;
84 }
85 }
86
87 if node.overflow.y == OverflowAxis::Scroll && delta.y != 0. {
88 // Is this node already scrolled all the way in the direction of the scroll?
89 let max = if delta.y > 0. {
90 scroll_position.y >= max_offset.y
91 } else {
92 scroll_position.y <= 0.
93 };
94
95 if !max {
96 scroll_position.y += delta.y;
97 // Consume the Y portion of the scroll delta.
98 delta.y = 0.;
99 }
100 }
101
102 // Stop propagating when the delta is fully consumed.
103 if *delta == Vec2::ZERO {
104 scroll.propagate(false);
105 }
106}pub fn contains_point(&self, transform: UiGlobalTransform, point: Vec2) -> bool
Sourcepub fn normalize_point(
&self,
transform: UiGlobalTransform,
point: Vec2,
) -> Option<Vec2>
pub fn normalize_point( &self, transform: UiGlobalTransform, point: Vec2, ) -> Option<Vec2>
Transform a point to normalized node space with the center of the node at the origin and the corners at [+/-0.5, +/-0.5]
Sourcepub fn resolve_clip_rect(
&self,
overflow: Overflow,
overflow_clip_margin: OverflowClipMargin,
) -> Rect
pub fn resolve_clip_rect( &self, overflow: Overflow, overflow_clip_margin: OverflowClipMargin, ) -> Rect
Resolve the node’s clipping rect in local space
Sourcepub fn border_box(&self) -> Rect
pub fn border_box(&self) -> Rect
Returns the node’s border-box in object-centered physical coordinates. This is the full rectangle enclosing the node.
Sourcepub fn padding_box(&self) -> Rect
pub fn padding_box(&self) -> Rect
Returns the node’s padding-box in object-centered physical coordinates. This is the region inside the border containing the node’s padding and content areas.
Sourcepub fn content_box(&self) -> Rect
pub fn content_box(&self) -> Rect
Returns the node’s content-box in object-centered physical coordinates. This is the innermost region of the node, where its content is placed.
Sourceimpl ComputedNode
impl ComputedNode
pub const DEFAULT: ComputedNode