pub struct EnvironmentMapLight {
pub diffuse_map: Handle<Image>,
pub specular_map: Handle<Image>,
pub intensity: f32,
pub rotation: Quat,
pub affects_lightmapped_mesh_diffuse: bool,
}Expand description
A pair of cubemap textures that represent the surroundings of a specific area in space.
See bevy_pbr::environment_map for detailed information.
Fields
diffuse_map: Handle<Image>The blurry image that represents diffuse radiance surrounding a region.
specular_map: Handle<Image>The typically-sharper, mipmapped image that represents specular radiance surrounding a region.
intensity: f32Scale factor applied to the diffuse and specular light generated by this component.
After applying this multiplier, the resulting values should be in units of cd/m^2.
See also https://google.github.io/filament/Filament.html#lighting/imagebasedlights/iblunit.
rotation: QuatWorld space rotation applied to the environment light cubemaps. This is useful for users who require a different axis, such as the Z-axis, to serve as the vertical axis.
affects_lightmapped_mesh_diffuse: boolWhether the light from this environment map contributes diffuse lighting to meshes with lightmaps.
Set this to false if your lightmap baking tool bakes the diffuse light from this environment light into the lightmaps in order to avoid counting the radiance from this environment map twice.
By default, this is set to true.