pub struct Extrusion<T>where
T: Primitive2d,{
pub base_shape: T,
pub half_depth: f32,
}Expand description
A 3D shape representing an extruded 2D base_shape.
Extruding a shape effectively “thickens” a 2D shapes, creating a shape with the same cross-section over the entire depth.
The resulting volumes are prisms. For example, a triangle becomes a triangular prism, while a circle becomes a cylinder.
Fields
base_shape: TThe base shape of the extrusion
half_depth: f32Half of the depth of the extrusion
Implementations