pub struct In<T>(pub T);Expand description
A SystemInput type which denotes that a System receives
an input value of type T from its caller.
Systems may take an optional input which they require to be passed to them when they
are being run. For FunctionSystems the input may be marked
with this In type, but only the first param of a function may be tagged as an input. This also
means a system can only have one or zero input parameters.
See SystemInput to learn more about system inputs in general.
Examples
Here is a simple example of a system that takes a usize and returns the square of it.
fn square(In(input): In<usize>) -> usize {
input * input
}
let mut world = World::new();
let mut square_system = IntoSystem::into_system(square);
square_system.initialize(&mut world);
assert_eq!(square_system.run(12, &mut world).unwrap(), 144);Tuple Fields
0: TTrait Implementations
Sourceimpl<T> SystemInput for In<T>where
T: 'static,
impl<T> SystemInput for In<T>where
T: 'static,
Sourcetype Param<'i> = In<T>
type Param<'i> = In<T>
The wrapper input type that is defined as the first argument to
FunctionSystems.Sourcetype Inner<'i> = T
type Inner<'i> = T
The inner input type that is passed to functions that run systems,
such as
System::run.Sourcefn wrap(
this: <In<T> as SystemInput>::Inner<'_>,
) -> <In<T> as SystemInput>::Param<'_>
fn wrap( this: <In<T> as SystemInput>::Inner<'_>, ) -> <In<T> as SystemInput>::Param<'_>
Converts a
SystemInput::Inner into a SystemInput::Param.Auto Trait Implementations
impl<T> Freeze for In<T>where
T: Freeze,
impl<T> RefUnwindSafe for In<T>where
T: RefUnwindSafe,
impl<T> Send for In<T>where
T: Send,
impl<T> Sync for In<T>where
T: Sync,
impl<T> Unpin for In<T>where
T: Unpin,
impl<T> UnsafeUnpin for In<T>where
T: UnsafeUnpin,
impl<T> UnwindSafe for In<T>where
T: UnwindSafe,