pub struct Leave {
pub hit: HitData,
pub was_in_bounds: bool,
}Expand description
Fires when a pointer crosses out of the bounds of a target entity.
Unlike Out, this event bubbles up through a subset of the
target entity’s ancestors
(traversed via the ChildOf relationship).
Event Propagation
An ancestor of a target entity will receive a Leave event
when the ancestor does not have a direct relation to any entity hovered by the
pointer in the current frame. For example, for a given pointer:
If the previously hovered entity C has the following entity ancestry: A -> B -> C
And the currently hovered entity E has the following entity ancestry: A -> D -> E
Leave events would be sent for both C and its direct ancestor B.
A Leave event would not be sent for A because it is a shared ancestor of both C and E.
Note: A Leave event may be fired for an ancestor even if the pointer does not leave
the ancestor’s bounds. More concretely, if a child’s bounds extend beyond the parent’s
and the pointer leaves from within those extended bounds,
two Leave events are still emitted for both the child and the parent.
This matches the triggering behavior of mouseleave events on the web.
To find out whether the pointer was within the target entity’s bounds
right before leaving, check the value of was_in_bounds.
Refer to pointer_events for more information on how these events are triggered.
Fields
hit: HitDataInformation about the latest prior picking intersection.
was_in_bounds: boolWhether this pointer directly exited out of the target entity’s bounds at the time of the event. This may be false if this entity’s child’s bounds extended beyond the entity and the pointer exited out of the child’s bounds only.