MeshMaterial2d

Struct MeshMaterial2d 

pub struct MeshMaterial2d<M>(pub Handle<M>)
where
    M: Material2d;
A material used for rendering a Mesh2d.

See Material2d for general information about 2D materials and how to implement your own materials.

Example

// Spawn an entity with a mesh using `ColorMaterial`.
fn setup(
    mut commands: Commands,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<ColorMaterial>>,
) {
    commands.spawn((
        Mesh2d(meshes.add(Circle::new(50.0))),
        MeshMaterial2d(materials.add(ColorMaterial::from_color(RED))),
    ));
}

Tuple Fields

0: Handle<M>

Methods from Deref<Target = Handle<M>>

pub fn id(&self) -> AssetId<A>

Returns the AssetId of this Asset.

Examples found in repository?
examples/shader_advanced/manual_material.rs (line 125)
124    fn as_asset_id(&self) -> AssetId<Self::Asset> {
125        self.0.id()
126    }
127}
128
129#[derive(Asset, TypePath, AsBindGroup, Debug, Clone)]
130struct ImageMaterial {
131    image: Handle<Image>,
132}
133
134impl ErasedRenderAsset for ImageMaterial {
135    type SourceAsset = ImageMaterial;
136    type ErasedAsset = PreparedMaterial;
137    type Param = (
138        SRes<DrawFunctions<Opaque3d>>,
139        SRes<ImageMaterialBindGroupLayout>,
140        SRes<AssetServer>,
141        SResMut<MaterialBindGroupAllocators>,
142        SResMut<RenderMaterialBindings>,
143        SRes<RenderAssets<GpuImage>>,
144        SRes<ImageMaterialBindGroupSampler>,
145    );
146
147    fn prepare_asset(
148        source_asset: Self::SourceAsset,
149        asset_id: AssetId<Self::SourceAsset>,
150        (
151            opaque_draw_functions,
152            material_layout,
153            asset_server,
154            bind_group_allocators,
155            render_material_bindings,
156            gpu_images,
157            image_material_sampler,
158        ): &mut SystemParamItem<Self::Param>,
159    ) -> std::result::Result<Self::ErasedAsset, PrepareAssetError<Self::SourceAsset>> {
160        let material_layout = material_layout.0.clone();
161        let draw_function_id = opaque_draw_functions.read().id::<DrawMaterial>();
162        let bind_group_allocator = bind_group_allocators
163            .get_mut(&TypeId::of::<ImageMaterial>())
164            .unwrap();
165        let Some(image) = gpu_images.get(&source_asset.image) else {
166            return Err(PrepareAssetError::RetryNextUpdate(source_asset));
167        };
168        let mut unprepared = BindGroupBuilder {
169            binding_resources: UnpreparedBindingResources(vec![
170                (
171                    0,
172                    UnpreparedBindingResource::TextureView(
173                        TextureViewDimension::D2,
174                        image.texture_view.clone(),
175                    ),
176                ),
177                (
178                    1,
179                    UnpreparedBindingResource::Sampler(
180                        SamplerBindingType::NonFiltering,
181                        image_material_sampler.0.clone(),
182                    ),
183                ),
184            ]),
185            data_buffer: vec![],
186        };
187        let binding = match render_material_bindings.entry(asset_id.into()) {
188            Entry::Occupied(mut occupied_entry) => {
189                bind_group_allocator.free(*occupied_entry.get());
190                let new_binding =
191                    bind_group_allocator.allocate_unprepared(&mut unprepared, &material_layout);
192                *occupied_entry.get_mut() = new_binding;
193                new_binding
194            }
195            Entry::Vacant(vacant_entry) => *vacant_entry.insert(
196                bind_group_allocator.allocate_unprepared(&mut unprepared, &material_layout),
197            ),
198        };
199
200        let mut properties = MaterialProperties {
201            material_layout: Some(material_layout),
202            mesh_pipeline_key_bits: ErasedMeshPipelineKey::new(MeshPipelineKey::empty()),
203            base_specialize: Some(base_specialize),
204            ..Default::default()
205        };
206        properties.add_draw_function(MainPassOpaqueDrawFunction, draw_function_id);
207        properties.add_shader(MaterialFragmentShader, asset_server.load(SHADER_ASSET_PATH));
208
209        Ok(PreparedMaterial {
210            binding,
211            properties: Arc::new(properties),
212        })
213    }
214}
215
216/// set up a simple 3D scene
217fn setup(
218    mut commands: Commands,
219    mut meshes: ResMut<Assets<Mesh>>,
220    mut materials: ResMut<Assets<ImageMaterial>>,
221    asset_server: Res<AssetServer>,
222) {
223    // cube
224    commands.spawn((
225        Mesh3d(meshes.add(Cuboid::new(2.0, 2.0, 2.0))),
226        ImageMaterial3d(materials.add(ImageMaterial {
227            image: asset_server.load("branding/icon.png"),
228        })),
229        Transform::from_xyz(0.0, 0.5, 0.0),
230    ));
231    // light
232    commands.spawn((
233        PointLight {
234            shadow_maps_enabled: true,
235            ..default()
236        },
237        Transform::from_xyz(4.0, 8.0, 4.0),
238    ));
239    // camera
240    commands.spawn((
241        Camera3d::default(),
242        Transform::from_xyz(-2.5, 4.5, 9.0).looking_at(Vec3::ZERO, Vec3::Y),
243    ));
244}
245
246fn extract_image_materials(
247    mut material_instances: ResMut<RenderMaterialInstances>,
248    changed_meshes_query: Extract<
249        Query<
250            (Entity, &ViewVisibility, &ImageMaterial3d),
251            Or<(Changed<ViewVisibility>, Changed<ImageMaterial3d>)>,
252        >,
253    >,
254) {
255    let last_change_tick = material_instances.current_change_tick;
256
257    for (entity, view_visibility, material) in &changed_meshes_query {
258        if view_visibility.get() {
259            material_instances.instances.insert(
260                entity.into(),
261                RenderMaterialInstance {
262                    asset_id: material.0.id().untyped(),
263                    last_change_tick,
264                },
265            );
266        } else {
267            material_instances
268                .instances
269                .remove(&MainEntity::from(entity));
270        }
271    }
272}
More examples
Hide additional examples
examples/2d/dynamic_mip_generation.rs (line 452)
446fn extract_mipmap_source_image(
447    mipmap_source_image: Extract<Res<MipmapSourceImage>>,
448    app_status: Extract<Res<AppStatus>>,
449    mut mip_generation_jobs: ResMut<MipGenerationJobs>,
450) {
451    if app_status.enable_mip_generation == EnableMipGeneration::On {
452        mip_generation_jobs.add(MIP_GENERATION_PHASE_ID, mipmap_source_image.id());
453    }
454}
examples/shader_advanced/compute_mesh.rs (line 194)
174fn prepare_chunks(
175    meshes_to_generate: Query<&GenerateMesh>,
176    mut chunks: ResMut<ChunksToProcess>,
177    pipeline_cache: Res<PipelineCache>,
178    pipeline: Res<ComputePipeline>,
179    mut processed: Local<HashSet<AssetId<Mesh>>>,
180) {
181    // If the pipeline isn't ready, then meshes
182    // won't be processed. So we want to wait until
183    // the pipeline is ready before considering any mesh processed.
184    if pipeline_cache
185        .get_compute_pipeline(pipeline.pipeline)
186        .is_some()
187    {
188        // get the AssetId for each Handle<Mesh>
189        // which we'll use later to get the relevant buffers
190        // from the mesh_allocator
191        let chunk_data: Vec<AssetId<Mesh>> = meshes_to_generate
192            .iter()
193            .filter_map(|gmesh| {
194                let id = gmesh.0.id();
195                processed.contains(&id).not().then_some(id)
196            })
197            .collect();
198
199        // Cache any meshes we're going to process this frame
200        for id in &chunk_data {
201            processed.insert(*id);
202        }
203
204        chunks.0 = chunk_data;
205    }
206}
examples/3d/tonemapping.rs (line 248)
227fn resize_image(
228    image_mesh: Query<(&MeshMaterial3d<StandardMaterial>, &Mesh3d), With<HDRViewer>>,
229    materials: Res<Assets<StandardMaterial>>,
230    mut meshes: ResMut<Assets<Mesh>>,
231    images: Res<Assets<Image>>,
232    mut image_event_reader: MessageReader<AssetEvent<Image>>,
233) {
234    for event in image_event_reader.read() {
235        let (AssetEvent::Added { id } | AssetEvent::Modified { id }) = event else {
236            continue;
237        };
238
239        for (mat_h, mesh_h) in &image_mesh {
240            let Some(mat) = materials.get(mat_h) else {
241                continue;
242            };
243
244            let Some(ref base_color_texture) = mat.base_color_texture else {
245                continue;
246            };
247
248            if *id != base_color_texture.id() {
249                continue;
250            };
251
252            let Some(image_changed) = images.get(*id) else {
253                continue;
254            };
255
256            let size = image_changed.size_f32().normalize_or_zero() * 1.4;
257            // Resize Mesh
258            let quad = Mesh::from(Rectangle::from_size(size));
259            meshes.insert(mesh_h, quad).unwrap();
260        }
261    }
262}
examples/3d/mirror.rs (line 409)
388fn handle_window_resize_messages(
389    windows_query: Query<&Window>,
390    mut mirror_cameras_query: Query<&mut RenderTarget, With<MirrorCamera>>,
391    mut images: ResMut<Assets<Image>>,
392    mut mirror_image: ResMut<MirrorImage>,
393    mut screen_space_texture_materials: ResMut<
394        Assets<ExtendedMaterial<StandardMaterial, ScreenSpaceTextureExtension>>,
395    >,
396    mut resize_messages: MessageReader<WindowResized>,
397) {
398    // We run at most once, regardless of the number of window resize messages
399    // there were this frame.
400    let Some(resize_message) = resize_messages.read().next() else {
401        return;
402    };
403    let Ok(window) = windows_query.get(resize_message.window) else {
404        return;
405    };
406
407    let window_size = uvec2(window.physical_width(), window.physical_height());
408    let image = create_mirror_texture_image(&mut images, window_size);
409    images.remove(mirror_image.0.id());
410
411    mirror_image.0 = image.clone();
412
413    for mut target in mirror_cameras_query.iter_mut() {
414        *target = image.clone().into();
415    }
416
417    for (_, material) in screen_space_texture_materials.iter_mut() {
418        material.base.emissive_texture = Some(image.clone());
419    }
420}
examples/2d/mesh2d_manual.rs (line 551)
517pub fn extract_colored_mesh2d(
518    mut commands: Commands,
519    mut previous_len: Local<usize>,
520    // When extracting, you must use `Extract` to mark the `SystemParam`s
521    // which should be taken from the main world.
522    query: Extract<
523        Query<
524            (
525                Entity,
526                RenderEntity,
527                &ViewVisibility,
528                &GlobalTransform,
529                &Mesh2d,
530            ),
531            With<ColoredMesh2d>,
532        >,
533    >,
534    mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
535) {
536    let mut values = Vec::with_capacity(*previous_len);
537    for (entity, render_entity, view_visibility, transform, handle) in &query {
538        if !view_visibility.get() {
539            continue;
540        }
541
542        let transforms = Mesh2dTransforms {
543            world_from_local: transform.affine().into(),
544            flags: MeshFlags::empty().bits(),
545        };
546
547        values.push((render_entity, ColoredMesh2d));
548        render_mesh_instances.insert(
549            entity.into(),
550            RenderMesh2dInstance {
551                mesh_asset_id: handle.0.id(),
552                transforms,
553                // This is unused here.
554                material_bindings_index: MaterialBindingId::default(),
555                automatic_batching: false,
556                tag: 0,
557            },
558        );
559    }
560    *previous_len = values.len();
561    commands.try_insert_batch(values);
562}
pub fn path(&self) -> Option<&AssetPath<'static>>

Returns the path if this is (1) a strong handle and (2) the asset has a path

pub fn is_uuid(&self) -> bool

Returns true if this is a UUID handle.

pub fn is_strong(&self) -> bool

Returns true if this is a strong handle.