MessageReader

Struct MessageReader 

pub struct MessageReader<'w, 's, M>
where M: Message,
{ /* private fields */ }
Reads Messages of type T in order and tracks which messages have already been read.

Use PopulatedMessageReader<T> to skip the system if there are no messages.

Usage


#[derive(Message)]
pub struct MyMessage(String); // Custom message type.
fn my_system(mut reader: MessageReader<MyMessage>) {
    for msg in reader.read() {
        println!("{}", msg.0)
    }
}

Concurrency

Unlike MessageWriter<T>, systems with MessageReader<T> param can be executed concurrently (but not concurrently with MessageWriter<T> or MessageMutator<T> systems for the same message type).

Implementations

Source

impl<'w, 's, M> MessageReader<'w, 's, M>
where M: Message,

pub fn read(&mut self) -> MessageIterator<'_, M>

Iterates over the messages this MessageReader has not seen yet. This updates the MessageReader’s message counter, which means subsequent message reads will not include messages that happened before now.

Examples found in repository?
examples/input/touch_input_events.rs (line 13)
12fn touch_event_system(mut touch_inputs: MessageReader<TouchInput>) {
13    for touch_input in touch_inputs.read() {
14        info!("{:?}", touch_input);
15    }
16}
More examples
Hide additional examples
examples/input/keyboard_input_events.rs (line 14)
13fn print_keyboard_event_system(mut keyboard_inputs: MessageReader<KeyboardInput>) {
14    for keyboard_input in keyboard_inputs.read() {
15        info!("{:?}", keyboard_input);
16    }
17}
examples/app/drag_and_drop.rs (line 13)
12fn file_drag_and_drop_system(mut drag_and_drop_reader: MessageReader<FileDragAndDrop>) {
13    for drag_and_drop in drag_and_drop_reader.read() {
14        info!("{:?}", drag_and_drop);
15    }
16}
examples/3d/clustered_decals.rs (line 380)
376fn handle_selection_change(
377    mut events: MessageReader<WidgetClickEvent<Selection>>,
378    mut app_status: ResMut<AppStatus>,
379) {
380    for event in events.read() {
381        app_status.selection = **event;
382    }
383}
examples/3d/light_textures.rs (line 441)
437fn handle_selection_change(
438    mut events: MessageReader<WidgetClickEvent<Selection>>,
439    mut app_status: ResMut<AppStatus>,
440) {
441    for event in events.read() {
442        app_status.selection = **event;
443    }
444}
445
446fn toggle_visibility(
447    mut events: MessageReader<WidgetClickEvent<Visibility>>,
448    app_status: Res<AppStatus>,
449    mut visibility: Query<(&mut Visibility, &Selection)>,
450) {
451    if let Some(vis) = events.read().last() {
452        for (mut visibility, selection) in visibility.iter_mut() {
453            if selection == &app_status.selection {
454                *visibility = **vis;
455            }
456        }
457    }
458}
examples/math/cubic_splines.rs (line 299)
295fn handle_mouse_move(
296    mut cursor_moved_reader: MessageReader<CursorMoved>,
297    mut mouse_position: ResMut<MousePosition>,
298) {
299    if let Some(cursor_moved) = cursor_moved_reader.read().last() {
300        mouse_position.0 = Some(cursor_moved.position);
301    }
302}
303
304/// This system handles updating the [`MouseEditMove`] resource, orchestrating the logical part
305/// of the click-and-drag motion which actually creates new control points.
306fn handle_mouse_press(
307    mut mouse_button_input_reader: MessageReader<MouseButtonInput>,
308    mouse_position: Res<MousePosition>,
309    mut edit_move: ResMut<MouseEditMove>,
310    mut control_points: ResMut<ControlPoints>,
311    camera: Single<(&Camera, &GlobalTransform)>,
312) {
313    let Some(mouse_pos) = mouse_position.0 else {
314        return;
315    };
316
317    // Handle click and drag behavior
318    for mouse_button_input in mouse_button_input_reader.read() {
319        if mouse_button_input.button != MouseButton::Left {
320            continue;
321        }
322
323        match mouse_button_input.state {
324            ButtonState::Pressed => {
325                if edit_move.start.is_some() {
326                    // If the edit move already has a start, press event should do nothing.
327                    continue;
328                }
329                // This press represents the start of the edit move.
330                edit_move.start = Some(mouse_pos);
331            }
332
333            ButtonState::Released => {
334                // Release is only meaningful if we started an edit move.
335                let Some(start) = edit_move.start else {
336                    continue;
337                };
338
339                let (camera, camera_transform) = *camera;
340
341                // Convert the starting point and end point (current mouse pos) into world coords:
342                let Ok(point) = camera.viewport_to_world_2d(camera_transform, start) else {
343                    continue;
344                };
345                let Ok(end_point) = camera.viewport_to_world_2d(camera_transform, mouse_pos) else {
346                    continue;
347                };
348                let tangent = end_point - point;
349
350                // The start of the click-and-drag motion represents the point to add,
351                // while the difference with the current position represents the tangent.
352                control_points.points_and_tangents.push((point, tangent));
353
354                // Reset the edit move since we've consumed it.
355                edit_move.start = None;
356            }
357        }
358    }
359}
pub fn read_with_id(&mut self) -> MessageIteratorWithId<'_, M>

Like read, except also returning the MessageId of the messages.

pub fn par_read(&mut self) -> MessageParIter<'_, M>

Available on crate feature multi_threaded only.

Returns a parallel iterator over the messages this MessageReader has not seen yet. See also for_each.

Example

#[derive(Message)]
struct MyMessage {
    value: usize,
}

#[derive(Resource, Default)]
struct Counter(AtomicUsize);

// setup
let mut world = World::new();
world.init_resource::<Messages<MyMessage>>();
world.insert_resource(Counter::default());

let mut schedule = Schedule::default();
schedule.add_systems(|mut messages: MessageReader<MyMessage>, counter: Res<Counter>| {
    messages.par_read().for_each(|MyMessage { value }| {
        counter.0.fetch_add(*value, Ordering::Relaxed);
    });
});
for value in 0..100 {
    world.write_message(MyMessage { value });
}
schedule.run(&mut world);
let Counter(counter) = world.remove_resource::<Counter>().unwrap();
// all messages were processed
assert_eq!(counter.into_inner(), 4950);
pub fn len(&self) -> usize

Determines the number of messages available to be read from this MessageReader without consuming any.

pub fn is_empty(&self) -> bool

Returns true if there are no messages available to read.

Example

The following example shows a useful pattern where some behavior is triggered if new messages are available. MessageReader::clear() is used so the same messages don’t re-trigger the behavior the next time the system runs.

#[derive(Message)]
struct Collision;

fn play_collision_sound(mut messages: MessageReader<Collision>) {
    if !messages.is_empty() {
        messages.clear();
        // Play a sound
    }
}
Examples found in repository?
examples/ecs/extraction.rs (line 123)
113fn extract_components(
114    b: Extract<Query<(RenderEntity, &B)>>,
115    c: Extract<Query<(RenderEntity, &C)>>,
116    mut reader: Extract<MessageReader<ExtractMessage>>,
117    mut commands: Commands,
118) {
119    for (entity, b) in &b {
120        commands.entity(entity).insert(b.clone());
121    }
122
123    if !reader.is_empty() {
124        for (entity, c) in &c {
125            commands.entity(entity).insert(c.clone());
126        }
127        reader.clear();
128    }
129}
More examples
Hide additional examples
examples/asset/processing/asset_processing.rs (line 254)
249fn print_text(
250    handles: Res<TextAssets>,
251    texts: Res<Assets<Text>>,
252    mut asset_events: MessageReader<AssetEvent<Text>>,
253) {
254    if !asset_events.is_empty() {
255        // This prints the current values of the assets
256        // Hot-reloading is supported, so try modifying the source assets (and their meta files)!
257        println!("Current Values:");
258        println!("  a: {:?}", texts.get(&handles.a));
259        println!("  b: {:?}", texts.get(&handles.b));
260        println!("  c: {:?}", texts.get(&handles.c));
261        println!("  d: {:?}", texts.get(&handles.d));
262        println!("  e: {:?}", texts.get(&handles.e));
263        println!("(You can modify source assets and their .meta files to hot-reload changes!)");
264        println!();
265        asset_events.clear();
266    }
267}
pub fn clear(&mut self)

Consumes all available messages.

This means these messages will not appear in calls to MessageReader::read() or MessageReader::read_with_id() and MessageReader::is_empty() will return true.

For usage, see MessageReader::is_empty().

Examples found in repository?
examples/ecs/extraction.rs (line 127)
113fn extract_components(
114    b: Extract<Query<(RenderEntity, &B)>>,
115    c: Extract<Query<(RenderEntity, &C)>>,
116    mut reader: Extract<MessageReader<ExtractMessage>>,
117    mut commands: Commands,
118) {
119    for (entity, b) in &b {
120        commands.entity(entity).insert(b.clone());
121    }
122
123    if !reader.is_empty() {
124        for (entity, c) in &c {
125            commands.entity(entity).insert(c.clone());
126        }
127        reader.clear();
128    }
129}
More examples
Hide additional examples
examples/asset/processing/asset_processing.rs (line 265)
249fn print_text(
250    handles: Res<TextAssets>,
251    texts: Res<Assets<Text>>,
252    mut asset_events: MessageReader<AssetEvent<Text>>,
253) {
254    if !asset_events.is_empty() {
255        // This prints the current values of the assets
256        // Hot-reloading is supported, so try modifying the source assets (and their meta files)!
257        println!("Current Values:");
258        println!("  a: {:?}", texts.get(&handles.a));
259        println!("  b: {:?}", texts.get(&handles.b));
260        println!("  c: {:?}", texts.get(&handles.c));
261        println!("  d: {:?}", texts.get(&handles.d));
262        println!("  e: {:?}", texts.get(&handles.e));
263        println!("(You can modify source assets and their .meta files to hot-reload changes!)");
264        println!();
265        asset_events.clear();
266    }
267}