MessageWriter

Struct MessageWriter 

pub struct MessageWriter<'w, E>
where E: Message,
{ /* private fields */ }
Writes Messages of type T.

Usage

MessageWriters are usually declared as a SystemParam.


#[derive(Message)]
pub struct MyMessage; // Custom message type.
fn my_system(mut writer: MessageWriter<MyMessage>) {
    writer.write(MyMessage);
}

Concurrency

MessageWriter param has ResMut<Messages<T>> inside. So two systems declaring MessageWriter<T> params for the same message type won’t be executed concurrently.

Untyped messages

MessageWriter can only write messages of one specific type, which must be known at compile-time. This is not a problem most of the time, but you may find a situation where you cannot know ahead of time every kind of message you’ll need to write. In this case, you can use the “type-erased message” pattern.

fn write_untyped(mut commands: Commands) {
    // Write a message of a specific type without having to declare that
    // type as a SystemParam.
    //
    // Effectively, we're just moving the type parameter from the /type/ to the /method/,
    // which allows one to do all kinds of clever things with type erasure, such as sending
    // custom messages to unknown 3rd party plugins (modding API).
    //
    // NOTE: the message won't actually be sent until commands get applied during
    // apply_deferred.
    commands.queue(|w: &mut World| {
        w.write_message(MyMessage);
    });
}

Note that this is considered non-idiomatic, and should only be used when MessageWriter will not work.

Implementations

Source

impl<'w, E> MessageWriter<'w, E>
where E: Message,

pub fn write(&mut self, message: E) -> MessageId<E>

Writes an message, which can later be read by MessageReaders. This method returns the ID of the written message.

See Messages for details.

Examples found in repository?
examples/ecs/send_and_receive_messages.rs (line 61)
59fn read_and_write_different_message_types(mut a: MessageWriter<A>, mut b: MessageReader<B>) {
60    for _ in b.read() {}
61    a.write(A);
62}
63
64/// A dummy message type.
65#[derive(Debug, Clone, Message)]
66struct DebugMessage {
67    resend_from_param_set: bool,
68    resend_from_local_message_reader: bool,
69    times_sent: u8,
70}
71
72/// A system that sends all combinations of messages.
73fn send_messages(mut debug_messages: MessageWriter<DebugMessage>, frame_count: Res<FrameCount>) {
74    println!("Sending messages for frame {}", frame_count.0);
75
76    debug_messages.write(DebugMessage {
77        resend_from_param_set: false,
78        resend_from_local_message_reader: false,
79        times_sent: 1,
80    });
81    debug_messages.write(DebugMessage {
82        resend_from_param_set: true,
83        resend_from_local_message_reader: false,
84        times_sent: 1,
85    });
86    debug_messages.write(DebugMessage {
87        resend_from_param_set: false,
88        resend_from_local_message_reader: true,
89        times_sent: 1,
90    });
91    debug_messages.write(DebugMessage {
92        resend_from_param_set: true,
93        resend_from_local_message_reader: true,
94        times_sent: 1,
95    });
96}
97
98/// A system that prints all messages sent since the last time this system ran.
99///
100/// Note that some messages will be printed twice, because they were sent twice.
101fn debug_messages(mut messages: MessageReader<DebugMessage>) {
102    for message in messages.read() {
103        println!("{message:?}");
104    }
105}
106
107/// A system that both sends and receives messages using [`ParamSet`].
108fn send_and_receive_param_set(
109    mut param_set: ParamSet<(MessageReader<DebugMessage>, MessageWriter<DebugMessage>)>,
110    frame_count: Res<FrameCount>,
111) {
112    println!(
113        "Sending and receiving messages for frame {} with a `ParamSet`",
114        frame_count.0
115    );
116
117    // We must collect the messages to resend, because we can't access the writer while we're iterating over the reader.
118    let mut messages_to_resend = Vec::new();
119
120    // This is p0, as the first parameter in the `ParamSet` is the reader.
121    for message in param_set.p0().read() {
122        if message.resend_from_param_set {
123            messages_to_resend.push(message.clone());
124        }
125    }
126
127    // This is p1, as the second parameter in the `ParamSet` is the writer.
128    for mut message in messages_to_resend {
129        message.times_sent += 1;
130        param_set.p1().write(message);
131    }
132}
More examples
Hide additional examples
examples/app/custom_loop.rs (line 38)
36fn exit_system(input: Res<Input>, mut app_exit_reader: MessageWriter<AppExit>) {
37    if input.0 == "exit" {
38        app_exit_reader.write(AppExit::Success);
39    }
40}
examples/async_tasks/external_source_external_thread.rs (line 49)
47fn read_stream(receiver: Res<StreamReceiver>, mut events: MessageWriter<StreamMessage>) {
48    for from_stream in receiver.try_iter() {
49        events.write(StreamMessage(from_stream));
50    }
51}
examples/ecs/message.rs (line 47)
40fn deal_damage_over_time(
41    time: Res<Time>,
42    mut state: ResMut<DamageTimer>,
43    mut deal_damage_writer: MessageWriter<DealDamage>,
44) {
45    if state.tick(time.delta()).is_finished() {
46        // Messages can be sent with 'write' and constructed just like any other object.
47        deal_damage_writer.write(DealDamage { amount: 10 });
48    }
49}
50
51// This system mutates the 'DealDamage' messages to apply some armor value
52// It also sends an 'ArmorBlockedDamage' message if the value of 'DealDamage' is zero
53//
54// Messages are mutated using an 'MessageMutator<T>' by calling 'read'. This returns an iterator
55// over all the &mut T that this system has not read yet. Note, you can have multiple
56// 'MessageReader', 'MessageWriter', and 'MessageMutator' in a given system, as long as the types (T) are different.
57fn apply_armor_to_damage(
58    mut dmg_messages: MessageMutator<DealDamage>,
59    mut armor_messages: MessageWriter<ArmorBlockedDamage>,
60) {
61    for message in dmg_messages.read() {
62        message.amount -= 1;
63        if message.amount <= 0 {
64            // Zero-sized messages can also be sent with 'send'
65            armor_messages.write(ArmorBlockedDamage);
66        }
67    }
68}
examples/ecs/ecs_guide.rs (line 171)
164fn game_over_system(
165    game_rules: Res<GameRules>,
166    game_state: Res<GameState>,
167    mut app_exit_writer: MessageWriter<AppExit>,
168) {
169    if let Some(ref player) = game_state.winning_player {
170        println!("{player} won the game!");
171        app_exit_writer.write(AppExit::Success);
172    } else if game_state.current_round == game_rules.max_rounds {
173        println!("Ran out of rounds. Nobody wins!");
174        app_exit_writer.write(AppExit::Success);
175    }
176}
examples/3d/../helpers/widgets.rs (line 178)
167pub fn handle_ui_interactions<T>(
168    mut interactions: Query<
169        (&Interaction, &WidgetClickSender<T>),
170        (With<Button>, With<RadioButton>),
171    >,
172    mut widget_click_events: MessageWriter<WidgetClickEvent<T>>,
173) where
174    T: Clone + Send + Sync + 'static,
175{
176    for (interaction, click_event) in interactions.iter_mut() {
177        if *interaction == Interaction::Pressed {
178            widget_click_events.write(WidgetClickEvent((**click_event).clone()));
179        }
180    }
181}
pub fn write_batch( &mut self, messages: impl IntoIterator<Item = E>, ) -> WriteBatchIds<E>

Writes a list of messages all at once, which can later be read by MessageReaders. This is more efficient than writing each message individually. This method returns the IDs of the written messages.

See Messages for details.

Examples found in repository?
examples/app/log_layers_ecs.rs (line 58)
53fn transfer_log_messages(
54    receiver: NonSend<CapturedLogMessages>,
55    mut message_writer: MessageWriter<LogMessage>,
56) {
57    // Make sure to use `try_iter()` and not `iter()` to prevent blocking.
58    message_writer.write_batch(receiver.try_iter());
59}
pub fn write_default(&mut self) -> MessageId<E>
where E: Default,

Writes the default value of the message. Useful when the message is an empty struct. This method returns the ID of the written message.

See Messages for details.

Examples found in repository?
examples/ecs/message.rs (line 84)
76fn apply_damage_to_health(
77    mut deal_damage_reader: MessageReader<DealDamage>,
78    mut damaged_received_writer: MessageWriter<DamageReceived>,
79) {
80    for deal_damage in deal_damage_reader.read() {
81        info!("Applying {} damage", deal_damage.amount);
82        if deal_damage.amount > 0 {
83            // Messages with a 'Default' implementation can be written with 'write_default'
84            damaged_received_writer.write_default();
85        }
86    }
87}