ParamSet

Struct ParamSet 

pub struct ParamSet<'w, 's, T>
where T: SystemParam,
{ /* private fields */ }
A collection of potentially conflicting SystemParams allowed by disjoint access.

Allows systems to safely access and interact with up to 8 mutually exclusive SystemParams, such as two queries that reference the same mutable data or an event reader and writer of the same type.

Each individual SystemParam can be accessed by using the functions p0(), p1(), …, p7(), according to the order they are defined in the ParamSet. This ensures that there’s either only one mutable reference to a parameter at a time or any number of immutable references.

Examples

The following system mutably accesses the same component two times, which is not allowed due to rust’s mutability rules.

// This will panic at runtime when the system gets initialized.
fn bad_system(
    mut enemies: Query<&mut Health, With<Enemy>>,
    mut allies: Query<&mut Health, With<Ally>>,
) {
    // ...
}

Conflicting SystemParams like these can be placed in a ParamSet, which leverages the borrow checker to ensure that only one of the contained parameters are accessed at a given time.

// Given the following system
fn fancy_system(
    mut set: ParamSet<(
        Query<&mut Health, With<Enemy>>,
        Query<&mut Health, With<Ally>>,
    )>
) {
    // This will access the first `SystemParam`.
    for mut health in set.p0().iter_mut() {
        // Do your fancy stuff here...
    }

    // The second `SystemParam`.
    // This would fail to compile if the previous parameter was still borrowed.
    for mut health in set.p1().iter_mut() {
        // Do even fancier stuff here...
    }
}

Of course, ParamSets can be used with any kind of SystemParam, not just queries.

fn message_system(
    mut set: ParamSet<(
        // PROBLEM: `MessageReader` and `MessageWriter` cannot be used together normally,
        // because they both need access to the same message queue.
        // SOLUTION: `ParamSet` allows these conflicting parameters to be used safely
        // by ensuring only one is accessed at a time.
        // Note that a better solution here is to use `MessageMutator`,
        // which both reads and writes messages with a single parameter.
        MessageReader<MyMessage>,
        MessageWriter<MyMessage>,
        // PROBLEM: `&World` needs read access to everything, which conflicts with
        // any mutable access in the same system.
        // SOLUTION: `ParamSet` ensures `&World` is only accessed when we're not
        // using the other mutable parameters.
        &World,
    )>,
) {
    for message in set.p0().read() {
        // ...
    }
    set.p1().write(MyMessage::new());

    let entities = set.p2().entities();
    // ...
}

Implementations

Source

impl<'w, 's, P0> ParamSet<'w, 's, (P0,)>
where P0: SystemParam,

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<'w, 's, P0, P1> ParamSet<'w, 's, (P0, P1)>
where P0: SystemParam, P1: SystemParam,

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/showcase/alien_cake_addict.rs (line 304)
268fn focus_camera(
269    time: Res<Time>,
270    mut game: ResMut<Game>,
271    mut transforms: ParamSet<(Query<&mut Transform, With<Camera3d>>, Query<&Transform>)>,
272) {
273    const SPEED: f32 = 2.0;
274    // if there is both a player and a bonus, target the mid-point of them
275    if let (Some(player_entity), Some(bonus_entity)) = (game.player.entity, game.bonus.entity) {
276        let transform_query = transforms.p1();
277        if let (Ok(player_transform), Ok(bonus_transform)) = (
278            transform_query.get(player_entity),
279            transform_query.get(bonus_entity),
280        ) {
281            game.camera_should_focus = player_transform
282                .translation
283                .lerp(bonus_transform.translation, 0.5);
284        }
285        // otherwise, if there is only a player, target the player
286    } else if let Some(player_entity) = game.player.entity {
287        if let Ok(player_transform) = transforms.p1().get(player_entity) {
288            game.camera_should_focus = player_transform.translation;
289        }
290        // otherwise, target the middle
291    } else {
292        game.camera_should_focus = Vec3::from(RESET_FOCUS);
293    }
294    // calculate the camera motion based on the difference between where the camera is looking
295    // and where it should be looking; the greater the distance, the faster the motion;
296    // smooth out the camera movement using the frame time
297    let mut camera_motion = game.camera_should_focus - game.camera_is_focus;
298    if camera_motion.length() > 0.2 {
299        camera_motion *= SPEED * time.delta_secs();
300        // set the new camera's actual focus
301        game.camera_is_focus += camera_motion;
302    }
303    // look at that new camera's actual focus
304    for mut transform in transforms.p0().iter_mut() {
305        *transform = transform.looking_at(game.camera_is_focus, Vec3::Y);
306    }
307}
More examples
Hide additional examples
examples/ui/text/multiline_text_input.rs (line 167)
28fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
29    commands.spawn(Camera2d);
30
31    commands
32        .spawn(Node {
33            width: percent(100.),
34            height: percent(100.),
35            justify_content: JustifyContent::Center,
36            align_items: AlignItems::Center,
37            ..default()
38        })
39        .with_children(|parent| {
40            parent
41                .spawn((
42                    Node {
43                        flex_direction: FlexDirection::Column,
44                        align_items: AlignItems::End,
45                        row_gap: px(10.),
46                        ..default()
47                    },
48                    TabGroup::default(),
49                ))
50                .with_children(|parent| {
51                    parent
52                        .spawn((
53                            Node {
54                                width: px(450.),
55                                border: px(2.).all(),
56                                padding: px(8.).all(),
57                                ..default()
58                            },
59                            EditableText {
60                                visible_lines: Some(8.),
61                                allow_newlines: true,
62                                ..default()
63                            },
64                            TextLayout {
65                                linebreak: LineBreak::WordOrCharacter,
66                                ..default()
67                            },
68                            TextCursorStyle {
69                                color: Color::WHITE,
70                                selected_text_color: Some(Color::BLACK),
71                                ..default()
72                            },
73                            TextFont {
74                                font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
75                                font_size: FontSize::Px(30.),
76                                ..default()
77                            },
78                            BackgroundColor(DARK_SLATE_GRAY.into()),
79                            BorderColor::all(SLATE_300),
80                            MultilineInput,
81                            TabIndex(0),
82                            AutoFocus,
83                        ))
84                        .observe(
85                            |on: On<FocusedInput<KeyboardInput>>,
86                             keys: Res<ButtonInput<Key>>,
87                             input_query: Query<&EditableText, With<MultilineInput>>| {
88                                if !(on.input.state.is_pressed()
89                                    && on.input.logical_key == Key::Enter
90                                    && keys.pressed(Key::Control))
91                                {
92                                    return;
93                                }
94                                let Ok(input) = input_query.get(on.focused_entity) else {
95                                    return;
96                                };
97
98                                let mut output = String::new();
99                                output.reserve(input.value().into_iter().map(str::len).sum());
100                                for sub_str in input.value() {
101                                    output.push_str(sub_str);
102                                }
103
104                                info!("{output}"                                    );
105                            },
106                        );
107
108                    parent
109                        .spawn((
110                            Node {
111                                flex_direction: FlexDirection::Row,
112                                column_gap: px(10.),
113                                ..default()
114                            },
115                            children![
116                                (
117                                    Text::new("visible lines:"),
118                                    TextFont {
119                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
120                                        font_size: FontSize::Px(30.),
121                                        ..default()
122                                    },
123                                ),
124                                (
125                                    Node {
126                                        width: px(100.),
127                                        border: px(2.).all(),
128                                        ..default()
129                                    },
130                                    TextFont {
131                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
132                                        font_size: FontSize::Px(30.),
133                                        ..default()
134                                    },
135                                    TextLayout {
136                                        justify: Justify::End,
137                                        ..default()
138                                    },
139                                    BackgroundColor(DARK_SLATE_GRAY.into()),
140                                    BorderColor::all(SLATE_300),
141                                    EditableText::new("8"),
142                                    EditableTextFilter::new(|c| c.is_ascii_digit() || c == '.'),
143                                    TextCursorStyle {
144                                        color: Color::WHITE,
145                                        selected_text_color: Some(Color::BLACK),
146                                        unfocused_selection_color: Color::NONE,
147                                        ..default()
148                                    },
149                                    SelectAllOnFocus,
150                                    VisibleLinesInput,
151                                    TabIndex(1),
152                                )
153                            ],
154                        ))
155                        .observe(
156                            |on: On<FocusedInput<KeyboardInput>>,
157                             mut query_set: ParamSet<(
158                                Query<&EditableText, With<VisibleLinesInput>>,
159                                Query<&mut EditableText, With<MultilineInput>>,
160                            )>| {
161                                if !(on.input.state.is_pressed()
162                                    && on.input.logical_key == Key::Enter)
163                                {
164                                    return;
165                                }
166
167                                let visible_lines_query = query_set.p0();
168                                let Ok(input) = visible_lines_query.get(on.original_event_target())
169                                else {
170                                    return;
171                                };
172
173                                let mut output = String::new();
174                                output.reserve(input.value().into_iter().map(str::len).sum());
175                                for sub_str in input.value() {
176                                    output.push_str(sub_str);
177                                }
178
179                                let Ok(lines) = output.parse::<f32>() else {
180                                    return;
181                                };
182
183                                let mut multiline_query = query_set.p1();
184                                let Ok(mut multiline_input) = multiline_query.single_mut() else {
185                                    return;
186                                };
187
188                                multiline_input.visible_lines = Some(lines.clamp(1., 10.));
189                            },
190                        );
191
192                    parent
193                        .spawn((
194                            Node {
195                                flex_direction: FlexDirection::Row,
196                                column_gap: px(10.),
197                                ..default()
198                            },
199                            children![
200                                (
201                                    Text::new("font size:"),
202                                    TextFont {
203                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
204                                        font_size: FontSize::Px(30.),
205                                        ..default()
206                                    },
207                                ),
208                                (
209                                    Node {
210                                        width: px(100.),
211                                        border: px(2.).all(),
212                                        ..default()
213                                    },
214                                    TextFont {
215                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
216                                        font_size: FontSize::Px(30.),
217                                        ..default()
218                                    },
219                                    TextLayout {
220                                        justify: Justify::End,
221                                        ..default()
222                                    },
223                                    BackgroundColor(DARK_SLATE_GRAY.into()),
224                                    BorderColor::all(SLATE_300),
225                                    EditableText::new("30"),
226                                    EditableTextFilter::new(|c| c.is_ascii_digit()),
227                                    TextCursorStyle {
228                                        color: Color::WHITE,
229                                        selected_text_color: Some(Color::BLACK),
230                                        unfocused_selection_color: Color::NONE,
231                                        ..default()
232                                    },
233                                    SelectAllOnFocus,
234                                    FontSizeInput,
235                                    TabIndex(2),
236                                )
237                            ],
238                        ))
239                        .observe(
240                            |on: On<FocusedInput<KeyboardInput>>,
241                             font_size_input_query: Query<&EditableText, With<FontSizeInput>>,
242                             mut multiline_input_font: Single<
243                                &mut TextFont,
244                                With<MultilineInput>,
245                            >| {
246                                if !(on.input.state.is_pressed()
247                                    && on.input.logical_key == Key::Enter)
248                                {
249                                    return;
250                                }
251
252                                let Ok(input) =
253                                    font_size_input_query.get(on.original_event_target())
254                                else {
255                                    return;
256                                };
257
258                                let mut output = String::new();
259                                output.reserve(input.value().into_iter().map(str::len).sum());
260                                for sub_str in input.value() {
261                                    output.push_str(sub_str);
262                                }
263
264                                let Ok(font_size) = output.parse::<f32>() else {
265                                    return;
266                                };
267
268                                multiline_input_font.font_size =
269                                    FontSize::Px(font_size.clamp(5., 50.));
270                            },
271                        );
272                });
273        });
274}
pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/showcase/alien_cake_addict.rs (line 276)
268fn focus_camera(
269    time: Res<Time>,
270    mut game: ResMut<Game>,
271    mut transforms: ParamSet<(Query<&mut Transform, With<Camera3d>>, Query<&Transform>)>,
272) {
273    const SPEED: f32 = 2.0;
274    // if there is both a player and a bonus, target the mid-point of them
275    if let (Some(player_entity), Some(bonus_entity)) = (game.player.entity, game.bonus.entity) {
276        let transform_query = transforms.p1();
277        if let (Ok(player_transform), Ok(bonus_transform)) = (
278            transform_query.get(player_entity),
279            transform_query.get(bonus_entity),
280        ) {
281            game.camera_should_focus = player_transform
282                .translation
283                .lerp(bonus_transform.translation, 0.5);
284        }
285        // otherwise, if there is only a player, target the player
286    } else if let Some(player_entity) = game.player.entity {
287        if let Ok(player_transform) = transforms.p1().get(player_entity) {
288            game.camera_should_focus = player_transform.translation;
289        }
290        // otherwise, target the middle
291    } else {
292        game.camera_should_focus = Vec3::from(RESET_FOCUS);
293    }
294    // calculate the camera motion based on the difference between where the camera is looking
295    // and where it should be looking; the greater the distance, the faster the motion;
296    // smooth out the camera movement using the frame time
297    let mut camera_motion = game.camera_should_focus - game.camera_is_focus;
298    if camera_motion.length() > 0.2 {
299        camera_motion *= SPEED * time.delta_secs();
300        // set the new camera's actual focus
301        game.camera_is_focus += camera_motion;
302    }
303    // look at that new camera's actual focus
304    for mut transform in transforms.p0().iter_mut() {
305        *transform = transform.looking_at(game.camera_is_focus, Vec3::Y);
306    }
307}
More examples
Hide additional examples
examples/ui/text/multiline_text_input.rs (line 183)
28fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
29    commands.spawn(Camera2d);
30
31    commands
32        .spawn(Node {
33            width: percent(100.),
34            height: percent(100.),
35            justify_content: JustifyContent::Center,
36            align_items: AlignItems::Center,
37            ..default()
38        })
39        .with_children(|parent| {
40            parent
41                .spawn((
42                    Node {
43                        flex_direction: FlexDirection::Column,
44                        align_items: AlignItems::End,
45                        row_gap: px(10.),
46                        ..default()
47                    },
48                    TabGroup::default(),
49                ))
50                .with_children(|parent| {
51                    parent
52                        .spawn((
53                            Node {
54                                width: px(450.),
55                                border: px(2.).all(),
56                                padding: px(8.).all(),
57                                ..default()
58                            },
59                            EditableText {
60                                visible_lines: Some(8.),
61                                allow_newlines: true,
62                                ..default()
63                            },
64                            TextLayout {
65                                linebreak: LineBreak::WordOrCharacter,
66                                ..default()
67                            },
68                            TextCursorStyle {
69                                color: Color::WHITE,
70                                selected_text_color: Some(Color::BLACK),
71                                ..default()
72                            },
73                            TextFont {
74                                font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
75                                font_size: FontSize::Px(30.),
76                                ..default()
77                            },
78                            BackgroundColor(DARK_SLATE_GRAY.into()),
79                            BorderColor::all(SLATE_300),
80                            MultilineInput,
81                            TabIndex(0),
82                            AutoFocus,
83                        ))
84                        .observe(
85                            |on: On<FocusedInput<KeyboardInput>>,
86                             keys: Res<ButtonInput<Key>>,
87                             input_query: Query<&EditableText, With<MultilineInput>>| {
88                                if !(on.input.state.is_pressed()
89                                    && on.input.logical_key == Key::Enter
90                                    && keys.pressed(Key::Control))
91                                {
92                                    return;
93                                }
94                                let Ok(input) = input_query.get(on.focused_entity) else {
95                                    return;
96                                };
97
98                                let mut output = String::new();
99                                output.reserve(input.value().into_iter().map(str::len).sum());
100                                for sub_str in input.value() {
101                                    output.push_str(sub_str);
102                                }
103
104                                info!("{output}"                                    );
105                            },
106                        );
107
108                    parent
109                        .spawn((
110                            Node {
111                                flex_direction: FlexDirection::Row,
112                                column_gap: px(10.),
113                                ..default()
114                            },
115                            children![
116                                (
117                                    Text::new("visible lines:"),
118                                    TextFont {
119                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
120                                        font_size: FontSize::Px(30.),
121                                        ..default()
122                                    },
123                                ),
124                                (
125                                    Node {
126                                        width: px(100.),
127                                        border: px(2.).all(),
128                                        ..default()
129                                    },
130                                    TextFont {
131                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
132                                        font_size: FontSize::Px(30.),
133                                        ..default()
134                                    },
135                                    TextLayout {
136                                        justify: Justify::End,
137                                        ..default()
138                                    },
139                                    BackgroundColor(DARK_SLATE_GRAY.into()),
140                                    BorderColor::all(SLATE_300),
141                                    EditableText::new("8"),
142                                    EditableTextFilter::new(|c| c.is_ascii_digit() || c == '.'),
143                                    TextCursorStyle {
144                                        color: Color::WHITE,
145                                        selected_text_color: Some(Color::BLACK),
146                                        unfocused_selection_color: Color::NONE,
147                                        ..default()
148                                    },
149                                    SelectAllOnFocus,
150                                    VisibleLinesInput,
151                                    TabIndex(1),
152                                )
153                            ],
154                        ))
155                        .observe(
156                            |on: On<FocusedInput<KeyboardInput>>,
157                             mut query_set: ParamSet<(
158                                Query<&EditableText, With<VisibleLinesInput>>,
159                                Query<&mut EditableText, With<MultilineInput>>,
160                            )>| {
161                                if !(on.input.state.is_pressed()
162                                    && on.input.logical_key == Key::Enter)
163                                {
164                                    return;
165                                }
166
167                                let visible_lines_query = query_set.p0();
168                                let Ok(input) = visible_lines_query.get(on.original_event_target())
169                                else {
170                                    return;
171                                };
172
173                                let mut output = String::new();
174                                output.reserve(input.value().into_iter().map(str::len).sum());
175                                for sub_str in input.value() {
176                                    output.push_str(sub_str);
177                                }
178
179                                let Ok(lines) = output.parse::<f32>() else {
180                                    return;
181                                };
182
183                                let mut multiline_query = query_set.p1();
184                                let Ok(mut multiline_input) = multiline_query.single_mut() else {
185                                    return;
186                                };
187
188                                multiline_input.visible_lines = Some(lines.clamp(1., 10.));
189                            },
190                        );
191
192                    parent
193                        .spawn((
194                            Node {
195                                flex_direction: FlexDirection::Row,
196                                column_gap: px(10.),
197                                ..default()
198                            },
199                            children![
200                                (
201                                    Text::new("font size:"),
202                                    TextFont {
203                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
204                                        font_size: FontSize::Px(30.),
205                                        ..default()
206                                    },
207                                ),
208                                (
209                                    Node {
210                                        width: px(100.),
211                                        border: px(2.).all(),
212                                        ..default()
213                                    },
214                                    TextFont {
215                                        font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
216                                        font_size: FontSize::Px(30.),
217                                        ..default()
218                                    },
219                                    TextLayout {
220                                        justify: Justify::End,
221                                        ..default()
222                                    },
223                                    BackgroundColor(DARK_SLATE_GRAY.into()),
224                                    BorderColor::all(SLATE_300),
225                                    EditableText::new("30"),
226                                    EditableTextFilter::new(|c| c.is_ascii_digit()),
227                                    TextCursorStyle {
228                                        color: Color::WHITE,
229                                        selected_text_color: Some(Color::BLACK),
230                                        unfocused_selection_color: Color::NONE,
231                                        ..default()
232                                    },
233                                    SelectAllOnFocus,
234                                    FontSizeInput,
235                                    TabIndex(2),
236                                )
237                            ],
238                        ))
239                        .observe(
240                            |on: On<FocusedInput<KeyboardInput>>,
241                             font_size_input_query: Query<&EditableText, With<FontSizeInput>>,
242                             mut multiline_input_font: Single<
243                                &mut TextFont,
244                                With<MultilineInput>,
245                            >| {
246                                if !(on.input.state.is_pressed()
247                                    && on.input.logical_key == Key::Enter)
248                                {
249                                    return;
250                                }
251
252                                let Ok(input) =
253                                    font_size_input_query.get(on.original_event_target())
254                                else {
255                                    return;
256                                };
257
258                                let mut output = String::new();
259                                output.reserve(input.value().into_iter().map(str::len).sum());
260                                for sub_str in input.value() {
261                                    output.push_str(sub_str);
262                                }
263
264                                let Ok(font_size) = output.parse::<f32>() else {
265                                    return;
266                                };
267
268                                multiline_input_font.font_size =
269                                    FontSize::Px(font_size.clamp(5., 50.));
270                            },
271                        );
272                });
273        });
274}
Source

impl<'w, 's, P0, P1, P2> ParamSet<'w, 's, (P0, P1, P2)>
where P0: SystemParam, P1: SystemParam, P2: SystemParam,

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<'w, 's, P0, P1, P2, P3> ParamSet<'w, 's, (P0, P1, P2, P3)>

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/3d/shadow_caster_receiver.rs (line 144)
132fn toggle_shadows(
133    mut commands: Commands,
134    input: Res<ButtonInput<KeyCode>>,
135    mut queries: ParamSet<(
136        Query<Entity, (With<Mesh3d>, With<NotShadowCaster>)>,
137        Query<Entity, (With<Mesh3d>, With<NotShadowReceiver>)>,
138        Query<Entity, (With<Mesh3d>, Without<NotShadowCaster>)>,
139        Query<Entity, (With<Mesh3d>, Without<NotShadowReceiver>)>,
140    )>,
141) {
142    if input.just_pressed(KeyCode::KeyC) {
143        println!("Toggling casters");
144        for entity in queries.p0().iter() {
145            commands.entity(entity).remove::<NotShadowCaster>();
146        }
147        for entity in queries.p2().iter() {
148            commands.entity(entity).insert(NotShadowCaster);
149        }
150    }
151    if input.just_pressed(KeyCode::KeyR) {
152        println!("Toggling receivers");
153        for entity in queries.p1().iter() {
154            commands.entity(entity).remove::<NotShadowReceiver>();
155        }
156        for entity in queries.p3().iter() {
157            commands.entity(entity).insert(NotShadowReceiver);
158        }
159    }
160}
pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/3d/shadow_caster_receiver.rs (line 153)
132fn toggle_shadows(
133    mut commands: Commands,
134    input: Res<ButtonInput<KeyCode>>,
135    mut queries: ParamSet<(
136        Query<Entity, (With<Mesh3d>, With<NotShadowCaster>)>,
137        Query<Entity, (With<Mesh3d>, With<NotShadowReceiver>)>,
138        Query<Entity, (With<Mesh3d>, Without<NotShadowCaster>)>,
139        Query<Entity, (With<Mesh3d>, Without<NotShadowReceiver>)>,
140    )>,
141) {
142    if input.just_pressed(KeyCode::KeyC) {
143        println!("Toggling casters");
144        for entity in queries.p0().iter() {
145            commands.entity(entity).remove::<NotShadowCaster>();
146        }
147        for entity in queries.p2().iter() {
148            commands.entity(entity).insert(NotShadowCaster);
149        }
150    }
151    if input.just_pressed(KeyCode::KeyR) {
152        println!("Toggling receivers");
153        for entity in queries.p1().iter() {
154            commands.entity(entity).remove::<NotShadowReceiver>();
155        }
156        for entity in queries.p3().iter() {
157            commands.entity(entity).insert(NotShadowReceiver);
158        }
159    }
160}
pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/3d/shadow_caster_receiver.rs (line 147)
132fn toggle_shadows(
133    mut commands: Commands,
134    input: Res<ButtonInput<KeyCode>>,
135    mut queries: ParamSet<(
136        Query<Entity, (With<Mesh3d>, With<NotShadowCaster>)>,
137        Query<Entity, (With<Mesh3d>, With<NotShadowReceiver>)>,
138        Query<Entity, (With<Mesh3d>, Without<NotShadowCaster>)>,
139        Query<Entity, (With<Mesh3d>, Without<NotShadowReceiver>)>,
140    )>,
141) {
142    if input.just_pressed(KeyCode::KeyC) {
143        println!("Toggling casters");
144        for entity in queries.p0().iter() {
145            commands.entity(entity).remove::<NotShadowCaster>();
146        }
147        for entity in queries.p2().iter() {
148            commands.entity(entity).insert(NotShadowCaster);
149        }
150    }
151    if input.just_pressed(KeyCode::KeyR) {
152        println!("Toggling receivers");
153        for entity in queries.p1().iter() {
154            commands.entity(entity).remove::<NotShadowReceiver>();
155        }
156        for entity in queries.p3().iter() {
157            commands.entity(entity).insert(NotShadowReceiver);
158        }
159    }
160}
pub fn p3<'a>(&'a mut self) -> <P3 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 3 in this ParamSet. No other parameters may be accessed while this one is active.

Examples found in repository?
examples/3d/shadow_caster_receiver.rs (line 156)
132fn toggle_shadows(
133    mut commands: Commands,
134    input: Res<ButtonInput<KeyCode>>,
135    mut queries: ParamSet<(
136        Query<Entity, (With<Mesh3d>, With<NotShadowCaster>)>,
137        Query<Entity, (With<Mesh3d>, With<NotShadowReceiver>)>,
138        Query<Entity, (With<Mesh3d>, Without<NotShadowCaster>)>,
139        Query<Entity, (With<Mesh3d>, Without<NotShadowReceiver>)>,
140    )>,
141) {
142    if input.just_pressed(KeyCode::KeyC) {
143        println!("Toggling casters");
144        for entity in queries.p0().iter() {
145            commands.entity(entity).remove::<NotShadowCaster>();
146        }
147        for entity in queries.p2().iter() {
148            commands.entity(entity).insert(NotShadowCaster);
149        }
150    }
151    if input.just_pressed(KeyCode::KeyR) {
152        println!("Toggling receivers");
153        for entity in queries.p1().iter() {
154            commands.entity(entity).remove::<NotShadowReceiver>();
155        }
156        for entity in queries.p3().iter() {
157            commands.entity(entity).insert(NotShadowReceiver);
158        }
159    }
160}
Source

impl<'w, 's, P0, P1, P2, P3, P4> ParamSet<'w, 's, (P0, P1, P2, P3, P4)>

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p3<'a>(&'a mut self) -> <P3 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 3 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p4<'a>(&'a mut self) -> <P4 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 4 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<'w, 's, P0, P1, P2, P3, P4, P5> ParamSet<'w, 's, (P0, P1, P2, P3, P4, P5)>

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p3<'a>(&'a mut self) -> <P3 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 3 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p4<'a>(&'a mut self) -> <P4 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 4 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p5<'a>(&'a mut self) -> <P5 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 5 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<'w, 's, P0, P1, P2, P3, P4, P5, P6> ParamSet<'w, 's, (P0, P1, P2, P3, P4, P5, P6)>

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p3<'a>(&'a mut self) -> <P3 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 3 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p4<'a>(&'a mut self) -> <P4 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 4 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p5<'a>(&'a mut self) -> <P5 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 5 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p6<'a>(&'a mut self) -> <P6 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 6 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<'w, 's, P0, P1, P2, P3, P4, P5, P6, P7> ParamSet<'w, 's, (P0, P1, P2, P3, P4, P5, P6, P7)>

pub fn p0<'a>(&'a mut self) -> <P0 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 0 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p1<'a>(&'a mut self) -> <P1 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 1 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p2<'a>(&'a mut self) -> <P2 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 2 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p3<'a>(&'a mut self) -> <P3 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 3 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p4<'a>(&'a mut self) -> <P4 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 4 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p5<'a>(&'a mut self) -> <P5 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 5 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p6<'a>(&'a mut self) -> <P6 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 6 in this ParamSet. No other parameters may be accessed while this one is active.

pub fn p7<'a>(&'a mut self) -> <P7 as SystemParam>::Item<'a, 'a>

Gets exclusive access to the parameter at index 7 in this ParamSet. No other parameters may be accessed while this one is active.

Source

impl<T> ParamSet<'_, '_, Vec<T>>
where T: SystemParam,

pub fn get_mut(&mut self, index: usize) -> <T as SystemParam>::Item<'_, '_>

Accesses the parameter at the given index. No other parameters may be accessed while this one is active.

pub fn for_each(&mut self, f: impl FnMut(<T as SystemParam>::Item<'_, '_>))

Calls a closure for each parameter in the set.