Struct PointLightBundle

pub struct PointLightBundle {
    pub point_light: PointLight,
    pub cubemap_visible_entities: CubemapVisibleEntities,
    pub cubemap_frusta: CubemapFrusta,
    pub transform: Transform,
    pub global_transform: GlobalTransform,
    pub visibility: Visibility,
    pub inherited_visibility: InheritedVisibility,
    pub view_visibility: ViewVisibility,
    pub sync: SyncToRenderWorld,
}
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
A component bundle for PointLight entities.

Fields

point_light: PointLight
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
cubemap_visible_entities: CubemapVisibleEntities
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
cubemap_frusta: CubemapFrusta
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
transform: Transform
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
global_transform: GlobalTransform
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.
visibility: Visibility
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.

Enables or disables the light

inherited_visibility: InheritedVisibility
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.

Inherited visibility of an entity.

view_visibility: ViewVisibility
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.

Algorithmically-computed indication of whether an entity is visible and should be extracted for rendering

sync: SyncToRenderWorld
๐Ÿ‘ŽDeprecated since 0.15.0: Use the PointLight component instead. Inserting it will now also insert the other components required by it automatically.

Marker component that indicates that its entity needs to be synchronized to the render world