pub struct Ray3d {
pub origin: Vec3,
pub direction: Dir3,
}Expand description
An infinite half-line starting at origin and going in direction in 3D space.
Fields
origin: Vec3The origin of the ray.
direction: Dir3The direction of the ray.
Implementations
sourceimpl Ray3d
impl Ray3d
sourcepub fn new(origin: Vec3, direction: Vec3) -> Ray3d
pub fn new(origin: Vec3, direction: Vec3) -> Ray3d
Create a new Ray3d from a given origin and direction
Panics
Panics if the given direction is zero (or very close to zero), or non-finite.
sourcepub fn get_point(&self, distance: f32) -> Vec3
pub fn get_point(&self, distance: f32) -> Vec3
Get a point at a given distance along the ray
Examples found in repository?
examples/3d/3d_viewport_to_world.rs (line 37)
13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41
fn draw_cursor(
camera_query: Query<(&Camera, &GlobalTransform)>,
ground_query: Query<&GlobalTransform, With<Ground>>,
windows: Query<&Window>,
mut gizmos: Gizmos,
) {
let (camera, camera_transform) = camera_query.single();
let ground = ground_query.single();
let Some(cursor_position) = windows.single().cursor_position() else {
return;
};
// Calculate a ray pointing from the camera into the world based on the cursor's position.
let Some(ray) = camera.viewport_to_world(camera_transform, cursor_position) else {
return;
};
// Calculate if and where the ray is hitting the ground plane.
let Some(distance) =
ray.intersect_plane(ground.translation(), InfinitePlane3d::new(ground.up()))
else {
return;
};
let point = ray.get_point(distance);
// Draw a circle just above the ground plane at that position.
gizmos.circle(point + ground.up() * 0.01, ground.up(), 0.2, Color::WHITE);
}sourcepub fn intersect_plane(
&self,
plane_origin: Vec3,
plane: InfinitePlane3d,
) -> Option<f32>
pub fn intersect_plane( &self, plane_origin: Vec3, plane: InfinitePlane3d, ) -> Option<f32>
Get the distance to a plane if the ray intersects it
Examples found in repository?
examples/3d/3d_viewport_to_world.rs (line 33)
13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41
fn draw_cursor(
camera_query: Query<(&Camera, &GlobalTransform)>,
ground_query: Query<&GlobalTransform, With<Ground>>,
windows: Query<&Window>,
mut gizmos: Gizmos,
) {
let (camera, camera_transform) = camera_query.single();
let ground = ground_query.single();
let Some(cursor_position) = windows.single().cursor_position() else {
return;
};
// Calculate a ray pointing from the camera into the world based on the cursor's position.
let Some(ray) = camera.viewport_to_world(camera_transform, cursor_position) else {
return;
};
// Calculate if and where the ray is hitting the ground plane.
let Some(distance) =
ray.intersect_plane(ground.translation(), InfinitePlane3d::new(ground.up()))
else {
return;
};
let point = ray.get_point(distance);
// Draw a circle just above the ground plane at that position.
gizmos.circle(point + ground.up() * 0.01, ground.up(), 0.2, Color::WHITE);
}More examples
examples/3d/irradiance_volumes.rs (line 489)
465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502
fn handle_mouse_clicks(
buttons: Res<ButtonInput<MouseButton>>,
windows: Query<&Window, With<PrimaryWindow>>,
cameras: Query<(&Camera, &GlobalTransform)>,
mut main_objects: Query<&mut Transform, With<MainObject>>,
) {
if !buttons.pressed(MouseButton::Left) {
return;
}
let Some(mouse_position) = windows
.iter()
.next()
.and_then(|window| window.cursor_position())
else {
return;
};
let Some((camera, camera_transform)) = cameras.iter().next() else {
return;
};
// Figure out where the user clicked on the plane.
let Some(ray) = camera.viewport_to_world(camera_transform, mouse_position) else {
return;
};
let Some(ray_distance) = ray.intersect_plane(Vec3::ZERO, InfinitePlane3d::new(Vec3::Y)) else {
return;
};
let plane_intersection = ray.origin + ray.direction.normalize() * ray_distance;
// Move all the main objeccts.
for mut transform in main_objects.iter_mut() {
transform.translation = vec3(
plane_intersection.x,
transform.translation.y,
plane_intersection.z,
);
}
}Trait Implementations
sourceimpl<'de> Deserialize<'de> for Ray3d
impl<'de> Deserialize<'de> for Ray3d
sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<Ray3d, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<Ray3d, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
sourceimpl FromReflect for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl FromReflect for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
sourcefn from_reflect(reflect: &(dyn Reflect + 'static)) -> Option<Ray3d>
fn from_reflect(reflect: &(dyn Reflect + 'static)) -> Option<Ray3d>
Constructs a concrete instance of
Self from a reflected value.sourcefn take_from_reflect(
reflect: Box<dyn Reflect>,
) -> Result<Self, Box<dyn Reflect>>
fn take_from_reflect( reflect: Box<dyn Reflect>, ) -> Result<Self, Box<dyn Reflect>>
Attempts to downcast the given value to
Self using,
constructing the value using from_reflect if that fails. Read moresourceimpl GetTypeRegistration for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl GetTypeRegistration for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
sourceimpl Reflect for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl Reflect for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
sourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any.sourcefn into_reflect(self: Box<Ray3d>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Ray3d>) -> Box<dyn Reflect>
Casts this type to a boxed reflected value.
sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a reflected value.
sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable reflected value.
sourcefn clone_value(&self) -> Box<dyn Reflect>
fn clone_value(&self) -> Box<dyn Reflect>
Clones the value as a
Reflect trait object. Read moresourcefn set(&mut self, value: Box<dyn Reflect>) -> Result<(), Box<dyn Reflect>>
fn set(&mut self, value: Box<dyn Reflect>) -> Result<(), Box<dyn Reflect>>
Performs a type-checked assignment of a reflected value to this value. Read more
sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
sourcefn reflect_owned(self: Box<Ray3d>) -> ReflectOwned
fn reflect_owned(self: Box<Ray3d>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
sourcefn reflect_partial_eq(&self, value: &(dyn Reflect + 'static)) -> Option<bool>
fn reflect_partial_eq(&self, value: &(dyn Reflect + 'static)) -> Option<bool>
Returns a “partial equality” comparison result. Read more
sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
sourcefn apply(&mut self, value: &(dyn Reflect + 'static))
fn apply(&mut self, value: &(dyn Reflect + 'static))
Applies a reflected value to this value. Read more
sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
sourcefn serializable(&self) -> Option<Serializable<'_>>
fn serializable(&self) -> Option<Serializable<'_>>
Returns a serializable version of the value. Read more
sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
sourceimpl Serialize for Ray3d
impl Serialize for Ray3d
sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Serialize this value into the given Serde serializer. Read more
sourceimpl Struct for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl Struct for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
sourcefn field(&self, name: &str) -> Option<&(dyn Reflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn Reflect + 'static)>
Returns a reference to the value of the field named
name as a &dyn Reflect.sourcefn field_mut(&mut self, name: &str) -> Option<&mut (dyn Reflect + 'static)>
fn field_mut(&mut self, name: &str) -> Option<&mut (dyn Reflect + 'static)>
Returns a mutable reference to the value of the field named
name as a
&mut dyn Reflect.sourcefn field_at(&self, index: usize) -> Option<&(dyn Reflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn Reflect + 'static)>
Returns a reference to the value of the field with index
index as a
&dyn Reflect.sourcefn field_at_mut(&mut self, index: usize) -> Option<&mut (dyn Reflect + 'static)>
fn field_at_mut(&mut self, index: usize) -> Option<&mut (dyn Reflect + 'static)>
Returns a mutable reference to the value of the field with index
index
as a &mut dyn Reflect.sourcefn name_at(&self, index: usize) -> Option<&str>
fn name_at(&self, index: usize) -> Option<&str>
Returns the name of the field with index
index.sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Returns an iterator over the values of the reflectable fields for this struct.
sourcefn clone_dynamic(&self) -> DynamicStruct
fn clone_dynamic(&self) -> DynamicStruct
Clones the struct into a
DynamicStruct.sourceimpl TypePath for Ray3d
impl TypePath for Ray3d
sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
sourceimpl Typed for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl Typed for Ray3dwhere
Ray3d: Any + Send + Sync,
Vec3: FromReflect + TypePath + RegisterForReflection,
Dir3: FromReflect + TypePath + RegisterForReflection,
impl Copy for Ray3d
impl StructuralPartialEq for Ray3d