Sprite

Struct Sprite 

pub struct Sprite {
    pub image: Handle<Image>,
    pub texture_atlas: Option<TextureAtlas>,
    pub color: Color,
    pub flip_x: bool,
    pub flip_y: bool,
    pub custom_size: Option<Vec2>,
    pub rect: Option<Rect>,
    pub image_mode: SpriteImageMode,
}
Describes a sprite to be rendered to a 2D camera

Fields

image: Handle<Image>

The image used to render the sprite

texture_atlas: Option<TextureAtlas>

The (optional) texture atlas used to render the sprite

color: Color

The sprite’s color tint

flip_x: bool

Flip the sprite along the X axis

flip_y: bool

Flip the sprite along the Y axis

custom_size: Option<Vec2>

An optional custom size for the sprite that will be used when rendering, instead of the size of the sprite’s image

rect: Option<Rect>

An optional rectangle representing the region of the sprite’s image to render, instead of rendering the full image. This is an easy one-off alternative to using a TextureAtlas.

When used with a TextureAtlas, the rect is offset by the atlas’s minimal (top-left) corner position.

image_mode: SpriteImageMode

How the sprite’s image will be scaled.

Implementations

Source

impl Sprite

pub fn sized(custom_size: Vec2) -> Sprite

Create a Sprite with a custom size

pub fn from_image(image: Handle<Image>) -> Sprite

Create a sprite from an image

Examples found in repository?
examples/window/transparent_window.rs (line 35)
33fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
34    commands.spawn(Camera2d);
35    commands.spawn(Sprite::from_image(asset_server.load("branding/icon.png")));
36}
More examples
Hide additional examples
examples/2d/move_sprite.rs (line 23)
19fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
20    commands.spawn(Camera2d);
21
22    commands.spawn((
23        Sprite::from_image(asset_server.load("branding/icon.png")),
24        Transform::from_xyz(0., 0., 0.),
25        Direction::Right,
26    ));
27}
examples/asset/web_asset.rs (line 21)
17fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
18    commands.spawn(Camera2d);
19    let url = "https://raw.githubusercontent.com/bevyengine/bevy/refs/heads/main/assets/branding/bevy_bird_dark.png";
20    // Simply use a url where you would normally use an asset folder relative path
21    commands.spawn(Sprite::from_image(asset_server.load(url)));
22}
examples/asset/extra_source.rs (line 45)
30fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
31    commands.spawn(Camera2d);
32
33    // Now we can load the asset using our new asset source.
34    //
35    // The actual file path relative to workspace root is
36    // "examples/asset/files/bevy_pixel_light.png".
37    let path = Path::new("bevy_pixel_light.png");
38    let source = AssetSourceId::from("example_files");
39    let asset_path = AssetPath::from_path(path).with_source(source);
40
41    // You could also parse this URL-like string representation for the asset
42    // path.
43    assert_eq!(asset_path, "example_files://bevy_pixel_light.png".into());
44
45    commands.spawn(Sprite::from_image(asset_server.load(asset_path)));
46}
examples/asset/embedded_asset.rs (line 57)
35fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
36    commands.spawn(Camera2d);
37
38    // Each example is its own crate (with name from [[example]] in Cargo.toml).
39    let crate_name = "embedded_asset";
40
41    // The actual file path relative to workspace root is
42    // "examples/asset/files/bevy_pixel_light.png".
43    //
44    // We omit the "examples/asset" from the embedded_asset! call and replace it
45    // with the crate name.
46    let path = Path::new(crate_name).join("files/bevy_pixel_light.png");
47    let source = AssetSourceId::from("embedded");
48    let asset_path = AssetPath::from_path(&path).with_source(source);
49
50    // You could also parse this URL-like string representation for the asset
51    // path.
52    assert_eq!(
53        asset_path,
54        "embedded://embedded_asset/files/bevy_pixel_light.png".into()
55    );
56
57    commands.spawn(Sprite::from_image(asset_server.load(asset_path)));
58}
pub fn from_atlas_image(image: Handle<Image>, atlas: TextureAtlas) -> Sprite

Create a sprite from an image, with an associated texture atlas

Examples found in repository?
examples/2d/sprite_sheet.rs (lines 56-62)
42fn setup(
43    mut commands: Commands,
44    asset_server: Res<AssetServer>,
45    mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
46) {
47    let texture = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
48    let layout = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
49    let texture_atlas_layout = texture_atlas_layouts.add(layout);
50    // Use only the subset of sprites in the sheet that make up the run animation
51    let animation_indices = AnimationIndices { first: 1, last: 6 };
52
53    commands.spawn(Camera2d);
54
55    commands.spawn((
56        Sprite::from_atlas_image(
57            texture,
58            TextureAtlas {
59                layout: texture_atlas_layout,
60                index: animation_indices.first,
61            },
62        ),
63        Transform::from_scale(Vec3::splat(6.0)),
64        animation_indices,
65        AnimationTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
66    ));
67}
pub fn from_color(color: impl Into<Color>, size: Vec2) -> Sprite

Create a sprite from a solid color

pub fn compute_pixel_space_point( &self, point_relative_to_sprite: Vec2, anchor: Anchor, images: &Assets<Image>, texture_atlases: &Assets<TextureAtlasLayout>, ) -> Result<Vec2, Vec2>

Computes the pixel point where point_relative_to_sprite is sampled from in this sprite. point_relative_to_sprite must be in the sprite’s local frame. Returns an Ok if the point is inside the bounds of the sprite (not just the image), and returns an Err otherwise.