pub struct TextFont {
pub font: Handle<Font>,
pub font_size: f32,
pub weight: FontWeight,
pub font_smoothing: FontSmoothing,
pub font_features: FontFeatures,
}Expand description
TextFont determines the style of a text span within a ComputedTextBlock, specifically
the font face, the font size, the line height, and the antialiasing method.
Fields
font: Handle<Font>The specific font face to use, as a Handle to a Font asset.
If the font is not specified, then
- if
default_fontfeature is enabled (enabled by default inbevycrate),FiraMono-subset.ttfcompiled into the library is used. - otherwise no text will be rendered, unless a custom font is loaded into the default font handle.
font_size: f32The vertical height of rasterized glyphs in the font atlas in pixels.
This is multiplied by the window scale factor and UiScale, but not the text entity
transform or camera projection.
A new font atlas is generated for every combination of font handle and scaled font size which can have a strong performance impact.
weight: FontWeightHow thick or bold the strokes of a font appear.
Font weights can be any value between 1 and 1000, inclusive.
Only supports variable weight fonts.
font_smoothing: FontSmoothingThe antialiasing method to use when rendering text.
font_features: FontFeaturesOpenType features for .otf fonts that support them.
Implementations
Sourceimpl TextFont
impl TextFont
Sourcepub fn from_font_size(font_size: f32) -> TextFont
pub fn from_font_size(font_size: f32) -> TextFont
Returns a new TextFont with the specified font size.
Examples found in repository?
29fn setup(
30 mut commands: Commands,
31 mut meshes: ResMut<Assets<Mesh>>,
32 mut materials: ResMut<Assets<ColorMaterial>>,
33) {
34 commands.spawn(Camera2d);
35
36 commands
37 .spawn((
38 Node {
39 width: Val::Percent(100.0),
40 height: Val::Percent(100.0),
41 align_items: AlignItems::Center,
42 justify_content: JustifyContent::Start,
43 ..default()
44 },
45 Pickable::IGNORE,
46 ))
47 .with_children(|parent| {
48 parent
49 .spawn((
50 DraggableButton,
51 Node {
52 width: Val::Px(BUTTON_WIDTH),
53 height: Val::Px(BUTTON_HEIGHT),
54 margin: UiRect::all(Val::Px(10.0)),
55 justify_content: JustifyContent::Center,
56 align_items: AlignItems::Center,
57 ..default()
58 },
59 BackgroundColor(Color::srgb(1.0, 0.0, 0.0)),
60 ))
61 .with_child((
62 Text::new("Drag from me"),
63 TextColor(Color::WHITE),
64 Pickable::IGNORE,
65 ))
66 .observe(
67 |mut event: On<Pointer<DragStart>>,
68 mut button_color: Single<&mut BackgroundColor, With<DraggableButton>>| {
69 button_color.0 = Color::srgb(1.0, 0.5, 0.0);
70 event.propagate(false);
71 },
72 )
73 .observe(
74 |mut event: On<Pointer<DragEnd>>,
75 mut button_color: Single<&mut BackgroundColor, With<DraggableButton>>| {
76 button_color.0 = Color::srgb(1.0, 0.0, 0.0);
77 event.propagate(false);
78 },
79 );
80 });
81
82 commands
83 .spawn((
84 DropArea,
85 Mesh2d(meshes.add(Rectangle::new(AREA_SIZE, AREA_SIZE))),
86 MeshMaterial2d(materials.add(Color::srgb(0.1, 0.4, 0.1))),
87 Transform::IDENTITY,
88 children![(
89 Text2d::new("Drop here"),
90 TextFont::from_font_size(50.),
91 TextColor(Color::BLACK),
92 Pickable::IGNORE,
93 Transform::from_translation(Vec3::Z),
94 )],
95 ))
96 .observe(on_drag_enter)
97 .observe(on_drag_over)
98 .observe(on_drag_drop)
99 .observe(on_drag_leave);
100}Sourcepub fn with_font(self, font: Handle<Font>) -> TextFont
pub fn with_font(self, font: Handle<Font>) -> TextFont
Returns this TextFont with the specified font face handle.
Sourcepub const fn with_font_size(self, font_size: f32) -> TextFont
pub const fn with_font_size(self, font_size: f32) -> TextFont
Returns this TextFont with the specified font size.
Sourcepub const fn with_font_smoothing(
self,
font_smoothing: FontSmoothing,
) -> TextFont
pub const fn with_font_smoothing( self, font_smoothing: FontSmoothing, ) -> TextFont
Returns this TextFont with the specified FontSmoothing.