pub struct TextureAtlasSources {
pub texture_ids: HashMap<AssetId<Image>, usize>,
}Expand description
Stores a mapping from sub texture handles to the related area index.
Generated by TextureAtlasBuilder.
Fields
texture_ids: HashMap<AssetId<Image>, usize>Maps from a specific image handle to the index in textures where they can be found.
Implementations
Sourceimpl TextureAtlasSources
impl TextureAtlasSources
Sourcepub fn texture_index(&self, texture: impl Into<AssetId<Image>>) -> Option<usize>
pub fn texture_index(&self, texture: impl Into<AssetId<Image>>) -> Option<usize>
Retrieves the texture section index of the given texture handle.
Sourcepub fn handle(
&self,
layout: Handle<TextureAtlasLayout>,
texture: impl Into<AssetId<Image>>,
) -> Option<TextureAtlas>
pub fn handle( &self, layout: Handle<TextureAtlasLayout>, texture: impl Into<AssetId<Image>>, ) -> Option<TextureAtlas>
Creates a TextureAtlas handle for the given texture handle.
Examples found in repository?
examples/2d/texture_atlas.rs (line 267)
251fn create_sprite_from_atlas(
252 commands: &mut Commands,
253 translation: (f32, f32, f32),
254 atlas_texture: Handle<Image>,
255 atlas_sources: TextureAtlasSources,
256 atlas_handle: Handle<TextureAtlasLayout>,
257 vendor_handle: &Handle<Image>,
258) {
259 commands.spawn((
260 Transform {
261 translation: Vec3::new(translation.0, translation.1, translation.2),
262 scale: Vec3::splat(3.0),
263 ..default()
264 },
265 Sprite::from_atlas_image(
266 atlas_texture,
267 atlas_sources.handle(atlas_handle, vendor_handle).unwrap(),
268 ),
269 ));
270}Sourcepub fn texture_rect(
&self,
layout: &TextureAtlasLayout,
texture: impl Into<AssetId<Image>>,
) -> Option<URect>
pub fn texture_rect( &self, layout: &TextureAtlasLayout, texture: impl Into<AssetId<Image>>, ) -> Option<URect>
Retrieves the texture section rectangle of the given texture handle in pixels.