pub struct UiPosition {
pub anchor: Vec2,
pub x: Val,
pub y: Val,
}Expand description
Responsive position relative to a UI node.
Fields
anchor: Vec2Normalized anchor point
x: ValResponsive horizontal position relative to the anchor point
y: ValResponsive vertical position relative to the anchor point
Implementations
Sourceimpl UiPosition
impl UiPosition
Sourcepub const TOP_LEFT: UiPosition
pub const TOP_LEFT: UiPosition
Position at the top-left corner
Sourcepub const LEFT: UiPosition
pub const LEFT: UiPosition
Position at the center of the left edge
Sourcepub const BOTTOM_LEFT: UiPosition
pub const BOTTOM_LEFT: UiPosition
Position at the bottom-left corner
Sourcepub const TOP: UiPosition
pub const TOP: UiPosition
Position at the center of the top edge
Sourcepub const CENTER: UiPosition
pub const CENTER: UiPosition
Position at the center of the element
Sourcepub const BOTTOM: UiPosition
pub const BOTTOM: UiPosition
Position at the center of the bottom edge
Sourcepub const TOP_RIGHT: UiPosition
pub const TOP_RIGHT: UiPosition
Position at the top-right corner
Sourcepub const RIGHT: UiPosition
pub const RIGHT: UiPosition
Position at the center of the right edge
Sourcepub const BOTTOM_RIGHT: UiPosition
pub const BOTTOM_RIGHT: UiPosition
Position at the bottom-right corner
Sourcepub const fn anchor(anchor: Vec2) -> UiPosition
pub const fn anchor(anchor: Vec2) -> UiPosition
Position at the given normalized anchor point
Sourcepub const fn at(self, x: Val, y: Val) -> UiPosition
pub const fn at(self, x: Val, y: Val) -> UiPosition
Creates a position from self with the given x and y coordinates
Sourcepub const fn at_x(self, x: Val) -> UiPosition
pub const fn at_x(self, x: Val) -> UiPosition
Creates a position from self with the given x coordinate
Examples found in repository?
16fn setup(mut commands: Commands) {
17 commands.spawn(Camera2d);
18 commands
19 .spawn(Node {
20 display: Display::Grid,
21 width: percent(100),
22 height: percent(100),
23
24 ..Default::default()
25 })
26 .with_children(|commands| {
27 commands.spawn((
28 Node {
29 width: percent(100),
30 height: percent(100),
31 ..Default::default()
32 },
33 BackgroundColor(Color::BLACK),
34 BackgroundGradient(vec![
35 LinearGradient::to_top_right(vec![
36 ColorStop::auto(RED),
37 ColorStop::auto(RED.with_alpha(0.)),
38 ])
39 .into(),
40 LinearGradient::to_top_left(vec![
41 ColorStop::auto(BLUE),
42 ColorStop::auto(BLUE.with_alpha(0.)),
43 ])
44 .into(),
45 ConicGradient {
46 start: 0.,
47 position: UiPosition::CENTER,
48 stops: vec![
49 AngularColorStop::auto(YELLOW.with_alpha(0.)),
50 AngularColorStop::auto(YELLOW.with_alpha(0.)),
51 AngularColorStop::auto(YELLOW),
52 AngularColorStop::auto(YELLOW.with_alpha(0.)),
53 AngularColorStop::auto(YELLOW.with_alpha(0.)),
54 ],
55 ..Default::default()
56 }
57 .into(),
58 RadialGradient {
59 position: UiPosition::TOP.at_x(percent(5)),
60 shape: RadialGradientShape::Circle(vh(30)),
61 stops: vec![
62 ColorStop::auto(Color::WHITE),
63 ColorStop::auto(YELLOW),
64 ColorStop::auto(YELLOW.with_alpha(0.1)),
65 ColorStop::auto(YELLOW.with_alpha(0.)),
66 ],
67 ..Default::default()
68 }
69 .into(),
70 LinearGradient {
71 angle: TAU / 16.,
72 stops: vec![
73 ColorStop::auto(Color::BLACK),
74 ColorStop::auto(Color::BLACK.with_alpha(0.)),
75 ],
76 ..Default::default()
77 }
78 .into(),
79 LinearGradient {
80 angle: 15. * TAU / 16.,
81 stops: vec![
82 ColorStop::auto(Color::BLACK),
83 ColorStop::auto(Color::BLACK.with_alpha(0.)),
84 ],
85 ..Default::default()
86 }
87 .into(),
88 ]),
89 ));
90 });
91}Sourcepub const fn at_y(self, y: Val) -> UiPosition
pub const fn at_y(self, y: Val) -> UiPosition
Creates a position from self with the given y coordinate
Sourcepub const fn at_px(self, x: f32, y: f32) -> UiPosition
pub const fn at_px(self, x: f32, y: f32) -> UiPosition
Creates a position in logical pixels from self with the given x and y coordinates
Sourcepub const fn at_percent(self, x: f32, y: f32) -> UiPosition
pub const fn at_percent(self, x: f32, y: f32) -> UiPosition
Creates a percentage position from self with the given x and y coordinates
Sourcepub const fn with_anchor(self, anchor: Vec2) -> UiPosition
pub const fn with_anchor(self, anchor: Vec2) -> UiPosition
Creates a position from self with the given anchor point
Sourcepub const fn top_left(x: Val, y: Val) -> UiPosition
pub const fn top_left(x: Val, y: Val) -> UiPosition
Position relative to the top-left corner
Sourcepub const fn left(x: Val, y: Val) -> UiPosition
pub const fn left(x: Val, y: Val) -> UiPosition
Position relative to the left edge
Sourcepub const fn bottom_left(x: Val, y: Val) -> UiPosition
pub const fn bottom_left(x: Val, y: Val) -> UiPosition
Position relative to the bottom-left corner
Sourcepub const fn top(x: Val, y: Val) -> UiPosition
pub const fn top(x: Val, y: Val) -> UiPosition
Position relative to the top edge
Sourcepub const fn center(x: Val, y: Val) -> UiPosition
pub const fn center(x: Val, y: Val) -> UiPosition
Position relative to the center
Sourcepub const fn bottom(x: Val, y: Val) -> UiPosition
pub const fn bottom(x: Val, y: Val) -> UiPosition
Position relative to the bottom edge
Sourcepub const fn top_right(x: Val, y: Val) -> UiPosition
pub const fn top_right(x: Val, y: Val) -> UiPosition
Position relative to the top-right corner
Sourcepub const fn right(x: Val, y: Val) -> UiPosition
pub const fn right(x: Val, y: Val) -> UiPosition
Position relative to the right edge
Sourcepub const fn bottom_right(x: Val, y: Val) -> UiPosition
pub const fn bottom_right(x: Val, y: Val) -> UiPosition
Position relative to the bottom-right corner