pub struct Val2 {
pub x: Val,
pub y: Val,
}Expand description
A pair of Vals used to represent a 2-dimensional size or offset.
Fields
x: ValTranslate the node along the x-axis.
Val::Percent values are resolved based on the computed width of the Ui Node.
Val::Auto is resolved to 0..
y: ValTranslate the node along the y-axis.
Val::Percent values are resolved based on the computed height of the UI Node.
Val::Auto is resolved to 0..
Implementations
Sourceimpl Val2
impl Val2
pub const ZERO: Val2
Sourcepub fn px<X, Y>(x: X, y: Y) -> Val2
pub fn px<X, Y>(x: X, y: Y) -> Val2
Creates a new Val2 where both components are in logical pixels
Examples found in repository?
74fn translation_system(
75 time: Res<Time>,
76 input: Res<ButtonInput<KeyCode>>,
77 mut translation_query: Query<&mut UiTransform, With<TargetNode>>,
78) {
79 let controls = [
80 (KeyCode::ArrowLeft, -Vec2::X),
81 (KeyCode::ArrowRight, Vec2::X),
82 (KeyCode::ArrowUp, -Vec2::Y),
83 (KeyCode::ArrowDown, Vec2::Y),
84 ];
85 for &(key_code, direction) in &controls {
86 if input.pressed(key_code) {
87 for mut transform in translation_query.iter_mut() {
88 let d = direction * 50.0 * time.delta_secs();
89 let (Val::Px(x), Val::Px(y)) = (transform.translation.x, transform.translation.y)
90 else {
91 continue;
92 };
93 let x = (x + d.x).clamp(-150., 150.);
94 let y = (y + d.y).clamp(-150., 150.);
95
96 transform.translation = Val2::px(x, y);
97 }
98 }
99 }
100}More examples
1384 pub fn setup(mut commands: Commands) {
1385 commands.spawn((Camera2d, DespawnOnExit(super::Scene::Transformations)));
1386 commands
1387 .spawn((
1388 Node {
1389 width: percent(100),
1390 height: percent(100),
1391 display: Display::Block,
1392 ..default()
1393 },
1394 DespawnOnExit(super::Scene::Transformations),
1395 ))
1396 .with_children(|parent| {
1397 for (transformation, label, background) in [
1398 (
1399 UiTransform::from_rotation(Rot2::degrees(45.)),
1400 "Rotate 45 degrees",
1401 RED,
1402 ),
1403 (
1404 UiTransform::from_scale(Vec2::new(2., 0.5)),
1405 "Scale 2.x 0.5y",
1406 GREEN,
1407 ),
1408 (
1409 UiTransform::from_translation(Val2::px(-50., 50.)),
1410 "Translate -50px x +50px y",
1411 BLUE,
1412 ),
1413 (
1414 UiTransform {
1415 translation: Val2::px(50., 0.),
1416 scale: Vec2::new(-1., 1.),
1417 rotation: Rot2::degrees(30.),
1418 },
1419 "T 50px x\nS -1.x (refl)\nR 30deg",
1420 DARK_CYAN,
1421 ),
1422 ] {
1423 parent
1424 .spawn((Node {
1425 width: percent(100),
1426 margin: UiRect {
1427 top: px(50),
1428 bottom: px(50),
1429 ..default()
1430 },
1431 align_items: AlignItems::Center,
1432 justify_content: JustifyContent::SpaceAround,
1433 ..default()
1434 },))
1435 .with_children(|row| {
1436 row.spawn((
1437 Text::new("Before Tf"),
1438 Node {
1439 width: px(100),
1440 height: px(100),
1441 border_radius: BorderRadius::bottom_right(px(25.)),
1442 ..default()
1443 },
1444 BackgroundColor(background.into()),
1445 TextFont::default(),
1446 ));
1447 row.spawn((
1448 Text::new(label),
1449 Node {
1450 width: px(100),
1451 height: px(100),
1452 border_radius: BorderRadius::bottom_right(px(25.)),
1453 ..default()
1454 },
1455 BackgroundColor(background.into()),
1456 transformation,
1457 TextFont::default(),
1458 ));
1459 });
1460 }
1461 });
1462 }16fn setup(mut commands: Commands) {
17 commands.spawn(Camera2d);
18 commands
19 .spawn((Node {
20 display: Display::Grid,
21 align_self: AlignSelf::Center,
22 justify_self: JustifySelf::Center,
23 ..Default::default()
24 }, Pickable::IGNORE, BackgroundColor(Color::srgb(0.4, 0.4, 0.4))))
25 .with_children(|parent| {
26 let tile_colors = [
27 Color::srgb(0.2, 0.2, 0.8),
28 Color::srgb(0.8, 0.2, 0.2)
29 ];
30 for column in 0..COLUMNS {
31 for row in 0..ROWS {
32 let i = column + row * COLUMNS;
33 let tile_color = tile_colors[((row % 2) + column) as usize % tile_colors.len()];
34 let tile_border_color = tile_color.darker(0.025);
35 parent
36 .spawn((
37 Node {
38 width: px(TILE_SIZE),
39 height: px(TILE_SIZE),
40 border: px(4.).all(),
41 grid_row: GridPlacement::start(row + 1),
42 grid_column: GridPlacement::start(column + 1),
43 align_items: AlignItems::Center,
44 justify_content: JustifyContent::Center,
45 ..Default::default()
46 },
47 BorderColor::all(tile_border_color),
48 BackgroundColor(tile_color),
49 Outline {
50 width: px(2.),
51 offset: Val::ZERO,
52 color: Color::NONE,
53 },
54 Pickable {
55 should_block_lower: false,
56 is_hoverable: true,
57 },
58 GlobalZIndex::default()
59 ))
60 .observe(move |on_over: On<Pointer<Over>>, mut query: Query<(&mut BackgroundColor, &mut BorderColor)>| {
61 if let Ok((mut background_color, mut border_color)) = query.get_mut(on_over.event_target()) {
62 background_color.0 = tile_color.lighter(0.1);
63 border_color.set_all(tile_border_color.lighter(0.1));
64 }
65 })
66 .observe(move |on_out: On<Pointer<Out>>, mut query: Query<(&mut BackgroundColor, &mut BorderColor)>| {
67 if let Ok((mut background_color, mut border_color)) = query.get_mut(on_out.event_target()) {
68 background_color.0 = tile_color;
69 border_color.set_all(tile_border_color);
70 }
71 })
72 .observe(|on_drag_start: On<Pointer<DragStart>>, mut query: Query<(&mut Outline, &mut GlobalZIndex)>| {
73 if let Ok((mut outline, mut global_zindex, )) = query.get_mut(on_drag_start.event_target()) {
74 outline.color = Color::WHITE;
75 global_zindex.0 = 1;
76 }
77 })
78 .observe(|on_drag: On<Pointer<Drag>>, mut query: Query<&mut UiTransform>| {
79 if let Ok(mut transform) = query.get_mut(on_drag.event_target()) {
80 transform.translation = Val2::px(on_drag.distance.x, on_drag.distance.y);
81 }
82 })
83 .observe(move |on_drag_end: On<Pointer<DragEnd>>, mut query: Query<(&mut UiTransform, &mut Outline, &mut GlobalZIndex)>| {
84 if let Ok((mut transform, mut outline, mut global_zindex)) = query.get_mut(on_drag_end.event_target()) {
85 transform.translation = Val2::ZERO;
86 outline.color = Color::NONE;
87 global_zindex.0 = 0;
88 }
89 })
90 .observe(|on_drag_drop: On<Pointer<DragDrop>>, mut query: Query<&mut Node>| {
91 if let Ok([mut a, mut b]) = query.get_many_mut([on_drag_drop.event_target(), on_drag_drop.dropped]) {
92 core::mem::swap(&mut a.grid_row, &mut b.grid_row);
93 core::mem::swap(&mut a.grid_column, &mut b.grid_column);
94 }
95 })
96 .with_child((Text::new(format!("{i}")), Pickable::IGNORE));
97 }
98 }
99 });
100}Sourcepub fn percent<X, Y>(x: X, y: Y) -> Val2
pub fn percent<X, Y>(x: X, y: Y) -> Val2
Creates a new Val2 where both components are percentage values
Sourcepub fn try_add(self, other: Val2) -> Result<Val2, ValArithmeticError>
pub fn try_add(self, other: Val2) -> Result<Val2, ValArithmeticError>
Try to add two Val2s component-wise.
Returns ValArithmeticError::IncompatibleUnits if either component has mismatched units.
assert_eq!(Val2::px(1., 2.).try_add(Val2::px(3., 4.)), Ok(Val2::px(4., 6.)));
assert_eq!(
Val2::new(Val::Px(1.), Val::Px(2.)).try_add(Val2::new(Val::Percent(3.), Val::Px(4.))),
Err(ValArithmeticError::IncompatibleUnits)
);Sourcepub fn try_sub(self, other: Val2) -> Result<Val2, ValArithmeticError>
pub fn try_sub(self, other: Val2) -> Result<Val2, ValArithmeticError>
Try to subtract two Val2s component-wise.
Returns ValArithmeticError::IncompatibleUnits if either component has mismatched units.
assert_eq!(Val2::px(3., 4.).try_sub(Val2::px(1., 2.)), Ok(Val2::px(2., 2.)));
assert_eq!(
Val2::new(Val::Px(1.), Val::Px(2.)).try_sub(Val2::new(Val::Percent(3.), Val::Px(4.))),
Err(ValArithmeticError::IncompatibleUnits)
);Trait Implementations
impl Copy for Val2
Sourceimpl<'de> Deserialize<'de> for Val2
impl<'de> Deserialize<'de> for Val2
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<Val2, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<Val2, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl FromReflect for Val2
impl FromReflect for Val2
Sourcefn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Val2>
fn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Val2>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetTypeRegistration for Val2
impl GetTypeRegistration for Val2
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for Val2
impl IntoReturn for Val2
Sourceimpl PartialReflect for Val2
impl PartialReflect for Val2
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<Val2>) -> ReflectOwned
fn reflect_owned(self: Box<Val2>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<Val2>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<Val2>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<Val2>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<Val2>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for Val2
impl Reflect for Val2
Sourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<Val2>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Val2>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl Serialize for Val2
impl Serialize for Val2
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Sourceimpl Struct for Val2
impl Struct for Val2
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.impl StructuralPartialEq for Val2
Sourceimpl TypePath for Val2
impl TypePath for Val2
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>