ViewVisibility

Struct ViewVisibility 

pub struct ViewVisibility(/* private fields */);
Algorithmically computed indication of whether an entity is visible and should be extracted for rendering.

Not to be confused with Visibility and InheritedVisibility. For more information, see module level documentation.

If you wish to add a custom visibility system that sets this value, be sure to add it to the CheckVisibility set.

Implementations

Source

impl ViewVisibility

pub const HIDDEN: ViewVisibility

An entity that cannot be seen from any views.

pub const VISIBLE: ViewVisibility

An entity that is visible and was visible last frame.

Entities that have opted out of CPU culling, if not hidden due to inherited visibility, will have their ViewVisibility set to this value.

pub fn get(self) -> bool

Returns true if the entity is visible in any view. Otherwise, returns false.

Examples found in repository?
examples/stress_tests/many_cubes.rs (line 574)
563fn print_mesh_count(
564    time: Res<Time>,
565    mut timer: Local<PrintingTimer>,
566    sprites: Query<(&Mesh3d, &ViewVisibility)>,
567) {
568    timer.tick(time.delta());
569
570    if timer.just_finished() {
571        info!(
572            "Meshes: {} - Visible Meshes {}",
573            sprites.iter().len(),
574            sprites.iter().filter(|(_, vis)| vis.get()).count(),
575        );
576    }
577}
More examples
Hide additional examples
examples/math/render_primitives.rs (line 642)
632fn rotate_primitive_2d_meshes(
633    mut primitives_2d: Query<
634        (&mut Transform, &ViewVisibility),
635        (With<PrimitiveData>, With<MeshDim2>),
636    >,
637    time: Res<Time>,
638) {
639    let rotation_2d = Quat::from_mat3(&Mat3::from_angle(time.elapsed_secs()));
640    primitives_2d
641        .iter_mut()
642        .filter(|(_, vis)| vis.get())
643        .for_each(|(mut transform, _)| {
644            transform.rotation = rotation_2d;
645        });
646}
647
648fn rotate_primitive_3d_meshes(
649    mut primitives_3d: Query<
650        (&mut Transform, &ViewVisibility),
651        (With<PrimitiveData>, With<MeshDim3>),
652    >,
653    time: Res<Time>,
654) {
655    let rotation_3d = Quat::from_rotation_arc(
656        Vec3::Z,
657        Vec3::new(
658            ops::sin(time.elapsed_secs()),
659            ops::cos(time.elapsed_secs()),
660            ops::sin(time.elapsed_secs()) * 0.5,
661        )
662        .try_normalize()
663        .unwrap_or(Vec3::Z),
664    );
665    primitives_3d
666        .iter_mut()
667        .filter(|(_, vis)| vis.get())
668        .for_each(|(mut transform, _)| {
669            transform.rotation = rotation_3d;
670        });
671}
examples/stress_tests/many_text2d.rs (line 198)
179fn print_counts(
180    time: Res<Time>,
181    mut timer: Local<PrintingTimer>,
182    texts: Query<&ViewVisibility, With<Text2d>>,
183    font_atlas_set: Res<FontAtlasSet>,
184    images: Res<Assets<Image>>,
185) {
186    timer.tick(time.delta());
187    if !timer.just_finished() {
188        return;
189    }
190
191    let num_atlases = font_atlas_set
192        .iter()
193        // Removed this filter for now as the keys no longer include the AssetIds
194        //        .filter(|(key, _)| key.0 == font_id)
195        .map(|(_, atlases)| atlases.len())
196        .sum::<usize>();
197
198    let visible_texts = texts.iter().filter(|visibility| visibility.get()).count();
199
200    info!(
201        "Texts: {} Visible: {} Atlases: {} Bytes: {}",
202        texts.iter().count(),
203        visible_texts,
204        num_atlases,
205        font_atlas_set.total_bytes(images.as_ref())
206    );
207}
examples/shader_advanced/manual_material.rs (line 254)
242fn extract_image_materials(
243    mut material_instances: ResMut<RenderMaterialInstances>,
244    changed_meshes_query: Extract<
245        Query<
246            (Entity, &ViewVisibility, &ImageMaterial3d),
247            Or<(Changed<ViewVisibility>, Changed<ImageMaterial3d>)>,
248        >,
249    >,
250) {
251    let last_change_tick = material_instances.current_change_tick;
252
253    for (entity, view_visibility, material) in &changed_meshes_query {
254        if view_visibility.get() {
255            material_instances.instances.insert(
256                entity.into(),
257                RenderMaterialInstance {
258                    asset_id: material.0.id().untyped(),
259                    last_change_tick,
260                },
261            );
262        } else {
263            material_instances
264                .instances
265                .remove(&MainEntity::from(entity));
266        }
267    }
268}
examples/2d/mesh2d_manual.rs (line 353)
332pub fn extract_colored_mesh2d(
333    mut commands: Commands,
334    mut previous_len: Local<usize>,
335    // When extracting, you must use `Extract` to mark the `SystemParam`s
336    // which should be taken from the main world.
337    query: Extract<
338        Query<
339            (
340                Entity,
341                RenderEntity,
342                &ViewVisibility,
343                &GlobalTransform,
344                &Mesh2d,
345            ),
346            With<ColoredMesh2d>,
347        >,
348    >,
349    mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
350) {
351    let mut values = Vec::with_capacity(*previous_len);
352    for (entity, render_entity, view_visibility, transform, handle) in &query {
353        if !view_visibility.get() {
354            continue;
355        }
356
357        let transforms = Mesh2dTransforms {
358            world_from_local: transform.affine().into(),
359            flags: MeshFlags::empty().bits(),
360        };
361
362        values.push((render_entity, ColoredMesh2d));
363        render_mesh_instances.insert(
364            entity.into(),
365            RenderMesh2dInstance {
366                mesh_asset_id: handle.0.id(),
367                transforms,
368                material_bind_group_id: Material2dBindGroupId::default(),
369                automatic_batching: false,
370                tag: 0,
371            },
372        );
373    }
374    *previous_len = values.len();
375    commands.try_insert_batch(values);
376}