pub struct WithOneRelated(pub Entity);Expand description
A wrapper over an Entity indicating that an entity should be added.
This is intended to be used for hierarchical spawning via traits like SpawnableList and SpawnRelated.
Unlike WithRelated this only adds one entity.
Also see the children and related macros that abstract over the Spawn API.
let mut world = World::new();
let child1 = world.spawn(Name::new("Child1")).id();
world.spawn((
Name::new("Root"),
Children::spawn((
// This adds the already existing entity as a child of Root.
WithOneRelated(child1),
)),
));Tuple Fields
0: EntityTrait Implementations
Sourceimpl<R> SpawnableList<R> for WithOneRelatedwhere
R: Relationship,
impl<R> SpawnableList<R> for WithOneRelatedwhere
R: Relationship,
Sourcefn size_hint(&self) -> usize
fn size_hint(&self) -> usize
Returns a size hint, which is used to reserve space for this list in a
RelationshipTarget. This should be
less than or equal to the actual size of the list. When in doubt, just use 0.Auto Trait Implementations
impl Freeze for WithOneRelated
impl RefUnwindSafe for WithOneRelated
impl Send for WithOneRelated
impl Sync for WithOneRelated
impl Unpin for WithOneRelated
impl UnwindSafe for WithOneRelated