pub struct WorldInstanceSpawner { /* private fields */ }Expand description
Handles spawning and despawning world instances, either synchronously or batched through the world_instance_spawner_system.
Synchronous methods: (WorldAsset operations will take effect immediately)
spawn_syncspawn_dynamic_syncdespawn_syncdespawn_dynamic_syncdespawn_instance_syncupdate_spawned_instancesupdate_spawned_dynamic_worldsspawn_queued_instancesdespawn_queued_world_asset_instancesdespawn_queued_dynamic_world_instances
Deferred methods: (WorldAsset operations will be processed when the world_instance_spawner_system is run)
Implementations
Sourceimpl WorldInstanceSpawner
impl WorldInstanceSpawner
Sourcepub fn spawn_dynamic(
&mut self,
id: impl Into<Handle<DynamicWorld>>,
) -> InstanceId
pub fn spawn_dynamic( &mut self, id: impl Into<Handle<DynamicWorld>>, ) -> InstanceId
Schedule the spawn of a new instance of the provided dynamic world.
Sourcepub fn spawn_dynamic_as_child(
&mut self,
id: impl Into<Handle<DynamicWorld>>,
parent: Entity,
) -> InstanceId
pub fn spawn_dynamic_as_child( &mut self, id: impl Into<Handle<DynamicWorld>>, parent: Entity, ) -> InstanceId
Schedule the spawn of a new instance of the provided dynamic world as a child of parent.
Sourcepub fn spawn(&mut self, id: impl Into<Handle<WorldAsset>>) -> InstanceId
pub fn spawn(&mut self, id: impl Into<Handle<WorldAsset>>) -> InstanceId
Schedule the spawn of a new instance of the provided world asset.
Sourcepub fn spawn_as_child(
&mut self,
id: impl Into<Handle<WorldAsset>>,
parent: Entity,
) -> InstanceId
pub fn spawn_as_child( &mut self, id: impl Into<Handle<WorldAsset>>, parent: Entity, ) -> InstanceId
Schedule the spawn of a new instance of the provided world asset as a child of parent.
Sourcepub fn despawn(&mut self, id: impl Into<AssetId<WorldAsset>>)
pub fn despawn(&mut self, id: impl Into<AssetId<WorldAsset>>)
Schedule the despawn of all instances of the provided world asset.
Sourcepub fn despawn_dynamic(&mut self, id: impl Into<AssetId<DynamicWorld>>)
pub fn despawn_dynamic(&mut self, id: impl Into<AssetId<DynamicWorld>>)
Schedule the despawn of all instances of the provided dynamic world.
Sourcepub fn despawn_instance(&mut self, instance_id: InstanceId)
pub fn despawn_instance(&mut self, instance_id: InstanceId)
Schedule the despawn of a world asset instance, removing all its entities from the world.
Note: this will despawn all entities associated with this instance, including those that have been removed from the hierarchy. To despawn only entities still in the hierarchy, despawn the relevant root entity directly.
Sourcepub fn unregister_instance(&mut self, instance_id: InstanceId)
pub fn unregister_instance(&mut self, instance_id: InstanceId)
This will remove all records of this instance, without despawning any entities.
Sourcepub fn despawn_sync(
&mut self,
world: &mut World,
id: impl Into<AssetId<WorldAsset>>,
) -> Result<(), WorldInstanceSpawnError>
pub fn despawn_sync( &mut self, world: &mut World, id: impl Into<AssetId<WorldAsset>>, ) -> Result<(), WorldInstanceSpawnError>
Immediately despawns all instances of a world asset.
Sourcepub fn despawn_dynamic_sync(
&mut self,
world: &mut World,
id: impl Into<AssetId<DynamicWorld>>,
) -> Result<(), WorldInstanceSpawnError>
pub fn despawn_dynamic_sync( &mut self, world: &mut World, id: impl Into<AssetId<DynamicWorld>>, ) -> Result<(), WorldInstanceSpawnError>
Immediately despawns all instances of a dynamic world.
Sourcepub fn despawn_instance_sync(
&mut self,
world: &mut World,
instance_id: &InstanceId,
)
pub fn despawn_instance_sync( &mut self, world: &mut World, instance_id: &InstanceId, )
Immediately despawns a world asset instance, removing all its entities from the world.
Sourcepub fn spawn_dynamic_sync(
&mut self,
world: &mut World,
id: impl Into<AssetId<DynamicWorld>>,
) -> Result<InstanceId, WorldInstanceSpawnError>
pub fn spawn_dynamic_sync( &mut self, world: &mut World, id: impl Into<AssetId<DynamicWorld>>, ) -> Result<InstanceId, WorldInstanceSpawnError>
Immediately spawns a new instance of the provided dynamic world.
Sourcepub fn spawn_sync(
&mut self,
world: &mut World,
id: impl Into<AssetId<WorldAsset>>,
) -> Result<InstanceId, WorldInstanceSpawnError>
pub fn spawn_sync( &mut self, world: &mut World, id: impl Into<AssetId<WorldAsset>>, ) -> Result<InstanceId, WorldInstanceSpawnError>
Immediately spawns a new instance of the provided world asset.
Sourcepub fn update_spawned_instances(
&mut self,
world: &mut World,
world_asset_ids: &[AssetId<WorldAsset>],
) -> Result<(), WorldInstanceSpawnError>
pub fn update_spawned_instances( &mut self, world: &mut World, world_asset_ids: &[AssetId<WorldAsset>], ) -> Result<(), WorldInstanceSpawnError>
Iterate through all instances of the provided worlds and update those immediately.
Useful for updating already spawned world asset instances after their corresponding world asset has been modified.
Sourcepub fn update_spawned_dynamic_worlds(
&mut self,
world: &mut World,
dynamic_world_ids: &[AssetId<DynamicWorld>],
) -> Result<(), WorldInstanceSpawnError>
pub fn update_spawned_dynamic_worlds( &mut self, world: &mut World, dynamic_world_ids: &[AssetId<DynamicWorld>], ) -> Result<(), WorldInstanceSpawnError>
Iterate through all instances of the provided dynamic worlds and update those immediately.
Useful for updating already spawned world asset instances after their corresponding dynamic world has been modified.
Sourcepub fn despawn_queued_world_asset_instances(
&mut self,
world: &mut World,
) -> Result<(), WorldInstanceSpawnError>
pub fn despawn_queued_world_asset_instances( &mut self, world: &mut World, ) -> Result<(), WorldInstanceSpawnError>
Immediately despawns all instances scheduled for despawn by despawning their instances.
Sourcepub fn despawn_queued_dynamic_world_instances(&mut self, world: &mut World)
pub fn despawn_queued_dynamic_world_instances(&mut self, world: &mut World)
Immediately despawns all world asset instances scheduled for despawn.
Sourcepub fn spawn_queued_instances(
&mut self,
world: &mut World,
) -> Result<(), WorldInstanceSpawnError>
pub fn spawn_queued_instances( &mut self, world: &mut World, ) -> Result<(), WorldInstanceSpawnError>
Immediately spawns all instances scheduled for spawn.
Sourcepub fn instance_is_ready(&self, instance_id: InstanceId) -> bool
pub fn instance_is_ready(&self, instance_id: InstanceId) -> bool
Check that a world asset instance spawned previously is ready to use
Sourcepub fn iter_instance_entities(
&self,
instance_id: InstanceId,
) -> impl Iterator<Item = Entity>
pub fn iter_instance_entities( &self, instance_id: InstanceId, ) -> impl Iterator<Item = Entity>
Get an iterator over the entities in an instance, once it’s spawned.
Before the world asset is spawned, the iterator will be empty. Use Self::instance_is_ready
to check if the instance is ready.