AnimationCompatibleCurve

Trait AnimationCompatibleCurve 

pub trait AnimationCompatibleCurve<T>:
    Curve<T>
    + Debug
    + Clone
    + Reflectable { }
This trait collects the additional requirements on top of Curve<T> needed for a curve to be used as an AnimationCurve.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<T, C> AnimationCompatibleCurve<T> for C
where C: Curve<T> + Debug + Clone + Reflectable,