pub enum ReflectCloneError {
NotImplemented {
type_path: Cow<'static, str>,
},
NotCloneable {
type_path: Cow<'static, str>,
},
FieldNotCloneable {
field: FieldId,
variant: Option<Cow<'static, str>>,
container_type_path: Cow<'static, str>,
},
FailedDowncast {
expected: Cow<'static, str>,
received: Cow<'static, str>,
},
}Expand description
An error that occurs when cloning a type via PartialReflect::reflect_clone.
Variants
NotImplemented
The type does not have a custom implementation for PartialReflect::reflect_clone.
Fields
type_path: Cow<'static, str>The fully qualified path of the type that PartialReflect::reflect_clone is not implemented for.
NotCloneable
The type cannot be cloned via PartialReflect::reflect_clone.
This type should be returned when a type is intentionally opting out of reflection cloning.
Fields
type_path: Cow<'static, str>The fully qualified path of the type that cannot be cloned via PartialReflect::reflect_clone.
FieldNotCloneable
The field cannot be cloned via PartialReflect::reflect_clone.
When deriving Reflect, this usually means that a field marked with #[reflect(ignore)]
is missing a #[reflect(clone)] attribute.
This may be intentional if the field is not meant/able to be cloned.
Fields
FailedDowncast
Could not downcast to the expected type.
Realistically this should only occur when a type has incorrectly implemented Reflect.