pub enum ReflectMut<'a> {
Struct(&'a mut (dyn Struct + 'static)),
TupleStruct(&'a mut (dyn TupleStruct + 'static)),
Tuple(&'a mut (dyn Tuple + 'static)),
List(&'a mut (dyn List + 'static)),
Array(&'a mut (dyn Array + 'static)),
Map(&'a mut (dyn Map + 'static)),
Set(&'a mut (dyn Set + 'static)),
Enum(&'a mut (dyn Enum + 'static)),
Function(&'a mut (dyn Function + 'static)),
Opaque(&'a mut (dyn PartialReflect + 'static)),
}Expand description
A mutable enumeration of “kinds” of a reflected type.
Each variant contains a trait object with methods specific to a kind of type.
A ReflectMut is obtained via PartialReflect::reflect_mut.
Variants
Struct(&'a mut (dyn Struct + 'static))
A mutable reference to a struct-like type.
TupleStruct(&'a mut (dyn TupleStruct + 'static))
A mutable reference to a tuple-struct-like type.
Tuple(&'a mut (dyn Tuple + 'static))
A mutable reference to a tuple-like type.
List(&'a mut (dyn List + 'static))
A mutable reference to a list-like type.
Array(&'a mut (dyn Array + 'static))
A mutable reference to an array-like type.
Map(&'a mut (dyn Map + 'static))
A mutable reference to a map-like type.
Set(&'a mut (dyn Set + 'static))
A mutable reference to a set-like type.
Enum(&'a mut (dyn Enum + 'static))
A mutable reference to an enum-like type.
Function(&'a mut (dyn Function + 'static))
functions only.A mutable reference to a function-like type.
Opaque(&'a mut (dyn PartialReflect + 'static))
A mutable reference to an opaque type.
Implementations
Sourceimpl ReflectMut<'_>
impl ReflectMut<'_>
Sourcepub fn kind(&self) -> ReflectKind
pub fn kind(&self) -> ReflectKind
Returns the “kind” of this reflected type without any information.
Sourceimpl<'a> ReflectMut<'a>
impl<'a> ReflectMut<'a>
Sourcepub fn as_struct(
self,
) -> Result<&'a mut (dyn Struct + 'static), ReflectKindMismatchError>
pub fn as_struct( self, ) -> Result<&'a mut (dyn Struct + 'static), ReflectKindMismatchError>
Attempts a cast to a Struct trait object.
Returns an error if self is not the Self::Struct variant.
Sourcepub fn as_tuple_struct(
self,
) -> Result<&'a mut (dyn TupleStruct + 'static), ReflectKindMismatchError>
pub fn as_tuple_struct( self, ) -> Result<&'a mut (dyn TupleStruct + 'static), ReflectKindMismatchError>
Attempts a cast to a TupleStruct trait object.
Returns an error if self is not the Self::TupleStruct variant.
Sourcepub fn as_tuple(
self,
) -> Result<&'a mut (dyn Tuple + 'static), ReflectKindMismatchError>
pub fn as_tuple( self, ) -> Result<&'a mut (dyn Tuple + 'static), ReflectKindMismatchError>
Attempts a cast to a Tuple trait object.
Returns an error if self is not the Self::Tuple variant.
Sourcepub fn as_list(
self,
) -> Result<&'a mut (dyn List + 'static), ReflectKindMismatchError>
pub fn as_list( self, ) -> Result<&'a mut (dyn List + 'static), ReflectKindMismatchError>
Attempts a cast to a List trait object.
Returns an error if self is not the Self::List variant.
Sourcepub fn as_array(
self,
) -> Result<&'a mut (dyn Array + 'static), ReflectKindMismatchError>
pub fn as_array( self, ) -> Result<&'a mut (dyn Array + 'static), ReflectKindMismatchError>
Attempts a cast to a Array trait object.
Returns an error if self is not the Self::Array variant.
Sourcepub fn as_map(
self,
) -> Result<&'a mut (dyn Map + 'static), ReflectKindMismatchError>
pub fn as_map( self, ) -> Result<&'a mut (dyn Map + 'static), ReflectKindMismatchError>
Sourcepub fn as_set(
self,
) -> Result<&'a mut (dyn Set + 'static), ReflectKindMismatchError>
pub fn as_set( self, ) -> Result<&'a mut (dyn Set + 'static), ReflectKindMismatchError>
Sourcepub fn as_enum(
self,
) -> Result<&'a mut (dyn Enum + 'static), ReflectKindMismatchError>
pub fn as_enum( self, ) -> Result<&'a mut (dyn Enum + 'static), ReflectKindMismatchError>
Attempts a cast to a Enum trait object.
Returns an error if self is not the Self::Enum variant.
Sourcepub fn as_opaque(
self,
) -> Result<&'a mut (dyn PartialReflect + 'static), ReflectKindMismatchError>
pub fn as_opaque( self, ) -> Result<&'a mut (dyn PartialReflect + 'static), ReflectKindMismatchError>
Attempts a cast to a PartialReflect trait object.
Returns an error if self is not the Self::Opaque variant.