ReflectDeserializer

Struct ReflectDeserializer 

pub struct ReflectDeserializer<'a, P = ()>{ /* private fields */ }
A general purpose deserializer for reflected types.

This is the deserializer counterpart to ReflectSerializer.

See TypedReflectDeserializer for a deserializer that expects a known type.

Input

This deserializer expects a map with a single entry, where the key is the full type path of the reflected type and the value is the serialized data.

Output

This deserializer will return a Box<dyn Reflect> containing the deserialized data.

For opaque types (i.e. ReflectKind::Opaque) or types that register ReflectDeserialize type data, this Box will contain the expected type. For example, deserializing an i32 will return a Box<i32> (as a Box<dyn Reflect>).

Otherwise, this Box will contain the dynamic equivalent. For example, a deserialized struct might return a Box<DynamicStruct> and a deserialized Vec might return a Box<DynamicList>.

This means that if the actual type is needed, these dynamic representations will need to be converted to the concrete type using FromReflect or ReflectFromReflect.

If you want to override deserialization for a specific TypeRegistration, you can pass in a reference to a ReflectDeserializerProcessor which will take priority over all other deserialization methods - see with_processor.

Example

#[derive(Reflect, PartialEq, Debug)]
#[type_path = "my_crate"]
struct MyStruct {
  value: i32
}

let mut registry = TypeRegistry::default();
registry.register::<MyStruct>();

let input = r#"{
  "my_crate::MyStruct": (
    value: 123
  )
}"#;

let mut deserializer = ron::Deserializer::from_str(input).unwrap();
let reflect_deserializer = ReflectDeserializer::new(&registry);

let output: Box<dyn PartialReflect> = reflect_deserializer.deserialize(&mut deserializer).unwrap();

// Since `MyStruct` is not an opaque type and does not register `ReflectDeserialize`,
// we know that its deserialized value will be a `DynamicStruct`,
// although it will represent `MyStruct`.
assert!(output.as_partial_reflect().represents::<MyStruct>());

// We can convert back to `MyStruct` using `FromReflect`.
let value: MyStruct = <MyStruct as FromReflect>::from_reflect(output.as_partial_reflect()).unwrap();
assert_eq!(value, MyStruct { value: 123 });

// We can also do this dynamically with `ReflectFromReflect`.
let type_id = output.get_represented_type_info().unwrap().type_id();
let reflect_from_reflect = registry.get_type_data::<ReflectFromReflect>(type_id).unwrap();
let value: Box<dyn Reflect> = reflect_from_reflect.from_reflect(output.as_partial_reflect()).unwrap();
assert!(value.is::<MyStruct>());
assert_eq!(value.take::<MyStruct>().unwrap(), MyStruct { value: 123 });

Implementations

Source

impl<'a> ReflectDeserializer<'a>

pub fn new(registry: &'a TypeRegistry) -> ReflectDeserializer<'a>

Creates a deserializer with no processor.

If you want to add custom logic for deserializing certain types, use with_processor.

Examples found in repository?
examples/reflection/serialization.rs (line 43)
35fn deserialize(type_registry: Res<AppTypeRegistry>) {
36    let type_registry = type_registry.read();
37
38    // a serde_json::Value that might have come from an API
39    let value: serde_json::Value = serde_json::from_str(PLAYER_JSON).unwrap();
40
41    // alternatively, `TypedReflectDeserializer` can be used if the type
42    // is known.
43    let deserializer = ReflectDeserializer::new(&type_registry);
44    // deserialize
45    let reflect_value = deserializer.deserialize(value).unwrap();
46    // If Player implemented additional functionality, like Component,
47    // this reflect_value could be used with commands.insert_reflect
48    info!(?reflect_value);
49}
More examples
Hide additional examples
examples/reflection/reflection.rs (line 99)
55fn setup(type_registry: Res<AppTypeRegistry>) {
56    let mut value = Foo {
57        a: 1,
58        _ignored: NonReflectedValue { _a: 10 },
59        nested: Bar { b: 8 },
60    };
61
62    // You can set field values like this. The type must match exactly or this will fail.
63    *value.get_field_mut("a").unwrap() = 2usize;
64    assert_eq!(value.a, 2);
65    assert_eq!(*value.get_field::<usize>("a").unwrap(), 2);
66
67    // You can also get the `&dyn PartialReflect` value of a field like this
68    let field = value.field("a").unwrap();
69
70    // But values introspected via `PartialReflect` will not return `dyn Reflect` trait objects
71    // (even if the containing type does implement `Reflect`), so we need to convert them:
72    let fully_reflected_field = field.try_as_reflect().unwrap();
73
74    // Now, you can downcast your `Reflect` value like this:
75    assert_eq!(*fully_reflected_field.downcast_ref::<usize>().unwrap(), 2);
76
77    // For this specific case, we also support the shortcut `try_downcast_ref`:
78    assert_eq!(*field.try_downcast_ref::<usize>().unwrap(), 2);
79
80    // `DynamicStruct` also implements the `Struct` and `Reflect` traits.
81    let mut patch = DynamicStruct::default();
82    patch.insert("a", 4usize);
83
84    // You can "apply" Reflect implementations on top of other Reflect implementations.
85    // This will only set fields with the same name, and it will fail if the types don't match.
86    // You can use this to "patch" your types with new values.
87    value.apply(&patch);
88    assert_eq!(value.a, 4);
89
90    let type_registry = type_registry.read();
91    // By default, all derived `Reflect` types can be Serialized using serde. No need to derive
92    // Serialize!
93    let serializer = ReflectSerializer::new(&value, &type_registry);
94    let ron_string =
95        ron::ser::to_string_pretty(&serializer, ron::ser::PrettyConfig::default()).unwrap();
96    info!("{}\n", ron_string);
97
98    // Dynamic properties can be deserialized
99    let reflect_deserializer = ReflectDeserializer::new(&type_registry);
100    let mut deserializer = ron::de::Deserializer::from_str(&ron_string).unwrap();
101    let reflect_value = reflect_deserializer.deserialize(&mut deserializer).unwrap();
102
103    // Deserializing returns a `Box<dyn PartialReflect>` value.
104    // Generally, deserializing a value will return the "dynamic" variant of a type.
105    // For example, deserializing a struct will return the DynamicStruct type.
106    // "Opaque types" will be deserialized as themselves.
107    assert_eq!(
108        reflect_value.reflect_type_path(),
109        DynamicStruct::type_path(),
110    );
111
112    // Reflect has its own `partial_eq` implementation, named `reflect_partial_eq`. This behaves
113    // like normal `partial_eq`, but it treats "dynamic" and "non-dynamic" types the same. The
114    // `Foo` struct and deserialized `DynamicStruct` are considered equal for this reason:
115    assert!(reflect_value.reflect_partial_eq(&value).unwrap());
116
117    // By "patching" `Foo` with the deserialized DynamicStruct, we can "Deserialize" Foo.
118    // This means we can serialize and deserialize with a single `Reflect` derive!
119    value.apply(&*reflect_value);
120}
Source

impl<'a, P> ReflectDeserializer<'a, P>

pub fn with_processor( registry: &'a TypeRegistry, processor: &'a mut P, ) -> ReflectDeserializer<'a, P>

Creates a deserializer with a processor.

If you do not need any custom logic for handling certain types, use new.