pub struct Type { /* private fields */ }Expand description
The base representation of a Rust type.
When possible, it is recommended to use &'static TypeInfo instead of this
as it provides more information as well as being smaller
(since a reference only takes the same number of bytes as a usize).
However, where a static reference to TypeInfo is not possible,
such as with trait objects and other types that can’t implement Typed,
this type can be used instead.
It only requires that the type implements TypePath.
And unlike TypeInfo, this type implements Copy, Eq, and Hash,
making it useful as a key type.
It’s especially helpful when compared to TypeId as it can provide the
actual [type path] when debugging, while still having the same performance
as hashing/comparing TypeId directly—at the cost of a little more memory.
Examples
use bevy_reflect::{Type, TypePath};
fn assert_char<T: ?Sized + TypePath>(t: &T) -> Result<(), String> {
let ty = Type::of::<T>();
if Type::of::<char>() == ty {
Ok(())
} else {
Err(format!("expected `char`, got `{}`", ty.path()))
}
}
assert_eq!(
assert_char(&'a'),
Ok(())
);
assert_eq!(
assert_char(&String::from("Hello, world!")),
Err(String::from("expected `char`, got `alloc::string::String`"))
);Implementations
Sourceimpl Type
impl Type
Sourcepub fn path(&self) -> &'static str
pub fn path(&self) -> &'static str
See TypePath::type_path.
Sourcepub fn short_path(&self) -> &'static str
pub fn short_path(&self) -> &'static str
Sourcepub fn ident(&self) -> Option<&'static str>
pub fn ident(&self) -> Option<&'static str>
See TypePath::type_ident.
Sourcepub fn crate_name(&self) -> Option<&'static str>
pub fn crate_name(&self) -> Option<&'static str>
See TypePath::crate_name.
Sourcepub fn module_path(&self) -> Option<&'static str>
pub fn module_path(&self) -> Option<&'static str>
Sourcepub fn type_path_table(&self) -> &TypePathTable
pub fn type_path_table(&self) -> &TypePathTable
A representation of the type path of this.
Provides dynamic access to all methods on TypePath.