TypeParamInfo

Struct TypeParamInfo 

pub struct TypeParamInfo { /* private fields */ }
Type information for a generic type parameter.

An example of a type parameter would be T in struct Foo<T>.

Implementations

Source

impl TypeParamInfo

pub fn new<T>(name: impl Into<Cow<'static, str>>) -> TypeParamInfo
where T: TypePath + ?Sized,

Creates a new type parameter with the given name.

pub fn with_default<T>(self) -> TypeParamInfo
where T: TypePath + ?Sized,

Sets the default type for the parameter.

pub fn name(&self) -> &Cow<'static, str>

The name of the type parameter.

pub fn default(&self) -> Option<&Type>

The default type for the parameter, if any.

Example
#[derive(Reflect)]
struct Foo<T = f32>(T);

let generics = Foo::<String>::type_info().generics();
let GenericInfo::Type(info) = generics.get_named("T").unwrap() else {
    panic!("expected a type parameter");
};

let default = info.default().unwrap();

assert!(default.is::<f32>());
pub fn ty(&self) -> &Type

The underlying Rust type.

pub fn type_id(&self) -> TypeId

The TypeId of this type.

pub fn type_path(&self) -> &'static str

The stable, full type path of this type.

Use type_path_table if you need access to the other methods on TypePath.

pub fn type_path_table(&self) -> &TypePathTable

A representation of the type path of this type.

Provides dynamic access to all methods on TypePath.

pub fn is<T>(&self) -> bool
where T: Any,

Check if the given type matches this one.

This only compares the TypeId of the types and does not verify they share the same TypePath (though it implies they do).