pub struct BrpJsonSchemaQueryFilter {
pub without_crates: Vec<String>,
pub with_crates: Vec<String>,
pub type_limit: JsonSchemaTypeLimit,
}Available on crate feature
bevy_remote only.Expand description
Constraints that can be placed on a query to include or exclude certain definitions.
Fields
without_crates: Vec<String>The crate name of the type name of each component that must not be present on the entity for it to be included in the results.
with_crates: Vec<String>The crate name of the type name of each component that must be present on the entity for it to be included in the results.
type_limit: JsonSchemaTypeLimitConstrain resource by type