Module http

Module http 

Available on crate feature http and non-target_family=wasm only.
The BRP transport using JSON-RPC over HTTP.

Adding the RemoteHttpPlugin to your App causes Bevy to accept connections over HTTP (by default, on port 15702) while your app is running.

Clients are expected to POST JSON requests to the root URL; see the client example for a trivial example of use.

Structs

Headers
A struct that holds a collection of HTTP headers.
HostAddress
A resource containing the IP address that Bevy will host on.
HostPort
A resource containing the port number that Bevy will listen on.
RemoteHttpPlugin
Add this plugin to your App to allow remote connections over HTTP to inspect and modify entities. It requires the RemotePlugin.

Constants

DEFAULT_ADDR
The default host address that Bevy will use for its server.
DEFAULT_PORT
The default port that Bevy will listen on.