RemoteWatchingMethodSystemId

Type Alias RemoteWatchingMethodSystemId 

pub type RemoteWatchingMethodSystemId = SystemId<In<Option<Value>>, Result<Option<Value>, BrpError>>;
Available on crate feature bevy_remote only.
The SystemId of a function that implements a remote watching method (world.get_components+watch, world.list_components+watch, etc.)

The first parameter is the JSON value of the params. Typically, an implementation will deserialize these as the first thing they do.

The optional returned JSON value will be sent as a response. If no changes were detected this should be None. Re-running of this handler is done in the RemotePlugin.

Aliased Type

pub struct RemoteWatchingMethodSystemId { /* private fields */ }