#[repr(C)]pub struct IndirectParametersNonIndexed {
pub vertex_count: u32,
pub instance_count: u32,
pub base_vertex: u32,
pub first_instance: u32,
}Expand description
The wgpu indirect parameters structure that specifies a GPU draw command.
This is the variant for non-indexed meshes. We generate the instances of
this structure in the build_indirect_params.wgsl compute shader.
Fields
vertex_count: u32The number of vertices that this mesh has.
instance_count: u32The number of instances we are to draw.
base_vertex: u32The offset of the first vertex for this mesh in the vertex buffer slab.
first_instance: u32The index of the first mesh instance in the Mesh buffer.