InstanceInputUniformBuffer

Struct InstanceInputUniformBuffer 

pub struct InstanceInputUniformBuffer<BDI>
where BDI: AtomicPod,
{ /* private fields */ }
Available on crate feature bevy_render only.
Holds the GPU buffer of instance input data, which is the data about each mesh instance that the CPU provides.

BDI is the buffer data input type, which the GPU mesh preprocessing shader is expected to expand to the full buffer data type.

Implementations

Source

impl<BDI> InstanceInputUniformBuffer<BDI>
where BDI: AtomicPod,

pub fn new() -> InstanceInputUniformBuffer<BDI>

Creates a new, empty buffer.

pub fn clear(&mut self)

Clears the buffer and entity list out.

pub fn buffer(&self) -> &AtomicSparseBufferVec<BDI>

Returns the AtomicSparseBufferVec corresponding to this input uniform buffer.

pub fn add(&mut self, element: BDI) -> u32

Adds a new piece of buffered data to the uniform buffer and returns its index.

pub fn remove(&mut self, uniform_index: u32)

Removes a piece of buffered data from the uniform buffer.

This simply marks the data as free.

pub fn get(&self, uniform_index: u32) -> Option<BDI>

Returns the piece of buffered data at the given index.

Returns None if the index is out of bounds or the data is removed.

pub fn get_unchecked(&self, uniform_index: u32) -> BDI

Returns the piece of buffered data at the given index. Can return data that has previously been removed.

Panics

if uniform_index is not in bounds of Self::buffer.

pub fn set(&self, uniform_index: u32, element: BDI)

Stores a piece of buffered data at the given index.

Panics

if uniform_index is not in bounds of Self::buffer.

pub fn ensure_nonempty(&mut self)

pub fn len(&self) -> usize

Returns the number of instances in this buffer.

pub fn is_empty(&self) -> bool

Returns true if this buffer has no instances or false if it contains any instances.

pub fn into_buffer(self) -> AtomicSparseBufferVec<BDI>

Consumes this InstanceInputUniformBuffer and returns the raw buffer ready to be uploaded to the GPU.