pub struct UntypedPhaseBatchedInstanceBuffers<BD>{
pub data_buffer: UninitBufferVec<BD>,
pub work_item_buffers: HashMap<RetainedViewEntity, PreprocessWorkItemBuffers>,
pub late_indexed_indirect_parameters_buffer: RawBufferVec<LatePreprocessWorkItemIndirectParameters>,
pub late_non_indexed_indirect_parameters_buffer: RawBufferVec<LatePreprocessWorkItemIndirectParameters>,
}Expand description
The GPU buffers holding the data needed to render batches for a single phase, without a type parameter for that phase.
Since this structure doesn’t have a type parameter, it can be placed in
BatchedInstanceBuffers::phase_instance_buffers.
Fields
data_buffer: UninitBufferVec<BD>A storage area for the buffer data that the GPU compute shader is expected to write to.
There will be one entry for each index.
work_item_buffers: HashMap<RetainedViewEntity, PreprocessWorkItemBuffers>The index of the buffer data in the current input buffer that corresponds to each instance.
This is keyed off each view. Each view has a separate buffer.
late_indexed_indirect_parameters_buffer: RawBufferVec<LatePreprocessWorkItemIndirectParameters>A buffer that holds the number of indexed meshes that weren’t visible in the previous frame, when GPU occlusion culling is in use.
There’s one set of LatePreprocessWorkItemIndirectParameters per
view. Bevy uses this value to determine how many threads to dispatch to
check meshes that weren’t visible next frame to see if they became newly
visible this frame.
late_non_indexed_indirect_parameters_buffer: RawBufferVec<LatePreprocessWorkItemIndirectParameters>A buffer that holds the number of non-indexed meshes that weren’t visible in the previous frame, when GPU occlusion culling is in use.
There’s one set of LatePreprocessWorkItemIndirectParameters per
view. Bevy uses this value to determine how many threads to dispatch to
check meshes that weren’t visible next frame to see if they became newly
visible this frame.
Implementations
Sourceimpl<BD> UntypedPhaseBatchedInstanceBuffers<BD>
impl<BD> UntypedPhaseBatchedInstanceBuffers<BD>
pub fn new() -> UntypedPhaseBatchedInstanceBuffers<BD>
Sourcepub fn instance_data_binding(&self) -> Option<BindingResource<'_>>
pub fn instance_data_binding(&self) -> Option<BindingResource<'_>>
Returns the binding of the buffer that contains the per-instance data.
This buffer needs to be filled in via a compute shader.