pub struct UntypedPhaseIndirectParametersBuffers {
pub indexed: MeshClassIndirectParametersBuffers<IndirectParametersIndexed>,
pub non_indexed: MeshClassIndirectParametersBuffers<IndirectParametersNonIndexed>,
}bevy_render only.Expand description
The buffers containing all the information that indirect draw commands use to draw the scene, for a single phase.
This is the version of the structure that doesn’t have a type parameter, so
that it can be inserted into IndirectParametersBuffers::buffers
See the IndirectParametersBuffers documentation for more information.
Fields
indexed: MeshClassIndirectParametersBuffers<IndirectParametersIndexed>Information that indirect draw commands use to draw indexed meshes in the scene.
non_indexed: MeshClassIndirectParametersBuffers<IndirectParametersNonIndexed>Information that indirect draw commands use to draw non-indexed meshes in the scene.
Implementations
Sourceimpl UntypedPhaseIndirectParametersBuffers
impl UntypedPhaseIndirectParametersBuffers
Sourcepub fn new(
allow_copies_from_indirect_parameter_buffers: bool,
) -> UntypedPhaseIndirectParametersBuffers
pub fn new( allow_copies_from_indirect_parameter_buffers: bool, ) -> UntypedPhaseIndirectParametersBuffers
Creates the indirect parameters buffers.
Sourcepub fn allocate(&mut self, indexed: bool, count: u32) -> u32
pub fn allocate(&mut self, indexed: bool, count: u32) -> u32
Reserves space for count new batches.
The indexed parameter specifies whether the meshes that these batches
correspond to are indexed or not.
Sourcepub fn batch_set_count(&self, indexed: bool) -> usize
pub fn batch_set_count(&self, indexed: bool) -> usize
Returns the number of batch sets currently allocated.
The indexed parameter specifies whether the meshes that these batch
sets correspond to are indexed or not.
Sourcepub fn add_batch_set(
&mut self,
indexed: bool,
retained_view_entity: &RetainedViewEntity,
indirect_parameters_range: Range<u32>,
)
pub fn add_batch_set( &mut self, indexed: bool, retained_view_entity: &RetainedViewEntity, indirect_parameters_range: Range<u32>, )
Adds a new batch set to Self::indexed_batch_sets or
Self::non_indexed_batch_sets as appropriate and updates the
view_to_indirect_parameters_batch_range.
indexed specifies whether the meshes that these batch sets correspond
to are indexed or not. retained_view_entity specifies the view that
this batch set is associated with. indirect_parameters_base specifies
the offset within Self::indexed_data or Self::non_indexed_data of
the first batch in this batch set.
Sourcepub fn get_next_batch_set_index(&self, indexed: bool) -> Option<NonMaxU32>
pub fn get_next_batch_set_index(&self, indexed: bool) -> Option<NonMaxU32>
Returns the index that a newly-added batch set will have.
The indexed parameter specifies whether the meshes in such a batch set
are indexed or not.