pub struct Viewport {
pub physical_position: UVec2,
pub physical_size: UVec2,
pub depth: Range<f32>,
}Expand description
Render viewport configuration for the Camera component.
The viewport defines the area on the render target to which the camera renders its image. You can overlay multiple cameras in a single window using viewports to create effects like split screen, minimaps, and character viewers.
Fields
physical_position: UVec2The physical position to render this viewport to within the RenderTarget of this Camera.
(0,0) corresponds to the top-left corner
physical_size: UVec2The physical size of the viewport rectangle to render to within the RenderTarget of this Camera.
The origin of the rectangle is in the top-left corner.
depth: Range<f32>The minimum and maximum depth to render (on a scale from 0.0 to 1.0).