pub struct PassSpanGuard<'a, R, P>where
R: ?Sized,{ /* private fields */ }Available on crate feature
bevy_render only.Expand description
Guard returned by RecordDiagnostics::pass_span.
Will panic on drop unless PassSpanGuard::end is called.
Implementations
Sourceimpl<R, P> PassSpanGuard<'_, R, P>where
R: RecordDiagnostics + ?Sized,
P: Pass,
impl<R, P> PassSpanGuard<'_, R, P>where
R: RecordDiagnostics + ?Sized,
P: Pass,
Sourcepub fn end(self, pass: &mut P)
pub fn end(self, pass: &mut P)
End the span. You have to provide the same pass which was used to begin the span.
Examples found in repository?
examples/2d/mesh2d_manual.rs (line 696)
637fn main_colored_transparent_pass_2d(
638 world: &World,
639 view: ViewQuery<(
640 &ExtractedCamera,
641 &ExtractedView,
642 &ViewTarget,
643 &ViewDepthStencilTexture,
644 )>,
645 transparent_phases: Res<ViewSortedRenderPhases<TransparentColoredMesh2d>>,
646 mut ctx: RenderContext,
647) {
648 let view_entity = view.entity();
649 let (camera, extracted_view, target, depth) = view.into_inner();
650
651 let Some(transparent_phase) = transparent_phases.get(&extracted_view.retained_view_entity)
652 else {
653 return;
654 };
655
656 #[cfg(feature = "trace")]
657 let _span = info_span!("main_colored_transparent_pass_2d").entered();
658
659 let diagnostics = ctx.diagnostic_recorder();
660 let diagnostics = diagnostics.as_deref();
661
662 let color_attachments = [Some(target.get_color_attachment())];
663 // NOTE: For the transparent pass we load the depth buffer. There should be no
664 // need to write to it, but store is set to `true` as a workaround for issue #3776,
665 // https://github.com/bevyengine/bevy/issues/3776
666 // so that wgpu does not clear the depth buffer.
667 // As the opaque and alpha mask passes run first, opaque meshes can occlude
668 // transparent ones.
669 let depth_stencil_attachment = Some(depth.get_attachment(StoreOp::Store));
670
671 {
672 let mut render_pass = ctx.begin_tracked_render_pass(RenderPassDescriptor {
673 label: Some("main_colored_transparent_pass_2d"),
674 color_attachments: &color_attachments,
675 depth_stencil_attachment,
676 timestamp_writes: None,
677 occlusion_query_set: None,
678 multiview_mask: None,
679 });
680 let pass_span = diagnostics.pass_span(&mut render_pass, "main_colored_transparent_pass_2d");
681
682 if let Some(viewport) = camera.viewport.as_ref() {
683 render_pass.set_camera_viewport(viewport);
684 }
685
686 if !transparent_phase.items.is_empty() {
687 #[cfg(feature = "trace")]
688 let _transparent_span = info_span!("colored_transparent_main_pass_2d").entered();
689 if let Err(err) = transparent_phase.render(&mut render_pass, world, view_entity) {
690 error!(
691 "Error encountered while rendering the colored transparent 2D phase {err:?}"
692 );
693 }
694 }
695
696 pass_span.end(&mut render_pass);
697 }
698}Trait Implementations
Sourceimpl<R, P> Drop for PassSpanGuard<'_, R, P>where
R: ?Sized,
impl<R, P> Drop for PassSpanGuard<'_, R, P>where
R: ?Sized,