pub struct DynamicUniformIndex<C>where
C: Component,{ /* private fields */ }Expand description
Stores the index of a uniform inside of ComponentUniforms.
Implementations
Sourceimpl<C> DynamicUniformIndex<C>where
C: Component,
impl<C> DynamicUniformIndex<C>where
C: Component,
Sourcepub fn index(&self) -> u32
pub fn index(&self) -> u32
Examples found in repository?
examples/shader_advanced/custom_post_processing.rs (line 162)
75fn post_process_system(
76 view: ViewQuery<(
77 &ViewTarget,
78 &PostProcessSettings,
79 &DynamicUniformIndex<PostProcessSettings>,
80 )>,
81 post_process_pipeline: Option<Res<PostProcessPipeline>>,
82 pipeline_cache: Res<PipelineCache>,
83 settings_uniforms: Res<ComponentUniforms<PostProcessSettings>>,
84 mut cache: Local<PostProcessBindGroupCache>,
85 mut ctx: RenderContext,
86) {
87 let Some(post_process_pipeline) = post_process_pipeline else {
88 return;
89 };
90
91 let (view_target, _post_process_settings, settings_index) = view.into_inner();
92
93 let Some(pipeline) = pipeline_cache.get_render_pipeline(post_process_pipeline.pipeline_id)
94 else {
95 return;
96 };
97
98 let Some(settings_binding) = settings_uniforms.uniforms().binding() else {
99 return;
100 };
101
102 // This will start a new "post process write", obtaining two texture
103 // views from the view target - a `source` and a `destination`.
104 // `source` is the "current" main texture and you _must_ write into
105 // `destination` because calling `post_process_write()` on the
106 // [`ViewTarget`] will internally flip the [`ViewTarget`]'s main
107 // texture to the `destination` texture. Failing to do so will cause
108 // the current main texture information to be lost.
109 let post_process = view_target.post_process_write();
110
111 let bind_group = match &mut cache.cached {
112 Some((texture_id, bind_group)) if post_process.source.id() == *texture_id => bind_group,
113 cached => {
114 // The bind_group gets created each frame.
115 //
116 // Normally, you would create a bind_group in the Queue set,
117 // but this doesn't work with the post_process_write().
118 // The reason it doesn't work is because each post_process_write will alternate the source/destination.
119 // The only way to have the correct source/destination for the bind_group
120 // is to make sure you get it during the node execution.
121 let bind_group = ctx.render_device().create_bind_group(
122 "post_process_bind_group",
123 &pipeline_cache.get_bind_group_layout(&post_process_pipeline.layout),
124 // It's important for this to match the BindGroupLayout defined in the PostProcessPipeline
125 &BindGroupEntries::sequential((
126 // Make sure to use the source view
127 post_process.source,
128 // Use the sampler created for the pipeline
129 &post_process_pipeline.sampler,
130 // Set the settings binding
131 settings_binding.clone(),
132 )),
133 );
134
135 let (_, bind_group) = cached.insert((post_process.source.id(), bind_group));
136 bind_group
137 }
138 };
139
140 let mut render_pass = ctx
141 .command_encoder()
142 .begin_render_pass(&RenderPassDescriptor {
143 label: Some("post_process_pass"),
144 color_attachments: &[Some(RenderPassColorAttachment {
145 // We need to specify the post process destination view here
146 // to make sure we write to the appropriate texture.
147 view: post_process.destination,
148 depth_slice: None,
149 resolve_target: None,
150 ops: Operations::default(),
151 })],
152 depth_stencil_attachment: None,
153 timestamp_writes: None,
154 occlusion_query_set: None,
155 multiview_mask: None,
156 });
157
158 render_pass.set_pipeline(pipeline);
159 // By passing in the index of the post process settings on this view, we ensure
160 // that in the event that multiple settings were sent to the GPU (as would be the
161 // case with multiple cameras), we use the correct one.
162 render_pass.set_bind_group(0, bind_group, &[settings_index.index()]);
163 render_pass.draw(0..3, 0..1);
164}Trait Implementations
Sourceimpl<C> Component for DynamicUniformIndex<C>
impl<C> Component for DynamicUniformIndex<C>
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
A constant indicating the storage type used for this component.
Sourcetype Mutability = Mutable
type Mutability = Mutable
A marker type to assist Bevy with determining if this component is
mutable, or immutable. Mutable components will have
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Registers required components. Read more
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Called when registering this component, allowing to override clone function (or disable cloning altogether) for this component. Read more
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<DynamicUniformIndex<C>>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<DynamicUniformIndex<C>>>
Returns
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
Maps the entities on this component using the given
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreAuto Trait Implementations
impl<C> Freeze for DynamicUniformIndex<C>
impl<C> RefUnwindSafe for DynamicUniformIndex<C>where
C: RefUnwindSafe,
impl<C> Send for DynamicUniformIndex<C>
impl<C> Sync for DynamicUniformIndex<C>
impl<C> Unpin for DynamicUniformIndex<C>where
C: Unpin,
impl<C> UnsafeUnpin for DynamicUniformIndex<C>
impl<C> UnwindSafe for DynamicUniformIndex<C>where
C: UnwindSafe,