Readback

Enum Readback 

pub enum Readback {
    Texture(Handle<Image>),
    Buffer {
        buffer: Handle<ShaderBuffer>,
        start_offset_and_size: Option<(u64, u64)>,
    },
}
A component that registers the wrapped handle for gpu readback, either a texture or a buffer.

Data is read asynchronously and will be triggered on the entity via the ReadbackComplete event when complete. If this component is not removed, the readback will be attempted every frame

Variants



Texture(Handle<Image>)



Buffer

Fields

start_offset_and_size: Option<(u64, u64)>

Implementations

Source

impl Readback

pub fn texture(image: Handle<Image>) -> Readback

Create a readback component for a texture using the given handle.

Examples found in repository?
examples/shader/gpu_readback.rs (line 128)
67fn setup(
68    mut commands: Commands,
69    mut images: ResMut<Assets<Image>>,
70    mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72    // Create a storage buffer with some data
73    let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74    let mut buffer = ShaderBuffer::from(buffer);
75    // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76    buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77    let buffer = buffers.add(buffer);
78
79    // Create a storage texture with some data
80    let size = Extent3d {
81        width: BUFFER_LEN as u32,
82        height: 1,
83        ..default()
84    };
85    // We create an uninitialized image since this texture will only be used for getting data out
86    // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87    let mut image = Image::new_uninit(
88        size,
89        TextureDimension::D2,
90        TextureFormat::R32Uint,
91        RenderAssetUsages::RENDER_WORLD,
92    );
93    // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94    // compute shader
95    image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96    let image = images.add(image);
97
98    // Spawn the readback components. For each frame, the data will be read back from the GPU
99    // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100    // to stop reading back the data.
101    commands
102        .spawn(Readback::buffer(buffer.clone()))
103        .observe(|event: On<ReadbackComplete>| {
104            // This matches the type which was used to create the `ShaderBuffer` above,
105            // and is a convenient way to interpret the data.
106            let data: Vec<u32> = event.to_shader_type();
107            info!("Buffer {:?}", data);
108        });
109
110    // It is also possible to read only a range of the buffer.
111    commands
112        .spawn(Readback::buffer_range(
113            buffer.clone(),
114            4 * u32::SHADER_SIZE.get(), // skip the first four elements
115            8 * u32::SHADER_SIZE.get(), // read eight elements
116        ))
117        .observe(|event: On<ReadbackComplete>| {
118            let data: Vec<u32> = event.to_shader_type();
119            info!("Buffer range {:?}", data);
120        });
121
122    // This is just a simple way to pass the buffer handle to the render app for our compute node
123    commands.insert_resource(ReadbackBuffer(buffer));
124
125    // Textures can also be read back from the GPU. Pay careful attention to the format of the
126    // texture, as it will affect how the data is interpreted.
127    commands
128        .spawn(Readback::texture(image.clone()))
129        .observe(|event: On<ReadbackComplete>| {
130            // You probably want to interpret the data as a color rather than a `ShaderType`,
131            // but in this case we know the data is a single channel storage texture, so we can
132            // interpret it as a `Vec<u32>`
133            let data: Vec<u32> = event.to_shader_type();
134            info!("Image {:?}", data);
135        });
136    commands.insert_resource(ReadbackImage(image));
137}
pub fn buffer(buffer: Handle<ShaderBuffer>) -> Readback

Create a readback component for a full buffer using the given handle.

Examples found in repository?
examples/shader/gpu_readback.rs (line 102)
67fn setup(
68    mut commands: Commands,
69    mut images: ResMut<Assets<Image>>,
70    mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72    // Create a storage buffer with some data
73    let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74    let mut buffer = ShaderBuffer::from(buffer);
75    // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76    buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77    let buffer = buffers.add(buffer);
78
79    // Create a storage texture with some data
80    let size = Extent3d {
81        width: BUFFER_LEN as u32,
82        height: 1,
83        ..default()
84    };
85    // We create an uninitialized image since this texture will only be used for getting data out
86    // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87    let mut image = Image::new_uninit(
88        size,
89        TextureDimension::D2,
90        TextureFormat::R32Uint,
91        RenderAssetUsages::RENDER_WORLD,
92    );
93    // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94    // compute shader
95    image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96    let image = images.add(image);
97
98    // Spawn the readback components. For each frame, the data will be read back from the GPU
99    // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100    // to stop reading back the data.
101    commands
102        .spawn(Readback::buffer(buffer.clone()))
103        .observe(|event: On<ReadbackComplete>| {
104            // This matches the type which was used to create the `ShaderBuffer` above,
105            // and is a convenient way to interpret the data.
106            let data: Vec<u32> = event.to_shader_type();
107            info!("Buffer {:?}", data);
108        });
109
110    // It is also possible to read only a range of the buffer.
111    commands
112        .spawn(Readback::buffer_range(
113            buffer.clone(),
114            4 * u32::SHADER_SIZE.get(), // skip the first four elements
115            8 * u32::SHADER_SIZE.get(), // read eight elements
116        ))
117        .observe(|event: On<ReadbackComplete>| {
118            let data: Vec<u32> = event.to_shader_type();
119            info!("Buffer range {:?}", data);
120        });
121
122    // This is just a simple way to pass the buffer handle to the render app for our compute node
123    commands.insert_resource(ReadbackBuffer(buffer));
124
125    // Textures can also be read back from the GPU. Pay careful attention to the format of the
126    // texture, as it will affect how the data is interpreted.
127    commands
128        .spawn(Readback::texture(image.clone()))
129        .observe(|event: On<ReadbackComplete>| {
130            // You probably want to interpret the data as a color rather than a `ShaderType`,
131            // but in this case we know the data is a single channel storage texture, so we can
132            // interpret it as a `Vec<u32>`
133            let data: Vec<u32> = event.to_shader_type();
134            info!("Image {:?}", data);
135        });
136    commands.insert_resource(ReadbackImage(image));
137}
pub fn buffer_range( buffer: Handle<ShaderBuffer>, start_offset: u64, size: u64, ) -> Readback

Create a readback component for a buffer range using the given handle, a start offset in bytes and a number of bytes to read.

Examples found in repository?
examples/shader/gpu_readback.rs (lines 112-116)
67fn setup(
68    mut commands: Commands,
69    mut images: ResMut<Assets<Image>>,
70    mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72    // Create a storage buffer with some data
73    let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74    let mut buffer = ShaderBuffer::from(buffer);
75    // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76    buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77    let buffer = buffers.add(buffer);
78
79    // Create a storage texture with some data
80    let size = Extent3d {
81        width: BUFFER_LEN as u32,
82        height: 1,
83        ..default()
84    };
85    // We create an uninitialized image since this texture will only be used for getting data out
86    // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87    let mut image = Image::new_uninit(
88        size,
89        TextureDimension::D2,
90        TextureFormat::R32Uint,
91        RenderAssetUsages::RENDER_WORLD,
92    );
93    // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94    // compute shader
95    image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96    let image = images.add(image);
97
98    // Spawn the readback components. For each frame, the data will be read back from the GPU
99    // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100    // to stop reading back the data.
101    commands
102        .spawn(Readback::buffer(buffer.clone()))
103        .observe(|event: On<ReadbackComplete>| {
104            // This matches the type which was used to create the `ShaderBuffer` above,
105            // and is a convenient way to interpret the data.
106            let data: Vec<u32> = event.to_shader_type();
107            info!("Buffer {:?}", data);
108        });
109
110    // It is also possible to read only a range of the buffer.
111    commands
112        .spawn(Readback::buffer_range(
113            buffer.clone(),
114            4 * u32::SHADER_SIZE.get(), // skip the first four elements
115            8 * u32::SHADER_SIZE.get(), // read eight elements
116        ))
117        .observe(|event: On<ReadbackComplete>| {
118            let data: Vec<u32> = event.to_shader_type();
119            info!("Buffer range {:?}", data);
120        });
121
122    // This is just a simple way to pass the buffer handle to the render app for our compute node
123    commands.insert_resource(ReadbackBuffer(buffer));
124
125    // Textures can also be read back from the GPU. Pay careful attention to the format of the
126    // texture, as it will affect how the data is interpreted.
127    commands
128        .spawn(Readback::texture(image.clone()))
129        .observe(|event: On<ReadbackComplete>| {
130            // You probably want to interpret the data as a color rather than a `ShaderType`,
131            // but in this case we know the data is a single channel storage texture, so we can
132            // interpret it as a `Vec<u32>`
133            let data: Vec<u32> = event.to_shader_type();
134            info!("Image {:?}", data);
135        });
136    commands.insert_resource(ReadbackImage(image));
137}