pub struct MeshDeserializer { /* private fields */ }Available on crate features
bevy_render and serialize only.Expand description
Use to specify extra options when deserializing a SerializedMesh into a Mesh.
Implementations
Sourceimpl MeshDeserializer
impl MeshDeserializer
Sourcepub fn new() -> MeshDeserializer
Available on crate feature bevy_mesh only.
pub fn new() -> MeshDeserializer
bevy_mesh only.Create a new MeshDeserializer.
Sourcepub fn add_custom_vertex_attribute(
&mut self,
name: &str,
attribute: MeshVertexAttribute,
) -> &mut MeshDeserializer
Available on crate feature bevy_mesh only.
pub fn add_custom_vertex_attribute( &mut self, name: &str, attribute: MeshVertexAttribute, ) -> &mut MeshDeserializer
bevy_mesh only.Register a custom vertex attribute to the deserializer. Custom vertex attributes that were not added with this method will be ignored while deserializing.
Sourcepub fn deserialize(&self, serialized_mesh: SerializedMesh) -> Mesh
Available on crate feature bevy_mesh only.
pub fn deserialize(&self, serialized_mesh: SerializedMesh) -> Mesh
bevy_mesh only.Deserialize a SerializedMesh into a Mesh.
See the documentation for SerializedMesh for caveats.