MeshVertexBufferLayout

Struct MeshVertexBufferLayout 

pub struct MeshVertexBufferLayout { /* private fields */ }

Implementations

Source

impl MeshVertexBufferLayout

pub fn new( attribute_ids: Vec<MeshVertexAttributeId>, layout: VertexBufferLayout, ) -> MeshVertexBufferLayout

pub fn contains(&self, attribute_id: impl Into<MeshVertexAttributeId>) -> bool

Examples found in repository?
examples/shader_advanced/custom_render_phase.rs (line 192)
185    fn specialize(
186        &self,
187        key: Self::Key,
188        layout: &MeshVertexBufferLayoutRef,
189    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
190        // We will only use the position of the mesh in our shader so we only need to specify that
191        let mut vertex_attributes = Vec::new();
192        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
193            // Make sure this matches the shader location
194            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
195        }
196        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
197        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
198        let view_layout = self
199            .mesh_pipeline
200            .get_view_layout(MeshPipelineViewLayoutKey::from(key));
201        Ok(RenderPipelineDescriptor {
202            label: Some("Specialized Mesh Pipeline".into()),
203            // We want to reuse the data from bevy so we use the same bind groups as the default
204            // mesh pipeline
205            layout: vec![
206                // Bind group 0 is the view uniform
207                view_layout.main_layout,
208                // Bind group 1 is empty
209                view_layout.empty_layout,
210                // Bind group 2 is the mesh uniform
211                self.mesh_pipeline.mesh_layouts.model_only.clone(),
212            ],
213            vertex: VertexState {
214                shader: self.shader_handle.clone(),
215                buffers: vec![vertex_buffer_layout],
216                ..default()
217            },
218            fragment: Some(FragmentState {
219                shader: self.shader_handle.clone(),
220                targets: vec![Some(ColorTargetState {
221                    format: key.target_format(),
222                    blend: None,
223                    write_mask: ColorWrites::ALL,
224                })],
225                ..default()
226            }),
227            primitive: PrimitiveState {
228                topology: key.primitive_topology(),
229                strip_index_format: key.strip_index_format(),
230                cull_mode: Some(Face::Back),
231                ..default()
232            },
233            // It's generally recommended to specialize your pipeline for MSAA,
234            // but it's not always possible
235            ..default()
236        })
237    }
More examples
Hide additional examples
examples/shader_advanced/specialized_mesh_pipeline.rs (line 199)
192    fn specialize(
193        &self,
194        mesh_key: Self::Key,
195        layout: &MeshVertexBufferLayoutRef,
196    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
197        // Define the vertex attributes based on a standard bevy [`Mesh`]
198        let mut vertex_attributes = Vec::new();
199        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
200            // Make sure this matches the shader location
201            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
202        }
203        if layout.0.contains(Mesh::ATTRIBUTE_COLOR) {
204            // Make sure this matches the shader location
205            vertex_attributes.push(Mesh::ATTRIBUTE_COLOR.at_shader_location(1));
206        }
207        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
208        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
209
210        let view_layout = self
211            .mesh_pipeline
212            .get_view_layout(MeshPipelineViewLayoutKey::from(mesh_key));
213
214        Ok(RenderPipelineDescriptor {
215            label: Some("Specialized Mesh Pipeline".into()),
216            layout: vec![
217                view_layout.main_layout,
218                view_layout.empty_layout,
219                self.mesh_pipeline.mesh_layouts.model_only.clone(),
220            ],
221            vertex: VertexState {
222                shader: self.shader_handle.clone(),
223                // Customize how to store the meshes' vertex attributes in the vertex buffer
224                buffers: vec![vertex_buffer_layout],
225                ..default()
226            },
227            fragment: Some(FragmentState {
228                shader: self.shader_handle.clone(),
229                targets: vec![Some(ColorTargetState {
230                    // This isn't required, but bevy supports rendering different formats
231                    // so it's generally recommended to specialize the pipeline for that
232                    format: mesh_key.target_format(),
233                    // For this example we only use opaque meshes,
234                    // but if you wanted to use alpha blending you would need to set it here
235                    blend: None,
236                    write_mask: ColorWrites::ALL,
237                })],
238                ..default()
239            }),
240            primitive: PrimitiveState {
241                topology: mesh_key.primitive_topology(),
242                strip_index_format: mesh_key.strip_index_format(),
243                front_face: FrontFace::Ccw,
244                cull_mode: Some(Face::Back),
245                polygon_mode: PolygonMode::Fill,
246                ..default()
247            },
248            // Note that if your view has no depth buffer this will need to be
249            // changed.
250            depth_stencil: Some(DepthStencilState {
251                format: CORE_3D_DEPTH_FORMAT,
252                depth_write_enabled: Some(true),
253                depth_compare: Some(CompareFunction::GreaterEqual),
254                stencil: default(),
255                bias: default(),
256            }),
257            // It's generally recommended to specialize your pipeline for MSAA,
258            // but it's not always possible
259            multisample: MultisampleState {
260                count: mesh_key.msaa_samples(),
261                ..default()
262            },
263            ..default()
264        })
265    }
pub fn attribute_ids(&self) -> &[MeshVertexAttributeId]

pub fn layout(&self) -> &VertexBufferLayout

pub fn get_layout( &self, attribute_descriptors: &[VertexAttributeDescriptor], ) -> Result<VertexBufferLayout, MissingVertexAttributeError>

Examples found in repository?
examples/shader_advanced/custom_vertex_attribute.rs (lines 80-83)
74    fn specialize(
75        _pipeline: &MaterialPipeline,
76        descriptor: &mut RenderPipelineDescriptor,
77        layout: &MeshVertexBufferLayoutRef,
78        _key: MaterialPipelineKey<Self>,
79    ) -> Result<(), SpecializedMeshPipelineError> {
80        let vertex_layout = layout.0.get_layout(&[
81            Mesh::ATTRIBUTE_POSITION.at_shader_location(0),
82            ATTRIBUTE_BLEND_COLOR.at_shader_location(1),
83        ])?;
84        descriptor.vertex.buffers = vec![vertex_layout];
85        Ok(())
86    }
More examples
Hide additional examples
examples/gltf/custom_gltf_vertex_attribute.rs (lines 87-91)
82    fn specialize(
83        descriptor: &mut RenderPipelineDescriptor,
84        layout: &MeshVertexBufferLayoutRef,
85        _key: Material2dKey<Self>,
86    ) -> Result<(), SpecializedMeshPipelineError> {
87        let vertex_layout = layout.0.get_layout(&[
88            Mesh::ATTRIBUTE_POSITION.at_shader_location(0),
89            Mesh::ATTRIBUTE_COLOR.at_shader_location(1),
90            ATTRIBUTE_BARYCENTRIC.at_shader_location(2),
91        ])?;
92        descriptor.vertex.buffers = vec![vertex_layout];
93        Ok(())
94    }
examples/gltf/gltf_extension_mesh_2d.rs (lines 138-142)
133    fn specialize(
134        descriptor: &mut RenderPipelineDescriptor,
135        layout: &MeshVertexBufferLayoutRef,
136        _key: Material2dKey<Self>,
137    ) -> Result<(), SpecializedMeshPipelineError> {
138        let vertex_layout = layout.0.get_layout(&[
139            Mesh::ATTRIBUTE_POSITION.at_shader_location(0),
140            Mesh::ATTRIBUTE_COLOR.at_shader_location(1),
141            ATTRIBUTE_BARYCENTRIC.at_shader_location(2),
142        ])?;
143        descriptor.vertex.buffers = vec![vertex_layout];
144        Ok(())
145    }
examples/shader_advanced/custom_render_phase.rs (line 197)
185    fn specialize(
186        &self,
187        key: Self::Key,
188        layout: &MeshVertexBufferLayoutRef,
189    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
190        // We will only use the position of the mesh in our shader so we only need to specify that
191        let mut vertex_attributes = Vec::new();
192        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
193            // Make sure this matches the shader location
194            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
195        }
196        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
197        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
198        let view_layout = self
199            .mesh_pipeline
200            .get_view_layout(MeshPipelineViewLayoutKey::from(key));
201        Ok(RenderPipelineDescriptor {
202            label: Some("Specialized Mesh Pipeline".into()),
203            // We want to reuse the data from bevy so we use the same bind groups as the default
204            // mesh pipeline
205            layout: vec![
206                // Bind group 0 is the view uniform
207                view_layout.main_layout,
208                // Bind group 1 is empty
209                view_layout.empty_layout,
210                // Bind group 2 is the mesh uniform
211                self.mesh_pipeline.mesh_layouts.model_only.clone(),
212            ],
213            vertex: VertexState {
214                shader: self.shader_handle.clone(),
215                buffers: vec![vertex_buffer_layout],
216                ..default()
217            },
218            fragment: Some(FragmentState {
219                shader: self.shader_handle.clone(),
220                targets: vec![Some(ColorTargetState {
221                    format: key.target_format(),
222                    blend: None,
223                    write_mask: ColorWrites::ALL,
224                })],
225                ..default()
226            }),
227            primitive: PrimitiveState {
228                topology: key.primitive_topology(),
229                strip_index_format: key.strip_index_format(),
230                cull_mode: Some(Face::Back),
231                ..default()
232            },
233            // It's generally recommended to specialize your pipeline for MSAA,
234            // but it's not always possible
235            ..default()
236        })
237    }
examples/shader_advanced/specialized_mesh_pipeline.rs (line 208)
192    fn specialize(
193        &self,
194        mesh_key: Self::Key,
195        layout: &MeshVertexBufferLayoutRef,
196    ) -> Result<RenderPipelineDescriptor, SpecializedMeshPipelineError> {
197        // Define the vertex attributes based on a standard bevy [`Mesh`]
198        let mut vertex_attributes = Vec::new();
199        if layout.0.contains(Mesh::ATTRIBUTE_POSITION) {
200            // Make sure this matches the shader location
201            vertex_attributes.push(Mesh::ATTRIBUTE_POSITION.at_shader_location(0));
202        }
203        if layout.0.contains(Mesh::ATTRIBUTE_COLOR) {
204            // Make sure this matches the shader location
205            vertex_attributes.push(Mesh::ATTRIBUTE_COLOR.at_shader_location(1));
206        }
207        // This will automatically generate the correct `VertexBufferLayout` based on the vertex attributes
208        let vertex_buffer_layout = layout.0.get_layout(&vertex_attributes)?;
209
210        let view_layout = self
211            .mesh_pipeline
212            .get_view_layout(MeshPipelineViewLayoutKey::from(mesh_key));
213
214        Ok(RenderPipelineDescriptor {
215            label: Some("Specialized Mesh Pipeline".into()),
216            layout: vec![
217                view_layout.main_layout,
218                view_layout.empty_layout,
219                self.mesh_pipeline.mesh_layouts.model_only.clone(),
220            ],
221            vertex: VertexState {
222                shader: self.shader_handle.clone(),
223                // Customize how to store the meshes' vertex attributes in the vertex buffer
224                buffers: vec![vertex_buffer_layout],
225                ..default()
226            },
227            fragment: Some(FragmentState {
228                shader: self.shader_handle.clone(),
229                targets: vec![Some(ColorTargetState {
230                    // This isn't required, but bevy supports rendering different formats
231                    // so it's generally recommended to specialize the pipeline for that
232                    format: mesh_key.target_format(),
233                    // For this example we only use opaque meshes,
234                    // but if you wanted to use alpha blending you would need to set it here
235                    blend: None,
236                    write_mask: ColorWrites::ALL,
237                })],
238                ..default()
239            }),
240            primitive: PrimitiveState {
241                topology: mesh_key.primitive_topology(),
242                strip_index_format: mesh_key.strip_index_format(),
243                front_face: FrontFace::Ccw,
244                cull_mode: Some(Face::Back),
245                polygon_mode: PolygonMode::Fill,
246                ..default()
247            },
248            // Note that if your view has no depth buffer this will need to be
249            // changed.
250            depth_stencil: Some(DepthStencilState {
251                format: CORE_3D_DEPTH_FORMAT,
252                depth_write_enabled: Some(true),
253                depth_compare: Some(CompareFunction::GreaterEqual),
254                stencil: default(),
255                bias: default(),
256            }),
257            // It's generally recommended to specialize your pipeline for MSAA,
258            // but it's not always possible
259            multisample: MultisampleState {
260                count: mesh_key.msaa_samples(),
261                ..default()
262            },
263            ..default()
264        })
265    }