pub struct ExtractSchedule;Expand description
Schedule in which data from the main world is ‘extracted’ into the render world.
This step should be kept as short as possible to increase the “pipelining potential” for running the next frame while rendering the current frame.
This schedule is run on the render world, but it also has access to the main world.
See MainWorld and Extract for details on how to access main world data from this schedule.