pub enum BindlessResourceType {
None,
Buffer,
SamplerFiltering,
SamplerNonFiltering,
SamplerComparison,
Texture1d,
Texture2d,
Texture2dArray,
Texture3d,
TextureCube,
TextureCubeArray,
DataBuffer,
}Expand description
The type of potentially-bindless resource.
Variants
None
No bindless resource.
This is used as a placeholder to fill holes in the
BindlessDescriptor::resources list.
Buffer
A storage buffer.
SamplerFiltering
A filtering sampler.
SamplerNonFiltering
A non-filtering sampler (nearest neighbor).
SamplerComparison
A comparison sampler (typically used for shadow maps).
Texture1d
A 1D texture.
Texture2d
A 2D texture.
Texture2dArray
A 2D texture array.
Note that this differs from a binding array. 2D texture arrays must all have the same size and format.
Texture3d
A 3D texture.
TextureCube
A cubemap texture.
TextureCubeArray
A cubemap texture array.
Note that this differs from a binding array. Cubemap texture arrays must all have the same size and format.
DataBuffer
Multiple instances of plain old data concatenated into a single buffer.
This corresponds to the #[data] declaration in
crate::render_resource::AsBindGroup.
Note that this resource doesn’t itself map to a GPU-level binding
resource and instead depends on the MaterialBindGroupAllocator to
create a binding resource for it.
Implementations
Sourceimpl BindlessResourceType
impl BindlessResourceType
Sourcepub fn binding_number(&self) -> Option<&'static BindingNumber>
pub fn binding_number(&self) -> Option<&'static BindingNumber>
Returns the binding number for the common array of this resource type.
For example, if you pass BindlessResourceType::Texture2d, this will
return 5, in order to match the @group(2) @binding(5) var bindless_textures_2d: binding_array<texture_2d<f32>> declaration in
bindless.wgsl.
Not all resource types have fixed binding numbers. If you call
Self::binding_number on such a resource type, it returns None.
Note that this returns a static reference to the binding number, not the
binding number itself. This is to conform to an idiosyncratic API in
wgpu whereby binding numbers for binding arrays are taken by &u32
reference, not by u32 value.