pub enum StorageTextureAccess {
WriteOnly,
ReadOnly,
ReadWrite,
Atomic,
}Expand description
Specific type of a sample in a texture binding.
For use in BindingType::StorageTexture.
Corresponds to WebGPU GPUStorageTextureAccess.
Variants
WriteOnly
The texture can only be written in the shader and it:
- may or may not be annotated with
write(WGSL). - must be annotated with
writeonly(GLSL).
Example WGSL syntax:
@group(0) @binding(0)
var my_storage_image: texture_storage_2d<r32float, write>;Example GLSL syntax:
layout(set=0, binding=0, r32f) writeonly uniform image2D myStorageImage;ReadOnly
The texture can only be read in the shader and it must be annotated with read (WGSL) or
readonly (GLSL).
Features::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES must be enabled to use this access
mode. This is a native-only extension.
Example WGSL syntax:
@group(0) @binding(0)
var my_storage_image: texture_storage_2d<r32float, read>;Example GLSL syntax:
layout(set=0, binding=0, r32f) readonly uniform image2D myStorageImage;ReadWrite
The texture can be both read and written in the shader and must be annotated with
read_write in WGSL.
Features::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES must be enabled to use this access
mode. This is a nonstandard, native-only extension.
Example WGSL syntax:
@group(0) @binding(0)
var my_storage_image: texture_storage_2d<r32float, read_write>;Example GLSL syntax:
layout(set=0, binding=0, r32f) uniform image2D myStorageImage;Atomic
The texture can be both read and written in the shader via atomics and must be annotated
with read_write in WGSL.
Features::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES must be enabled to use this access
mode. This is a nonstandard, native-only extension.
Example WGSL syntax:
@group(0) @binding(0)
var my_storage_image: texture_storage_2d<r32uint, atomic>;Trait Implementations
Sourceimpl Clone for StorageTextureAccess
impl Clone for StorageTextureAccess
Sourcefn clone(&self) -> StorageTextureAccess
fn clone(&self) -> StorageTextureAccess
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for StorageTextureAccess
impl Debug for StorageTextureAccess
Sourceimpl<'de> Deserialize<'de> for StorageTextureAccess
impl<'de> Deserialize<'de> for StorageTextureAccess
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<StorageTextureAccess, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<StorageTextureAccess, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl Hash for StorageTextureAccess
impl Hash for StorageTextureAccess
Sourceimpl PartialEq for StorageTextureAccess
impl PartialEq for StorageTextureAccess
Sourceimpl Serialize for StorageTextureAccess
impl Serialize for StorageTextureAccess
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
impl Copy for StorageTextureAccess
impl Eq for StorageTextureAccess
impl StructuralPartialEq for StorageTextureAccess