pub struct RenderPipeline { /* private fields */ }Expand description
A RenderPipeline represents a graphics pipeline and its stages (shaders), bindings and vertex buffers.
May be converted from and dereferences to a wgpu RenderPipeline.
Can be created via RenderDevice::create_render_pipeline.
Implementations
Sourceimpl RenderPipeline
impl RenderPipeline
pub fn id(&self) -> RenderPipelineId
Methods from Deref<Target = RenderPipeline>
Sourcepub fn get_bind_group_layout(&self, index: u32) -> BindGroupLayout
pub fn get_bind_group_layout(&self, index: u32) -> BindGroupLayout
Get an object representing the bind group layout at a given index.
If this pipeline was created with a default layout, then
bind groups created with the returned BindGroupLayout can only be used with this pipeline.
This method will raise a validation error if there is no bind group layout at index.