pub struct StorageBuffer<T>where
T: ShaderType,{ /* private fields */ }bevy_render only.Expand description
Stores data to be transferred to the GPU and made accessible to shaders as a storage buffer.
Storage buffers can be made available to shaders in some combination of read/write mode, and can store large amounts of data. Note however that WebGL2 does not support storage buffers, so consider alternative options in this case.
Storage buffers can store runtime-sized arrays, but only if they are the last field in a structure.
The contained data is stored in system RAM. write_buffer queues
copying of the data from system RAM to VRAM. Storage buffers must conform to std430 alignment/padding requirements, which
is automatically enforced by this structure.
Other options for storing GPU-accessible data are:
Implementations
Sourceimpl<T> StorageBuffer<T>where
T: ShaderType + WriteInto,
impl<T> StorageBuffer<T>where
T: ShaderType + WriteInto,
pub fn buffer(&self) -> Option<&Buffer>
pub fn binding(&self) -> Option<BindingResource<'_>>
pub fn set(&mut self, value: T)
pub fn get(&self) -> &T
pub fn get_mut(&mut self) -> &mut T
pub fn set_label(&mut self, label: Option<&str>)
pub fn get_label(&self) -> Option<&str>
Sourcepub fn add_usages(&mut self, usage: BufferUsages)
pub fn add_usages(&mut self, usage: BufferUsages)
Add more BufferUsages to the buffer.
This method only allows addition of flags to the default usage flags.
The default values for buffer usage are BufferUsages::COPY_DST and BufferUsages::STORAGE.
Sourcepub fn write_buffer(&mut self, device: &RenderDevice, queue: &RenderQueue)
pub fn write_buffer(&mut self, device: &RenderDevice, queue: &RenderQueue)
Queues writing of data from system RAM to VRAM using the RenderDevice
and the provided RenderQueue.
If there is no GPU-side buffer allocated to hold the data currently stored, or if a GPU-side buffer previously allocated does not have enough capacity, a new GPU-side buffer is created.