Struct bevy::render::render_resource::TextureViewDescriptor [−]
pub struct TextureViewDescriptor<'a> {
pub label: Option<&'a str>,
pub format: Option<TextureFormat>,
pub dimension: Option<TextureViewDimension>,
pub aspect: TextureAspect,
pub base_mip_level: u32,
pub mip_level_count: Option<NonZeroU32>,
pub base_array_layer: u32,
pub array_layer_count: Option<NonZeroU32>,
}Expand description
Describes a [TextureView].
Fields
label: Option<&'a str>Debug label of the texture view. This will show up in graphics debuggers for easy identification.
format: Option<TextureFormat>Format of the texture view. At this time, it must be the same as the underlying format of the texture.
dimension: Option<TextureViewDimension>The dimension of the texture view. For 1D textures, this must be 1D. For 2D textures it must be one of
D2, D2Array, Cube, and CubeArray. For 3D textures it must be 3D
aspect: TextureAspectAspect of the texture. Color textures must be TextureAspect::All.
base_mip_level: u32Base mip level.
mip_level_count: Option<NonZeroU32>Mip level count.
If Some(count), base_mip_level + count must be less or equal to underlying texture mip count.
If None, considered to include the rest of the mipmap levels, but at least 1 in total.
base_array_layer: u32Base array layer.
array_layer_count: Option<NonZeroU32>Layer count.
If Some(count), base_array_layer + count must be less or equal to the underlying array count.
If None, considered to include the rest of the array layers, but at least 1 in total.