Struct bevy::render::render_resource::WgpuAdapterInfo
pub struct WgpuAdapterInfo {
pub name: String,
pub vendor: usize,
pub device: usize,
pub device_type: DeviceType,
pub driver: String,
pub driver_info: String,
pub backend: Backend,
}Expand description
Information about an adapter.
Fields
name: StringAdapter name
vendor: usizeVendor PCI id of the adapter
If the vendor has no PCI id, then this value will be the backend’s vendor id equivalent. On Vulkan,
Mesa would have a vendor id equivalent to it’s VkVendorId value.
device: usizePCI id of the adapter
device_type: DeviceTypeType of device
driver: StringDriver name
driver_info: StringDriver info
backend: BackendBackend used for device