pub struct WgpuTextureView { /* private fields */ }Expand description
Handle to a texture view.
A TextureView object refers to a Texture, or a subset of its layers and mip levels, and
specifies an interpretation of the texture’s texels, which is needed to use a texture as a
binding in a BindGroup or as an attachment in a RenderPass.
It can be created using Texture::create_view(), which accepts a TextureViewDescriptor
specifying the properties of the view.
Corresponds to WebGPU GPUTextureView.
Implementations
Sourceimpl TextureView
impl TextureView
Sourcepub unsafe fn as_hal<A>(
&self,
) -> Option<impl Deref<Target = <A as Api>::TextureView>>where
A: Api,
Available on wgpu_core only.
pub unsafe fn as_hal<A>(
&self,
) -> Option<impl Deref<Target = <A as Api>::TextureView>>where
A: Api,
wgpu_core only.Get the wgpu_hal texture view from this TextureView.
Find the Api struct corresponding to the active backend in wgpu_hal::api,
and pass that struct to the to the A type parameter.
Returns a guard that dereferences to the type of the hal backend
which implements A::TextureView.
Types
The returned type depends on the backend:
hal::api::Vulkanuseshal::vulkan::TextureViewhal::api::Metaluseshal::metal::TextureViewhal::api::Dx12useshal::dx12::TextureViewhal::api::Glesuseshal::gles::TextureView
Deadlocks
- The returned guard holds a read-lock on a device-local “destruction”
lock, which will cause all calls to
destroyto block until the guard is released.
Errors
This method will return None if:
- The texture view is not from the backend specified by
A. - The texture view is from the
webgpuorcustombackend. - The texture this view points to has had
Texture::destroy()called on it.
Safety
- The returned resource must not be destroyed unless the guard is the last reference to it and it is not in use by the GPU. The guard and handle may be dropped at any time however.
- All the safety requirements of wgpu-hal must be upheld.