Struct bevy::render::renderer::RenderInstance
pub struct RenderInstance(pub Instance);Expand description
The GPU instance is used to initialize the RenderQueue and RenderDevice,
as well as to create WindowSurfaces.
Tuple Fields
0: InstanceMethods from Deref<Target = Instance>
pub fn enumerate_adapters(
&self,
backends: Backends
) -> impl Iterator<Item = Adapter>
pub fn enumerate_adapters(
&self,
backends: Backends
) -> impl Iterator<Item = Adapter>
pub fn request_adapter(
&self,
options: &RequestAdapterOptions<&Surface>
) -> impl Future<Output = Option<Adapter>> + Send
pub fn request_adapter(
&self,
options: &RequestAdapterOptions<&Surface>
) -> impl Future<Output = Option<Adapter>> + Send
Retrieves an [Adapter] which matches the given [RequestAdapterOptions].
Some options are “soft”, so treated as non-mandatory. Others are “hard”.
If no adapters are found that suffice all the “hard” options, None is returned.
pub unsafe fn create_adapter_from_hal<A>(
&self,
hal_adapter: ExposedAdapter<A>
) -> Adapterwhere
A: HalApi,
pub unsafe fn create_adapter_from_hal<A>(
&self,
hal_adapter: ExposedAdapter<A>
) -> Adapterwhere
A: HalApi,
Converts a wgpu-hal ExposedAdapter to a wgpu [Adapter].
Safety
hal_adapter must be created from this instance internal handle.
pub unsafe fn create_surface<W>(&self, window: &W) -> Surfacewhere
W: HasRawWindowHandle + HasRawDisplayHandle,
pub unsafe fn create_surface<W>(&self, window: &W) -> Surfacewhere
W: HasRawWindowHandle + HasRawDisplayHandle,
Creates a surface from a raw window handle.
If the specified display and window handle are not supported by any of the backends, then the surface will not be supported by any adapters.
Safety
- Raw Window Handle must be a valid object to create a surface upon and must remain valid for the lifetime of the returned surface.
- If not called on the main thread, metal backend will panic.
pub fn poll_all(&self, force_wait: bool) -> bool
pub fn poll_all(&self, force_wait: bool) -> bool
Polls all devices.
If force_wait is true and this is not running on the web, then this
function will block until all in-flight buffers have been mapped and
all submitted commands have finished execution.
Return true if all devices’ queues are empty, or false if there are
queue submissions still in flight. (Note that, unless access to all
Queues associated with this [Instance] is coordinated somehow,
this information could be out of date by the time the caller receives
it. Queues can be shared between threads, and other threads could
submit new work at any time.)
On the web, this is a no-op. Devices are automatically polled.
pub fn generate_report(&self) -> GlobalReport
pub fn generate_report(&self) -> GlobalReport
Generates memory report.