pub struct DeallocationStage<'a, I>where
I: SlabItem,{
pub allocator: &'a mut SlabAllocator<I>,
/* private fields */
}Expand description
An object that enables batched deallocation.
To free objects from a SlabAllocator, call
SlabAllocator::stage_deallocation to create a DeallocationStage,
call Self::free to deallocate objects, and finally call
Self::commit. You must call Self::commit in order to ensure that
newly-empty slabs are deallocated.
Fields
allocator: &'a mut SlabAllocator<I>The allocator in which objects are to be freed.
Implementations
Sourceimpl<'a, I> DeallocationStage<'a, I>where
I: SlabItem,
impl<'a, I> DeallocationStage<'a, I>where
I: SlabItem,
Sourcepub fn free(&mut self, key: &<I as SlabItem>::Key)
pub fn free(&mut self, key: &<I as SlabItem>::Key)
Schedules a free operation for the allocation with the given key.
The key must correspond to a live allocation. An error will be emitted to the log otherwise.
Sourcepub fn commit(self)
pub fn commit(self)
Performs all the free operations.
You must call this method if you called Self::free.