pub struct DepthStencilViewAttachment {
pub depth_stencil_views: DepthStencilViews,
/* private fields */
}Available on crate feature
bevy_render only.Expand description
A wrapper for a TextureView that is used as a RenderPassDepthStencilAttachment.
Fields
depth_stencil_views: DepthStencilViewsImplementations
Sourceimpl DepthStencilViewAttachment
impl DepthStencilViewAttachment
pub fn new( depth_stencil_views: DepthStencilViews, depth_clear_value: Option<f32>, stencil_clear_value: Option<u32>, ) -> DepthStencilViewAttachment
Sourcepub fn get_attachment(
&self,
store: StoreOp,
) -> RenderPassDepthStencilAttachment<'_>
pub fn get_attachment( &self, store: StoreOp, ) -> RenderPassDepthStencilAttachment<'_>
Get this texture view as an attachment. The attachment will be cleared with a value of
clear_value if this is the first time calling this function with store == StoreOp::Store,
and a clear value was provided, otherwise it will be loaded.
Examples found in repository?
examples/shader_advanced/deferred_raymarch.rs (line 378)
360fn draw_raymarch_shadow(
361 ctx: &mut RenderContext,
362 pipeline_cache: &PipelineCache,
363 layout: &BindGroupLayoutDescriptor,
364 view_uniforms: &ViewUniforms,
365 globals: &GlobalsBuffer,
366 render_pipeline: &RenderPipeline,
367 shadow_view: &ShadowView,
368 view_uniform_offset: &ViewUniformOffset,
369) {
370 let Some(bind_group) = raymarch_bind_group(ctx, pipeline_cache, layout, view_uniforms, globals)
371 else {
372 return;
373 };
374
375 let mut pass = ctx.begin_tracked_render_pass(RenderPassDescriptor {
376 label: Some("raymarch_shadow_pass"),
377 color_attachments: &[],
378 depth_stencil_attachment: Some(shadow_view.depth_attachment.get_attachment(StoreOp::Store)),
379 timestamp_writes: None,
380 occlusion_query_set: None,
381 multiview_mask: None,
382 });
383
384 pass.set_render_pipeline(render_pipeline);
385 pass.set_bind_group(0, &bind_group, &[view_uniform_offset.offset]);
386 pass.draw(0..3, 0..1);
387}Sourcepub fn prepare_for_new_frame(&self)
pub fn prepare_for_new_frame(&self)
Marks this depth attachment as unused this frame so that it’ll be cleared at first use.