pub struct ColorGrading {
pub global: ColorGradingGlobal,
pub shadows: ColorGradingSection,
pub midtones: ColorGradingSection,
pub highlights: ColorGradingSection,
}Expand description
Configures filmic color grading parameters to adjust the image appearance.
Color grading is applied just before tonemapping for a given
Camera entity, with the sole exception of the
post_saturation value in ColorGradingGlobal, which is applied after
tonemapping.
Fields
global: ColorGradingGlobalFilmic color grading values applied to the image as a whole (as opposed to individual sections, like shadows and highlights).
shadows: ColorGradingSectionColor grading values that are applied to the darker parts of the image.
The cutoff points can be customized with the
ColorGradingGlobal::midtones_range field.
midtones: ColorGradingSectionColor grading values that are applied to the parts of the image with intermediate brightness.
The cutoff points can be customized with the
ColorGradingGlobal::midtones_range field.
highlights: ColorGradingSectionColor grading values that are applied to the lighter parts of the image.
The cutoff points can be customized with the
ColorGradingGlobal::midtones_range field.
Implementations
Sourceimpl ColorGrading
impl ColorGrading
Sourcepub fn with_identical_sections(
global: ColorGradingGlobal,
section: ColorGradingSection,
) -> ColorGrading
pub fn with_identical_sections( global: ColorGradingGlobal, section: ColorGradingSection, ) -> ColorGrading
Creates a new ColorGrading instance in which shadows, midtones, and
highlights all have the same set of color grading values.
Examples found in repository?
556 fn basic_scene_recommendation(method: Tonemapping) -> ColorGrading {
557 match method {
558 Tonemapping::Reinhard | Tonemapping::ReinhardLuminance => ColorGrading {
559 global: ColorGradingGlobal {
560 exposure: 0.5,
561 ..default()
562 },
563 ..default()
564 },
565 Tonemapping::AcesFitted => ColorGrading {
566 global: ColorGradingGlobal {
567 exposure: 0.35,
568 ..default()
569 },
570 ..default()
571 },
572 Tonemapping::AgX => ColorGrading::with_identical_sections(
573 ColorGradingGlobal {
574 exposure: -0.2,
575 post_saturation: 1.1,
576 ..default()
577 },
578 ColorGradingSection {
579 saturation: 1.1,
580 ..default()
581 },
582 ),
583 _ => ColorGrading::default(),
584 }
585 }Sourcepub fn all_sections(&self) -> impl Iterator<Item = &ColorGradingSection>
pub fn all_sections(&self) -> impl Iterator<Item = &ColorGradingSection>
Returns an iterator that visits the shadows, midtones, and highlights sections, in that order.
Sourcepub fn all_sections_mut(
&mut self,
) -> impl Iterator<Item = &mut ColorGradingSection>
pub fn all_sections_mut( &mut self, ) -> impl Iterator<Item = &mut ColorGradingSection>
Applies the given mutating function to the shadows, midtones, and highlights sections, in that order.
Returns an array composed of the results of such evaluation, in that order.
Examples found in repository?
340fn update_color_grading_settings(
341 keys: Res<ButtonInput<KeyCode>>,
342 time: Res<Time>,
343 mut per_method_settings: ResMut<PerMethodSettings>,
344 tonemapping: Single<&Tonemapping>,
345 current_scene: Res<CurrentScene>,
346 mut selected_parameter: ResMut<SelectedParameter>,
347) {
348 let color_grading = per_method_settings.settings.get_mut(*tonemapping).unwrap();
349 let mut dt = time.delta_secs() * 0.25;
350 if keys.pressed(KeyCode::ArrowLeft) {
351 dt = -dt;
352 }
353
354 if keys.just_pressed(KeyCode::ArrowDown) {
355 selected_parameter.next();
356 }
357 if keys.just_pressed(KeyCode::ArrowUp) {
358 selected_parameter.prev();
359 }
360 if keys.pressed(KeyCode::ArrowLeft) || keys.pressed(KeyCode::ArrowRight) {
361 match selected_parameter.value {
362 0 => {
363 color_grading.global.exposure += dt;
364 }
365 1 => {
366 color_grading
367 .all_sections_mut()
368 .for_each(|section| section.gamma += dt);
369 }
370 2 => {
371 color_grading
372 .all_sections_mut()
373 .for_each(|section| section.saturation += dt);
374 }
375 3 => {
376 color_grading.global.post_saturation += dt;
377 }
378 _ => {}
379 }
380 }
381
382 if keys.just_pressed(KeyCode::Space) {
383 for (_, grading) in per_method_settings.settings.iter_mut() {
384 *grading = ColorGrading::default();
385 }
386 }
387
388 if keys.just_pressed(KeyCode::Enter) && current_scene.0 == 1 {
389 for (mapper, grading) in per_method_settings.settings.iter_mut() {
390 *grading = PerMethodSettings::basic_scene_recommendation(*mapper);
391 }
392 }
393}Trait Implementations
Sourceimpl Clone for ColorGrading
impl Clone for ColorGrading
Sourcefn clone(&self) -> ColorGrading
fn clone(&self) -> ColorGrading
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Component for ColorGrading
impl Component for ColorGrading
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<ColorGrading>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<ColorGrading>>
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceimpl Debug for ColorGrading
impl Debug for ColorGrading
Sourceimpl Default for ColorGrading
impl Default for ColorGrading
Sourcefn default() -> ColorGrading
fn default() -> ColorGrading
Sourceimpl From<ColorGrading> for ColorGradingUniform
impl From<ColorGrading> for ColorGradingUniform
Sourcefn from(component: ColorGrading) -> ColorGradingUniform
fn from(component: ColorGrading) -> ColorGradingUniform
Sourceimpl FromArg for ColorGrading
impl FromArg for ColorGrading
Sourceimpl FromReflect for ColorGrading
impl FromReflect for ColorGrading
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<ColorGrading>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<ColorGrading>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for ColorGrading
impl GetOwnership for ColorGrading
Sourceimpl GetTypeRegistration for ColorGrading
impl GetTypeRegistration for ColorGrading
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for ColorGrading
impl IntoReturn for ColorGrading
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
ColorGrading: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
ColorGrading: 'into_return,
Sourceimpl PartialReflect for ColorGrading
impl PartialReflect for ColorGrading
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<ColorGrading>) -> ReflectOwned
fn reflect_owned(self: Box<ColorGrading>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<ColorGrading>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<ColorGrading>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<ColorGrading>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<ColorGrading>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for ColorGrading
impl Reflect for ColorGrading
Sourcefn into_any(self: Box<ColorGrading>) -> Box<dyn Any>
fn into_any(self: Box<ColorGrading>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<ColorGrading>) -> Box<dyn Reflect>
fn into_reflect(self: Box<ColorGrading>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl Struct for ColorGrading
impl Struct for ColorGrading
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.Sourceimpl TypePath for ColorGrading
impl TypePath for ColorGrading
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>