pub struct ExtractedView {
pub retained_view_entity: RetainedViewEntity,
pub clip_from_view: Mat4,
pub world_from_view: GlobalTransform,
pub clip_from_world: Option<Mat4>,
pub hdr: bool,
pub viewport: UVec4,
pub color_grading: ColorGrading,
pub invert_culling: bool,
}Expand description
Describes a camera in the render world.
Each entity in the main world can potentially extract to multiple subviews,
each of which has a RetainedViewEntity::subview_index. For instance, 3D
cameras extract to both a 3D camera subview with index 0 and a special UI
subview with index 1. Likewise, point lights with shadows extract to 6
subviews, one for each side of the shadow cubemap.
Fields
retained_view_entity: RetainedViewEntityThe entity in the main world corresponding to this render world view.
clip_from_view: Mat4Typically a column-major right-handed projection matrix, one of either:
Perspective (infinite reverse z)
f = 1 / tan(fov_y_radians / 2)
⎡ f / aspect 0 0 0 ⎤
⎢ 0 f 0 0 ⎥
⎢ 0 0 0 near ⎥
⎣ 0 0 -1 0 ⎦Orthographic
w = right - left
h = top - bottom
d = far - near
cw = -right - left
ch = -top - bottom
⎡ 2 / w 0 0 cw / w ⎤
⎢ 0 2 / h 0 ch / h ⎥
⎢ 0 0 1 / d far / d ⎥
⎣ 0 0 0 1 ⎦clip_from_view[3][3] == 1.0 is the standard way to check if a projection is orthographic
Glam matrices are column major, so for example getting the near plane of a perspective projection is clip_from_view[3][2]
Custom projections are also possible however.
world_from_view: GlobalTransformclip_from_world: Option<Mat4>hdr: boolviewport: UVec4color_grading: ColorGradinginvert_culling: boolWhether to switch culling mode so that materials that request backface culling cull front faces, and vice versa.
This is typically used for cameras that mirror the world that they render across a plane, because doing that flips the winding of each polygon.
This setting doesn’t affect materials that disable backface culling.
Implementations
Sourceimpl ExtractedView
impl ExtractedView
Sourcepub fn rangefinder3d(&self) -> ViewRangefinder3d
pub fn rangefinder3d(&self) -> ViewRangefinder3d
Creates a 3D rangefinder for a view