pub struct ScreenshotCaptured {
pub entity: Entity,
pub image: Image,
}Fields
entity: Entityimage: ImageMethods from Deref<Target = Image>
Sourcepub fn width(&self) -> u32
pub fn width(&self) -> u32
Returns the width of a 2D image.
Examples found in repository?
39fn atlas_render_system(
40 mut commands: Commands,
41 mut state: ResMut<State>,
42 font_atlas_set: Res<FontAtlasSet>,
43 images: Res<Assets<Image>>,
44) {
45 if let Some(font_atlases) = font_atlas_set.values().next() {
46 let x_offset = state.atlas_count as f32;
47 if state.atlas_count == font_atlases.len() as u32 {
48 return;
49 }
50 let font_atlas = &font_atlases[state.atlas_count as usize];
51 let image = images.get(&font_atlas.texture).unwrap();
52 state.atlas_count += 1;
53 commands.spawn((
54 ImageNode::new(font_atlas.texture.clone()),
55 Node {
56 position_type: PositionType::Absolute,
57 top: Val::ZERO,
58 left: px(image.width() as f32 * x_offset),
59 ..default()
60 },
61 ));
62 }
63}More examples
148fn asset_loaded(
149 asset_server: Res<AssetServer>,
150 mut images: ResMut<Assets<Image>>,
151 mut cubemap: ResMut<Cubemap>,
152 mut skyboxes: Query<&mut Skybox>,
153) {
154 if !cubemap.is_loaded && asset_server.load_state(&cubemap.image_handle).is_loaded() {
155 info!("Swapping to {}...", CUBEMAPS[cubemap.index].0);
156 let mut image = images.get_mut(&cubemap.image_handle).unwrap();
157 // NOTE: PNGs do not have any metadata that could indicate they contain a cubemap texture,
158 // so they appear as one texture. The following code reconfigures the texture as necessary.
159 if image.texture_descriptor.array_layer_count() == 1 {
160 let layers = image.height() / image.width();
161 image
162 .reinterpret_stacked_2d_as_array(layers)
163 .expect("asset should be 2d texture and height will always be evenly divisible with the given layers");
164 image.texture_view_descriptor = Some(TextureViewDescriptor {
165 dimension: Some(TextureViewDimension::Cube),
166 ..default()
167 });
168 }
169
170 for mut skybox in &mut skyboxes {
171 skybox.image = Some(cubemap.image_handle.clone());
172 }
173
174 cubemap.is_loaded = true;
175 }
176}450fn update(
451 images_to_save: Query<&ImageToSave>,
452 receiver: Res<MainWorldReceiver>,
453 mut images: ResMut<Assets<Image>>,
454 mut scene_controller: ResMut<SceneController>,
455 mut app_exit_writer: MessageWriter<AppExit>,
456 mut file_number: Local<u32>,
457) {
458 if let SceneState::Render(n) = scene_controller.state {
459 if n < 1 {
460 // We don't want to block the main world on this,
461 // so we use try_recv which attempts to receive without blocking
462 let mut image_data = Vec::new();
463 while let Ok(data) = receiver.try_recv() {
464 // image generation could be faster than saving to fs,
465 // that's why use only last of them
466 image_data = data;
467 }
468 if !image_data.is_empty() {
469 for image in images_to_save.iter() {
470 // Fill correct data from channel to image
471 let mut img_bytes = images.get_mut(image.id()).unwrap();
472
473 // We need to ensure that this works regardless of the image dimensions
474 // If the image became wider when copying from the texture to the buffer,
475 // then the data is reduced to its original size when copying from the buffer to the image.
476 let row_bytes = img_bytes.width() as usize
477 * img_bytes.texture_descriptor.format.pixel_size().unwrap();
478 let aligned_row_bytes = RenderDevice::align_copy_bytes_per_row(row_bytes);
479 if row_bytes == aligned_row_bytes {
480 img_bytes.data.as_mut().unwrap().clone_from(&image_data);
481 } else {
482 // shrink data to original image size
483 img_bytes.data = Some(
484 image_data
485 .chunks(aligned_row_bytes)
486 .take(img_bytes.height() as usize)
487 .flat_map(|row| &row[..row_bytes.min(row.len())])
488 .cloned()
489 .collect(),
490 );
491 }
492
493 // Create RGBA Image Buffer
494 let img = match img_bytes.clone().try_into_dynamic() {
495 Ok(img) => img.to_rgba8(),
496 Err(e) => panic!("Failed to create image buffer {e:?}"),
497 };
498
499 // Prepare directory for images, test_images in bevy folder is used here for example
500 // You should choose the path depending on your needs
501 let images_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_images");
502 info!("Saving image to: {images_dir:?}");
503 std::fs::create_dir_all(&images_dir).unwrap();
504
505 // Choose filename starting from 000.png
506 let image_path = images_dir.join(format!("{:03}.png", file_number.deref()));
507 *file_number.deref_mut() += 1;
508
509 // Finally saving image to file, this heavy blocking operation is kept here
510 // for example simplicity, but in real app you should move it to a separate task
511 if let Err(e) = img.save(image_path) {
512 panic!("Failed to save image: {e}");
513 };
514 }
515 if scene_controller.single_image {
516 app_exit_writer.write(AppExit::Success);
517 }
518 }
519 } else {
520 // clears channel for skipped frames
521 while receiver.try_recv().is_ok() {}
522 scene_controller.state = SceneState::Render(n - 1);
523 }
524 }
525}Sourcepub fn height(&self) -> u32
pub fn height(&self) -> u32
Returns the height of a 2D image.
Examples found in repository?
148fn asset_loaded(
149 asset_server: Res<AssetServer>,
150 mut images: ResMut<Assets<Image>>,
151 mut cubemap: ResMut<Cubemap>,
152 mut skyboxes: Query<&mut Skybox>,
153) {
154 if !cubemap.is_loaded && asset_server.load_state(&cubemap.image_handle).is_loaded() {
155 info!("Swapping to {}...", CUBEMAPS[cubemap.index].0);
156 let mut image = images.get_mut(&cubemap.image_handle).unwrap();
157 // NOTE: PNGs do not have any metadata that could indicate they contain a cubemap texture,
158 // so they appear as one texture. The following code reconfigures the texture as necessary.
159 if image.texture_descriptor.array_layer_count() == 1 {
160 let layers = image.height() / image.width();
161 image
162 .reinterpret_stacked_2d_as_array(layers)
163 .expect("asset should be 2d texture and height will always be evenly divisible with the given layers");
164 image.texture_view_descriptor = Some(TextureViewDescriptor {
165 dimension: Some(TextureViewDimension::Cube),
166 ..default()
167 });
168 }
169
170 for mut skybox in &mut skyboxes {
171 skybox.image = Some(cubemap.image_handle.clone());
172 }
173
174 cubemap.is_loaded = true;
175 }
176}More examples
450fn update(
451 images_to_save: Query<&ImageToSave>,
452 receiver: Res<MainWorldReceiver>,
453 mut images: ResMut<Assets<Image>>,
454 mut scene_controller: ResMut<SceneController>,
455 mut app_exit_writer: MessageWriter<AppExit>,
456 mut file_number: Local<u32>,
457) {
458 if let SceneState::Render(n) = scene_controller.state {
459 if n < 1 {
460 // We don't want to block the main world on this,
461 // so we use try_recv which attempts to receive without blocking
462 let mut image_data = Vec::new();
463 while let Ok(data) = receiver.try_recv() {
464 // image generation could be faster than saving to fs,
465 // that's why use only last of them
466 image_data = data;
467 }
468 if !image_data.is_empty() {
469 for image in images_to_save.iter() {
470 // Fill correct data from channel to image
471 let mut img_bytes = images.get_mut(image.id()).unwrap();
472
473 // We need to ensure that this works regardless of the image dimensions
474 // If the image became wider when copying from the texture to the buffer,
475 // then the data is reduced to its original size when copying from the buffer to the image.
476 let row_bytes = img_bytes.width() as usize
477 * img_bytes.texture_descriptor.format.pixel_size().unwrap();
478 let aligned_row_bytes = RenderDevice::align_copy_bytes_per_row(row_bytes);
479 if row_bytes == aligned_row_bytes {
480 img_bytes.data.as_mut().unwrap().clone_from(&image_data);
481 } else {
482 // shrink data to original image size
483 img_bytes.data = Some(
484 image_data
485 .chunks(aligned_row_bytes)
486 .take(img_bytes.height() as usize)
487 .flat_map(|row| &row[..row_bytes.min(row.len())])
488 .cloned()
489 .collect(),
490 );
491 }
492
493 // Create RGBA Image Buffer
494 let img = match img_bytes.clone().try_into_dynamic() {
495 Ok(img) => img.to_rgba8(),
496 Err(e) => panic!("Failed to create image buffer {e:?}"),
497 };
498
499 // Prepare directory for images, test_images in bevy folder is used here for example
500 // You should choose the path depending on your needs
501 let images_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("test_images");
502 info!("Saving image to: {images_dir:?}");
503 std::fs::create_dir_all(&images_dir).unwrap();
504
505 // Choose filename starting from 000.png
506 let image_path = images_dir.join(format!("{:03}.png", file_number.deref()));
507 *file_number.deref_mut() += 1;
508
509 // Finally saving image to file, this heavy blocking operation is kept here
510 // for example simplicity, but in real app you should move it to a separate task
511 if let Err(e) = img.save(image_path) {
512 panic!("Failed to save image: {e}");
513 };
514 }
515 if scene_controller.single_image {
516 app_exit_writer.write(AppExit::Success);
517 }
518 }
519 } else {
520 // clears channel for skipped frames
521 while receiver.try_recv().is_ok() {}
522 scene_controller.state = SceneState::Render(n - 1);
523 }
524 }
525}Sourcepub fn aspect_ratio(&self) -> AspectRatio
pub fn aspect_ratio(&self) -> AspectRatio
Returns the aspect ratio (width / height) of a 2D image.
Sourcepub fn size_f32(&self) -> Vec2
pub fn size_f32(&self) -> Vec2
Returns the size of a 2D image as f32.
Examples found in repository?
227fn resize_image(
228 image_mesh: Query<(&MeshMaterial3d<StandardMaterial>, &Mesh3d), With<HDRViewer>>,
229 materials: Res<Assets<StandardMaterial>>,
230 mut meshes: ResMut<Assets<Mesh>>,
231 images: Res<Assets<Image>>,
232 mut image_event_reader: MessageReader<AssetEvent<Image>>,
233) {
234 for event in image_event_reader.read() {
235 let (AssetEvent::Added { id } | AssetEvent::Modified { id }) = event else {
236 continue;
237 };
238
239 for (mat_h, mesh_h) in &image_mesh {
240 let Some(mat) = materials.get(mat_h) else {
241 continue;
242 };
243
244 let Some(ref base_color_texture) = mat.base_color_texture else {
245 continue;
246 };
247
248 if *id != base_color_texture.id() {
249 continue;
250 };
251
252 let Some(image_changed) = images.get(*id) else {
253 continue;
254 };
255
256 let size = image_changed.size_f32().normalize_or_zero() * 1.4;
257 // Resize Mesh
258 let quad = Mesh::from(Rectangle::from_size(size));
259 meshes.insert(mesh_h, quad).unwrap();
260 }
261 }
262}Sourcepub fn resize(&mut self, size: Extent3d)
pub fn resize(&mut self, size: Extent3d)
Resizes the image to the new size, by removing information or appending 0 to the data.
Does not properly scale the contents of the image.
If you need to keep pixel data intact, use Image::resize_in_place.
Examples found in repository?
84fn setup_camera(mut commands: Commands, mut images: ResMut<Assets<Image>>) {
85 let canvas_size = Extent3d {
86 width: RES_WIDTH,
87 height: RES_HEIGHT,
88 ..default()
89 };
90
91 // This Image serves as a canvas representing the low-resolution game screen
92 let mut canvas = Image {
93 texture_descriptor: TextureDescriptor {
94 label: None,
95 size: canvas_size,
96 dimension: TextureDimension::D2,
97 format: TextureFormat::Bgra8UnormSrgb,
98 mip_level_count: 1,
99 sample_count: 1,
100 usage: TextureUsages::TEXTURE_BINDING
101 | TextureUsages::COPY_DST
102 | TextureUsages::RENDER_ATTACHMENT,
103 view_formats: &[],
104 },
105 ..default()
106 };
107
108 // Fill image.data with zeroes
109 canvas.resize(canvas_size);
110
111 let image_handle = images.add(canvas);
112
113 // This camera renders whatever is on `PIXEL_PERFECT_LAYERS` to the canvas
114 commands.spawn((
115 Camera2d,
116 Camera {
117 // Render before the "main pass" camera
118 order: -1,
119 clear_color: ClearColorConfig::Custom(GRAY.into()),
120 ..default()
121 },
122 RenderTarget::Image(image_handle.clone().into()),
123 Msaa::Off,
124 InGameCamera,
125 PIXEL_PERFECT_LAYERS,
126 ));
127
128 // Spawn the canvas
129 commands.spawn((Sprite::from_image(image_handle), Canvas, HIGH_RES_LAYERS));
130
131 // The "outer" camera renders whatever is on `HIGH_RES_LAYERS` to the screen.
132 // here, the canvas and one of the sample sprites will be rendered by this camera
133 commands.spawn((Camera2d, Msaa::Off, OuterCamera, HIGH_RES_LAYERS));
134}Sourcepub fn reinterpret_size(
&mut self,
new_size: Extent3d,
) -> Result<(), TextureReinterpretationError>
pub fn reinterpret_size( &mut self, new_size: Extent3d, ) -> Result<(), TextureReinterpretationError>
Changes the size if the total number of data elements (pixels) remains the same.
If not, returns TextureReinterpretationError::IncompatibleSizes.
Sourcepub fn resize_in_place(&mut self, new_size: Extent3d)
pub fn resize_in_place(&mut self, new_size: Extent3d)
Resizes the image to the new size, keeping the pixel data intact, anchored at the top-left. When growing, the new space is filled with 0. When shrinking, the image is clipped.
For faster resizing when keeping pixel data intact is not important, use Image::resize.
Sourcepub fn reinterpret_stacked_2d_as_array(
&mut self,
layers: u32,
) -> Result<(), TextureReinterpretationError>
pub fn reinterpret_stacked_2d_as_array( &mut self, layers: u32, ) -> Result<(), TextureReinterpretationError>
Takes a 2D image containing vertically stacked images of the same size, and reinterprets
it as a 2D array texture, where each of the stacked images becomes one layer of the
array. This is primarily for use with the texture2DArray shader uniform type.
Errors
Returns TextureReinterpretationError if the texture is not 2D, has more than one layers
or is not evenly dividable into the layers.
Examples found in repository?
148fn asset_loaded(
149 asset_server: Res<AssetServer>,
150 mut images: ResMut<Assets<Image>>,
151 mut cubemap: ResMut<Cubemap>,
152 mut skyboxes: Query<&mut Skybox>,
153) {
154 if !cubemap.is_loaded && asset_server.load_state(&cubemap.image_handle).is_loaded() {
155 info!("Swapping to {}...", CUBEMAPS[cubemap.index].0);
156 let mut image = images.get_mut(&cubemap.image_handle).unwrap();
157 // NOTE: PNGs do not have any metadata that could indicate they contain a cubemap texture,
158 // so they appear as one texture. The following code reconfigures the texture as necessary.
159 if image.texture_descriptor.array_layer_count() == 1 {
160 let layers = image.height() / image.width();
161 image
162 .reinterpret_stacked_2d_as_array(layers)
163 .expect("asset should be 2d texture and height will always be evenly divisible with the given layers");
164 image.texture_view_descriptor = Some(TextureViewDescriptor {
165 dimension: Some(TextureViewDimension::Cube),
166 ..default()
167 });
168 }
169
170 for mut skybox in &mut skyboxes {
171 skybox.image = Some(cubemap.image_handle.clone());
172 }
173
174 cubemap.is_loaded = true;
175 }
176}Sourcepub fn create_stacked_array_from_2d_grid(
&self,
rows: u32,
columns: u32,
) -> Result<Image, TextureReinterpretationError>
pub fn create_stacked_array_from_2d_grid( &self, rows: u32, columns: u32, ) -> Result<Image, TextureReinterpretationError>
Returns a newly constructed 2D image using the same properties as &self from a grid of tiles of the specified size, The new image is constructed in a vertical stack of tiles to be used as a 2D array texture.
This is primarily for preparing grid based tilesets.
Errors
Returns TextureReinterpretationError if the texture is not 2D, has more than one layers
or is not evenly dividable by size_in_tiles.
Sourcepub fn convert(&self, new_format: TextureFormat) -> Option<Image>
pub fn convert(&self, new_format: TextureFormat) -> Option<Image>
Convert a texture from a format to another. Only a few formats are supported as input and output:
TextureFormat::R8UnormTextureFormat::Rg8UnormTextureFormat::Rgba8UnormSrgb
To get Image as a image::DynamicImage see:
Image::try_into_dynamic.
Sourcepub fn is_compressed(&self) -> bool
pub fn is_compressed(&self) -> bool
Whether the texture format is compressed or uncompressed
Sourcepub fn pixel_data_offset(
&self,
coords: UVec3,
) -> Result<usize, TextureAccessError>
pub fn pixel_data_offset( &self, coords: UVec3, ) -> Result<usize, TextureAccessError>
Compute the byte offset where the data of a specific pixel is stored
Returns an error if the provided coordinates are out of bounds.
For 2D textures, Z is the layer number. For 1D textures, Y and Z are ignored.
Sourcepub fn pixel_bytes(&self, coords: UVec3) -> Result<&[u8], TextureAccessError>
pub fn pixel_bytes(&self, coords: UVec3) -> Result<&[u8], TextureAccessError>
Get a reference to the data bytes where a specific pixel’s value is stored.
Sourcepub fn pixel_bytes_mut(
&mut self,
coords: UVec3,
) -> Result<&mut [u8], TextureAccessError>
pub fn pixel_bytes_mut( &mut self, coords: UVec3, ) -> Result<&mut [u8], TextureAccessError>
Get a mutable reference to the data bytes where a specific pixel’s value is stored.
Examples found in repository?
38fn setup(mut commands: Commands, mut images: ResMut<Assets<Image>>) {
39 commands.spawn(Camera2d);
40
41 // Create an image that we are going to draw into
42 let mut image = Image::new_fill(
43 // 2D image of size 256x256
44 Extent3d {
45 width: IMAGE_WIDTH,
46 height: IMAGE_HEIGHT,
47 depth_or_array_layers: 1,
48 },
49 TextureDimension::D2,
50 // Initialize it with a beige color
51 &(css::BEIGE.to_u8_array()),
52 // Use the same encoding as the color we set
53 TextureFormat::Rgba8UnormSrgb,
54 RenderAssetUsages::MAIN_WORLD | RenderAssetUsages::RENDER_WORLD,
55 );
56
57 // To make it extra fancy, we can set the Alpha of each pixel,
58 // so that it fades out in a circular fashion.
59 for y in 0..IMAGE_HEIGHT {
60 for x in 0..IMAGE_WIDTH {
61 let center = Vec2::new(IMAGE_WIDTH as f32 / 2.0, IMAGE_HEIGHT as f32 / 2.0);
62 let max_radius = IMAGE_HEIGHT.min(IMAGE_WIDTH) as f32 / 2.0;
63 let r = Vec2::new(x as f32, y as f32).distance(center);
64 let a = 1.0 - (r / max_radius).clamp(0.0, 1.0);
65
66 // Here we will set the A value by accessing the raw data bytes.
67 // (it is the 4th byte of each pixel, as per our `TextureFormat`)
68
69 // Find our pixel by its coordinates
70 let pixel_bytes = image.pixel_bytes_mut(UVec3::new(x, y, 0)).unwrap();
71 // Convert our f32 to u8
72 pixel_bytes[3] = (a * u8::MAX as f32) as u8;
73 }
74 }
75
76 // Add it to Bevy's assets, so it can be used for rendering
77 // this will give us a handle we can use
78 // (to display it in a sprite, or as part of UI, etc.)
79 let handle = images.add(image);
80
81 // Create a sprite entity using our image
82 commands.spawn(Sprite::from_image(handle.clone()));
83 commands.insert_resource(MyProcGenImage(handle));
84
85 // We're seeding the PRNG here to make this example deterministic for testing purposes.
86 // This isn't strictly required in practical use unless you need your app to be deterministic.
87 let seeded_rng = ChaCha8Rng::seed_from_u64(19878367467712);
88 commands.insert_resource(SeededRng(seeded_rng));
89}Sourcepub fn clear(&mut self, pixel: &[u8])
pub fn clear(&mut self, pixel: &[u8])
Clears the content of the image with the given pixel. The image needs to be initialized on the cpu otherwise this is a noop.
This does nothing if the image data is not already initialized
Sourcepub fn get_color_at_1d(&self, x: u32) -> Result<Color, TextureAccessError>
pub fn get_color_at_1d(&self, x: u32) -> Result<Color, TextureAccessError>
Read the color of a specific pixel (1D texture).
See get_color_at for more details.
Sourcepub fn get_color_at(&self, x: u32, y: u32) -> Result<Color, TextureAccessError>
pub fn get_color_at(&self, x: u32, y: u32) -> Result<Color, TextureAccessError>
Read the color of a specific pixel (2D texture).
This function will find the raw byte data of a specific pixel and
decode it into a user-friendly Color struct for you.
Supports many of the common TextureFormats:
- RGBA/BGRA 8-bit unsigned integer, both sRGB and Linear
- 16-bit and 32-bit unsigned integer
- 16-bit and 32-bit float
Be careful: as the data is converted to Color (which uses f32 internally),
there may be issues with precision when using non-f32 TextureFormats.
If you read a value you previously wrote using set_color_at, it will not match.
If you are working with a 32-bit integer TextureFormat, the value will be
inaccurate (as f32 does not have enough bits to represent it exactly).
Single channel (R) formats are assumed to represent grayscale, so the value
will be copied to all three RGB channels in the resulting Color.
Other TextureFormats are unsupported, such as:
- block-compressed formats
- non-byte-aligned formats like 10-bit
- signed integer formats
Examples found in repository?
92fn draw(
93 my_handle: Res<MyProcGenImage>,
94 mut images: ResMut<Assets<Image>>,
95 // Used to keep track of where we are
96 mut i: Local<u32>,
97 mut draw_color: Local<Color>,
98 mut seeded_rng: ResMut<SeededRng>,
99) {
100 if *i == 0 {
101 // Generate a random color on first run.
102 *draw_color = Color::linear_rgb(
103 seeded_rng.0.random(),
104 seeded_rng.0.random(),
105 seeded_rng.0.random(),
106 );
107 }
108
109 // Get the image from Bevy's asset storage.
110 let mut image = images.get_mut(&my_handle.0).expect("Image not found");
111
112 // Compute the position of the pixel to draw.
113
114 let center = Vec2::new(IMAGE_WIDTH as f32 / 2.0, IMAGE_HEIGHT as f32 / 2.0);
115 let max_radius = IMAGE_HEIGHT.min(IMAGE_WIDTH) as f32 / 2.0;
116 let rot_speed = 0.0123;
117 let period = 0.12345;
118
119 let r = ops::sin(*i as f32 * period) * max_radius;
120 let xy = Vec2::from_angle(*i as f32 * rot_speed) * r + center;
121 let (x, y) = (xy.x as u32, xy.y as u32);
122
123 // Get the old color of that pixel.
124 let old_color = image.get_color_at(x, y).unwrap();
125
126 // If the old color is our current color, change our drawing color.
127 let tolerance = 1.0 / 255.0;
128 if old_color.distance(&draw_color) <= tolerance {
129 *draw_color = Color::linear_rgb(
130 seeded_rng.0.random(),
131 seeded_rng.0.random(),
132 seeded_rng.0.random(),
133 );
134 }
135
136 // Set the new color, but keep old alpha value from image.
137 image
138 .set_color_at(x, y, draw_color.with_alpha(old_color.alpha()))
139 .unwrap();
140
141 *i += 1;
142}Sourcepub fn get_color_at_3d(
&self,
x: u32,
y: u32,
z: u32,
) -> Result<Color, TextureAccessError>
pub fn get_color_at_3d( &self, x: u32, y: u32, z: u32, ) -> Result<Color, TextureAccessError>
Read the color of a specific pixel (2D texture with layers or 3D texture).
See get_color_at for more details.
Sourcepub fn set_color_at_1d(
&mut self,
x: u32,
color: Color,
) -> Result<(), TextureAccessError>
pub fn set_color_at_1d( &mut self, x: u32, color: Color, ) -> Result<(), TextureAccessError>
Change the color of a specific pixel (1D texture).
See set_color_at for more details.
Sourcepub fn set_color_at(
&mut self,
x: u32,
y: u32,
color: Color,
) -> Result<(), TextureAccessError>
pub fn set_color_at( &mut self, x: u32, y: u32, color: Color, ) -> Result<(), TextureAccessError>
Change the color of a specific pixel (2D texture).
This function will find the raw byte data of a specific pixel and
change it according to a Color you provide. The Color struct
will be encoded into the Image’s TextureFormat.
Supports many of the common TextureFormats:
- RGBA/BGRA 8-bit unsigned integer, both sRGB and Linear
- 16-bit and 32-bit unsigned integer (with possibly-limited precision, as
Colorusesf32) - 16-bit and 32-bit float
Be careful: writing to non-f32 TextureFormats is lossy! The data has to be converted,
so if you read it back using get_color_at, the Color you get will not equal the value
you used when writing it using this function.
For RG formats, only the respective values from the linear RGB Color will be used.
For R formats the linear RGB Color will be converted to grayscale
and the R channel will be the luminance.
Other TextureFormats are unsupported, such as:
- block-compressed formats
- non-byte-aligned formats like 10-bit
- signed integer formats
Examples found in repository?
208fn try_plot(
209 event: On<TryPlot>,
210 sprite: Query<(&Sprite, &Anchor, &GlobalTransform), With<SpriteToSave>>,
211 camera: Single<(&Camera, &GlobalTransform)>,
212 texture_atlases: Res<Assets<TextureAtlasLayout>>,
213 draw_color: Res<DrawColor>,
214 mut images: ResMut<Assets<Image>>,
215) {
216 let Ok((sprite, anchor, sprite_transform)) = sprite.get(event.entity) else {
217 return;
218 };
219 let (camera, camera_transform) = camera.into_inner();
220 let Ok(world_position) = camera.viewport_to_world_2d(camera_transform, event.location.position)
221 else {
222 return;
223 };
224 let relative_to_sprite = sprite_transform
225 .affine()
226 .inverse()
227 .transform_point3(world_position.extend(0.0));
228 let Ok(pixel_space) = sprite.compute_pixel_space_point(
229 relative_to_sprite.xy(),
230 *anchor,
231 &images,
232 &texture_atlases,
233 ) else {
234 return;
235 };
236 let pixel_coordinates = pixel_space.floor().as_uvec2();
237 let mut image = images.get_mut(&sprite.image).unwrap();
238 // For an actual drawing app, you'd at least draw a line from the last point, but this is
239 // simpler.
240 image
241 .set_color_at(pixel_coordinates.x, pixel_coordinates.y, draw_color.0)
242 .unwrap();
243}More examples
92fn draw(
93 my_handle: Res<MyProcGenImage>,
94 mut images: ResMut<Assets<Image>>,
95 // Used to keep track of where we are
96 mut i: Local<u32>,
97 mut draw_color: Local<Color>,
98 mut seeded_rng: ResMut<SeededRng>,
99) {
100 if *i == 0 {
101 // Generate a random color on first run.
102 *draw_color = Color::linear_rgb(
103 seeded_rng.0.random(),
104 seeded_rng.0.random(),
105 seeded_rng.0.random(),
106 );
107 }
108
109 // Get the image from Bevy's asset storage.
110 let mut image = images.get_mut(&my_handle.0).expect("Image not found");
111
112 // Compute the position of the pixel to draw.
113
114 let center = Vec2::new(IMAGE_WIDTH as f32 / 2.0, IMAGE_HEIGHT as f32 / 2.0);
115 let max_radius = IMAGE_HEIGHT.min(IMAGE_WIDTH) as f32 / 2.0;
116 let rot_speed = 0.0123;
117 let period = 0.12345;
118
119 let r = ops::sin(*i as f32 * period) * max_radius;
120 let xy = Vec2::from_angle(*i as f32 * rot_speed) * r + center;
121 let (x, y) = (xy.x as u32, xy.y as u32);
122
123 // Get the old color of that pixel.
124 let old_color = image.get_color_at(x, y).unwrap();
125
126 // If the old color is our current color, change our drawing color.
127 let tolerance = 1.0 / 255.0;
128 if old_color.distance(&draw_color) <= tolerance {
129 *draw_color = Color::linear_rgb(
130 seeded_rng.0.random(),
131 seeded_rng.0.random(),
132 seeded_rng.0.random(),
133 );
134 }
135
136 // Set the new color, but keep old alpha value from image.
137 image
138 .set_color_at(x, y, draw_color.with_alpha(old_color.alpha()))
139 .unwrap();
140
141 *i += 1;
142}Sourcepub fn set_color_at_3d(
&mut self,
x: u32,
y: u32,
z: u32,
color: Color,
) -> Result<(), TextureAccessError>
pub fn set_color_at_3d( &mut self, x: u32, y: u32, z: u32, color: Color, ) -> Result<(), TextureAccessError>
Change the color of a specific pixel (2D texture with layers or 3D texture).
See set_color_at for more details.