pub struct DynamicScene {
pub resources: Vec<Box<dyn PartialReflect>>,
pub entities: Vec<DynamicEntity>,
}Expand description
A collection of serializable resources and dynamic entities.
Each dynamic entity in the collection contains its own run-time defined set of components. To spawn a dynamic scene, you can use either:
SceneSpawner::spawn_dynamic- adding the
DynamicSceneRootcomponent to an entity. - using the
DynamicSceneBuilderto construct aDynamicScenefromWorld.
Fields
resources: Vec<Box<dyn PartialReflect>>Resources stored in the dynamic scene.
entities: Vec<DynamicEntity>Entities contained in the dynamic scene.
Implementations
Sourceimpl DynamicScene
impl DynamicScene
Sourcepub fn from_scene(scene: &Scene) -> DynamicScene
pub fn from_scene(scene: &Scene) -> DynamicScene
Create a new dynamic scene from a given scene.
Sourcepub fn from_world(world: &World) -> DynamicScene
pub fn from_world(world: &World) -> DynamicScene
Create a new dynamic scene from a given world.
Sourcepub fn write_to_world_with(
&self,
world: &mut World,
entity_map: &mut EntityHashMap<Entity>,
type_registry: &AppTypeRegistry,
) -> Result<(), SceneSpawnError>
pub fn write_to_world_with( &self, world: &mut World, entity_map: &mut EntityHashMap<Entity>, type_registry: &AppTypeRegistry, ) -> Result<(), SceneSpawnError>
Write the resources, the dynamic entities, and their corresponding components to the given world.
This method will return a SceneSpawnError if a type either is not registered
in the provided AppTypeRegistry resource, or doesn’t reflect the
Component or Resource trait.
Sourcepub fn write_to_world(
&self,
world: &mut World,
entity_map: &mut EntityHashMap<Entity>,
) -> Result<(), SceneSpawnError>
pub fn write_to_world( &self, world: &mut World, entity_map: &mut EntityHashMap<Entity>, ) -> Result<(), SceneSpawnError>
Write the resources, the dynamic entities, and their corresponding components to the given world.
This method will return a SceneSpawnError if a type either is not registered
in the world’s AppTypeRegistry resource, or doesn’t reflect the
Component trait.
Sourcepub fn serialize(&self, registry: &TypeRegistry) -> Result<String, Error>
Available on crate feature serialize only.
pub fn serialize(&self, registry: &TypeRegistry) -> Result<String, Error>
serialize only.Serialize this dynamic scene into the official Bevy scene format (.scn / .scn.ron).
The Bevy scene format is based on Rusty Object Notation (RON). It describes the scene
in a human-friendly format. To deserialize the scene, use the SceneLoader.