pub struct ScenePatch {
pub scene: Option<Box<dyn Scene>>,
pub dependencies: Vec<UntypedHandle>,
pub resolved: Option<Arc<ResolvedSceneRoot>>,
}Expand description
An Asset that holds a Scene, tracks its dependencies, and holds the ResolvedSceneRoot (after the Scene has been loaded and resolved).
Fields
scene: Option<Box<dyn Scene>>A Scene.
dependencies: Vec<UntypedHandle>The dependencies of scene (populated using Scene::register_dependencies). These are “asset dependencies” and will affect the load state.
resolved: Option<Arc<ResolvedSceneRoot>>The ResolvedSceneRoot, if exists. This is populated after the Scene has been loaded and resolved
Implementations
Sourceimpl ScenePatch
impl ScenePatch
Sourcepub fn load<P>(assets: &AssetServer, scene: P) -> ScenePatchwhere
P: Scene,
pub fn load<P>(assets: &AssetServer, scene: P) -> ScenePatchwhere
P: Scene,
Kicks off a load of the scene. This enumerates the scene’s dependencies using Scene::register_dependencies, loads
them using the given AssetServer, and assigns the resulting asset handles to ScenePatch::dependencies.
Sourcepub fn load_with<P>(
load_from_path: &mut impl LoadFromPath,
scene: P,
) -> ScenePatchwhere
P: Scene,
pub fn load_with<P>(
load_from_path: &mut impl LoadFromPath,
scene: P,
) -> ScenePatchwhere
P: Scene,
Same as Self::load, but allows passing in any LoadFromPath impl for more general
loading cases.
Sourcepub fn resolve(
&mut self,
assets: &AssetServer,
patches: &Assets<ScenePatch>,
) -> Result<(), ResolveSceneError>
pub fn resolve( &mut self, assets: &AssetServer, patches: &Assets<ScenePatch>, ) -> Result<(), ResolveSceneError>
Resolves the current scene (using Scene::resolve). This should only be called after every dependency has loaded from the scene’s
Scene::register_dependencies. If successful, it will store the resolved result in ScenePatch::resolved.
Sourcepub fn spawn<'w>(
&self,
world: &'w mut World,
) -> Result<EntityWorldMut<'w>, SpawnSceneError>
pub fn spawn<'w>( &self, world: &'w mut World, ) -> Result<EntityWorldMut<'w>, SpawnSceneError>
Spawns the scene in world as a new entity. This should only be called after ScenePatch::resolve.
Sourcepub fn apply<'w>(
&self,
entity: &'w mut EntityWorldMut<'_>,
) -> Result<(), SpawnSceneError>
pub fn apply<'w>( &self, entity: &'w mut EntityWorldMut<'_>, ) -> Result<(), SpawnSceneError>
Applies the scene to the given entity. This should only be called after ScenePatch::resolve