Crate settings

Crate settings 

Framework for saving and loading user settings files in Bevy applications.

Refer to SettingsPlugin for detailed usage information.

Structs

ReflectSettingsGroup
Reflected data from a SettingsGroup.
SaveSettingsDeferred
A Command which saves changed settings after a delay. This is debounced: issuing this command multiple times resets the delay timer each time. This is meant to be used for settings which change at a high frequency, such as dragging a slider which controls the game’s audio volume. The default delay is 1.0 seconds.
SettingsPlugin
Plugin to orchestrate loading and saving settings.

Enums

SaveSettings
A Command which saves settings to disk. Actual file system operations happen in another thread.
SaveSettingsSync
A Command which saves settings to disk. This blocks the command queue until saving is complete.

Traits

SettingsGroup
Trait which identifies a type as corresponding to a section with a settings file.

Derive Macros

SettingsGroup
Cheat sheet for derive syntax,