Ring

Struct Ring 

pub struct Ring<P>
where P: Primitive2d,
{ pub outer_shape: P, pub inner_shape: P, }
A 2D shape representing the ring version of a base shape.

The inner_shape forms the “hollow” of the outer_shape.

The resulting shapes are rings or hollow shapes. For example, a circle becomes an annulus.

Warning

The outer_shape must contain the inner_shape for the generated meshes to be accurate.

If there are vertices in the inner_shape that escape the outer_shape (for example, if the inner_shape is in fact larger), it may result in incorrect geometries.

Fields

outer_shape: P

The outer shape

inner_shape: P

The inner shape (the same shape of a different size)

Implementations

Source

impl<P> Ring<P>
where P: Primitive2d,

pub const fn new(outer_shape: P, inner_shape: P) -> Ring<P>

Create a new Ring from a given outer_shape and inner_shape.

If the primitive implements Inset and you would like a uniform thickness, consider using ToRing::to_ring

Examples found in repository?
examples/math/custom_primitives.rs (line 22)
22const RING: Ring<Heart> = Ring::new(HEART, HOLLOW);
More examples
Hide additional examples
examples/2d/2d_shapes.rs (lines 121-124)
68fn setup(
69    mut commands: Commands,
70    mut meshes: ResMut<Assets<Mesh>>,
71    mut materials: ResMut<Assets<ColorMaterial>>,
72) {
73    commands.spawn(Camera2d);
74
75    let shapes = [
76        meshes.add(Circle::new(50.0)),
77        meshes.add(CircularSector::new(50.0, 1.0)),
78        meshes.add(CircularSegment::new(50.0, 1.25)),
79        meshes.add(Ellipse::new(25.0, 50.0)),
80        meshes.add(Annulus::new(25.0, 50.0)),
81        meshes.add(Capsule2d::new(25.0, 50.0)),
82        meshes.add(Rhombus::new(75.0, 100.0)),
83        meshes.add(Rectangle::new(50.0, 100.0)),
84        meshes.add(RegularPolygon::new(50.0, 6)),
85        meshes.add(Triangle2d::new(
86            Vec2::Y * 50.0,
87            Vec2::new(-50.0, -50.0),
88            Vec2::new(50.0, -50.0),
89        )),
90        meshes.add(Segment2d::new(
91            Vec2::new(-50.0, 50.0),
92            Vec2::new(50.0, -50.0),
93        )),
94        meshes.add(Polyline2d::new(vec![
95            Vec2::new(-50.0, 50.0),
96            Vec2::new(0.0, -50.0),
97            Vec2::new(50.0, 50.0),
98        ])),
99    ];
100    let num_shapes = shapes.len();
101
102    for (i, shape) in shapes.into_iter().enumerate() {
103        // Distribute colors evenly across the rainbow.
104        let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
105
106        commands.spawn((
107            Mesh2d(shape),
108            MeshMaterial2d(materials.add(color)),
109            Transform::from_xyz(
110                // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
111                -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
112                Y_EXTENT / 2.,
113                0.0,
114            ),
115        ));
116    }
117
118    let rings = [
119        meshes.add(Circle::new(50.0).to_ring(THICKNESS)),
120        // this visually produces an arc segment but this is not technically accurate
121        meshes.add(Ring::new(
122            CircularSector::new(50.0, 1.0),
123            CircularSector::new(45.0, 1.0),
124        )),
125        meshes.add(CircularSegment::new(50.0, 1.25).to_ring(THICKNESS)),
126        meshes.add({
127            // This is an approximation; Ellipse does not implement Inset as concentric ellipses do not have parallel curves
128            let outer = Ellipse::new(25.0, 50.0);
129            let mut inner = outer;
130            inner.half_size -= Vec2::splat(THICKNESS);
131            Ring::new(outer, inner)
132        }),
133        // this is equivalent to the Annulus::new(25.0, 50.0) above
134        meshes.add(Ring::new(Circle::new(50.0), Circle::new(25.0))),
135        meshes.add(Capsule2d::new(25.0, 50.0).to_ring(THICKNESS)),
136        meshes.add(Rhombus::new(75.0, 100.0).to_ring(THICKNESS)),
137        meshes.add(Rectangle::new(50.0, 100.0).to_ring(THICKNESS)),
138        meshes.add(RegularPolygon::new(50.0, 6).to_ring(THICKNESS)),
139        meshes.add(
140            Triangle2d::new(
141                Vec2::Y * 50.0,
142                Vec2::new(-50.0, -50.0),
143                Vec2::new(50.0, -50.0),
144            )
145            .to_ring(THICKNESS),
146        ),
147    ];
148    // Allow for 2 empty spaces
149    let num_rings = rings.len() + 2;
150
151    for (i, shape) in rings.into_iter().enumerate() {
152        // Distribute colors evenly across the rainbow.
153        let color = Color::hsl(360. * i as f32 / num_rings as f32, 0.95, 0.7);
154
155        commands.spawn((
156            Mesh2d(shape),
157            MeshMaterial2d(materials.add(color)),
158            Transform::from_xyz(
159                // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
160                -X_EXTENT / 2. + i as f32 / (num_rings - 1) as f32 * X_EXTENT,
161                -Y_EXTENT / 2.,
162                0.0,
163            ),
164        ));
165    }
166    spawn_buttons(&mut commands);
167}
examples/3d/3d_shapes.rs (line 111)
57fn setup(
58    mut commands: Commands,
59    mut meshes: ResMut<Assets<Mesh>>,
60    mut images: ResMut<Assets<Image>>,
61    mut materials: ResMut<Assets<StandardMaterial>>,
62) {
63    let debug_material = materials.add(StandardMaterial {
64        base_color_texture: Some(images.add(uv_debug_texture())),
65        ..default()
66    });
67
68    let shapes = [
69        meshes.add(Cuboid::default()),
70        meshes.add(Tetrahedron::default()),
71        meshes.add(Capsule3d::default()),
72        meshes.add(Torus::default()),
73        meshes.add(Cylinder::default()),
74        meshes.add(Cone::default()),
75        meshes.add(ConicalFrustum::default()),
76        meshes.add(Sphere::default().mesh().ico(5).unwrap()),
77        meshes.add(Sphere::default().mesh().uv(32, 18)),
78        meshes.add(Segment3d::default()),
79        meshes.add(Polyline3d::new(vec![
80            Vec3::new(-0.5, 0.0, 0.0),
81            Vec3::new(0.5, 0.0, 0.0),
82            Vec3::new(0.0, 0.5, 0.0),
83        ])),
84    ];
85
86    let extrusions = [
87        meshes.add(Extrusion::new(Rectangle::default(), 1.)),
88        meshes.add(Extrusion::new(Capsule2d::default(), 1.)),
89        meshes.add(Extrusion::new(Annulus::default(), 1.)),
90        meshes.add(Extrusion::new(Circle::default(), 1.)),
91        meshes.add(Extrusion::new(Ellipse::default(), 1.)),
92        meshes.add(Extrusion::new(RegularPolygon::default(), 1.)),
93        meshes.add(Extrusion::new(Triangle2d::default(), 1.)),
94        meshes.add(Extrusion::new(
95            ConvexPolygon::new(vec![
96                Vec2::new(0.0, 0.8),
97                Vec2::new(-0.47, 0.25),
98                Vec2::new(-0.47, -0.65),
99                Vec2::new(0.47, -0.65),
100                Vec2::new(0.47, 0.25),
101            ])
102            .unwrap(),
103            1.0,
104        )),
105    ];
106
107    let ring_extrusions = [
108        meshes.add(Extrusion::new(Rectangle::default().to_ring(THICKNESS), 1.)),
109        meshes.add(Extrusion::new(Capsule2d::default().to_ring(THICKNESS), 1.)),
110        meshes.add(Extrusion::new(
111            Ring::new(Circle::new(1.0), Circle::new(0.5)),
112            1.,
113        )),
114        meshes.add(Extrusion::new(Circle::default().to_ring(THICKNESS), 1.)),
115        meshes.add(Extrusion::new(
116            {
117                // This is an approximation; Ellipse does not implement Inset as concentric ellipses do not have parallel curves
118                let outer = Ellipse::default();
119                let mut inner = outer;
120                inner.half_size -= Vec2::splat(THICKNESS);
121                Ring::new(outer, inner)
122            },
123            1.,
124        )),
125        meshes.add(Extrusion::new(
126            RegularPolygon::default().to_ring(THICKNESS),
127            1.,
128        )),
129        meshes.add(Extrusion::new(Triangle2d::default().to_ring(THICKNESS), 1.)),
130    ];
131
132    let num_shapes = shapes.len();
133
134    for (i, shape) in shapes.into_iter().enumerate() {
135        commands.spawn((
136            Mesh3d(shape),
137            MeshMaterial3d(debug_material.clone()),
138            Transform::from_xyz(
139                -SHAPES_X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * SHAPES_X_EXTENT,
140                2.0,
141                Row::Front.z(),
142            )
143            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
144            Shape,
145            Row::Front,
146        ));
147    }
148
149    let num_extrusions = extrusions.len();
150
151    for (i, shape) in extrusions.into_iter().enumerate() {
152        commands.spawn((
153            Mesh3d(shape),
154            MeshMaterial3d(debug_material.clone()),
155            Transform::from_xyz(
156                -EXTRUSION_X_EXTENT / 2.
157                    + i as f32 / (num_extrusions - 1) as f32 * EXTRUSION_X_EXTENT,
158                2.0,
159                Row::Middle.z(),
160            )
161            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
162            Shape,
163            Row::Middle,
164        ));
165    }
166
167    let num_ring_extrusions = ring_extrusions.len();
168
169    for (i, shape) in ring_extrusions.into_iter().enumerate() {
170        commands.spawn((
171            Mesh3d(shape),
172            MeshMaterial3d(debug_material.clone()),
173            Transform::from_xyz(
174                -EXTRUSION_X_EXTENT / 2.
175                    + i as f32 / (num_ring_extrusions - 1) as f32 * EXTRUSION_X_EXTENT,
176                2.0,
177                Row::Rear.z(),
178            )
179            .with_rotation(Quat::from_rotation_x(-PI / 4.)),
180            Shape,
181            Row::Rear,
182        ));
183    }
184
185    commands.spawn((
186        PointLight {
187            shadow_maps_enabled: true,
188            intensity: 10_000_000.,
189            range: 100.0,
190            shadow_depth_bias: 0.2,
191            ..default()
192        },
193        Transform::from_xyz(8.0, 16.0, 8.0),
194    ));
195
196    // ground plane
197    commands.spawn((
198        Mesh3d(meshes.add(Plane3d::default().mesh().size(50.0, 50.0).subdivisions(10))),
199        MeshMaterial3d(materials.add(Color::from(SILVER))),
200    ));
201
202    commands.spawn((
203        Camera3d::default(),
204        Transform::from_xyz(0.0, 7., 14.0).looking_at(Vec3::new(0., 1., 0.), Vec3::Y),
205    ));
206
207    let mut text = "\
208        Press 'R' to pause/resume rotation\n\
209        Press 'Tab' to cycle through rows"
210        .to_string();
211    #[cfg(not(target_arch = "wasm32"))]
212    text.push_str("\nPress 'Space' to toggle wireframes");
213
214    commands.spawn((
215        Text::new(text),
216        Node {
217            position_type: PositionType::Absolute,
218            top: px(12),
219            left: px(12),
220            ..default()
221        },
222    ));
223}
Source

impl<P> Ring<P>
where P: Primitive2d + Clone + Inset,

pub fn from_primitive_and_thickness(primitive: P, thickness: f32) -> Ring<P>

Generate a Ring from a given primitive and a thickness.