pub struct RegularPolygon {
pub circumcircle: Circle,
pub sides: u32,
}Expand description
A polygon centered on the origin where all vertices lie on a circle, equally far apart.
Fields
circumcircle: CircleThe circumcircle on which all vertices lie
sides: u32The number of sides
Implementations
Sourceimpl RegularPolygon
impl RegularPolygon
Sourcepub const fn new(circumradius: f32, sides: u32) -> RegularPolygon
pub const fn new(circumradius: f32, sides: u32) -> RegularPolygon
Create a new RegularPolygon
from the radius of the circumcircle and a number of sides
Panics
Panics if circumradius is negative
Examples found in repository?
104fn setup_scene(
105 mut commands: Commands,
106 mut meshes: ResMut<Assets<Mesh>>,
107 mut materials: ResMut<Assets<ColorMaterial>>,
108) {
109 commands.spawn(Camera2d);
110
111 let named_shapes = [
112 (Name::new("Annulus"), meshes.add(Annulus::new(25.0, 50.0))),
113 (
114 Name::new("Bestagon"),
115 meshes.add(RegularPolygon::new(50.0, 6)),
116 ),
117 (Name::new("Rhombus"), meshes.add(Rhombus::new(75.0, 100.0))),
118 ];
119 let num_shapes = named_shapes.len();
120
121 for (i, (name, shape)) in named_shapes.into_iter().enumerate() {
122 // Distribute colors evenly across the rainbow.
123 let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
124
125 commands.spawn((
126 name,
127 DisableOnClick,
128 Mesh2d(shape),
129 MeshMaterial2d(materials.add(color)),
130 Transform::from_xyz(
131 // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
132 -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
133 0.0,
134 0.0,
135 ),
136 ));
137 }
138}More examples
121 pub fn setup(
122 mut commands: Commands,
123 mut meshes: ResMut<Assets<Mesh>>,
124 mut materials: ResMut<Assets<ColorMaterial>>,
125 ) {
126 commands.spawn((Camera2d, DespawnOnExit(super::Scene::Shapes)));
127
128 let shapes = [
129 meshes.add(Circle::new(50.0)),
130 meshes.add(CircularSector::new(50.0, 1.0)),
131 meshes.add(CircularSegment::new(50.0, 1.25)),
132 meshes.add(Ellipse::new(25.0, 50.0)),
133 meshes.add(Annulus::new(25.0, 50.0)),
134 meshes.add(Capsule2d::new(25.0, 50.0)),
135 meshes.add(Rhombus::new(75.0, 100.0)),
136 meshes.add(Rectangle::new(50.0, 100.0)),
137 meshes.add(RegularPolygon::new(50.0, 6)),
138 meshes.add(Triangle2d::new(
139 Vec2::Y * 50.0,
140 Vec2::new(-50.0, -50.0),
141 Vec2::new(50.0, -50.0),
142 )),
143 ];
144 let num_shapes = shapes.len();
145
146 for (i, shape) in shapes.into_iter().enumerate() {
147 // Distribute colors evenly across the rainbow.
148 let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
149
150 commands.spawn((
151 Mesh2d(shape),
152 MeshMaterial2d(materials.add(color)),
153 Transform::from_xyz(
154 // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
155 -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
156 0.0,
157 0.0,
158 ),
159 DespawnOnExit(super::Scene::Shapes),
160 ));
161 }
162 }
163}
164
165mod bloom {
166 use bevy::{core_pipeline::tonemapping::Tonemapping, post_process::bloom::Bloom, prelude::*};
167
168 pub fn setup(
169 mut commands: Commands,
170 mut meshes: ResMut<Assets<Mesh>>,
171 mut materials: ResMut<Assets<ColorMaterial>>,
172 ) {
173 commands.spawn((
174 Camera2d,
175 Tonemapping::TonyMcMapface,
176 Bloom::default(),
177 DespawnOnExit(super::Scene::Bloom),
178 ));
179
180 commands.spawn((
181 Mesh2d(meshes.add(Circle::new(100.))),
182 MeshMaterial2d(materials.add(Color::srgb(7.5, 0.0, 7.5))),
183 Transform::from_translation(Vec3::new(-200., 0., 0.)),
184 DespawnOnExit(super::Scene::Bloom),
185 ));
186
187 commands.spawn((
188 Mesh2d(meshes.add(RegularPolygon::new(100., 6))),
189 MeshMaterial2d(materials.add(Color::srgb(6.25, 9.4, 9.1))),
190 Transform::from_translation(Vec3::new(200., 0., 0.)),
191 DespawnOnExit(super::Scene::Bloom),
192 ));
193 }202fn setup(mut commands: Commands) {
203 commands.spawn(Camera2d);
204
205 commands.spawn((
206 Transform::from_xyz(-OFFSET_X, OFFSET_Y, 0.),
207 Shape::Circle(Circle::new(45.)),
208 DesiredVolume::Aabb,
209 Intersects::default(),
210 ));
211
212 commands.spawn((
213 Transform::from_xyz(0., OFFSET_Y, 0.),
214 Shape::Rectangle(Rectangle::new(80., 80.)),
215 Spin,
216 DesiredVolume::Circle,
217 Intersects::default(),
218 ));
219
220 commands.spawn((
221 Transform::from_xyz(OFFSET_X, OFFSET_Y, 0.),
222 Shape::Triangle(Triangle2d::new(
223 Vec2::new(-40., -40.),
224 Vec2::new(-20., 40.),
225 Vec2::new(40., 50.),
226 )),
227 Spin,
228 DesiredVolume::Aabb,
229 Intersects::default(),
230 ));
231
232 commands.spawn((
233 Transform::from_xyz(-OFFSET_X, -OFFSET_Y, 0.),
234 Shape::Line(Segment2d::from_direction_and_length(
235 Dir2::from_xy(1., 0.3).unwrap(),
236 90.,
237 )),
238 Spin,
239 DesiredVolume::Circle,
240 Intersects::default(),
241 ));
242
243 commands.spawn((
244 Transform::from_xyz(0., -OFFSET_Y, 0.),
245 Shape::Capsule(Capsule2d::new(25., 50.)),
246 Spin,
247 DesiredVolume::Aabb,
248 Intersects::default(),
249 ));
250
251 commands.spawn((
252 Transform::from_xyz(OFFSET_X, -OFFSET_Y, 0.),
253 Shape::Polygon(RegularPolygon::new(50., 6)),
254 Spin,
255 DesiredVolume::Circle,
256 Intersects::default(),
257 ));
258
259 commands.spawn((
260 Text::default(),
261 Node {
262 position_type: PositionType::Absolute,
263 top: px(12),
264 left: px(12),
265 ..default()
266 },
267 ));
268}17fn setup(
18 mut commands: Commands,
19 mut meshes: ResMut<Assets<Mesh>>,
20 mut materials: ResMut<Assets<ColorMaterial>>,
21 asset_server: Res<AssetServer>,
22) {
23 commands.spawn((
24 Camera2d,
25 Camera {
26 clear_color: ClearColorConfig::Custom(Color::BLACK),
27 ..default()
28 },
29 Tonemapping::TonyMcMapface, // 1. Using a tonemapper that desaturates to white is recommended
30 Bloom::default(), // 2. Enable bloom for the camera
31 DebandDither::Enabled, // Optional: bloom causes gradients which cause banding
32 ));
33
34 // Sprite
35 commands.spawn(Sprite {
36 image: asset_server.load("branding/bevy_bird_dark.png"),
37 color: Color::srgb(5.0, 5.0, 5.0), // 3. Put something bright in a dark environment to see the effect
38 custom_size: Some(Vec2::splat(160.0)),
39 ..default()
40 });
41
42 // Circle mesh
43 commands.spawn((
44 Mesh2d(meshes.add(Circle::new(100.))),
45 // 3. Put something bright in a dark environment to see the effect
46 MeshMaterial2d(materials.add(Color::srgb(7.5, 0.0, 7.5))),
47 Transform::from_translation(Vec3::new(-200., 0., 0.)),
48 ));
49
50 // Hexagon mesh
51 commands.spawn((
52 Mesh2d(meshes.add(RegularPolygon::new(100., 6))),
53 // 3. Put something bright in a dark environment to see the effect
54 MeshMaterial2d(materials.add(Color::srgb(6.25, 9.4, 9.1))),
55 Transform::from_translation(Vec3::new(200., 0., 0.)),
56 ));
57
58 // UI
59 commands.spawn((
60 Text::default(),
61 Node {
62 position_type: PositionType::Absolute,
63 top: px(12),
64 left: px(12),
65 ..default()
66 },
67 ));
68}68fn setup(
69 mut commands: Commands,
70 mut meshes: ResMut<Assets<Mesh>>,
71 mut materials: ResMut<Assets<ColorMaterial>>,
72) {
73 commands.spawn(Camera2d);
74
75 let shapes = [
76 meshes.add(Circle::new(50.0)),
77 meshes.add(CircularSector::new(50.0, 1.0)),
78 meshes.add(CircularSegment::new(50.0, 1.25)),
79 meshes.add(Ellipse::new(25.0, 50.0)),
80 meshes.add(Annulus::new(25.0, 50.0)),
81 meshes.add(Capsule2d::new(25.0, 50.0)),
82 meshes.add(Rhombus::new(75.0, 100.0)),
83 meshes.add(Rectangle::new(50.0, 100.0)),
84 meshes.add(RegularPolygon::new(50.0, 6)),
85 meshes.add(Triangle2d::new(
86 Vec2::Y * 50.0,
87 Vec2::new(-50.0, -50.0),
88 Vec2::new(50.0, -50.0),
89 )),
90 meshes.add(Segment2d::new(
91 Vec2::new(-50.0, 50.0),
92 Vec2::new(50.0, -50.0),
93 )),
94 meshes.add(Polyline2d::new(vec![
95 Vec2::new(-50.0, 50.0),
96 Vec2::new(0.0, -50.0),
97 Vec2::new(50.0, 50.0),
98 ])),
99 ];
100 let num_shapes = shapes.len();
101
102 for (i, shape) in shapes.into_iter().enumerate() {
103 // Distribute colors evenly across the rainbow.
104 let color = Color::hsl(360. * i as f32 / num_shapes as f32, 0.95, 0.7);
105
106 commands.spawn((
107 Mesh2d(shape),
108 MeshMaterial2d(materials.add(color)),
109 Transform::from_xyz(
110 // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
111 -X_EXTENT / 2. + i as f32 / (num_shapes - 1) as f32 * X_EXTENT,
112 Y_EXTENT / 2.,
113 0.0,
114 ),
115 ));
116 }
117
118 let rings = [
119 meshes.add(Circle::new(50.0).to_ring(THICKNESS)),
120 // this visually produces an arc segment but this is not technically accurate
121 meshes.add(Ring::new(
122 CircularSector::new(50.0, 1.0),
123 CircularSector::new(45.0, 1.0),
124 )),
125 meshes.add(CircularSegment::new(50.0, 1.25).to_ring(THICKNESS)),
126 meshes.add({
127 // This is an approximation; Ellipse does not implement Inset as concentric ellipses do not have parallel curves
128 let outer = Ellipse::new(25.0, 50.0);
129 let mut inner = outer;
130 inner.half_size -= Vec2::splat(THICKNESS);
131 Ring::new(outer, inner)
132 }),
133 // this is equivalent to the Annulus::new(25.0, 50.0) above
134 meshes.add(Ring::new(Circle::new(50.0), Circle::new(25.0))),
135 meshes.add(Capsule2d::new(25.0, 50.0).to_ring(THICKNESS)),
136 meshes.add(Rhombus::new(75.0, 100.0).to_ring(THICKNESS)),
137 meshes.add(Rectangle::new(50.0, 100.0).to_ring(THICKNESS)),
138 meshes.add(RegularPolygon::new(50.0, 6).to_ring(THICKNESS)),
139 meshes.add(
140 Triangle2d::new(
141 Vec2::Y * 50.0,
142 Vec2::new(-50.0, -50.0),
143 Vec2::new(50.0, -50.0),
144 )
145 .to_ring(THICKNESS),
146 ),
147 ];
148 // Allow for 2 empty spaces
149 let num_rings = rings.len() + 2;
150
151 for (i, shape) in rings.into_iter().enumerate() {
152 // Distribute colors evenly across the rainbow.
153 let color = Color::hsl(360. * i as f32 / num_rings as f32, 0.95, 0.7);
154
155 commands.spawn((
156 Mesh2d(shape),
157 MeshMaterial2d(materials.add(color)),
158 Transform::from_xyz(
159 // Distribute shapes from -X_EXTENT/2 to +X_EXTENT/2.
160 -X_EXTENT / 2. + i as f32 / (num_rings - 1) as f32 * X_EXTENT,
161 -Y_EXTENT / 2.,
162 0.0,
163 ),
164 ));
165 }
166 spawn_buttons(&mut commands);
167}Sourcepub const fn circumradius(&self) -> f32
pub const fn circumradius(&self) -> f32
Get the radius of the circumcircle on which all vertices of the regular polygon lie
Sourcepub fn inradius(&self) -> f32
pub fn inradius(&self) -> f32
Get the inradius or apothem of the regular polygon. This is the radius of the largest circle that can be drawn within the polygon
Sourcepub fn side_length(&self) -> f32
pub fn side_length(&self) -> f32
Get the length of one side of the regular polygon
Sourcepub const fn internal_angle_degrees(&self) -> f32
pub const fn internal_angle_degrees(&self) -> f32
Get the internal angle of the regular polygon in degrees.
This is the angle formed by two adjacent sides with points within the angle being in the interior of the polygon
Sourcepub const fn internal_angle_radians(&self) -> f32
pub const fn internal_angle_radians(&self) -> f32
Get the internal angle of the regular polygon in radians.
This is the angle formed by two adjacent sides with points within the angle being in the interior of the polygon
Sourcepub const fn external_angle_degrees(&self) -> f32
pub const fn external_angle_degrees(&self) -> f32
Get the external angle of the regular polygon in degrees.
This is the angle formed by two adjacent sides with points within the angle being in the exterior of the polygon
Sourcepub const fn external_angle_radians(&self) -> f32
pub const fn external_angle_radians(&self) -> f32
Get the external angle of the regular polygon in radians.
This is the angle formed by two adjacent sides with points within the angle being in the exterior of the polygon
Sourcepub fn vertices(self, rotation: f32) -> impl IntoIterator<Item = Vec2>
pub fn vertices(self, rotation: f32) -> impl IntoIterator<Item = Vec2>
Returns an iterator over the vertices of the regular polygon, rotated counterclockwise by the given angle in radians.
With a rotation of 0, a vertex will be placed at the top (0.0, circumradius).