pub struct Scope<'scope, 'env, T>where
'env: 'scope,{ /* private fields */ }Expand description
A TaskPool scope for running one or more non-'static futures.
For more information, see TaskPool::scope.
Implementations
Sourceimpl<'scope, 'env, T> Scope<'scope, 'env, T>where
T: Send + 'scope,
impl<'scope, 'env, T> Scope<'scope, 'env, T>where
T: Send + 'scope,
Sourcepub fn spawn<Fut>(&self, f: Fut)
pub fn spawn<Fut>(&self, f: Fut)
Spawns a scoped future onto the thread pool. The scope must outlive
the provided future. The results of the future will be returned as a part of
TaskPool::scope’s return value.
For futures that should run on the thread scope is called on Scope::spawn_on_scope should be used
instead.
For more information, see TaskPool::scope.
Sourcepub fn spawn_on_scope<Fut>(&self, f: Fut)
pub fn spawn_on_scope<Fut>(&self, f: Fut)
Spawns a scoped future onto the thread the scope is run on. The scope must outlive
the provided future. The results of the future will be returned as a part of
TaskPool::scope’s return value. Users should generally prefer to use
Scope::spawn instead, unless the provided future needs to run on the scope’s thread.
For more information, see TaskPool::scope.
Sourcepub fn spawn_on_external<Fut>(&self, f: Fut)
pub fn spawn_on_external<Fut>(&self, f: Fut)
Spawns a scoped future onto the thread of the external thread executor.
This is typically the main thread. The scope must outlive
the provided future. The results of the future will be returned as a part of
TaskPool::scope’s return value. Users should generally prefer to use
Scope::spawn instead, unless the provided future needs to run on the external thread.
For more information, see TaskPool::scope.