pub enum FontSource {
}Expand description
Determines how the font face for a text sections is selected.
A FontSource can be a handle to a font asset, a font family name,
or a generic font category that is resolved using Parley’s font database.
Font family fallback (selection of a font when the requested font is not found)
is automatically handled by parley::fontique.
Be sure to enable the parley/system feature for automatic discovery of system fonts.
Generally speaking, these fallbacks are OS-specific, and do not require manual configuration.
You can check which font family is used for a given FontSource
by calling FontCx::get_family.
Variants
Handle(Handle<Font>)
Use a specific font face referenced by a Font asset handle.
If the default font handle is used, then
- if
default_fontfeature is enabled (enabled by default inbevycrate),FiraMono-subset.ttfcompiled into the library is used. - otherwise no text will be rendered, unless a custom font is loaded into the default font handle.
Family(SmolStr)
Resolve the font by family name using the font database.
Serif
Fonts with serifs — small decorative strokes at the ends of letterforms.
Serif fonts are typically used for long passages of text and represent a more traditional or formal typographic style.
SansSerif
Fonts without serifs.
Sans-serif fonts generally have low stroke contrast and plain stroke endings, making them common for UI text and on-screen reading.
Cursive
Fonts that use a cursive or handwritten style.
Glyphs often resemble connected or flowing pen or brush strokes rather than printed letterforms.
Fantasy
Decorative or expressive fonts.
Fantasy fonts are primarily intended for display purposes and may prioritize visual style over readability.
Monospace
Fonts in which all glyphs have the same fixed advance width.
Monospace fonts are commonly used for code, tabular data, and text where vertical alignment is important.
SystemUi
The default user interface system font.
UiSerif
Alternative serif font for user interfaces.
UiSansSerif
Alternative sans-erif font for user interfaces.
UiMonospace
Alternative monospace font for user interfaces.
UiRounded
Fonts that have rounded features.
Emoji
Fonts that are specifically designed to render emoji.
Math
This is for the particular stylistic concerns of representing mathematics: superscript and subscript, brackets that cross several lines, nesting expressions, and double struck glyphs with distinct meanings.
FangSong
A particular style of Chinese characters that are between serif-style Song and cursive-style Kai forms. This style is often used for government documents.