pub struct FontAtlas {
pub dynamic_texture_atlas_builder: DynamicTextureAtlasBuilder,
pub glyph_to_atlas_index: HashMap<CacheKey, GlyphAtlasLocation>,
pub texture_atlas: Handle<TextureAtlasLayout>,
pub texture: Handle<Image>,
}Expand description
Rasterized glyphs are cached, stored in, and retrieved from, a FontAtlas.
A FontAtlas contains one or more textures, each of which contains one or more glyphs packed into them.
A FontAtlasSet contains a FontAtlas for each font size in the same font face.
For the same font face and font size, a glyph will be rasterized differently for different subpixel offsets. In practice, ranges of subpixel offsets are grouped into subpixel bins to limit the number of rasterized glyphs, providing a trade-off between visual quality and performance.
A CacheKey encodes all of the information of a subpixel-offset glyph and is used to
find that glyphs raster in a TextureAtlas through its corresponding GlyphAtlasLocation.
Fields
dynamic_texture_atlas_builder: DynamicTextureAtlasBuilderUsed to update the TextureAtlasLayout.
glyph_to_atlas_index: HashMap<CacheKey, GlyphAtlasLocation>A mapping between subpixel-offset glyphs and their GlyphAtlasLocation.
texture_atlas: Handle<TextureAtlasLayout>The handle to the TextureAtlasLayout that holds the rasterized glyphs.
texture: Handle<Image>The texture where this font atlas is located
Implementations
Sourceimpl FontAtlas
impl FontAtlas
Sourcepub fn new(
textures: &mut Assets<Image>,
texture_atlases_layout: &mut Assets<TextureAtlasLayout>,
size: UVec2,
font_smoothing: FontSmoothing,
) -> FontAtlas
pub fn new( textures: &mut Assets<Image>, texture_atlases_layout: &mut Assets<TextureAtlasLayout>, size: UVec2, font_smoothing: FontSmoothing, ) -> FontAtlas
Create a new FontAtlas with the given size, adding it to the appropriate asset collections.
Sourcepub fn get_glyph_index(&self, cache_key: CacheKey) -> Option<GlyphAtlasLocation>
pub fn get_glyph_index(&self, cache_key: CacheKey) -> Option<GlyphAtlasLocation>
Get the GlyphAtlasLocation for a subpixel-offset glyph.
Sourcepub fn has_glyph(&self, cache_key: CacheKey) -> bool
pub fn has_glyph(&self, cache_key: CacheKey) -> bool
Checks if the given subpixel-offset glyph is contained in this FontAtlas.
Sourcepub fn add_glyph(
&mut self,
textures: &mut Assets<Image>,
atlas_layouts: &mut Assets<TextureAtlasLayout>,
cache_key: CacheKey,
texture: &Image,
offset: IVec2,
) -> Result<(), TextError>
pub fn add_glyph( &mut self, textures: &mut Assets<Image>, atlas_layouts: &mut Assets<TextureAtlasLayout>, cache_key: CacheKey, texture: &Image, offset: IVec2, ) -> Result<(), TextError>
Add a glyph to the atlas, updating both its texture and layout.
The glyph is represented by glyph, and its image content is glyph_texture.
This content is copied into the atlas texture, and the atlas layout is updated
to store the location of that glyph into the atlas.
Returns
Returns () if the glyph is successfully added, or TextError::FailedToAddGlyph otherwise.
In that case, neither the atlas texture nor the atlas layout are
modified.