pub struct FontFeatures { /* private fields */ }Expand description
OpenType features for .otf fonts that support them.
Examples features include ligatures, small-caps, and fractional number display. For the complete
list of OpenType features, see the spec at
<https://learn.microsoft.com/en-us/typography/opentype/spec/featurelist>.
Usage:
use bevy_text::{FontFeatureTag, FontFeatures};
// Create using the builder
let font_features = FontFeatures::builder()
.enable(FontFeatureTag::STANDARD_LIGATURES)
.set(FontFeatureTag::WEIGHT, 300)
.build();
// Create from a list
let more_font_features: FontFeatures = [
FontFeatureTag::STANDARD_LIGATURES,
FontFeatureTag::OLDSTYLE_FIGURES,
FontFeatureTag::TABULAR_FIGURES
].into();Implementations
Sourceimpl FontFeatures
impl FontFeatures
Sourcepub fn builder() -> FontFeaturesBuilder
pub fn builder() -> FontFeaturesBuilder
Create a new FontFeaturesBuilder.
Examples found in repository?
examples/ui/text/text.rs (line 140)
29fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
30 // UI camera
31 commands.spawn(Camera2d);
32 // Text with one section
33 commands.spawn((
34 // Accepts a `String` or any type that converts into a `String`, such as `&str`
35 Text::new("hello\nbevy!"),
36 Underline,
37 TextFont {
38 // This font is loaded and will be used instead of the default font.
39 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
40 // The size of the text will be 20% of the viewport height.
41 font_size: FontSize::Vh(20.0),
42 ..default()
43 },
44 TextShadow::default(),
45 // Set the justification of the Text
46 TextLayout::justify(Justify::Center),
47 // Set the style of the Node itself.
48 Node {
49 position_type: PositionType::Absolute,
50 bottom: px(5),
51 right: px(5),
52 ..default()
53 },
54 AnimatedText,
55 ));
56
57 // Text with multiple sections
58 commands
59 .spawn((
60 // Create a Text with multiple child spans.
61 Text::new("FPS: "),
62 TextFont {
63 // This font is loaded and will be used instead of the default font.
64 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
65 font_size: FontSize::Px(42.0),
66 ..default()
67 },
68 ))
69 .with_child((
70 TextSpan::default(),
71 (
72 TextFont {
73 // If the "default_font" feature is unavailable, load a font to use instead.
74 #[cfg(not(feature = "default_font"))]
75 font: asset_server.load("fonts/FiraMono-Medium.ttf").into(),
76 font_size: FontSize::Px(33.0),
77 ..Default::default()
78 },
79 TextColor(GOLD.into()),
80 ),
81 FpsText,
82 ));
83
84 // Text with OpenType features
85 let opentype_font_handle: FontSource =
86 asset_server.load("fonts/EBGaramond12-Regular.otf").into();
87 commands
88 .spawn((
89 Node {
90 margin: UiRect::all(px(12.0)),
91 position_type: PositionType::Absolute,
92 top: px(5.0),
93 right: px(5.0),
94 ..default()
95 },
96 Text::new("Opentype features:\n"),
97 TextFont {
98 font: opentype_font_handle.clone(),
99 font_size: FontSize::Px(32.0),
100 ..default()
101 },
102 ))
103 .with_children(|parent| {
104 let text_rows = [
105 ("Smallcaps: ", FontFeatureTag::SMALL_CAPS, "Hello World"),
106 (
107 "Ligatures: ",
108 FontFeatureTag::STANDARD_LIGATURES,
109 "fi fl ff ffi ffl",
110 ),
111 ("Fractions: ", FontFeatureTag::FRACTIONS, "12/134"),
112 ("Superscript: ", FontFeatureTag::SUPERSCRIPT, "Up here!"),
113 ("Subscript: ", FontFeatureTag::SUBSCRIPT, "Down here!"),
114 (
115 "Oldstyle figures: ",
116 FontFeatureTag::OLDSTYLE_FIGURES,
117 "1234567890",
118 ),
119 (
120 "Lining figures: ",
121 FontFeatureTag::LINING_FIGURES,
122 "1234567890",
123 ),
124 ];
125
126 for (title, feature, text) in text_rows {
127 parent.spawn((
128 TextSpan::new(title),
129 TextFont {
130 font: opentype_font_handle.clone(),
131 font_size: FontSize::Px(24.0),
132 ..default()
133 },
134 ));
135 parent.spawn((
136 TextSpan::new(format!("{text}\n")),
137 TextFont {
138 font: opentype_font_handle.clone(),
139 font_size: FontSize::Px(24.0),
140 font_features: FontFeatures::builder().enable(feature).build(),
141 ..default()
142 },
143 ));
144 }
145 });
146
147 #[cfg(feature = "default_font")]
148 commands.spawn((
149 // Here we are able to call the `From` method instead of creating a new `TextSection`.
150 // This will use the default font (a minimal subset of FiraMono) and apply the default styling.
151 Text::new("From an &str into a Text with the default font!"),
152 Node {
153 position_type: PositionType::Absolute,
154 bottom: px(5),
155 left: px(15),
156 ..default()
157 },
158 ));
159
160 #[cfg(not(feature = "default_font"))]
161 commands.spawn((
162 Text::new("Default font disabled"),
163 TextFont {
164 font: asset_server.load("fonts/FiraMono-Medium.ttf"),
165 ..default()
166 },
167 Node {
168 position_type: PositionType::Absolute,
169 bottom: px(5),
170 left: px(15),
171 ..default()
172 },
173 ));
174}Trait Implementations
Sourceimpl Clone for FontFeatures
impl Clone for FontFeatures
Sourcefn clone(&self) -> FontFeatures
fn clone(&self) -> FontFeatures
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for FontFeatures
impl Debug for FontFeatures
Sourceimpl Default for FontFeatures
impl Default for FontFeatures
Sourcefn default() -> FontFeatures
fn default() -> FontFeatures
Returns the “default value” for a type. Read more
Sourceimpl<T> From<T> for FontFeatureswhere
T: IntoIterator<Item = FontFeatureTag>,
Allow FontFeatures to be built from a list. This is suitable for the standard case when each
listed feature is a boolean type. If any features require a numeric value (like “wght”), use
FontFeaturesBuilder instead.
impl<T> From<T> for FontFeatureswhere
T: IntoIterator<Item = FontFeatureTag>,
Allow FontFeatures to be built from a list. This is suitable for the standard case when each
listed feature is a boolean type. If any features require a numeric value (like “wght”), use
FontFeaturesBuilder instead.
Sourcefn from(value: T) -> FontFeatures
fn from(value: T) -> FontFeatures
Converts to this type from the input type.
Sourceimpl FromArg for FontFeatures
impl FromArg for FontFeatures
Sourceimpl FromReflect for FontFeatures
impl FromReflect for FontFeatures
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<FontFeatures>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<FontFeatures>
Constructs a concrete instance of
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Attempts to downcast the given value to
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for FontFeatures
impl GetOwnership for FontFeatures
Sourceimpl GetTypeRegistration for FontFeatures
impl GetTypeRegistration for FontFeatures
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl IntoReturn for FontFeatures
impl IntoReturn for FontFeatures
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
FontFeatures: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
FontFeatures: 'into_return,
Sourceimpl PartialEq for FontFeatures
impl PartialEq for FontFeatures
Sourcefn eq(&self, other: &FontFeatures) -> bool
fn eq(&self, other: &FontFeatures) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl PartialReflect for FontFeatures
impl PartialReflect for FontFeatures
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<FontFeatures>) -> ReflectOwned
fn reflect_owned(self: Box<FontFeatures>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<FontFeatures>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<FontFeatures>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(self: Box<FontFeatures>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<FontFeatures>) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl Reflect for FontFeatures
impl Reflect for FontFeatures
Sourcefn into_any(self: Box<FontFeatures>) -> Box<dyn Any>
fn into_any(self: Box<FontFeatures>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<FontFeatures>) -> Box<dyn Reflect>
fn into_reflect(self: Box<FontFeatures>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
Sourceimpl Struct for FontFeatures
impl Struct for FontFeatures
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field named
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field named
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field with index
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field with index
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Gets the index of the field with the given name.
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Returns an iterator over the values of the reflectable fields for this struct.
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
Creates a new
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
Will return
None if TypeInfo is not available.impl StructuralPartialEq for FontFeatures
Sourceimpl TypePath for FontFeatures
impl TypePath for FontFeatures
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>