pub struct TextBounds {
pub width: Option<f32>,
pub height: Option<f32>,
}Expand description
The maximum width and height of text. The text will wrap according to the specified size.
Characters out of the bounds after wrapping will be truncated. Text is aligned according to the
specified Justify.
Note: only characters that are completely out of the bounds will be truncated, so this is not a reliable limit if it is necessary to contain the text strictly in the bounds. Currently this component is mainly useful for text wrapping only.
Fields
width: Option<f32>The maximum width of text in logical pixels.
If None, the width is unbounded.
height: Option<f32>The maximum height of text in logical pixels.
If None, the height is unbounded.
Implementations
Sourceimpl TextBounds
impl TextBounds
Sourcepub const UNBOUNDED: TextBounds
pub const UNBOUNDED: TextBounds
Unbounded text will not be truncated or wrapped.
Sourcepub const fn new(width: f32, height: f32) -> TextBounds
pub const fn new(width: f32, height: f32) -> TextBounds
Creates a new TextBounds, bounded with the specified width and height values.
Examples found in repository?
193 pub fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
194 commands.spawn((Camera2d, DespawnOnExit(super::Scene::Text)));
195
196 for (i, justify) in [
197 Justify::Left,
198 Justify::Right,
199 Justify::Center,
200 Justify::Justified,
201 ]
202 .into_iter()
203 .enumerate()
204 {
205 let y = 230. - 150. * i as f32;
206 spawn_anchored_text(&mut commands, -300. * Vec3::X + y * Vec3::Y, justify, None);
207 spawn_anchored_text(
208 &mut commands,
209 300. * Vec3::X + y * Vec3::Y,
210 justify,
211 Some(TextBounds::new(150., 60.)),
212 );
213 }
214
215 let sans_serif = TextFont::from(asset_server.load("fonts/FiraSans-Bold.ttf"));
216
217 const NUM_ITERATIONS: usize = 10;
218 for i in 0..NUM_ITERATIONS {
219 let fraction = i as f32 / (NUM_ITERATIONS - 1) as f32;
220
221 commands.spawn((
222 Text2d::new("Bevy"),
223 sans_serif.clone(),
224 Transform::from_xyz(0.0, fraction * 200.0, i as f32)
225 .with_scale(1.0 + Vec2::splat(fraction).extend(1.))
226 .with_rotation(Quat::from_rotation_z(fraction * core::f32::consts::PI)),
227 TextColor(Color::hsla(fraction * 360.0, 0.8, 0.8, 0.8)),
228 DespawnOnExit(super::Scene::Text),
229 ));
230 }
231
232 commands.spawn((
233 Text2d::new("This text is invisible."),
234 Visibility::Hidden,
235 DespawnOnExit(super::Scene::Text),
236 ));
237 }Sourcepub const fn new_horizontal(width: f32) -> TextBounds
pub const fn new_horizontal(width: f32) -> TextBounds
Creates a new TextBounds, bounded with the specified width value and unbounded on height.
Examples found in repository?
64fn setup(mut commands: Commands, args: Res<Args>) {
65 warn!(include_str!("warning_string.txt"));
66
67 commands.spawn(Camera2d);
68 let text_string = "0123456789".repeat(10_000);
69 let text_font = TextFont {
70 font_size: FontSize::Px(4.),
71 ..Default::default()
72 };
73 let text_block = TextLayout {
74 justify: Justify::Left,
75 linebreak: LineBreak::AnyCharacter,
76 };
77
78 if !args.no_ui {
79 commands
80 .spawn(Node {
81 width: percent(100),
82 align_items: AlignItems::Center,
83 justify_content: JustifyContent::Center,
84 ..default()
85 })
86 .with_children(|commands| {
87 commands
88 .spawn(Node {
89 width: px(1000),
90 ..Default::default()
91 })
92 .with_child((Text(text_string.clone()), text_font.clone(), text_block));
93 });
94 }
95
96 if !args.no_text2d {
97 commands.spawn((
98 Text2d::new(text_string),
99 text_font.clone(),
100 TextColor(RED.into()),
101 bevy::sprite::Anchor::CENTER,
102 TextBounds::new_horizontal(1000.),
103 text_block,
104 ));
105 }
106}Sourcepub const fn new_vertical(height: f32) -> TextBounds
pub const fn new_vertical(height: f32) -> TextBounds
Creates a new TextBounds, bounded with the specified height value and unbounded on width.