pub struct TextPipeline {
pub map_handle_to_font_id: HashMap<AssetId<Font>, (ID, Arc<str>)>,
/* private fields */
}Expand description
The TextPipeline is used to layout and render text blocks (see Text/Text2d).
See the crate-level documentation for more information.
Fields
map_handle_to_font_id: HashMap<AssetId<Font>, (ID, Arc<str>)>Implementations
Sourceimpl TextPipeline
impl TextPipeline
Sourcepub fn update_buffer<'a>(
&mut self,
fonts: &Assets<Font>,
text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>,
linebreak: LineBreak,
justify: Justify,
bounds: TextBounds,
scale_factor: f64,
computed: &mut ComputedTextBlock,
font_system: &mut CosmicFontSystem,
) -> Result<(), TextError>
pub fn update_buffer<'a>( &mut self, fonts: &Assets<Font>, text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>, linebreak: LineBreak, justify: Justify, bounds: TextBounds, scale_factor: f64, computed: &mut ComputedTextBlock, font_system: &mut CosmicFontSystem, ) -> Result<(), TextError>
Utilizes cosmic_text::Buffer to shape and layout text
Negative or 0.0 font sizes will not be laid out.
Sourcepub fn queue_text<'a>(
&mut self,
layout_info: &mut TextLayoutInfo,
fonts: &Assets<Font>,
text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>,
scale_factor: f64,
layout: &TextLayout,
bounds: TextBounds,
font_atlas_sets: &mut FontAtlasSets,
texture_atlases: &mut Assets<TextureAtlasLayout>,
textures: &mut Assets<Image>,
computed: &mut ComputedTextBlock,
font_system: &mut CosmicFontSystem,
swash_cache: &mut SwashCache,
) -> Result<(), TextError>
pub fn queue_text<'a>( &mut self, layout_info: &mut TextLayoutInfo, fonts: &Assets<Font>, text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>, scale_factor: f64, layout: &TextLayout, bounds: TextBounds, font_atlas_sets: &mut FontAtlasSets, texture_atlases: &mut Assets<TextureAtlasLayout>, textures: &mut Assets<Image>, computed: &mut ComputedTextBlock, font_system: &mut CosmicFontSystem, swash_cache: &mut SwashCache, ) -> Result<(), TextError>
Queues text for rendering
Produces a TextLayoutInfo, containing PositionedGlyphs
which contain information for rendering the text.
Sourcepub fn create_text_measure<'a>(
&mut self,
entity: Entity,
fonts: &Assets<Font>,
text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>,
scale_factor: f64,
layout: &TextLayout,
computed: &mut ComputedTextBlock,
font_system: &mut CosmicFontSystem,
) -> Result<TextMeasureInfo, TextError>
pub fn create_text_measure<'a>( &mut self, entity: Entity, fonts: &Assets<Font>, text_spans: impl Iterator<Item = (Entity, usize, &'a str, &'a TextFont, Color)>, scale_factor: f64, layout: &TextLayout, computed: &mut ComputedTextBlock, font_system: &mut CosmicFontSystem, ) -> Result<TextMeasureInfo, TextError>
Queues text for measurement
Produces a TextMeasureInfo which can be used by a layout system
to measure the text area on demand.
Sourcepub fn get_font_id(&self, asset_id: AssetId<Font>) -> Option<ID>
pub fn get_font_id(&self, asset_id: AssetId<Font>) -> Option<ID>
Returns the cosmic_text::fontdb::ID for a given Font asset.