pub enum TrySendError<T> {
Full(T),
Disconnected(T),
}Expand description
An error returned from the try_send method.
The error contains the message being sent so it can be recovered.
Variants
Full(T)
The message could not be sent because the channel is full.
If this is a zero-capacity channel, then the error indicates that there was no receiver available to receive the message at the time.
Disconnected(T)
The message could not be sent because the channel is disconnected.
Implementations
Sourceimpl<T> TrySendError<T>
impl<T> TrySendError<T>
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Unwraps the message.
Examples
use crossbeam_channel::bounded;
let (s, r) = bounded(0);
if let Err(err) = s.try_send("foo") {
assert_eq!(err.into_inner(), "foo");
}Sourcepub fn is_full(&self) -> bool
pub fn is_full(&self) -> bool
Returns true if the send operation failed because the channel is full.
Sourcepub fn is_disconnected(&self) -> bool
pub fn is_disconnected(&self) -> bool
Returns true if the send operation failed because the channel is disconnected.