Outline

Struct Outline 

pub struct Outline {
    pub width: Val,
    pub offset: Val,
    pub color: Color,
}
The Outline component adds an outline outside the edge of a UI node. Outlines do not take up space in the layout.

To add an Outline to a ui node you can spawn a (Node, Outline) tuple bundle:

fn setup_ui(mut commands: Commands) {
    commands.spawn((
        Node {
            width: Val::Px(100.),
            height: Val::Px(100.),
            ..Default::default()
        },
        BackgroundColor(BLUE.into()),
        Outline::new(Val::Px(10.), Val::ZERO, RED.into())
    ));
}

Outline components can also be added later to existing UI nodes:

fn outline_hovered_button_system(
    mut commands: Commands,
    mut node_query: Query<(Entity, &Interaction, Option<&mut Outline>), Changed<Interaction>>,
) {
    for (entity, interaction, mut maybe_outline) in node_query.iter_mut() {
        let outline_color =
            if matches!(*interaction, Interaction::Hovered) {
                Color::WHITE
            } else {
                Color::NONE
            };
        if let Some(mut outline) = maybe_outline {
            outline.color = outline_color;
        } else {
            commands.entity(entity).insert(Outline::new(Val::Px(10.), Val::ZERO, outline_color));
        }
    }
}

Inserting and removing an Outline component repeatedly will result in table moves, so it is generally preferable to set Outline::color to Color::NONE to hide an outline.

Fields

width: Val

The width of the outline.

Percentage Val values are resolved based on the width of the outlined Node.

offset: Val

The amount of space between a node’s outline the edge of the node.

Percentage Val values are resolved based on the width of the outlined Node.

color: Color

The color of the outline.

If you are frequently toggling outlines for a UI node on and off it is recommended to set Color::NONE to hide the outline. This avoids the table moves that would occur from the repeated insertion and removal of the Outline component.

Implementations

Source

impl Outline

pub const fn new(width: Val, offset: Val, color: Color) -> Outline

Create a new outline

Examples found in repository?
examples/ui/ui_texture_atlas.rs (line 55)
18fn setup(
19    mut commands: Commands,
20    asset_server: Res<AssetServer>,
21    mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
22) {
23    // Camera
24    commands.spawn(Camera2d);
25
26    let text_font = TextFont::default();
27
28    let texture_handle = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
29    let texture_atlas = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
30    let texture_atlas_handle = texture_atlases.add(texture_atlas);
31
32    // root node
33    commands
34        .spawn(Node {
35            width: percent(100),
36            height: percent(100),
37            flex_direction: FlexDirection::Column,
38            justify_content: JustifyContent::Center,
39            align_items: AlignItems::Center,
40            row_gap: px(text_font.font_size * 2.),
41            ..default()
42        })
43        .with_children(|parent| {
44            parent.spawn((
45                ImageNode::from_atlas_image(
46                    texture_handle,
47                    TextureAtlas::from(texture_atlas_handle),
48                ),
49                Node {
50                    width: px(256),
51                    height: px(256),
52                    ..default()
53                },
54                BackgroundColor(ANTIQUE_WHITE.into()),
55                Outline::new(px(8), Val::ZERO, CRIMSON.into()),
56            ));
57            parent
58                .spawn((Text::new("press "), text_font.clone()))
59                .with_child((
60                    TextSpan::new("space"),
61                    TextColor(YELLOW.into()),
62                    text_font.clone(),
63                ))
64                .with_child((TextSpan::new(" to advance frames"), text_font));
65        });
66}
More examples
Hide additional examples
examples/animation/animation_graph.rs (line 305)
271fn setup_node_rects(commands: &mut Commands) {
272    for (node_rect, node_type) in NODE_RECTS.iter().zip(NODE_TYPES.iter()) {
273        let node_string = match *node_type {
274            NodeType::Clip(ref clip) => clip.text,
275            NodeType::Blend(text) => text,
276        };
277
278        let text = commands
279            .spawn((
280                Text::new(node_string),
281                TextFont {
282                    font_size: 16.0,
283                    ..default()
284                },
285                TextColor(ANTIQUE_WHITE.into()),
286                TextLayout::new_with_justify(Justify::Center),
287            ))
288            .id();
289
290        let container = {
291            let mut container = commands.spawn((
292                Node {
293                    position_type: PositionType::Absolute,
294                    bottom: px(node_rect.bottom),
295                    left: px(node_rect.left),
296                    height: px(node_rect.height),
297                    width: px(node_rect.width),
298                    align_items: AlignItems::Center,
299                    justify_items: JustifyItems::Center,
300                    align_content: AlignContent::Center,
301                    justify_content: JustifyContent::Center,
302                    ..default()
303                },
304                BorderColor::all(WHITE),
305                Outline::new(px(1), Val::ZERO, Color::WHITE),
306            ));
307
308            if let NodeType::Clip(clip) = node_type {
309                container.insert((
310                    Interaction::None,
311                    RelativeCursorPosition::default(),
312                    (*clip).clone(),
313                ));
314            }
315
316            container.id()
317        };
318
319        // Create the background color.
320        if let NodeType::Clip(_) = node_type {
321            let background = commands
322                .spawn((
323                    Node {
324                        position_type: PositionType::Absolute,
325                        top: px(0),
326                        left: px(0),
327                        height: px(node_rect.height),
328                        width: px(node_rect.width),
329                        ..default()
330                    },
331                    BackgroundColor(DARK_GREEN.into()),
332                ))
333                .id();
334
335            commands.entity(container).add_child(background);
336        }
337
338        commands.entity(container).add_child(text);
339    }
340}