pub struct OverflowClipMargin {
pub visual_box: VisualBox,
pub margin: f32,
}Expand description
The bounds of the visible area when a UI node is clipped.
Fields
visual_box: VisualBoxVisible unclipped area
margin: f32Width of the margin on each edge of the visual box in logical pixels. The width of the margin will be zero if a negative value is set.
Implementations
Sourceimpl OverflowClipMargin
impl OverflowClipMargin
pub const DEFAULT: OverflowClipMargin
Sourcepub const fn content_box() -> OverflowClipMargin
pub const fn content_box() -> OverflowClipMargin
Clip any content that overflows outside the content box
Examples found in repository?
examples/ui/scroll_and_overflow/overflow_clip_margin.rs (line 35)
12fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
13 commands.spawn(Camera2d);
14
15 let image = asset_server.load("branding/icon.png");
16
17 commands
18 .spawn((
19 Node {
20 width: percent(100),
21 height: percent(100),
22 align_items: AlignItems::Center,
23 justify_content: JustifyContent::Center,
24 row_gap: px(40),
25 flex_direction: FlexDirection::Column,
26 ..default()
27 },
28 BackgroundColor(ANTIQUE_WHITE.into()),
29 ))
30 .with_children(|parent| {
31 for overflow_clip_margin in [
32 OverflowClipMargin::border_box().with_margin(25.),
33 OverflowClipMargin::border_box(),
34 OverflowClipMargin::padding_box(),
35 OverflowClipMargin::content_box(),
36 ] {
37 parent
38 .spawn(Node {
39 flex_direction: FlexDirection::Row,
40 column_gap: px(20),
41 ..default()
42 })
43 .with_children(|parent| {
44 parent
45 .spawn((
46 Node {
47 padding: UiRect::all(px(10)),
48 margin: UiRect::bottom(px(25)),
49 ..default()
50 },
51 BackgroundColor(Color::srgb(0.25, 0.25, 0.25)),
52 ))
53 .with_child(Text(format!("{overflow_clip_margin:#?}")));
54
55 parent
56 .spawn((
57 Node {
58 margin: UiRect::top(px(10)),
59 width: px(100),
60 height: px(100),
61 padding: UiRect::all(px(20)),
62 border: UiRect::all(px(5)),
63 overflow: Overflow::clip(),
64 overflow_clip_margin,
65 ..default()
66 },
67 BackgroundColor(GRAY.into()),
68 BorderColor::all(Color::BLACK),
69 ))
70 .with_children(|parent| {
71 parent
72 .spawn((
73 Node {
74 min_width: px(50),
75 min_height: px(50),
76 ..default()
77 },
78 BackgroundColor(LIGHT_CYAN.into()),
79 ))
80 .with_child((
81 ImageNode::new(image.clone()),
82 Node {
83 min_width: px(100),
84 min_height: px(100),
85 ..default()
86 },
87 ));
88 });
89 });
90 }
91 });
92}Sourcepub const fn padding_box() -> OverflowClipMargin
pub const fn padding_box() -> OverflowClipMargin
Clip any content that overflows outside the padding box
Examples found in repository?
examples/ui/scroll_and_overflow/overflow_clip_margin.rs (line 34)
12fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
13 commands.spawn(Camera2d);
14
15 let image = asset_server.load("branding/icon.png");
16
17 commands
18 .spawn((
19 Node {
20 width: percent(100),
21 height: percent(100),
22 align_items: AlignItems::Center,
23 justify_content: JustifyContent::Center,
24 row_gap: px(40),
25 flex_direction: FlexDirection::Column,
26 ..default()
27 },
28 BackgroundColor(ANTIQUE_WHITE.into()),
29 ))
30 .with_children(|parent| {
31 for overflow_clip_margin in [
32 OverflowClipMargin::border_box().with_margin(25.),
33 OverflowClipMargin::border_box(),
34 OverflowClipMargin::padding_box(),
35 OverflowClipMargin::content_box(),
36 ] {
37 parent
38 .spawn(Node {
39 flex_direction: FlexDirection::Row,
40 column_gap: px(20),
41 ..default()
42 })
43 .with_children(|parent| {
44 parent
45 .spawn((
46 Node {
47 padding: UiRect::all(px(10)),
48 margin: UiRect::bottom(px(25)),
49 ..default()
50 },
51 BackgroundColor(Color::srgb(0.25, 0.25, 0.25)),
52 ))
53 .with_child(Text(format!("{overflow_clip_margin:#?}")));
54
55 parent
56 .spawn((
57 Node {
58 margin: UiRect::top(px(10)),
59 width: px(100),
60 height: px(100),
61 padding: UiRect::all(px(20)),
62 border: UiRect::all(px(5)),
63 overflow: Overflow::clip(),
64 overflow_clip_margin,
65 ..default()
66 },
67 BackgroundColor(GRAY.into()),
68 BorderColor::all(Color::BLACK),
69 ))
70 .with_children(|parent| {
71 parent
72 .spawn((
73 Node {
74 min_width: px(50),
75 min_height: px(50),
76 ..default()
77 },
78 BackgroundColor(LIGHT_CYAN.into()),
79 ))
80 .with_child((
81 ImageNode::new(image.clone()),
82 Node {
83 min_width: px(100),
84 min_height: px(100),
85 ..default()
86 },
87 ));
88 });
89 });
90 }
91 });
92}Sourcepub const fn border_box() -> OverflowClipMargin
pub const fn border_box() -> OverflowClipMargin
Clip any content that overflows outside the border box
Examples found in repository?
examples/ui/scroll_and_overflow/overflow_clip_margin.rs (line 32)
12fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
13 commands.spawn(Camera2d);
14
15 let image = asset_server.load("branding/icon.png");
16
17 commands
18 .spawn((
19 Node {
20 width: percent(100),
21 height: percent(100),
22 align_items: AlignItems::Center,
23 justify_content: JustifyContent::Center,
24 row_gap: px(40),
25 flex_direction: FlexDirection::Column,
26 ..default()
27 },
28 BackgroundColor(ANTIQUE_WHITE.into()),
29 ))
30 .with_children(|parent| {
31 for overflow_clip_margin in [
32 OverflowClipMargin::border_box().with_margin(25.),
33 OverflowClipMargin::border_box(),
34 OverflowClipMargin::padding_box(),
35 OverflowClipMargin::content_box(),
36 ] {
37 parent
38 .spawn(Node {
39 flex_direction: FlexDirection::Row,
40 column_gap: px(20),
41 ..default()
42 })
43 .with_children(|parent| {
44 parent
45 .spawn((
46 Node {
47 padding: UiRect::all(px(10)),
48 margin: UiRect::bottom(px(25)),
49 ..default()
50 },
51 BackgroundColor(Color::srgb(0.25, 0.25, 0.25)),
52 ))
53 .with_child(Text(format!("{overflow_clip_margin:#?}")));
54
55 parent
56 .spawn((
57 Node {
58 margin: UiRect::top(px(10)),
59 width: px(100),
60 height: px(100),
61 padding: UiRect::all(px(20)),
62 border: UiRect::all(px(5)),
63 overflow: Overflow::clip(),
64 overflow_clip_margin,
65 ..default()
66 },
67 BackgroundColor(GRAY.into()),
68 BorderColor::all(Color::BLACK),
69 ))
70 .with_children(|parent| {
71 parent
72 .spawn((
73 Node {
74 min_width: px(50),
75 min_height: px(50),
76 ..default()
77 },
78 BackgroundColor(LIGHT_CYAN.into()),
79 ))
80 .with_child((
81 ImageNode::new(image.clone()),
82 Node {
83 min_width: px(100),
84 min_height: px(100),
85 ..default()
86 },
87 ));
88 });
89 });
90 }
91 });
92}Sourcepub const fn with_margin(self, margin: f32) -> OverflowClipMargin
pub const fn with_margin(self, margin: f32) -> OverflowClipMargin
Add a margin on each edge of the visual box in logical pixels. The width of the margin will be zero if a negative value is set.
Examples found in repository?
examples/ui/scroll_and_overflow/overflow_clip_margin.rs (line 32)
12fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
13 commands.spawn(Camera2d);
14
15 let image = asset_server.load("branding/icon.png");
16
17 commands
18 .spawn((
19 Node {
20 width: percent(100),
21 height: percent(100),
22 align_items: AlignItems::Center,
23 justify_content: JustifyContent::Center,
24 row_gap: px(40),
25 flex_direction: FlexDirection::Column,
26 ..default()
27 },
28 BackgroundColor(ANTIQUE_WHITE.into()),
29 ))
30 .with_children(|parent| {
31 for overflow_clip_margin in [
32 OverflowClipMargin::border_box().with_margin(25.),
33 OverflowClipMargin::border_box(),
34 OverflowClipMargin::padding_box(),
35 OverflowClipMargin::content_box(),
36 ] {
37 parent
38 .spawn(Node {
39 flex_direction: FlexDirection::Row,
40 column_gap: px(20),
41 ..default()
42 })
43 .with_children(|parent| {
44 parent
45 .spawn((
46 Node {
47 padding: UiRect::all(px(10)),
48 margin: UiRect::bottom(px(25)),
49 ..default()
50 },
51 BackgroundColor(Color::srgb(0.25, 0.25, 0.25)),
52 ))
53 .with_child(Text(format!("{overflow_clip_margin:#?}")));
54
55 parent
56 .spawn((
57 Node {
58 margin: UiRect::top(px(10)),
59 width: px(100),
60 height: px(100),
61 padding: UiRect::all(px(20)),
62 border: UiRect::all(px(5)),
63 overflow: Overflow::clip(),
64 overflow_clip_margin,
65 ..default()
66 },
67 BackgroundColor(GRAY.into()),
68 BorderColor::all(Color::BLACK),
69 ))
70 .with_children(|parent| {
71 parent
72 .spawn((
73 Node {
74 min_width: px(50),
75 min_height: px(50),
76 ..default()
77 },
78 BackgroundColor(LIGHT_CYAN.into()),
79 ))
80 .with_child((
81 ImageNode::new(image.clone()),
82 Node {
83 min_width: px(100),
84 min_height: px(100),
85 ..default()
86 },
87 ));
88 });
89 });
90 }
91 });
92}Trait Implementations
Sourceimpl Clone for OverflowClipMargin
impl Clone for OverflowClipMargin
Sourcefn clone(&self) -> OverflowClipMargin
fn clone(&self) -> OverflowClipMargin
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreimpl Copy for OverflowClipMargin
Sourceimpl Debug for OverflowClipMargin
impl Debug for OverflowClipMargin
Sourceimpl Default for OverflowClipMargin
impl Default for OverflowClipMargin
Sourcefn default() -> OverflowClipMargin
fn default() -> OverflowClipMargin
Returns the “default value” for a type. Read more
Sourceimpl<'de> Deserialize<'de> for OverflowClipMargin
impl<'de> Deserialize<'de> for OverflowClipMargin
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<OverflowClipMargin, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<OverflowClipMargin, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl FromArg for OverflowClipMargin
impl FromArg for OverflowClipMargin
Sourceimpl FromReflect for OverflowClipMargin
impl FromReflect for OverflowClipMargin
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<OverflowClipMargin>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<OverflowClipMargin>
Constructs a concrete instance of
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Attempts to downcast the given value to
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for OverflowClipMargin
impl GetOwnership for OverflowClipMargin
Sourceimpl GetTypeRegistration for OverflowClipMargin
impl GetTypeRegistration for OverflowClipMargin
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl IntoReturn for OverflowClipMargin
impl IntoReturn for OverflowClipMargin
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
OverflowClipMargin: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
OverflowClipMargin: 'into_return,
Sourceimpl PartialEq for OverflowClipMargin
impl PartialEq for OverflowClipMargin
Sourcefn eq(&self, other: &OverflowClipMargin) -> bool
fn eq(&self, other: &OverflowClipMargin) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl PartialReflect for OverflowClipMargin
impl PartialReflect for OverflowClipMargin
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<OverflowClipMargin>) -> ReflectOwned
fn reflect_owned(self: Box<OverflowClipMargin>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<OverflowClipMargin>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<OverflowClipMargin>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(
self: Box<OverflowClipMargin>,
) -> Box<dyn PartialReflect>
fn into_partial_reflect( self: Box<OverflowClipMargin>, ) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl Reflect for OverflowClipMargin
impl Reflect for OverflowClipMargin
Sourcefn into_any(self: Box<OverflowClipMargin>) -> Box<dyn Any>
fn into_any(self: Box<OverflowClipMargin>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<OverflowClipMargin>) -> Box<dyn Reflect>
fn into_reflect(self: Box<OverflowClipMargin>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
Sourceimpl Serialize for OverflowClipMargin
impl Serialize for OverflowClipMargin
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Serialize this value into the given Serde serializer. Read more
Sourceimpl Struct for OverflowClipMargin
impl Struct for OverflowClipMargin
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field named
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field named
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
Gets a reference to the value of the field with index
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Gets a mutable reference to the value of the field with index
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Gets the index of the field with the given name.
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Returns an iterator over the values of the reflectable fields for this struct.
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
Creates a new
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
Will return
None if TypeInfo is not available.impl StructuralPartialEq for OverflowClipMargin
Sourceimpl TypePath for OverflowClipMargin
impl TypePath for OverflowClipMargin
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>