UiTransform

Struct UiTransform 

pub struct UiTransform {
    pub translation: Val2,
    pub scale: Vec2,
    pub rotation: Rot2,
}
Relative 2D transform for UI nodes

UiGlobalTransform is automatically inserted whenever UiTransform is inserted.

Fields

translation: Val2

Translate the node.

scale: Vec2

Scale the node. A negative value reflects the node in that axis.

rotation: Rot2

Rotate the node clockwise.

Implementations

Source

impl UiTransform

pub const IDENTITY: UiTransform

pub const fn from_rotation(rotation: Rot2) -> UiTransform

Creates a UI transform representing a rotation.

Examples found in repository?
examples/testbed/ui.rs (line 1399)
1384    pub fn setup(mut commands: Commands) {
1385        commands.spawn((Camera2d, DespawnOnExit(super::Scene::Transformations)));
1386        commands
1387            .spawn((
1388                Node {
1389                    width: percent(100),
1390                    height: percent(100),
1391                    display: Display::Block,
1392                    ..default()
1393                },
1394                DespawnOnExit(super::Scene::Transformations),
1395            ))
1396            .with_children(|parent| {
1397                for (transformation, label, background) in [
1398                    (
1399                        UiTransform::from_rotation(Rot2::degrees(45.)),
1400                        "Rotate 45 degrees",
1401                        RED,
1402                    ),
1403                    (
1404                        UiTransform::from_scale(Vec2::new(2., 0.5)),
1405                        "Scale 2.x 0.5y",
1406                        GREEN,
1407                    ),
1408                    (
1409                        UiTransform::from_translation(Val2::px(-50., 50.)),
1410                        "Translate -50px x +50px y",
1411                        BLUE,
1412                    ),
1413                    (
1414                        UiTransform {
1415                            translation: Val2::px(50., 0.),
1416                            scale: Vec2::new(-1., 1.),
1417                            rotation: Rot2::degrees(30.),
1418                        },
1419                        "T 50px x\nS -1.x (refl)\nR 30deg",
1420                        DARK_CYAN,
1421                    ),
1422                ] {
1423                    parent
1424                        .spawn((Node {
1425                            width: percent(100),
1426                            margin: UiRect {
1427                                top: px(50),
1428                                bottom: px(50),
1429                                ..default()
1430                            },
1431                            align_items: AlignItems::Center,
1432                            justify_content: JustifyContent::SpaceAround,
1433                            ..default()
1434                        },))
1435                        .with_children(|row| {
1436                            row.spawn((
1437                                Text::new("Before Tf"),
1438                                Node {
1439                                    width: px(100),
1440                                    height: px(100),
1441                                    border_radius: BorderRadius::bottom_right(px(25.)),
1442                                    ..default()
1443                                },
1444                                BackgroundColor(background.into()),
1445                                TextFont::default(),
1446                            ));
1447                            row.spawn((
1448                                Text::new(label),
1449                                Node {
1450                                    width: px(100),
1451                                    height: px(100),
1452                                    border_radius: BorderRadius::bottom_right(px(25.)),
1453                                    ..default()
1454                                },
1455                                BackgroundColor(background.into()),
1456                                transformation,
1457                                TextFont::default(),
1458                            ));
1459                        });
1460                }
1461            });
1462    }
1463}
1464
1465#[cfg(feature = "bevy_ui_debug")]
1466mod debug_outlines {
1467    use bevy::{
1468        color::palettes::css::{BLUE, GRAY, RED},
1469        prelude::*,
1470        ui_render::UiDebugOptions,
1471    };
1472
1473    pub fn setup(mut commands: Commands, mut debug_options: ResMut<GlobalUiDebugOptions>) {
1474        debug_options.enabled = true;
1475        debug_options.line_width = 5.;
1476        debug_options.line_color_override = Some(LinearRgba::GREEN);
1477        debug_options.show_hidden = true;
1478        debug_options.show_clipped = true;
1479
1480        let debug_options: UiDebugOptions = (*debug_options.as_ref()).into();
1481
1482        commands.spawn((Camera2d, DespawnOnExit(super::Scene::DebugOutlines)));
1483        commands
1484            .spawn((
1485                Node {
1486                    width: percent(100),
1487                    height: percent(50),
1488                    align_items: AlignItems::Center,
1489                    justify_content: JustifyContent::SpaceAround,
1490                    ..default()
1491                },
1492                DespawnOnExit(super::Scene::DebugOutlines),
1493            ))
1494            .with_children(|parent| {
1495                parent.spawn((
1496                    Node {
1497                        width: px(100),
1498                        height: px(100),
1499                        ..default()
1500                    },
1501                    BackgroundColor(GRAY.into()),
1502                    UiTransform::from_rotation(Rot2::degrees(45.)),
1503                ));
1504
1505                parent.spawn((Text::new("Regular Text"), TextFont::default()));
1506
1507                parent.spawn((
1508                    Node {
1509                        width: px(100),
1510                        height: px(100),
1511                        ..default()
1512                    },
1513                    Text::new("Invisible"),
1514                    BackgroundColor(GRAY.into()),
1515                    TextFont::default(),
1516                    Visibility::Hidden,
1517                ));
1518
1519                parent
1520                    .spawn((
1521                        Node {
1522                            width: px(100),
1523                            height: px(100),
1524                            padding: UiRect {
1525                                left: px(25),
1526                                top: px(25),
1527                                ..Default::default()
1528                            },
1529                            overflow: Overflow::clip(),
1530                            ..default()
1531                        },
1532                        BackgroundColor(RED.into()),
1533                    ))
1534                    .with_children(|child| {
1535                        child.spawn((
1536                            Node {
1537                                min_width: px(100),
1538                                min_height: px(100),
1539                                ..default()
1540                            },
1541                            BackgroundColor(BLUE.into()),
1542                        ));
1543                    });
1544            });
1545
1546        commands
1547            .spawn((
1548                Node {
1549                    width: percent(100),
1550                    height: percent(50),
1551                    top: percent(50),
1552                    align_items: AlignItems::Center,
1553                    justify_content: JustifyContent::SpaceAround,
1554                    ..default()
1555                },
1556                DespawnOnExit(super::Scene::DebugOutlines),
1557            ))
1558            .with_children(|parent| {
1559                parent.spawn((
1560                    Node {
1561                        width: px(200),
1562                        height: px(200),
1563                        border: UiRect {
1564                            top: px(10),
1565                            bottom: px(20),
1566                            left: px(30),
1567                            right: px(40),
1568                        },
1569                        border_radius: BorderRadius::bottom_right(px(10)),
1570                        padding: UiRect {
1571                            top: px(40),
1572                            bottom: px(30),
1573                            left: px(20),
1574                            right: px(10),
1575                        },
1576                        ..default()
1577                    },
1578                    children![(
1579                        Text::new("border padding content outlines"),
1580                        TextFont::default(),
1581                        UiDebugOptions {
1582                            enabled: false,
1583                            ..default()
1584                        }
1585                    )],
1586                    UiDebugOptions {
1587                        outline_border_box: true,
1588                        outline_padding_box: true,
1589                        outline_content_box: true,
1590                        ignore_border_radius: false,
1591                        ..debug_options
1592                    },
1593                ));
1594
1595                // Vertical scrollbar (non-functional)
1596                parent.spawn((
1597                    Node {
1598                        flex_direction: FlexDirection::Column,
1599                        width: px(90),
1600                        height: px(230),
1601                        overflow: Overflow::scroll_y(),
1602                        scrollbar_width: 20.,
1603                        ..default()
1604                    },
1605                    ScrollPosition(Vec2::new(180., 180.)),
1606                    UiDebugOptions {
1607                        line_width: 3.,
1608                        outline_scrollbars: true,
1609                        show_hidden: false,
1610                        show_clipped: false,
1611                        ..debug_options
1612                    },
1613                    Children::spawn(SpawnIter((0..20).map(move |i| {
1614                        (
1615                            Node::default(),
1616                            children![(
1617                                Text(format!("Item {i}")),
1618                                UiDebugOptions {
1619                                    enabled: false,
1620                                    ..default()
1621                                }
1622                            )],
1623                            UiDebugOptions {
1624                                enabled: false,
1625                                ..default()
1626                            },
1627                        )
1628                    }))),
1629                ));
1630
1631                // Horizontal scrollbar (non-functional)
1632                parent.spawn((
1633                    Node {
1634                        flex_direction: FlexDirection::Row,
1635                        width: px(156),
1636                        height: px(70),
1637                        overflow: Overflow::scroll_x(),
1638                        scrollbar_width: 10.,
1639                        ..default()
1640                    },
1641                    UiDebugOptions {
1642                        line_width: 3.,
1643                        outline_scrollbars: true,
1644                        show_hidden: false,
1645                        show_clipped: false,
1646                        ..debug_options
1647                    },
1648                    Children::spawn(SpawnIter((0..20).map(move |i| {
1649                        (
1650                            Node::default(),
1651                            children![(
1652                                Text(format!("Item {i}")),
1653                                UiDebugOptions {
1654                                    enabled: false,
1655                                    ..default()
1656                                }
1657                            )],
1658                            UiDebugOptions {
1659                                enabled: false,
1660                                ..default()
1661                            },
1662                        )
1663                    }))),
1664                ));
1665
1666                // bi-directional scrollbar (non-functional)
1667                parent.spawn((
1668                    Node {
1669                        flex_direction: FlexDirection::Column,
1670                        width: px(230),
1671                        height: px(125),
1672                        overflow: Overflow::scroll(),
1673                        scrollbar_width: 20.,
1674                        ..default()
1675                    },
1676                    ScrollPosition(Vec2::new(300., 0.)),
1677                    UiDebugOptions {
1678                        line_width: 3.,
1679                        outline_scrollbars: true,
1680                        show_hidden: false,
1681                        show_clipped: false,
1682                        ..debug_options
1683                    },
1684                    Children::spawn(SpawnIter((0..6).map(move |i| {
1685                        (
1686                            Node {
1687                                flex_direction: FlexDirection::Row,
1688                                ..default()
1689                            },
1690                            Children::spawn(SpawnIter((0..6).map({
1691                                move |j| {
1692                                    (
1693                                        Text(format!("Item {}", (i * 5) + j)),
1694                                        UiDebugOptions {
1695                                            enabled: false,
1696                                            ..default()
1697                                        },
1698                                    )
1699                                }
1700                            }))),
1701                            UiDebugOptions {
1702                                enabled: false,
1703                                ..default()
1704                            },
1705                        )
1706                    }))),
1707                ));
1708            });
1709    }
More examples
Hide additional examples
examples/ui/ui_transform.rs (lines 209-211)
102fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
103    // UI camera
104    commands.spawn(Camera2d);
105
106    // Root node filling the whole screen
107    commands.spawn((
108        Node {
109            width: percent(100),
110            height: percent(100),
111            align_items: AlignItems::Center,
112            justify_content: JustifyContent::Center,
113            ..default()
114        },
115        BackgroundColor(Color::BLACK),
116        children![(
117            Node {
118                align_items: AlignItems::Center,
119                justify_content: JustifyContent::SpaceEvenly,
120                column_gap: px(25),
121                row_gap: px(25),
122                ..default()
123            },
124            BackgroundColor(Color::BLACK),
125            children![
126                (
127                    Node {
128                        flex_direction: FlexDirection::Column,
129                        justify_content: JustifyContent::Center,
130                        row_gap: px(10),
131                        column_gap: px(10),
132                        padding: UiRect::all(px(10)),
133                        ..default()
134                    },
135                    BackgroundColor(Color::BLACK),
136                    GlobalZIndex(1),
137                    children![
138                        (
139                            Button,
140                            Node {
141                                height: px(50),
142                                width: px(50),
143                                align_items: AlignItems::Center,
144                                justify_content: JustifyContent::Center,
145                                ..default()
146                            },
147                            BackgroundColor(Color::WHITE),
148                            RotateButton(Rot2::radians(-FRAC_PI_8)),
149                            children![(Text::new("<--"), TextColor(Color::BLACK),)]
150                        ),
151                        (
152                            Button,
153                            Node {
154                                height: px(50),
155                                width: px(50),
156                                align_items: AlignItems::Center,
157                                justify_content: JustifyContent::Center,
158                                ..default()
159                            },
160                            BackgroundColor(Color::WHITE),
161                            ScaleButton(-0.25),
162                            children![(Text::new("-"), TextColor(Color::BLACK),)]
163                        ),
164                    ]
165                ),
166                // Target node with its own set of buttons
167                (
168                    Node {
169                        flex_direction: FlexDirection::Column,
170                        justify_content: JustifyContent::SpaceBetween,
171                        align_items: AlignItems::Center,
172                        width: px(300),
173                        height: px(300),
174                        ..default()
175                    },
176                    BackgroundColor(DARK_GRAY.into()),
177                    TargetNode,
178                    children![
179                        (
180                            Button,
181                            Node {
182                                width: px(80),
183                                height: px(80),
184                                align_items: AlignItems::Center,
185                                justify_content: JustifyContent::Center,
186                                ..default()
187                            },
188                            BackgroundColor(Color::WHITE),
189                            children![(Text::new("Top"), TextColor(Color::BLACK))]
190                        ),
191                        (
192                            Node {
193                                align_self: AlignSelf::Stretch,
194                                justify_content: JustifyContent::SpaceBetween,
195                                align_items: AlignItems::Center,
196                                ..default()
197                            },
198                            children![
199                                (
200                                    Button,
201                                    Node {
202                                        width: px(80),
203                                        height: px(80),
204                                        align_items: AlignItems::Center,
205                                        justify_content: JustifyContent::Center,
206                                        ..default()
207                                    },
208                                    BackgroundColor(Color::WHITE),
209                                    UiTransform::from_rotation(Rot2::radians(
210                                        -std::f32::consts::FRAC_PI_2
211                                    )),
212                                    children![(Text::new("Left"), TextColor(Color::BLACK),)]
213                                ),
214                                (
215                                    Node {
216                                        width: px(100),
217                                        height: px(100),
218                                        ..Default::default()
219                                    },
220                                    ImageNode {
221                                        image: asset_server.load("branding/icon.png"),
222                                        image_mode: NodeImageMode::Stretch,
223                                        ..default()
224                                    }
225                                ),
226                                (
227                                    Button,
228                                    Node {
229                                        width: px(80),
230                                        height: px(80),
231                                        align_items: AlignItems::Center,
232                                        justify_content: JustifyContent::Center,
233                                        ..default()
234                                    },
235                                    UiTransform::from_rotation(Rot2::radians(
236                                        core::f32::consts::FRAC_PI_2
237                                    )),
238                                    BackgroundColor(Color::WHITE),
239                                    children![(Text::new("Right"), TextColor(Color::BLACK))]
240                                ),
241                            ]
242                        ),
243                        (
244                            Button,
245                            Node {
246                                width: px(80),
247                                height: px(80),
248                                align_items: AlignItems::Center,
249                                justify_content: JustifyContent::Center,
250                                ..default()
251                            },
252                            BackgroundColor(Color::WHITE),
253                            UiTransform::from_rotation(Rot2::radians(std::f32::consts::PI)),
254                            children![(Text::new("Bottom"), TextColor(Color::BLACK),)]
255                        ),
256                    ]
257                ),
258                // Right column of controls
259                (
260                    Node {
261                        flex_direction: FlexDirection::Column,
262                        justify_content: JustifyContent::Center,
263                        row_gap: px(10),
264                        column_gap: px(10),
265                        padding: UiRect::all(px(10)),
266                        ..default()
267                    },
268                    BackgroundColor(Color::BLACK),
269                    GlobalZIndex(1),
270                    children![
271                        (
272                            Button,
273                            Node {
274                                height: px(50),
275                                width: px(50),
276                                align_items: AlignItems::Center,
277                                justify_content: JustifyContent::Center,
278                                ..default()
279                            },
280                            BackgroundColor(Color::WHITE),
281                            RotateButton(Rot2::radians(FRAC_PI_8)),
282                            children![(Text::new("-->"), TextColor(Color::BLACK),)]
283                        ),
284                        (
285                            Button,
286                            Node {
287                                height: px(50),
288                                width: px(50),
289                                align_items: AlignItems::Center,
290                                justify_content: JustifyContent::Center,
291                                ..default()
292                            },
293                            BackgroundColor(Color::WHITE),
294                            ScaleButton(0.25),
295                            children![(Text::new("+"), TextColor(Color::BLACK),)]
296                        ),
297                    ]
298                )
299            ]
300        )],
301    ));
302}
pub const fn from_translation(translation: Val2) -> UiTransform

Creates a UI transform representing a responsive translation.

Examples found in repository?
examples/testbed/ui.rs (line 1409)
1384    pub fn setup(mut commands: Commands) {
1385        commands.spawn((Camera2d, DespawnOnExit(super::Scene::Transformations)));
1386        commands
1387            .spawn((
1388                Node {
1389                    width: percent(100),
1390                    height: percent(100),
1391                    display: Display::Block,
1392                    ..default()
1393                },
1394                DespawnOnExit(super::Scene::Transformations),
1395            ))
1396            .with_children(|parent| {
1397                for (transformation, label, background) in [
1398                    (
1399                        UiTransform::from_rotation(Rot2::degrees(45.)),
1400                        "Rotate 45 degrees",
1401                        RED,
1402                    ),
1403                    (
1404                        UiTransform::from_scale(Vec2::new(2., 0.5)),
1405                        "Scale 2.x 0.5y",
1406                        GREEN,
1407                    ),
1408                    (
1409                        UiTransform::from_translation(Val2::px(-50., 50.)),
1410                        "Translate -50px x +50px y",
1411                        BLUE,
1412                    ),
1413                    (
1414                        UiTransform {
1415                            translation: Val2::px(50., 0.),
1416                            scale: Vec2::new(-1., 1.),
1417                            rotation: Rot2::degrees(30.),
1418                        },
1419                        "T 50px x\nS -1.x (refl)\nR 30deg",
1420                        DARK_CYAN,
1421                    ),
1422                ] {
1423                    parent
1424                        .spawn((Node {
1425                            width: percent(100),
1426                            margin: UiRect {
1427                                top: px(50),
1428                                bottom: px(50),
1429                                ..default()
1430                            },
1431                            align_items: AlignItems::Center,
1432                            justify_content: JustifyContent::SpaceAround,
1433                            ..default()
1434                        },))
1435                        .with_children(|row| {
1436                            row.spawn((
1437                                Text::new("Before Tf"),
1438                                Node {
1439                                    width: px(100),
1440                                    height: px(100),
1441                                    border_radius: BorderRadius::bottom_right(px(25.)),
1442                                    ..default()
1443                                },
1444                                BackgroundColor(background.into()),
1445                                TextFont::default(),
1446                            ));
1447                            row.spawn((
1448                                Text::new(label),
1449                                Node {
1450                                    width: px(100),
1451                                    height: px(100),
1452                                    border_radius: BorderRadius::bottom_right(px(25.)),
1453                                    ..default()
1454                                },
1455                                BackgroundColor(background.into()),
1456                                transformation,
1457                                TextFont::default(),
1458                            ));
1459                        });
1460                }
1461            });
1462    }
pub const fn from_scale(scale: Vec2) -> UiTransform

Creates a UI transform representing a scaling.

Examples found in repository?
examples/testbed/ui.rs (line 1404)
1384    pub fn setup(mut commands: Commands) {
1385        commands.spawn((Camera2d, DespawnOnExit(super::Scene::Transformations)));
1386        commands
1387            .spawn((
1388                Node {
1389                    width: percent(100),
1390                    height: percent(100),
1391                    display: Display::Block,
1392                    ..default()
1393                },
1394                DespawnOnExit(super::Scene::Transformations),
1395            ))
1396            .with_children(|parent| {
1397                for (transformation, label, background) in [
1398                    (
1399                        UiTransform::from_rotation(Rot2::degrees(45.)),
1400                        "Rotate 45 degrees",
1401                        RED,
1402                    ),
1403                    (
1404                        UiTransform::from_scale(Vec2::new(2., 0.5)),
1405                        "Scale 2.x 0.5y",
1406                        GREEN,
1407                    ),
1408                    (
1409                        UiTransform::from_translation(Val2::px(-50., 50.)),
1410                        "Translate -50px x +50px y",
1411                        BLUE,
1412                    ),
1413                    (
1414                        UiTransform {
1415                            translation: Val2::px(50., 0.),
1416                            scale: Vec2::new(-1., 1.),
1417                            rotation: Rot2::degrees(30.),
1418                        },
1419                        "T 50px x\nS -1.x (refl)\nR 30deg",
1420                        DARK_CYAN,
1421                    ),
1422                ] {
1423                    parent
1424                        .spawn((Node {
1425                            width: percent(100),
1426                            margin: UiRect {
1427                                top: px(50),
1428                                bottom: px(50),
1429                                ..default()
1430                            },
1431                            align_items: AlignItems::Center,
1432                            justify_content: JustifyContent::SpaceAround,
1433                            ..default()
1434                        },))
1435                        .with_children(|row| {
1436                            row.spawn((
1437                                Text::new("Before Tf"),
1438                                Node {
1439                                    width: px(100),
1440                                    height: px(100),
1441                                    border_radius: BorderRadius::bottom_right(px(25.)),
1442                                    ..default()
1443                                },
1444                                BackgroundColor(background.into()),
1445                                TextFont::default(),
1446                            ));
1447                            row.spawn((
1448                                Text::new(label),
1449                                Node {
1450                                    width: px(100),
1451                                    height: px(100),
1452                                    border_radius: BorderRadius::bottom_right(px(25.)),
1453                                    ..default()
1454                                },
1455                                BackgroundColor(background.into()),
1456                                transformation,
1457                                TextFont::default(),
1458                            ));
1459                        });
1460                }
1461            });
1462    }
pub const fn from_xy(x: Val, y: Val) -> UiTransform

Create a new UI transform at the position (x, y)

pub fn compute_affine( &self, scale_factor: f32, base_size: Vec2, target_size: Vec2, ) -> Affine2

Resolves the translation from the given scale_factor, base_value, and target_size and returns a 2d affine transform from the resolved translation, and the UiTransform’s rotation, and scale.