pub struct InlineImage {
pub color: Color,
pub image: Handle<Image>,
pub width: Option<f32>,
pub height: Option<f32>,
pub flip_x: bool,
pub flip_y: bool,
}bevy_ui only.Expand description
An inline image
Fields
color: ColorThe tint color used to draw the image.
This is multiplied by the color of each pixel in the image. The field value defaults to solid white, which will pass the image through unmodified.
image: Handle<Image>Handle to the texture.
This defaults to a TRANSPARENT_IMAGE_HANDLE, which points to a fully transparent 1x1 texture.
width: Option<f32>Set a fixed width in logical pixels for the image’s inline box target.
height: Option<f32>Set a fixed height in logical pixels for the image’s inline box target.
flip_x: boolFlip the image on the x-axis.
flip_y: boolFlip the image on the y-axis.
Implementations
Sourceimpl InlineImage
impl InlineImage
Sourcepub fn resolve_inline_box_size(&self, image_size: Vec2) -> Vec2
pub fn resolve_inline_box_size(&self, image_size: Vec2) -> Vec2
Resolve the inline box size. Preserves the image’s aspect ratio if dimensions unset.