pub struct MenuPopup {
pub layout: MenuLayout,
}Expand description
Component that defines a popup menu container.
Menus are automatically dismissed when the user clicks outside the menu bounds. Unlike a modal dialog, where the click event is intercepted, we don’t want to actually prevent the click event from triggering its normal action. The easiest way to detect this kind of click is to look for keyboard focus loss. When a menu is opened, one of its children will gain focus, and the menu remains open so long as at least one descendant is focused. Arrow keys can be used to navigate between menu items.
This means that popup menu must contain at least one focusable entity. It also means that two menus cannot be displayed at the same time unless one is an ancestor of the other.
Some care needs to be taken in implementing a menu button: we normally want menu buttons to toggle the open state of the menu; but clicking on the button will cause focus loss which means that the menu will always be closed by the time the click event is processed.
Fields
layout: MenuLayoutThe layout orientation of the menu