pub struct ScrollbarThumb {
pub border_radius: BorderRadius,
pub border: UiRect,
}Expand description
This component indicates that the entity is a scrollbar thumb (the moving, draggable part of the scrollbar). This should be a child of the scrollbar entity.
A ScrollbarThumb UI node does not have a Node component. It only has Border and BorderRadius styling properties.
Its layout is handled after ui_layout_system in update_scroll_thumb so that its size and position can be set relative to the scrolling area’s
size and scroll position.
Fields
border_radius: BorderRadiusBorder radius of the scrollbar thumb, used to update ComputedNode::border_radius in UiSystems::Layout.
border: UiRectThickness of the thumb node’s border.
If the border is too large to fit inside the thumb node, the width of the border will be scaled to fit.