pub struct Slider {
pub track_click: TrackClick,
pub orientation: SliderOrientation,
}Expand description
A headless slider widget, which can be used to build custom sliders. Sliders have a value
(represented by the SliderValue component) and a range (represented by SliderRange). An
optional step size can be specified via SliderStep, and you can control the rounding
during dragging with SliderPrecision.
You can also control the slider remotely by triggering a SetSliderValue event on it. This
can be useful in a console environment for controlling the value gamepad inputs.
Typically a slider will contain entities representing the “track” and “thumb” elements. The core
slider makes no assumptions about the hierarchical structure of these elements, but expects that
the thumb will be marked with a SliderThumb component.
The core slider does not modify the visible position of the thumb: that is the responsibility of the stylist. This can be done either in percent or pixel units as desired. To prevent overhang at the ends of the slider, the positioning should take into account the thumb width, by reducing the amount of travel. So for example, in a slider 100px wide, with a thumb that is 10px, the amount of travel is 90px. The core slider’s calculations for clicking and dragging assume this is the case, and will reduce the travel by the measured size of the thumb entity, which allows the movement of the thumb to be perfectly synchronized with the movement of the mouse.
In cases where overhang is desired for artistic reasons, the thumb may have additional decorative child elements, absolutely positioned, which don’t affect the size measurement.
Fields
track_click: TrackClickSet the track-clicking behavior for this slider.
orientation: SliderOrientationSet the slider orientation. Defaults to auto-detection based on dimensions.