pub enum ExitCondition {
OnPrimaryClosed,
OnAllClosed,
DontExit,
}Expand description
Defines the specific conditions the application should exit on
Variants
OnPrimaryClosed
Close application when the primary window is closed
The plugin will add exit_on_primary_closed to PostUpdate.
OnAllClosed
Close application when all windows are closed
The plugin will add exit_on_all_closed to PostUpdate.
DontExit
Keep application running headless even after closing all windows
If selecting this, ensure that you send the bevy_app::AppExit
event when the app should exit. If this does not occur, you will
create ‘headless’ processes (processes without windows), which may
surprise your users.