pub enum WindowEvent {
Show 27 variants
AppLifecycle(AppLifecycle),
CursorEntered(CursorEntered),
CursorLeft(CursorLeft),
CursorMoved(CursorMoved),
FileDragAndDrop(FileDragAndDrop),
Ime(Ime),
RequestRedraw(RequestRedraw),
WindowBackendScaleFactorChanged(WindowBackendScaleFactorChanged),
WindowCloseRequested(WindowCloseRequested),
WindowCreated(WindowCreated),
WindowDestroyed(WindowDestroyed),
WindowFocused(WindowFocused),
WindowMoved(WindowMoved),
WindowOccluded(WindowOccluded),
WindowResized(WindowResized),
WindowScaleFactorChanged(WindowScaleFactorChanged),
WindowThemeChanged(WindowThemeChanged),
MouseButtonInput(MouseButtonInput),
MouseMotion(MouseMotion),
MouseWheel(MouseWheel),
PinchGesture(PinchGesture),
RotationGesture(RotationGesture),
DoubleTapGesture(DoubleTapGesture),
PanGesture(PanGesture),
TouchInput(TouchInput),
KeyboardInput(KeyboardInput),
KeyboardFocusLost(KeyboardFocusLost),
}Expand description
Wraps all bevy_window and bevy_input events in a common enum.
Read these events with MessageReader<WindowEvent> if you need to
access window events in the order they were received from the
operating system. Otherwise, the event types are individually
readable with MessageReader<E> (e.g. MessageReader<KeyboardInput>).
Variants
AppLifecycle(AppLifecycle)
An application lifecycle event.
CursorEntered(CursorEntered)
The user’s cursor has entered a window.
CursorLeft(CursorLeft)
The user’s cursor has left a window.
CursorMoved(CursorMoved)
The user’s cursor has moved inside a window.
FileDragAndDrop(FileDragAndDrop)
A file drag and drop event.
Ime(Ime)
An Input Method Editor event.
RequestRedraw(RequestRedraw)
A redraw of all of the application’s windows has been requested.
WindowBackendScaleFactorChanged(WindowBackendScaleFactorChanged)
The window’s OS-reported scale factor has changed.
WindowCloseRequested(WindowCloseRequested)
The OS has requested that a window be closed.
WindowCreated(WindowCreated)
A new window has been created.
WindowDestroyed(WindowDestroyed)
A window has been destroyed by the underlying windowing system.
WindowFocused(WindowFocused)
A window has received or lost focus.
WindowMoved(WindowMoved)
A window has been moved.
WindowOccluded(WindowOccluded)
A window has started or stopped being occluded.
WindowResized(WindowResized)
A window’s logical size has changed.
WindowScaleFactorChanged(WindowScaleFactorChanged)
A window’s scale factor has changed.
WindowThemeChanged(WindowThemeChanged)
Sent for windows that are using the system theme when the system theme changes.
MouseButtonInput(MouseButtonInput)
The state of a mouse button has changed.
MouseMotion(MouseMotion)
The physical position of a pointing device has changed.
MouseWheel(MouseWheel)
The mouse wheel has moved.
PinchGesture(PinchGesture)
A two finger pinch gesture.
RotationGesture(RotationGesture)
A two finger rotation gesture.
DoubleTapGesture(DoubleTapGesture)
A double tap gesture.
PanGesture(PanGesture)
A pan gesture.
TouchInput(TouchInput)
A touch input state change.
KeyboardInput(KeyboardInput)
A keyboard input.
KeyboardFocusLost(KeyboardFocusLost)
Sent when focus has been lost for all Bevy windows.
Used to clear pressed key state.