pub struct CursorMoved {
pub window: Entity,
pub position: Vec2,
pub delta: Option<Vec2>,
}Expand description
An event reporting that the mouse cursor has moved inside a window.
The event is sent only if the cursor is over one of the application’s windows.
It is the translated version of WindowEvent::CursorMoved from the winit crate with the addition of delta.
Not to be confused with the MouseMotion event from bevy_input.
Because the range of data is limited by the window area and it may have been transformed by the OS to implement certain effects like acceleration,
you should not use it for non-cursor-like behavior such as 3D camera control. Please see MouseMotion instead.
Fields
window: EntityWindow that the cursor moved inside.
position: Vec2The cursor position in logical pixels.
delta: Option<Vec2>The change in the position of the cursor since the last event was sent.
This value is None if the cursor was outside the window area during the last frame.