pub struct WindowPlugin {
pub primary_window: Option<Window>,
pub primary_cursor_options: Option<CursorOptions>,
pub exit_condition: ExitCondition,
pub close_when_requested: bool,
}Expand description
A Plugin that defines an interface for windowing support in Bevy.
Fields
primary_window: Option<Window>Settings for the primary window.
Some(custom_window) will spawn an entity with custom_window and PrimaryWindow as components.
None will not spawn a primary window.
Defaults to Some(Window::default()).
Note that if there are no windows the App will exit (by default) due to
exit_on_all_closed.
primary_cursor_options: Option<CursorOptions>Settings for the cursor on the primary window.
Defaults to Some(CursorOptions::default()).
Has no effect if WindowPlugin::primary_window is None.
exit_condition: ExitConditionWhether to exit the app when there are no open windows.
If disabling this, ensure that you send the bevy_app::AppExit
event when the app should exit. If this does not occur, you will
create ‘headless’ processes (processes without windows), which may
surprise your users. It is recommended to leave this setting to
either ExitCondition::OnAllClosed or ExitCondition::OnPrimaryClosed.
ExitCondition::OnAllClosed will add exit_on_all_closed to Update.
ExitCondition::OnPrimaryClosed will add exit_on_primary_closed to Update.
close_when_requested: boolWhether to close windows when they are requested to be closed (i.e. when the close button is pressed).
If true, this plugin will add close_when_requested to Update.
If this system (or a replacement) is not running, the close button will have no effect.
This may surprise your users. It is recommended to leave this setting as true.