pub struct CustomCursorImage {
pub handle: Handle<Image>,
pub texture_atlas: Option<TextureAtlas>,
pub flip_x: bool,
pub flip_y: bool,
pub rect: Option<URect>,
pub hotspot: (u16, u16),
}Expand description
A custom cursor created from an image.
Fields
handle: Handle<Image>Handle to the image to use as the cursor. The image must be in 8 bit int or 32 bit float rgba. PNG images work well for this.
texture_atlas: Option<TextureAtlas>An optional texture atlas used to render the image.
flip_x: boolWhether the image should be flipped along its x-axis.
If true, the cursor’s hotspot automatically flips along with the
image.
flip_y: boolWhether the image should be flipped along its y-axis.
If true, the cursor’s hotspot automatically flips along with the
image.
rect: Option<URect>An optional rectangle representing the region of the image to render,
instead of rendering the full image. This is an easy one-off alternative
to using a TextureAtlas.
When used with a TextureAtlas, the rect is offset by the atlas’s
minimal (top-left) corner position.
hotspot: (u16, u16)X and Y coordinates of the hotspot in pixels. The hotspot must be within the image bounds.
If you are flipping the image using flip_x or flip_y, you don’t need
to adjust this field to account for the flip because it is adjusted
automatically.