WorldFilter

Enum WorldFilter 

pub enum WorldFilter {
    Unset,
    Allowlist(HashSet<TypeId>),
    Denylist(HashSet<TypeId>),
}
Available on crate feature bevy_world_serialization only.
A filter used to control which types can be added to a DynamicWorld.

This world filter can be used more generically to represent a filter for any given type; however, note that its intended usage with DynamicWorld only considers components and resources. Adding types that are not a component or resource will have no effect when used with DynamicWorld.

Variants



Unset

Represents an unset filter.

This is the equivalent of an empty Denylist or an Allowlist containing every type— essentially, all types are permissible.

Allowing a type will convert this filter to an Allowlist. Similarly, denying a type will convert this filter to a Denylist.



Allowlist(HashSet<TypeId>)

Contains the set of permitted types by their TypeId.

Types not contained within this set should not be allowed to be saved to an associated DynamicWorld.



Denylist(HashSet<TypeId>)

Contains the set of prohibited types by their TypeId.

Types contained within this set should not be allowed to be saved to an associated DynamicWorld.

Implementations

Source

impl WorldFilter

pub fn allow_all() -> WorldFilter

Creates a filter where all types are allowed.

This is the equivalent of creating an empty Denylist.

pub fn deny_all() -> WorldFilter

Creates a filter where all types are denied.

This is the equivalent of creating an empty Allowlist.

pub fn allow<T>(self) -> WorldFilter
where T: Any,

Allow the given type, T.

If this filter is already set as a Denylist, then the given type will be removed from the denied set.

If this filter is Unset, then it will be completely replaced by a new Allowlist.

pub fn allow_by_id(self, type_id: TypeId) -> WorldFilter

Allow the given type.

If this filter is already set as a Denylist, then the given type will be removed from the denied set.

If this filter is Unset, then it will be completely replaced by a new Allowlist.

pub fn deny<T>(self) -> WorldFilter
where T: Any,

Deny the given type, T.

If this filter is already set as an Allowlist, then the given type will be removed from the allowed set.

If this filter is Unset, then it will be completely replaced by a new Denylist.

pub fn deny_by_id(self, type_id: TypeId) -> WorldFilter

Deny the given type.

If this filter is already set as an Allowlist, then the given type will be removed from the allowed set.

If this filter is Unset, then it will be completely replaced by a new Denylist.

pub fn is_allowed<T>(&self) -> bool
where T: Any,

Returns true if the given type, T, is allowed by the filter.

If the filter is Unset, this will always return true.

pub fn is_allowed_by_id(&self, type_id: TypeId) -> bool

Returns true if the given type is allowed by the filter.

If the filter is Unset, this will always return true.

pub fn is_denied<T>(&self) -> bool
where T: Any,

Returns true if the given type, T, is denied by the filter.

If the filter is Unset, this will always return false.

pub fn is_denied_by_id(&self, type_id: TypeId) -> bool

Returns true if the given type is denied by the filter.

If the filter is Unset, this will always return false.

pub fn iter(&self) -> Box<dyn ExactSizeIterator<Item = &TypeId> + '_>

Returns an iterator over the items in the filter.

If the filter is Unset, this will return an empty iterator.

pub fn len(&self) -> usize

Returns the number of items in the filter.

If the filter is Unset, this will always return a length of zero.

pub fn is_empty(&self) -> bool

Returns true if there are zero items in the filter.

If the filter is Unset, this will always return true.