AnimationPlayer

Struct AnimationPlayer 

pub struct AnimationPlayer { /* private fields */ }
Animation controls.

Automatically added to any root animations of a scene when it is spawned.

Implementations

Source

impl AnimationPlayer

pub fn start(&mut self, animation: NodeIndex) -> &mut ActiveAnimation

Start playing an animation, restarting it if necessary.

pub fn play(&mut self, animation: NodeIndex) -> &mut ActiveAnimation

Start playing an animation, unless the requested animation is already playing.

Examples found in repository?
examples/3d/irradiance_volumes.rs (line 524)
515fn play_animations(
516    mut commands: Commands,
517    assets: Res<ExampleAssets>,
518    mut players: Query<(Entity, &mut AnimationPlayer), Without<AnimationGraphHandle>>,
519) {
520    for (entity, mut player) in players.iter_mut() {
521        commands
522            .entity(entity)
523            .insert(AnimationGraphHandle(assets.fox_animation_graph.clone()));
524        player.play(assets.fox_animation_node).repeat();
525    }
526}
More examples
Hide additional examples
examples/animation/animation_graph.rs (line 395)
379fn init_animations(
380    mut commands: Commands,
381    mut query: Query<(Entity, &mut AnimationPlayer)>,
382    animation_graph: Res<ExampleAnimationGraph>,
383    mut done: Local<bool>,
384) {
385    if *done {
386        return;
387    }
388
389    for (entity, mut player) in query.iter_mut() {
390        commands.entity(entity).insert((
391            AnimationGraphHandle(animation_graph.0.clone()),
392            ExampleAnimationWeights::default(),
393        ));
394        for &node_index in &CLIP_NODE_INDICES {
395            player.play(node_index.into()).repeat();
396        }
397
398        *done = true;
399    }
400}
401
402/// Read cursor position relative to clip nodes, allowing the user to change weights
403/// when dragging the node UI widgets.
404fn handle_weight_drag(
405    mut interaction_query: Query<(&Interaction, &RelativeCursorPosition, &ClipNode)>,
406    mut animation_weights_query: Query<&mut ExampleAnimationWeights>,
407) {
408    for (interaction, relative_cursor, clip_node) in &mut interaction_query {
409        if !matches!(*interaction, Interaction::Pressed) {
410            continue;
411        }
412
413        let Some(pos) = relative_cursor.normalized else {
414            continue;
415        };
416
417        for mut animation_weights in animation_weights_query.iter_mut() {
418            animation_weights.weights[clip_node.index] = pos.x.clamp(0., 1.);
419        }
420    }
421}
422
423// Updates the UI based on the weights that the user has chosen.
424fn update_ui(
425    mut text_query: Query<&mut Text>,
426    mut background_query: Query<&mut Node, Without<Text>>,
427    container_query: Query<(&Children, &ClipNode)>,
428    animation_weights_query: Query<&ExampleAnimationWeights, Changed<ExampleAnimationWeights>>,
429) {
430    for animation_weights in animation_weights_query.iter() {
431        for (children, clip_node) in &container_query {
432            // Draw the green background color to visually indicate the weight.
433            let mut bg_iter = background_query.iter_many_mut(children);
434            if let Some(mut node) = bg_iter.fetch_next() {
435                // All nodes are the same width, so `NODE_RECTS[0]` is as good as any other.
436                node.width = px(NODE_RECTS[0].width * animation_weights.weights[clip_node.index]);
437            }
438
439            // Update the node labels with the current weights.
440            let mut text_iter = text_query.iter_many_mut(children);
441            if let Some(mut text) = text_iter.fetch_next() {
442                **text = format!(
443                    "{}\n{:.2}",
444                    clip_node.text, animation_weights.weights[clip_node.index]
445                );
446            }
447        }
448    }
449}
450
451/// Takes the weights that were set in the UI and assigns them to the actual
452/// playing animation.
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454    for (mut animation_player, animation_weights) in query.iter_mut() {
455        for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456            .iter()
457            .zip(animation_weights.weights.iter())
458        {
459            // If the animation happens to be no longer active, restart it.
460            if !animation_player.is_playing_animation(animation_node_index.into()) {
461                animation_player.play(animation_node_index.into());
462            }
463
464            // Set the weight.
465            if let Some(active_animation) =
466                animation_player.animation_mut(animation_node_index.into())
467            {
468                active_animation.set_weight(animation_weight);
469            }
470        }
471    }
472}
examples/animation/morph_targets.rs (line 68)
58fn play_animation_when_ready(
59    scene_ready: On<WorldInstanceReady>,
60    mut commands: Commands,
61    children: Query<&Children>,
62    animations_to_play: Query<&AnimationToPlay>,
63    mut players: Query<&mut AnimationPlayer>,
64) {
65    if let Ok(animation_to_play) = animations_to_play.get(scene_ready.entity) {
66        for child in children.iter_descendants(scene_ready.entity) {
67            if let Ok(mut player) = players.get_mut(child) {
68                player.play(animation_to_play.index).repeat();
69
70                commands
71                    .entity(child)
72                    .insert(AnimationGraphHandle(animation_to_play.graph_handle.clone()));
73            }
74        }
75    }
76}
tests/3d/test_skinned_mesh_bounds.rs (line 107)
97fn play_animation(
98    trigger: On<WorldInstanceReady>,
99    mut commands: Commands,
100    children: Query<&Children>,
101    animations: Query<&PendingAnimation>,
102    mut players: Query<&mut AnimationPlayer>,
103) {
104    if let Ok(PendingAnimation((graph_handle, graph_node_index))) = animations.get(trigger.entity) {
105        for child in children.iter_descendants(trigger.entity) {
106            if let Ok(mut player) = players.get_mut(child) {
107                player.play(*graph_node_index).set_speed(0.6).repeat();
108
109                commands
110                    .entity(child)
111                    .insert(AnimationGraphHandle(graph_handle.clone()));
112            }
113        }
114    }
115
116    commands.entity(trigger.entity).remove::<PendingAnimation>();
117}
examples/stress_tests/many_foxes.rs (line 261)
251fn setup_scene_once_loaded(
252    scene_ready: On<WorldInstanceReady>,
253    animations: Res<Animations>,
254    foxes: Res<Foxes>,
255    mut commands: Commands,
256    children: Query<&Children>,
257    mut players: Query<&mut AnimationPlayer>,
258) {
259    for child in children.iter_descendants(scene_ready.entity) {
260        if let Ok(mut player) = players.get_mut(child) {
261            let playing_animation = player.play(animations.node_indices[0]).repeat();
262            if !foxes.sync {
263                playing_animation.seek_to(scene_ready.entity.index_u32() as f32 / 10.0);
264            }
265            commands.entity(child).insert((
266                AnimationGraphHandle(animations.graph.clone()),
267                AnimationTransitions::default(),
268            ));
269        }
270    }
271}
examples/stress_tests/many_morph_targets.rs (line 384)
366fn play_animation(
367    trigger: On<WorldInstanceReady>,
368    mut commands: Commands,
369    args: Res<Args>,
370    children: Query<&Children>,
371    animations_to_play: Query<&AnimationToPlay>,
372    mut players: Query<&mut AnimationPlayer>,
373) {
374    if args.weights == ArgWeights::Animated
375        && let Ok(animation_to_play) = animations_to_play.get(trigger.entity)
376    {
377        for child in children.iter_descendants(trigger.entity) {
378            if let Ok(mut player) = players.get_mut(child) {
379                commands
380                    .entity(child)
381                    .insert(AnimationGraphHandle(animation_to_play.graph_handle.clone()));
382
383                player
384                    .play(animation_to_play.index)
385                    .repeat()
386                    .set_speed(animation_to_play.speed);
387            }
388        }
389    }
390}
pub fn stop(&mut self, animation: NodeIndex) -> &mut AnimationPlayer

Stops playing the given animation, removing it from the list of playing animations.

pub fn stop_all(&mut self) -> &mut AnimationPlayer

Stops all currently-playing animations.

pub fn playing_animations( &self, ) -> impl Iterator<Item = (&NodeIndex, &ActiveAnimation)>

Iterates through all animations that this AnimationPlayer is currently playing.

Examples found in repository?
examples/animation/animated_mesh_control.rs (line 172)
165fn keyboard_control(
166    keyboard_input: Res<ButtonInput<KeyCode>>,
167    mut animation_players: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
168    animations: Res<Animations>,
169    mut current_animation: Local<usize>,
170) {
171    for (mut player, mut transitions) in &mut animation_players {
172        let Some((&playing_animation_index, _)) = player.playing_animations().next() else {
173            continue;
174        };
175
176        if keyboard_input.just_pressed(KeyCode::Space) {
177            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
178            if playing_animation.is_paused() {
179                playing_animation.resume();
180            } else {
181                playing_animation.pause();
182            }
183        }
184
185        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
186            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
187            let speed = playing_animation.speed();
188            playing_animation.set_speed(speed * 1.2);
189        }
190
191        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
192            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
193            let speed = playing_animation.speed();
194            playing_animation.set_speed(speed * 0.8);
195        }
196
197        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
198            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
199            let elapsed = playing_animation.seek_time();
200            playing_animation.seek_to(elapsed - 0.1);
201        }
202
203        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
204            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
205            let elapsed = playing_animation.seek_time();
206            playing_animation.seek_to(elapsed + 0.1);
207        }
208
209        if keyboard_input.just_pressed(KeyCode::Enter) {
210            *current_animation = (*current_animation + 1) % animations.animations.len();
211
212            transitions
213                .play(
214                    &mut player,
215                    animations.animations[*current_animation],
216                    Duration::from_millis(250),
217                )
218                .repeat();
219        }
220
221        if keyboard_input.just_pressed(KeyCode::Digit1) {
222            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
223            playing_animation
224                .set_repeat(RepeatAnimation::Count(1))
225                .replay();
226        }
227
228        if keyboard_input.just_pressed(KeyCode::Digit2) {
229            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
230            playing_animation
231                .set_repeat(RepeatAnimation::Count(2))
232                .replay();
233        }
234
235        if keyboard_input.just_pressed(KeyCode::Digit3) {
236            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
237            playing_animation
238                .set_repeat(RepeatAnimation::Count(3))
239                .replay();
240        }
241
242        if keyboard_input.just_pressed(KeyCode::KeyL) {
243            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
244            playing_animation.set_repeat(RepeatAnimation::Forever);
245        }
246    }
247}
pub fn playing_animations_mut( &mut self, ) -> impl Iterator<Item = (&NodeIndex, &mut ActiveAnimation)>

Iterates through all animations that this AnimationPlayer is currently playing, mutably.

pub fn is_playing_animation(&self, animation: NodeIndex) -> bool

Returns true if the animation is currently playing or paused, or false if the animation is stopped.

Examples found in repository?
examples/animation/animation_graph.rs (line 460)
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454    for (mut animation_player, animation_weights) in query.iter_mut() {
455        for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456            .iter()
457            .zip(animation_weights.weights.iter())
458        {
459            // If the animation happens to be no longer active, restart it.
460            if !animation_player.is_playing_animation(animation_node_index.into()) {
461                animation_player.play(animation_node_index.into());
462            }
463
464            // Set the weight.
465            if let Some(active_animation) =
466                animation_player.animation_mut(animation_node_index.into())
467            {
468                active_animation.set_weight(animation_weight);
469            }
470        }
471    }
472}
pub fn all_finished(&self) -> bool

Check if all playing animations have finished, according to the repetition behavior.

pub fn all_paused(&self) -> bool

Check if all playing animations are paused.

Examples found in repository?
examples/stress_tests/many_foxes.rs (line 314)
289fn keyboard_animation_control(
290    keyboard_input: Res<ButtonInput<KeyCode>>,
291    mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292    animations: Res<Animations>,
293    mut current_animation: Local<usize>,
294    mut foxes: ResMut<Foxes>,
295) {
296    if keyboard_input.just_pressed(KeyCode::Space) {
297        foxes.moving = !foxes.moving;
298    }
299
300    if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301        foxes.speed *= 1.25;
302    }
303
304    if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305        foxes.speed *= 0.8;
306    }
307
308    if keyboard_input.just_pressed(KeyCode::Enter) {
309        *current_animation = (*current_animation + 1) % animations.node_indices.len();
310    }
311
312    for (mut player, mut transitions) in &mut animation_player {
313        if keyboard_input.just_pressed(KeyCode::Space) {
314            if player.all_paused() {
315                player.resume_all();
316            } else {
317                player.pause_all();
318            }
319        }
320
321        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322            player.adjust_speeds(1.25);
323        }
324
325        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326            player.adjust_speeds(0.8);
327        }
328
329        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330            player.seek_all_by(-0.1);
331        }
332
333        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334            player.seek_all_by(0.1);
335        }
336
337        if keyboard_input.just_pressed(KeyCode::Enter) {
338            transitions
339                .play(
340                    &mut player,
341                    animations.node_indices[*current_animation],
342                    Duration::from_millis(250),
343                )
344                .repeat();
345        }
346    }
347}
pub fn pause_all(&mut self) -> &mut AnimationPlayer

Pause all playing animations.

Examples found in repository?
examples/stress_tests/many_foxes.rs (line 317)
289fn keyboard_animation_control(
290    keyboard_input: Res<ButtonInput<KeyCode>>,
291    mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292    animations: Res<Animations>,
293    mut current_animation: Local<usize>,
294    mut foxes: ResMut<Foxes>,
295) {
296    if keyboard_input.just_pressed(KeyCode::Space) {
297        foxes.moving = !foxes.moving;
298    }
299
300    if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301        foxes.speed *= 1.25;
302    }
303
304    if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305        foxes.speed *= 0.8;
306    }
307
308    if keyboard_input.just_pressed(KeyCode::Enter) {
309        *current_animation = (*current_animation + 1) % animations.node_indices.len();
310    }
311
312    for (mut player, mut transitions) in &mut animation_player {
313        if keyboard_input.just_pressed(KeyCode::Space) {
314            if player.all_paused() {
315                player.resume_all();
316            } else {
317                player.pause_all();
318            }
319        }
320
321        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322            player.adjust_speeds(1.25);
323        }
324
325        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326            player.adjust_speeds(0.8);
327        }
328
329        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330            player.seek_all_by(-0.1);
331        }
332
333        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334            player.seek_all_by(0.1);
335        }
336
337        if keyboard_input.just_pressed(KeyCode::Enter) {
338            transitions
339                .play(
340                    &mut player,
341                    animations.node_indices[*current_animation],
342                    Duration::from_millis(250),
343                )
344                .repeat();
345        }
346    }
347}
pub fn resume_all(&mut self) -> &mut AnimationPlayer

Resume all active animations.

Examples found in repository?
examples/stress_tests/many_foxes.rs (line 315)
289fn keyboard_animation_control(
290    keyboard_input: Res<ButtonInput<KeyCode>>,
291    mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292    animations: Res<Animations>,
293    mut current_animation: Local<usize>,
294    mut foxes: ResMut<Foxes>,
295) {
296    if keyboard_input.just_pressed(KeyCode::Space) {
297        foxes.moving = !foxes.moving;
298    }
299
300    if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301        foxes.speed *= 1.25;
302    }
303
304    if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305        foxes.speed *= 0.8;
306    }
307
308    if keyboard_input.just_pressed(KeyCode::Enter) {
309        *current_animation = (*current_animation + 1) % animations.node_indices.len();
310    }
311
312    for (mut player, mut transitions) in &mut animation_player {
313        if keyboard_input.just_pressed(KeyCode::Space) {
314            if player.all_paused() {
315                player.resume_all();
316            } else {
317                player.pause_all();
318            }
319        }
320
321        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322            player.adjust_speeds(1.25);
323        }
324
325        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326            player.adjust_speeds(0.8);
327        }
328
329        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330            player.seek_all_by(-0.1);
331        }
332
333        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334            player.seek_all_by(0.1);
335        }
336
337        if keyboard_input.just_pressed(KeyCode::Enter) {
338            transitions
339                .play(
340                    &mut player,
341                    animations.node_indices[*current_animation],
342                    Duration::from_millis(250),
343                )
344                .repeat();
345        }
346    }
347}
pub fn rewind_all(&mut self) -> &mut AnimationPlayer

Rewinds all active animations.

pub fn adjust_speeds(&mut self, factor: f32) -> &mut AnimationPlayer

Multiplies the speed of all active animations by the given factor.

Examples found in repository?
examples/stress_tests/many_foxes.rs (line 322)
289fn keyboard_animation_control(
290    keyboard_input: Res<ButtonInput<KeyCode>>,
291    mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292    animations: Res<Animations>,
293    mut current_animation: Local<usize>,
294    mut foxes: ResMut<Foxes>,
295) {
296    if keyboard_input.just_pressed(KeyCode::Space) {
297        foxes.moving = !foxes.moving;
298    }
299
300    if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301        foxes.speed *= 1.25;
302    }
303
304    if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305        foxes.speed *= 0.8;
306    }
307
308    if keyboard_input.just_pressed(KeyCode::Enter) {
309        *current_animation = (*current_animation + 1) % animations.node_indices.len();
310    }
311
312    for (mut player, mut transitions) in &mut animation_player {
313        if keyboard_input.just_pressed(KeyCode::Space) {
314            if player.all_paused() {
315                player.resume_all();
316            } else {
317                player.pause_all();
318            }
319        }
320
321        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322            player.adjust_speeds(1.25);
323        }
324
325        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326            player.adjust_speeds(0.8);
327        }
328
329        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330            player.seek_all_by(-0.1);
331        }
332
333        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334            player.seek_all_by(0.1);
335        }
336
337        if keyboard_input.just_pressed(KeyCode::Enter) {
338            transitions
339                .play(
340                    &mut player,
341                    animations.node_indices[*current_animation],
342                    Duration::from_millis(250),
343                )
344                .repeat();
345        }
346    }
347}
pub fn seek_all_by(&mut self, amount: f32) -> &mut AnimationPlayer

Seeks all active animations forward or backward by the same amount.

To seek forward, pass a positive value; to seek negative, pass a negative value. Values below 0.0 or beyond the end of the animation clip are clamped appropriately.

Examples found in repository?
examples/stress_tests/many_foxes.rs (line 330)
289fn keyboard_animation_control(
290    keyboard_input: Res<ButtonInput<KeyCode>>,
291    mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292    animations: Res<Animations>,
293    mut current_animation: Local<usize>,
294    mut foxes: ResMut<Foxes>,
295) {
296    if keyboard_input.just_pressed(KeyCode::Space) {
297        foxes.moving = !foxes.moving;
298    }
299
300    if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301        foxes.speed *= 1.25;
302    }
303
304    if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305        foxes.speed *= 0.8;
306    }
307
308    if keyboard_input.just_pressed(KeyCode::Enter) {
309        *current_animation = (*current_animation + 1) % animations.node_indices.len();
310    }
311
312    for (mut player, mut transitions) in &mut animation_player {
313        if keyboard_input.just_pressed(KeyCode::Space) {
314            if player.all_paused() {
315                player.resume_all();
316            } else {
317                player.pause_all();
318            }
319        }
320
321        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322            player.adjust_speeds(1.25);
323        }
324
325        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326            player.adjust_speeds(0.8);
327        }
328
329        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330            player.seek_all_by(-0.1);
331        }
332
333        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334            player.seek_all_by(0.1);
335        }
336
337        if keyboard_input.just_pressed(KeyCode::Enter) {
338            transitions
339                .play(
340                    &mut player,
341                    animations.node_indices[*current_animation],
342                    Duration::from_millis(250),
343                )
344                .repeat();
345        }
346    }
347}
pub fn animation(&self, animation: NodeIndex) -> Option<&ActiveAnimation>

Returns the ActiveAnimation associated with the given animation node if it’s currently playing.

If the animation isn’t currently active, returns None.

pub fn animation_mut( &mut self, animation: NodeIndex, ) -> Option<&mut ActiveAnimation>

Returns a mutable reference to the ActiveAnimation associated with the given animation node if it’s currently active.

If the animation isn’t currently active, returns None.

Examples found in repository?
examples/animation/animation_graph.rs (line 466)
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454    for (mut animation_player, animation_weights) in query.iter_mut() {
455        for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456            .iter()
457            .zip(animation_weights.weights.iter())
458        {
459            // If the animation happens to be no longer active, restart it.
460            if !animation_player.is_playing_animation(animation_node_index.into()) {
461                animation_player.play(animation_node_index.into());
462            }
463
464            // Set the weight.
465            if let Some(active_animation) =
466                animation_player.animation_mut(animation_node_index.into())
467            {
468                active_animation.set_weight(animation_weight);
469            }
470        }
471    }
472}
More examples
Hide additional examples
examples/animation/animated_mesh_control.rs (line 177)
165fn keyboard_control(
166    keyboard_input: Res<ButtonInput<KeyCode>>,
167    mut animation_players: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
168    animations: Res<Animations>,
169    mut current_animation: Local<usize>,
170) {
171    for (mut player, mut transitions) in &mut animation_players {
172        let Some((&playing_animation_index, _)) = player.playing_animations().next() else {
173            continue;
174        };
175
176        if keyboard_input.just_pressed(KeyCode::Space) {
177            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
178            if playing_animation.is_paused() {
179                playing_animation.resume();
180            } else {
181                playing_animation.pause();
182            }
183        }
184
185        if keyboard_input.just_pressed(KeyCode::ArrowUp) {
186            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
187            let speed = playing_animation.speed();
188            playing_animation.set_speed(speed * 1.2);
189        }
190
191        if keyboard_input.just_pressed(KeyCode::ArrowDown) {
192            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
193            let speed = playing_animation.speed();
194            playing_animation.set_speed(speed * 0.8);
195        }
196
197        if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
198            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
199            let elapsed = playing_animation.seek_time();
200            playing_animation.seek_to(elapsed - 0.1);
201        }
202
203        if keyboard_input.just_pressed(KeyCode::ArrowRight) {
204            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
205            let elapsed = playing_animation.seek_time();
206            playing_animation.seek_to(elapsed + 0.1);
207        }
208
209        if keyboard_input.just_pressed(KeyCode::Enter) {
210            *current_animation = (*current_animation + 1) % animations.animations.len();
211
212            transitions
213                .play(
214                    &mut player,
215                    animations.animations[*current_animation],
216                    Duration::from_millis(250),
217                )
218                .repeat();
219        }
220
221        if keyboard_input.just_pressed(KeyCode::Digit1) {
222            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
223            playing_animation
224                .set_repeat(RepeatAnimation::Count(1))
225                .replay();
226        }
227
228        if keyboard_input.just_pressed(KeyCode::Digit2) {
229            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
230            playing_animation
231                .set_repeat(RepeatAnimation::Count(2))
232                .replay();
233        }
234
235        if keyboard_input.just_pressed(KeyCode::Digit3) {
236            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
237            playing_animation
238                .set_repeat(RepeatAnimation::Count(3))
239                .replay();
240        }
241
242        if keyboard_input.just_pressed(KeyCode::KeyL) {
243            let playing_animation = player.animation_mut(playing_animation_index).unwrap();
244            playing_animation.set_repeat(RepeatAnimation::Forever);
245        }
246    }
247}