pub struct AnimationPlayer { /* private fields */ }Expand description
Animation controls.
Automatically added to any root animations of a scene when it is spawned.
Implementations
Sourceimpl AnimationPlayer
impl AnimationPlayer
Sourcepub fn start(&mut self, animation: NodeIndex) -> &mut ActiveAnimation
pub fn start(&mut self, animation: NodeIndex) -> &mut ActiveAnimation
Start playing an animation, restarting it if necessary.
Sourcepub fn play(&mut self, animation: NodeIndex) -> &mut ActiveAnimation
pub fn play(&mut self, animation: NodeIndex) -> &mut ActiveAnimation
Start playing an animation, unless the requested animation is already playing.
Examples found in repository?
515fn play_animations(
516 mut commands: Commands,
517 assets: Res<ExampleAssets>,
518 mut players: Query<(Entity, &mut AnimationPlayer), Without<AnimationGraphHandle>>,
519) {
520 for (entity, mut player) in players.iter_mut() {
521 commands
522 .entity(entity)
523 .insert(AnimationGraphHandle(assets.fox_animation_graph.clone()));
524 player.play(assets.fox_animation_node).repeat();
525 }
526}More examples
379fn init_animations(
380 mut commands: Commands,
381 mut query: Query<(Entity, &mut AnimationPlayer)>,
382 animation_graph: Res<ExampleAnimationGraph>,
383 mut done: Local<bool>,
384) {
385 if *done {
386 return;
387 }
388
389 for (entity, mut player) in query.iter_mut() {
390 commands.entity(entity).insert((
391 AnimationGraphHandle(animation_graph.0.clone()),
392 ExampleAnimationWeights::default(),
393 ));
394 for &node_index in &CLIP_NODE_INDICES {
395 player.play(node_index.into()).repeat();
396 }
397
398 *done = true;
399 }
400}
401
402/// Read cursor position relative to clip nodes, allowing the user to change weights
403/// when dragging the node UI widgets.
404fn handle_weight_drag(
405 mut interaction_query: Query<(&Interaction, &RelativeCursorPosition, &ClipNode)>,
406 mut animation_weights_query: Query<&mut ExampleAnimationWeights>,
407) {
408 for (interaction, relative_cursor, clip_node) in &mut interaction_query {
409 if !matches!(*interaction, Interaction::Pressed) {
410 continue;
411 }
412
413 let Some(pos) = relative_cursor.normalized else {
414 continue;
415 };
416
417 for mut animation_weights in animation_weights_query.iter_mut() {
418 animation_weights.weights[clip_node.index] = pos.x.clamp(0., 1.);
419 }
420 }
421}
422
423// Updates the UI based on the weights that the user has chosen.
424fn update_ui(
425 mut text_query: Query<&mut Text>,
426 mut background_query: Query<&mut Node, Without<Text>>,
427 container_query: Query<(&Children, &ClipNode)>,
428 animation_weights_query: Query<&ExampleAnimationWeights, Changed<ExampleAnimationWeights>>,
429) {
430 for animation_weights in animation_weights_query.iter() {
431 for (children, clip_node) in &container_query {
432 // Draw the green background color to visually indicate the weight.
433 let mut bg_iter = background_query.iter_many_mut(children);
434 if let Some(mut node) = bg_iter.fetch_next() {
435 // All nodes are the same width, so `NODE_RECTS[0]` is as good as any other.
436 node.width = px(NODE_RECTS[0].width * animation_weights.weights[clip_node.index]);
437 }
438
439 // Update the node labels with the current weights.
440 let mut text_iter = text_query.iter_many_mut(children);
441 if let Some(mut text) = text_iter.fetch_next() {
442 **text = format!(
443 "{}\n{:.2}",
444 clip_node.text, animation_weights.weights[clip_node.index]
445 );
446 }
447 }
448 }
449}
450
451/// Takes the weights that were set in the UI and assigns them to the actual
452/// playing animation.
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454 for (mut animation_player, animation_weights) in query.iter_mut() {
455 for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456 .iter()
457 .zip(animation_weights.weights.iter())
458 {
459 // If the animation happens to be no longer active, restart it.
460 if !animation_player.is_playing_animation(animation_node_index.into()) {
461 animation_player.play(animation_node_index.into());
462 }
463
464 // Set the weight.
465 if let Some(active_animation) =
466 animation_player.animation_mut(animation_node_index.into())
467 {
468 active_animation.set_weight(animation_weight);
469 }
470 }
471 }
472}58fn play_animation_when_ready(
59 scene_ready: On<WorldInstanceReady>,
60 mut commands: Commands,
61 children: Query<&Children>,
62 animations_to_play: Query<&AnimationToPlay>,
63 mut players: Query<&mut AnimationPlayer>,
64) {
65 if let Ok(animation_to_play) = animations_to_play.get(scene_ready.entity) {
66 for child in children.iter_descendants(scene_ready.entity) {
67 if let Ok(mut player) = players.get_mut(child) {
68 player.play(animation_to_play.index).repeat();
69
70 commands
71 .entity(child)
72 .insert(AnimationGraphHandle(animation_to_play.graph_handle.clone()));
73 }
74 }
75 }
76}97fn play_animation(
98 trigger: On<WorldInstanceReady>,
99 mut commands: Commands,
100 children: Query<&Children>,
101 animations: Query<&PendingAnimation>,
102 mut players: Query<&mut AnimationPlayer>,
103) {
104 if let Ok(PendingAnimation((graph_handle, graph_node_index))) = animations.get(trigger.entity) {
105 for child in children.iter_descendants(trigger.entity) {
106 if let Ok(mut player) = players.get_mut(child) {
107 player.play(*graph_node_index).set_speed(0.6).repeat();
108
109 commands
110 .entity(child)
111 .insert(AnimationGraphHandle(graph_handle.clone()));
112 }
113 }
114 }
115
116 commands.entity(trigger.entity).remove::<PendingAnimation>();
117}251fn setup_scene_once_loaded(
252 scene_ready: On<WorldInstanceReady>,
253 animations: Res<Animations>,
254 foxes: Res<Foxes>,
255 mut commands: Commands,
256 children: Query<&Children>,
257 mut players: Query<&mut AnimationPlayer>,
258) {
259 for child in children.iter_descendants(scene_ready.entity) {
260 if let Ok(mut player) = players.get_mut(child) {
261 let playing_animation = player.play(animations.node_indices[0]).repeat();
262 if !foxes.sync {
263 playing_animation.seek_to(scene_ready.entity.index_u32() as f32 / 10.0);
264 }
265 commands.entity(child).insert((
266 AnimationGraphHandle(animations.graph.clone()),
267 AnimationTransitions::default(),
268 ));
269 }
270 }
271}366fn play_animation(
367 trigger: On<WorldInstanceReady>,
368 mut commands: Commands,
369 args: Res<Args>,
370 children: Query<&Children>,
371 animations_to_play: Query<&AnimationToPlay>,
372 mut players: Query<&mut AnimationPlayer>,
373) {
374 if args.weights == ArgWeights::Animated
375 && let Ok(animation_to_play) = animations_to_play.get(trigger.entity)
376 {
377 for child in children.iter_descendants(trigger.entity) {
378 if let Ok(mut player) = players.get_mut(child) {
379 commands
380 .entity(child)
381 .insert(AnimationGraphHandle(animation_to_play.graph_handle.clone()));
382
383 player
384 .play(animation_to_play.index)
385 .repeat()
386 .set_speed(animation_to_play.speed);
387 }
388 }
389 }
390}Sourcepub fn stop(&mut self, animation: NodeIndex) -> &mut AnimationPlayer
pub fn stop(&mut self, animation: NodeIndex) -> &mut AnimationPlayer
Stops playing the given animation, removing it from the list of playing animations.
Sourcepub fn stop_all(&mut self) -> &mut AnimationPlayer
pub fn stop_all(&mut self) -> &mut AnimationPlayer
Stops all currently-playing animations.
Sourcepub fn playing_animations(
&self,
) -> impl Iterator<Item = (&NodeIndex, &ActiveAnimation)>
pub fn playing_animations( &self, ) -> impl Iterator<Item = (&NodeIndex, &ActiveAnimation)>
Iterates through all animations that this AnimationPlayer is
currently playing.
Examples found in repository?
165fn keyboard_control(
166 keyboard_input: Res<ButtonInput<KeyCode>>,
167 mut animation_players: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
168 animations: Res<Animations>,
169 mut current_animation: Local<usize>,
170) {
171 for (mut player, mut transitions) in &mut animation_players {
172 let Some((&playing_animation_index, _)) = player.playing_animations().next() else {
173 continue;
174 };
175
176 if keyboard_input.just_pressed(KeyCode::Space) {
177 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
178 if playing_animation.is_paused() {
179 playing_animation.resume();
180 } else {
181 playing_animation.pause();
182 }
183 }
184
185 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
186 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
187 let speed = playing_animation.speed();
188 playing_animation.set_speed(speed * 1.2);
189 }
190
191 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
192 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
193 let speed = playing_animation.speed();
194 playing_animation.set_speed(speed * 0.8);
195 }
196
197 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
198 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
199 let elapsed = playing_animation.seek_time();
200 playing_animation.seek_to(elapsed - 0.1);
201 }
202
203 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
204 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
205 let elapsed = playing_animation.seek_time();
206 playing_animation.seek_to(elapsed + 0.1);
207 }
208
209 if keyboard_input.just_pressed(KeyCode::Enter) {
210 *current_animation = (*current_animation + 1) % animations.animations.len();
211
212 transitions
213 .play(
214 &mut player,
215 animations.animations[*current_animation],
216 Duration::from_millis(250),
217 )
218 .repeat();
219 }
220
221 if keyboard_input.just_pressed(KeyCode::Digit1) {
222 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
223 playing_animation
224 .set_repeat(RepeatAnimation::Count(1))
225 .replay();
226 }
227
228 if keyboard_input.just_pressed(KeyCode::Digit2) {
229 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
230 playing_animation
231 .set_repeat(RepeatAnimation::Count(2))
232 .replay();
233 }
234
235 if keyboard_input.just_pressed(KeyCode::Digit3) {
236 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
237 playing_animation
238 .set_repeat(RepeatAnimation::Count(3))
239 .replay();
240 }
241
242 if keyboard_input.just_pressed(KeyCode::KeyL) {
243 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
244 playing_animation.set_repeat(RepeatAnimation::Forever);
245 }
246 }
247}Sourcepub fn playing_animations_mut(
&mut self,
) -> impl Iterator<Item = (&NodeIndex, &mut ActiveAnimation)>
pub fn playing_animations_mut( &mut self, ) -> impl Iterator<Item = (&NodeIndex, &mut ActiveAnimation)>
Iterates through all animations that this AnimationPlayer is
currently playing, mutably.
Sourcepub fn is_playing_animation(&self, animation: NodeIndex) -> bool
pub fn is_playing_animation(&self, animation: NodeIndex) -> bool
Returns true if the animation is currently playing or paused, or false if the animation is stopped.
Examples found in repository?
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454 for (mut animation_player, animation_weights) in query.iter_mut() {
455 for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456 .iter()
457 .zip(animation_weights.weights.iter())
458 {
459 // If the animation happens to be no longer active, restart it.
460 if !animation_player.is_playing_animation(animation_node_index.into()) {
461 animation_player.play(animation_node_index.into());
462 }
463
464 // Set the weight.
465 if let Some(active_animation) =
466 animation_player.animation_mut(animation_node_index.into())
467 {
468 active_animation.set_weight(animation_weight);
469 }
470 }
471 }
472}Sourcepub fn all_finished(&self) -> bool
pub fn all_finished(&self) -> bool
Check if all playing animations have finished, according to the repetition behavior.
Sourcepub fn all_paused(&self) -> bool
pub fn all_paused(&self) -> bool
Check if all playing animations are paused.
Examples found in repository?
289fn keyboard_animation_control(
290 keyboard_input: Res<ButtonInput<KeyCode>>,
291 mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292 animations: Res<Animations>,
293 mut current_animation: Local<usize>,
294 mut foxes: ResMut<Foxes>,
295) {
296 if keyboard_input.just_pressed(KeyCode::Space) {
297 foxes.moving = !foxes.moving;
298 }
299
300 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301 foxes.speed *= 1.25;
302 }
303
304 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305 foxes.speed *= 0.8;
306 }
307
308 if keyboard_input.just_pressed(KeyCode::Enter) {
309 *current_animation = (*current_animation + 1) % animations.node_indices.len();
310 }
311
312 for (mut player, mut transitions) in &mut animation_player {
313 if keyboard_input.just_pressed(KeyCode::Space) {
314 if player.all_paused() {
315 player.resume_all();
316 } else {
317 player.pause_all();
318 }
319 }
320
321 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322 player.adjust_speeds(1.25);
323 }
324
325 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326 player.adjust_speeds(0.8);
327 }
328
329 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330 player.seek_all_by(-0.1);
331 }
332
333 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334 player.seek_all_by(0.1);
335 }
336
337 if keyboard_input.just_pressed(KeyCode::Enter) {
338 transitions
339 .play(
340 &mut player,
341 animations.node_indices[*current_animation],
342 Duration::from_millis(250),
343 )
344 .repeat();
345 }
346 }
347}Sourcepub fn pause_all(&mut self) -> &mut AnimationPlayer
pub fn pause_all(&mut self) -> &mut AnimationPlayer
Pause all playing animations.
Examples found in repository?
289fn keyboard_animation_control(
290 keyboard_input: Res<ButtonInput<KeyCode>>,
291 mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292 animations: Res<Animations>,
293 mut current_animation: Local<usize>,
294 mut foxes: ResMut<Foxes>,
295) {
296 if keyboard_input.just_pressed(KeyCode::Space) {
297 foxes.moving = !foxes.moving;
298 }
299
300 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301 foxes.speed *= 1.25;
302 }
303
304 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305 foxes.speed *= 0.8;
306 }
307
308 if keyboard_input.just_pressed(KeyCode::Enter) {
309 *current_animation = (*current_animation + 1) % animations.node_indices.len();
310 }
311
312 for (mut player, mut transitions) in &mut animation_player {
313 if keyboard_input.just_pressed(KeyCode::Space) {
314 if player.all_paused() {
315 player.resume_all();
316 } else {
317 player.pause_all();
318 }
319 }
320
321 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322 player.adjust_speeds(1.25);
323 }
324
325 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326 player.adjust_speeds(0.8);
327 }
328
329 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330 player.seek_all_by(-0.1);
331 }
332
333 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334 player.seek_all_by(0.1);
335 }
336
337 if keyboard_input.just_pressed(KeyCode::Enter) {
338 transitions
339 .play(
340 &mut player,
341 animations.node_indices[*current_animation],
342 Duration::from_millis(250),
343 )
344 .repeat();
345 }
346 }
347}Sourcepub fn resume_all(&mut self) -> &mut AnimationPlayer
pub fn resume_all(&mut self) -> &mut AnimationPlayer
Resume all active animations.
Examples found in repository?
289fn keyboard_animation_control(
290 keyboard_input: Res<ButtonInput<KeyCode>>,
291 mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292 animations: Res<Animations>,
293 mut current_animation: Local<usize>,
294 mut foxes: ResMut<Foxes>,
295) {
296 if keyboard_input.just_pressed(KeyCode::Space) {
297 foxes.moving = !foxes.moving;
298 }
299
300 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301 foxes.speed *= 1.25;
302 }
303
304 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305 foxes.speed *= 0.8;
306 }
307
308 if keyboard_input.just_pressed(KeyCode::Enter) {
309 *current_animation = (*current_animation + 1) % animations.node_indices.len();
310 }
311
312 for (mut player, mut transitions) in &mut animation_player {
313 if keyboard_input.just_pressed(KeyCode::Space) {
314 if player.all_paused() {
315 player.resume_all();
316 } else {
317 player.pause_all();
318 }
319 }
320
321 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322 player.adjust_speeds(1.25);
323 }
324
325 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326 player.adjust_speeds(0.8);
327 }
328
329 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330 player.seek_all_by(-0.1);
331 }
332
333 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334 player.seek_all_by(0.1);
335 }
336
337 if keyboard_input.just_pressed(KeyCode::Enter) {
338 transitions
339 .play(
340 &mut player,
341 animations.node_indices[*current_animation],
342 Duration::from_millis(250),
343 )
344 .repeat();
345 }
346 }
347}Sourcepub fn rewind_all(&mut self) -> &mut AnimationPlayer
pub fn rewind_all(&mut self) -> &mut AnimationPlayer
Rewinds all active animations.
Sourcepub fn adjust_speeds(&mut self, factor: f32) -> &mut AnimationPlayer
pub fn adjust_speeds(&mut self, factor: f32) -> &mut AnimationPlayer
Multiplies the speed of all active animations by the given factor.
Examples found in repository?
289fn keyboard_animation_control(
290 keyboard_input: Res<ButtonInput<KeyCode>>,
291 mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292 animations: Res<Animations>,
293 mut current_animation: Local<usize>,
294 mut foxes: ResMut<Foxes>,
295) {
296 if keyboard_input.just_pressed(KeyCode::Space) {
297 foxes.moving = !foxes.moving;
298 }
299
300 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301 foxes.speed *= 1.25;
302 }
303
304 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305 foxes.speed *= 0.8;
306 }
307
308 if keyboard_input.just_pressed(KeyCode::Enter) {
309 *current_animation = (*current_animation + 1) % animations.node_indices.len();
310 }
311
312 for (mut player, mut transitions) in &mut animation_player {
313 if keyboard_input.just_pressed(KeyCode::Space) {
314 if player.all_paused() {
315 player.resume_all();
316 } else {
317 player.pause_all();
318 }
319 }
320
321 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322 player.adjust_speeds(1.25);
323 }
324
325 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326 player.adjust_speeds(0.8);
327 }
328
329 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330 player.seek_all_by(-0.1);
331 }
332
333 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334 player.seek_all_by(0.1);
335 }
336
337 if keyboard_input.just_pressed(KeyCode::Enter) {
338 transitions
339 .play(
340 &mut player,
341 animations.node_indices[*current_animation],
342 Duration::from_millis(250),
343 )
344 .repeat();
345 }
346 }
347}Sourcepub fn seek_all_by(&mut self, amount: f32) -> &mut AnimationPlayer
pub fn seek_all_by(&mut self, amount: f32) -> &mut AnimationPlayer
Seeks all active animations forward or backward by the same amount.
To seek forward, pass a positive value; to seek negative, pass a negative value. Values below 0.0 or beyond the end of the animation clip are clamped appropriately.
Examples found in repository?
289fn keyboard_animation_control(
290 keyboard_input: Res<ButtonInput<KeyCode>>,
291 mut animation_player: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
292 animations: Res<Animations>,
293 mut current_animation: Local<usize>,
294 mut foxes: ResMut<Foxes>,
295) {
296 if keyboard_input.just_pressed(KeyCode::Space) {
297 foxes.moving = !foxes.moving;
298 }
299
300 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
301 foxes.speed *= 1.25;
302 }
303
304 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
305 foxes.speed *= 0.8;
306 }
307
308 if keyboard_input.just_pressed(KeyCode::Enter) {
309 *current_animation = (*current_animation + 1) % animations.node_indices.len();
310 }
311
312 for (mut player, mut transitions) in &mut animation_player {
313 if keyboard_input.just_pressed(KeyCode::Space) {
314 if player.all_paused() {
315 player.resume_all();
316 } else {
317 player.pause_all();
318 }
319 }
320
321 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
322 player.adjust_speeds(1.25);
323 }
324
325 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
326 player.adjust_speeds(0.8);
327 }
328
329 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
330 player.seek_all_by(-0.1);
331 }
332
333 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
334 player.seek_all_by(0.1);
335 }
336
337 if keyboard_input.just_pressed(KeyCode::Enter) {
338 transitions
339 .play(
340 &mut player,
341 animations.node_indices[*current_animation],
342 Duration::from_millis(250),
343 )
344 .repeat();
345 }
346 }
347}Sourcepub fn animation(&self, animation: NodeIndex) -> Option<&ActiveAnimation>
pub fn animation(&self, animation: NodeIndex) -> Option<&ActiveAnimation>
Returns the ActiveAnimation associated with the given animation
node if it’s currently playing.
If the animation isn’t currently active, returns None.
Sourcepub fn animation_mut(
&mut self,
animation: NodeIndex,
) -> Option<&mut ActiveAnimation>
pub fn animation_mut( &mut self, animation: NodeIndex, ) -> Option<&mut ActiveAnimation>
Returns a mutable reference to the ActiveAnimation associated with
the given animation node if it’s currently active.
If the animation isn’t currently active, returns None.
Examples found in repository?
453fn sync_weights(mut query: Query<(&mut AnimationPlayer, &ExampleAnimationWeights)>) {
454 for (mut animation_player, animation_weights) in query.iter_mut() {
455 for (&animation_node_index, &animation_weight) in CLIP_NODE_INDICES
456 .iter()
457 .zip(animation_weights.weights.iter())
458 {
459 // If the animation happens to be no longer active, restart it.
460 if !animation_player.is_playing_animation(animation_node_index.into()) {
461 animation_player.play(animation_node_index.into());
462 }
463
464 // Set the weight.
465 if let Some(active_animation) =
466 animation_player.animation_mut(animation_node_index.into())
467 {
468 active_animation.set_weight(animation_weight);
469 }
470 }
471 }
472}More examples
165fn keyboard_control(
166 keyboard_input: Res<ButtonInput<KeyCode>>,
167 mut animation_players: Query<(&mut AnimationPlayer, &mut AnimationTransitions)>,
168 animations: Res<Animations>,
169 mut current_animation: Local<usize>,
170) {
171 for (mut player, mut transitions) in &mut animation_players {
172 let Some((&playing_animation_index, _)) = player.playing_animations().next() else {
173 continue;
174 };
175
176 if keyboard_input.just_pressed(KeyCode::Space) {
177 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
178 if playing_animation.is_paused() {
179 playing_animation.resume();
180 } else {
181 playing_animation.pause();
182 }
183 }
184
185 if keyboard_input.just_pressed(KeyCode::ArrowUp) {
186 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
187 let speed = playing_animation.speed();
188 playing_animation.set_speed(speed * 1.2);
189 }
190
191 if keyboard_input.just_pressed(KeyCode::ArrowDown) {
192 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
193 let speed = playing_animation.speed();
194 playing_animation.set_speed(speed * 0.8);
195 }
196
197 if keyboard_input.just_pressed(KeyCode::ArrowLeft) {
198 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
199 let elapsed = playing_animation.seek_time();
200 playing_animation.seek_to(elapsed - 0.1);
201 }
202
203 if keyboard_input.just_pressed(KeyCode::ArrowRight) {
204 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
205 let elapsed = playing_animation.seek_time();
206 playing_animation.seek_to(elapsed + 0.1);
207 }
208
209 if keyboard_input.just_pressed(KeyCode::Enter) {
210 *current_animation = (*current_animation + 1) % animations.animations.len();
211
212 transitions
213 .play(
214 &mut player,
215 animations.animations[*current_animation],
216 Duration::from_millis(250),
217 )
218 .repeat();
219 }
220
221 if keyboard_input.just_pressed(KeyCode::Digit1) {
222 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
223 playing_animation
224 .set_repeat(RepeatAnimation::Count(1))
225 .replay();
226 }
227
228 if keyboard_input.just_pressed(KeyCode::Digit2) {
229 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
230 playing_animation
231 .set_repeat(RepeatAnimation::Count(2))
232 .replay();
233 }
234
235 if keyboard_input.just_pressed(KeyCode::Digit3) {
236 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
237 playing_animation
238 .set_repeat(RepeatAnimation::Count(3))
239 .replay();
240 }
241
242 if keyboard_input.just_pressed(KeyCode::KeyL) {
243 let playing_animation = player.animation_mut(playing_animation_index).unwrap();
244 playing_animation.set_repeat(RepeatAnimation::Forever);
245 }
246 }
247}Trait Implementations
Sourceimpl Clone for AnimationPlayer
impl Clone for AnimationPlayer
Sourcefn clone(&self) -> AnimationPlayer
fn clone(&self) -> AnimationPlayer
Sourcefn clone_from(&mut self, source: &AnimationPlayer)
fn clone_from(&mut self, source: &AnimationPlayer)
source. Read moreSourceimpl Component for AnimationPlayer
impl Component for AnimationPlayer
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<AnimationPlayer>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<AnimationPlayer>>
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceimpl Default for AnimationPlayer
impl Default for AnimationPlayer
Sourcefn default() -> AnimationPlayer
fn default() -> AnimationPlayer
Sourceimpl FromArg for AnimationPlayer
impl FromArg for AnimationPlayer
Sourceimpl FromReflect for AnimationPlayer
impl FromReflect for AnimationPlayer
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<AnimationPlayer>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<AnimationPlayer>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for AnimationPlayer
impl GetOwnership for AnimationPlayer
Sourceimpl GetTypeRegistration for AnimationPlayer
impl GetTypeRegistration for AnimationPlayer
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for AnimationPlayer
impl IntoReturn for AnimationPlayer
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
AnimationPlayer: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
AnimationPlayer: 'into_return,
Sourceimpl PartialReflect for AnimationPlayer
impl PartialReflect for AnimationPlayer
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<AnimationPlayer>) -> ReflectOwned
fn reflect_owned(self: Box<AnimationPlayer>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<AnimationPlayer>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<AnimationPlayer>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<AnimationPlayer>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<AnimationPlayer>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for AnimationPlayer
impl Reflect for AnimationPlayer
Sourcefn into_any(self: Box<AnimationPlayer>) -> Box<dyn Any>
fn into_any(self: Box<AnimationPlayer>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<AnimationPlayer>) -> Box<dyn Reflect>
fn into_reflect(self: Box<AnimationPlayer>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl Struct for AnimationPlayer
impl Struct for AnimationPlayer
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn index_of_name(&self, name: &str) -> Option<usize>
fn index_of_name(&self, name: &str) -> Option<usize>
Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.Sourceimpl TypePath for AnimationPlayer
impl TypePath for AnimationPlayer
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>