pub struct SubApp {
pub update_schedule: Option<Interned<dyn ScheduleLabel>>,
/* private fields */
}Expand description
A secondary application with its own World. These can run independently of each other.
These are useful for situations where certain processes (e.g. a render thread) need to be kept separate from the main application.
Example
#[derive(Resource, Default)]
struct Val(pub i32);
#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, AppLabel)]
struct ExampleApp;
// Create an app with a certain resource.
let mut app = App::new();
app.insert_resource(Val(10));
// Create a sub-app with the same resource and a single schedule.
let mut sub_app = SubApp::new();
sub_app.update_schedule = Some(Main.intern());
sub_app.insert_resource(Val(100));
// Setup an extract function to copy the resource's value in the main world.
sub_app.set_extract(|main_world, sub_world| {
sub_world.resource_mut::<Val>().0 = main_world.resource::<Val>().0;
});
// Schedule a system that will verify extraction is working.
sub_app.add_systems(Main, |counter: Res<Val>| {
// The value will be copied during extraction, so we should see 10 instead of 100.
assert_eq!(counter.0, 10);
});
// Add the sub-app to the main app.
app.insert_sub_app(ExampleApp, sub_app);
// Update the application once (using the default runner).
app.run();Fields
update_schedule: Option<Interned<dyn ScheduleLabel>>The schedule that will be run by update.
Implementations
Sourceimpl SubApp
impl SubApp
Sourcepub fn world(&self) -> &World
pub fn world(&self) -> &World
Returns a reference to the World.
Examples found in repository?
105 fn update(&mut self) {
106 self.0.update();
107 // Wait for frame to finish rendering by wait polling the device
108 self.0
109 .main
110 .world()
111 .resource::<RenderDevice>()
112 .wgpu_device()
113 .poll(PollType::Wait {
114 submission_index: None,
115 timeout: None,
116 })
117 .unwrap();
118 }More examples
91fn count_ambiguities(sub_app: &mut SubApp) -> AmbiguitiesCount {
92 let schedule_labels = sub_app
93 .world()
94 .resource::<Schedules>()
95 .iter()
96 .map(|(_, schedule)| schedule.label())
97 .collect::<Vec<_>>();
98 let mut ambiguities = <HashMap<_, _>>::default();
99 for label in schedule_labels {
100 let ambiguities_in_schedule =
101 sub_app
102 .world_mut()
103 .schedule_scope(label, |world, schedule| {
104 schedule.initialize(world).unwrap().unwrap();
105 schedule.graph().conflicting_systems().len()
106 });
107 ambiguities.insert(label, ambiguities_in_schedule);
108 }
109 AmbiguitiesCount(ambiguities)
110}Sourcepub fn world_mut(&mut self) -> &mut World
pub fn world_mut(&mut self) -> &mut World
Returns a mutable reference to the World.
Examples found in repository?
62 fn finish(&self, app: &mut App) {
63 let Some(render_app) = app.get_sub_app_mut(RenderApp) else {
64 return;
65 };
66 render_app
67 .world_mut()
68 .resource_mut::<MeshAllocatorSettings>()
69 // This allows using the mesh allocator slabs as
70 // storage buffers directly in the compute shader.
71 // Which means that we can write from our compute
72 // shader directly to the allocated mesh slabs.
73 .extra_buffer_usages = BufferUsages::STORAGE;
74 }More examples
75 fn new_render_target(&mut self, width: u32, height: u32) -> RenderTarget {
76 let mut target = Image::new_uninit(
77 Extent3d {
78 width,
79 height,
80 depth_or_array_layers: 1,
81 },
82 TextureDimension::D2,
83 TextureFormat::Rgba8UnormSrgb,
84 RenderAssetUsages::RENDER_WORLD,
85 );
86 // We're going to render to this image, mark it as such
87 target.texture_descriptor.usage |= TextureUsages::RENDER_ATTACHMENT;
88 self.0
89 .main
90 .world_mut()
91 .resource_mut::<Assets<Image>>()
92 .add(target)
93 .into()
94 }
95
96 fn spawn_camera(&mut self, target: RenderTarget) -> Entity {
97 self.0
98 .main
99 .world_mut()
100 .spawn((Camera3d::default(), target, Transform::IDENTITY))
101 .id()
102 }
103
104 // Run one world update and wait for rendering to finish.
105 fn update(&mut self) {
106 self.0.update();
107 // Wait for frame to finish rendering by wait polling the device
108 self.0
109 .main
110 .world()
111 .resource::<RenderDevice>()
112 .wgpu_device()
113 .poll(PollType::Wait {
114 submission_index: None,
115 timeout: None,
116 })
117 .unwrap();
118 }
119
120 // Schedules a screenshot to be captured on the next update.
121 fn screenshot(&mut self, target: RenderTarget, i: u32) {
122 self.0
123 .main
124 .world_mut()
125 .spawn(Screenshot::image(target.as_image().unwrap().clone()))
126 .observe(save_to_disk(format!("test_images/screenshot{i}.png")));
127 }70fn configure_ambiguity_detection(sub_app: &mut SubApp) {
71 let mut schedules = sub_app.world_mut().resource_mut::<Schedules>();
72 for (_, schedule) in schedules.iter_mut() {
73 schedule.set_build_settings(ScheduleBuildSettings {
74 // NOTE: you can change this to `LogLevel::Ignore` to easily see the current number of ambiguities.
75 ambiguity_detection: LogLevel::Warn,
76 // With auto-inserted apply_deferred stages, these can cause two ambiguous systems to
77 // become accidentally ordered by one of the apply_deferred stages. Disabling requires
78 // us to meet a higher bar. We don't just want no ambiguities - we also don't want
79 // changes to systems or the auto-insert code from "creating" new ambiguities (by
80 // reordering the graph). However, the cost is that the graph is no longer runnable,
81 // since Bevy crates often rely on auto-insert apply_deferred to not panic (e.g.,
82 // because a resource wasn't inserted).
83 auto_insert_apply_deferred: false,
84 use_shortnames: false,
85 ..default()
86 });
87 }
88}
89
90/// Returns the number of conflicting systems per schedule.
91fn count_ambiguities(sub_app: &mut SubApp) -> AmbiguitiesCount {
92 let schedule_labels = sub_app
93 .world()
94 .resource::<Schedules>()
95 .iter()
96 .map(|(_, schedule)| schedule.label())
97 .collect::<Vec<_>>();
98 let mut ambiguities = <HashMap<_, _>>::default();
99 for label in schedule_labels {
100 let ambiguities_in_schedule =
101 sub_app
102 .world_mut()
103 .schedule_scope(label, |world, schedule| {
104 schedule.initialize(world).unwrap().unwrap();
105 schedule.graph().conflicting_systems().len()
106 });
107 ambiguities.insert(label, ambiguities_in_schedule);
108 }
109 AmbiguitiesCount(ambiguities)
110}Sourcepub fn run_default_schedule(&mut self)
pub fn run_default_schedule(&mut self)
Runs the default schedule.
Does not clear internal trackers used for change detection.
Sourcepub fn extract(&mut self, world: &mut World)
pub fn extract(&mut self, world: &mut World)
Extracts data from world into the app’s world using the registered extract method.
Note: There is no default extract method. Calling extract does nothing if
set_extract has not been called.
Sourcepub fn set_extract<F>(&mut self, extract: F) -> &mut SubApp
pub fn set_extract<F>(&mut self, extract: F) -> &mut SubApp
Sets the method that will be called by extract.
The first argument is the World to extract data from, the second argument is the app World.
Sourcepub fn take_extract(
&mut self,
) -> Option<Box<dyn FnMut(&mut World, &mut World) + Send>>
pub fn take_extract( &mut self, ) -> Option<Box<dyn FnMut(&mut World, &mut World) + Send>>
Take the function that will be called by extract out of the app, if any was set,
and replace it with None.
If you use Bevy, bevy_render will set a default extract function used to extract data from
the main world into the render world as part of the Extract phase. In that case, you cannot replace
it with your own function. Instead, take the Bevy default function with this, and install your own
instead which calls the Bevy default.
let mut default_fn = app.take_extract();
app.set_extract(move |main, render| {
// Do pre-extract custom logic
// [...]
// Call Bevy's default, which executes the Extract phase
if let Some(f) = default_fn.as_mut() {
f(main, render);
}
// Do post-extract custom logic
// [...]
});Sourcepub fn insert_resource<R>(&mut self, resource: R) -> &mut SubAppwhere
R: Resource,
pub fn insert_resource<R>(&mut self, resource: R) -> &mut SubAppwhere
R: Resource,
See App::insert_resource.
Examples found in repository?
205 fn build(&self, app: &mut App) {
206 let (s, r) = crossbeam_channel::unbounded();
207
208 let render_app = app
209 .insert_resource(MainWorldReceiver(r))
210 .sub_app_mut(RenderApp);
211
212 render_app
213 .insert_resource(RenderWorldSender(s))
214 // Make ImageCopiers accessible in RenderWorld system and plugin
215 .add_systems(ExtractSchedule, image_copy_extract)
216 // Receives image data from buffer to channel
217 // so we need to run it after the render graph is done
218 .add_systems(
219 Render,
220 receive_image_from_buffer.after(RenderSystems::Render),
221 )
222 .add_systems(RenderGraph, image_copy_driver);
223 }More examples
46fn main() {
47 let mut app = App::new();
48
49 // Main World
50 app.insert_resource(WorldName("Main World".into()))
51 .add_plugins((
52 DefaultPlugins,
53 // Plugin for automatically extracting A.
54 ExtractComponentPlugin::<A>::default(),
55 ))
56 .add_message::<ExtractMessage>()
57 .add_systems(Startup, setup)
58 .add_systems(Update, (set_time, trigger_extraction, display_state));
59
60 let Some(render_app) = app.get_sub_app_mut(RenderApp) else {
61 return;
62 };
63
64 // Render World
65 render_app
66 .insert_resource(WorldName("Render World".into()))
67 .add_systems(ExtractSchedule, extract_components)
68 .add_systems(Render, display_state);
69
70 app.run();
71}300 fn build(&self, app: &mut App) {
301 // Load our custom shader
302 let mut shaders = app.world_mut().resource_mut::<Assets<Shader>>();
303 // Here, we construct and add the shader asset manually. There are many ways to load this
304 // shader, including `embedded_asset`/`load_embedded_asset`.
305 let shader = shaders.add(Shader::from_wgsl(COLORED_MESH2D_SHADER, file!()));
306
307 app.add_plugins(SyncComponentPlugin::<ColoredMesh2d>::default());
308
309 // Register our custom draw function, and add our render systems
310 app.get_sub_app_mut(RenderApp)
311 .unwrap()
312 .insert_resource(ColoredMesh2dShader(shader))
313 .add_render_command::<Transparent2d, DrawColoredMesh2d>()
314 .init_resource::<SpecializedRenderPipelines<ColoredMesh2dPipeline>>()
315 .init_resource::<RenderColoredMesh2dInstances>()
316 .add_systems(
317 RenderStartup,
318 init_colored_mesh_2d_pipeline.after(init_mesh_2d_pipeline),
319 )
320 .add_systems(
321 ExtractSchedule,
322 extract_colored_mesh2d.after(extract_mesh2d),
323 )
324 .add_systems(
325 Render,
326 queue_colored_mesh2d.in_set(RenderSystems::QueueMeshes),
327 );
328 }197 fn build(&self, app: &mut App) {
198 // Create the `SavedIndirectParameters` resource that we're going to use
199 // to communicate between the thread that the GPU-to-CPU readback
200 // callback runs on and the main application threads. This resource is
201 // atomically reference counted. We store one reference to the
202 // `SavedIndirectParameters` in the main app and another reference in
203 // the render app.
204 let saved_indirect_parameters = SavedIndirectParameters::new();
205 app.insert_resource(saved_indirect_parameters.clone());
206
207 // Fetch the render app.
208 let Some(render_app) = app.get_sub_app_mut(RenderApp) else {
209 return;
210 };
211
212 render_app
213 // Insert another reference to the `SavedIndirectParameters`.
214 .insert_resource(saved_indirect_parameters)
215 // Setup the parameters in RenderStartup.
216 .add_systems(RenderStartup, init_saved_indirect_parameters)
217 .init_resource::<IndirectParametersStagingBuffers>()
218 .add_systems(ExtractSchedule, readback_indirect_parameters)
219 .add_systems(
220 Render,
221 create_indirect_parameters_staging_buffers
222 .in_set(RenderSystems::PrepareResourcesFlush),
223 )
224 .add_systems(
225 Core3d,
226 // Add the node that allows us to read the indirect parameters back
227 // from the GPU to the CPU, which allows us to determine how many
228 // meshes were culled.
229 readback_indirect_parameters_node
230 // We read back the indirect parameters any time after
231 // `MainPass`. Readback doesn't particularly need to execute
232 // before PostProcess, but we order it that way anyway.
233 .after(Core3dSystems::MainPass)
234 .before(Core3dSystems::PostProcess),
235 );
236 }Sourcepub fn init_resource<R>(&mut self) -> &mut SubApp
pub fn init_resource<R>(&mut self) -> &mut SubApp
See App::init_resource.
Examples found in repository?
More examples
110 fn build(&self, app: &mut App) {
111 app.add_plugins(ExtractComponentPlugin::<InstanceMaterialData>::default());
112 app.sub_app_mut(RenderApp)
113 .add_render_command::<Transparent3d, DrawCustom>()
114 .init_resource::<SpecializedMeshPipelines<CustomPipeline>>()
115 .add_systems(
116 RenderStartup,
117 init_custom_pipeline.after(MeshPipelineSystems),
118 )
119 .add_systems(
120 Render,
121 (
122 queue_custom.in_set(RenderSystems::QueueMeshes),
123 prepare_instance_buffers.in_set(RenderSystems::PrepareResources),
124 ),
125 );
126 }164fn main() {
165 let mut app = App::new();
166 app.add_plugins(DefaultPlugins)
167 .add_plugins(ExtractComponentPlugin::<CustomRenderedEntity>::default())
168 .add_systems(Startup, setup);
169
170 // We make sure to add these to the render app, not the main app.
171 app.sub_app_mut(RenderApp)
172 .init_resource::<CustomPhasePipeline>()
173 .init_resource::<PendingCustomPhaseItemQueues>()
174 .add_render_command::<Opaque3d, DrawCustomPhaseItemCommands>()
175 .add_systems(
176 Render,
177 prepare_custom_phase_item_buffers.in_set(RenderSystems::Prepare),
178 )
179 .add_systems(Render, queue_custom_phase_item.in_set(RenderSystems::Queue));
180
181 app.run();
182}94 fn build(&self, app: &mut App) {
95 // Extract the game of life image resource from the main world into the render world
96 // for operation on by the compute shader and display on the sprite.
97 app.add_plugins((
98 ExtractResourcePlugin::<GameOfLifeImages>::default(),
99 ExtractResourcePlugin::<GameOfLifeUniforms>::default(),
100 ));
101 let render_app = app.sub_app_mut(RenderApp);
102 render_app
103 .init_resource::<GameOfLifeState>()
104 .add_systems(RenderStartup, init_game_of_life_pipeline)
105 .add_systems(
106 Render,
107 prepare_bind_group.in_set(RenderSystems::PrepareBindGroups),
108 )
109 .add_systems(Render, update.in_set(RenderSystems::Prepare))
110 .add_systems(RenderGraph, game_of_life.before(camera_driver));
111 }106 fn build(&self, app: &mut App) {
107 app.add_plugins(ExtractComponentPlugin::<CustomRenderedEntity>::default());
108
109 // We make sure to add these to the render app, not the main app.
110 let Some(render_app) = app.get_sub_app_mut(RenderApp) else {
111 return;
112 };
113 render_app
114 // This is needed to tell bevy about your custom pipeline
115 .init_resource::<SpecializedMeshPipelines<CustomMeshPipeline>>()
116 .init_resource::<PendingCustomMeshQueues>()
117 // We need to use a custom draw command so we need to register it
118 .add_render_command::<Opaque3d, DrawSpecializedPipelineCommands>()
119 .add_systems(
120 RenderStartup,
121 init_custom_mesh_pipeline.after(MeshPipelineSystems),
122 )
123 .add_systems(
124 Render,
125 queue_custom_mesh_pipeline.in_set(RenderSystems::Queue),
126 );
127 }300 fn build(&self, app: &mut App) {
301 // Load our custom shader
302 let mut shaders = app.world_mut().resource_mut::<Assets<Shader>>();
303 // Here, we construct and add the shader asset manually. There are many ways to load this
304 // shader, including `embedded_asset`/`load_embedded_asset`.
305 let shader = shaders.add(Shader::from_wgsl(COLORED_MESH2D_SHADER, file!()));
306
307 app.add_plugins(SyncComponentPlugin::<ColoredMesh2d>::default());
308
309 // Register our custom draw function, and add our render systems
310 app.get_sub_app_mut(RenderApp)
311 .unwrap()
312 .insert_resource(ColoredMesh2dShader(shader))
313 .add_render_command::<Transparent2d, DrawColoredMesh2d>()
314 .init_resource::<SpecializedRenderPipelines<ColoredMesh2dPipeline>>()
315 .init_resource::<RenderColoredMesh2dInstances>()
316 .add_systems(
317 RenderStartup,
318 init_colored_mesh_2d_pipeline.after(init_mesh_2d_pipeline),
319 )
320 .add_systems(
321 ExtractSchedule,
322 extract_colored_mesh2d.after(extract_mesh2d),
323 )
324 .add_systems(
325 Render,
326 queue_colored_mesh2d.in_set(RenderSystems::QueueMeshes),
327 );
328 }Sourcepub fn add_systems<M>(
&mut self,
schedule: impl ScheduleLabel,
systems: impl IntoScheduleConfigs<Box<dyn System<Out = (), In = ()>>, M>,
) -> &mut SubApp
pub fn add_systems<M>( &mut self, schedule: impl ScheduleLabel, systems: impl IntoScheduleConfigs<Box<dyn System<Out = (), In = ()>>, M>, ) -> &mut SubApp
See App::add_systems.
Examples found in repository?
More examples
20fn main() {
21 let mut app = App::new();
22 app.add_plugins((
23 DefaultPlugins,
24 ExtractResourcePlugin::<RenderError>::default(),
25 ))
26 .add_systems(Startup, setup)
27 .add_systems(Update, (update_camera, input))
28 .init_resource::<RenderError>()
29 .sub_app_mut(RenderApp)
30 .add_systems(Render, cause_error);
31 app.run();
32}44 fn build(&self, app: &mut App) {
45 let Some(render_app) = app.get_sub_app_mut(RenderApp) else {
46 return;
47 };
48 render_app
49 .add_systems(RenderStartup, init_compute_pipeline)
50 .add_systems(
51 Render,
52 prepare_bind_group
53 .in_set(RenderSystems::PrepareBindGroups)
54 // We don't need to recreate the bind group every frame
55 .run_if(not(resource_exists::<GpuBufferBindGroup>)),
56 )
57 .add_systems(RenderGraph, compute);
58 }- examples/shader_advanced/custom_shader_instancing.rs
- examples/shader_advanced/custom_phase_item.rs
- examples/app/headless_renderer.rs
- examples/ecs/extraction.rs
- examples/shader_advanced/render_depth_to_texture.rs
- examples/shader/compute_shader_game_of_life.rs
- examples/shader_advanced/specialized_mesh_pipeline.rs
- examples/shader_advanced/manual_material.rs
- examples/shader_advanced/custom_post_processing.rs
- examples/2d/mesh2d_manual.rs
- examples/shader_advanced/custom_render_phase.rs
- examples/2d/dynamic_mip_generation.rs
- examples/3d/occlusion_culling.rs
Sourcepub fn remove_systems_in_set<M>(
&mut self,
schedule: impl ScheduleLabel,
set: impl IntoSystemSet<M>,
policy: ScheduleCleanupPolicy,
) -> Result<usize, ScheduleError>
pub fn remove_systems_in_set<M>( &mut self, schedule: impl ScheduleLabel, set: impl IntoSystemSet<M>, policy: ScheduleCleanupPolicy, ) -> Result<usize, ScheduleError>
Sourcepub fn register_system<I, O, M>(
&mut self,
system: impl IntoSystem<I, O, M> + 'static,
) -> SystemId<I, O>where
I: SystemInput + 'static,
O: 'static,
pub fn register_system<I, O, M>(
&mut self,
system: impl IntoSystem<I, O, M> + 'static,
) -> SystemId<I, O>where
I: SystemInput + 'static,
O: 'static,
See App::register_system.
Sourcepub fn register_tracked_system<I, O, M>(
&mut self,
system: impl IntoSystem<I, O, M> + 'static,
) -> SystemHandle<I, O>where
I: SystemInput + 'static,
O: 'static,
pub fn register_tracked_system<I, O, M>(
&mut self,
system: impl IntoSystem<I, O, M> + 'static,
) -> SystemHandle<I, O>where
I: SystemInput + 'static,
O: 'static,
Sourcepub fn configure_sets<M>(
&mut self,
schedule: impl ScheduleLabel,
sets: impl IntoScheduleConfigs<Interned<dyn SystemSet>, M>,
) -> &mut SubApp
pub fn configure_sets<M>( &mut self, schedule: impl ScheduleLabel, sets: impl IntoScheduleConfigs<Interned<dyn SystemSet>, M>, ) -> &mut SubApp
See App::configure_sets.
Sourcepub fn add_schedule(&mut self, schedule: Schedule) -> &mut SubApp
pub fn add_schedule(&mut self, schedule: Schedule) -> &mut SubApp
See App::add_schedule.
Sourcepub fn init_schedule(&mut self, label: impl ScheduleLabel) -> &mut SubApp
pub fn init_schedule(&mut self, label: impl ScheduleLabel) -> &mut SubApp
See App::init_schedule.
Sourcepub fn get_schedule(&self, label: impl ScheduleLabel) -> Option<&Schedule>
pub fn get_schedule(&self, label: impl ScheduleLabel) -> Option<&Schedule>
See App::get_schedule.
Sourcepub fn get_schedule_mut(
&mut self,
label: impl ScheduleLabel,
) -> Option<&mut Schedule>
pub fn get_schedule_mut( &mut self, label: impl ScheduleLabel, ) -> Option<&mut Schedule>
Sourcepub fn edit_schedule(
&mut self,
label: impl ScheduleLabel,
f: impl FnMut(&mut Schedule),
) -> &mut SubApp
pub fn edit_schedule( &mut self, label: impl ScheduleLabel, f: impl FnMut(&mut Schedule), ) -> &mut SubApp
See App::edit_schedule.
Sourcepub fn configure_schedules(
&mut self,
schedule_build_settings: ScheduleBuildSettings,
) -> &mut SubApp
pub fn configure_schedules( &mut self, schedule_build_settings: ScheduleBuildSettings, ) -> &mut SubApp
Sourcepub fn allow_ambiguous_component<T>(&mut self) -> &mut SubAppwhere
T: Component,
pub fn allow_ambiguous_component<T>(&mut self) -> &mut SubAppwhere
T: Component,
Sourcepub fn allow_ambiguous_resource<T>(&mut self) -> &mut SubAppwhere
T: Resource,
pub fn allow_ambiguous_resource<T>(&mut self) -> &mut SubAppwhere
T: Resource,
Sourcepub fn ignore_ambiguity<M1, M2, S1, S2>(
&mut self,
schedule: impl ScheduleLabel,
a: S1,
b: S2,
) -> &mut SubAppwhere
S1: IntoSystemSet<M1>,
S2: IntoSystemSet<M2>,
pub fn ignore_ambiguity<M1, M2, S1, S2>(
&mut self,
schedule: impl ScheduleLabel,
a: S1,
b: S2,
) -> &mut SubAppwhere
S1: IntoSystemSet<M1>,
S2: IntoSystemSet<M2>,
Sourcepub fn add_observer<M>(&mut self, observer: impl IntoObserver<M>) -> &mut SubApp
pub fn add_observer<M>(&mut self, observer: impl IntoObserver<M>) -> &mut SubApp
See App::add_observer.
Sourcepub fn add_message<T>(&mut self) -> &mut SubAppwhere
T: Message,
pub fn add_message<T>(&mut self) -> &mut SubAppwhere
T: Message,
See App::add_message.
Sourcepub fn add_plugins<M>(&mut self, plugins: impl Plugins<M>) -> &mut SubApp
pub fn add_plugins<M>(&mut self, plugins: impl Plugins<M>) -> &mut SubApp
See App::add_plugins.
Sourcepub fn is_plugin_added<T>(&self) -> boolwhere
T: Plugin,
pub fn is_plugin_added<T>(&self) -> boolwhere
T: Plugin,
See App::is_plugin_added.
Sourcepub fn get_added_plugins<T>(&self) -> Vec<&T>where
T: Plugin,
pub fn get_added_plugins<T>(&self) -> Vec<&T>where
T: Plugin,
Sourcepub fn plugins_state(&mut self) -> PluginsState
pub fn plugins_state(&mut self) -> PluginsState
Return the state of plugins.
Sourcepub fn finish(&mut self)
pub fn finish(&mut self)
Runs Plugin::finish for each plugin.
Sourcepub fn cleanup(&mut self)
pub fn cleanup(&mut self)
Runs Plugin::cleanup for each plugin.
Sourcepub fn register_type<T>(&mut self) -> &mut SubAppwhere
T: GetTypeRegistration,
Available on crate feature bevy_reflect only.
pub fn register_type<T>(&mut self) -> &mut SubAppwhere
T: GetTypeRegistration,
bevy_reflect only.See App::register_type.
Sourcepub fn register_type_data<T, D>(&mut self) -> &mut SubApp
Available on crate feature bevy_reflect only.
pub fn register_type_data<T, D>(&mut self) -> &mut SubApp
bevy_reflect only.Sourcepub fn register_type_conversion<T, U, F>(&mut self, function: F) -> &mut SubApp
Available on crate feature bevy_reflect only.
pub fn register_type_conversion<T, U, F>(&mut self, function: F) -> &mut SubApp
bevy_reflect only.Sourcepub fn register_into_type_conversion<T, U>(&mut self) -> &mut SubApp
Available on crate feature bevy_reflect only.
pub fn register_into_type_conversion<T, U>(&mut self) -> &mut SubApp
bevy_reflect only.Sourcepub fn register_function<F, Marker>(&mut self, function: F) -> &mut SubAppwhere
F: IntoFunction<'static, Marker> + 'static,
Available on crate feature reflect_functions only.
pub fn register_function<F, Marker>(&mut self, function: F) -> &mut SubAppwhere
F: IntoFunction<'static, Marker> + 'static,
reflect_functions only.Sourcepub fn register_function_with_name<F, Marker>(
&mut self,
name: impl Into<Cow<'static, str>>,
function: F,
) -> &mut SubAppwhere
F: IntoFunction<'static, Marker> + 'static,
Available on crate feature reflect_functions only.
pub fn register_function_with_name<F, Marker>(
&mut self,
name: impl Into<Cow<'static, str>>,
function: F,
) -> &mut SubAppwhere
F: IntoFunction<'static, Marker> + 'static,
reflect_functions only.Trait Implementations
Sourceimpl AddRenderCommand for SubApp
impl AddRenderCommand for SubApp
Sourcefn add_render_command<P, C>(&mut self) -> &mut SubAppwhere
P: PhaseItem,
C: RenderCommand<P> + Send + Sync + 'static,
<C as RenderCommand<P>>::Param: ReadOnlySystemParam,
fn add_render_command<P, C>(&mut self) -> &mut SubAppwhere
P: PhaseItem,
C: RenderCommand<P> + Send + Sync + 'static,
<C as RenderCommand<P>>::Param: ReadOnlySystemParam,
RenderCommand for the specified render phase to the app.Sourceimpl AppExtStates for SubApp
impl AppExtStates for SubApp
Sourcefn init_state<S>(&mut self) -> &mut SubAppwhere
S: FreelyMutableState + FromWorld,
fn init_state<S>(&mut self) -> &mut SubAppwhere
S: FreelyMutableState + FromWorld,
Sourcefn insert_state<S>(&mut self, state: S) -> &mut SubAppwhere
S: FreelyMutableState,
fn insert_state<S>(&mut self, state: S) -> &mut SubAppwhere
S: FreelyMutableState,
Sourcefn add_computed_state<S>(&mut self) -> &mut SubAppwhere
S: ComputedStates,
fn add_computed_state<S>(&mut self) -> &mut SubAppwhere
S: ComputedStates,
ComputedStates. Read moreSourcefn add_sub_state<S>(&mut self) -> &mut SubAppwhere
S: SubStates,
fn add_sub_state<S>(&mut self) -> &mut SubAppwhere
S: SubStates,
Sourcefn register_type_state<S>(&mut self) -> &mut SubApp
fn register_type_state<S>(&mut self) -> &mut SubApp
bevy_reflect only.T using App::register_type,
and adds ReflectState type data to T in the type registry. Read moreSourcefn register_type_mutable_state<S>(&mut self) -> &mut SubApp
fn register_type_mutable_state<S>(&mut self) -> &mut SubApp
bevy_reflect only.T using App::register_type,
and adds crate::reflect::ReflectState and crate::reflect::ReflectFreelyMutableState type data to T in the type registry. Read moreSourceimpl GpuResourceAppExt for SubApp
impl GpuResourceAppExt for SubApp
Sourcefn init_gpu_resource<R>(&mut self) -> &mut SubApp
fn init_gpu_resource<R>(&mut self) -> &mut SubApp
RenderStartup. Read moreSourceimpl RegisterDiagnostic for SubApp
impl RegisterDiagnostic for SubApp
Sourcefn register_diagnostic(&mut self, diagnostic: Diagnostic) -> &mut SubApp
fn register_diagnostic(&mut self, diagnostic: Diagnostic) -> &mut SubApp
Sourceimpl StateScopedMessagesAppExt for SubApp
impl StateScopedMessagesAppExt for SubApp