PluginGroupBuilder

Struct PluginGroupBuilder 

pub struct PluginGroupBuilder { /* private fields */ }
Facilitates the creation and configuration of a PluginGroup.

Provides a build ordering to ensure that Plugins which produce/require a Resource are built before/after dependent/depending Plugins. Plugins inside the group can be disabled, enabled or reordered.

Implementations

Source

impl PluginGroupBuilder

pub fn start<PG>() -> PluginGroupBuilder
where PG: PluginGroup,

Start a new builder for the PluginGroup.

Examples found in repository?
examples/app/plugin_group.rs (line 31)
30    fn build(self) -> PluginGroupBuilder {
31        PluginGroupBuilder::start::<Self>()
32            .add(PrintHelloPlugin)
33            .add(PrintWorldPlugin)
34    }
pub fn contains<T>(&self) -> bool
where T: Plugin,

Checks if the PluginGroupBuilder contains the given Plugin.

pub fn enabled<T>(&self) -> bool
where T: Plugin,

Returns true if the PluginGroupBuilder contains the given Plugin and it’s enabled.

pub fn set<T>(self, plugin: T) -> PluginGroupBuilder
where T: Plugin,

Sets the value of the given Plugin, if it exists.

Panics

Panics if the Plugin does not exist.

pub fn try_set<T>( self, plugin: T, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>
where T: Plugin,

Tries to set the value of the given Plugin, if it exists.

If the given plugin doesn’t exist returns self and the passed in Plugin.

pub fn add<T>(self, plugin: T) -> PluginGroupBuilder
where T: Plugin,

Adds the plugin Plugin at the end of this PluginGroupBuilder. If the plugin was already in the group, it is removed from its previous place.

Examples found in repository?
examples/app/plugin_group.rs (line 32)
30    fn build(self) -> PluginGroupBuilder {
31        PluginGroupBuilder::start::<Self>()
32            .add(PrintHelloPlugin)
33            .add(PrintWorldPlugin)
34    }
pub fn try_add<T>( self, plugin: T, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>
where T: Plugin,

Attempts to add the plugin Plugin at the end of this PluginGroupBuilder.

If the plugin was already in the group the addition fails.

pub fn add_group(self, group: impl PluginGroup) -> PluginGroupBuilder

Adds a PluginGroup at the end of this PluginGroupBuilder. If the plugin was already in the group, it is removed from its previous place.

pub fn add_before<Target>(self, plugin: impl Plugin) -> PluginGroupBuilder
where Target: Plugin,

Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.

If the plugin was already the group, it is removed from its previous place.

Panics

Panics if Target is not already in this PluginGroupBuilder.

pub fn try_add_before<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
where Target: Plugin, Insert: Plugin,

Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.

If the plugin was already in the group the add fails. If there isn’t a plugin of type Target in the group the plugin we’re trying to insert is returned.

pub fn try_add_before_overwrite<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
where Target: Plugin, Insert: Plugin,

Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.

If the plugin was already in the group, it is removed from its previous places. If there isn’t a plugin of type Target in the group the plugin we’re trying to insert is returned.

pub fn add_after<Target>(self, plugin: impl Plugin) -> PluginGroupBuilder
where Target: Plugin,

Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.

If the plugin was already the group, it is removed from its previous place.

Panics

Panics if Target is not already in this PluginGroupBuilder.

pub fn try_add_after<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
where Target: Plugin, Insert: Plugin,

Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.

If the plugin was already in the group the add fails. If there isn’t a plugin of type Target in the group the plugin we’re trying to insert is returned.

pub fn try_add_after_overwrite<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
where Target: Plugin, Insert: Plugin,

Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.

If the plugin was already in the group, it is removed from its previous places. If there isn’t a plugin of type Target in the group the plugin we’re trying to insert is returned.

pub fn enable<T>(self) -> PluginGroupBuilder
where T: Plugin,

Enables a Plugin.

Plugins within a PluginGroup are enabled by default. This function is used to opt back in to a Plugin after disabling it. If there are no plugins of type T in this group, it will panic.

pub fn disable<T>(self) -> PluginGroupBuilder
where T: Plugin,

Disables a Plugin, preventing it from being added to the App with the rest of the PluginGroup. The disabled Plugin keeps its place in the PluginGroup, so it can still be used for ordering with add_before or add_after, or it can be re-enabled. If there are no plugins of type T in this group, it will panic.

Examples found in repository?
examples/2d/mesh2d_repeated_texture.rs (line 20)
18fn main() {
19    App::new()
20        .add_plugins(DefaultPlugins.build().disable::<AudioPlugin>())
21        .add_systems(Startup, setup)
22        .run();
23}
More examples
Hide additional examples
examples/app/headless.rs (line 42)
16fn main() {
17    if cfg!(feature = "bevy_window") {
18        println!("This example is running with the bevy_window feature enabled and will not run headless.");
19        println!("Disable the default features and rerun the example to run headless.");
20        println!("To do so, run:");
21        println!();
22        println!("    cargo run --example headless --no-default-features --features bevy_log");
23        return;
24    }
25
26    // This app runs once
27    App::new()
28        .add_plugins(DefaultPlugins.set(ScheduleRunnerPlugin::run_once()))
29        .add_systems(Update, hello_world_system)
30        .run();
31
32    // This app loops forever at 60 fps
33    App::new()
34        .add_plugins(
35            DefaultPlugins
36                .set(ScheduleRunnerPlugin::run_loop(Duration::from_secs_f64(
37                    1.0 / 60.0,
38                )))
39                // The log and ctrl+c plugin can only be registered once globally,
40                // which means we need to disable it here, because it was already registered with the
41                // app that runs once.
42                .disable::<LogPlugin>(),
43        )
44        .add_systems(Update, counter)
45        .run();
46}
pub fn finish(self, app: &mut App)

Consumes the PluginGroupBuilder and builds the contained Plugins in the order specified.

Panics

Panics if one of the plugin in the group was already added to the application.