pub struct PluginGroupBuilder { /* private fields */ }Expand description
Facilitates the creation and configuration of a PluginGroup.
Provides a build ordering to ensure that Plugins which produce/require a Resource
are built before/after dependent/depending Plugins. Plugins inside the group
can be disabled, enabled or reordered.
Implementations
Sourceimpl PluginGroupBuilder
impl PluginGroupBuilder
Sourcepub fn start<PG>() -> PluginGroupBuilderwhere
PG: PluginGroup,
pub fn start<PG>() -> PluginGroupBuilderwhere
PG: PluginGroup,
Start a new builder for the PluginGroup.
Sourcepub fn contains<T>(&self) -> boolwhere
T: Plugin,
pub fn contains<T>(&self) -> boolwhere
T: Plugin,
Checks if the PluginGroupBuilder contains the given Plugin.
Sourcepub fn enabled<T>(&self) -> boolwhere
T: Plugin,
pub fn enabled<T>(&self) -> boolwhere
T: Plugin,
Returns true if the PluginGroupBuilder contains the given Plugin and it’s enabled.
Sourcepub fn set<T>(self, plugin: T) -> PluginGroupBuilderwhere
T: Plugin,
pub fn set<T>(self, plugin: T) -> PluginGroupBuilderwhere
T: Plugin,
Sourcepub fn try_set<T>(
self,
plugin: T,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>where
T: Plugin,
pub fn try_set<T>(
self,
plugin: T,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>where
T: Plugin,
Sourcepub fn add<T>(self, plugin: T) -> PluginGroupBuilderwhere
T: Plugin,
pub fn add<T>(self, plugin: T) -> PluginGroupBuilderwhere
T: Plugin,
Adds the plugin Plugin at the end of this PluginGroupBuilder. If the plugin was
already in the group, it is removed from its previous place.
Sourcepub fn try_add<T>(
self,
plugin: T,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>where
T: Plugin,
pub fn try_add<T>(
self,
plugin: T,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, T)>where
T: Plugin,
Attempts to add the plugin Plugin at the end of this PluginGroupBuilder.
If the plugin was already in the group the addition fails.
Sourcepub fn add_group(self, group: impl PluginGroup) -> PluginGroupBuilder
pub fn add_group(self, group: impl PluginGroup) -> PluginGroupBuilder
Adds a PluginGroup at the end of this PluginGroupBuilder. If the plugin was
already in the group, it is removed from its previous place.
Sourcepub fn add_before<Target>(self, plugin: impl Plugin) -> PluginGroupBuilderwhere
Target: Plugin,
pub fn add_before<Target>(self, plugin: impl Plugin) -> PluginGroupBuilderwhere
Target: Plugin,
Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.
If the plugin was already the group, it is removed from its previous place.
Panics
Panics if Target is not already in this PluginGroupBuilder.
Sourcepub fn try_add_before<Target, Insert>(
self,
plugin: Insert,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
pub fn try_add_before<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.
If the plugin was already in the group the add fails. If there isn’t a plugin
of type Target in the group the plugin we’re trying to insert is returned.
Sourcepub fn try_add_before_overwrite<Target, Insert>(
self,
plugin: Insert,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
pub fn try_add_before_overwrite<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
Adds a Plugin in this PluginGroupBuilder before the plugin of type Target.
If the plugin was already in the group, it is removed from its previous places.
If there isn’t a plugin of type Target in the group the plugin we’re trying to insert
is returned.
Sourcepub fn add_after<Target>(self, plugin: impl Plugin) -> PluginGroupBuilderwhere
Target: Plugin,
pub fn add_after<Target>(self, plugin: impl Plugin) -> PluginGroupBuilderwhere
Target: Plugin,
Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.
If the plugin was already the group, it is removed from its previous place.
Panics
Panics if Target is not already in this PluginGroupBuilder.
Sourcepub fn try_add_after<Target, Insert>(
self,
plugin: Insert,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
pub fn try_add_after<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.
If the plugin was already in the group the add fails. If there isn’t a plugin
of type Target in the group the plugin we’re trying to insert is returned.
Sourcepub fn try_add_after_overwrite<Target, Insert>(
self,
plugin: Insert,
) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
pub fn try_add_after_overwrite<Target, Insert>( self, plugin: Insert, ) -> Result<PluginGroupBuilder, (PluginGroupBuilder, Insert)>
Adds a Plugin in this PluginGroupBuilder after the plugin of type Target.
If the plugin was already in the group, it is removed from its previous places.
If there isn’t a plugin of type Target in the group the plugin we’re trying to insert
is returned.
Sourcepub fn enable<T>(self) -> PluginGroupBuilderwhere
T: Plugin,
pub fn enable<T>(self) -> PluginGroupBuilderwhere
T: Plugin,
Enables a Plugin.
Plugins within a PluginGroup are enabled by default. This function is used to
opt back in to a Plugin after disabling it. If there are no plugins
of type T in this group, it will panic.
Sourcepub fn disable<T>(self) -> PluginGroupBuilderwhere
T: Plugin,
pub fn disable<T>(self) -> PluginGroupBuilderwhere
T: Plugin,
Disables a Plugin, preventing it from being added to the App with the rest of the
PluginGroup. The disabled Plugin keeps its place in the PluginGroup, so it can
still be used for ordering with add_before or
add_after, or it can be re-enabled. If there are no
plugins of type T in this group, it will panic.
Examples found in repository?
More examples
16fn main() {
17 if cfg!(feature = "bevy_window") {
18 println!("This example is running with the bevy_window feature enabled and will not run headless.");
19 println!("Disable the default features and rerun the example to run headless.");
20 println!("To do so, run:");
21 println!();
22 println!(" cargo run --example headless --no-default-features --features bevy_log");
23 return;
24 }
25
26 // This app runs once
27 App::new()
28 .add_plugins(DefaultPlugins.set(ScheduleRunnerPlugin::run_once()))
29 .add_systems(Update, hello_world_system)
30 .run();
31
32 // This app loops forever at 60 fps
33 App::new()
34 .add_plugins(
35 DefaultPlugins
36 .set(ScheduleRunnerPlugin::run_loop(Duration::from_secs_f64(
37 1.0 / 60.0,
38 )))
39 // The log and ctrl+c plugin can only be registered once globally,
40 // which means we need to disable it here, because it was already registered with the
41 // app that runs once.
42 .disable::<LogPlugin>(),
43 )
44 .add_systems(Update, counter)
45 .run();
46}Trait Implementations
Sourceimpl PluginGroup for PluginGroupBuilder
impl PluginGroup for PluginGroupBuilder
Sourcefn build(self) -> PluginGroupBuilder
fn build(self) -> PluginGroupBuilder
Plugins that are to be added.Sourcefn name() -> String
fn name() -> String
PluginGroup which is primarily used for debugging.Auto Trait Implementations
impl Freeze for PluginGroupBuilder
impl !RefUnwindSafe for PluginGroupBuilder
impl Send for PluginGroupBuilder
impl Sync for PluginGroupBuilder
impl Unpin for PluginGroupBuilder
impl UnsafeUnpin for PluginGroupBuilder
impl !UnwindSafe for PluginGroupBuilder