pub enum LoadState {
NotLoaded,
Loading,
Loaded,
Failed(Arc<AssetLoadError>),
}Expand description
The load state of an asset.
Variants
NotLoaded
The asset has not started loading yet
Loading
The asset is in the process of loading.
Loaded
The asset has been loaded and has been added to the World
Failed(Arc<AssetLoadError>)
The asset failed to load. The underlying AssetLoadError is
referenced by Arc clones in all related DependencyLoadStates
and RecursiveDependencyLoadStates in the asset’s dependency tree.
Implementations
Sourceimpl LoadState
impl LoadState
Sourcepub fn is_loading(&self) -> bool
pub fn is_loading(&self) -> bool
Returns true if this instance is LoadState::Loading
Sourcepub fn is_loaded(&self) -> bool
pub fn is_loaded(&self) -> bool
Returns true if this instance is LoadState::Loaded
Examples found in repository?
examples/3d/pbr.rs (line 135)
127fn environment_map_load_finish(
128 mut commands: Commands,
129 asset_server: Res<AssetServer>,
130 environment_map: Single<&EnvironmentMapLight>,
131 label_entity: Option<Single<Entity, With<EnvironmentMapLabel>>>,
132) {
133 if asset_server
134 .load_state(&environment_map.diffuse_map)
135 .is_loaded()
136 && asset_server
137 .load_state(&environment_map.specular_map)
138 .is_loaded()
139 {
140 // Do not attempt to remove `label_entity` if it has already been removed.
141 if let Some(label_entity) = label_entity {
142 commands.entity(*label_entity).despawn();
143 }
144 }
145}More examples
examples/3d/skybox.rs (line 154)
148fn asset_loaded(
149 asset_server: Res<AssetServer>,
150 mut images: ResMut<Assets<Image>>,
151 mut cubemap: ResMut<Cubemap>,
152 mut skyboxes: Query<&mut Skybox>,
153) {
154 if !cubemap.is_loaded && asset_server.load_state(&cubemap.image_handle).is_loaded() {
155 info!("Swapping to {}...", CUBEMAPS[cubemap.index].0);
156 let image = images.get_mut(&cubemap.image_handle).unwrap();
157 // NOTE: PNGs do not have any metadata that could indicate they contain a cubemap texture,
158 // so they appear as one texture. The following code reconfigures the texture as necessary.
159 if image.texture_descriptor.array_layer_count() == 1 {
160 image.reinterpret_stacked_2d_as_array(image.height() / image.width());
161 image.texture_view_descriptor = Some(TextureViewDescriptor {
162 dimension: Some(TextureViewDimension::Cube),
163 ..default()
164 });
165 }
166
167 for mut skybox in &mut skyboxes {
168 skybox.image = cubemap.image_handle.clone();
169 }
170
171 cubemap.is_loaded = true;
172 }
173}examples/shader/array_texture.rs (line 59)
48fn create_array_texture(
49 mut commands: Commands,
50 asset_server: Res<AssetServer>,
51 mut loading_texture: ResMut<LoadingTexture>,
52 mut images: ResMut<Assets<Image>>,
53 mut meshes: ResMut<Assets<Mesh>>,
54 mut materials: ResMut<Assets<ArrayTextureMaterial>>,
55) {
56 if loading_texture.is_loaded
57 || !asset_server
58 .load_state(loading_texture.handle.id())
59 .is_loaded()
60 {
61 return;
62 }
63 loading_texture.is_loaded = true;
64 let image = images.get_mut(&loading_texture.handle).unwrap();
65
66 // Create a new array texture asset from the loaded texture.
67 let array_layers = 4;
68 image.reinterpret_stacked_2d_as_array(array_layers);
69
70 // Spawn some cubes using the array texture
71 let mesh_handle = meshes.add(Cuboid::default());
72 let material_handle = materials.add(ArrayTextureMaterial {
73 array_texture: loading_texture.handle.clone(),
74 });
75 for x in -5..=5 {
76 commands.spawn((
77 Mesh3d(mesh_handle.clone()),
78 MeshMaterial3d(material_handle.clone()),
79 Transform::from_xyz(x as f32 + 0.5, 0.0, 0.0),
80 ));
81 }
82}Sourcepub fn is_failed(&self) -> bool
pub fn is_failed(&self) -> bool
Returns true if this instance is LoadState::Failed