pub enum Volume {
Linear(f32),
Decibels(f32),
}Expand description
A Volume represents an audio source’s volume level.
To create a new Volume from a linear scale value, use
Volume::Linear.
To create a new Volume from decibels, use Volume::Decibels.
Variants
Linear(f32)
Create a new Volume from the given volume in the linear scale.
In a linear scale, the value 1.0 represents the “normal” volume,
meaning the audio is played at its original level. Values greater than
1.0 increase the volume, while values between 0.0 and 1.0 decrease
the volume. A value of 0.0 effectively mutes the audio.
Examples
let volume = Volume::Linear(0.5);
assert_eq!(volume.to_linear(), 0.5);
assert!(ops::abs(volume.to_decibels() - -6.0206) < EPSILON);
let volume = Volume::Linear(0.0);
assert_eq!(volume.to_linear(), 0.0);
assert_eq!(volume.to_decibels(), f32::NEG_INFINITY);
let volume = Volume::Linear(1.0);
assert_eq!(volume.to_linear(), 1.0);
assert!(ops::abs(volume.to_decibels() - 0.0) < EPSILON);Decibels(f32)
Create a new Volume from the given volume in decibels.
In a decibel scale, the value 0.0 represents the “normal” volume,
meaning the audio is played at its original level. Values greater than
0.0 increase the volume, while values less than 0.0 decrease the
volume. A value of f32::NEG_INFINITY decibels effectively mutes the
audio.
Examples
let volume = Volume::Decibels(-5.998);
assert!(ops::abs(volume.to_linear() - 0.5) < EPSILON);
let volume = Volume::Decibels(f32::NEG_INFINITY);
assert_eq!(volume.to_linear(), 0.0);
let volume = Volume::Decibels(0.0);
assert_eq!(volume.to_linear(), 1.0);
let volume = Volume::Decibels(20.0);
assert_eq!(volume.to_linear(), 10.0);Implementations
Sourceimpl Volume
impl Volume
Sourcepub fn to_decibels(&self) -> f32
pub fn to_decibels(&self) -> f32
Returns the volume in decibels as a float.
If the volume is silent / off / muted, i.e., its underlying linear scale
is 0.0, this method returns negative infinity.
Sourcepub fn increase_by_percentage(&self, percentage: f32) -> Volume
pub fn increase_by_percentage(&self, percentage: f32) -> Volume
Increases the volume by the specified percentage.
This method works in the linear domain, where a 100% increase means doubling the volume (equivalent to +6.02dB).
Arguments
percentage- The percentage to increase (50.0 means 50% increase)
Examples
use bevy_audio::Volume;
let volume = Volume::Linear(1.0);
let increased = volume.increase_by_percentage(100.0);
assert_eq!(increased.to_linear(), 2.0);Examples found in repository?
98fn volume(
99 keyboard_input: Res<ButtonInput<KeyCode>>,
100 mut music_controller: Query<&mut AudioSink, With<MyMusic>>,
101) {
102 let Ok(mut sink) = music_controller.single_mut() else {
103 return;
104 };
105
106 if keyboard_input.just_pressed(KeyCode::Equal) {
107 let current_volume = sink.volume();
108 sink.set_volume(current_volume.increase_by_percentage(10.0));
109 } else if keyboard_input.just_pressed(KeyCode::Minus) {
110 let current_volume = sink.volume();
111 sink.set_volume(current_volume.increase_by_percentage(-10.0));
112 }
113}Sourcepub fn decrease_by_percentage(&self, percentage: f32) -> Volume
pub fn decrease_by_percentage(&self, percentage: f32) -> Volume
Decreases the volume by the specified percentage.
This method works in the linear domain, where a 50% decrease means halving the volume (equivalent to -6.02dB).
Arguments
percentage- The percentage to decrease (50.0 means 50% decrease)
Examples
use bevy_audio::Volume;
let volume = Volume::Linear(1.0);
let decreased = volume.decrease_by_percentage(50.0);
assert_eq!(decreased.to_linear(), 0.5);Sourcepub fn scale_to_factor(&self, factor: f32) -> Volume
pub fn scale_to_factor(&self, factor: f32) -> Volume
Scales the volume to a specific linear factor relative to the current volume.
This is different from adjust_by_linear as it sets the volume to be
exactly the factor times the original volume, rather than applying
the factor to the current volume.
Arguments
factor- The scaling factor (2.0 = twice as loud, 0.5 = half as loud)
Examples
use bevy_audio::Volume;
let volume = Volume::Linear(0.8);
let scaled = volume.scale_to_factor(1.25);
assert_eq!(scaled.to_linear(), 1.0);Sourcepub fn fade_towards(&self, target: Volume, factor: f32) -> Volume
pub fn fade_towards(&self, target: Volume, factor: f32) -> Volume
Creates a fade effect by interpolating between current volume and target volume.
This method performs linear interpolation in the linear domain, which provides a more natural-sounding fade effect.
Arguments
target- The target volume to fade towardsfactor- The interpolation factor (0.0 = current volume, 1.0 = target volume)
Examples
use bevy_audio::Volume;
let current = Volume::Linear(1.0);
let target = Volume::Linear(0.0);
let faded = current.fade_towards(target, 0.5);
assert_eq!(faded.to_linear(), 0.5);Examples found in repository?
101fn fade_in(
102 mut commands: Commands,
103 mut audio_sink: Query<(&mut AudioSink, Entity), With<FadeIn>>,
104 timer: Res<GameStateTimer>,
105) {
106 for (mut audio, entity) in audio_sink.iter_mut() {
107 audio.set_volume(
108 Volume::SILENT.fade_towards(Volume::Linear(1.0), timer.0.elapsed_secs() / FADE_TIME),
109 );
110 if timer.0.elapsed_secs() >= FADE_TIME {
111 audio.set_volume(Volume::Linear(1.0));
112 commands.entity(entity).remove::<FadeIn>();
113 }
114 }
115}
116
117// Fades out the audio of entities that has the FadeOut component. Despawns the entities once audio
118// volume reaches zero.
119fn fade_out(
120 mut commands: Commands,
121 mut audio_sink: Query<(&mut AudioSink, Entity), With<FadeOut>>,
122 timer: Res<GameStateTimer>,
123) {
124 for (mut audio, entity) in audio_sink.iter_mut() {
125 audio.set_volume(
126 Volume::Linear(1.0).fade_towards(Volume::SILENT, timer.0.elapsed_secs() / FADE_TIME),
127 );
128 if timer.0.elapsed_secs() >= FADE_TIME {
129 commands.entity(entity).despawn();
130 }
131 }
132}Trait Implementations
impl Copy for Volume
Sourceimpl DivAssign for Volume
impl DivAssign for Volume
Sourcefn div_assign(&mut self, rhs: Volume)
fn div_assign(&mut self, rhs: Volume)
/= operation. Read moreSourceimpl Enum for Volume
impl Enum for Volume
Sourcefn field(&self, __name_param: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, __name_param: &str) -> Option<&(dyn PartialReflect + 'static)>
Sourcefn field_at(
&self,
__index_param: usize,
) -> Option<&(dyn PartialReflect + 'static)>
fn field_at( &self, __index_param: usize, ) -> Option<&(dyn PartialReflect + 'static)>
Sourcefn field_mut(
&mut self,
__name_param: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, __name_param: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Sourcefn field_at_mut(
&mut self,
__index_param: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, __index_param: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Sourcefn index_of(&self, __name_param: &str) -> Option<usize>
fn index_of(&self, __name_param: &str) -> Option<usize>
Sourcefn name_at(&self, __index_param: usize) -> Option<&str>
fn name_at(&self, __index_param: usize) -> Option<&str>
Sourcefn iter_fields(&self) -> VariantFieldIter<'_> ⓘ
fn iter_fields(&self) -> VariantFieldIter<'_> ⓘ
Sourcefn variant_name(&self) -> &str
fn variant_name(&self) -> &str
Sourcefn variant_index(&self) -> usize
fn variant_index(&self) -> usize
Sourcefn variant_type(&self) -> VariantType
fn variant_type(&self) -> VariantType
Sourcefn to_dynamic_enum(&self) -> DynamicEnum
fn to_dynamic_enum(&self) -> DynamicEnum
DynamicEnum from this enum.Sourcefn is_variant(&self, variant_type: VariantType) -> bool
fn is_variant(&self, variant_type: VariantType) -> bool
Sourcefn variant_path(&self) -> String
fn variant_path(&self) -> String
Sourceimpl From<Volume> for GlobalVolume
impl From<Volume> for GlobalVolume
Sourcefn from(volume: Volume) -> GlobalVolume
fn from(volume: Volume) -> GlobalVolume
Sourceimpl FromReflect for Volume
impl FromReflect for Volume
Sourcefn from_reflect(__param0: &(dyn PartialReflect + 'static)) -> Option<Volume>
fn from_reflect(__param0: &(dyn PartialReflect + 'static)) -> Option<Volume>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetTypeRegistration for Volume
impl GetTypeRegistration for Volume
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for Volume
impl IntoReturn for Volume
Sourceimpl MulAssign for Volume
impl MulAssign for Volume
Sourcefn mul_assign(&mut self, rhs: Volume)
fn mul_assign(&mut self, rhs: Volume)
*= operation. Read moreSourceimpl PartialOrd for Volume
impl PartialOrd for Volume
Sourceimpl PartialReflect for Volume
impl PartialReflect for Volume
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
__value_param: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, __value_param: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<Volume>) -> ReflectOwned
fn reflect_owned(self: Box<Volume>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<Volume>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<Volume>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<Volume>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<Volume>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for Volume
impl Reflect for Volume
Sourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<Volume>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Volume>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl TypePath for Volume
impl TypePath for Volume
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>