Skip to main content Crate camera Copy item path Source prelude The camera prelude. primitives visibility Components that control the visibility of entities. Camera The defining Component for camera entities,
storing information about how and what to render through this camera. Camera2d A 2D camera component. Enables the 2D render graph for a Camera . Camera3d A 3D camera component. Enables the main 3D render graph for a Camera . Camera3dDepthTextureUsage CameraMainTextureUsages This component lets you control the TextureUsages field of the main texture generated for the camera CameraPlugin CameraProjectionPlugin Adds Camera driver systems for a given projection type. CameraUpdateSystems Label for camera_system<T>, shared across all T. ClearColor A Resource that stores the default color that cameras use to clear the screen between frames. ComputedCameraValues Holds internally computed Camera values. CustomProjection Holds a dynamic CameraProjection trait object. Use Projection::custom() to construct a
custom projection. Exposure How much energy a Camera3d absorbs from incoming light. Hdr If this component is added to a camera, the camera will use an intermediate “high dynamic range” render texture.
This allows rendering with a wider range of lighting values. However, this does not affect
whether the camera will render with hdr display output (which bevy does not support currently)
and only affects the intermediate render texture. ImageRenderTarget A render target that renders to an Image . MainPassResolutionOverride Override the resolution a 3d camera’s main pass is rendered at. ManualTextureViewHandle A unique id that corresponds to a specific ManualTextureView in the ManualTextureViews collection. ManualTextureViewHandleTemplate OrthographicProjection Project a 3D space onto a 2D surface using parallel lines, i.e., unlike PerspectiveProjection ,
the size of objects remains the same regardless of their distance to the camera. PerspectiveProjection A 3D camera projection in which distant objects appear smaller than close objects. PhysicalCameraParameters Parameters based on physical camera characteristics for calculating EV100
values for use with Exposure . This is also used for depth of field. RenderTargetInfo Information about the current RenderTarget . ShadowLodOrigin The entity that Bevy uses to resolve visibility ranges when no specific
camera is applicable. SubCameraView Settings to define a camera sub view. Viewport Render viewport configuration for the Camera component. Camera3dDepthLoadOp The depth clear operation to perform for the main 3d pass. CameraOutputMode Control how this Camera outputs once rendering is completed. ClearColorConfig For a camera, specifies the color used to clear the viewport
before rendering
or when writing to the final render target texture . CompositingSpace Color space for alpha compositing. Affects how overlapping semi-transparent layers blend. MsaaWriteback Controls when MSAA writeback occurs for a camera. NormalizedRenderTarget Normalized version of the render target. Projection Component that defines how to compute a Camera ’s projection matrix. RenderTarget The “target” that a Camera will render to. For example, this could be a Window
swapchain or an Image . ScalingMode Scaling mode for OrthographicProjection . ViewportConversionError Error returned when a conversion between world-space and viewport-space coordinates fails. CameraProjection Describes a type that can generate a projection matrix, allowing it to be added to a
Camera ’s Projection component.