pub struct EasyScreenshotPlugin {
pub trigger: KeyCode,
pub format: ScreenshotFormat,
}Available on crate feature
bevy_dev_tools only.Expand description
Add this plugin to your app to enable easy screenshotting.
Add this plugin, press the key, and you have a screenshot ๐
Fields
trigger: KeyCodeKey that will trigger a screenshot
format: ScreenshotFormatFormat of the screenshot
The corresponding image format must be supported by bevy renderer