pub struct LogDiagnosticsState { /* private fields */ }Expand description
State used by the LogDiagnosticsPlugin
Implementations
Sourceimpl LogDiagnosticsState
impl LogDiagnosticsState
Sourcepub fn set_timer_duration(&mut self, duration: Duration)
pub fn set_timer_duration(&mut self, duration: Duration)
Sets a new duration for the log timer
Sourcepub fn add_filter(&mut self, diagnostic_path: DiagnosticPath) -> bool
pub fn add_filter(&mut self, diagnostic_path: DiagnosticPath) -> bool
Add a filter to the log state, returning true if the DiagnosticPath
was not present
Sourcepub fn extend_filter(&mut self, iter: impl IntoIterator<Item = DiagnosticPath>)
pub fn extend_filter(&mut self, iter: impl IntoIterator<Item = DiagnosticPath>)
Extends the filter of the log state with multiple DiagnosticPaths
Sourcepub fn remove_filter(&mut self, diagnostic_path: &DiagnosticPath) -> bool
pub fn remove_filter(&mut self, diagnostic_path: &DiagnosticPath) -> bool
Removes a filter from the log state, returning true if it was present
Sourcepub fn clear_filter(&mut self)
pub fn clear_filter(&mut self)
Clears the filters of the log state
Sourcepub fn enable_filtering(&mut self)
pub fn enable_filtering(&mut self)
Enables filtering with empty filters
Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 116)
103fn filters_inputs(
104 keys: Res<ButtonInput<KeyCode>>,
105 mut status: ResMut<LogDiagnosticsStatus>,
106 mut filters: ResMut<LogDiagnosticsFilters>,
107 mut log_state: ResMut<LogDiagnosticsState>,
108) {
109 if keys.just_pressed(KeyCode::KeyQ) {
110 *status = match *status {
111 LogDiagnosticsStatus::Enabled => {
112 log_state.disable_filtering();
113 LogDiagnosticsStatus::Disabled
114 }
115 LogDiagnosticsStatus::Disabled => {
116 log_state.enable_filtering();
117 if filters.frame_time {
118 enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
119 }
120 if filters.entity_count {
121 enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
122 }
123 if filters.system_info {
124 enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
125 }
126 LogDiagnosticsStatus::Enabled
127 }
128 };
129 }
130
131 let enabled = *status == LogDiagnosticsStatus::Enabled;
132 if keys.just_pressed(KeyCode::Digit1) {
133 filters.frame_time = !filters.frame_time;
134 if enabled {
135 if filters.frame_time {
136 enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
137 } else {
138 disable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
139 }
140 }
141 }
142 if keys.just_pressed(KeyCode::Digit2) {
143 filters.entity_count = !filters.entity_count;
144 if enabled {
145 if filters.entity_count {
146 enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
147 } else {
148 disable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
149 }
150 }
151 }
152 if keys.just_pressed(KeyCode::Digit3) {
153 filters.system_info = !filters.system_info;
154 if enabled {
155 if filters.system_info {
156 enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
157 } else {
158 disable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
159 }
160 }
161 }
162}Sourcepub fn disable_filtering(&mut self)
pub fn disable_filtering(&mut self)
Disables filtering
Examples found in repository?
examples/diagnostics/log_diagnostics.rs (line 112)
103fn filters_inputs(
104 keys: Res<ButtonInput<KeyCode>>,
105 mut status: ResMut<LogDiagnosticsStatus>,
106 mut filters: ResMut<LogDiagnosticsFilters>,
107 mut log_state: ResMut<LogDiagnosticsState>,
108) {
109 if keys.just_pressed(KeyCode::KeyQ) {
110 *status = match *status {
111 LogDiagnosticsStatus::Enabled => {
112 log_state.disable_filtering();
113 LogDiagnosticsStatus::Disabled
114 }
115 LogDiagnosticsStatus::Disabled => {
116 log_state.enable_filtering();
117 if filters.frame_time {
118 enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
119 }
120 if filters.entity_count {
121 enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
122 }
123 if filters.system_info {
124 enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
125 }
126 LogDiagnosticsStatus::Enabled
127 }
128 };
129 }
130
131 let enabled = *status == LogDiagnosticsStatus::Enabled;
132 if keys.just_pressed(KeyCode::Digit1) {
133 filters.frame_time = !filters.frame_time;
134 if enabled {
135 if filters.frame_time {
136 enable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
137 } else {
138 disable_filters(&mut log_state, FRAME_TIME_DIAGNOSTICS);
139 }
140 }
141 }
142 if keys.just_pressed(KeyCode::Digit2) {
143 filters.entity_count = !filters.entity_count;
144 if enabled {
145 if filters.entity_count {
146 enable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
147 } else {
148 disable_filters(&mut log_state, ENTITY_COUNT_DIAGNOSTICS);
149 }
150 }
151 }
152 if keys.just_pressed(KeyCode::Digit3) {
153 filters.system_info = !filters.system_info;
154 if enabled {
155 if filters.system_info {
156 enable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
157 } else {
158 disable_filters(&mut log_state, SYSTEM_INFO_DIAGNOSTICS);
159 }
160 }
161 }
162}Trait Implementations
Sourceimpl Component for LogDiagnosticsState
impl Component for LogDiagnosticsState
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::SparseSet
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::SparseSet
A constant indicating the storage type used for this component.
Sourcetype Mutability = Mutable
type Mutability = Mutable
A marker type to assist Bevy with determining if this component is
mutable, or immutable. Mutable components will have
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Registers required components. Read more
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Called when registering this component, allowing to override clone function (or disable cloning altogether) for this component. Read more
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<LogDiagnosticsState>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<LogDiagnosticsState>>
Returns
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
Maps the entities on this component using the given
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceconst HAS_SUMMARY_TICK: bool = false
const HAS_SUMMARY_TICK: bool = false
Set this constant to true if the component should track a summary
tick. Read more
impl Resource for LogDiagnosticsState