pub struct Archetype { /* private fields */ }Expand description
Metadata for a single archetype within a World.
For more information, see the module level documentation.
Implementations
Sourceimpl Archetype
impl Archetype
Sourcepub fn id(&self) -> ArchetypeId
pub fn id(&self) -> ArchetypeId
Fetches the ID for the archetype.
Sourcepub fn entities(&self) -> &[ArchetypeEntity]
pub fn entities(&self) -> &[ArchetypeEntity]
Fetches the entities contained in this archetype.
Sourcepub fn entities_with_location(
&self,
) -> impl Iterator<Item = (Entity, EntityLocation)>
pub fn entities_with_location( &self, ) -> impl Iterator<Item = (Entity, EntityLocation)>
Fetches the entities contained in this archetype.
Sourcepub fn table_components(&self) -> impl Iterator<Item = ComponentId>
pub fn table_components(&self) -> impl Iterator<Item = ComponentId>
Gets an iterator of all of the components stored in Tables.
All of the IDs are unique.
Sourcepub fn sparse_set_components(&self) -> impl Iterator<Item = ComponentId>
pub fn sparse_set_components(&self) -> impl Iterator<Item = ComponentId>
Gets an iterator of all of the components stored in ComponentSparseSets.
All of the IDs are unique.
Sourcepub fn components(&self) -> &[ComponentId]
pub fn components(&self) -> &[ComponentId]
Returns a slice of all of the components in the archetype.
All of the IDs are unique.
Sourcepub fn iter_components(&self) -> impl Iterator<Item = ComponentId> + Clone
pub fn iter_components(&self) -> impl Iterator<Item = ComponentId> + Clone
Gets an iterator of all of the components in the archetype.
All of the IDs are unique.
Sourcepub fn component_count(&self) -> usize
pub fn component_count(&self) -> usize
Returns the total number of components in the archetype
Sourcepub fn edges(&self) -> &Edges
pub fn edges(&self) -> &Edges
Fetches an immutable reference to the archetype’s Edges, a cache of
archetypal relationships.
Sourcepub fn entity_table_row(&self, row: ArchetypeRow) -> TableRow
pub fn entity_table_row(&self, row: ArchetypeRow) -> TableRow
Fetches the row in the Table where the components for the entity at index
is stored.
An entity’s archetype row can be fetched from EntityLocation::archetype_row, which
can be retrieved from Entities::get.
Panics
This function will panic if index >= self.len().
Sourcepub fn contains(&self, component_id: ComponentId) -> bool
pub fn contains(&self, component_id: ComponentId) -> bool
Checks if the archetype contains a specific component. This runs in O(1) time.
Sourcepub fn get_storage_type(&self, component_id: ComponentId) -> Option<StorageType>
pub fn get_storage_type(&self, component_id: ComponentId) -> Option<StorageType>
Gets the type of storage where a component in the archetype can be found.
Returns None if the component is not part of the archetype.
This runs in O(1) time.
Sourcepub fn has_add_hook(&self) -> bool
pub fn has_add_hook(&self) -> bool
Returns true if any of the components in this archetype have on_add hooks
Sourcepub fn has_insert_hook(&self) -> bool
pub fn has_insert_hook(&self) -> bool
Returns true if any of the components in this archetype have on_insert hooks
Sourcepub fn has_replace_hook(&self) -> bool
pub fn has_replace_hook(&self) -> bool
Returns true if any of the components in this archetype have on_replace hooks
Sourcepub fn has_remove_hook(&self) -> bool
pub fn has_remove_hook(&self) -> bool
Returns true if any of the components in this archetype have on_remove hooks
Sourcepub fn has_despawn_hook(&self) -> bool
pub fn has_despawn_hook(&self) -> bool
Returns true if any of the components in this archetype have on_despawn hooks
Sourcepub fn has_add_observer(&self) -> bool
pub fn has_add_observer(&self) -> bool
Returns true if any of the components in this archetype have at least one Add observer
Sourcepub fn has_insert_observer(&self) -> bool
pub fn has_insert_observer(&self) -> bool
Returns true if any of the components in this archetype have at least one Insert observer
Sourcepub fn has_replace_observer(&self) -> bool
pub fn has_replace_observer(&self) -> bool
Returns true if any of the components in this archetype have at least one Replace observer
Sourcepub fn has_remove_observer(&self) -> bool
pub fn has_remove_observer(&self) -> bool
Returns true if any of the components in this archetype have at least one Remove observer
Sourcepub fn has_despawn_observer(&self) -> bool
pub fn has_despawn_observer(&self) -> bool
Returns true if any of the components in this archetype have at least one Despawn observer