pub struct Ref<'w, T>where
T: ?Sized,{ /* private fields */ }Expand description
Shared borrow of an entity’s component with access to change detection.
Similar to Mut but is immutable and so doesn’t require unique access.
Examples
These two systems produce the same output.
fn how_many_changed_1(query: Query<(), Changed<MyComponent>>) {
println!("{} changed", query.iter().count());
}
fn how_many_changed_2(query: Query<Ref<MyComponent>>) {
println!("{} changed", query.iter().filter(|c| c.is_changed()).count());
}Implementations
Sourceimpl<'w, T> Ref<'w, T>where
T: ?Sized,
impl<'w, T> Ref<'w, T>where
T: ?Sized,
Sourcepub fn into_inner(self) -> &'w T
pub fn into_inner(self) -> &'w T
Returns the reference wrapped by this type. The reference is allowed to outlive self, which makes this method more flexible than simply borrowing self.
Sourcepub fn map<U>(self, f: impl FnOnce(&T) -> &U) -> Ref<'w, U>where
U: ?Sized,
pub fn map<U>(self, f: impl FnOnce(&T) -> &U) -> Ref<'w, U>where
U: ?Sized,
Map Ref to a different type using f.
This doesn’t do anything else than call f on the wrapped value.
This is equivalent to Mut::map_unchanged.
Sourcepub fn new(
value: &'w T,
added: &'w Tick,
changed: &'w Tick,
last_run: Tick,
this_run: Tick,
caller: MaybeLocation<&'w &'static Location<'static>>,
) -> Ref<'w, T>
pub fn new( value: &'w T, added: &'w Tick, changed: &'w Tick, last_run: Tick, this_run: Tick, caller: MaybeLocation<&'w &'static Location<'static>>, ) -> Ref<'w, T>
Create a new Ref using provided values.
This is an advanced feature, Refs are designed to be created by
engine-internal code and consumed by end-user code.
value- The value wrapped byRef.added- ATickthat stores the tick when the wrapped value was created.changed- ATickthat stores the last time the wrapped value was changed.last_run- ATick, occurring beforethis_run, which is used as a reference to determine whether the wrapped value is newly added or changed.this_run- ATickcorresponding to the current point in time – “now”.