pub struct ComponentsQueuedRegistrator<'w> { /* private fields */ }Expand description
A type that enables queuing registration in Components.
Note
These queued registrations return ComponentIds.
These ids are not yet valid, but they will become valid
when either ComponentsRegistrator::apply_queued_registrations is called or the same registration is made directly.
In either case, the returned ComponentIds will be correct, but they are not correct yet.
Generally, that means these ComponentIds can be safely used for read-only purposes.
Modifying the contents of the world through these ComponentIds directly without waiting for them to be fully registered
and without then confirming that they have been fully registered is not supported.
Hence, extra care is needed with these ComponentIds to ensure all safety rules are followed.
As a rule of thumb, if you have mutable access to ComponentsRegistrator, prefer to use that instead.
Use this only if you need to know the id of a component but do not need to modify the contents of the world based on that id.
Implementations
Sourceimpl<'w> ComponentsQueuedRegistrator<'w>
impl<'w> ComponentsQueuedRegistrator<'w>
Sourcepub unsafe fn new(
components: &'w Components,
ids: &'w ComponentIds,
) -> ComponentsQueuedRegistrator<'w>
pub unsafe fn new( components: &'w Components, ids: &'w ComponentIds, ) -> ComponentsQueuedRegistrator<'w>
Constructs a new ComponentsQueuedRegistrator.
Safety
The Components and ComponentIds must match.
For example, they must be from the same world.
Sourcepub fn queue_register_component<T>(&self) -> ComponentIdwhere
T: Component,
pub fn queue_register_component<T>(&self) -> ComponentIdwhere
T: Component,
This is a queued version of ComponentsRegistrator::register_component.
This will reserve an id and queue the registration.
These registrations will be carried out at the next opportunity.
If this has already been registered or queued, this returns the previous ComponentId.
Note
Technically speaking, the returned ComponentId is not valid, but it will become valid later.
See type level docs for details.
Sourcepub fn queue_register_component_with_descriptor(
&self,
descriptor: ComponentDescriptor,
) -> ComponentId
pub fn queue_register_component_with_descriptor( &self, descriptor: ComponentDescriptor, ) -> ComponentId
This is a queued version of ComponentsRegistrator::register_component_with_descriptor.
This will reserve an id and queue the registration.
These registrations will be carried out at the next opportunity.
This can also be used to register resources and non-send data.
Note
Technically speaking, the returned ComponentId is not valid, but it will become valid later.
See type level docs for details.
Warning
When registering a custom resource be sure to add crate::resource::IsResource as a required component,
Otherwise it will not function as a resource.
Sourcepub fn queue_register_non_send<T>(&self) -> ComponentIdwhere
T: Any,
pub fn queue_register_non_send<T>(&self) -> ComponentIdwhere
T: Any,
This is a queued version of ComponentsRegistrator::register_non_send.
This will reserve an id and queue the registration.
These registrations will be carried out at the next opportunity.
If this has already been registered or queued, this returns the previous ComponentId.
Note
Technically speaking, the returned ComponentId is not valid, but it will become valid later.
See type level docs for details.
Methods from Deref<Target = Components>
Sourcepub fn len(&self) -> usize
pub fn len(&self) -> usize
Returns the number of components registered or queued with this instance.
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Returns true if there are no components registered or queued with this instance. Otherwise, this returns false.
Sourcepub fn num_queued(&self) -> usize
pub fn num_queued(&self) -> usize
Returns the number of components registered with this instance.
Sourcepub fn any_queued(&self) -> bool
pub fn any_queued(&self) -> bool
Returns true if there are any components registered with this instance. Otherwise, this returns false.
Sourcepub fn num_registered(&self) -> usize
pub fn num_registered(&self) -> usize
Returns the number of components registered with this instance.
Sourcepub fn any_registered(&self) -> bool
pub fn any_registered(&self) -> bool
Returns true if there are any components registered with this instance. Otherwise, this returns false.
Sourcepub fn get_info(&self, id: ComponentId) -> Option<&ComponentInfo>
pub fn get_info(&self, id: ComponentId) -> Option<&ComponentInfo>
Gets the metadata associated with the given component, if it is registered.
This will return None if the id is not registered or is queued.
This will return an incorrect result if id did not come from the same world as self. It may return None or a garbage value.
Examples found in repository?
69fn main() {
70 let mut world = World::new();
71 let mut lines = std::io::stdin().lines();
72 let mut component_names = HashMap::<String, ComponentId>::new();
73 let mut component_info = HashMap::<ComponentId, ComponentInfo>::new();
74 let mut event_names = HashMap::<String, EventKey>::new();
75
76 println!("{PROMPT}");
77 loop {
78 print!("\n> ");
79 let _ = std::io::stdout().flush();
80 let Some(Ok(line)) = lines.next() else {
81 return;
82 };
83
84 if line.is_empty() {
85 return;
86 };
87
88 let Some((first, rest)) = line.trim().split_once(|c: char| c.is_whitespace()) else {
89 match &line.chars().next() {
90 Some('c') => println!("{COMPONENT_PROMPT}"),
91 Some('s') => println!("{ENTITY_PROMPT}"),
92 Some('q') => println!("{QUERY_PROMPT}"),
93 Some('e') => println!("{EVENT_PROMPT}"),
94 Some('t') => println!("{EMIT_PROMPT}"),
95 _ => println!("{PROMPT}"),
96 }
97 continue;
98 };
99
100 match &first[0..1] {
101 "c" => {
102 rest.split(',').for_each(|component| {
103 let mut component = component.split_whitespace();
104 let Some(name) = component.next() else {
105 return;
106 };
107 let size = match component.next().map(str::parse) {
108 Some(Ok(size)) => size,
109 _ => 0,
110 };
111 // Register our new component to the world with a layout specified by its size
112 // SAFETY: [u64] is Send + Sync
113 let id = world.register_component_with_descriptor(unsafe {
114 ComponentDescriptor::new_with_layout(
115 name.to_string(),
116 StorageType::Table,
117 Layout::array::<u64>(size).unwrap(),
118 None,
119 true,
120 false,
121 ComponentCloneBehavior::Default,
122 None,
123 )
124 });
125 let Some(info) = world.components().get_info(id) else {
126 return;
127 };
128 component_names.insert(name.to_string(), id);
129 component_info.insert(id, info.clone());
130 println!("Component {} created with id: {}", name, id.index());
131 });
132 }
133 "s" => {
134 let mut to_insert_ids = Vec::new();
135 let mut to_insert_data = Vec::new();
136 rest.split(',').for_each(|component| {
137 let mut component = component.split_whitespace();
138 let Some(name) = component.next() else {
139 return;
140 };
141
142 // Get the id for the component with the given name
143 let Some(&id) = component_names.get(name) else {
144 println!("Component {name} does not exist");
145 return;
146 };
147
148 // Calculate the length for the array based on the layout created for this component id
149 let info = world.components().get_info(id).unwrap();
150 let len = info.layout().size() / size_of::<u64>();
151 let mut values: Vec<u64> = component
152 .take(len)
153 .filter_map(|value| value.parse::<u64>().ok())
154 .collect();
155 values.resize(len, 0);
156
157 // Collect the id and array to be inserted onto our entity
158 to_insert_ids.push(id);
159 to_insert_data.push(values);
160 });
161
162 let mut entity = world.spawn_empty();
163
164 // Construct an `OwningPtr` for each component in `to_insert_data`
165 let to_insert_ptr = to_owning_ptrs(&mut to_insert_data);
166
167 // SAFETY:
168 // - Component ids have been taken from the same world
169 // - Each array is created to the layout specified in the world
170 unsafe {
171 entity.insert_by_ids(&to_insert_ids, to_insert_ptr.into_iter());
172 }
173
174 println!("Entity spawned with id: {}", entity.id());
175 }
176 "q" => {
177 let mut builder = QueryBuilder::<FilteredEntityMut>::new(&mut world);
178 parse_query(rest, &mut builder, &component_names);
179 let mut query = builder.build();
180 query.iter_mut(&mut world).for_each(|filtered_entity| {
181 let terms = filtered_entity
182 .access()
183 .try_iter_access()
184 .unwrap()
185 .map(|component_access| {
186 let id = *component_access.index();
187 let ptr = filtered_entity.get_by_id(id).unwrap();
188 let info = component_info.get(&id).unwrap();
189 let len = info.layout().size() / size_of::<u64>();
190
191 // SAFETY:
192 // - All components are created with layout [u64]
193 // - len is calculated from the component descriptor
194 let data = unsafe {
195 std::slice::from_raw_parts_mut(
196 ptr.assert_unique().as_ptr().cast::<u64>(),
197 len,
198 )
199 };
200
201 // If we have write access, increment each value once
202 if matches!(component_access, ComponentAccessKind::Exclusive(_)) {
203 data.iter_mut().for_each(|data| {
204 *data += 1;
205 });
206 }
207
208 format!("{}: {:?}", info.name(), data[0..len].to_vec())
209 })
210 .collect::<Vec<_>>()
211 .join(", ");
212
213 println!("{}: {}", filtered_entity.id(), terms);
214 });
215 }
216 "e" => {
217 rest.split(',').for_each(|event| {
218 let name = event.trim();
219 if name.is_empty() {
220 return;
221 }
222
223 // Register a ComponentId for this event, no Rust type needed.
224 // SAFETY: ZST with no drop
225 let event_component_id = world.register_component_with_descriptor(unsafe {
226 ComponentDescriptor::new_with_layout(
227 format!("event:{name}"),
228 StorageType::Table,
229 Layout::new::<()>(),
230 None,
231 false,
232 false,
233 ComponentCloneBehavior::Ignore,
234 None,
235 )
236 });
237 // SAFETY: event_component_id was just registered for this event
238 let event_key = unsafe { EventKey::new(event_component_id) };
239 event_names.insert(name.to_string(), event_key);
240
241 // Build a dynamic observer that prints when the event fires.
242 let runner: ObserverRunner = |mut world, _observer, ctx, _event, _trigger| {
243 println!(" Observer fired!");
244 if let Some(mut counts) = world.get_resource_mut::<EventFireCount>() {
245 *counts.0.entry(ctx.event_key).or_insert(0) += 1;
246 }
247 };
248
249 // SAFETY: event_key was just registered, runner ignores pointers
250 let observer =
251 unsafe { Observer::with_dynamic_runner(runner).with_event_key(event_key) };
252 world.spawn(observer);
253
254 println!(
255 "Event '{name}' registered (key: {}) with a dynamic observer",
256 event_component_id.index()
257 );
258 });
259
260 // Ensure the counter resource exists.
261 world.init_resource::<EventFireCount>();
262 }
263 "t" => {
264 let name = rest.trim();
265 let Some(&event_key) = event_names.get(name) else {
266 println!(
267 "Event '{name}' does not exist. Register it first with 'event {name}'"
268 );
269 continue;
270 };
271
272 let mut event_data = ();
273 let mut trigger_data = ();
274 // SAFETY: event_key was registered in this world, both pointers are valid ZSTs
275 unsafe {
276 world.trigger_dynamic(
277 event_key,
278 PtrMut::from(&mut event_data),
279 PtrMut::from(&mut trigger_data),
280 );
281 }
282
283 let count = world
284 .get_resource::<EventFireCount>()
285 .map_or(0, |c| c.0.get(&event_key).copied().unwrap_or(0));
286 println!("Event '{name}' triggered ({count} fires)");
287 }
288 _ => continue,
289 }
290 }
291}Sourcepub fn get_descriptor<'a>(
&'a self,
id: ComponentId,
) -> Option<Cow<'a, ComponentDescriptor>>
pub fn get_descriptor<'a>( &'a self, id: ComponentId, ) -> Option<Cow<'a, ComponentDescriptor>>
Gets the ComponentDescriptor of the component with this ComponentId if it is present.
This will return None only if the id is neither registered nor queued to be registered.
Currently, the Cow will be Cow::Owned if and only if the component is queued. It will be Cow::Borrowed otherwise.
This will return an incorrect result if id did not come from the same world as self. It may return None or a garbage value.
Sourcepub fn get_name<'a>(&'a self, id: ComponentId) -> Option<DebugName>
pub fn get_name<'a>(&'a self, id: ComponentId) -> Option<DebugName>
Gets the name of the component with this ComponentId if it is present.
This will return None only if the id is neither registered nor queued to be registered.
This will return an incorrect result if id did not come from the same world as self. It may return None or a garbage value.
Sourcepub unsafe fn get_info_unchecked(&self, id: ComponentId) -> &ComponentInfo
pub unsafe fn get_info_unchecked(&self, id: ComponentId) -> &ComponentInfo
Gets the metadata associated with the given component.
Safety
id must be a valid and fully registered ComponentId.
Sourcepub fn is_id_valid(&self, id: ComponentId) -> bool
pub fn is_id_valid(&self, id: ComponentId) -> bool
Returns true if the ComponentId is fully registered and valid.
Ids may be invalid if they are still queued to be registered.
Those ids are still correct, but they are not usable in every context yet.
Sourcepub fn get_valid_id(&self, type_id: TypeId) -> Option<ComponentId>
pub fn get_valid_id(&self, type_id: TypeId) -> Option<ComponentId>
Type-erased equivalent of Components::valid_component_id().
Sourcepub fn valid_component_id<T>(&self) -> Option<ComponentId>where
T: Component,
pub fn valid_component_id<T>(&self) -> Option<ComponentId>where
T: Component,
Returns the ComponentId of the given Component type T if it is fully registered.
If you want to include queued registration, see Components::component_id().
use bevy_ecs::prelude::*;
let mut world = World::new();
#[derive(Component)]
struct ComponentA;
let component_a_id = world.register_component::<ComponentA>();
assert_eq!(component_a_id, world.components().valid_component_id::<ComponentA>().unwrap())See also
Sourcepub fn get_id(&self, type_id: TypeId) -> Option<ComponentId>
pub fn get_id(&self, type_id: TypeId) -> Option<ComponentId>
Type-erased equivalent of Components::component_id().
Sourcepub fn component_id<T>(&self) -> Option<ComponentId>where
T: Component,
pub fn component_id<T>(&self) -> Option<ComponentId>where
T: Component,
Returns the ComponentId of the given Component type T.
The returned ComponentId is specific to the Components instance
it was retrieved from and should not be used with another Components
instance.
Returns None if the Component type has not yet been initialized using
ComponentsRegistrator::register_component() or
ComponentsQueuedRegistrator::queue_register_component().
use bevy_ecs::prelude::*;
let mut world = World::new();
#[derive(Component)]
struct ComponentA;
let component_a_id = world.register_component::<ComponentA>();
assert_eq!(component_a_id, world.components().component_id::<ComponentA>().unwrap())See also
Sourcepub fn iter_registered(
&self,
) -> impl Iterator<Item = (ComponentId, &ComponentInfo)>
pub fn iter_registered( &self, ) -> impl Iterator<Item = (ComponentId, &ComponentInfo)>
Gets an iterator over all components fully registered with this instance.