Struct bevy::ecs::schedule::SimpleExecutor
source · pub struct SimpleExecutor { /* private fields */ }Expand description
A variant of SingleThreadedExecutor that calls
apply_deferred immediately after running each system.
Implementations
sourceimpl SimpleExecutor
impl SimpleExecutor
sourcepub const fn new() -> SimpleExecutor
pub const fn new() -> SimpleExecutor
Creates a new simple executor for use in a Schedule.
This calls each system in order and immediately calls System::apply_deferred.