pub struct TableId(/* private fields */);Expand description
An opaque unique ID for a Table within a World.
Can be used with Tables::get to fetch the corresponding
table.
Each Archetype always points to a table via Archetype::table_id.
Multiple archetypes can point to the same table so long as the components
stored in the table are identical, but do not share the same sparse set
components.
Implementations
Sourceimpl TableId
impl TableId
Sourcepub const fn from_u32(index: u32) -> TableId
pub const fn from_u32(index: u32) -> TableId
Creates a new TableId.
index must be retrieved from calling TableId::as_u32 on a TableId you got
from a table of a given World or the created ID may be invalid.
Sourcepub const fn from_usize(index: usize) -> TableId
pub const fn from_usize(index: usize) -> TableId
Creates a new TableId.
index must be retrieved from calling TableId::as_usize on a TableId you got
from a table of a given World or the created ID may be invalid.
Panics
Will panic if the provided value does not fit within a u32.