pub struct FilteredEntityMut<'w, 's> { /* private fields */ }Expand description
Provides mutable access to a single entity and some of its components defined by the contained Access.
To define the access when used as a QueryData,
use a QueryBuilder or QueryParamBuilder.
The FilteredEntityMut must be the entire QueryData, and not nested inside a tuple with other data.
// This gives the `FilteredEntityMut` access to `&mut A`.
let mut query = QueryBuilder::<FilteredEntityMut>::new(&mut world)
.data::<&mut A>()
.build();
let mut filtered_entity: FilteredEntityMut = query.single_mut(&mut world).unwrap();
let component: Mut<A> = filtered_entity.get_mut().unwrap();Also see UnsafeFilteredEntityMut for a way to bypass borrow-checker restrictions.
Implementations
Sourceimpl<'w, 's> FilteredEntityMut<'w, 's>
impl<'w, 's> FilteredEntityMut<'w, 's>
Sourcepub fn reborrow(&mut self) -> FilteredEntityMut<'_, 's>
pub fn reborrow(&mut self) -> FilteredEntityMut<'_, 's>
Returns a new instance with a shorter lifetime.
This is useful if you have &mut FilteredEntityMut, but you need FilteredEntityMut.
Sourcepub fn into_readonly(self) -> FilteredEntityRef<'w, 's>
pub fn into_readonly(self) -> FilteredEntityRef<'w, 's>
Consumes self and returns read-only access to all of the entity’s
components, with the world 'w lifetime.
Sourcepub fn as_readonly(&self) -> FilteredEntityRef<'_, 's>
pub fn as_readonly(&self) -> FilteredEntityRef<'_, 's>
Gets read-only access to all of the entity’s components.
Sourcepub fn try_into_all(self) -> Result<EntityMut<'w>, TryFromFilteredError>
pub fn try_into_all(self) -> Result<EntityMut<'w>, TryFromFilteredError>
Consumes self and returns mutable access to the entity and all of
its components, with the world 'w lifetime. Returns an error if the
access does not include read and write access to all components.
Errors
TryFromFilteredError::MissingReadAllAccess- if the access does not include read access to all components.TryFromFilteredError::MissingWriteAllAccess- if the access does not include write access to all components.
Sourcepub fn as_unsafe_entity_cell(&mut self) -> UnsafeEntityCell<'_>
pub fn as_unsafe_entity_cell(&mut self) -> UnsafeEntityCell<'_>
Get access to the underlying UnsafeEntityCell.
Sourcepub fn id(&self) -> Entity
pub fn id(&self) -> Entity
Returns the ID of the current entity.
Examples found in repository?
69fn main() {
70 let mut world = World::new();
71 let mut lines = std::io::stdin().lines();
72 let mut component_names = HashMap::<String, ComponentId>::new();
73 let mut component_info = HashMap::<ComponentId, ComponentInfo>::new();
74 let mut event_names = HashMap::<String, EventKey>::new();
75
76 println!("{PROMPT}");
77 loop {
78 print!("\n> ");
79 let _ = std::io::stdout().flush();
80 let Some(Ok(line)) = lines.next() else {
81 return;
82 };
83
84 if line.is_empty() {
85 return;
86 };
87
88 let Some((first, rest)) = line.trim().split_once(|c: char| c.is_whitespace()) else {
89 match &line.chars().next() {
90 Some('c') => println!("{COMPONENT_PROMPT}"),
91 Some('s') => println!("{ENTITY_PROMPT}"),
92 Some('q') => println!("{QUERY_PROMPT}"),
93 Some('e') => println!("{EVENT_PROMPT}"),
94 Some('t') => println!("{EMIT_PROMPT}"),
95 _ => println!("{PROMPT}"),
96 }
97 continue;
98 };
99
100 match &first[0..1] {
101 "c" => {
102 rest.split(',').for_each(|component| {
103 let mut component = component.split_whitespace();
104 let Some(name) = component.next() else {
105 return;
106 };
107 let size = match component.next().map(str::parse) {
108 Some(Ok(size)) => size,
109 _ => 0,
110 };
111 // Register our new component to the world with a layout specified by it's size
112 // SAFETY: [u64] is Send + Sync
113 let id = world.register_component_with_descriptor(unsafe {
114 ComponentDescriptor::new_with_layout(
115 name.to_string(),
116 StorageType::Table,
117 Layout::array::<u64>(size).unwrap(),
118 None,
119 true,
120 ComponentCloneBehavior::Default,
121 None,
122 )
123 });
124 let Some(info) = world.components().get_info(id) else {
125 return;
126 };
127 component_names.insert(name.to_string(), id);
128 component_info.insert(id, info.clone());
129 println!("Component {} created with id: {}", name, id.index());
130 });
131 }
132 "s" => {
133 let mut to_insert_ids = Vec::new();
134 let mut to_insert_data = Vec::new();
135 rest.split(',').for_each(|component| {
136 let mut component = component.split_whitespace();
137 let Some(name) = component.next() else {
138 return;
139 };
140
141 // Get the id for the component with the given name
142 let Some(&id) = component_names.get(name) else {
143 println!("Component {name} does not exist");
144 return;
145 };
146
147 // Calculate the length for the array based on the layout created for this component id
148 let info = world.components().get_info(id).unwrap();
149 let len = info.layout().size() / size_of::<u64>();
150 let mut values: Vec<u64> = component
151 .take(len)
152 .filter_map(|value| value.parse::<u64>().ok())
153 .collect();
154 values.resize(len, 0);
155
156 // Collect the id and array to be inserted onto our entity
157 to_insert_ids.push(id);
158 to_insert_data.push(values);
159 });
160
161 let mut entity = world.spawn_empty();
162
163 // Construct an `OwningPtr` for each component in `to_insert_data`
164 let to_insert_ptr = to_owning_ptrs(&mut to_insert_data);
165
166 // SAFETY:
167 // - Component ids have been taken from the same world
168 // - Each array is created to the layout specified in the world
169 unsafe {
170 entity.insert_by_ids(&to_insert_ids, to_insert_ptr.into_iter());
171 }
172
173 println!("Entity spawned with id: {}", entity.id());
174 }
175 "q" => {
176 let mut builder = QueryBuilder::<FilteredEntityMut>::new(&mut world);
177 parse_query(rest, &mut builder, &component_names);
178 let mut query = builder.build();
179 query.iter_mut(&mut world).for_each(|filtered_entity| {
180 let terms = filtered_entity
181 .access()
182 .try_iter_access()
183 .unwrap()
184 .map(|component_access| {
185 let id = *component_access.index();
186 let ptr = filtered_entity.get_by_id(id).unwrap();
187 let info = component_info.get(&id).unwrap();
188 let len = info.layout().size() / size_of::<u64>();
189
190 // SAFETY:
191 // - All components are created with layout [u64]
192 // - len is calculated from the component descriptor
193 let data = unsafe {
194 std::slice::from_raw_parts_mut(
195 ptr.assert_unique().as_ptr().cast::<u64>(),
196 len,
197 )
198 };
199
200 // If we have write access, increment each value once
201 if matches!(component_access, ComponentAccessKind::Exclusive(_)) {
202 data.iter_mut().for_each(|data| {
203 *data += 1;
204 });
205 }
206
207 format!("{}: {:?}", info.name(), data[0..len].to_vec())
208 })
209 .collect::<Vec<_>>()
210 .join(", ");
211
212 println!("{}: {}", filtered_entity.id(), terms);
213 });
214 }
215 "e" => {
216 rest.split(',').for_each(|event| {
217 let name = event.trim();
218 if name.is_empty() {
219 return;
220 }
221
222 // Register a ComponentId for this event, no Rust type needed.
223 // SAFETY: ZST with no drop
224 let event_component_id = world.register_component_with_descriptor(unsafe {
225 ComponentDescriptor::new_with_layout(
226 format!("event:{name}"),
227 StorageType::Table,
228 Layout::new::<()>(),
229 None,
230 false,
231 ComponentCloneBehavior::Ignore,
232 None,
233 )
234 });
235 // SAFETY: event_component_id was just registered for this event
236 let event_key = unsafe { EventKey::new(event_component_id) };
237 event_names.insert(name.to_string(), event_key);
238
239 // Build a dynamic observer that prints when the event fires.
240 let runner: ObserverRunner = |mut world, _observer, ctx, _event, _trigger| {
241 println!(" Observer fired!");
242 if let Some(mut counts) = world.get_resource_mut::<EventFireCount>() {
243 *counts.0.entry(ctx.event_key).or_insert(0) += 1;
244 }
245 };
246
247 // SAFETY: event_key was just registered, runner ignores pointers
248 let observer =
249 unsafe { Observer::with_dynamic_runner(runner).with_event_key(event_key) };
250 world.spawn(observer);
251
252 println!(
253 "Event '{name}' registered (key: {}) with a dynamic observer",
254 event_component_id.index()
255 );
256 });
257
258 // Ensure the counter resource exists.
259 world.init_resource::<EventFireCount>();
260 }
261 "t" => {
262 let name = rest.trim();
263 let Some(&event_key) = event_names.get(name) else {
264 println!(
265 "Event '{name}' does not exist. Register it first with 'event {name}'"
266 );
267 continue;
268 };
269
270 let mut event_data = ();
271 let mut trigger_data = ();
272 // SAFETY: event_key was registered in this world, both pointers are valid ZSTs
273 unsafe {
274 world.trigger_dynamic(
275 event_key,
276 PtrMut::from(&mut event_data),
277 PtrMut::from(&mut trigger_data),
278 );
279 }
280
281 let count = world
282 .get_resource::<EventFireCount>()
283 .map_or(0, |c| c.0.get(&event_key).copied().unwrap_or(0));
284 println!("Event '{name}' triggered ({count} fires)");
285 }
286 _ => continue,
287 }
288 }
289}Sourcepub fn location(&self) -> EntityLocation
pub fn location(&self) -> EntityLocation
Gets metadata indicating the location where the current entity is stored.
Sourcepub fn archetype(&self) -> &Archetype
pub fn archetype(&self) -> &Archetype
Returns the archetype that the current entity belongs to.
Sourcepub fn access(&self) -> &Access
pub fn access(&self) -> &Access
Returns a reference to the underlying Access.
Examples found in repository?
69fn main() {
70 let mut world = World::new();
71 let mut lines = std::io::stdin().lines();
72 let mut component_names = HashMap::<String, ComponentId>::new();
73 let mut component_info = HashMap::<ComponentId, ComponentInfo>::new();
74 let mut event_names = HashMap::<String, EventKey>::new();
75
76 println!("{PROMPT}");
77 loop {
78 print!("\n> ");
79 let _ = std::io::stdout().flush();
80 let Some(Ok(line)) = lines.next() else {
81 return;
82 };
83
84 if line.is_empty() {
85 return;
86 };
87
88 let Some((first, rest)) = line.trim().split_once(|c: char| c.is_whitespace()) else {
89 match &line.chars().next() {
90 Some('c') => println!("{COMPONENT_PROMPT}"),
91 Some('s') => println!("{ENTITY_PROMPT}"),
92 Some('q') => println!("{QUERY_PROMPT}"),
93 Some('e') => println!("{EVENT_PROMPT}"),
94 Some('t') => println!("{EMIT_PROMPT}"),
95 _ => println!("{PROMPT}"),
96 }
97 continue;
98 };
99
100 match &first[0..1] {
101 "c" => {
102 rest.split(',').for_each(|component| {
103 let mut component = component.split_whitespace();
104 let Some(name) = component.next() else {
105 return;
106 };
107 let size = match component.next().map(str::parse) {
108 Some(Ok(size)) => size,
109 _ => 0,
110 };
111 // Register our new component to the world with a layout specified by it's size
112 // SAFETY: [u64] is Send + Sync
113 let id = world.register_component_with_descriptor(unsafe {
114 ComponentDescriptor::new_with_layout(
115 name.to_string(),
116 StorageType::Table,
117 Layout::array::<u64>(size).unwrap(),
118 None,
119 true,
120 ComponentCloneBehavior::Default,
121 None,
122 )
123 });
124 let Some(info) = world.components().get_info(id) else {
125 return;
126 };
127 component_names.insert(name.to_string(), id);
128 component_info.insert(id, info.clone());
129 println!("Component {} created with id: {}", name, id.index());
130 });
131 }
132 "s" => {
133 let mut to_insert_ids = Vec::new();
134 let mut to_insert_data = Vec::new();
135 rest.split(',').for_each(|component| {
136 let mut component = component.split_whitespace();
137 let Some(name) = component.next() else {
138 return;
139 };
140
141 // Get the id for the component with the given name
142 let Some(&id) = component_names.get(name) else {
143 println!("Component {name} does not exist");
144 return;
145 };
146
147 // Calculate the length for the array based on the layout created for this component id
148 let info = world.components().get_info(id).unwrap();
149 let len = info.layout().size() / size_of::<u64>();
150 let mut values: Vec<u64> = component
151 .take(len)
152 .filter_map(|value| value.parse::<u64>().ok())
153 .collect();
154 values.resize(len, 0);
155
156 // Collect the id and array to be inserted onto our entity
157 to_insert_ids.push(id);
158 to_insert_data.push(values);
159 });
160
161 let mut entity = world.spawn_empty();
162
163 // Construct an `OwningPtr` for each component in `to_insert_data`
164 let to_insert_ptr = to_owning_ptrs(&mut to_insert_data);
165
166 // SAFETY:
167 // - Component ids have been taken from the same world
168 // - Each array is created to the layout specified in the world
169 unsafe {
170 entity.insert_by_ids(&to_insert_ids, to_insert_ptr.into_iter());
171 }
172
173 println!("Entity spawned with id: {}", entity.id());
174 }
175 "q" => {
176 let mut builder = QueryBuilder::<FilteredEntityMut>::new(&mut world);
177 parse_query(rest, &mut builder, &component_names);
178 let mut query = builder.build();
179 query.iter_mut(&mut world).for_each(|filtered_entity| {
180 let terms = filtered_entity
181 .access()
182 .try_iter_access()
183 .unwrap()
184 .map(|component_access| {
185 let id = *component_access.index();
186 let ptr = filtered_entity.get_by_id(id).unwrap();
187 let info = component_info.get(&id).unwrap();
188 let len = info.layout().size() / size_of::<u64>();
189
190 // SAFETY:
191 // - All components are created with layout [u64]
192 // - len is calculated from the component descriptor
193 let data = unsafe {
194 std::slice::from_raw_parts_mut(
195 ptr.assert_unique().as_ptr().cast::<u64>(),
196 len,
197 )
198 };
199
200 // If we have write access, increment each value once
201 if matches!(component_access, ComponentAccessKind::Exclusive(_)) {
202 data.iter_mut().for_each(|data| {
203 *data += 1;
204 });
205 }
206
207 format!("{}: {:?}", info.name(), data[0..len].to_vec())
208 })
209 .collect::<Vec<_>>()
210 .join(", ");
211
212 println!("{}: {}", filtered_entity.id(), terms);
213 });
214 }
215 "e" => {
216 rest.split(',').for_each(|event| {
217 let name = event.trim();
218 if name.is_empty() {
219 return;
220 }
221
222 // Register a ComponentId for this event, no Rust type needed.
223 // SAFETY: ZST with no drop
224 let event_component_id = world.register_component_with_descriptor(unsafe {
225 ComponentDescriptor::new_with_layout(
226 format!("event:{name}"),
227 StorageType::Table,
228 Layout::new::<()>(),
229 None,
230 false,
231 ComponentCloneBehavior::Ignore,
232 None,
233 )
234 });
235 // SAFETY: event_component_id was just registered for this event
236 let event_key = unsafe { EventKey::new(event_component_id) };
237 event_names.insert(name.to_string(), event_key);
238
239 // Build a dynamic observer that prints when the event fires.
240 let runner: ObserverRunner = |mut world, _observer, ctx, _event, _trigger| {
241 println!(" Observer fired!");
242 if let Some(mut counts) = world.get_resource_mut::<EventFireCount>() {
243 *counts.0.entry(ctx.event_key).or_insert(0) += 1;
244 }
245 };
246
247 // SAFETY: event_key was just registered, runner ignores pointers
248 let observer =
249 unsafe { Observer::with_dynamic_runner(runner).with_event_key(event_key) };
250 world.spawn(observer);
251
252 println!(
253 "Event '{name}' registered (key: {}) with a dynamic observer",
254 event_component_id.index()
255 );
256 });
257
258 // Ensure the counter resource exists.
259 world.init_resource::<EventFireCount>();
260 }
261 "t" => {
262 let name = rest.trim();
263 let Some(&event_key) = event_names.get(name) else {
264 println!(
265 "Event '{name}' does not exist. Register it first with 'event {name}'"
266 );
267 continue;
268 };
269
270 let mut event_data = ();
271 let mut trigger_data = ();
272 // SAFETY: event_key was registered in this world, both pointers are valid ZSTs
273 unsafe {
274 world.trigger_dynamic(
275 event_key,
276 PtrMut::from(&mut event_data),
277 PtrMut::from(&mut trigger_data),
278 );
279 }
280
281 let count = world
282 .get_resource::<EventFireCount>()
283 .map_or(0, |c| c.0.get(&event_key).copied().unwrap_or(0));
284 println!("Event '{name}' triggered ({count} fires)");
285 }
286 _ => continue,
287 }
288 }
289}Sourcepub fn contains<T>(&self) -> boolwhere
T: Component,
pub fn contains<T>(&self) -> boolwhere
T: Component,
Returns true if the current entity has a component of type T.
Otherwise, this returns false.
Notes
If you do not know the concrete type of a component, consider using
Self::contains_id or Self::contains_type_id.
Sourcepub fn contains_id(&self, component_id: ComponentId) -> bool
pub fn contains_id(&self, component_id: ComponentId) -> bool
Returns true if the current entity has a component identified by component_id.
Otherwise, this returns false.
Notes
- If you know the concrete type of the component, you should prefer
Self::contains. - If you know the component’s
TypeIdbut not itsComponentId, consider usingSelf::contains_type_id.
Sourcepub fn contains_type_id(&self, type_id: TypeId) -> bool
pub fn contains_type_id(&self, type_id: TypeId) -> bool
Returns true if the current entity has a component with the type identified by type_id.
Otherwise, this returns false.
Notes
- If you know the concrete type of the component, you should prefer
Self::contains. - If you have a
ComponentIdinstead of aTypeId, consider usingSelf::contains_id.
Sourcepub fn get<T>(&self) -> Option<&T>where
T: Component,
pub fn get<T>(&self) -> Option<&T>where
T: Component,
Gets access to the component of type T for the current entity.
Returns None if the entity does not have a component of type T.
Sourcepub fn get_ref<T>(&self) -> Option<Ref<'_, T>>where
T: Component,
pub fn get_ref<T>(&self) -> Option<Ref<'_, T>>where
T: Component,
Gets access to the component of type T for the current entity,
including change detection information as a Ref.
Returns None if the entity does not have a component of type T.
Sourcepub fn get_mut<T>(&mut self) -> Option<Mut<'_, T>>
pub fn get_mut<T>(&mut self) -> Option<Mut<'_, T>>
Gets mutable access to the component of type T for the current entity.
Returns None if the entity does not have a component of type T or if
the access does not include write access to T.
Sourcepub unsafe fn get_mut_unchecked<T>(&self) -> Option<Mut<'_, T>>
pub unsafe fn get_mut_unchecked<T>(&self) -> Option<Mut<'_, T>>
Gets mutable access to the component of type T for the current entity.
Returns None if the entity does not have a component of type T or if
the access does not include write access to T.
This only requires &self, and so may be used to get mutable access to multiple components.
Example
#[derive(Component)]
struct X(usize);
#[derive(Component)]
struct Y(usize);
let mut entity = world.spawn((X(0), Y(0))).into_mutable();
// This gives the `FilteredEntityMut` access to `&mut X` and `&mut Y`.
let mut query = QueryBuilder::<FilteredEntityMut>::new(&mut world)
.data::<(&mut X, &mut Y)>()
.build();
let mut filtered_entity: FilteredEntityMut = query.single_mut(&mut world).unwrap();
// Get mutable access to two components at once
// SAFETY: We don't take any other references to `X` from this entity
let mut x = unsafe { filtered_entity.get_mut_unchecked::<X>() }.unwrap();
// SAFETY: We don't take any other references to `Y` from this entity
let mut y = unsafe { filtered_entity.get_mut_unchecked::<Y>() }.unwrap();
*x = X(1);
*y = Y(1);Safety
No other references to the same component may exist at the same time as the returned reference.
See also
get_mutfor the safe version.
Sourcepub fn into_mut<T>(self) -> Option<Mut<'w, T>>
pub fn into_mut<T>(self) -> Option<Mut<'w, T>>
Consumes self and gets mutable access to the component of type T
with the world 'w lifetime for the current entity.
Returns None if the entity does not have a component of type T.
Sourcepub unsafe fn into_mut_assume_mutable<T>(self) -> Option<Mut<'w, T>>where
T: Component,
pub unsafe fn into_mut_assume_mutable<T>(self) -> Option<Mut<'w, T>>where
T: Component,
Consumes self and gets mutable access to the component of type T
with the world 'w lifetime for the current entity.
Returns None if the entity does not have a component of type T.
Safety
Tmust be a mutable component
Sourcepub fn get_change_ticks<T>(&self) -> Option<ComponentTicks>where
T: Component,
pub fn get_change_ticks<T>(&self) -> Option<ComponentTicks>where
T: Component,
Retrieves the change ticks for the given component. This can be useful for implementing change detection in custom runtimes.
Sourcepub fn get_change_ticks_by_id(
&self,
component_id: ComponentId,
) -> Option<ComponentTicks>
pub fn get_change_ticks_by_id( &self, component_id: ComponentId, ) -> Option<ComponentTicks>
Retrieves the change ticks for the given ComponentId. This can be useful for implementing change
detection in custom runtimes.
You should prefer to use the typed API Self::get_change_ticks where possible and only
use this in cases where the actual component types are not known at
compile time.
Sourcepub fn get_by_id(&self, component_id: ComponentId) -> Option<Ptr<'_>>
pub fn get_by_id(&self, component_id: ComponentId) -> Option<Ptr<'_>>
Gets the component of the given ComponentId from the entity.
You should prefer to use the typed API Self::get where possible and only
use this in cases where the actual component types are not known at
compile time.
Unlike FilteredEntityMut::get, this returns a raw pointer to the component,
which is only valid while the FilteredEntityMut is alive.
Examples found in repository?
38fn base_system(access_components: In<Vec<ComponentId>>, mut query: Query<FilteredEntityMut>) {
39 #[cfg(feature = "trace")]
40 let _span = tracing::info_span!("base_system", components = ?access_components.0, count = query.iter().len()).entered();
41
42 for mut filtered_entity in &mut query {
43 // We calculate Faulhaber's formula mod 256 with n = value and p = exponent.
44 // See https://en.wikipedia.org/wiki/Faulhaber%27s_formula
45 // The time is takes to compute this depends on the number of entities and the values in
46 // each entity. This is to ensure that each system takes a different amount of time.
47 let mut total: Wrapping<u8> = Wrapping(0);
48 for (exponent, component_id) in (1_u32..).zip(access_components.0.iter()) {
49 // find the value of the component
50 let ptr = filtered_entity.get_by_id(*component_id).unwrap();
51
52 // SAFETY: All components have a u8 layout
53 let value: u8 = unsafe { *ptr.deref::<u8>() };
54
55 for i in 0..=value {
56 let mut product = Wrapping(1);
57 for _ in 1..=exponent {
58 product *= Wrapping(i);
59 }
60 total += product;
61 }
62 }
63
64 // we assign this value to all the components we can write to
65 for component_id in &access_components.0 {
66 if let Some(ptr) = filtered_entity.get_mut_by_id(*component_id) {
67 // SAFETY: All components have a u8 layout
68 unsafe {
69 let mut value = ptr.with_type::<u8>();
70 *value = total.0;
71 }
72 }
73 }
74 }
75}More examples
69fn main() {
70 let mut world = World::new();
71 let mut lines = std::io::stdin().lines();
72 let mut component_names = HashMap::<String, ComponentId>::new();
73 let mut component_info = HashMap::<ComponentId, ComponentInfo>::new();
74 let mut event_names = HashMap::<String, EventKey>::new();
75
76 println!("{PROMPT}");
77 loop {
78 print!("\n> ");
79 let _ = std::io::stdout().flush();
80 let Some(Ok(line)) = lines.next() else {
81 return;
82 };
83
84 if line.is_empty() {
85 return;
86 };
87
88 let Some((first, rest)) = line.trim().split_once(|c: char| c.is_whitespace()) else {
89 match &line.chars().next() {
90 Some('c') => println!("{COMPONENT_PROMPT}"),
91 Some('s') => println!("{ENTITY_PROMPT}"),
92 Some('q') => println!("{QUERY_PROMPT}"),
93 Some('e') => println!("{EVENT_PROMPT}"),
94 Some('t') => println!("{EMIT_PROMPT}"),
95 _ => println!("{PROMPT}"),
96 }
97 continue;
98 };
99
100 match &first[0..1] {
101 "c" => {
102 rest.split(',').for_each(|component| {
103 let mut component = component.split_whitespace();
104 let Some(name) = component.next() else {
105 return;
106 };
107 let size = match component.next().map(str::parse) {
108 Some(Ok(size)) => size,
109 _ => 0,
110 };
111 // Register our new component to the world with a layout specified by it's size
112 // SAFETY: [u64] is Send + Sync
113 let id = world.register_component_with_descriptor(unsafe {
114 ComponentDescriptor::new_with_layout(
115 name.to_string(),
116 StorageType::Table,
117 Layout::array::<u64>(size).unwrap(),
118 None,
119 true,
120 ComponentCloneBehavior::Default,
121 None,
122 )
123 });
124 let Some(info) = world.components().get_info(id) else {
125 return;
126 };
127 component_names.insert(name.to_string(), id);
128 component_info.insert(id, info.clone());
129 println!("Component {} created with id: {}", name, id.index());
130 });
131 }
132 "s" => {
133 let mut to_insert_ids = Vec::new();
134 let mut to_insert_data = Vec::new();
135 rest.split(',').for_each(|component| {
136 let mut component = component.split_whitespace();
137 let Some(name) = component.next() else {
138 return;
139 };
140
141 // Get the id for the component with the given name
142 let Some(&id) = component_names.get(name) else {
143 println!("Component {name} does not exist");
144 return;
145 };
146
147 // Calculate the length for the array based on the layout created for this component id
148 let info = world.components().get_info(id).unwrap();
149 let len = info.layout().size() / size_of::<u64>();
150 let mut values: Vec<u64> = component
151 .take(len)
152 .filter_map(|value| value.parse::<u64>().ok())
153 .collect();
154 values.resize(len, 0);
155
156 // Collect the id and array to be inserted onto our entity
157 to_insert_ids.push(id);
158 to_insert_data.push(values);
159 });
160
161 let mut entity = world.spawn_empty();
162
163 // Construct an `OwningPtr` for each component in `to_insert_data`
164 let to_insert_ptr = to_owning_ptrs(&mut to_insert_data);
165
166 // SAFETY:
167 // - Component ids have been taken from the same world
168 // - Each array is created to the layout specified in the world
169 unsafe {
170 entity.insert_by_ids(&to_insert_ids, to_insert_ptr.into_iter());
171 }
172
173 println!("Entity spawned with id: {}", entity.id());
174 }
175 "q" => {
176 let mut builder = QueryBuilder::<FilteredEntityMut>::new(&mut world);
177 parse_query(rest, &mut builder, &component_names);
178 let mut query = builder.build();
179 query.iter_mut(&mut world).for_each(|filtered_entity| {
180 let terms = filtered_entity
181 .access()
182 .try_iter_access()
183 .unwrap()
184 .map(|component_access| {
185 let id = *component_access.index();
186 let ptr = filtered_entity.get_by_id(id).unwrap();
187 let info = component_info.get(&id).unwrap();
188 let len = info.layout().size() / size_of::<u64>();
189
190 // SAFETY:
191 // - All components are created with layout [u64]
192 // - len is calculated from the component descriptor
193 let data = unsafe {
194 std::slice::from_raw_parts_mut(
195 ptr.assert_unique().as_ptr().cast::<u64>(),
196 len,
197 )
198 };
199
200 // If we have write access, increment each value once
201 if matches!(component_access, ComponentAccessKind::Exclusive(_)) {
202 data.iter_mut().for_each(|data| {
203 *data += 1;
204 });
205 }
206
207 format!("{}: {:?}", info.name(), data[0..len].to_vec())
208 })
209 .collect::<Vec<_>>()
210 .join(", ");
211
212 println!("{}: {}", filtered_entity.id(), terms);
213 });
214 }
215 "e" => {
216 rest.split(',').for_each(|event| {
217 let name = event.trim();
218 if name.is_empty() {
219 return;
220 }
221
222 // Register a ComponentId for this event, no Rust type needed.
223 // SAFETY: ZST with no drop
224 let event_component_id = world.register_component_with_descriptor(unsafe {
225 ComponentDescriptor::new_with_layout(
226 format!("event:{name}"),
227 StorageType::Table,
228 Layout::new::<()>(),
229 None,
230 false,
231 ComponentCloneBehavior::Ignore,
232 None,
233 )
234 });
235 // SAFETY: event_component_id was just registered for this event
236 let event_key = unsafe { EventKey::new(event_component_id) };
237 event_names.insert(name.to_string(), event_key);
238
239 // Build a dynamic observer that prints when the event fires.
240 let runner: ObserverRunner = |mut world, _observer, ctx, _event, _trigger| {
241 println!(" Observer fired!");
242 if let Some(mut counts) = world.get_resource_mut::<EventFireCount>() {
243 *counts.0.entry(ctx.event_key).or_insert(0) += 1;
244 }
245 };
246
247 // SAFETY: event_key was just registered, runner ignores pointers
248 let observer =
249 unsafe { Observer::with_dynamic_runner(runner).with_event_key(event_key) };
250 world.spawn(observer);
251
252 println!(
253 "Event '{name}' registered (key: {}) with a dynamic observer",
254 event_component_id.index()
255 );
256 });
257
258 // Ensure the counter resource exists.
259 world.init_resource::<EventFireCount>();
260 }
261 "t" => {
262 let name = rest.trim();
263 let Some(&event_key) = event_names.get(name) else {
264 println!(
265 "Event '{name}' does not exist. Register it first with 'event {name}'"
266 );
267 continue;
268 };
269
270 let mut event_data = ();
271 let mut trigger_data = ();
272 // SAFETY: event_key was registered in this world, both pointers are valid ZSTs
273 unsafe {
274 world.trigger_dynamic(
275 event_key,
276 PtrMut::from(&mut event_data),
277 PtrMut::from(&mut trigger_data),
278 );
279 }
280
281 let count = world
282 .get_resource::<EventFireCount>()
283 .map_or(0, |c| c.0.get(&event_key).copied().unwrap_or(0));
284 println!("Event '{name}' triggered ({count} fires)");
285 }
286 _ => continue,
287 }
288 }
289}Sourcepub fn get_mut_by_id(
&mut self,
component_id: ComponentId,
) -> Option<MutUntyped<'_>>
pub fn get_mut_by_id( &mut self, component_id: ComponentId, ) -> Option<MutUntyped<'_>>
Gets a MutUntyped of the component of the given ComponentId from the entity.
You should prefer to use the typed API Self::get_mut where possible and only
use this in cases where the actual component types are not known at
compile time.
Unlike FilteredEntityMut::get_mut, this returns a raw pointer to the component,
which is only valid while the FilteredEntityMut is alive.
Examples found in repository?
38fn base_system(access_components: In<Vec<ComponentId>>, mut query: Query<FilteredEntityMut>) {
39 #[cfg(feature = "trace")]
40 let _span = tracing::info_span!("base_system", components = ?access_components.0, count = query.iter().len()).entered();
41
42 for mut filtered_entity in &mut query {
43 // We calculate Faulhaber's formula mod 256 with n = value and p = exponent.
44 // See https://en.wikipedia.org/wiki/Faulhaber%27s_formula
45 // The time is takes to compute this depends on the number of entities and the values in
46 // each entity. This is to ensure that each system takes a different amount of time.
47 let mut total: Wrapping<u8> = Wrapping(0);
48 for (exponent, component_id) in (1_u32..).zip(access_components.0.iter()) {
49 // find the value of the component
50 let ptr = filtered_entity.get_by_id(*component_id).unwrap();
51
52 // SAFETY: All components have a u8 layout
53 let value: u8 = unsafe { *ptr.deref::<u8>() };
54
55 for i in 0..=value {
56 let mut product = Wrapping(1);
57 for _ in 1..=exponent {
58 product *= Wrapping(i);
59 }
60 total += product;
61 }
62 }
63
64 // we assign this value to all the components we can write to
65 for component_id in &access_components.0 {
66 if let Some(ptr) = filtered_entity.get_mut_by_id(*component_id) {
67 // SAFETY: All components have a u8 layout
68 unsafe {
69 let mut value = ptr.with_type::<u8>();
70 *value = total.0;
71 }
72 }
73 }
74 }
75}Sourcepub unsafe fn get_mut_by_id_unchecked(
&self,
component_id: ComponentId,
) -> Option<MutUntyped<'_>>
pub unsafe fn get_mut_by_id_unchecked( &self, component_id: ComponentId, ) -> Option<MutUntyped<'_>>
Gets a MutUntyped of the component of the given ComponentId from the entity.
You should prefer to use the typed API Self::get_mut where possible and only
use this in cases where the actual component types are not known at
compile time.
Unlike FilteredEntityMut::get_mut, this returns a raw pointer to the component,
which is only valid while the FilteredEntityMut is alive.
This only requires &self, and so may be used to get mutable access to multiple components.
Safety
No other references to the same component may exist at the same time as the returned reference.
See also
get_mut_by_idfor the safe version.
Sourcepub fn spawned_by(&self) -> MaybeLocation
pub fn spawned_by(&self) -> MaybeLocation
Returns the source code location from which this entity has last been spawned.
Sourcepub fn spawn_tick(&self) -> Tick
pub fn spawn_tick(&self) -> Tick
Returns the Tick at which this entity has been spawned.