pub enum ImageSampler {
Default,
Descriptor(ImageSamplerDescriptor),
}Expand description
Used in Image, this determines what image sampler to use when rendering. The default setting,
ImageSampler::Default, will read the sampler from the ImagePlugin at setup.
Setting this to ImageSampler::Descriptor will override the global default descriptor for this Image.
Variants
Default
Default image sampler, derived from the ImagePlugin setup.
Descriptor(ImageSamplerDescriptor)
Custom sampler for this image which will override global default.
Implementations
Sourceimpl ImageSampler
impl ImageSampler
Sourcepub fn linear() -> ImageSampler
pub fn linear() -> ImageSampler
Returns an image sampler with ImageFilterMode::Linear min and mag filters
Sourcepub fn nearest() -> ImageSampler
pub fn nearest() -> ImageSampler
Returns an image sampler with ImageFilterMode::Nearest min and mag filters
Examples found in repository?
17fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
18 let image = asset_server.load_with_settings(
19 "textures/fantasy_ui_borders/numbered_slices.png",
20 |settings: &mut ImageLoaderSettings| {
21 // Need to use nearest filtering to avoid bleeding between the slices with tiling
22 settings.sampler = ImageSampler::nearest();
23 },
24 );
25
26 let slicer = TextureSlicer {
27 // `numbered_slices.png` is 48 pixels square. `BorderRect::square(16.)` insets the slicing line from each edge by 16 pixels, resulting in nine slices that are each 16 pixels square.
28 border: BorderRect::all(16.),
29 // With `SliceScaleMode::Tile` the side and center slices are tiled to fill the side and center sections of the target.
30 // And with a `stretch_value` of `1.` the tiles will have the same size as the corresponding slices in the source image.
31 center_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
32 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
33 ..default()
34 };
35
36 // ui camera
37 commands.spawn(Camera2d);
38
39 commands
40 .spawn(Node {
41 width: percent(100),
42 height: percent(100),
43 justify_content: JustifyContent::Center,
44 align_content: AlignContent::Center,
45 flex_wrap: FlexWrap::Wrap,
46 column_gap: px(10),
47 row_gap: px(10),
48 ..default()
49 })
50 .with_children(|parent| {
51 for [columns, rows] in [[3, 3], [4, 4], [5, 4], [4, 5], [5, 5]] {
52 for (flip_x, flip_y) in [(false, false), (false, true), (true, false), (true, true)]
53 {
54 parent.spawn((
55 ImageNode {
56 image: image.clone(),
57 flip_x,
58 flip_y,
59 image_mode: NodeImageMode::Sliced(slicer.clone()),
60 ..default()
61 },
62 Node {
63 width: px(16 * columns),
64 height: px(16 * rows),
65 ..default()
66 },
67 ));
68 }
69 }
70 });
71}More examples
21fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
22 // Without any .meta file specifying settings, the default sampler [ImagePlugin::default()] is used for loading images.
23 // If you are using a very small image and rendering it larger like seen here, the default linear filtering will result in a blurry image.
24 // Useful note: The default sampler specified by the ImagePlugin is *not* the same as the default implementation of sampler. This is why
25 // everything uses linear by default but if you look at the default of sampler, it uses nearest.
26 commands.spawn((
27 Sprite {
28 image: asset_server.load("bevy_pixel_dark.png"),
29 custom_size: Some(Vec2 { x: 160.0, y: 120.0 }),
30 ..Default::default()
31 },
32 Transform::from_xyz(-100.0, 0.0, 0.0),
33 ));
34
35 // When a .meta file is added with the same name as the asset and a '.meta' extension
36 // you can (and must) specify all fields of the asset loader's settings for that
37 // particular asset, in this case [ImageLoaderSettings]. Take a look at
38 // examples/asset/files/bevy_pixel_dark_with_meta.png.meta
39 // for the format and you'll notice, the only non-default option is setting Nearest
40 // filtering. This tends to work much better for pixel art assets.
41 // A good reference when filling this out is to check out [ImageLoaderSettings::default()]
42 // and follow to the default implementation of each fields type.
43 // https://docs.rs/bevy/latest/bevy/image/struct.ImageLoaderSettings.html
44 commands.spawn((
45 Sprite {
46 image: asset_server.load("bevy_pixel_dark_with_meta.png"),
47 custom_size: Some(Vec2 { x: 160.0, y: 120.0 }),
48 ..Default::default()
49 },
50 Transform::from_xyz(100.0, 0.0, 0.0),
51 ));
52
53 // Another option is to use the AssetServers load_with_settings function.
54 // With this you can specify the same settings upon loading your asset with a
55 // couple of differences. A big one is that you aren't required to set *every*
56 // setting, just modify the ones that you need. It works by passing in a function
57 // (in this case an anonymous closure) that takes a reference to the settings type
58 // that is then modified in the function.
59 // Do note that if you want to load the same asset with different settings, the
60 // settings changes from any loads after the first of the same asset will be ignored.
61 // This is why this one loads a differently named copy of the asset instead of using
62 // same one as without a .meta file.
63 commands.spawn((
64 Sprite {
65 image: asset_server.load_with_settings(
66 "bevy_pixel_dark_with_settings.png",
67 |settings: &mut ImageLoaderSettings| {
68 settings.sampler = ImageSampler::nearest();
69 },
70 ),
71 custom_size: Some(Vec2 { x: 160.0, y: 120.0 }),
72 ..Default::default()
73 },
74 Transform::from_xyz(0.0, 150.0, 0.0),
75 ));
76
77 commands.spawn(Camera2d);
78}Sourcepub fn get_or_init_descriptor(&mut self) -> &mut ImageSamplerDescriptor
pub fn get_or_init_descriptor(&mut self) -> &mut ImageSamplerDescriptor
Initialize the descriptor if it is not already initialized.
Descriptor is typically initialized by Bevy when the image is loaded, so this is convenient shortcut for updating the descriptor.