pub struct Gamepad { /* private fields */ }Expand description
Stores a connected gamepad’s metadata such as the name and its GamepadButton and GamepadAxis.
An entity with this component is spawned automatically after GamepadConnectionEvent
and updated by gamepad_event_processing_system.
See also GamepadSettings for configuration.
Examples
fn gamepad_usage_system(gamepads: Query<(&Name, &Gamepad)>) {
for (name, gamepad) in &gamepads {
println!("{name}");
if gamepad.just_pressed(GamepadButton::North) {
println!("{} just pressed North", name)
}
if let Some(left_stick_x) = gamepad.get(GamepadAxis::LeftStickX) {
println!("left stick X: {}", left_stick_x)
}
}
}Implementations
Sourceimpl Gamepad
impl Gamepad
Sourcepub fn vendor_id(&self) -> Option<u16>
pub fn vendor_id(&self) -> Option<u16>
Returns the USB vendor ID as assigned by the USB-IF, if available.
Sourcepub fn product_id(&self) -> Option<u16>
pub fn product_id(&self) -> Option<u16>
Returns the USB product ID as assigned by the vendor, if available.
Sourcepub fn get(&self, input: impl Into<GamepadInput>) -> Option<f32>
pub fn get(&self, input: impl Into<GamepadInput>) -> Option<f32>
Returns the analog data of the provided GamepadAxis or GamepadButton.
Examples found in repository?
12fn gamepad_system(gamepads: Query<(Entity, &Gamepad)>) {
13 for (entity, gamepad) in &gamepads {
14 if gamepad.just_pressed(GamepadButton::South) {
15 info!("{} just pressed South", entity);
16 } else if gamepad.just_released(GamepadButton::South) {
17 info!("{} just released South", entity);
18 }
19
20 let right_trigger = gamepad.get(GamepadButton::RightTrigger2).unwrap();
21 if right_trigger.abs() > 0.01 {
22 info!("{} RightTrigger2 value is {}", entity, right_trigger);
23 }
24
25 let left_stick_x = gamepad.get(GamepadAxis::LeftStickX).unwrap();
26 if left_stick_x.abs() > 0.01 {
27 info!("{} LeftStickX value is {}", entity, left_stick_x);
28 }
29 }
30}Sourcepub fn get_unclamped(&self, input: impl Into<GamepadInput>) -> Option<f32>
pub fn get_unclamped(&self, input: impl Into<GamepadInput>) -> Option<f32>
Returns the unclamped analog data of the provided GamepadAxis or GamepadButton.
This value may be outside the Axis::MIN and Axis::MAX range.
Sourcepub fn left_stick(&self) -> Vec2
pub fn left_stick(&self) -> Vec2
Returns the left stick as a Vec2.
Sourcepub fn right_stick(&self) -> Vec2
pub fn right_stick(&self) -> Vec2
Returns the right stick as a Vec2.
Sourcepub fn pressed(&self, button_type: GamepadButton) -> bool
pub fn pressed(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been pressed.
Sourcepub fn any_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has been pressed.
Sourcepub fn all_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have been pressed.
Sourcepub fn just_pressed(&self, button_type: GamepadButton) -> bool
pub fn just_pressed(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been pressed during the current frame.
Note: This function does not imply information regarding the current state of ButtonInput::pressed or ButtonInput::just_released.
Examples found in repository?
12fn gamepad_system(gamepads: Query<(Entity, &Gamepad)>) {
13 for (entity, gamepad) in &gamepads {
14 if gamepad.just_pressed(GamepadButton::South) {
15 info!("{} just pressed South", entity);
16 } else if gamepad.just_released(GamepadButton::South) {
17 info!("{} just released South", entity);
18 }
19
20 let right_trigger = gamepad.get(GamepadButton::RightTrigger2).unwrap();
21 if right_trigger.abs() > 0.01 {
22 info!("{} RightTrigger2 value is {}", entity, right_trigger);
23 }
24
25 let left_stick_x = gamepad.get(GamepadAxis::LeftStickX).unwrap();
26 if left_stick_x.abs() > 0.01 {
27 info!("{} LeftStickX value is {}", entity, left_stick_x);
28 }
29 }
30}More examples
281fn process_inputs(
282 mut action_state: ResMut<ActionState>,
283 keyboard_input: Res<ButtonInput<KeyCode>>,
284 gamepad_input: Query<&Gamepad>,
285) {
286 // Reset the set of pressed actions each frame
287 // to ensure that we only process each action once
288 action_state.pressed_actions.clear();
289
290 for action in DirectionalNavigationAction::variants() {
291 // Use just_pressed to ensure that we only process each action once
292 // for each time it is pressed
293 if keyboard_input.just_pressed(action.keycode()) {
294 action_state.pressed_actions.insert(action);
295 }
296 }
297
298 // We're treating this like a single-player game:
299 // if multiple gamepads are connected, we don't care which one is being used
300 for gamepad in gamepad_input.iter() {
301 for action in DirectionalNavigationAction::variants() {
302 // Unlike keyboard input, gamepads are bound to a specific controller
303 if gamepad.just_pressed(action.gamepad_button()) {
304 action_state.pressed_actions.insert(action);
305 }
306 }
307 }
308}17fn gamepad_system(
18 gamepads: Query<(Entity, &Gamepad)>,
19 mut rumble_requests: MessageWriter<GamepadRumbleRequest>,
20) {
21 for (entity, gamepad) in &gamepads {
22 if gamepad.just_pressed(GamepadButton::North) {
23 info!(
24 "North face button: strong (low-frequency) with low intensity for rumble for 5 seconds. Press multiple times to increase intensity."
25 );
26 rumble_requests.write(GamepadRumbleRequest::Add {
27 gamepad: entity,
28 intensity: GamepadRumbleIntensity::strong_motor(0.1),
29 duration: Duration::from_secs(5),
30 });
31 }
32
33 if gamepad.just_pressed(GamepadButton::East) {
34 info!("East face button: maximum rumble on both motors for 5 seconds");
35 rumble_requests.write(GamepadRumbleRequest::Add {
36 gamepad: entity,
37 duration: Duration::from_secs(5),
38 intensity: GamepadRumbleIntensity::MAX,
39 });
40 }
41
42 if gamepad.just_pressed(GamepadButton::South) {
43 info!("South face button: low-intensity rumble on the weak motor for 0.5 seconds");
44 rumble_requests.write(GamepadRumbleRequest::Add {
45 gamepad: entity,
46 duration: Duration::from_secs_f32(0.5),
47 intensity: GamepadRumbleIntensity::weak_motor(0.25),
48 });
49 }
50
51 if gamepad.just_pressed(GamepadButton::West) {
52 info!("West face button: custom rumble intensity for 5 second");
53 rumble_requests.write(GamepadRumbleRequest::Add {
54 gamepad: entity,
55 intensity: GamepadRumbleIntensity {
56 // intensity low-frequency motor, usually on the left-hand side
57 strong_motor: 0.5,
58 // intensity of high-frequency motor, usually on the right-hand side
59 weak_motor: 0.25,
60 },
61 duration: Duration::from_secs(5),
62 });
63 }
64
65 if gamepad.just_pressed(GamepadButton::Start) {
66 info!("Start button: Interrupt the current rumble");
67 rumble_requests.write(GamepadRumbleRequest::Stop { gamepad: entity });
68 }
69 }
70}Sourcepub fn any_just_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_just_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has been pressed during the current frame.
Sourcepub fn all_just_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_just_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have been just pressed.
Sourcepub fn just_released(&self, button_type: GamepadButton) -> bool
pub fn just_released(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been released during the current frame.
Note: This function does not imply information regarding the current state of ButtonInput::pressed or ButtonInput::just_pressed.
Examples found in repository?
12fn gamepad_system(gamepads: Query<(Entity, &Gamepad)>) {
13 for (entity, gamepad) in &gamepads {
14 if gamepad.just_pressed(GamepadButton::South) {
15 info!("{} just pressed South", entity);
16 } else if gamepad.just_released(GamepadButton::South) {
17 info!("{} just released South", entity);
18 }
19
20 let right_trigger = gamepad.get(GamepadButton::RightTrigger2).unwrap();
21 if right_trigger.abs() > 0.01 {
22 info!("{} RightTrigger2 value is {}", entity, right_trigger);
23 }
24
25 let left_stick_x = gamepad.get(GamepadAxis::LeftStickX).unwrap();
26 if left_stick_x.abs() > 0.01 {
27 info!("{} LeftStickX value is {}", entity, left_stick_x);
28 }
29 }
30}Sourcepub fn any_just_released(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_just_released( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has just been released.
Sourcepub fn all_just_released(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_just_released( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have just been released.
Sourcepub fn get_pressed(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_pressed(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that are pressed.
Sourcepub fn get_just_pressed(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_just_pressed(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that were just pressed.
Sourcepub fn get_just_released(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_just_released(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that were just released.
Sourcepub fn get_analog_axes(&self) -> impl Iterator<Item = &GamepadInput>
pub fn get_analog_axes(&self) -> impl Iterator<Item = &GamepadInput>
Returns an iterator over all analog axes.
Sourcepub fn digital(&self) -> &ButtonInput<GamepadButton>
pub fn digital(&self) -> &ButtonInput<GamepadButton>
ButtonInput of GamepadButton representing their digital state.
Sourcepub fn digital_mut(&mut self) -> &mut ButtonInput<GamepadButton>
pub fn digital_mut(&mut self) -> &mut ButtonInput<GamepadButton>
Mutable ButtonInput of GamepadButton representing their digital state. Useful for mocking inputs.
Sourcepub fn analog(&self) -> &Axis<GamepadInput>
pub fn analog(&self) -> &Axis<GamepadInput>
Axis of GamepadButton representing their analog state.
Sourcepub fn analog_mut(&mut self) -> &mut Axis<GamepadInput>
pub fn analog_mut(&mut self) -> &mut Axis<GamepadInput>
Mutable Axis of GamepadButton representing their analog state. Useful for mocking inputs.