pub struct CascadeShadowConfigBuilder {
pub num_cascades: usize,
pub minimum_distance: f32,
pub maximum_distance: f32,
pub first_cascade_far_bound: f32,
pub overlap_proportion: f32,
}Expand description
Builder for CascadeShadowConfig.
Fields
num_cascades: usizeThe number of shadow cascades. More cascades increases shadow quality by mitigating perspective aliasing - a phenomenon where areas nearer the camera are covered by fewer shadow map texels than areas further from the camera, causing blocky looking shadows.
This does come at the cost increased rendering overhead, however this overhead is still less than if you were to use fewer cascades and much larger shadow map textures to achieve the same quality level.
In case rendered geometry covers a relatively narrow and static depth relative to camera, it may
make more sense to use fewer cascades and a higher resolution shadow map texture as perspective aliasing
is not as much an issue. Be sure to adjust minimum_distance and maximum_distance appropriately.
minimum_distance: f32The minimum shadow distance, which can help improve the texel resolution of the first cascade. Areas nearer to the camera than this will likely receive no shadows.
NOTE: Due to implementation details, this usually does not impact shadow quality as much as
first_cascade_far_bound and maximum_distance. At many view frustum field-of-views, the
texel resolution of the first cascade is dominated by the width / height of the view frustum plane
at first_cascade_far_bound rather than the depth of the frustum from minimum_distance to
first_cascade_far_bound.
maximum_distance: f32The maximum shadow distance. Areas further from the camera than this will likely receive no shadows.
first_cascade_far_bound: f32Sets the far bound of the first cascade, relative to the view origin. In-between cascades will be exponentially spaced relative to the maximum shadow distance. NOTE: This is ignored if there is only one cascade, the maximum distance takes precedence.
overlap_proportion: f32Sets the overlap proportion between cascades. The overlap is used to make the transition from one cascade’s shadow map to the next less abrupt by blending between both shadow maps.
Implementations
Sourceimpl CascadeShadowConfigBuilder
impl CascadeShadowConfigBuilder
Sourcepub fn build(&self) -> CascadeShadowConfig
pub fn build(&self) -> CascadeShadowConfig
Returns the cascade config as specified by this builder.
Examples found in repository?
98fn setup_camera_and_environment(
99 mut commands: Commands,
100 mut meshes: ResMut<Assets<Mesh>>,
101 mut materials: ResMut<Assets<StandardMaterial>>,
102) {
103 // Camera
104 commands.spawn((
105 Camera3d::default(),
106 Transform::from_xyz(100.0, 100.0, 150.0).looking_at(Vec3::new(0.0, 20.0, 0.0), Vec3::Y),
107 ));
108
109 // Plane
110 commands.spawn((
111 Mesh3d(meshes.add(Plane3d::default().mesh().size(500000.0, 500000.0))),
112 MeshMaterial3d(materials.add(Color::srgb(0.3, 0.5, 0.3))),
113 ));
114
115 // Light
116 commands.spawn((
117 Transform::from_rotation(Quat::from_euler(EulerRot::ZYX, 0.0, 1.0, -PI / 4.)),
118 DirectionalLight {
119 shadows_enabled: true,
120 ..default()
121 },
122 CascadeShadowConfigBuilder {
123 first_cascade_far_bound: 200.0,
124 maximum_distance: 400.0,
125 ..default()
126 }
127 .build(),
128 ));
129}More examples
89fn spawn_scene(commands: &mut Commands, asset_server: &AssetServer) {
90 // Spawn the main scene.
91 commands.spawn(SceneRoot(asset_server.load(
92 GltfAssetLabel::Scene(0).from_asset("models/TonemappingTest/TonemappingTest.gltf"),
93 )));
94
95 // Spawn the flight helmet.
96 commands.spawn((
97 SceneRoot(
98 asset_server
99 .load(GltfAssetLabel::Scene(0).from_asset("models/FlightHelmet/FlightHelmet.gltf")),
100 ),
101 Transform::from_xyz(0.5, 0.0, -0.5).with_rotation(Quat::from_rotation_y(-0.15 * PI)),
102 ));
103
104 // Spawn the light.
105 commands.spawn((
106 DirectionalLight {
107 illuminance: 15000.0,
108 shadows_enabled: true,
109 ..default()
110 },
111 Transform::from_rotation(Quat::from_euler(EulerRot::ZYX, 0.0, PI * -0.15, PI * -0.15)),
112 CascadeShadowConfigBuilder {
113 maximum_distance: 3.0,
114 first_cascade_far_bound: 0.9,
115 ..default()
116 }
117 .build(),
118 ));
119}344fn add_basic_scene(commands: &mut Commands, asset_server: &AssetServer) {
345 // Spawn the main scene.
346 commands.spawn(SceneRoot(asset_server.load(
347 GltfAssetLabel::Scene(0).from_asset("models/TonemappingTest/TonemappingTest.gltf"),
348 )));
349
350 // Spawn the flight helmet.
351 commands.spawn((
352 SceneRoot(
353 asset_server
354 .load(GltfAssetLabel::Scene(0).from_asset("models/FlightHelmet/FlightHelmet.gltf")),
355 ),
356 Transform::from_xyz(0.5, 0.0, -0.5).with_rotation(Quat::from_rotation_y(-0.15 * PI)),
357 ));
358
359 // Spawn the light.
360 commands.spawn((
361 DirectionalLight {
362 illuminance: 15000.0,
363 shadows_enabled: true,
364 ..default()
365 },
366 Transform::from_rotation(Quat::from_euler(EulerRot::ZYX, 0.0, PI * -0.15, PI * -0.15)),
367 CascadeShadowConfigBuilder {
368 maximum_distance: 3.0,
369 first_cascade_far_bound: 0.9,
370 ..default()
371 }
372 .build(),
373 ));
374}99fn setup_basic_scene(mut commands: Commands, asset_server: Res<AssetServer>) {
100 // Main scene
101 commands.spawn((
102 SceneRoot(asset_server.load(
103 GltfAssetLabel::Scene(0).from_asset("models/TonemappingTest/TonemappingTest.gltf"),
104 )),
105 SceneNumber(1),
106 ));
107
108 // Flight Helmet
109 commands.spawn((
110 SceneRoot(
111 asset_server
112 .load(GltfAssetLabel::Scene(0).from_asset("models/FlightHelmet/FlightHelmet.gltf")),
113 ),
114 Transform::from_xyz(0.5, 0.0, -0.5).with_rotation(Quat::from_rotation_y(-0.15 * PI)),
115 SceneNumber(1),
116 ));
117
118 // light
119 commands.spawn((
120 DirectionalLight {
121 illuminance: 15_000.,
122 shadows_enabled: true,
123 ..default()
124 },
125 Transform::from_rotation(Quat::from_euler(EulerRot::ZYX, 0.0, PI * -0.15, PI * -0.15)),
126 CascadeShadowConfigBuilder {
127 maximum_distance: 3.0,
128 first_cascade_far_bound: 0.9,
129 ..default()
130 }
131 .build(),
132 SceneNumber(1),
133 ));
134}18fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
19 commands.spawn((
20 Camera3d::default(),
21 Transform::from_xyz(0.7, 0.7, 1.0).looking_at(Vec3::new(0.0, 0.3, 0.0), Vec3::Y),
22 EnvironmentMapLight {
23 diffuse_map: asset_server.load("environment_maps/pisa_diffuse_rgb9e5_zstd.ktx2"),
24 specular_map: asset_server.load("environment_maps/pisa_specular_rgb9e5_zstd.ktx2"),
25 intensity: 250.0,
26 ..default()
27 },
28 ));
29
30 commands.spawn((
31 DirectionalLight {
32 shadows_enabled: true,
33 ..default()
34 },
35 // This is a relatively small scene, so use tighter shadow
36 // cascade bounds than the default for better quality.
37 // We also adjusted the shadow map to be larger since we're
38 // only using a single cascade.
39 CascadeShadowConfigBuilder {
40 num_cascades: 1,
41 maximum_distance: 1.6,
42 ..default()
43 }
44 .build(),
45 ));
46 commands.spawn(SceneRoot(asset_server.load(
47 GltfAssetLabel::Scene(0).from_asset("models/FlightHelmet/FlightHelmet.gltf"),
48 )));
49}43fn setup_terrain_scene(
44 mut commands: Commands,
45 mut meshes: ResMut<Assets<Mesh>>,
46 mut materials: ResMut<Assets<StandardMaterial>>,
47 asset_server: Res<AssetServer>,
48) {
49 // Configure a properly scaled cascade shadow map for this scene (defaults are too large, mesh units are in km)
50 let cascade_shadow_config = CascadeShadowConfigBuilder {
51 first_cascade_far_bound: 0.3,
52 maximum_distance: 3.0,
53 ..default()
54 }
55 .build();
56
57 // Sun
58 commands.spawn((
59 DirectionalLight {
60 color: Color::srgb(0.98, 0.95, 0.82),
61 shadows_enabled: true,
62 ..default()
63 },
64 Transform::from_xyz(0.0, 0.0, 0.0).looking_at(Vec3::new(-0.15, -0.05, 0.25), Vec3::Y),
65 cascade_shadow_config,
66 ));
67
68 // Terrain
69 commands.spawn(SceneRoot(asset_server.load(
70 GltfAssetLabel::Scene(0).from_asset("models/terrain/Mountains.gltf"),
71 )));
72
73 // Sky
74 commands.spawn((
75 Mesh3d(meshes.add(Cuboid::new(2.0, 1.0, 1.0))),
76 MeshMaterial3d(materials.add(StandardMaterial {
77 base_color: Srgba::hex("888888").unwrap().into(),
78 unlit: true,
79 cull_mode: None,
80 ..default()
81 })),
82 Transform::from_scale(Vec3::splat(20.0)),
83 NotShadowCaster,
84 ));
85}- examples/animation/animated_mesh_events.rs
- examples/3d/visibility_range.rs
- examples/animation/animated_mesh_control.rs
- examples/3d/atmosphere.rs
- examples/3d/shadow_caster_receiver.rs
- examples/3d/anti_aliasing.rs
- examples/3d/meshlet.rs
- examples/3d/split_screen.rs
- examples/stress_tests/many_foxes.rs
- examples/3d/deferred_rendering.rs
- examples/3d/lighting.rs