pub struct Isometry3d {
pub rotation: Quat,
pub translation: Vec3A,
}Expand description
An isometry in three dimensions, representing a rotation followed by a translation. This can often be useful for expressing relative positions and transformations from one position to another.
In particular, this type represents a distance-preserving transformation known as a rigid motion or a direct motion, and belongs to the special Euclidean group SE(3). This includes translation and rotation, but excludes reflection.
For the two-dimensional version, see Isometry2d.
Example
Isometries can be created from a given translation and rotation:
let iso = Isometry3d::new(Vec3::new(2.0, 1.0, 3.0), Quat::from_rotation_z(FRAC_PI_2));Or from separate parts:
let iso1 = Isometry3d::from_translation(Vec3::new(2.0, 1.0, 3.0));
let iso2 = Isometry3d::from_rotation(Quat::from_rotation_z(FRAC_PI_2));The isometries can be used to transform points:
let iso = Isometry3d::new(Vec3::new(2.0, 1.0, 3.0), Quat::from_rotation_z(FRAC_PI_2));
let point = Vec3::new(4.0, 4.0, 4.0);
// These are equivalent
let result = iso.transform_point(point);
let result = iso * point;
assert_relative_eq!(result, Vec3::new(-2.0, 5.0, 7.0));Isometries can also be composed together:
assert_eq!(iso1 * iso2, iso);One common operation is to compute an isometry representing the relative positions of two objects for things like intersection tests. This can be done with an inverse transformation:
let sphere_iso = Isometry3d::from_translation(Vec3::new(2.0, 1.0, 3.0));
let cuboid_iso = Isometry3d::from_rotation(Quat::from_rotation_z(FRAC_PI_2));
// Compute the relative position and orientation between the two shapes
let relative_iso = sphere_iso.inverse() * cuboid_iso;
// Or alternatively, to skip an extra rotation operation:
let relative_iso = sphere_iso.inverse_mul(cuboid_iso);Fields
rotation: QuatThe rotational part of a three-dimensional isometry.
translation: Vec3AThe translational part of a three-dimensional isometry.
Implementations
Sourceimpl Isometry3d
impl Isometry3d
Sourcepub const IDENTITY: Isometry3d
pub const IDENTITY: Isometry3d
The identity isometry which represents the rigid motion of not doing anything.
Sourcepub fn new(translation: impl Into<Vec3A>, rotation: Quat) -> Isometry3d
pub fn new(translation: impl Into<Vec3A>, rotation: Quat) -> Isometry3d
Create a three-dimensional isometry from a rotation and a translation.
Examples found in repository?
25fn hello_world(mut text_gizmos: Gizmos, time: Res<Time>) {
26 let t = 0.2 * time.elapsed_secs();
27
28 text_gizmos.text(
29 Isometry3d::new(Vec3::new(0.0, 1.5, 0.0), Quat::from_rotation_y(-t)),
30 "Hello",
31 1.,
32 Vec2::ZERO,
33 RED,
34 );
35
36 text_gizmos.text(
37 Isometry3d::new(Vec3::new(0.0, 0.0, 0.0), Quat::from_rotation_y(t + 0.25)),
38 "Text",
39 1.,
40 Vec2::ZERO,
41 ORANGE,
42 );
43
44 text_gizmos.text(
45 Isometry3d::new(Vec3::new(0.0, -1.5, 0.0), Quat::from_rotation_y(-t - 0.5)),
46 "Gizmos",
47 1.,
48 Vec2::ZERO,
49 YELLOW,
50 );
51}More examples
346fn draw_camera_gizmo(cameras: Query<(&Camera, &GlobalTransform)>, mut gizmos: Gizmos) {
347 for (camera, transform) in &cameras {
348 // As above, we use the order as a cheap tag to tell the depth texture
349 // apart from the main texture.
350 if camera.order >= 0 {
351 continue;
352 }
353
354 // Draw a cone representing the camera.
355 gizmos.primitive_3d(
356 &Cone {
357 radius: 1.0,
358 height: 3.0,
359 },
360 Isometry3d::new(
361 transform.translation(),
362 // We have to rotate here because `Cone` primitives are oriented
363 // along +Y and cameras point along +Z.
364 transform.rotation() * Quat::from_rotation_x(FRAC_PI_2),
365 ),
366 LIME,
367 );
368 }
369}13fn draw_cursor(
14 camera_query: Single<(&Camera, &GlobalTransform)>,
15 ground: Single<&GlobalTransform, With<Ground>>,
16 window: Single<&Window>,
17 mut gizmos: Gizmos,
18) {
19 let (camera, camera_transform) = *camera_query;
20
21 if let Some(cursor_position) = window.cursor_position()
22 // Calculate a ray pointing from the camera into the world based on the cursor's position.
23 && let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
24 // Calculate if and where the ray is hitting the ground plane.
25 && let Some(point) = ray.plane_intersection_point(ground.translation(), InfinitePlane3d::new(ground.up()))
26 {
27 // Draw a circle just above the ground plane at that position.
28 gizmos.circle(
29 Isometry3d::new(
30 point + ground.up() * 0.01,
31 Quat::from_rotation_arc(Vec3::Z, ground.up().as_vec3()),
32 ),
33 0.2,
34 Color::WHITE,
35 );
36 }
37}673fn draw_gizmos_3d(mut gizmos: Gizmos, state: Res<State<PrimitiveSelected>>, time: Res<Time>) {
674 const POSITION: Vec3 = Vec3::new(LEFT_RIGHT_OFFSET_3D, 0.0, 0.0);
675 let rotation = Quat::from_rotation_arc(
676 Vec3::Z,
677 Vec3::new(
678 ops::sin(time.elapsed_secs()),
679 ops::cos(time.elapsed_secs()),
680 ops::sin(time.elapsed_secs()) * 0.5,
681 )
682 .try_normalize()
683 .unwrap_or(Vec3::Z),
684 );
685 let isometry = Isometry3d::new(POSITION, rotation);
686 let color = Color::WHITE;
687 let resolution = 10;
688
689 #[expect(
690 clippy::match_same_arms,
691 reason = "Certain primitives don't have any 3D rendering support yet."
692 )]
693 match state.get() {
694 PrimitiveSelected::RectangleAndCuboid => {
695 gizmos.primitive_3d(&CUBOID, isometry, color);
696 }
697 PrimitiveSelected::CircleAndSphere => drop(
698 gizmos
699 .primitive_3d(&SPHERE, isometry, color)
700 .resolution(resolution),
701 ),
702 PrimitiveSelected::Ellipse => {}
703 PrimitiveSelected::Triangle => gizmos.primitive_3d(&TRIANGLE_3D, isometry, color),
704 PrimitiveSelected::Plane => drop(gizmos.primitive_3d(&PLANE_3D, isometry, color)),
705 PrimitiveSelected::Line => gizmos.primitive_3d(&LINE_3D, isometry, color),
706 PrimitiveSelected::Segment => gizmos.primitive_3d(&SEGMENT_3D, isometry, color),
707 PrimitiveSelected::Polyline => gizmos.primitive_3d(
708 &Polyline3d {
709 vertices: POLYLINE_3D_VERTICES.to_vec(),
710 },
711 isometry,
712 color,
713 ),
714 PrimitiveSelected::Polygon => {}
715 PrimitiveSelected::ConvexPolygon => {}
716 PrimitiveSelected::RegularPolygon => {}
717 PrimitiveSelected::Capsule => drop(
718 gizmos
719 .primitive_3d(&CAPSULE_3D, isometry, color)
720 .resolution(resolution),
721 ),
722 PrimitiveSelected::Cylinder => drop(
723 gizmos
724 .primitive_3d(&CYLINDER, isometry, color)
725 .resolution(resolution),
726 ),
727 PrimitiveSelected::Cone => drop(
728 gizmos
729 .primitive_3d(&CONE, isometry, color)
730 .resolution(resolution),
731 ),
732 PrimitiveSelected::ConicalFrustum => {
733 gizmos.primitive_3d(&CONICAL_FRUSTUM, isometry, color);
734 }
735
736 PrimitiveSelected::Torus => drop(
737 gizmos
738 .primitive_3d(&TORUS, isometry, color)
739 .minor_resolution(resolution)
740 .major_resolution(resolution),
741 ),
742 PrimitiveSelected::Tetrahedron => {
743 gizmos.primitive_3d(&TETRAHEDRON, isometry, color);
744 }
745
746 PrimitiveSelected::Arc => {}
747 PrimitiveSelected::CircularSector => {}
748 PrimitiveSelected::CircularSegment => {}
749 }
750}99fn draw_example_collection(
100 mut gizmos: Gizmos,
101 mut my_gizmos: Gizmos<MyRoundGizmos>,
102 time: Res<Time>,
103) {
104 gizmos.grid(
105 Quat::from_rotation_x(PI / 2.),
106 UVec2::splat(20),
107 Vec2::new(2., 2.),
108 // Light gray
109 LinearRgba::gray(0.65),
110 );
111 gizmos.grid(
112 Isometry3d::new(Vec3::splat(10.0), Quat::from_rotation_x(PI / 3. * 2.)),
113 UVec2::splat(20),
114 Vec2::new(2., 2.),
115 PURPLE,
116 );
117 gizmos.sphere(Vec3::splat(10.0), 1.0, PURPLE);
118
119 gizmos
120 .primitive_3d(
121 &Plane3d {
122 normal: Dir3::Y,
123 half_size: Vec2::splat(1.0),
124 },
125 Isometry3d::new(
126 Vec3::splat(4.0) + Vec2::from(ops::sin_cos(time.elapsed_secs())).extend(0.0),
127 Quat::from_rotation_x(PI / 2. + time.elapsed_secs()),
128 ),
129 GREEN,
130 )
131 .cell_count(UVec2::new(5, 10))
132 .spacing(Vec2::new(0.2, 0.1));
133
134 gizmos.cube(
135 Transform::from_translation(Vec3::Y * 0.5).with_scale(Vec3::splat(1.25)),
136 BLACK,
137 );
138 gizmos.rect(
139 Isometry3d::new(
140 Vec3::new(ops::cos(time.elapsed_secs()) * 2.5, 1., 0.),
141 Quat::from_rotation_y(PI / 2.),
142 ),
143 Vec2::splat(2.),
144 LIME,
145 );
146
147 gizmos.cross(Vec3::new(-1., 1., 1.), 0.5, FUCHSIA);
148
149 let domain = Interval::EVERYWHERE;
150 let curve = FunctionCurve::new(domain, |t| {
151 (Vec2::from(ops::sin_cos(t * 10.0))).extend(t - 6.0)
152 });
153 let resolution = ((ops::sin(time.elapsed_secs()) + 1.0) * 100.0) as usize;
154 let times_and_colors = (0..=resolution)
155 .map(|n| n as f32 / resolution as f32)
156 .map(|t| t * 5.0)
157 .map(|t| (t, TEAL.mix(&HOT_PINK, t / 5.0)));
158 gizmos.curve_gradient_3d(curve, times_and_colors);
159
160 my_gizmos.sphere(Vec3::new(1., 0.5, 0.), 0.5, RED);
161
162 my_gizmos
163 .rounded_cuboid(Vec3::new(-2.0, 0.75, -0.75), Vec3::splat(0.9), TURQUOISE)
164 .edge_radius(0.1)
165 .arc_resolution(4);
166
167 for y in [0., 0.5, 1.] {
168 gizmos.ray(
169 Vec3::new(1., y, 0.),
170 Vec3::new(-3., ops::sin(time.elapsed_secs() * 3.), 0.),
171 BLUE,
172 );
173 }
174
175 my_gizmos
176 .arc_3d(
177 180.0_f32.to_radians(),
178 0.2,
179 Isometry3d::new(
180 Vec3::ONE,
181 Quat::from_rotation_arc(Vec3::Y, Vec3::ONE.normalize()),
182 ),
183 ORANGE,
184 )
185 .resolution(10);
186
187 // Circles have 32 line-segments by default.
188 my_gizmos.circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3., BLACK);
189
190 // You may want to increase this for larger circles or spheres.
191 my_gizmos
192 .circle(Quat::from_rotation_arc(Vec3::Z, Vec3::Y), 3.1, NAVY)
193 .resolution(64);
194 my_gizmos
195 .sphere(Isometry3d::IDENTITY, 3.2, BLACK)
196 .resolution(64);
197
198 gizmos.arrow(Vec3::ZERO, Vec3::splat(1.5), YELLOW);
199
200 // You can create more complex arrows using the arrow builder.
201 gizmos
202 .arrow(Vec3::new(2., 0., 2.), Vec3::new(2., 2., 2.), ORANGE_RED)
203 .with_double_end()
204 .with_tip_length(0.5);
205}Sourcepub const fn from_rotation(rotation: Quat) -> Isometry3d
pub const fn from_rotation(rotation: Quat) -> Isometry3d
Create a three-dimensional isometry from a rotation.
Sourcepub fn from_translation(translation: impl Into<Vec3A>) -> Isometry3d
pub fn from_translation(translation: impl Into<Vec3A>) -> Isometry3d
Create a three-dimensional isometry from a translation.
Examples found in repository?
379 pub fn draw_gizmos(mut gizmos: Gizmos) {
380 gizmos.cube(
381 Transform::from_translation(Vec3::X * -1.75).with_scale(Vec3::splat(1.25)),
382 RED,
383 );
384 gizmos
385 .sphere(Isometry3d::from_translation(Vec3::X * -3.5), 0.75, GREEN)
386 .resolution(30_000 / 3);
387
388 gizmos.text(
389 Isometry3d::from_translation(Vec3::Y * 1.5),
390 "text gizmo",
391 0.3,
392 Vec2 { x: 0., y: 0. },
393 Color::WHITE,
394 );
395
396 // 3d grids with all variations of outer edges on or off
397 for i in 0..8 {
398 let x = 1.5 * (i % 4) as f32;
399 let y = 1.0 * (0.5 - (i / 4) as f32);
400 let mut grid = gizmos.grid_3d(
401 Isometry3d::from_translation(Vec3::new(x, y, 0.0)),
402 UVec3::new(5, 4, 3),
403 Vec3::splat(0.175),
404 Color::WHITE,
405 );
406 if i & 1 > 0 {
407 grid = grid.outer_edges_x();
408 }
409 if i & 2 > 0 {
410 grid = grid.outer_edges_y();
411 }
412 if i & 4 > 0 {
413 grid.outer_edges_z();
414 }
415 }
416 }Sourcepub const fn from_xyz(x: f32, y: f32, z: f32) -> Isometry3d
pub const fn from_xyz(x: f32, y: f32, z: f32) -> Isometry3d
Create a three-dimensional isometry from a translation with the given x, y, and z components.
Sourcepub fn inverse(&self) -> Isometry3d
pub fn inverse(&self) -> Isometry3d
The inverse isometry that undoes this one.
Sourcepub fn inverse_mul(&self, rhs: Isometry3d) -> Isometry3d
pub fn inverse_mul(&self, rhs: Isometry3d) -> Isometry3d
Compute iso1.inverse() * iso2 in a more efficient way for one-shot cases.
If the same isometry is used multiple times, it is more efficient to instead compute the inverse once and use that for each transformation.
Sourcepub fn transform_point(&self, point: impl Into<Vec3A>) -> Vec3A
pub fn transform_point(&self, point: impl Into<Vec3A>) -> Vec3A
Transform a point by rotating and translating it using this isometry.
Sourcepub fn inverse_transform_point(&self, point: impl Into<Vec3A>) -> Vec3A
pub fn inverse_transform_point(&self, point: impl Into<Vec3A>) -> Vec3A
Transform a point by rotating and translating it using the inverse of this isometry.
This is more efficient than iso.inverse().transform_point(point) for one-shot cases.
If the same isometry is used multiple times, it is more efficient to instead compute
the inverse once and use that for each transformation.