UVec2

Struct UVec2 

#[repr(C)]
pub struct UVec2 { pub x: u32, pub y: u32, }
A 2-dimensional vector.

Fields

x: u32y: u32

Implementations

Source

impl UVec2

pub const ZERO: UVec2

All zeroes.

pub const ONE: UVec2

All ones.

pub const MIN: UVec2

All u32::MIN.

pub const MAX: UVec2

All u32::MAX.

pub const X: UVec2

A unit vector pointing along the positive X axis.

pub const Y: UVec2

A unit vector pointing along the positive Y axis.

pub const AXES: [UVec2; 2]

The unit axes.

pub const fn new(x: u32, y: u32) -> UVec2

Creates a new vector.

Examples found in repository?
examples/shader/compute_shader_game_of_life.rs (line 29)
29const SIZE: UVec2 = UVec2::new(1280 / DISPLAY_FACTOR, 720 / DISPLAY_FACTOR);
More examples
Hide additional examples
examples/window/custom_cursor_image.rs (line 201)
195    const SECTIONS: [Option<URect>; 5] = [
196        Some(URect {
197            min: UVec2::ZERO,
198            max: UVec2::splat(RECT_SIZE),
199        }),
200        Some(URect {
201            min: UVec2::new(RECT_SIZE, 0),
202            max: UVec2::new(2 * RECT_SIZE, RECT_SIZE),
203        }),
204        Some(URect {
205            min: UVec2::new(0, RECT_SIZE),
206            max: UVec2::new(RECT_SIZE, 2 * RECT_SIZE),
207        }),
208        Some(URect {
209            min: UVec2::new(RECT_SIZE, RECT_SIZE),
210            max: UVec2::splat(2 * RECT_SIZE),
211        }),
212        None, // reset to None
213    ];
examples/2d/tilemap_chunk.rs (line 71)
65fn spawn_fake_player(
66    mut commands: Commands,
67    mut meshes: ResMut<Assets<Mesh>>,
68    mut materials: ResMut<Assets<ColorMaterial>>,
69    chunk: Single<&TilemapChunk>,
70) {
71    let mut transform = chunk.calculate_tile_transform(UVec2::new(0, 0));
72    transform.translation.z = 1.;
73
74    commands.spawn((
75        Mesh2d(meshes.add(Rectangle::new(8., 8.))),
76        MeshMaterial2d(materials.add(Color::from(RED_400))),
77        transform,
78        MovePlayer,
79    ));
80
81    let mut transform = chunk.calculate_tile_transform(UVec2::new(5, 6));
82    transform.translation.z = 1.;
83
84    // second "player" to visually test a non-zero position
85    commands.spawn((
86        Mesh2d(meshes.add(Rectangle::new(8., 8.))),
87        MeshMaterial2d(materials.add(Color::from(RED_400))),
88        transform,
89    ));
90}
91
92fn move_player(
93    mut player: Single<&mut Transform, With<MovePlayer>>,
94    time: Res<Time>,
95    chunk: Single<&TilemapChunk>,
96) {
97    let t = (ops::sin(time.elapsed_secs()) + 1.) / 2.;
98
99    let origin = chunk
100        .calculate_tile_transform(UVec2::new(0, 0))
101        .translation
102        .x;
103    let destination = chunk
104        .calculate_tile_transform(UVec2::new(63, 0))
105        .translation
106        .x;
107
108    player.translation.x = origin.lerp(destination, t);
109}
110
111fn update_tileset_image(
112    chunk_query: Single<&TilemapChunk>,
113    mut events: MessageReader<AssetEvent<Image>>,
114    mut images: ResMut<Assets<Image>>,
115) {
116    let chunk = *chunk_query;
117    for event in events.read() {
118        if event.is_loaded_with_dependencies(chunk.tileset.id()) {
119            let image = images.get_mut(&chunk.tileset).unwrap();
120            image.reinterpret_stacked_2d_as_array(4);
121        }
122    }
123}
124
125fn update_tilemap(
126    time: Res<Time>,
127    mut query: Query<(&mut TilemapChunkTileData, &mut UpdateTimer)>,
128    mut rng: ResMut<SeededRng>,
129) {
130    for (mut tile_data, mut timer) in query.iter_mut() {
131        timer.tick(time.delta());
132
133        if timer.just_finished() {
134            for _ in 0..50 {
135                let index = rng.random_range(0..tile_data.len());
136                tile_data[index] = Some(TileData::from_tileset_index(rng.random_range(0..5)));
137            }
138        }
139    }
140}
141
142// find the data for an arbitrary tile in the chunk and log its data
143fn log_tile(tilemap: Single<(&TilemapChunk, &TilemapChunkTileData)>, mut local: Local<u16>) {
144    let (chunk, data) = tilemap.into_inner();
145    let Some(tile_data) = data.tile_data_from_tile_pos(chunk.chunk_size, UVec2::new(3, 4)) else {
146        return;
147    };
148    // log when the tile changes
149    if tile_data.tileset_index != *local {
150        info!(?tile_data, "tile_data changed");
151        *local = tile_data.tileset_index;
152    }
153}
examples/testbed/2d.rs (line 310)
290    pub fn draw_gizmos(mut gizmos: Gizmos) {
291        gizmos.rect_2d(
292            Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
293            Vec2::new(200.0, 200.0),
294            RED,
295        );
296        gizmos
297            .circle_2d(
298                Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
299                200.0,
300                GREEN,
301            )
302            .resolution(64);
303
304        // 2d grids with all variations of outer edges on or off
305        for i in 0..4 {
306            let x = 200.0 * (1.0 + (i % 2) as f32);
307            let y = 150.0 * (0.5 - (i / 2) as f32);
308            let mut grid = gizmos.grid(
309                Vec3::new(x, y, 0.0),
310                UVec2::new(5, 4),
311                Vec2::splat(30.),
312                Color::WHITE,
313            );
314            if i & 1 > 0 {
315                grid = grid.outer_edges_x();
316            }
317            if i & 2 > 0 {
318                grid.outer_edges_y();
319            }
320        }
321    }
examples/picking/sprite_picking.rs (line 128)
122fn setup_atlas(
123    mut commands: Commands,
124    asset_server: Res<AssetServer>,
125    mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
126) {
127    let texture_handle = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
128    let layout = TextureAtlasLayout::from_grid(UVec2::new(24, 24), 7, 1, None, None);
129    let texture_atlas_layout_handle = texture_atlas_layouts.add(layout);
130    // Use only the subset of sprites in the sheet that make up the run animation
131    let animation_indices = AnimationIndices { first: 1, last: 6 };
132    commands
133        .spawn((
134            Sprite::from_atlas_image(
135                texture_handle,
136                TextureAtlas {
137                    layout: texture_atlas_layout_handle,
138                    index: animation_indices.first,
139                },
140            ),
141            Transform::from_xyz(300.0, 0.0, 0.0).with_scale(Vec3::splat(6.0)),
142            animation_indices,
143            AnimationTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
144            Pickable::default(),
145        ))
146        .observe(recolor_on::<Pointer<Over>>(Color::srgb(0.0, 1.0, 1.0)))
147        .observe(recolor_on::<Pointer<Out>>(Color::srgb(1.0, 1.0, 1.0)))
148        .observe(recolor_on::<Pointer<Press>>(Color::srgb(1.0, 1.0, 0.0)))
149        .observe(recolor_on::<Pointer<Release>>(Color::srgb(0.0, 1.0, 1.0)));
150}
examples/shader/automatic_instancing.rs (line 46)
26fn setup(
27    mut commands: Commands,
28    assets: Res<AssetServer>,
29    mut meshes: ResMut<Assets<Mesh>>,
30    mut materials: ResMut<Assets<CustomMaterial>>,
31) {
32    // We will use this image as our external data for our material to sample from in the vertex shader
33    let image = assets.load("branding/icon.png");
34
35    // Our single mesh handle that will be instanced
36    let mesh_handle = meshes.add(Cuboid::from_size(Vec3::splat(0.01)));
37
38    // Create the custom material with a reference to our texture
39    // Automatic instancing works with any Material, including the `StandardMaterial`.
40    // This custom material is used to demonstrate the optional `MeshTag` feature.
41    let material_handle = materials.add(CustomMaterial {
42        image: image.clone(),
43    });
44
45    // We're hardcoding the image dimensions for simplicity
46    let image_dims = UVec2::new(256, 256);
47    let total_pixels = image_dims.x * image_dims.y;
48
49    for index in 0..total_pixels {
50        // Get x,y from index - x goes left to right, y goes top to bottom
51        let x = index % image_dims.x;
52        let y = index / image_dims.x;
53
54        // Convert to centered world coordinates
55        let world_x = (x as f32 - image_dims.x as f32 / 2.0) / 50.0;
56        let world_y = -((y as f32 - image_dims.y as f32 / 2.0) / 50.0); // Still need negative for world space
57
58        commands.spawn((
59            // For automatic instancing to take effect you need to
60            // use the same mesh handle and material handle for each instance
61            Mesh3d(mesh_handle.clone()),
62            MeshMaterial3d(material_handle.clone()),
63            // This is an optional component that can be used to help tie external data to a mesh instance
64            MeshTag(index),
65            Transform::from_xyz(world_x, world_y, 0.0),
66        ));
67    }
68
69    // Camera
70    commands.spawn((
71        Camera3d::default(),
72        Transform::from_xyz(0.0, 0.0, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
73    ));
74}
pub const fn splat(v: u32) -> UVec2

Creates a vector with all elements set to v.

Examples found in repository?
examples/2d/sprite_sheet.rs (line 48)
42fn setup(
43    mut commands: Commands,
44    asset_server: Res<AssetServer>,
45    mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
46) {
47    let texture = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
48    let layout = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
49    let texture_atlas_layout = texture_atlas_layouts.add(layout);
50    // Use only the subset of sprites in the sheet that make up the run animation
51    let animation_indices = AnimationIndices { first: 1, last: 6 };
52
53    commands.spawn(Camera2d);
54
55    commands.spawn((
56        Sprite::from_atlas_image(
57            texture,
58            TextureAtlas {
59                layout: texture_atlas_layout,
60                index: animation_indices.first,
61            },
62        ),
63        Transform::from_scale(Vec3::splat(6.0)),
64        animation_indices,
65        AnimationTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
66    ));
67}
More examples
Hide additional examples
examples/2d/tilemap_chunk.rs (line 33)
28fn setup(mut commands: Commands, assets: Res<AssetServer>) {
29    // We're seeding the PRNG here to make this example deterministic for testing purposes.
30    // This isn't strictly required in practical use unless you need your app to be deterministic.
31    let mut rng = ChaCha8Rng::seed_from_u64(42);
32
33    let chunk_size = UVec2::splat(64);
34    let tile_display_size = UVec2::splat(8);
35    let tile_data: Vec<Option<TileData>> = (0..chunk_size.element_product())
36        .map(|_| rng.random_range(0..5))
37        .map(|i| {
38            if i == 0 {
39                None
40            } else {
41                Some(TileData::from_tileset_index(i - 1))
42            }
43        })
44        .collect();
45
46    commands.spawn((
47        TilemapChunk {
48            chunk_size,
49            tile_display_size,
50            tileset: assets.load("textures/array_texture.png"),
51            ..default()
52        },
53        TilemapChunkTileData(tile_data),
54        UpdateTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
55    ));
56
57    commands.spawn(Camera2d);
58
59    commands.insert_resource(SeededRng(rng));
60}
examples/shader_advanced/render_depth_to_texture.rs (line 229)
219fn spawn_depth_only_camera(commands: &mut Commands) {
220    commands.spawn((
221        Camera3d::default(),
222        Transform::from_xyz(-4.0, -5.0, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
223        Camera {
224            // We specify no color render target, for maximum efficiency.
225            target: RenderTarget::None {
226                // When specifying no render target, we must manually specify
227                // the viewport size. Otherwise, Bevy won't know how big to make
228                // the depth buffer.
229                size: UVec2::splat(DEPTH_TEXTURE_SIZE),
230            },
231            // Make sure that we render from this depth-only camera *before*
232            // rendering from the main camera.
233            order: -1,
234            ..Camera::default()
235        },
236        // We need to disable multisampling or the depth texture will be
237        // multisampled, which adds complexity we don't care about for this
238        // demo.
239        Msaa::Off,
240        // Cameras with no render target render *nothing* by default. To get
241        // them to render something, we must add a prepass that specifies what
242        // we want to render: in this case, depth.
243        DepthPrepass,
244    ));
245}
examples/window/custom_cursor_image.rs (line 38)
31fn setup_cursor_icon(
32    mut commands: Commands,
33    asset_server: Res<AssetServer>,
34    mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
35    window: Single<Entity, With<Window>>,
36) {
37    let layout =
38        TextureAtlasLayout::from_grid(UVec2::splat(64), 20, 10, Some(UVec2::splat(5)), None);
39    let texture_atlas_layout = texture_atlas_layouts.add(layout);
40
41    let animation_config = AnimationConfig::new(0, 199, 1, 4);
42
43    commands.entity(*window).insert((
44        CursorIcon::Custom(CustomCursor::Image(CustomCursorImage {
45            // Image to use as the cursor.
46            handle: asset_server
47                .load("cursors/kenney_crosshairPack/Tilesheet/crosshairs_tilesheet_white.png"),
48            // Optional texture atlas allows you to pick a section of the image
49            // and animate it.
50            texture_atlas: Some(TextureAtlas {
51                layout: texture_atlas_layout.clone(),
52                index: animation_config.first_sprite_index,
53            }),
54            flip_x: false,
55            flip_y: false,
56            // Optional section of the image to use as the cursor.
57            rect: None,
58            // The hotspot is the point in the cursor image that will be
59            // positioned at the mouse cursor's position.
60            hotspot: (0, 0),
61        })),
62        animation_config,
63    ));
64}
65
66fn setup_camera(mut commands: Commands) {
67    commands.spawn(Camera3d::default());
68}
69
70fn setup_instructions(mut commands: Commands) {
71    commands.spawn((
72        Text::new(
73            "Press T to toggle the cursor's `texture_atlas`.\n
74Press X to toggle the cursor's `flip_x` setting.\n
75Press Y to toggle the cursor's `flip_y` setting.\n
76Press C to cycle through the sections of the cursor's image using `rect`.",
77        ),
78        Node {
79            position_type: PositionType::Absolute,
80            bottom: px(12),
81            left: px(12),
82            ..default()
83        },
84    ));
85}
86
87#[derive(Component)]
88struct AnimationConfig {
89    first_sprite_index: usize,
90    last_sprite_index: usize,
91    increment: usize,
92    fps: u8,
93    frame_timer: Timer,
94}
95
96impl AnimationConfig {
97    fn new(first: usize, last: usize, increment: usize, fps: u8) -> Self {
98        Self {
99            first_sprite_index: first,
100            last_sprite_index: last,
101            increment,
102            fps,
103            frame_timer: Self::timer_from_fps(fps),
104        }
105    }
106
107    fn timer_from_fps(fps: u8) -> Timer {
108        Timer::new(Duration::from_secs_f32(1.0 / (fps as f32)), TimerMode::Once)
109    }
110}
111
112/// This system loops through all the sprites in the [`CursorIcon`]'s
113/// [`TextureAtlas`], from [`AnimationConfig`]'s `first_sprite_index` to
114/// `last_sprite_index`.
115fn execute_animation(time: Res<Time>, mut query: Query<(&mut AnimationConfig, &mut CursorIcon)>) {
116    for (mut config, mut cursor_icon) in &mut query {
117        if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
118            config.frame_timer.tick(time.delta());
119
120            if config.frame_timer.is_finished()
121                && let Some(atlas) = image.texture_atlas.as_mut()
122            {
123                atlas.index += config.increment;
124
125                if atlas.index > config.last_sprite_index {
126                    atlas.index = config.first_sprite_index;
127                }
128
129                config.frame_timer = AnimationConfig::timer_from_fps(config.fps);
130            }
131        }
132    }
133}
134
135fn toggle_texture_atlas(
136    input: Res<ButtonInput<KeyCode>>,
137    mut query: Query<&mut CursorIcon, With<Window>>,
138    mut cached_atlas: Local<Option<TextureAtlas>>, // this lets us restore the previous value
139) {
140    if input.just_pressed(KeyCode::KeyT) {
141        for mut cursor_icon in &mut query {
142            if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
143                match image.texture_atlas.take() {
144                    Some(a) => {
145                        // Save the current texture atlas.
146                        *cached_atlas = Some(a.clone());
147                    }
148                    None => {
149                        // Restore the cached texture atlas.
150                        if let Some(cached_a) = cached_atlas.take() {
151                            image.texture_atlas = Some(cached_a);
152                        }
153                    }
154                }
155            }
156        }
157    }
158}
159
160fn toggle_flip_x(
161    input: Res<ButtonInput<KeyCode>>,
162    mut query: Query<&mut CursorIcon, With<Window>>,
163) {
164    if input.just_pressed(KeyCode::KeyX) {
165        for mut cursor_icon in &mut query {
166            if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
167                image.flip_x = !image.flip_x;
168            }
169        }
170    }
171}
172
173fn toggle_flip_y(
174    input: Res<ButtonInput<KeyCode>>,
175    mut query: Query<&mut CursorIcon, With<Window>>,
176) {
177    if input.just_pressed(KeyCode::KeyY) {
178        for mut cursor_icon in &mut query {
179            if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
180                image.flip_y = !image.flip_y;
181            }
182        }
183    }
184}
185
186/// This system alternates the [`CursorIcon`]'s `rect` field between `None` and
187/// 4 sections/rectangles of the cursor's image.
188fn cycle_rect(input: Res<ButtonInput<KeyCode>>, mut query: Query<&mut CursorIcon, With<Window>>) {
189    if !input.just_pressed(KeyCode::KeyC) {
190        return;
191    }
192
193    const RECT_SIZE: u32 = 32; // half the size of a tile in the texture atlas
194
195    const SECTIONS: [Option<URect>; 5] = [
196        Some(URect {
197            min: UVec2::ZERO,
198            max: UVec2::splat(RECT_SIZE),
199        }),
200        Some(URect {
201            min: UVec2::new(RECT_SIZE, 0),
202            max: UVec2::new(2 * RECT_SIZE, RECT_SIZE),
203        }),
204        Some(URect {
205            min: UVec2::new(0, RECT_SIZE),
206            max: UVec2::new(RECT_SIZE, 2 * RECT_SIZE),
207        }),
208        Some(URect {
209            min: UVec2::new(RECT_SIZE, RECT_SIZE),
210            max: UVec2::splat(2 * RECT_SIZE),
211        }),
212        None, // reset to None
213    ];
examples/ui/ui_texture_atlas.rs (line 29)
18fn setup(
19    mut commands: Commands,
20    asset_server: Res<AssetServer>,
21    mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
22) {
23    // Camera
24    commands.spawn(Camera2d);
25
26    let text_font = TextFont::default();
27
28    let texture_handle = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
29    let texture_atlas = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
30    let texture_atlas_handle = texture_atlases.add(texture_atlas);
31
32    // root node
33    commands
34        .spawn(Node {
35            width: percent(100),
36            height: percent(100),
37            flex_direction: FlexDirection::Column,
38            justify_content: JustifyContent::Center,
39            align_items: AlignItems::Center,
40            row_gap: px(text_font.font_size * 2.),
41            ..default()
42        })
43        .with_children(|parent| {
44            parent.spawn((
45                ImageNode::from_atlas_image(
46                    texture_handle,
47                    TextureAtlas::from(texture_atlas_handle),
48                ),
49                Node {
50                    width: px(256),
51                    height: px(256),
52                    ..default()
53                },
54                BackgroundColor(ANTIQUE_WHITE.into()),
55                Outline::new(px(8), Val::ZERO, CRIMSON.into()),
56            ));
57            parent
58                .spawn((Text::new("press "), text_font.clone()))
59                .with_child((
60                    TextSpan::new("space"),
61                    TextColor(YELLOW.into()),
62                    text_font.clone(),
63                ))
64                .with_child((TextSpan::new(" to advance frames"), text_font));
65        });
66}
examples/usage/cooldown.rs (line 29)
22fn setup(
23    mut commands: Commands,
24    mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
25    asset_server: Res<AssetServer>,
26) {
27    commands.spawn(Camera2d);
28    let texture = asset_server.load("textures/food_kenney.png");
29    let layout = TextureAtlasLayout::from_grid(UVec2::splat(64), 7, 7, None, None);
30    let texture_atlas_layout = texture_atlas_layouts.add(layout);
31    commands.spawn((
32        Node {
33            width: percent(100),
34            height: percent(100),
35            align_items: AlignItems::Center,
36            justify_content: JustifyContent::Center,
37            column_gap: px(15),
38            ..default()
39        },
40        Children::spawn(SpawnIter(
41            [
42                FoodItem {
43                    name: "an apple",
44                    cooldown: 2.,
45                    index: 2,
46                },
47                FoodItem {
48                    name: "a burger",
49                    cooldown: 1.,
50                    index: 23,
51                },
52                FoodItem {
53                    name: "chocolate",
54                    cooldown: 10.,
55                    index: 32,
56                },
57                FoodItem {
58                    name: "cherries",
59                    cooldown: 4.,
60                    index: 41,
61                },
62            ]
63            .into_iter()
64            .map(move |food| build_ability(food, texture.clone(), texture_atlas_layout.clone())),
65        )),
66    ));
67    commands.spawn((
68        Text::new("*Click some food to eat it*"),
69        Node {
70            position_type: PositionType::Absolute,
71            top: px(12),
72            left: px(12),
73            ..default()
74        },
75    ));
76}
pub fn map<F>(self, f: F) -> UVec2
where F: Fn(u32) -> u32,

Returns a vector containing each element of self modified by a mapping function f.

pub fn select(mask: BVec2, if_true: UVec2, if_false: UVec2) -> UVec2

Creates a vector from the elements in if_true and if_false, selecting which to use for each element of self.

A true element in the mask uses the corresponding element from if_true, and false uses the element from if_false.

pub const fn from_array(a: [u32; 2]) -> UVec2

Creates a new vector from an array.

pub const fn to_array(&self) -> [u32; 2]

Converts self to [x, y]

pub const fn from_slice(slice: &[u32]) -> UVec2

Creates a vector from the first 2 values in slice.

Panics

Panics if slice is less than 2 elements long.

pub fn write_to_slice(self, slice: &mut [u32])

Writes the elements of self to the first 2 elements in slice.

Panics

Panics if slice is less than 2 elements long.

pub const fn extend(self, z: u32) -> UVec3

Creates a 3D vector from self and the given z value.

pub fn with_x(self, x: u32) -> UVec2

Creates a 2D vector from self with the given value of x.

pub fn with_y(self, y: u32) -> UVec2

Creates a 2D vector from self with the given value of y.

pub fn dot(self, rhs: UVec2) -> u32

Computes the dot product of self and rhs.

pub fn dot_into_vec(self, rhs: UVec2) -> UVec2

Returns a vector where every component is the dot product of self and rhs.

pub fn min(self, rhs: UVec2) -> UVec2

Returns a vector containing the minimum values for each element of self and rhs.

In other words this computes [min(x, rhs.x), min(self.y, rhs.y), ..].

Examples found in repository?
examples/2d/2d_viewport_to_world.rs (line 97)
42fn controls(
43    camera_query: Single<(&mut Camera, &mut Transform, &mut Projection)>,
44    window: Single<&Window>,
45    input: Res<ButtonInput<KeyCode>>,
46    time: Res<Time<Fixed>>,
47) {
48    let (mut camera, mut transform, mut projection) = camera_query.into_inner();
49
50    let fspeed = 600.0 * time.delta_secs();
51    let uspeed = fspeed as u32;
52    let window_size = window.resolution.physical_size();
53
54    // Camera movement controls
55    if input.pressed(KeyCode::ArrowUp) {
56        transform.translation.y += fspeed;
57    }
58    if input.pressed(KeyCode::ArrowDown) {
59        transform.translation.y -= fspeed;
60    }
61    if input.pressed(KeyCode::ArrowLeft) {
62        transform.translation.x -= fspeed;
63    }
64    if input.pressed(KeyCode::ArrowRight) {
65        transform.translation.x += fspeed;
66    }
67
68    // Camera zoom controls
69    if let Projection::Orthographic(projection2d) = &mut *projection {
70        if input.pressed(KeyCode::Comma) {
71            projection2d.scale *= powf(4.0f32, time.delta_secs());
72        }
73
74        if input.pressed(KeyCode::Period) {
75            projection2d.scale *= powf(0.25f32, time.delta_secs());
76        }
77    }
78
79    if let Some(viewport) = camera.viewport.as_mut() {
80        // Viewport movement controls
81        if input.pressed(KeyCode::KeyW) {
82            viewport.physical_position.y = viewport.physical_position.y.saturating_sub(uspeed);
83        }
84        if input.pressed(KeyCode::KeyS) {
85            viewport.physical_position.y += uspeed;
86        }
87        if input.pressed(KeyCode::KeyA) {
88            viewport.physical_position.x = viewport.physical_position.x.saturating_sub(uspeed);
89        }
90        if input.pressed(KeyCode::KeyD) {
91            viewport.physical_position.x += uspeed;
92        }
93
94        // Bound viewport position so it doesn't go off-screen
95        viewport.physical_position = viewport
96            .physical_position
97            .min(window_size - viewport.physical_size);
98
99        // Viewport size controls
100        if input.pressed(KeyCode::KeyI) {
101            viewport.physical_size.y = viewport.physical_size.y.saturating_sub(uspeed);
102        }
103        if input.pressed(KeyCode::KeyK) {
104            viewport.physical_size.y += uspeed;
105        }
106        if input.pressed(KeyCode::KeyJ) {
107            viewport.physical_size.x = viewport.physical_size.x.saturating_sub(uspeed);
108        }
109        if input.pressed(KeyCode::KeyL) {
110            viewport.physical_size.x += uspeed;
111        }
112
113        // Bound viewport size so it doesn't go off-screen
114        viewport.physical_size = viewport
115            .physical_size
116            .min(window_size - viewport.physical_position)
117            .max(UVec2::new(20, 20));
118    }
119}
pub fn max(self, rhs: UVec2) -> UVec2

Returns a vector containing the maximum values for each element of self and rhs.

In other words this computes [max(self.x, rhs.x), max(self.y, rhs.y), ..].

Examples found in repository?
examples/2d/2d_viewport_to_world.rs (line 117)
42fn controls(
43    camera_query: Single<(&mut Camera, &mut Transform, &mut Projection)>,
44    window: Single<&Window>,
45    input: Res<ButtonInput<KeyCode>>,
46    time: Res<Time<Fixed>>,
47) {
48    let (mut camera, mut transform, mut projection) = camera_query.into_inner();
49
50    let fspeed = 600.0 * time.delta_secs();
51    let uspeed = fspeed as u32;
52    let window_size = window.resolution.physical_size();
53
54    // Camera movement controls
55    if input.pressed(KeyCode::ArrowUp) {
56        transform.translation.y += fspeed;
57    }
58    if input.pressed(KeyCode::ArrowDown) {
59        transform.translation.y -= fspeed;
60    }
61    if input.pressed(KeyCode::ArrowLeft) {
62        transform.translation.x -= fspeed;
63    }
64    if input.pressed(KeyCode::ArrowRight) {
65        transform.translation.x += fspeed;
66    }
67
68    // Camera zoom controls
69    if let Projection::Orthographic(projection2d) = &mut *projection {
70        if input.pressed(KeyCode::Comma) {
71            projection2d.scale *= powf(4.0f32, time.delta_secs());
72        }
73
74        if input.pressed(KeyCode::Period) {
75            projection2d.scale *= powf(0.25f32, time.delta_secs());
76        }
77    }
78
79    if let Some(viewport) = camera.viewport.as_mut() {
80        // Viewport movement controls
81        if input.pressed(KeyCode::KeyW) {
82            viewport.physical_position.y = viewport.physical_position.y.saturating_sub(uspeed);
83        }
84        if input.pressed(KeyCode::KeyS) {
85            viewport.physical_position.y += uspeed;
86        }
87        if input.pressed(KeyCode::KeyA) {
88            viewport.physical_position.x = viewport.physical_position.x.saturating_sub(uspeed);
89        }
90        if input.pressed(KeyCode::KeyD) {
91            viewport.physical_position.x += uspeed;
92        }
93
94        // Bound viewport position so it doesn't go off-screen
95        viewport.physical_position = viewport
96            .physical_position
97            .min(window_size - viewport.physical_size);
98
99        // Viewport size controls
100        if input.pressed(KeyCode::KeyI) {
101            viewport.physical_size.y = viewport.physical_size.y.saturating_sub(uspeed);
102        }
103        if input.pressed(KeyCode::KeyK) {
104            viewport.physical_size.y += uspeed;
105        }
106        if input.pressed(KeyCode::KeyJ) {
107            viewport.physical_size.x = viewport.physical_size.x.saturating_sub(uspeed);
108        }
109        if input.pressed(KeyCode::KeyL) {
110            viewport.physical_size.x += uspeed;
111        }
112
113        // Bound viewport size so it doesn't go off-screen
114        viewport.physical_size = viewport
115            .physical_size
116            .min(window_size - viewport.physical_position)
117            .max(UVec2::new(20, 20));
118    }
119}
pub fn clamp(self, min: UVec2, max: UVec2) -> UVec2

Component-wise clamping of values, similar to u32::clamp.

Each element in min must be less-or-equal to the corresponding element in max.

Panics

Will panic if min is greater than max when glam_assert is enabled.

pub fn min_element(self) -> u32

Returns the horizontal minimum of self.

In other words this computes min(x, y, ..).

pub fn max_element(self) -> u32

Returns the horizontal maximum of self.

In other words this computes max(x, y, ..).

pub fn min_position(self) -> usize

Returns the index of the first minimum element of self.

pub fn max_position(self) -> usize

Returns the index of the first maximum element of self.

pub fn element_sum(self) -> u32

Returns the sum of all elements of self.

In other words, this computes self.x + self.y + ...

pub fn element_product(self) -> u32

Returns the product of all elements of self.

In other words, this computes self.x * self.y * ...

Examples found in repository?
examples/2d/tilemap_chunk.rs (line 35)
28fn setup(mut commands: Commands, assets: Res<AssetServer>) {
29    // We're seeding the PRNG here to make this example deterministic for testing purposes.
30    // This isn't strictly required in practical use unless you need your app to be deterministic.
31    let mut rng = ChaCha8Rng::seed_from_u64(42);
32
33    let chunk_size = UVec2::splat(64);
34    let tile_display_size = UVec2::splat(8);
35    let tile_data: Vec<Option<TileData>> = (0..chunk_size.element_product())
36        .map(|_| rng.random_range(0..5))
37        .map(|i| {
38            if i == 0 {
39                None
40            } else {
41                Some(TileData::from_tileset_index(i - 1))
42            }
43        })
44        .collect();
45
46    commands.spawn((
47        TilemapChunk {
48            chunk_size,
49            tile_display_size,
50            tileset: assets.load("textures/array_texture.png"),
51            ..default()
52        },
53        TilemapChunkTileData(tile_data),
54        UpdateTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
55    ));
56
57    commands.spawn(Camera2d);
58
59    commands.insert_resource(SeededRng(rng));
60}
pub fn cmpeq(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a == comparison for each element of self and rhs.

In other words, this computes [self.x == rhs.x, self.y == rhs.y, ..] for all elements.

pub fn cmpne(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a != comparison for each element of self and rhs.

In other words this computes [self.x != rhs.x, self.y != rhs.y, ..] for all elements.

pub fn cmpge(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a >= comparison for each element of self and rhs.

In other words this computes [self.x >= rhs.x, self.y >= rhs.y, ..] for all elements.

pub fn cmpgt(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a > comparison for each element of self and rhs.

In other words this computes [self.x > rhs.x, self.y > rhs.y, ..] for all elements.

pub fn cmple(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a <= comparison for each element of self and rhs.

In other words this computes [self.x <= rhs.x, self.y <= rhs.y, ..] for all elements.

pub fn cmplt(self, rhs: UVec2) -> BVec2

Returns a vector mask containing the result of a < comparison for each element of self and rhs.

In other words this computes [self.x < rhs.x, self.y < rhs.y, ..] for all elements.

pub fn length_squared(self) -> u32

Computes the squared length of self.

pub fn manhattan_distance(self, rhs: UVec2) -> u32

Computes the manhattan distance between two points.

Overflow

This method may overflow if the result is greater than u32::MAX.

See also checked_manhattan_distance.

pub fn checked_manhattan_distance(self, rhs: UVec2) -> Option<u32>

Computes the manhattan distance between two points.

This will returns None if the result is greater than u32::MAX.

pub fn chebyshev_distance(self, rhs: UVec2) -> u32

Computes the chebyshev distance between two points.

pub fn as_vec2(&self) -> Vec2

Casts all elements of self to f32.

Examples found in repository?
examples/shader/compute_shader_game_of_life.rs (line 65)
54fn setup(mut commands: Commands, mut images: ResMut<Assets<Image>>) {
55    let mut image = Image::new_target_texture(SIZE.x, SIZE.y, TextureFormat::Rgba32Float);
56    image.asset_usage = RenderAssetUsages::RENDER_WORLD;
57    image.texture_descriptor.usage =
58        TextureUsages::COPY_DST | TextureUsages::STORAGE_BINDING | TextureUsages::TEXTURE_BINDING;
59    let image0 = images.add(image.clone());
60    let image1 = images.add(image);
61
62    commands.spawn((
63        Sprite {
64            image: image0.clone(),
65            custom_size: Some(SIZE.as_vec2()),
66            ..default()
67        },
68        Transform::from_scale(Vec3::splat(DISPLAY_FACTOR as f32)),
69    ));
70    commands.spawn(Camera2d);
71
72    commands.insert_resource(GameOfLifeImages {
73        texture_a: image0,
74        texture_b: image1,
75    });
76
77    commands.insert_resource(GameOfLifeUniforms {
78        alive_color: LinearRgba::RED,
79    });
80}
More examples
Hide additional examples
examples/2d/2d_viewport_to_world.rs (line 127)
121fn setup(
122    mut commands: Commands,
123    mut meshes: ResMut<Assets<Mesh>>,
124    mut materials: ResMut<Assets<ColorMaterial>>,
125    window: Single<&Window>,
126) {
127    let window_size = window.resolution.physical_size().as_vec2();
128
129    // Initialize centered, non-window-filling viewport
130    commands.spawn((
131        Camera2d,
132        Camera {
133            viewport: Some(Viewport {
134                physical_position: (window_size * 0.125).as_uvec2(),
135                physical_size: (window_size * 0.75).as_uvec2(),
136                ..default()
137            }),
138            ..default()
139        },
140    ));
141
142    // Create a minimal UI explaining how to interact with the example
143    commands.spawn((
144        Text::new(
145            "Move the mouse to see the circle follow your cursor.\n\
146                    Use the arrow keys to move the camera.\n\
147                    Use the comma and period keys to zoom in and out.\n\
148                    Use the WASD keys to move the viewport.\n\
149                    Use the IJKL keys to resize the viewport.",
150        ),
151        Node {
152            position_type: PositionType::Absolute,
153            top: px(12),
154            left: px(12),
155            ..default()
156        },
157    ));
158
159    // Add mesh to make camera movement visible
160    commands.spawn((
161        Mesh2d(meshes.add(Rectangle::new(40.0, 20.0))),
162        MeshMaterial2d(materials.add(Color::from(GREEN))),
163    ));
164
165    // Add background to visualize viewport bounds
166    commands.spawn((
167        Mesh2d(meshes.add(Rectangle::new(50000.0, 50000.0))),
168        MeshMaterial2d(materials.add(Color::linear_rgb(0.01, 0.01, 0.01))),
169        Transform::from_translation(Vec3::new(0.0, 0.0, -200.0)),
170    ));
171}
examples/ui/render_ui_to_texture.rs (line 197)
174fn drive_diegetic_pointer(
175    mut cursor_last: Local<Vec2>,
176    mut raycast: MeshRayCast,
177    rays: Res<RayMap>,
178    cubes: Query<&Mesh3d, With<Cube>>,
179    ui_camera: Query<&Camera, With<Camera2d>>,
180    primary_window: Query<Entity, With<PrimaryWindow>>,
181    windows: Query<(Entity, &Window)>,
182    images: Res<Assets<Image>>,
183    manual_texture_views: Res<ManualTextureViews>,
184    mut window_events: MessageReader<WindowEvent>,
185    mut pointer_inputs: MessageWriter<PointerInput>,
186) -> Result {
187    // Get the size of the texture, so we can convert from dimensionless UV coordinates that span
188    // from 0 to 1, to pixel coordinates.
189    let target = ui_camera
190        .single()?
191        .target
192        .normalize(primary_window.single().ok())
193        .unwrap();
194    let target_info = target
195        .get_render_target_info(windows, &images, &manual_texture_views)
196        .unwrap();
197    let size = target_info.physical_size.as_vec2();
198
199    // Find raycast hits and update the virtual pointer.
200    let raycast_settings = MeshRayCastSettings {
201        visibility: RayCastVisibility::VisibleInView,
202        filter: &|entity| cubes.contains(entity),
203        early_exit_test: &|_| false,
204    };
205    for (_id, ray) in rays.iter() {
206        for (_cube, hit) in raycast.cast_ray(*ray, &raycast_settings) {
207            let position = size * hit.uv.unwrap();
208            if position != *cursor_last {
209                pointer_inputs.write(PointerInput::new(
210                    CUBE_POINTER_ID,
211                    Location {
212                        target: target.clone(),
213                        position,
214                    },
215                    PointerAction::Move {
216                        delta: position - *cursor_last,
217                    },
218                ));
219                *cursor_last = position;
220            }
221        }
222    }
223
224    // Pipe pointer button presses to the virtual pointer on the UI texture.
225    for window_event in window_events.read() {
226        if let WindowEvent::MouseButtonInput(input) = window_event {
227            let button = match input.button {
228                MouseButton::Left => PointerButton::Primary,
229                MouseButton::Right => PointerButton::Secondary,
230                MouseButton::Middle => PointerButton::Middle,
231                _ => continue,
232            };
233            let action = match input.state {
234                ButtonState::Pressed => PointerAction::Press(button),
235                ButtonState::Released => PointerAction::Release(button),
236            };
237            pointer_inputs.write(PointerInput::new(
238                CUBE_POINTER_ID,
239                Location {
240                    target: target.clone(),
241                    position: *cursor_last,
242                },
243                action,
244            ));
245        }
246    }
247
248    Ok(())
249}
pub fn as_dvec2(&self) -> DVec2

Casts all elements of self to f64.

pub fn as_i8vec2(&self) -> I8Vec2

Casts all elements of self to i8.

pub fn as_u8vec2(&self) -> U8Vec2

Casts all elements of self to u8.

pub fn as_i16vec2(&self) -> I16Vec2

Casts all elements of self to i16.

pub fn as_u16vec2(&self) -> U16Vec2

Casts all elements of self to u16.

pub fn as_ivec2(&self) -> IVec2

Casts all elements of self to i32.

pub fn as_i64vec2(&self) -> I64Vec2

Casts all elements of self to i64.

pub fn as_u64vec2(&self) -> U64Vec2

Casts all elements of self to u64.

pub fn as_usizevec2(&self) -> USizeVec2

Casts all elements of self to usize.

pub const fn checked_add(self, rhs: UVec2) -> Option<UVec2>

Returns a vector containing the wrapping addition of self and rhs.

In other words this computes Some([self.x + rhs.x, self.y + rhs.y, ..]) but returns None on any overflow.

pub const fn checked_sub(self, rhs: UVec2) -> Option<UVec2>

Returns a vector containing the wrapping subtraction of self and rhs.

In other words this computes Some([self.x - rhs.x, self.y - rhs.y, ..]) but returns None on any overflow.

pub const fn checked_mul(self, rhs: UVec2) -> Option<UVec2>

Returns a vector containing the wrapping multiplication of self and rhs.

In other words this computes Some([self.x * rhs.x, self.y * rhs.y, ..]) but returns None on any overflow.

pub const fn checked_div(self, rhs: UVec2) -> Option<UVec2>

Returns a vector containing the wrapping division of self and rhs.

In other words this computes Some([self.x / rhs.x, self.y / rhs.y, ..]) but returns None on any division by zero.

pub const fn wrapping_add(self, rhs: UVec2) -> UVec2

Returns a vector containing the wrapping addition of self and rhs.

In other words this computes [self.x.wrapping_add(rhs.x), self.y.wrapping_add(rhs.y), ..].

pub const fn wrapping_sub(self, rhs: UVec2) -> UVec2

Returns a vector containing the wrapping subtraction of self and rhs.

In other words this computes [self.x.wrapping_sub(rhs.x), self.y.wrapping_sub(rhs.y), ..].

pub const fn wrapping_mul(self, rhs: UVec2) -> UVec2

Returns a vector containing the wrapping multiplication of self and rhs.

In other words this computes [self.x.wrapping_mul(rhs.x), self.y.wrapping_mul(rhs.y), ..].

pub const fn wrapping_div(self, rhs: UVec2) -> UVec2

Returns a vector containing the wrapping division of self and rhs.

In other words this computes [self.x.wrapping_div(rhs.x), self.y.wrapping_div(rhs.y), ..].

pub const fn saturating_add(self, rhs: UVec2) -> UVec2

Returns a vector containing the saturating addition of self and rhs.

In other words this computes [self.x.saturating_add(rhs.x), self.y.saturating_add(rhs.y), ..].

pub const fn saturating_sub(self, rhs: UVec2) -> UVec2

Returns a vector containing the saturating subtraction of self and rhs.

In other words this computes [self.x.saturating_sub(rhs.x), self.y.saturating_sub(rhs.y), ..].

pub const fn saturating_mul(self, rhs: UVec2) -> UVec2

Returns a vector containing the saturating multiplication of self and rhs.

In other words this computes [self.x.saturating_mul(rhs.x), self.y.saturating_mul(rhs.y), ..].

pub const fn saturating_div(self, rhs: UVec2) -> UVec2

Returns a vector containing the saturating division of self and rhs.

In other words this computes [self.x.saturating_div(rhs.x), self.y.saturating_div(rhs.y), ..].

pub const fn checked_add_signed(self, rhs: IVec2) -> Option<UVec2>

Returns a vector containing the wrapping addition of self and signed vector rhs.

In other words this computes Some([self.x + rhs.x, self.y + rhs.y, ..]) but returns None on any overflow.

pub const fn wrapping_add_signed(self, rhs: IVec2) -> UVec2

Returns a vector containing the wrapping addition of self and signed vector rhs.

In other words this computes [self.x.wrapping_add_signed(rhs.x), self.y.wrapping_add_signed(rhs.y), ..].

pub const fn saturating_add_signed(self, rhs: IVec2) -> UVec2

Returns a vector containing the saturating addition of self and signed vector rhs.

In other words this computes [self.x.saturating_add_signed(rhs.x), self.y.saturating_add_signed(rhs.y), ..].