pub struct Vec4(/* private fields */);Expand description
A 4-dimensional vector.
SIMD vector types are used for storage on supported platforms.
This type is 16 byte aligned.
Implementations
Sourceimpl Vec4
impl Vec4
Sourcepub const NEG_INFINITY: Vec4
pub const NEG_INFINITY: Vec4
All f32::NEG_INFINITY.
Sourcepub const USES_CORE_SIMD: bool = false
pub const USES_CORE_SIMD: bool = false
Vec4 uses Rust Portable SIMD
Sourcepub const USES_SCALAR_MATH: bool = false
pub const USES_SCALAR_MATH: bool = false
Vec4 uses scalar math
Sourcepub const USES_WASM_SIMD: bool = false
pub const USES_WASM_SIMD: bool = false
Vec4 uses WebAssembly 128-bit SIMD
pub const USES_WASM32_SIMD: bool = false
Renamed to USES_WASM_SIMD
Sourcepub const fn splat(v: f32) -> Vec4
pub const fn splat(v: f32) -> Vec4
Creates a vector with all elements set to v.
Examples found in repository?
20fn setup(
21 mut commands: Commands,
22 mut ui_materials: ResMut<Assets<CustomUiMaterial>>,
23 asset_server: Res<AssetServer>,
24) {
25 // Camera so we can see UI
26 commands.spawn(Camera2d);
27
28 commands
29 .spawn(Node {
30 width: percent(100),
31 height: percent(100),
32 align_items: AlignItems::Center,
33 justify_content: JustifyContent::Center,
34 ..default()
35 })
36 .with_children(|parent| {
37 let banner_scale_factor = 0.5;
38 parent.spawn((
39 Node {
40 position_type: PositionType::Absolute,
41 width: px(905.0 * banner_scale_factor),
42 height: px(363.0 * banner_scale_factor),
43 border: UiRect::all(px(20)),
44 border_radius: BorderRadius::all(px(20)),
45 ..default()
46 },
47 MaterialNode(ui_materials.add(CustomUiMaterial {
48 color: LinearRgba::WHITE.to_f32_array().into(),
49 slider: Vec4::splat(0.5),
50 color_texture: asset_server.load("branding/banner.png"),
51 border_color: LinearRgba::WHITE.to_f32_array().into(),
52 })),
53 // UI material nodes can have outlines and shadows like any other UI node
54 Outline {
55 width: px(2),
56 offset: px(100),
57 color: DARK_BLUE.into(),
58 },
59 ));
60 });
61}More examples
3606 pub fn setup(
3607 mut commands: Commands,
3608 mut default_ui_materials: ResMut<Assets<DefaultUiMaterial>>,
3609 mut ui_materials: ResMut<Assets<CustomUiMaterial>>,
3610 asset_server: Res<AssetServer>,
3611 ) {
3612 let default_material = default_ui_materials.add(DefaultUiMaterial {});
3613 let custom_material = |slider| CustomUiMaterial {
3614 color: LinearRgba::from(CYAN_100).to_vec4(),
3615 slider: Vec4::splat(slider),
3616 color_texture: asset_server.load("branding/banner.png"),
3617 border_color: LinearRgba::from(YELLOW_100).to_vec4(),
3618 };
3619 let full_material = ui_materials.add(custom_material(1.));
3620 let material_node = Node {
3621 width: px(200),
3622 height: px(80),
3623 border: px(10).all(),
3624 border_radius: BorderRadius::all(px(20)),
3625 ..default()
3626 };
3627 let material_label = |text: &str| (Text::new(text), TextFont::from_font_size(px(10.)));
3628
3629 commands.spawn((Camera2d, DespawnOnExit(super::Scene::NodeMaterial)));
3630 commands.spawn((
3631 Node {
3632 display: Display::Grid,
3633 // two column grid, labels then materials, first row is spanned by title.
3634 width: percent(100),
3635 height: percent(100),
3636 grid_template_columns: vec![GridTrack::auto(), GridTrack::px(200.)],
3637 align_items: AlignItems::Center,
3638 align_content: AlignContent::Center,
3639 justify_content: JustifyContent::Center,
3640 row_gap: px(10.),
3641 column_gap: px(20.),
3642 ..default()
3643 },
3644 DespawnOnExit(super::Scene::NodeMaterial),
3645 children![
3646 (
3647 Text::new("MaterialNode"),
3648 TextFont::from_font_size(px(30.)),
3649 TextLayout::justify(Justify::Center),
3650 Node {
3651 grid_column: GridPlacement::span(2),
3652 margin: px(10.).bottom(),
3653 ..default()
3654 },
3655 ),
3656 // Default UI material (just white with the default ui_material.wgsl shader for the bevy_ui crate)
3657 material_label("'ui_material.wgsl' default material"),
3658 (material_node.clone(), MaterialNode(default_material),),
3659 // Custom UI material, 1./3 full.
3660 material_label("'custom_ui_material.wgsl' 1/3"),
3661 (
3662 material_node.clone(),
3663 MaterialNode(ui_materials.add(custom_material(1. / 3.))),
3664 ),
3665 // Custom UI material, 2./3 full.
3666 material_label("'custom_ui_material.wgsl' 2/3"),
3667 (
3668 material_node.clone(),
3669 MaterialNode(ui_materials.add(custom_material(2. / 3.)))
3670 ),
3671 // Custom UI material, full.
3672 material_label("'custom_ui_material.wgsl' Full"),
3673 (material_node.clone(), MaterialNode(full_material.clone()),),
3674 // Custom UI material, full. Clipped using overflow so bottom 50% missing
3675 material_label("'custom_ui_material.wgsl' clipped vertically"),
3676 (
3677 Node {
3678 width: material_node.width,
3679 height: material_node.height,
3680 ..default()
3681 },
3682 Outline {
3683 width: px(1),
3684 color: RED_500.into(),
3685 ..default()
3686 },
3687 children![(
3688 Node {
3689 width: percent(100),
3690 height: percent(50),
3691 overflow: Overflow::clip(),
3692 ..default()
3693 },
3694 children![(material_node.clone(), MaterialNode(full_material.clone()))],
3695 )],
3696 ),
3697 // Custom UI material clipped on both axis so bottom 25% and right 25% not visible.
3698 material_label("'custom_ui_material.wgsl' end clipped on both axis"),
3699 (
3700 Node {
3701 width: material_node.width,
3702 height: material_node.height,
3703 ..default()
3704 },
3705 Outline {
3706 width: px(1),
3707 color: RED_500.into(),
3708 ..default()
3709 },
3710 children![(
3711 Node {
3712 width: percent(75),
3713 height: percent(75),
3714 overflow: Overflow::clip(),
3715 ..default()
3716 },
3717 children![(
3718 Node {
3719 position_type: PositionType::Absolute,
3720 width: px(200),
3721 height: px(80),
3722 border: px(10).all(),
3723 border_radius: BorderRadius::all(px(20)),
3724 ..default()
3725 },
3726 MaterialNode(full_material.clone()),
3727 )],
3728 )],
3729 ),
3730 // Custom UI material clipped on both axis so top 25% and left 25% not visible.
3731 material_label("'custom_ui_material.wgsl' start clipped on both axis"),
3732 (
3733 Node {
3734 width: material_node.width,
3735 height: material_node.height,
3736 align_items: AlignItems::End,
3737 justify_content: JustifyContent::End,
3738 ..default()
3739 },
3740 Outline {
3741 width: px(1),
3742 color: RED_500.into(),
3743 ..default()
3744 },
3745 children![(
3746 Node {
3747 width: percent(75),
3748 height: percent(75),
3749 overflow: Overflow::clip(),
3750 ..default()
3751 },
3752 children![(
3753 Node {
3754 position_type: PositionType::Absolute,
3755 width: px(200),
3756 height: px(80),
3757 right: px(0),
3758 bottom: px(0),
3759 border: px(10).all(),
3760 border_radius: BorderRadius::all(px(20)),
3761 ..default()
3762 },
3763 MaterialNode(full_material),
3764 )],
3765 )],
3766 ),
3767 ],
3768 ));
3769 }Sourcepub fn map<F>(self, f: F) -> Vec4
pub fn map<F>(self, f: F) -> Vec4
Returns a vector containing each element of self modified by a mapping function f.
Sourcepub fn select(mask: BVec4A, if_true: Vec4, if_false: Vec4) -> Vec4
pub fn select(mask: BVec4A, if_true: Vec4, if_false: Vec4) -> Vec4
Creates a vector from the elements in if_true and if_false, selecting which to use
for each element of self.
A true element in the mask uses the corresponding element from if_true, and false
uses the element from if_false.
Sourcepub const fn from_array(a: [f32; 4]) -> Vec4
pub const fn from_array(a: [f32; 4]) -> Vec4
Creates a new vector from an array.
Sourcepub const fn from_slice(slice: &[f32]) -> Vec4
pub const fn from_slice(slice: &[f32]) -> Vec4
Creates a vector from the first 4 values in slice.
Panics
Panics if slice is less than 4 elements long.
Sourcepub fn write_to_slice(self, slice: &mut [f32])
pub fn write_to_slice(self, slice: &mut [f32])
Writes the elements of self to the first 4 elements in slice.
Panics
Panics if slice is less than 4 elements long.
Sourcepub fn truncate(self) -> Vec3
pub fn truncate(self) -> Vec3
Creates a 3D vector from the x, y and z elements of self, discarding w.
Truncation to Vec3 may also be performed by using self.xyz().
To truncate to Vec3A use Vec3A::from_vec4().
Sourcepub fn project(self) -> Vec3
pub fn project(self) -> Vec3
Projects a homogeneous coordinate to 3D space by performing perspective divide.
To project to Vec3A use Vec3A::from_homogeneous().
Panics
Will panic if self.w is 0 when glam_assert is enabled.
Sourcepub fn dot_into_vec(self, rhs: Vec4) -> Vec4
pub fn dot_into_vec(self, rhs: Vec4) -> Vec4
Returns a vector where every component is the dot product of self and rhs.
Sourcepub fn min(self, rhs: Vec4) -> Vec4
pub fn min(self, rhs: Vec4) -> Vec4
Returns a vector containing the minimum values for each element of self and rhs.
In other words this computes [min(x, rhs.x), min(self.y, rhs.y), ..].
NaN propogation does not follow IEEE 754-2008 semantics for minNum and may differ on different SIMD architectures.
Sourcepub fn max(self, rhs: Vec4) -> Vec4
pub fn max(self, rhs: Vec4) -> Vec4
Returns a vector containing the maximum values for each element of self and rhs.
In other words this computes [max(self.x, rhs.x), max(self.y, rhs.y), ..].
NaN propogation does not follow IEEE 754-2008 semantics for maxNum and may differ on different SIMD architectures.
Sourcepub fn clamp(self, min: Vec4, max: Vec4) -> Vec4
pub fn clamp(self, min: Vec4, max: Vec4) -> Vec4
Component-wise clamping of values, similar to f32::clamp.
Each element in min must be less-or-equal to the corresponding element in max.
NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.
Panics
Will panic if min is greater than max when glam_assert is enabled.
Sourcepub fn min_element(self) -> f32
pub fn min_element(self) -> f32
Returns the horizontal minimum of self.
In other words this computes min(x, y, ..).
NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.
Sourcepub fn max_element(self) -> f32
pub fn max_element(self) -> f32
Returns the horizontal maximum of self.
In other words this computes max(x, y, ..).
NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.
Sourcepub fn min_position(self) -> usize
pub fn min_position(self) -> usize
Returns the index of the first minimum element of self.
Sourcepub fn max_position(self) -> usize
pub fn max_position(self) -> usize
Returns the index of the first maximum element of self.
Sourcepub fn element_sum(self) -> f32
pub fn element_sum(self) -> f32
Returns the sum of all elements of self.
In other words, this computes self.x + self.y + ...
Sourcepub fn element_product(self) -> f32
pub fn element_product(self) -> f32
Returns the product of all elements of self.
In other words, this computes self.x * self.y * ...
Sourcepub fn cmpeq(self, rhs: Vec4) -> BVec4A
pub fn cmpeq(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a == comparison for each element of
self and rhs.
In other words, this computes [self.x == rhs.x, self.y == rhs.y, ..] for all
elements.
Sourcepub fn cmpne(self, rhs: Vec4) -> BVec4A
pub fn cmpne(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a != comparison for each element of
self and rhs.
In other words this computes [self.x != rhs.x, self.y != rhs.y, ..] for all
elements.
Sourcepub fn cmpge(self, rhs: Vec4) -> BVec4A
pub fn cmpge(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a >= comparison for each element of
self and rhs.
In other words this computes [self.x >= rhs.x, self.y >= rhs.y, ..] for all
elements.
Sourcepub fn cmpgt(self, rhs: Vec4) -> BVec4A
pub fn cmpgt(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a > comparison for each element of
self and rhs.
In other words this computes [self.x > rhs.x, self.y > rhs.y, ..] for all
elements.
Sourcepub fn cmple(self, rhs: Vec4) -> BVec4A
pub fn cmple(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a <= comparison for each element of
self and rhs.
In other words this computes [self.x <= rhs.x, self.y <= rhs.y, ..] for all
elements.
Sourcepub fn cmplt(self, rhs: Vec4) -> BVec4A
pub fn cmplt(self, rhs: Vec4) -> BVec4A
Returns a vector mask containing the result of a < comparison for each element of
self and rhs.
In other words this computes [self.x < rhs.x, self.y < rhs.y, ..] for all
elements.
Sourcepub fn abs(self) -> Vec4
pub fn abs(self) -> Vec4
Returns a vector containing the absolute value of each element of self.
Sourcepub fn signum(self) -> Vec4
pub fn signum(self) -> Vec4
Returns a vector with elements representing the sign of self.
1.0if the number is positive,+0.0orINFINITY-1.0if the number is negative,-0.0orNEG_INFINITYNANif the number isNAN
Sourcepub fn copysign(self, rhs: Vec4) -> Vec4
pub fn copysign(self, rhs: Vec4) -> Vec4
Returns a vector with signs of rhs and the magnitudes of self.
Sourcepub fn is_negative_bitmask(self) -> u32
pub fn is_negative_bitmask(self) -> u32
Returns a bitmask with the lowest 4 bits set to the sign bits from the elements of self.
A negative element results in a 1 bit and a positive element in a 0 bit. Element x goes
into the first lowest bit, element y into the second, etc.
An element is negative if it has a negative sign, including -0.0, NaNs with negative sign bit and negative infinity.
Sourcepub fn is_negative_mask(self) -> BVec4A
pub fn is_negative_mask(self) -> BVec4A
Returns a mask indicating which components are negative.
An element is negative if it has a negative sign, including -0.0, NaNs with negative sign bit and negative infinity.
Sourcepub fn is_finite(self) -> bool
pub fn is_finite(self) -> bool
Returns true if, and only if, all elements are finite. If any element is either
NaN, positive or negative infinity, this will return false.
Sourcepub fn is_finite_mask(self) -> BVec4A
pub fn is_finite_mask(self) -> BVec4A
Performs is_finite on each element of self, returning a vector mask of the results.
In other words, this computes [x.is_finite(), y.is_finite(), ...].
Sourcepub fn is_nan_mask(self) -> BVec4A
pub fn is_nan_mask(self) -> BVec4A
Performs is_nan on each element of self, returning a vector mask of the results.
In other words, this computes [x.is_nan(), y.is_nan(), ...].
Sourcepub fn length_squared(self) -> f32
pub fn length_squared(self) -> f32
Computes the squared length of self.
This is faster than length() as it avoids a square root operation.
Sourcepub fn length_recip(self) -> f32
pub fn length_recip(self) -> f32
Computes 1.0 / length().
For valid results, self must not be of length zero.
Sourcepub fn distance(self, rhs: Vec4) -> f32
pub fn distance(self, rhs: Vec4) -> f32
Computes the Euclidean distance between two points in space.
Sourcepub fn distance_squared(self, rhs: Vec4) -> f32
pub fn distance_squared(self, rhs: Vec4) -> f32
Compute the squared euclidean distance between two points in space.
Sourcepub fn div_euclid(self, rhs: Vec4) -> Vec4
pub fn div_euclid(self, rhs: Vec4) -> Vec4
Returns the element-wise quotient of [Euclidean division] of self by rhs.
Sourcepub fn rem_euclid(self, rhs: Vec4) -> Vec4
pub fn rem_euclid(self, rhs: Vec4) -> Vec4
Returns the element-wise remainder of Euclidean division of self by rhs.
Sourcepub fn normalize(self) -> Vec4
pub fn normalize(self) -> Vec4
Returns self normalized to length 1.0.
For valid results, self must be finite and not of length zero, nor very close to zero.
See also Self::try_normalize() and Self::normalize_or_zero().
Panics
Will panic if the resulting normalized vector is not finite when glam_assert is enabled.
Sourcepub fn try_normalize(self) -> Option<Vec4>
pub fn try_normalize(self) -> Option<Vec4>
Returns self normalized to length 1.0 if possible, else returns None.
In particular, if the input is zero (or very close to zero), or non-finite,
the result of this operation will be None.
See also Self::normalize_or_zero().
Sourcepub fn normalize_or(self, fallback: Vec4) -> Vec4
pub fn normalize_or(self, fallback: Vec4) -> Vec4
Returns self normalized to length 1.0 if possible, else returns a
fallback value.
In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be the fallback value.
See also Self::try_normalize().
Sourcepub fn normalize_or_zero(self) -> Vec4
pub fn normalize_or_zero(self) -> Vec4
Returns self normalized to length 1.0 if possible, else returns zero.
In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be zero.
See also Self::try_normalize().
Sourcepub fn normalize_and_length(self) -> (Vec4, f32)
pub fn normalize_and_length(self) -> (Vec4, f32)
Returns self normalized to length 1.0 and the length of self.
If self is zero length then (Self::X, 0.0) is returned.
Sourcepub fn is_normalized(self) -> bool
pub fn is_normalized(self) -> bool
Returns whether self is length 1.0 or not.
Uses a precision threshold of approximately 1e-4.
Sourcepub fn project_onto(self, rhs: Vec4) -> Vec4
pub fn project_onto(self, rhs: Vec4) -> Vec4
Returns the vector projection of self onto rhs.
rhs must be of non-zero length.
Panics
Will panic if rhs is zero length when glam_assert is enabled.
Sourcepub fn reject_from(self, rhs: Vec4) -> Vec4
pub fn reject_from(self, rhs: Vec4) -> Vec4
Returns the vector rejection of self from rhs.
The vector rejection is the vector perpendicular to the projection of self onto
rhs, in rhs words the result of self - self.project_onto(rhs).
rhs must be of non-zero length.
Panics
Will panic if rhs has a length of zero when glam_assert is enabled.
Sourcepub fn project_onto_normalized(self, rhs: Vec4) -> Vec4
pub fn project_onto_normalized(self, rhs: Vec4) -> Vec4
Returns the vector projection of self onto rhs.
rhs must be normalized.
Panics
Will panic if rhs is not normalized when glam_assert is enabled.
Sourcepub fn reject_from_normalized(self, rhs: Vec4) -> Vec4
pub fn reject_from_normalized(self, rhs: Vec4) -> Vec4
Returns the vector rejection of self from rhs.
The vector rejection is the vector perpendicular to the projection of self onto
rhs, in rhs words the result of self - self.project_onto(rhs).
rhs must be normalized.
Panics
Will panic if rhs is not normalized when glam_assert is enabled.
Sourcepub fn round(self) -> Vec4
pub fn round(self) -> Vec4
Returns a vector containing the nearest integer to a number for each element of self.
Round half-way cases away from 0.0.
Sourcepub fn floor(self) -> Vec4
pub fn floor(self) -> Vec4
Returns a vector containing the largest integer less than or equal to a number for each
element of self.
Sourcepub fn ceil(self) -> Vec4
pub fn ceil(self) -> Vec4
Returns a vector containing the smallest integer greater than or equal to a number for
each element of self.
Sourcepub fn trunc(self) -> Vec4
pub fn trunc(self) -> Vec4
Returns a vector containing the integer part each element of self. This means numbers are
always truncated towards zero.
Sourcepub fn step(self, rhs: Vec4) -> Vec4
pub fn step(self, rhs: Vec4) -> Vec4
Returns a vector containing 0.0 if rhs < self and 1.0 otherwise.
Similar to glsl’s step(edge, x), which translates into edge.step(x)
Sourcepub fn smoothstep(self, edge0: Vec4, edge1: Vec4) -> Vec4
pub fn smoothstep(self, edge0: Vec4, edge1: Vec4) -> Vec4
Performs Hermite interpolation between 0.0 and 1.0 using x normalized to [edge0, edge1].
This is equivalent to t * t * (3.0 - 2.0 * t), where t is clamped to [0.0, 1.0].
Results are undefined if any element of edge0 is greater than or equal to the corresponding
element of edge1.
Panics
Will panic if any element of edge0 is greater than or equal to the corresponding element
of edge1, when glam_assert is enabled.
Sourcepub fn saturate(self) -> Vec4
pub fn saturate(self) -> Vec4
Returns a vector containing all elements of self clamped to the range of [0, 1].
Sourcepub fn fract(self) -> Vec4
pub fn fract(self) -> Vec4
Returns a vector containing the fractional part of the vector as self - self.trunc().
Note that this differs from the GLSL implementation of fract which returns
self - self.floor().
Note that this is fast but not precise for large numbers.
Sourcepub fn fract_gl(self) -> Vec4
pub fn fract_gl(self) -> Vec4
Returns a vector containing the fractional part of the vector as self - self.floor().
Note that this differs from the Rust implementation of fract which returns
self - self.trunc().
Note that this is fast but not precise for large numbers.
Sourcepub fn exp(self) -> Vec4
pub fn exp(self) -> Vec4
Returns a vector containing e^self (the exponential function) for each element of
self.
Sourcepub fn ln(self) -> Vec4
pub fn ln(self) -> Vec4
Returns a vector containing the natural logarithm for each element of self.
This returns NaN when the element is negative and negative infinity when the element is zero.
Sourcepub fn log2(self) -> Vec4
pub fn log2(self) -> Vec4
Returns a vector containing the base 2 logarithm for each element of self.
This returns NaN when the element is negative and negative infinity when the element is zero.
Sourcepub fn powf(self, n: f32) -> Vec4
pub fn powf(self, n: f32) -> Vec4
Returns a vector containing each element of self raised to the power of n.
Sourcepub fn sqrt(self) -> Vec4
pub fn sqrt(self) -> Vec4
Returns a vector containing the square root for each element of self.
This returns NaN when the element is negative.
Sourcepub fn sin_cos(self) -> (Vec4, Vec4)
pub fn sin_cos(self) -> (Vec4, Vec4)
Returns a tuple of two vectors containing the sine and cosine for each element of self.
Sourcepub fn recip(self) -> Vec4
pub fn recip(self) -> Vec4
Returns a vector containing the reciprocal 1.0/n of each element of self.
Sourcepub fn lerp(self, rhs: Vec4, s: f32) -> Vec4
pub fn lerp(self, rhs: Vec4, s: f32) -> Vec4
Performs a linear interpolation between self and rhs based on the value s, using the
form self * (1.0 - s) + rhs * s.
When s is 0.0, the result will be equal to self. When s is 1.0, the result will
be equal to rhs. When s is outside of the range [0, 1], the result is linearly
extrapolated.
The result is guaranteed to be self at s == 0.0 and rhs at s == 1.0, even when the
values differ greatly in magnitude, but it is not monotonic in s for nearly equal inputs
and may not preserve equal inputs exactly. Consider lerp_monotonic
instead when interpolating between values that may be equal or nearly equal.
Sourcepub fn lerp_monotonic(self, rhs: Vec4, s: f32) -> Vec4
pub fn lerp_monotonic(self, rhs: Vec4, s: f32) -> Vec4
Performs a linear interpolation between self and rhs based on the value s, using the
monotonic form self + (rhs - self) * s.
When s is 0.0, the result will be equal to self. When s is 1.0, the result will
be equal to rhs. When s is outside of the range [0, 1], the result is linearly
extrapolated.
Prefer this over lerp when interpolating between values that may be equal or
nearly equal: the result is monotonic in s and equal inputs are preserved exactly, avoiding
the rounding jitter that lerp can introduce. The tradeoff is that
rhs - self is evaluated first, so this is less accurate than lerp when
self and rhs differ greatly in magnitude, and overflows to infinity when they have
opposite signs and large magnitudes.
On SIMD back-ends the multiply and add are fused when the target supports it, which has a single rounding step and can be more accurate than a separate multiply and add.
Sourcepub fn move_towards(self, rhs: Vec4, d: f32) -> Vec4
pub fn move_towards(self, rhs: Vec4, d: f32) -> Vec4
Moves towards rhs based on the value d.
When d is 0.0, the result will be equal to self. When d is equal to
self.distance(rhs), the result will be equal to rhs. Will not go past rhs.
Sourcepub fn midpoint(self, rhs: Vec4) -> Vec4
pub fn midpoint(self, rhs: Vec4) -> Vec4
Calculates the midpoint between self and rhs.
The midpoint is the average of, or halfway point between, two vectors.
a.midpoint(b) should yield the same result as a.lerp(b, 0.5)
while being slightly cheaper to compute.
Sourcepub fn abs_diff_eq(self, rhs: Vec4, max_abs_diff: f32) -> bool
pub fn abs_diff_eq(self, rhs: Vec4, max_abs_diff: f32) -> bool
Returns true if the absolute difference of all elements between self and rhs is
less than or equal to max_abs_diff.
This can be used to compare if two vectors contain similar elements. It works best when
comparing with a known value. The max_abs_diff that should be used used depends on
the values being compared against.
For more see comparing floating point numbers.
Sourcepub fn clamp_length(self, min: f32, max: f32) -> Vec4
pub fn clamp_length(self, min: f32, max: f32) -> Vec4
Returns a vector with a length no less than min and no more than max.
Panics
Will panic if min is greater than max, or if either min or max is negative, when glam_assert is enabled.
Sourcepub fn clamp_length_max(self, max: f32) -> Vec4
pub fn clamp_length_max(self, max: f32) -> Vec4
Returns a vector with a length no more than max.
Panics
Will panic if max is negative when glam_assert is enabled.
Sourcepub fn clamp_length_min(self, min: f32) -> Vec4
pub fn clamp_length_min(self, min: f32) -> Vec4
Returns a vector with a length no less than min.
Panics
Will panic if min is negative when glam_assert is enabled.
Sourcepub fn mul_add(self, a: Vec4, b: Vec4) -> Vec4
pub fn mul_add(self, a: Vec4, b: Vec4) -> Vec4
Fused multiply-add. Computes (self * a) + b element-wise with only one rounding
error, yielding a more accurate result than an unfused multiply-add.
Using mul_add may be more performant than an unfused multiply-add if the target
architecture has a dedicated fma CPU instruction. However, this is not always true,
and will be heavily dependant on designing algorithms with specific target hardware in
mind.
Sourcepub fn reflect(self, normal: Vec4) -> Vec4
pub fn reflect(self, normal: Vec4) -> Vec4
Returns the reflection vector for a given incident vector self and surface normal
normal.
normal must be normalized.
Panics
Will panic if normal is not normalized when glam_assert is enabled.
Sourcepub fn refract(self, normal: Vec4, eta: f32) -> Vec4
pub fn refract(self, normal: Vec4, eta: f32) -> Vec4
Returns the refraction direction for a given incident vector self, surface normal
normal and ratio of indices of refraction, eta. When total internal reflection occurs,
a zero vector will be returned.
self and normal must be normalized.
Panics
Will panic if self or normal is not normalized when glam_assert is enabled.
Sourcepub fn as_dvec4(self) -> DVec4
Available on crate feature f64 only.
pub fn as_dvec4(self) -> DVec4
f64 only.Casts all elements of self to f64.
Sourcepub fn as_i8vec4(self) -> I8Vec4
Available on crate feature i8 only.
pub fn as_i8vec4(self) -> I8Vec4
i8 only.Casts all elements of self to i8.
Sourcepub fn as_u8vec4(self) -> U8Vec4
Available on crate feature u8 only.
pub fn as_u8vec4(self) -> U8Vec4
u8 only.Casts all elements of self to u8.
Sourcepub fn as_i16vec4(self) -> I16Vec4
Available on crate feature i16 only.
pub fn as_i16vec4(self) -> I16Vec4
i16 only.Casts all elements of self to i16.
Sourcepub fn as_u16vec4(self) -> U16Vec4
Available on crate feature u16 only.
pub fn as_u16vec4(self) -> U16Vec4
u16 only.Casts all elements of self to u16.
Sourcepub fn as_ivec4(self) -> IVec4
Available on crate feature i32 only.
pub fn as_ivec4(self) -> IVec4
i32 only.Casts all elements of self to i32.
Sourcepub fn as_uvec4(self) -> UVec4
Available on crate feature u32 only.
pub fn as_uvec4(self) -> UVec4
u32 only.Casts all elements of self to u32.
Sourcepub fn as_i64vec4(self) -> I64Vec4
Available on crate feature i64 only.
pub fn as_i64vec4(self) -> I64Vec4
i64 only.Casts all elements of self to i64.
Sourcepub fn as_u64vec4(self) -> U64Vec4
Available on crate feature u64 only.
pub fn as_u64vec4(self) -> U64Vec4
u64 only.Casts all elements of self to u64.