pub struct BaseMeshPipelineKey(/* private fields */);bevy_mesh only.Expand description
Our base mesh pipeline key bits start from the highest bit and go
downward. The PBR mesh pipeline key bits start from the lowest bit and
go upward. This allows the PBR bits in the downstream crate bevy_pbr
to coexist in the same field without any shifts.
Implementations
Sourceimpl BaseMeshPipelineKey
impl BaseMeshPipelineKey
pub const MORPH_TARGETS: BaseMeshPipelineKey
pub const PRIMITIVE_TOPOLOGY_RESERVED_BITS: BaseMeshPipelineKey
pub const STRIP_INDEX_FORMAT_RESERVED_BITS: BaseMeshPipelineKey
pub const STRIP_INDEX_FORMAT_NONE: BaseMeshPipelineKey
pub const STRIP_INDEX_FORMAT_U32: BaseMeshPipelineKey
pub const STRIP_INDEX_FORMAT_U16: BaseMeshPipelineKey
Sourceimpl BaseMeshPipelineKey
impl BaseMeshPipelineKey
Sourcepub const fn empty() -> BaseMeshPipelineKey
pub const fn empty() -> BaseMeshPipelineKey
Get a flags value with all bits unset.
Sourcepub const fn all() -> BaseMeshPipelineKey
pub const fn all() -> BaseMeshPipelineKey
Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Examples found in repository?
565fn queue_colored_mesh2d(
566 transparent_draw_functions: Res<DrawFunctions<TransparentColoredMesh2d>>,
567 colored_mesh2d_pipeline: Res<ColoredMesh2dPipeline>,
568 mut pipelines: ResMut<SpecializedRenderPipelines<ColoredMesh2dPipeline>>,
569 pipeline_cache: Res<PipelineCache>,
570 render_meshes: Res<RenderAssets<RenderMesh>>,
571 render_mesh_instances: Res<RenderColoredMesh2dInstances>,
572 mut transparent_render_phases: ResMut<ViewSortedRenderPhases<TransparentColoredMesh2d>>,
573 views: Query<(&RenderVisibleEntities, &ExtractedView, &Msaa)>,
574) {
575 if render_mesh_instances.is_empty() {
576 return;
577 }
578
579 // Iterate each view (a camera is a view)
580 for (visible_entities, view, msaa) in &views {
581 let Some(transparent_phase) = transparent_render_phases.get_mut(&view.retained_view_entity)
582 else {
583 continue;
584 };
585
586 let draw_colored_mesh2d = transparent_draw_functions
587 .read()
588 .id::<DrawTransparentColoredMesh2d>();
589
590 let mesh_key = Mesh2dPipelineKey::from_msaa_samples(msaa.samples())
591 | Mesh2dPipelineKey::from_target_format(view.target_format);
592
593 // Queue all entities visible to that view
594 let Some(visible_entities) = visible_entities.get::<Mesh2d>() else {
595 continue;
596 };
597 for (render_entity, visible_entity) in visible_entities.iter_visible() {
598 if let Some(mesh_instance) = render_mesh_instances.get(visible_entity) {
599 let mesh2d_handle = mesh_instance.mesh_asset_id;
600 let mesh2d_transforms = &mesh_instance.transforms;
601 // Get our specialized pipeline
602 let mut mesh2d_key = mesh_key;
603 let Some(mesh) = render_meshes.get(mesh2d_handle) else {
604 continue;
605 };
606 mesh2d_key |= Mesh2dPipelineKey::from(
607 BaseMeshPipelineKey::from_primitive_topology_and_strip_index(
608 mesh.primitive_topology(),
609 mesh.index_format(),
610 )
611 .bits(),
612 );
613
614 let pipeline_id =
615 pipelines.specialize(&pipeline_cache, &colored_mesh2d_pipeline, mesh2d_key);
616
617 let mesh_z = mesh2d_transforms.world_from_local.translation.z;
618 transparent_phase.add_retained(TransparentColoredMesh2d {
619 entity: (*render_entity, *visible_entity),
620 draw_function: draw_colored_mesh2d,
621 pipeline: pipeline_id,
622 // The 2d render items are sorted according to their z value before rendering,
623 // in order to get correct transparency
624 sort_key: FloatOrd(mesh_z),
625 // This material is not batched
626 batch_range: 0..1,
627 extra_index: PhaseItemExtraIndex::None,
628 indexed: mesh.indexed(),
629 });
630 }
631 }
632 }
633}Sourcepub const fn from_bits(bits: u64) -> Option<BaseMeshPipelineKey>
pub const fn from_bits(bits: u64) -> Option<BaseMeshPipelineKey>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> BaseMeshPipelineKey
pub const fn from_bits_truncate(bits: u64) -> BaseMeshPipelineKey
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> BaseMeshPipelineKey
pub const fn from_bits_retain(bits: u64) -> BaseMeshPipelineKey
Convert from a bits value exactly.
Examples found in repository?
174 fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
175 // Customize how to store the meshes' vertex attributes in the vertex buffer
176 // Our meshes only have position and color
177 let formats = vec![
178 // Position
179 VertexFormat::Float32x3,
180 // Color
181 VertexFormat::Uint32,
182 ];
183
184 let vertex_layout =
185 VertexBufferLayout::from_vertex_formats(VertexStepMode::Vertex, formats);
186
187 let format = key.target_format();
188
189 RenderPipelineDescriptor {
190 vertex: VertexState {
191 // Use our custom shader
192 shader: self.shader.clone(),
193 // Use our custom vertex buffer
194 buffers: vec![vertex_layout],
195 ..default()
196 },
197 fragment: Some(FragmentState {
198 // Use our custom shader
199 shader: self.shader.clone(),
200 targets: vec![Some(ColorTargetState {
201 format,
202 blend: Some(BlendState::ALPHA_BLENDING),
203 write_mask: ColorWrites::ALL,
204 })],
205 ..default()
206 }),
207 // Use the two standard uniforms for 2d meshes
208 layout: vec![
209 // Bind group 0 is the view uniform
210 self.mesh2d_pipeline.view_layout.clone(),
211 // Bind group 1 is the mesh uniform
212 self.mesh2d_pipeline.mesh_layout.clone(),
213 ],
214 primitive: PrimitiveState {
215 cull_mode: Some(Face::Back),
216 topology: BaseMeshPipelineKey::from_bits_retain(key.bits()).primitive_topology(),
217 strip_index_format: BaseMeshPipelineKey::from_bits_retain(key.bits())
218 .strip_index_format(),
219 ..default()
220 },
221 depth_stencil: Some(DepthStencilState {
222 format: CORE_2D_DEPTH_FORMAT,
223 depth_write_enabled: Some(false),
224 depth_compare: Some(CompareFunction::GreaterEqual),
225 stencil: StencilState {
226 front: StencilFaceState::IGNORE,
227 back: StencilFaceState::IGNORE,
228 read_mask: 0,
229 write_mask: 0,
230 },
231 bias: DepthBiasState {
232 constant: 0,
233 slope_scale: 0.0,
234 clamp: 0.0,
235 },
236 }),
237 multisample: MultisampleState {
238 count: key.msaa_samples(),
239 mask: !0,
240 alpha_to_coverage_enabled: false,
241 },
242 label: Some("colored_mesh2d_pipeline".into()),
243 ..default()
244 }
245 }Sourcepub fn from_name(name: &str) -> Option<BaseMeshPipelineKey>
pub fn from_name(name: &str) -> Option<BaseMeshPipelineKey>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: BaseMeshPipelineKey) -> bool
pub const fn intersects(&self, other: BaseMeshPipelineKey) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: BaseMeshPipelineKey) -> bool
pub const fn contains(&self, other: BaseMeshPipelineKey) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn insert(&mut self, other: BaseMeshPipelineKey)
pub fn insert(&mut self, other: BaseMeshPipelineKey)
The bitwise or (|) of the bits in self and other.
Sourcepub fn remove(&mut self, other: BaseMeshPipelineKey)
pub fn remove(&mut self, other: BaseMeshPipelineKey)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: BaseMeshPipelineKey)
pub fn toggle(&mut self, other: BaseMeshPipelineKey)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: BaseMeshPipelineKey, value: bool)
pub fn set(&mut self, other: BaseMeshPipelineKey, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(
self,
other: BaseMeshPipelineKey,
) -> BaseMeshPipelineKey
pub const fn intersection( self, other: BaseMeshPipelineKey, ) -> BaseMeshPipelineKey
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
pub const fn union(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
pub const fn difference(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(
self,
other: BaseMeshPipelineKey,
) -> BaseMeshPipelineKey
pub const fn symmetric_difference( self, other: BaseMeshPipelineKey, ) -> BaseMeshPipelineKey
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> BaseMeshPipelineKey
pub const fn complement(self) -> BaseMeshPipelineKey
The bitwise negation (!) of the bits in self, truncating the result.
Sourceimpl BaseMeshPipelineKey
impl BaseMeshPipelineKey
Sourcepub const fn iter(&self) -> Iter<BaseMeshPipelineKey> ⓘ
pub const fn iter(&self) -> Iter<BaseMeshPipelineKey> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<BaseMeshPipelineKey> ⓘ
pub const fn iter_names(&self) -> IterNames<BaseMeshPipelineKey> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.
Sourceimpl BaseMeshPipelineKey
impl BaseMeshPipelineKey
pub const MORPH_TARGETS_SHIFT_BITS: u64
pub const PRIMITIVE_TOPOLOGY_MASK_BITS: u64 = 0b111
pub const PRIMITIVE_TOPOLOGY_SHIFT_BITS: u64
pub const STRIP_INDEX_FORMAT_MASK_BITS: u64 = 0b11
pub const STRIP_INDEX_FORMAT_SHIFT_BITS: u64
Sourcepub fn from_primitive_topology_and_strip_index(
primitive_topology: PrimitiveTopology,
strip_index_format: Option<IndexFormat>,
) -> BaseMeshPipelineKey
pub fn from_primitive_topology_and_strip_index( primitive_topology: PrimitiveTopology, strip_index_format: Option<IndexFormat>, ) -> BaseMeshPipelineKey
Create a BaseMeshPipelineKey from mesh primitive topology and index format.
For non-strip topologies, Self::STRIP_INDEX_FORMAT_NONE is set regardless of the strip_index_format argument.
Examples found in repository?
565fn queue_colored_mesh2d(
566 transparent_draw_functions: Res<DrawFunctions<TransparentColoredMesh2d>>,
567 colored_mesh2d_pipeline: Res<ColoredMesh2dPipeline>,
568 mut pipelines: ResMut<SpecializedRenderPipelines<ColoredMesh2dPipeline>>,
569 pipeline_cache: Res<PipelineCache>,
570 render_meshes: Res<RenderAssets<RenderMesh>>,
571 render_mesh_instances: Res<RenderColoredMesh2dInstances>,
572 mut transparent_render_phases: ResMut<ViewSortedRenderPhases<TransparentColoredMesh2d>>,
573 views: Query<(&RenderVisibleEntities, &ExtractedView, &Msaa)>,
574) {
575 if render_mesh_instances.is_empty() {
576 return;
577 }
578
579 // Iterate each view (a camera is a view)
580 for (visible_entities, view, msaa) in &views {
581 let Some(transparent_phase) = transparent_render_phases.get_mut(&view.retained_view_entity)
582 else {
583 continue;
584 };
585
586 let draw_colored_mesh2d = transparent_draw_functions
587 .read()
588 .id::<DrawTransparentColoredMesh2d>();
589
590 let mesh_key = Mesh2dPipelineKey::from_msaa_samples(msaa.samples())
591 | Mesh2dPipelineKey::from_target_format(view.target_format);
592
593 // Queue all entities visible to that view
594 let Some(visible_entities) = visible_entities.get::<Mesh2d>() else {
595 continue;
596 };
597 for (render_entity, visible_entity) in visible_entities.iter_visible() {
598 if let Some(mesh_instance) = render_mesh_instances.get(visible_entity) {
599 let mesh2d_handle = mesh_instance.mesh_asset_id;
600 let mesh2d_transforms = &mesh_instance.transforms;
601 // Get our specialized pipeline
602 let mut mesh2d_key = mesh_key;
603 let Some(mesh) = render_meshes.get(mesh2d_handle) else {
604 continue;
605 };
606 mesh2d_key |= Mesh2dPipelineKey::from(
607 BaseMeshPipelineKey::from_primitive_topology_and_strip_index(
608 mesh.primitive_topology(),
609 mesh.index_format(),
610 )
611 .bits(),
612 );
613
614 let pipeline_id =
615 pipelines.specialize(&pipeline_cache, &colored_mesh2d_pipeline, mesh2d_key);
616
617 let mesh_z = mesh2d_transforms.world_from_local.translation.z;
618 transparent_phase.add_retained(TransparentColoredMesh2d {
619 entity: (*render_entity, *visible_entity),
620 draw_function: draw_colored_mesh2d,
621 pipeline: pipeline_id,
622 // The 2d render items are sorted according to their z value before rendering,
623 // in order to get correct transparency
624 sort_key: FloatOrd(mesh_z),
625 // This material is not batched
626 batch_range: 0..1,
627 extra_index: PhaseItemExtraIndex::None,
628 indexed: mesh.indexed(),
629 });
630 }
631 }
632 }
633}Sourcepub fn primitive_topology(&self) -> PrimitiveTopology
pub fn primitive_topology(&self) -> PrimitiveTopology
Examples found in repository?
174 fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
175 // Customize how to store the meshes' vertex attributes in the vertex buffer
176 // Our meshes only have position and color
177 let formats = vec![
178 // Position
179 VertexFormat::Float32x3,
180 // Color
181 VertexFormat::Uint32,
182 ];
183
184 let vertex_layout =
185 VertexBufferLayout::from_vertex_formats(VertexStepMode::Vertex, formats);
186
187 let format = key.target_format();
188
189 RenderPipelineDescriptor {
190 vertex: VertexState {
191 // Use our custom shader
192 shader: self.shader.clone(),
193 // Use our custom vertex buffer
194 buffers: vec![vertex_layout],
195 ..default()
196 },
197 fragment: Some(FragmentState {
198 // Use our custom shader
199 shader: self.shader.clone(),
200 targets: vec![Some(ColorTargetState {
201 format,
202 blend: Some(BlendState::ALPHA_BLENDING),
203 write_mask: ColorWrites::ALL,
204 })],
205 ..default()
206 }),
207 // Use the two standard uniforms for 2d meshes
208 layout: vec![
209 // Bind group 0 is the view uniform
210 self.mesh2d_pipeline.view_layout.clone(),
211 // Bind group 1 is the mesh uniform
212 self.mesh2d_pipeline.mesh_layout.clone(),
213 ],
214 primitive: PrimitiveState {
215 cull_mode: Some(Face::Back),
216 topology: BaseMeshPipelineKey::from_bits_retain(key.bits()).primitive_topology(),
217 strip_index_format: BaseMeshPipelineKey::from_bits_retain(key.bits())
218 .strip_index_format(),
219 ..default()
220 },
221 depth_stencil: Some(DepthStencilState {
222 format: CORE_2D_DEPTH_FORMAT,
223 depth_write_enabled: Some(false),
224 depth_compare: Some(CompareFunction::GreaterEqual),
225 stencil: StencilState {
226 front: StencilFaceState::IGNORE,
227 back: StencilFaceState::IGNORE,
228 read_mask: 0,
229 write_mask: 0,
230 },
231 bias: DepthBiasState {
232 constant: 0,
233 slope_scale: 0.0,
234 clamp: 0.0,
235 },
236 }),
237 multisample: MultisampleState {
238 count: key.msaa_samples(),
239 mask: !0,
240 alpha_to_coverage_enabled: false,
241 },
242 label: Some("colored_mesh2d_pipeline".into()),
243 ..default()
244 }
245 }Sourcepub fn strip_index_format(&self) -> Option<IndexFormat>
pub fn strip_index_format(&self) -> Option<IndexFormat>
Returns the IndexFormat that this mesh pipeline key specifies.
Examples found in repository?
174 fn specialize(&self, key: Self::Key) -> RenderPipelineDescriptor {
175 // Customize how to store the meshes' vertex attributes in the vertex buffer
176 // Our meshes only have position and color
177 let formats = vec![
178 // Position
179 VertexFormat::Float32x3,
180 // Color
181 VertexFormat::Uint32,
182 ];
183
184 let vertex_layout =
185 VertexBufferLayout::from_vertex_formats(VertexStepMode::Vertex, formats);
186
187 let format = key.target_format();
188
189 RenderPipelineDescriptor {
190 vertex: VertexState {
191 // Use our custom shader
192 shader: self.shader.clone(),
193 // Use our custom vertex buffer
194 buffers: vec![vertex_layout],
195 ..default()
196 },
197 fragment: Some(FragmentState {
198 // Use our custom shader
199 shader: self.shader.clone(),
200 targets: vec![Some(ColorTargetState {
201 format,
202 blend: Some(BlendState::ALPHA_BLENDING),
203 write_mask: ColorWrites::ALL,
204 })],
205 ..default()
206 }),
207 // Use the two standard uniforms for 2d meshes
208 layout: vec![
209 // Bind group 0 is the view uniform
210 self.mesh2d_pipeline.view_layout.clone(),
211 // Bind group 1 is the mesh uniform
212 self.mesh2d_pipeline.mesh_layout.clone(),
213 ],
214 primitive: PrimitiveState {
215 cull_mode: Some(Face::Back),
216 topology: BaseMeshPipelineKey::from_bits_retain(key.bits()).primitive_topology(),
217 strip_index_format: BaseMeshPipelineKey::from_bits_retain(key.bits())
218 .strip_index_format(),
219 ..default()
220 },
221 depth_stencil: Some(DepthStencilState {
222 format: CORE_2D_DEPTH_FORMAT,
223 depth_write_enabled: Some(false),
224 depth_compare: Some(CompareFunction::GreaterEqual),
225 stencil: StencilState {
226 front: StencilFaceState::IGNORE,
227 back: StencilFaceState::IGNORE,
228 read_mask: 0,
229 write_mask: 0,
230 },
231 bias: DepthBiasState {
232 constant: 0,
233 slope_scale: 0.0,
234 clamp: 0.0,
235 },
236 }),
237 multisample: MultisampleState {
238 count: key.msaa_samples(),
239 mask: !0,
240 alpha_to_coverage_enabled: false,
241 },
242 label: Some("colored_mesh2d_pipeline".into()),
243 ..default()
244 }
245 }Trait Implementations
Sourceimpl Binary for BaseMeshPipelineKey
impl Binary for BaseMeshPipelineKey
Sourceimpl BitAnd for BaseMeshPipelineKey
impl BitAnd for BaseMeshPipelineKey
Sourcefn bitand(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
fn bitand(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The bitwise and (&) of the bits in self and other.
Sourcetype Output = BaseMeshPipelineKey
type Output = BaseMeshPipelineKey
& operator.Sourceimpl BitAndAssign for BaseMeshPipelineKey
impl BitAndAssign for BaseMeshPipelineKey
Sourcefn bitand_assign(&mut self, other: BaseMeshPipelineKey)
fn bitand_assign(&mut self, other: BaseMeshPipelineKey)
The bitwise and (&) of the bits in self and other.
Sourceimpl BitOr for BaseMeshPipelineKey
impl BitOr for BaseMeshPipelineKey
Sourcefn bitor(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
fn bitor(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The bitwise or (|) of the bits in self and other.
Sourcetype Output = BaseMeshPipelineKey
type Output = BaseMeshPipelineKey
| operator.Sourceimpl BitOrAssign for BaseMeshPipelineKey
impl BitOrAssign for BaseMeshPipelineKey
Sourcefn bitor_assign(&mut self, other: BaseMeshPipelineKey)
fn bitor_assign(&mut self, other: BaseMeshPipelineKey)
The bitwise or (|) of the bits in self and other.
Sourceimpl BitXor for BaseMeshPipelineKey
impl BitXor for BaseMeshPipelineKey
Sourcefn bitxor(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
fn bitxor(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The bitwise exclusive-or (^) of the bits in self and other.
Sourcetype Output = BaseMeshPipelineKey
type Output = BaseMeshPipelineKey
^ operator.Sourceimpl BitXorAssign for BaseMeshPipelineKey
impl BitXorAssign for BaseMeshPipelineKey
Sourcefn bitxor_assign(&mut self, other: BaseMeshPipelineKey)
fn bitxor_assign(&mut self, other: BaseMeshPipelineKey)
The bitwise exclusive-or (^) of the bits in self and other.
Sourceimpl Clone for BaseMeshPipelineKey
impl Clone for BaseMeshPipelineKey
Sourcefn clone(&self) -> BaseMeshPipelineKey
fn clone(&self) -> BaseMeshPipelineKey
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for BaseMeshPipelineKey
impl Debug for BaseMeshPipelineKey
Sourceimpl Extend<BaseMeshPipelineKey> for BaseMeshPipelineKey
impl Extend<BaseMeshPipelineKey> for BaseMeshPipelineKey
Sourcefn extend<T>(&mut self, iterator: T)where
T: IntoIterator<Item = BaseMeshPipelineKey>,
fn extend<T>(&mut self, iterator: T)where
T: IntoIterator<Item = BaseMeshPipelineKey>,
The bitwise or (|) of the bits in each flags value.
Sourcefn extend_one(&mut self, item: T)
fn extend_one(&mut self, item: T)
extend_one)Sourcefn extend_reserve(&mut self, additional: usize)
fn extend_reserve(&mut self, additional: usize)
extend_one)Sourceimpl Flags for BaseMeshPipelineKey
impl Flags for BaseMeshPipelineKey
Sourceconst FLAGS: &'static [Flag<BaseMeshPipelineKey>]
const FLAGS: &'static [Flag<BaseMeshPipelineKey>]
Sourcefn from_bits_retain(bits: u64) -> BaseMeshPipelineKey
fn from_bits_retain(bits: u64) -> BaseMeshPipelineKey
Sourcefn all_named() -> BaseMeshPipelineKey
fn all_named() -> BaseMeshPipelineKey
Sourcefn known_bits(&self) -> Self::Bits
fn known_bits(&self) -> Self::Bits
Sourcefn unknown_bits(&self) -> Self::Bits
fn unknown_bits(&self) -> Self::Bits
Sourcefn contains_unknown_bits(&self) -> bool
fn contains_unknown_bits(&self) -> bool
true if any unknown bits are set.Sourcefn from_bits_truncate(bits: Self::Bits) -> Self
fn from_bits_truncate(bits: Self::Bits) -> Self
Sourcefn from_name(name: &str) -> Option<Self>
fn from_name(name: &str) -> Option<Self>
Sourcefn iter_names(&self) -> IterNames<Self> ⓘ
fn iter_names(&self) -> IterNames<Self> ⓘ
Sourcefn iter_defined_names() -> IterDefinedNames<Self> ⓘ
fn iter_defined_names() -> IterDefinedNames<Self> ⓘ
Self::FLAGS.Sourcefn iter_equal_names(&self) -> IterEqualNames<Self> ⓘ
fn iter_equal_names(&self) -> IterEqualNames<Self> ⓘ
Sourcefn intersects(&self, other: Self) -> boolwhere
Self: Sized,
fn intersects(&self, other: Self) -> boolwhere
Self: Sized,
other are also set in self.Sourcefn contains(&self, other: Self) -> boolwhere
Self: Sized,
fn contains(&self, other: Self) -> boolwhere
Self: Sized,
other are also set in self.Sourcefn insert(&mut self, other: Self)where
Self: Sized,
fn insert(&mut self, other: Self)where
Self: Sized,
|) of the bits in self and other.Sourcefn toggle(&mut self, other: Self)where
Self: Sized,
fn toggle(&mut self, other: Self)where
Self: Sized,
^) of the bits in self and other.Sourcefn intersection(self, other: Self) -> Self
fn intersection(self, other: Self) -> Self
&) of the bits in self and other.Sourcefn difference(self, other: Self) -> Self
fn difference(self, other: Self) -> Self
Sourcefn symmetric_difference(self, other: Self) -> Self
fn symmetric_difference(self, other: Self) -> Self
^) of the bits in self and other.Sourcefn complement(self) -> Self
fn complement(self) -> Self
!) of the bits in self, truncating the result.Sourceimpl FromIterator<BaseMeshPipelineKey> for BaseMeshPipelineKey
impl FromIterator<BaseMeshPipelineKey> for BaseMeshPipelineKey
Sourcefn from_iter<T>(iterator: T) -> BaseMeshPipelineKeywhere
T: IntoIterator<Item = BaseMeshPipelineKey>,
fn from_iter<T>(iterator: T) -> BaseMeshPipelineKeywhere
T: IntoIterator<Item = BaseMeshPipelineKey>,
The bitwise or (|) of the bits in each flags value.
Sourceimpl IntoIterator for BaseMeshPipelineKey
impl IntoIterator for BaseMeshPipelineKey
Sourcetype Item = BaseMeshPipelineKey
type Item = BaseMeshPipelineKey
Sourcetype IntoIter = Iter<BaseMeshPipelineKey>
type IntoIter = Iter<BaseMeshPipelineKey>
Sourcefn into_iter(self) -> <BaseMeshPipelineKey as IntoIterator>::IntoIter
fn into_iter(self) -> <BaseMeshPipelineKey as IntoIterator>::IntoIter
Sourceimpl LowerHex for BaseMeshPipelineKey
impl LowerHex for BaseMeshPipelineKey
Sourceimpl Not for BaseMeshPipelineKey
impl Not for BaseMeshPipelineKey
Sourcefn not(self) -> BaseMeshPipelineKey
fn not(self) -> BaseMeshPipelineKey
The bitwise negation (!) of the bits in self, truncating the result.
Sourcetype Output = BaseMeshPipelineKey
type Output = BaseMeshPipelineKey
! operator.Sourceimpl Octal for BaseMeshPipelineKey
impl Octal for BaseMeshPipelineKey
Sourceimpl Sub for BaseMeshPipelineKey
impl Sub for BaseMeshPipelineKey
Sourcefn sub(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
fn sub(self, other: BaseMeshPipelineKey) -> BaseMeshPipelineKey
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcetype Output = BaseMeshPipelineKey
type Output = BaseMeshPipelineKey
- operator.Sourceimpl SubAssign for BaseMeshPipelineKey
impl SubAssign for BaseMeshPipelineKey
Sourcefn sub_assign(&mut self, other: BaseMeshPipelineKey)
fn sub_assign(&mut self, other: BaseMeshPipelineKey)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.