pub struct Hovered(pub bool);Expand description
A component that allows users to use regular Bevy change detection to determine when the pointer enters or leaves an entity. Users should insert this component on an entity to indicate interest in knowing about hover state changes.
The component’s boolean value will be true whenever the pointer is currently directly hovering
over the entity, or any of the entity’s descendants (as defined by the ChildOf
relationship). This is consistent with the behavior of the CSS :hover pseudo-class, which
applies to the element and all of its descendants.
The contained boolean value is guaranteed to only be mutated when the pointer enters or leaves
the entity, allowing Bevy change detection to be used efficiently. This is in contrast to the
HoverMap resource, which is updated every frame.
Typically, a simple hoverable entity or widget will have this component added to it. More complex widgets can have this component added to each hoverable part.
The computational cost of keeping the Hovered components up to date is relatively cheap, and
linear in the number of entities that have the Hovered component inserted.
Tuple Fields
0: boolImplementations