pub struct MeshRayCast<'w, 's> { /* private fields */ }Expand description
Add this ray casting SystemParam to your system to cast rays into the world with an
immediate-mode API. Call cast_ray to immediately perform a ray cast and get a result.
Under the hood, this is a collection of regular bevy queries, resources, and local parameters that are added to your system.
Usage
The following system casts a ray into the world with the ray positioned at the origin, pointing in the X-direction, and returns a list of intersections:
fn ray_cast_system(mut ray_cast: MeshRayCast) {
let ray = Ray3d::new(Vec3::ZERO, Dir3::X);
let hits = ray_cast.cast_ray(ray, &MeshRayCastSettings::default());
}Configuration
You can specify the behavior of the ray cast using MeshRayCastSettings. This allows you to filter out
entities, configure early-out behavior, and set whether the Visibility
of an entity should be considered.
fn ray_cast_system(mut ray_cast: MeshRayCast, foo_query: Query<(), With<Foo>>) {
let ray = Ray3d::new(Vec3::ZERO, Dir3::X);
// Only ray cast against entities with the `Foo` component.
let filter = |entity| foo_query.contains(entity);
// Never early-exit. Note that you can change behavior per-entity.
let early_exit_test = |_entity| false;
// Ignore the visibility of entities. This allows ray casting hidden entities.
let visibility = RayCastVisibility::Any;
let settings = MeshRayCastSettings::default()
.with_filter(&filter)
.with_early_exit_test(&early_exit_test)
.with_visibility(visibility);
// Cast the ray with the settings, returning a list of intersections.
let hits = ray_cast.cast_ray(ray, &settings);
}Implementations
Sourceimpl<'w, 's> MeshRayCast<'w, 's>
impl<'w, 's> MeshRayCast<'w, 's>
Sourcepub fn cast_ray(
&mut self,
ray: Ray3d,
settings: &MeshRayCastSettings<'_>,
) -> &[(Entity, RayMeshHit)]
pub fn cast_ray( &mut self, ray: Ray3d, settings: &MeshRayCastSettings<'_>, ) -> &[(Entity, RayMeshHit)]
Casts the ray into the world and returns a sorted list of intersections, nearest first.
Examples found in repository?
examples/3d/mesh_ray_cast.rs (line 52)
45fn bounce_ray(mut ray: Ray3d, ray_cast: &mut MeshRayCast, gizmos: &mut Gizmos, color: Color) {
46 let mut intersections = Vec::with_capacity(MAX_BOUNCES + 1);
47 intersections.push((ray.origin, Color::srgb(30.0, 0.0, 0.0)));
48
49 for i in 0..MAX_BOUNCES {
50 // Cast the ray and get the first hit
51 let Some((_, hit)) = ray_cast
52 .cast_ray(ray, &MeshRayCastSettings::default())
53 .first()
54 else {
55 break;
56 };
57
58 // Draw the point of intersection and add it to the list
59 let brightness = 1.0 + 10.0 * (1.0 - i as f32 / MAX_BOUNCES as f32);
60 intersections.push((hit.point, Color::BLACK.mix(&color, brightness)));
61 gizmos.sphere(hit.point, 0.005, Color::BLACK.mix(&color, brightness * 2.0));
62
63 // Reflect the ray off of the surface
64 ray.direction = Dir3::new(ray.direction.reflect(hit.normal)).unwrap();
65 ray.origin = hit.point + ray.direction * 1e-6;
66 }
67 gizmos.linestrip_gradient(intersections);
68}More examples
examples/ui/render_ui_to_texture.rs (line 206)
174fn drive_diegetic_pointer(
175 mut cursor_last: Local<Vec2>,
176 mut raycast: MeshRayCast,
177 rays: Res<RayMap>,
178 cubes: Query<&Mesh3d, With<Cube>>,
179 ui_camera: Query<&Camera, With<Camera2d>>,
180 primary_window: Query<Entity, With<PrimaryWindow>>,
181 windows: Query<(Entity, &Window)>,
182 images: Res<Assets<Image>>,
183 manual_texture_views: Res<ManualTextureViews>,
184 mut window_events: MessageReader<WindowEvent>,
185 mut pointer_inputs: MessageWriter<PointerInput>,
186) -> Result {
187 // Get the size of the texture, so we can convert from dimensionless UV coordinates that span
188 // from 0 to 1, to pixel coordinates.
189 let target = ui_camera
190 .single()?
191 .target
192 .normalize(primary_window.single().ok())
193 .unwrap();
194 let target_info = target
195 .get_render_target_info(windows, &images, &manual_texture_views)
196 .unwrap();
197 let size = target_info.physical_size.as_vec2();
198
199 // Find raycast hits and update the virtual pointer.
200 let raycast_settings = MeshRayCastSettings {
201 visibility: RayCastVisibility::VisibleInView,
202 filter: &|entity| cubes.contains(entity),
203 early_exit_test: &|_| false,
204 };
205 for (_id, ray) in rays.iter() {
206 for (_cube, hit) in raycast.cast_ray(*ray, &raycast_settings) {
207 let position = size * hit.uv.unwrap();
208 if position != *cursor_last {
209 pointer_inputs.write(PointerInput::new(
210 CUBE_POINTER_ID,
211 Location {
212 target: target.clone(),
213 position,
214 },
215 PointerAction::Move {
216 delta: position - *cursor_last,
217 },
218 ));
219 *cursor_last = position;
220 }
221 }
222 }
223
224 // Pipe pointer button presses to the virtual pointer on the UI texture.
225 for window_event in window_events.read() {
226 if let WindowEvent::MouseButtonInput(input) = window_event {
227 let button = match input.button {
228 MouseButton::Left => PointerButton::Primary,
229 MouseButton::Right => PointerButton::Secondary,
230 MouseButton::Middle => PointerButton::Middle,
231 _ => continue,
232 };
233 let action = match input.state {
234 ButtonState::Pressed => PointerAction::Press(button),
235 ButtonState::Released => PointerAction::Release(button),
236 };
237 pointer_inputs.write(PointerInput::new(
238 CUBE_POINTER_ID,
239 Location {
240 target: target.clone(),
241 position: *cursor_last,
242 },
243 action,
244 ));
245 }
246 }
247
248 Ok(())
249}Trait Implementations
Sourceimpl SystemParam for MeshRayCast<'_, '_>
impl SystemParam for MeshRayCast<'_, '_>
Sourcetype Item<'w, 's> = MeshRayCast<'w, 's>
type Item<'w, 's> = MeshRayCast<'w, 's>
The item type returned when constructing this system param.
The value of this associated type should be
Self, instantiated with new lifetimes. Read moreSourcefn init_state(world: &mut World) -> <MeshRayCast<'_, '_> as SystemParam>::State
fn init_state(world: &mut World) -> <MeshRayCast<'_, '_> as SystemParam>::State
Creates a new instance of this param’s
State.Sourcefn init_access(
state: &<MeshRayCast<'_, '_> as SystemParam>::State,
system_meta: &mut SystemMeta,
component_access_set: &mut FilteredAccessSet,
world: &mut World,
)
fn init_access( state: &<MeshRayCast<'_, '_> as SystemParam>::State, system_meta: &mut SystemMeta, component_access_set: &mut FilteredAccessSet, world: &mut World, )
Registers any
World access used by this SystemParamSourcefn apply(
state: &mut <MeshRayCast<'_, '_> as SystemParam>::State,
system_meta: &SystemMeta,
world: &mut World,
)
fn apply( state: &mut <MeshRayCast<'_, '_> as SystemParam>::State, system_meta: &SystemMeta, world: &mut World, )
Applies any deferred mutations stored in this
SystemParam’s state.
This is used to apply Commands during ApplyDeferred.Sourcefn queue(
state: &mut <MeshRayCast<'_, '_> as SystemParam>::State,
system_meta: &SystemMeta,
world: DeferredWorld<'_>,
)
fn queue( state: &mut <MeshRayCast<'_, '_> as SystemParam>::State, system_meta: &SystemMeta, world: DeferredWorld<'_>, )
Queues any deferred mutations to be applied at the next
ApplyDeferred.Sourceunsafe fn validate_param<'w, 's>(
state: &'s mut <MeshRayCast<'_, '_> as SystemParam>::State,
_system_meta: &SystemMeta,
_world: UnsafeWorldCell<'w>,
) -> Result<(), SystemParamValidationError>
unsafe fn validate_param<'w, 's>( state: &'s mut <MeshRayCast<'_, '_> as SystemParam>::State, _system_meta: &SystemMeta, _world: UnsafeWorldCell<'w>, ) -> Result<(), SystemParamValidationError>
Sourceunsafe fn get_param<'w, 's>(
state: &'s mut <MeshRayCast<'_, '_> as SystemParam>::State,
system_meta: &SystemMeta,
world: UnsafeWorldCell<'w>,
change_tick: Tick,
) -> <MeshRayCast<'_, '_> as SystemParam>::Item<'w, 's>
unsafe fn get_param<'w, 's>( state: &'s mut <MeshRayCast<'_, '_> as SystemParam>::State, system_meta: &SystemMeta, world: UnsafeWorldCell<'w>, change_tick: Tick, ) -> <MeshRayCast<'_, '_> as SystemParam>::Item<'w, 's>
Creates a parameter to be passed into a
SystemParamFunction. Read moreimpl<'w, 's> ReadOnlySystemParam for MeshRayCast<'w, 's>where
Res<'w, Assets<Mesh>>: ReadOnlySystemParam,
Local<'s, Vec<(FloatOrd, (Entity, RayMeshHit))>>: ReadOnlySystemParam,
Local<'s, Vec<(Entity, RayMeshHit)>>: ReadOnlySystemParam,
Local<'s, Vec<(FloatOrd, Entity)>>: ReadOnlySystemParam,
Query<'w, 's, (&'static InheritedVisibility, &'static ViewVisibility, &'static Aabb, &'static GlobalTransform, Entity), Or<(With<Mesh3d>, With<Mesh2d>, With<SimplifiedMesh>)>>: ReadOnlySystemParam,
Query<'w, 's, (Option<&'static Mesh2d>, Option<&'static Mesh3d>, Option<&'static SimplifiedMesh>, Has<RayCastBackfaces>, &'static GlobalTransform), Or<(With<Mesh3d>, With<Mesh2d>, With<SimplifiedMesh>)>>: ReadOnlySystemParam,