pub enum AssetId<A>where
A: Asset,{
Index {
index: AssetIndex,
marker: PhantomData<fn() -> A>,
},
Uuid {
uuid: Uuid,
},
}Expand description
A unique runtime-only identifier for an Asset. This is cheap to Copy/Clone and is not directly tied to the
lifetime of the Asset. This means it can point to an Asset that no longer exists.
For an identifier tied to the lifetime of an asset, see Handle.
For an “untyped” / “generic-less” id, see UntypedAssetId.
Variants
Index
A small / efficient runtime identifier that can be used to efficiently look up an asset stored in Assets. This is
the “default” identifier used for assets. The alternative(s) (ex: AssetId::Uuid) will only be used if assets are
explicitly registered that way.
Fields
index: AssetIndexThe unstable, opaque index of the asset.
marker: PhantomData<fn() -> A>A marker to store the type information of the asset.
Uuid
A stable-across-runs / const asset identifier. This will only be used if an asset is explicitly registered in Assets
with one.
Implementations
Sourceimpl<A> AssetId<A>where
A: Asset,
impl<A> AssetId<A>where
A: Asset,
Sourcepub const DEFAULT_UUID: Uuid
pub const DEFAULT_UUID: Uuid
Sourcepub const INVALID_UUID: Uuid
pub const INVALID_UUID: Uuid
This asset id should never be valid. Assigning a value to this in Assets will
produce undefined behavior, so don’t do it!
Sourcepub const fn invalid() -> AssetId<A>
pub const fn invalid() -> AssetId<A>
Returns an AssetId with Self::INVALID_UUID, which should never be assigned to.
Examples found in repository?
227fn queue_custom_phase_item(
228 pipeline_cache: Res<PipelineCache>,
229 mut pipeline: ResMut<CustomPhasePipeline>,
230 mut opaque_render_phases: ResMut<ViewBinnedRenderPhases<Opaque3d>>,
231 opaque_draw_functions: Res<DrawFunctions<Opaque3d>>,
232 views: Query<(&ExtractedView, &RenderVisibleEntities, &Msaa)>,
233 dirty_specializations: Res<DirtySpecializations>,
234 mut pending_custom_phase_item_queues: ResMut<PendingCustomPhaseItemQueues>,
235) {
236 let draw_custom_phase_item = opaque_draw_functions
237 .read()
238 .id::<DrawCustomPhaseItemCommands>();
239
240 // Render phases are per-view, so we need to iterate over all views so that
241 // the entity appears in them. (In this example, we have only one view, but
242 // it's good practice to loop over all views anyway.)
243 for (view, view_visible_entities, msaa) in views.iter() {
244 let Some(opaque_phase) = opaque_render_phases.get_mut(&view.retained_view_entity) else {
245 continue;
246 };
247
248 // Fetch the list of visible entities in the `CustomRenderedEntity`
249 // class. If there are no such entities, then we have no entities to
250 // render, and we're done.
251 let Some(render_visible_mesh_entities) =
252 view_visible_entities.get::<CustomRenderedEntity>()
253 else {
254 continue;
255 };
256
257 let view_pending_custom_phase_item_queues =
258 pending_custom_phase_item_queues.prepare_for_new_frame(view.retained_view_entity);
259
260 // First, remove meshes that need to be respecialized, and those that
261 // were removed, from the bins.
262 for &main_entity in dirty_specializations
263 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
264 {
265 opaque_phase.remove(main_entity);
266 }
267
268 // Find all the custom rendered entities that are visible from this
269 // view.
270 for (render_entity, main_entity) in dirty_specializations.iter_to_queue(
271 view.retained_view_entity,
272 render_visible_mesh_entities,
273 &view_pending_custom_phase_item_queues.prev_frame,
274 ) {
275 // Ordinarily, the [`SpecializedRenderPipeline::Key`] would contain
276 // some per-view settings, such as whether the view is HDR, but for
277 // simplicity's sake we simply hard-code the view's characteristics,
278 // with the exception of number of MSAA samples.
279 let Ok(pipeline_id) = pipeline
280 .variants
281 .specialize(&pipeline_cache, CustomPhaseKey(*msaa))
282 else {
283 continue;
284 };
285
286 // Add the custom render item. We use the
287 // [`BinnedRenderPhaseType::NonMesh`] type to skip the special
288 // handling that Bevy has for meshes (preprocessing, indirect
289 // draws, etc.)
290 //
291 // The asset ID is arbitrary; we simply use [`AssetId::invalid`],
292 // but you can use anything you like. Note that the asset ID need
293 // not be the ID of a [`Mesh`].
294 opaque_phase.add(
295 Opaque3dBatchSetKey {
296 draw_function: draw_custom_phase_item,
297 pipeline: pipeline_id,
298 material_bind_group_index: None,
299 lightmap_slab: None,
300 slabs: MeshSlabs::default(),
301 },
302 Opaque3dBinKey {
303 asset_id: AssetId::<Mesh>::invalid().untyped(),
304 },
305 (*render_entity, *main_entity),
306 InputUniformIndex::default(),
307 BinnedRenderPhaseType::NonMesh,
308 );
309 }
310 }
311}Sourcepub fn untyped(self) -> UntypedAssetId
pub fn untyped(self) -> UntypedAssetId
Converts this to an “untyped” / “generic-less” Asset identifier that stores the type information
inside the UntypedAssetId.
Examples found in repository?
242fn extract_image_materials(
243 mut material_instances: ResMut<RenderMaterialInstances>,
244 changed_meshes_query: Extract<
245 Query<
246 (Entity, &ViewVisibility, &ImageMaterial3d),
247 Or<(Changed<ViewVisibility>, Changed<ImageMaterial3d>)>,
248 >,
249 >,
250) {
251 let last_change_tick = material_instances.current_change_tick;
252
253 for (entity, view_visibility, material) in &changed_meshes_query {
254 if view_visibility.get() {
255 material_instances.instances.insert(
256 entity.into(),
257 RenderMaterialInstance {
258 asset_id: material.0.id().untyped(),
259 last_change_tick,
260 },
261 );
262 } else {
263 material_instances
264 .instances
265 .remove(&MainEntity::from(entity));
266 }
267 }
268}More examples
227fn queue_custom_phase_item(
228 pipeline_cache: Res<PipelineCache>,
229 mut pipeline: ResMut<CustomPhasePipeline>,
230 mut opaque_render_phases: ResMut<ViewBinnedRenderPhases<Opaque3d>>,
231 opaque_draw_functions: Res<DrawFunctions<Opaque3d>>,
232 views: Query<(&ExtractedView, &RenderVisibleEntities, &Msaa)>,
233 dirty_specializations: Res<DirtySpecializations>,
234 mut pending_custom_phase_item_queues: ResMut<PendingCustomPhaseItemQueues>,
235) {
236 let draw_custom_phase_item = opaque_draw_functions
237 .read()
238 .id::<DrawCustomPhaseItemCommands>();
239
240 // Render phases are per-view, so we need to iterate over all views so that
241 // the entity appears in them. (In this example, we have only one view, but
242 // it's good practice to loop over all views anyway.)
243 for (view, view_visible_entities, msaa) in views.iter() {
244 let Some(opaque_phase) = opaque_render_phases.get_mut(&view.retained_view_entity) else {
245 continue;
246 };
247
248 // Fetch the list of visible entities in the `CustomRenderedEntity`
249 // class. If there are no such entities, then we have no entities to
250 // render, and we're done.
251 let Some(render_visible_mesh_entities) =
252 view_visible_entities.get::<CustomRenderedEntity>()
253 else {
254 continue;
255 };
256
257 let view_pending_custom_phase_item_queues =
258 pending_custom_phase_item_queues.prepare_for_new_frame(view.retained_view_entity);
259
260 // First, remove meshes that need to be respecialized, and those that
261 // were removed, from the bins.
262 for &main_entity in dirty_specializations
263 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
264 {
265 opaque_phase.remove(main_entity);
266 }
267
268 // Find all the custom rendered entities that are visible from this
269 // view.
270 for (render_entity, main_entity) in dirty_specializations.iter_to_queue(
271 view.retained_view_entity,
272 render_visible_mesh_entities,
273 &view_pending_custom_phase_item_queues.prev_frame,
274 ) {
275 // Ordinarily, the [`SpecializedRenderPipeline::Key`] would contain
276 // some per-view settings, such as whether the view is HDR, but for
277 // simplicity's sake we simply hard-code the view's characteristics,
278 // with the exception of number of MSAA samples.
279 let Ok(pipeline_id) = pipeline
280 .variants
281 .specialize(&pipeline_cache, CustomPhaseKey(*msaa))
282 else {
283 continue;
284 };
285
286 // Add the custom render item. We use the
287 // [`BinnedRenderPhaseType::NonMesh`] type to skip the special
288 // handling that Bevy has for meshes (preprocessing, indirect
289 // draws, etc.)
290 //
291 // The asset ID is arbitrary; we simply use [`AssetId::invalid`],
292 // but you can use anything you like. Note that the asset ID need
293 // not be the ID of a [`Mesh`].
294 opaque_phase.add(
295 Opaque3dBatchSetKey {
296 draw_function: draw_custom_phase_item,
297 pipeline: pipeline_id,
298 material_bind_group_index: None,
299 lightmap_slab: None,
300 slabs: MeshSlabs::default(),
301 },
302 Opaque3dBinKey {
303 asset_id: AssetId::<Mesh>::invalid().untyped(),
304 },
305 (*render_entity, *main_entity),
306 InputUniformIndex::default(),
307 BinnedRenderPhaseType::NonMesh,
308 );
309 }
310 }
311}Trait Implementations
impl<A> Copy for AssetId<A>where
A: Asset,
Sourceimpl<'de, A> Deserialize<'de> for AssetId<A>where
A: Asset,
impl<'de, A> Deserialize<'de> for AssetId<A>where
A: Asset,
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<AssetId<A>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<AssetId<A>, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl<A> Enum for AssetId<A>
impl<A> Enum for AssetId<A>
Sourcefn field(&self, __name_param: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, __name_param: &str) -> Option<&(dyn PartialReflect + 'static)>
Sourcefn field_at(
&self,
__index_param: usize,
) -> Option<&(dyn PartialReflect + 'static)>
fn field_at( &self, __index_param: usize, ) -> Option<&(dyn PartialReflect + 'static)>
Sourcefn field_mut(
&mut self,
__name_param: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, __name_param: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
Sourcefn field_at_mut(
&mut self,
__index_param: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, __index_param: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Sourcefn index_of(&self, __name_param: &str) -> Option<usize>
fn index_of(&self, __name_param: &str) -> Option<usize>
Sourcefn name_at(&self, __index_param: usize) -> Option<&str>
fn name_at(&self, __index_param: usize) -> Option<&str>
Sourcefn iter_fields(&self) -> VariantFieldIter<'_> ⓘ
fn iter_fields(&self) -> VariantFieldIter<'_> ⓘ
Sourcefn variant_name(&self) -> &str
fn variant_name(&self) -> &str
Sourcefn variant_index(&self) -> usize
fn variant_index(&self) -> usize
Sourcefn variant_type(&self) -> VariantType
fn variant_type(&self) -> VariantType
Sourcefn to_dynamic_enum(&self) -> DynamicEnum
fn to_dynamic_enum(&self) -> DynamicEnum
DynamicEnum from this enum.Sourcefn is_variant(&self, variant_type: VariantType) -> bool
fn is_variant(&self, variant_type: VariantType) -> bool
Sourcefn variant_path(&self) -> String
fn variant_path(&self) -> String
impl<A> Eq for AssetId<A>where
A: Asset,
Sourceimpl<T> Equivalent<Handle<T>> for AssetId<T>where
T: Asset,
impl<T> Equivalent<Handle<T>> for AssetId<T>where
T: Asset,
Sourcefn equivalent(&self, key: &Handle<T>) -> bool
fn equivalent(&self, key: &Handle<T>) -> bool
key and return true if they are equal.Sourceimpl From<&AnimationGraphHandle> for AssetId<AnimationGraph>
impl From<&AnimationGraphHandle> for AssetId<AnimationGraph>
Sourcefn from(handle: &AnimationGraphHandle) -> AssetId<AnimationGraph>
fn from(handle: &AnimationGraphHandle) -> AssetId<AnimationGraph>
Sourceimpl<M> From<&MaterialNode<M>> for AssetId<M>where
M: UiMaterial,
impl<M> From<&MaterialNode<M>> for AssetId<M>where
M: UiMaterial,
Sourcefn from(material: &MaterialNode<M>) -> AssetId<M>
fn from(material: &MaterialNode<M>) -> AssetId<M>
Sourceimpl<M> From<&MeshMaterial2d<M>> for AssetId<M>where
M: Material2d,
impl<M> From<&MeshMaterial2d<M>> for AssetId<M>where
M: Material2d,
Sourcefn from(material: &MeshMaterial2d<M>) -> AssetId<M>
fn from(material: &MeshMaterial2d<M>) -> AssetId<M>
Sourceimpl<M> From<&MeshMaterial3d<M>> for AssetId<M>where
M: Material,
impl<M> From<&MeshMaterial3d<M>> for AssetId<M>where
M: Material,
Sourcefn from(material: &MeshMaterial3d<M>) -> AssetId<M>
fn from(material: &MeshMaterial3d<M>) -> AssetId<M>
Sourceimpl From<&MeshletMesh3d> for AssetId<MeshletMesh>
impl From<&MeshletMesh3d> for AssetId<MeshletMesh>
Sourcefn from(mesh: &MeshletMesh3d) -> AssetId<MeshletMesh>
fn from(mesh: &MeshletMesh3d) -> AssetId<MeshletMesh>
Sourceimpl<A> From<(AssetIndex, PhantomData<fn() -> A>)> for AssetId<A>where
A: Asset,
impl<A> From<(AssetIndex, PhantomData<fn() -> A>)> for AssetId<A>where
A: Asset,
Sourcefn from(value: (AssetIndex, PhantomData<fn() -> A>)) -> AssetId<A>
fn from(value: (AssetIndex, PhantomData<fn() -> A>)) -> AssetId<A>
Sourceimpl From<AnimationGraphHandle> for AssetId<AnimationGraph>
impl From<AnimationGraphHandle> for AssetId<AnimationGraph>
Sourcefn from(handle: AnimationGraphHandle) -> AssetId<AnimationGraph>
fn from(handle: AnimationGraphHandle) -> AssetId<AnimationGraph>
Sourceimpl<A> From<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
impl<A> From<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
Sourcefn from(value: AssetId<A>) -> UntypedAssetId
fn from(value: AssetId<A>) -> UntypedAssetId
Sourceimpl<A> From<AssetIndex> for AssetId<A>where
A: Asset,
impl<A> From<AssetIndex> for AssetId<A>where
A: Asset,
Sourcefn from(value: AssetIndex) -> AssetId<A>
fn from(value: AssetIndex) -> AssetId<A>
Sourceimpl<M> From<MaterialNode<M>> for AssetId<M>where
M: UiMaterial,
impl<M> From<MaterialNode<M>> for AssetId<M>where
M: UiMaterial,
Sourcefn from(material: MaterialNode<M>) -> AssetId<M>
fn from(material: MaterialNode<M>) -> AssetId<M>
Sourceimpl<M> From<MeshMaterial2d<M>> for AssetId<M>where
M: Material2d,
impl<M> From<MeshMaterial2d<M>> for AssetId<M>where
M: Material2d,
Sourcefn from(material: MeshMaterial2d<M>) -> AssetId<M>
fn from(material: MeshMaterial2d<M>) -> AssetId<M>
Sourceimpl<M> From<MeshMaterial3d<M>> for AssetId<M>where
M: Material,
impl<M> From<MeshMaterial3d<M>> for AssetId<M>where
M: Material,
Sourcefn from(material: MeshMaterial3d<M>) -> AssetId<M>
fn from(material: MeshMaterial3d<M>) -> AssetId<M>
Sourceimpl From<MeshletMesh3d> for AssetId<MeshletMesh>
impl From<MeshletMesh3d> for AssetId<MeshletMesh>
Sourcefn from(mesh: MeshletMesh3d) -> AssetId<MeshletMesh>
fn from(mesh: MeshletMesh3d) -> AssetId<MeshletMesh>
Sourceimpl<A> FromReflect for AssetId<A>
impl<A> FromReflect for AssetId<A>
Sourcefn from_reflect(__param0: &(dyn PartialReflect + 'static)) -> Option<AssetId<A>>
fn from_reflect(__param0: &(dyn PartialReflect + 'static)) -> Option<AssetId<A>>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl<A> GetOwnership for AssetId<A>
impl<A> GetOwnership for AssetId<A>
Sourceimpl<A> GetTypeRegistration for AssetId<A>
impl<A> GetTypeRegistration for AssetId<A>
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl<A> IntoReturn for AssetId<A>
impl<A> IntoReturn for AssetId<A>
Sourceimpl<A> Ord for AssetId<A>where
A: Asset,
impl<A> Ord for AssetId<A>where
A: Asset,
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Sourceimpl<A> PartialEq for AssetId<A>where
A: Asset,
impl<A> PartialEq for AssetId<A>where
A: Asset,
Sourceimpl<A> PartialEq<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
impl<A> PartialEq<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
Sourceimpl<A> PartialEq<UntypedAssetId> for AssetId<A>where
A: Asset,
impl<A> PartialEq<UntypedAssetId> for AssetId<A>where
A: Asset,
Sourcefn eq(&self, other: &UntypedAssetId) -> bool
fn eq(&self, other: &UntypedAssetId) -> bool
self and other values to be equal, and is used by ==.Sourceimpl<A> PartialOrd for AssetId<A>where
A: Asset,
impl<A> PartialOrd for AssetId<A>where
A: Asset,
Sourceimpl<A> PartialOrd<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
impl<A> PartialOrd<AssetId<A>> for UntypedAssetIdwhere
A: Asset,
Sourceimpl<A> PartialOrd<UntypedAssetId> for AssetId<A>where
A: Asset,
impl<A> PartialOrd<UntypedAssetId> for AssetId<A>where
A: Asset,
Sourceimpl<A> PartialReflect for AssetId<A>
impl<A> PartialReflect for AssetId<A>
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
__value_param: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, __value_param: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<AssetId<A>>) -> ReflectOwned
fn reflect_owned(self: Box<AssetId<A>>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<AssetId<A>>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<AssetId<A>>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<AssetId<A>>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<AssetId<A>>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl<A> Reflect for AssetId<A>
impl<A> Reflect for AssetId<A>
Sourcefn into_any(self: Box<AssetId<A>>) -> Box<dyn Any>
fn into_any(self: Box<AssetId<A>>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<AssetId<A>>) -> Box<dyn Reflect>
fn into_reflect(self: Box<AssetId<A>>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl<A> Serialize for AssetId<A>where
A: Asset,
impl<A> Serialize for AssetId<A>where
A: Asset,
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Sourceimpl<A> TryFrom<UntypedAssetId> for AssetId<A>where
A: Asset,
impl<A> TryFrom<UntypedAssetId> for AssetId<A>where
A: Asset,
Sourcetype Error = UntypedAssetIdConversionError
type Error = UntypedAssetIdConversionError
Sourcefn try_from(
value: UntypedAssetId,
) -> Result<AssetId<A>, <AssetId<A> as TryFrom<UntypedAssetId>>::Error>
fn try_from( value: UntypedAssetId, ) -> Result<AssetId<A>, <AssetId<A> as TryFrom<UntypedAssetId>>::Error>
Sourceimpl<A> TypePath for AssetId<A>
impl<A> TypePath for AssetId<A>
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>