pub struct BorderRect {
pub min_inset: Vec2,
pub max_inset: Vec2,
}Expand description
Defines border insets that shrink a rectangle from its minimum and maximum corners.
This struct is used to represent thickness or offsets from the four edges of a rectangle, with values increasing inwards.
Fields
min_inset: Vec2Inset applied to the rectangle’s minimum corner
max_inset: Vec2Inset applied to the rectangle’s maximum corner
Implementations
Sourceimpl BorderRect
impl BorderRect
Sourcepub const ZERO: BorderRect
pub const ZERO: BorderRect
An empty border with zero thickness along each edge
Sourcepub const fn all(inset: f32) -> BorderRect
pub const fn all(inset: f32) -> BorderRect
Creates a border with the same inset along each edge
Examples found in repository?
examples/testbed/2d.rs (line 361)
342 pub fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
343 commands.spawn((Camera2d, DespawnOnExit(super::Scene::SpriteSlicing)));
344
345 let texture = asset_server.load("textures/slice_square_2.png");
346 let font = asset_server.load("fonts/FiraSans-Bold.ttf");
347
348 commands.spawn((
349 Sprite {
350 image: texture.clone(),
351 ..default()
352 },
353 Transform::from_translation(Vec3::new(-150.0, 50.0, 0.0)).with_scale(Vec3::splat(2.0)),
354 DespawnOnExit(super::Scene::SpriteSlicing),
355 ));
356
357 commands.spawn((
358 Sprite {
359 image: texture,
360 image_mode: SpriteImageMode::Sliced(TextureSlicer {
361 border: BorderRect::all(20.0),
362 center_scale_mode: SliceScaleMode::Stretch,
363 ..default()
364 }),
365 custom_size: Some(Vec2::new(200.0, 200.0)),
366 ..default()
367 },
368 Transform::from_translation(Vec3::new(150.0, 50.0, 1.0)),
369 DespawnOnExit(super::Scene::SpriteSlicing),
370 ));
371
372 commands.spawn((
373 Text2d::new("Original"),
374 TextFont {
375 font: FontSource::from(font.clone()),
376 font_size: FontSize::Px(20.0),
377 ..default()
378 },
379 Transform::from_translation(Vec3::new(-150.0, -80.0, 0.0)),
380 DespawnOnExit(super::Scene::SpriteSlicing),
381 ));
382
383 commands.spawn((
384 Text2d::new("Sliced"),
385 TextFont {
386 font: FontSource::from(font.clone()),
387 font_size: FontSize::Px(20.0),
388 ..default()
389 },
390 Transform::from_translation(Vec3::new(150.0, -80.0, 0.0)),
391 DespawnOnExit(super::Scene::SpriteSlicing),
392 ));
393 }More examples
examples/ui/images/ui_texture_slice.rs (line 92)
88fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
89 let image = asset_server.load("textures/fantasy_ui_borders/panel-border-010.png");
90
91 let slicer = TextureSlicer {
92 border: BorderRect::all(22.0),
93 center_scale_mode: SliceScaleMode::Stretch,
94 sides_scale_mode: SliceScaleMode::Stretch,
95 max_corner_scale: 1.0,
96 };
97 // ui camera
98 commands.spawn(Camera2d);
99 commands
100 .spawn(Node {
101 width: percent(100),
102 height: percent(100),
103 align_items: AlignItems::Center,
104 justify_content: JustifyContent::Center,
105 ..default()
106 })
107 .with_children(|parent| {
108 for [w, h] in [[150.0, 150.0], [300.0, 150.0], [150.0, 300.0]] {
109 parent.spawn((
110 Button,
111 // Required so the button tracks hover state (used by `button_system`).
112 Hovered::default(),
113 ImageNode {
114 image: image.clone(),
115 image_mode: NodeImageMode::Sliced(slicer.clone()),
116 ..default()
117 },
118 Node {
119 width: px(w),
120 height: px(h),
121 // horizontally center child text
122 justify_content: JustifyContent::Center,
123 // vertically center child text
124 align_items: AlignItems::Center,
125 margin: UiRect::all(px(20)),
126 ..default()
127 },
128 children![(
129 Text::new("Button"),
130 TextFont {
131 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
132 font_size: FontSize::Px(33.0),
133 ..default()
134 },
135 TextColor(Color::srgb(0.9, 0.9, 0.9)),
136 )],
137 ));
138 }
139 });
140}examples/ui/images/ui_texture_slice_flip_and_tile.rs (line 28)
17fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
18 let image = asset_server
19 .load_builder()
20 .with_settings(|settings: &mut ImageLoaderSettings| {
21 // Need to use nearest filtering to avoid bleeding between the slices with tiling
22 settings.sampler = ImageSampler::nearest();
23 })
24 .load("textures/fantasy_ui_borders/numbered_slices.png");
25
26 let slicer = TextureSlicer {
27 // `numbered_slices.png` is 48 pixels square. `BorderRect::square(16.)` insets the slicing line from each edge by 16 pixels, resulting in nine slices that are each 16 pixels square.
28 border: BorderRect::all(16.),
29 // With `SliceScaleMode::Tile` the side and center slices are tiled to fill the side and center sections of the target.
30 // And with a `stretch_value` of `1.` the tiles will have the same size as the corresponding slices in the source image.
31 center_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
32 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 1. },
33 ..default()
34 };
35
36 // ui camera
37 commands.spawn(Camera2d);
38
39 commands
40 .spawn(Node {
41 width: percent(100),
42 height: percent(100),
43 justify_content: JustifyContent::Center,
44 align_content: AlignContent::Center,
45 flex_wrap: FlexWrap::Wrap,
46 column_gap: px(10),
47 row_gap: px(10),
48 ..default()
49 })
50 .with_children(|parent| {
51 for [columns, rows] in [[3, 3], [4, 4], [5, 4], [4, 5], [5, 5]] {
52 for (flip_x, flip_y) in [(false, false), (false, true), (true, false), (true, true)]
53 {
54 parent.spawn((
55 ImageNode {
56 image: image.clone(),
57 flip_x,
58 flip_y,
59 image_mode: NodeImageMode::Sliced(slicer.clone()),
60 ..default()
61 },
62 Node {
63 width: px(16 * columns),
64 height: px(16 * rows),
65 ..default()
66 },
67 ));
68 }
69 }
70 });
71}examples/ui/images/ui_texture_atlas_slice.rs (line 98)
87fn setup(
88 mut commands: Commands,
89 asset_server: Res<AssetServer>,
90 mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
91) {
92 let texture_handle = asset_server.load("textures/fantasy_ui_borders/border_sheet.png");
93 let atlas_layout =
94 TextureAtlasLayout::from_grid(UVec2::new(50, 50), 6, 6, Some(UVec2::splat(2)), None);
95 let atlas_layout_handle = texture_atlases.add(atlas_layout);
96
97 let slicer = TextureSlicer {
98 border: BorderRect::all(24.0),
99 center_scale_mode: SliceScaleMode::Stretch,
100 sides_scale_mode: SliceScaleMode::Stretch,
101 max_corner_scale: 1.0,
102 };
103 // ui camera
104 commands.spawn(Camera2d);
105 commands
106 .spawn(Node {
107 width: percent(100),
108 height: percent(100),
109 align_items: AlignItems::Center,
110 justify_content: JustifyContent::Center,
111 ..default()
112 })
113 .with_children(|parent| {
114 for (idx, [w, h]) in [
115 (0, [150.0, 150.0]),
116 (7, [300.0, 150.0]),
117 (13, [150.0, 300.0]),
118 ] {
119 parent
120 .spawn((
121 Button,
122 Hovered::default(),
123 ImageNode::from_atlas_image(
124 texture_handle.clone(),
125 TextureAtlas {
126 index: idx,
127 layout: atlas_layout_handle.clone(),
128 },
129 )
130 .with_mode(NodeImageMode::Sliced(slicer.clone())),
131 Node {
132 width: px(w),
133 height: px(h),
134 // horizontally center child text
135 justify_content: JustifyContent::Center,
136 // vertically center child text
137 align_items: AlignItems::Center,
138 margin: px(20).all(),
139 ..default()
140 },
141 ))
142 .with_children(|parent| {
143 parent.spawn((
144 Text::new("Button"),
145 TextFont {
146 font: asset_server.load("fonts/FiraSans-Bold.ttf").into(),
147 font_size: FontSize::Px(33.0),
148 ..default()
149 },
150 TextColor(Color::srgb(0.9, 0.9, 0.9)),
151 ));
152 });
153 }
154 });
155}examples/2d/sprite_slice.rs (line 41)
12fn spawn_sprites(
13 commands: &mut Commands,
14 texture_handle: Handle<Image>,
15 mut position: Vec3,
16 slice_border: f32,
17 style: TextFont,
18 gap: f32,
19) {
20 let cases = [
21 // Reference sprite
22 (
23 "Original",
24 style.clone(),
25 Vec2::splat(100.0),
26 SpriteImageMode::Auto,
27 ),
28 // Scaled regular sprite
29 (
30 "Stretched",
31 style.clone(),
32 Vec2::new(100.0, 200.0),
33 SpriteImageMode::Auto,
34 ),
35 // Stretched Scaled sliced sprite
36 (
37 "With Slicing",
38 style.clone(),
39 Vec2::new(100.0, 200.0),
40 SpriteImageMode::Sliced(TextureSlicer {
41 border: BorderRect::all(slice_border),
42 center_scale_mode: SliceScaleMode::Stretch,
43 ..default()
44 }),
45 ),
46 // Scaled sliced sprite
47 (
48 "With Tiling",
49 style.clone(),
50 Vec2::new(100.0, 200.0),
51 SpriteImageMode::Sliced(TextureSlicer {
52 border: BorderRect::all(slice_border),
53 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.5 },
54 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
55 ..default()
56 }),
57 ),
58 // Scaled sliced sprite horizontally
59 (
60 "With Tiling",
61 style.clone(),
62 Vec2::new(300.0, 200.0),
63 SpriteImageMode::Sliced(TextureSlicer {
64 border: BorderRect::all(slice_border),
65 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
66 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.3 },
67 ..default()
68 }),
69 ),
70 // Scaled sliced sprite horizontally with max scale
71 (
72 "With Corners Constrained",
73 style,
74 Vec2::new(300.0, 200.0),
75 SpriteImageMode::Sliced(TextureSlicer {
76 border: BorderRect::all(slice_border),
77 center_scale_mode: SliceScaleMode::Tile { stretch_value: 0.1 },
78 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 0.2 },
79 max_corner_scale: 0.2,
80 }),
81 ),
82 ];
83
84 for (label, text_style, size, scale_mode) in cases {
85 position.x += 0.5 * size.x;
86 commands.spawn((
87 Sprite {
88 image: texture_handle.clone(),
89 custom_size: Some(size),
90 image_mode: scale_mode,
91 ..default()
92 },
93 Transform::from_translation(position),
94 children![(
95 Text2d::new(label),
96 text_style,
97 TextLayout::justify(Justify::Center),
98 Transform::from_xyz(0., -0.5 * size.y - 10., 0.0),
99 bevy::sprite::Anchor::TOP_CENTER,
100 )],
101 ));
102 position.x += 0.5 * size.x + gap;
103 }
104}examples/testbed/ui.rs (line 1998)
1993 pub fn setup(mut commands: Commands, asset_server: Res<AssetServer>) {
1994 commands.spawn((Camera2d, DespawnOnExit(super::Scene::Slice)));
1995 let image = asset_server.load("textures/fantasy_ui_borders/numbered_slices.png");
1996
1997 let slicer = TextureSlicer {
1998 border: BorderRect::all(16.0),
1999 center_scale_mode: SliceScaleMode::Tile { stretch_value: 1.0 },
2000 sides_scale_mode: SliceScaleMode::Tile { stretch_value: 1.0 },
2001 ..default()
2002 };
2003 commands
2004 .spawn((
2005 Node {
2006 width: percent(100),
2007 height: percent(100),
2008 flex_direction: FlexDirection::Column,
2009 justify_content: JustifyContent::SpaceAround,
2010 align_content: AlignContent::Center,
2011 ..default()
2012 },
2013 DespawnOnExit(super::Scene::Slice),
2014 ))
2015 .with_children(|parent| {
2016 for visual_box in [
2017 VisualBox::BorderBox,
2018 VisualBox::PaddingBox,
2019 VisualBox::ContentBox,
2020 ] {
2021 parent
2022 .spawn(Node {
2023 justify_content: JustifyContent::SpaceAround,
2024 ..default()
2025 })
2026 .with_children(|parent| {
2027 for [w, h] in [[200.0, 200.0], [300.0, 200.0], [150., 200.0]] {
2028 parent.spawn((
2029 ImageNode {
2030 image: image.clone(),
2031 image_mode: NodeImageMode::Sliced(slicer.clone()),
2032 visual_box,
2033 ..default()
2034 },
2035 Node {
2036 width: px(w),
2037 height: px(h),
2038 border: px(20.).all(),
2039 padding: px(20.).all(),
2040 ..default()
2041 },
2042 Outline {
2043 width: px(2.),
2044 ..default()
2045 },
2046 ));
2047 }
2048
2049 parent.spawn((
2050 ImageNode {
2051 image: asset_server
2052 .load("textures/fantasy_ui_borders/panel-border-010.png"),
2053 image_mode: NodeImageMode::Sliced(TextureSlicer {
2054 border: BorderRect::all(22.0),
2055 center_scale_mode: SliceScaleMode::Stretch,
2056 sides_scale_mode: SliceScaleMode::Stretch,
2057 max_corner_scale: 1.0,
2058 }),
2059 visual_box,
2060 ..Default::default()
2061 },
2062 Node {
2063 width: px(200),
2064 height: px(200),
2065 border: px(20.).all(),
2066 padding: px(20.).all(),
2067 ..default()
2068 },
2069 Outline {
2070 color: bevy::color::palettes::css::DARK_CYAN.into(),
2071 width: px(2.),
2072 ..default()
2073 },
2074 BackgroundColor(bevy::color::palettes::css::NAVY.into()),
2075 ));
2076 });
2077 }
2078 });
2079 }Sourcepub const fn axes(horizontal: f32, vertical: f32) -> BorderRect
pub const fn axes(horizontal: f32, vertical: f32) -> BorderRect
Creates a new border with the min.x and max.x insets equal to horizontal, and the min.y and max.y insets equal to vertical.