pub struct Camera2dBundle {Show 13 fields
pub camera: Camera,
pub camera_render_graph: CameraRenderGraph,
pub projection: OrthographicProjection,
pub visible_entities: VisibleEntities,
pub frustum: Frustum,
pub transform: Transform,
pub global_transform: GlobalTransform,
pub camera_2d: Camera2d,
pub tonemapping: Tonemapping,
pub deband_dither: DebandDither,
pub main_texture_usages: CameraMainTextureUsages,
pub msaa: Msaa,
pub sync: SyncToRenderWorld,
}๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.Fields
camera: Camera๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.camera_render_graph: CameraRenderGraph๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.projection: OrthographicProjection๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.visible_entities: VisibleEntities๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.frustum: Frustum๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.transform: Transform๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.global_transform: GlobalTransform๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.camera_2d: Camera2d๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.tonemapping: Tonemapping๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.deband_dither: DebandDither๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.main_texture_usages: CameraMainTextureUsages๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.msaa: Msaa๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.sync: SyncToRenderWorld๐Deprecated since 0.15.0: Use the
Camera2d component instead. Inserting it will now also insert the other components required by it automatically.Marker component that indicates that its entity needs to be synchronized to the render world
Implementations
Sourceimpl Camera2dBundle
impl Camera2dBundle
Sourcepub fn new_with_far(far: f32) -> Camera2dBundle
pub fn new_with_far(far: f32) -> Camera2dBundle
Create an orthographic projection camera with a custom Z position.
The camera is placed at Z=far-0.1, looking toward the world origin (0,0,0).
Its orthographic projection extends from 0.0 to -far in camera view space,
corresponding to Z=far-0.1 (closest to camera) to Z=-0.1 (furthest away from
camera) in world space.