pub struct PointLightBundle {
pub point_light: PointLight,
pub cubemap_visible_entities: CubemapVisibleEntities,
pub cubemap_frusta: CubemapFrusta,
pub transform: Transform,
pub global_transform: GlobalTransform,
pub visibility: Visibility,
pub inherited_visibility: InheritedVisibility,
pub view_visibility: ViewVisibility,
pub sync: SyncToRenderWorld,
}๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.Expand description
A component bundle for PointLight entities.
Fields
point_light: PointLight๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.cubemap_visible_entities: CubemapVisibleEntities๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.cubemap_frusta: CubemapFrusta๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.transform: Transform๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.global_transform: GlobalTransform๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.visibility: Visibility๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.Enables or disables the light
inherited_visibility: InheritedVisibility๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.Inherited visibility of an entity.
view_visibility: ViewVisibility๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.Algorithmically-computed indication of whether an entity is visible and should be extracted for rendering
sync: SyncToRenderWorld๐Deprecated since 0.15.0: Use the
PointLight component instead. Inserting it will now also insert the other components required by it automatically.Marker component that indicates that its entity needs to be synchronized to the render world