#[repr(C)]pub struct UVec2 {
pub x: u32,
pub y: u32,
}Expand description
A 2-dimensional vector.
Fields
x: u32y: u32Implementations
Sourceimpl UVec2
impl UVec2
Sourcepub const fn new(x: u32, y: u32) -> UVec2
pub const fn new(x: u32, y: u32) -> UVec2
Creates a new vector.
Examples found in repository?
More examples
115fn store_window_settings(mut window_settings: ResMut<WindowSettings>, window: &Window) -> bool {
116 window_settings.set_if_neq(WindowSettings {
117 position: match window.position {
118 WindowPosition::At(pos) => Some(pos),
119 _ => None,
120 },
121 size: Some(UVec2::new(
122 window.resolution.width() as u32,
123 window.resolution.height() as u32,
124 )),
125 fullscreen: window.mode != WindowMode::Windowed,
126 })
127}195 const SECTIONS: [Option<URect>; 5] = [
196 Some(URect {
197 min: UVec2::ZERO,
198 max: UVec2::splat(RECT_SIZE),
199 }),
200 Some(URect {
201 min: UVec2::new(RECT_SIZE, 0),
202 max: UVec2::new(2 * RECT_SIZE, RECT_SIZE),
203 }),
204 Some(URect {
205 min: UVec2::new(0, RECT_SIZE),
206 max: UVec2::new(RECT_SIZE, 2 * RECT_SIZE),
207 }),
208 Some(URect {
209 min: UVec2::new(RECT_SIZE, RECT_SIZE),
210 max: UVec2::splat(2 * RECT_SIZE),
211 }),
212 None, // reset to None
213 ];70fn spawn_fake_player(
71 mut commands: Commands,
72 mut meshes: ResMut<Assets<Mesh>>,
73 mut materials: ResMut<Assets<ColorMaterial>>,
74 chunk: Single<&TilemapChunk>,
75) {
76 let mut transform = chunk.calculate_tile_transform(UVec2::new(0, 0));
77 transform.translation.z = 1.;
78
79 commands.spawn((
80 Mesh2d(meshes.add(Rectangle::new(8., 8.))),
81 MeshMaterial2d(materials.add(Color::from(RED_400))),
82 transform,
83 MovePlayer,
84 ));
85
86 let mut transform = chunk.calculate_tile_transform(UVec2::new(5, 6));
87 transform.translation.z = 1.;
88
89 // second "player" to visually test a non-zero position
90 commands.spawn((
91 Mesh2d(meshes.add(Rectangle::new(8., 8.))),
92 MeshMaterial2d(materials.add(Color::from(RED_400))),
93 transform,
94 ));
95}
96
97fn move_player(
98 mut player: Single<&mut Transform, With<MovePlayer>>,
99 time: Res<Time>,
100 chunk: Single<&TilemapChunk>,
101) {
102 let t = (ops::sin(time.elapsed_secs()) + 1.) / 2.;
103
104 let origin = chunk
105 .calculate_tile_transform(UVec2::new(0, 0))
106 .translation
107 .x;
108 let destination = chunk
109 .calculate_tile_transform(UVec2::new(63, 0))
110 .translation
111 .x;
112
113 player.translation.x = origin.lerp(destination, t);
114}
115
116fn update_tilemap(
117 time: Res<Time>,
118 mut query: Query<(&mut TilemapChunkTileData, &mut UpdateTimer)>,
119 mut rng: ResMut<SeededRng>,
120) {
121 for (mut tile_data, mut timer) in query.iter_mut() {
122 timer.tick(time.delta());
123
124 if timer.just_finished() {
125 for _ in 0..50 {
126 let index = rng.random_range(0..tile_data.len());
127 tile_data[index] = Some(TileData::from_tileset_index(rng.random_range(0..5)));
128 }
129 }
130 }
131}
132
133// find the data for an arbitrary tile in the chunk and log its data
134fn log_tile(tilemap: Single<(&TilemapChunk, &TilemapChunkTileData)>, mut local: Local<u16>) {
135 let (chunk, data) = tilemap.into_inner();
136 let Some(tile_data) = data.tile_data_from_tile_pos(chunk.chunk_size, UVec2::new(3, 4)) else {
137 return;
138 };
139 // log when the tile changes
140 if tile_data.tileset_index != *local {
141 info!(?tile_data, "tile_data changed");
142 *local = tile_data.tileset_index;
143 }
144}394 pub fn draw_gizmos(mut gizmos: Gizmos) {
395 gizmos.rect_2d(
396 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
397 Vec2::new(200.0, 200.0),
398 RED,
399 );
400 gizmos
401 .circle_2d(
402 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
403 200.0,
404 GREEN,
405 )
406 .resolution(64);
407
408 gizmos.text_2d(
409 Isometry2d::from_translation(Vec2::new(-200.0, 0.0)),
410 "text_2d gizmo",
411 15.,
412 Vec2 { x: 0., y: 0. },
413 Color::WHITE,
414 );
415
416 // 2d grids with all variations of outer edges on or off
417 for i in 0..4 {
418 let x = 200.0 * (1.0 + (i % 2) as f32);
419 let y = 150.0 * (0.5 - (i / 2) as f32);
420 let mut grid = gizmos.grid(
421 Vec3::new(x, y, 0.0),
422 UVec2::new(5, 4),
423 Vec2::splat(30.),
424 Color::WHITE,
425 );
426 if i & 1 > 0 {
427 grid = grid.outer_edges_x();
428 }
429 if i & 2 > 0 {
430 grid.outer_edges_y();
431 }
432 }
433 }122fn setup_atlas(
123 mut commands: Commands,
124 asset_server: Res<AssetServer>,
125 mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
126) {
127 let texture_handle = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
128 let layout = TextureAtlasLayout::from_grid(UVec2::new(24, 24), 7, 1, None, None);
129 let texture_atlas_layout_handle = texture_atlas_layouts.add(layout);
130 // Use only the subset of sprites in the sheet that make up the run animation
131 let animation_indices = AnimationIndices { first: 1, last: 6 };
132 commands
133 .spawn((
134 Sprite::from_atlas_image(
135 texture_handle,
136 TextureAtlas {
137 layout: texture_atlas_layout_handle,
138 index: animation_indices.first,
139 },
140 ),
141 Transform::from_xyz(300.0, 0.0, 0.0).with_scale(Vec3::splat(6.0)),
142 animation_indices,
143 AnimationTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
144 Pickable::default(),
145 ))
146 .observe(recolor_on::<Pointer<Over>>(Color::srgb(0.0, 1.0, 1.0)))
147 .observe(recolor_on::<Pointer<Out>>(Color::srgb(1.0, 1.0, 1.0)))
148 .observe(recolor_on::<Pointer<Press>>(Color::srgb(1.0, 1.0, 0.0)))
149 .observe(recolor_on::<Pointer<Release>>(Color::srgb(0.0, 1.0, 1.0)));
150}- examples/shader/automatic_instancing.rs
- examples/stress_tests/many_cameras_lights.rs
- examples/ui/images/ui_texture_atlas_slice.rs
- examples/gizmos/2d_gizmos.rs
- examples/2d/2d_viewport_to_world.rs
- examples/gizmos/3d_gizmos.rs
- examples/testbed/3d.rs
- examples/3d/split_screen.rs
- examples/2d/texture_atlas.rs
- examples/3d/camera_sub_view.rs
Sourcepub const fn splat(v: u32) -> UVec2
pub const fn splat(v: u32) -> UVec2
Creates a vector with all elements set to v.
Examples found in repository?
More examples
406 fn next(&mut self) -> Option<Self::Item> {
407 // The size of mipmap level N + 1 is equal to half the size of mipmap
408 // level N, rounding down, except that the size can never go below 1
409 // pixel on either axis.
410 let result = self.size;
411 if let Some(size) = self.size {
412 self.size = if size == UVec2::splat(1) {
413 None
414 } else {
415 Some((size / 2).max(UVec2::splat(1)))
416 };
417 }
418 result
419 }42fn setup(
43 mut commands: Commands,
44 asset_server: Res<AssetServer>,
45 mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
46) {
47 let texture = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
48 let layout = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
49 let texture_atlas_layout = texture_atlas_layouts.add(layout);
50 // Use only the subset of sprites in the sheet that make up the run animation
51 let animation_indices = AnimationIndices { first: 1, last: 6 };
52
53 commands.spawn(Camera2d);
54
55 commands.spawn((
56 Sprite::from_atlas_image(
57 texture,
58 TextureAtlas {
59 layout: texture_atlas_layout,
60 index: animation_indices.first,
61 },
62 ),
63 Transform::from_scale(Vec3::splat(6.0)),
64 animation_indices,
65 AnimationTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
66 ));
67}240fn spawn_depth_only_camera(commands: &mut Commands) {
241 commands.spawn((
242 Camera3d::default(),
243 Transform::from_xyz(-4.0, -5.0, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
244 Camera {
245 // Make sure that we render from this depth-only camera *before*
246 // rendering from the main camera.
247 order: -1,
248 ..Camera::default()
249 },
250 // We specify no color render target, for maximum efficiency.
251 RenderTarget::None {
252 // When specifying no render target, we must manually specify
253 // the viewport size. Otherwise, Bevy won't know how big to make
254 // the depth buffer.
255 size: UVec2::splat(DEPTH_TEXTURE_SIZE),
256 },
257 // We need to disable multisampling or the depth texture will be
258 // multisampled, which adds complexity we don't care about for this
259 // demo.
260 Msaa::Off,
261 // Cameras with no render target render *nothing* by default. To get
262 // them to render something, we must add a prepass that specifies what
263 // we want to render: in this case, depth.
264 DepthPrepass,
265 ));
266}31fn setup_cursor_icon(
32 mut commands: Commands,
33 asset_server: Res<AssetServer>,
34 mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
35 window: Single<Entity, With<Window>>,
36) {
37 let layout =
38 TextureAtlasLayout::from_grid(UVec2::splat(64), 20, 10, Some(UVec2::splat(5)), None);
39 let texture_atlas_layout = texture_atlas_layouts.add(layout);
40
41 let animation_config = AnimationConfig::new(0, 199, 1, 4);
42
43 commands.entity(*window).insert((
44 CursorIcon::Custom(CustomCursor::Image(CustomCursorImage {
45 // Image to use as the cursor.
46 handle: asset_server
47 .load("cursors/kenney_crosshairPack/Tilesheet/crosshairs_tilesheet_white.png"),
48 // Optional texture atlas allows you to pick a section of the image
49 // and animate it.
50 texture_atlas: Some(TextureAtlas {
51 layout: texture_atlas_layout.clone(),
52 index: animation_config.first_sprite_index,
53 }),
54 flip_x: false,
55 flip_y: false,
56 // Optional section of the image to use as the cursor.
57 rect: None,
58 // The hotspot is the point in the cursor image that will be
59 // positioned at the mouse cursor's position.
60 hotspot: (0, 0),
61 })),
62 animation_config,
63 ));
64}
65
66fn setup_camera(mut commands: Commands) {
67 commands.spawn(Camera3d::default());
68}
69
70fn setup_instructions(mut commands: Commands) {
71 commands.spawn((
72 Text::new(
73 "Press T to toggle the cursor's `texture_atlas`.\n
74Press X to toggle the cursor's `flip_x` setting.\n
75Press Y to toggle the cursor's `flip_y` setting.\n
76Press C to cycle through the sections of the cursor's image using `rect`.",
77 ),
78 Node {
79 position_type: PositionType::Absolute,
80 bottom: px(12),
81 left: px(12),
82 ..default()
83 },
84 ));
85}
86
87#[derive(Component)]
88struct AnimationConfig {
89 first_sprite_index: usize,
90 last_sprite_index: usize,
91 increment: usize,
92 fps: u8,
93 frame_timer: Timer,
94}
95
96impl AnimationConfig {
97 fn new(first: usize, last: usize, increment: usize, fps: u8) -> Self {
98 Self {
99 first_sprite_index: first,
100 last_sprite_index: last,
101 increment,
102 fps,
103 frame_timer: Self::timer_from_fps(fps),
104 }
105 }
106
107 fn timer_from_fps(fps: u8) -> Timer {
108 Timer::new(Duration::from_secs_f32(1.0 / (fps as f32)), TimerMode::Once)
109 }
110}
111
112/// This system loops through all the sprites in the [`CursorIcon`]'s
113/// [`TextureAtlas`], from [`AnimationConfig`]'s `first_sprite_index` to
114/// `last_sprite_index`.
115fn execute_animation(time: Res<Time>, mut query: Query<(&mut AnimationConfig, &mut CursorIcon)>) {
116 for (mut config, mut cursor_icon) in &mut query {
117 if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
118 config.frame_timer.tick(time.delta());
119
120 if config.frame_timer.is_finished()
121 && let Some(atlas) = image.texture_atlas.as_mut()
122 {
123 atlas.index += config.increment;
124
125 if atlas.index > config.last_sprite_index {
126 atlas.index = config.first_sprite_index;
127 }
128
129 config.frame_timer = AnimationConfig::timer_from_fps(config.fps);
130 }
131 }
132 }
133}
134
135fn toggle_texture_atlas(
136 input: Res<ButtonInput<KeyCode>>,
137 mut query: Query<&mut CursorIcon, With<Window>>,
138 mut cached_atlas: Local<Option<TextureAtlas>>, // this lets us restore the previous value
139) {
140 if input.just_pressed(KeyCode::KeyT) {
141 for mut cursor_icon in &mut query {
142 if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
143 match image.texture_atlas.take() {
144 Some(a) => {
145 // Save the current texture atlas.
146 *cached_atlas = Some(a.clone());
147 }
148 None => {
149 // Restore the cached texture atlas.
150 if let Some(cached_a) = cached_atlas.take() {
151 image.texture_atlas = Some(cached_a);
152 }
153 }
154 }
155 }
156 }
157 }
158}
159
160fn toggle_flip_x(
161 input: Res<ButtonInput<KeyCode>>,
162 mut query: Query<&mut CursorIcon, With<Window>>,
163) {
164 if input.just_pressed(KeyCode::KeyX) {
165 for mut cursor_icon in &mut query {
166 if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
167 image.flip_x = !image.flip_x;
168 }
169 }
170 }
171}
172
173fn toggle_flip_y(
174 input: Res<ButtonInput<KeyCode>>,
175 mut query: Query<&mut CursorIcon, With<Window>>,
176) {
177 if input.just_pressed(KeyCode::KeyY) {
178 for mut cursor_icon in &mut query {
179 if let CursorIcon::Custom(CustomCursor::Image(ref mut image)) = *cursor_icon {
180 image.flip_y = !image.flip_y;
181 }
182 }
183 }
184}
185
186/// This system alternates the [`CursorIcon`]'s `rect` field between `None` and
187/// 4 sections/rectangles of the cursor's image.
188fn cycle_rect(input: Res<ButtonInput<KeyCode>>, mut query: Query<&mut CursorIcon, With<Window>>) {
189 if !input.just_pressed(KeyCode::KeyC) {
190 return;
191 }
192
193 const RECT_SIZE: u32 = 32; // half the size of a tile in the texture atlas
194
195 const SECTIONS: [Option<URect>; 5] = [
196 Some(URect {
197 min: UVec2::ZERO,
198 max: UVec2::splat(RECT_SIZE),
199 }),
200 Some(URect {
201 min: UVec2::new(RECT_SIZE, 0),
202 max: UVec2::new(2 * RECT_SIZE, RECT_SIZE),
203 }),
204 Some(URect {
205 min: UVec2::new(0, RECT_SIZE),
206 max: UVec2::new(RECT_SIZE, 2 * RECT_SIZE),
207 }),
208 Some(URect {
209 min: UVec2::new(RECT_SIZE, RECT_SIZE),
210 max: UVec2::splat(2 * RECT_SIZE),
211 }),
212 None, // reset to None
213 ];26fn setup(mut commands: Commands, assets: Res<AssetServer>) {
27 // We're seeding the PRNG here to make this example deterministic for testing purposes.
28 // This isn't strictly required in practical use unless you need your app to be deterministic.
29 let mut rng = ChaCha8Rng::seed_from_u64(42);
30
31 let chunk_size = UVec2::splat(64);
32 let tile_display_size = UVec2::splat(8);
33 let tile_data: Vec<Option<TileData>> = (0..chunk_size.element_product())
34 .map(|_| rng.random_range(0..5))
35 .map(|i| {
36 if i == 0 {
37 None
38 } else {
39 Some(TileData::from_tileset_index(i - 1))
40 }
41 })
42 .collect();
43
44 commands.spawn((
45 TilemapChunk {
46 chunk_size,
47 tile_display_size,
48 tileset: assets
49 .load_builder()
50 .with_settings(|settings: &mut ImageLoaderSettings| {
51 // The tileset texture is expected to be an array of tile textures, so we tell the
52 // `ImageLoader` that our texture is composed of 4 stacked tile images.
53 settings.array_layout = Some(ImageArrayLayout::RowCount { rows: 4 });
54 })
55 .load("textures/array_texture.png"),
56 ..default()
57 },
58 TilemapChunkTileData(tile_data),
59 UpdateTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
60 ));
61
62 commands.spawn(Camera2d);
63
64 commands.insert_resource(SeededRng(rng));
65}- examples/usage/cooldown.rs
- examples/ui/images/ui_texture_atlas.rs
- examples/stress_tests/many_animated_sprites.rs
- examples/stress_tests/many_animated_sprite_meshes.rs
- examples/2d/sprite_animation.rs
- examples/2d/tilemap_chunk_orientation.rs
- examples/ui/images/ui_texture_atlas_slice.rs
- examples/gizmos/3d_gizmos.rs
- examples/2d/sprite_scale.rs
Sourcepub fn map<F>(self, f: F) -> UVec2
pub fn map<F>(self, f: F) -> UVec2
Returns a vector containing each element of self modified by a mapping function f.
Sourcepub fn select(mask: BVec2, if_true: UVec2, if_false: UVec2) -> UVec2
pub fn select(mask: BVec2, if_true: UVec2, if_false: UVec2) -> UVec2
Creates a vector from the elements in if_true and if_false, selecting which to use
for each element of self.
A true element in the mask uses the corresponding element from if_true, and false
uses the element from if_false.
Sourcepub const fn from_array(a: [u32; 2]) -> UVec2
pub const fn from_array(a: [u32; 2]) -> UVec2
Creates a new vector from an array.
Sourcepub const fn from_slice(slice: &[u32]) -> UVec2
pub const fn from_slice(slice: &[u32]) -> UVec2
Creates a vector from the first 2 values in slice.
Panics
Panics if slice is less than 2 elements long.
Sourcepub fn write_to_slice(self, slice: &mut [u32])
pub fn write_to_slice(self, slice: &mut [u32])
Writes the elements of self to the first 2 elements in slice.
Panics
Panics if slice is less than 2 elements long.
Sourcepub const fn extend(self, z: u32) -> UVec3
pub const fn extend(self, z: u32) -> UVec3
Creates a 3D vector from self and the given z value.
Sourcepub fn dot_into_vec(self, rhs: UVec2) -> UVec2
pub fn dot_into_vec(self, rhs: UVec2) -> UVec2
Returns a vector where every component is the dot product of self and rhs.
Sourcepub fn min(self, rhs: UVec2) -> UVec2
pub fn min(self, rhs: UVec2) -> UVec2
Returns a vector containing the minimum values for each element of self and rhs.
In other words this computes [min(x, rhs.x), min(self.y, rhs.y), ..].
Examples found in repository?
42fn controls(
43 camera_query: Single<(&mut Camera, &mut Transform, &mut Projection)>,
44 window: Single<&Window>,
45 input: Res<ButtonInput<KeyCode>>,
46 time: Res<Time<Fixed>>,
47) {
48 let (mut camera, mut transform, mut projection) = camera_query.into_inner();
49
50 let fspeed = 600.0 * time.delta_secs();
51 let uspeed = fspeed as u32;
52 let window_size = window.resolution.physical_size();
53
54 // Camera movement controls
55 if input.pressed(KeyCode::ArrowUp) {
56 transform.translation.y += fspeed;
57 }
58 if input.pressed(KeyCode::ArrowDown) {
59 transform.translation.y -= fspeed;
60 }
61 if input.pressed(KeyCode::ArrowLeft) {
62 transform.translation.x -= fspeed;
63 }
64 if input.pressed(KeyCode::ArrowRight) {
65 transform.translation.x += fspeed;
66 }
67
68 // Camera zoom controls
69 if let Projection::Orthographic(projection2d) = &mut *projection {
70 if input.pressed(KeyCode::Comma) {
71 projection2d.scale *= powf(4.0f32, time.delta_secs());
72 }
73
74 if input.pressed(KeyCode::Period) {
75 projection2d.scale *= powf(0.25f32, time.delta_secs());
76 }
77 }
78
79 if let Some(viewport) = camera.viewport.as_mut() {
80 // Reset viewport size on window resize
81 if viewport.physical_size.x > window_size.x || viewport.physical_size.y > window_size.y {
82 viewport.physical_size = (window_size.as_vec2() * 0.75).as_uvec2();
83 }
84
85 // Viewport movement controls
86 if input.pressed(KeyCode::KeyW) {
87 viewport.physical_position.y = viewport.physical_position.y.saturating_sub(uspeed);
88 }
89 if input.pressed(KeyCode::KeyS) {
90 viewport.physical_position.y += uspeed;
91 }
92 if input.pressed(KeyCode::KeyA) {
93 viewport.physical_position.x = viewport.physical_position.x.saturating_sub(uspeed);
94 }
95 if input.pressed(KeyCode::KeyD) {
96 viewport.physical_position.x += uspeed;
97 }
98
99 // Bound viewport position so it doesn't go off-screen
100 viewport.physical_position = viewport
101 .physical_position
102 .min(window_size - viewport.physical_size);
103
104 // Viewport size controls
105 if input.pressed(KeyCode::KeyI) {
106 viewport.physical_size.y = viewport.physical_size.y.saturating_sub(uspeed);
107 }
108 if input.pressed(KeyCode::KeyK) {
109 viewport.physical_size.y += uspeed;
110 }
111 if input.pressed(KeyCode::KeyJ) {
112 viewport.physical_size.x = viewport.physical_size.x.saturating_sub(uspeed);
113 }
114 if input.pressed(KeyCode::KeyL) {
115 viewport.physical_size.x += uspeed;
116 }
117
118 // Bound viewport size so it doesn't go off-screen
119 viewport.physical_size = viewport
120 .physical_size
121 .min(window_size - viewport.physical_position)
122 .max(UVec2::new(20, 20));
123 }
124}Sourcepub fn max(self, rhs: UVec2) -> UVec2
pub fn max(self, rhs: UVec2) -> UVec2
Returns a vector containing the maximum values for each element of self and rhs.
In other words this computes [max(self.x, rhs.x), max(self.y, rhs.y), ..].
Examples found in repository?
406 fn next(&mut self) -> Option<Self::Item> {
407 // The size of mipmap level N + 1 is equal to half the size of mipmap
408 // level N, rounding down, except that the size can never go below 1
409 // pixel on either axis.
410 let result = self.size;
411 if let Some(size) = self.size {
412 self.size = if size == UVec2::splat(1) {
413 None
414 } else {
415 Some((size / 2).max(UVec2::splat(1)))
416 };
417 }
418 result
419 }More examples
42fn controls(
43 camera_query: Single<(&mut Camera, &mut Transform, &mut Projection)>,
44 window: Single<&Window>,
45 input: Res<ButtonInput<KeyCode>>,
46 time: Res<Time<Fixed>>,
47) {
48 let (mut camera, mut transform, mut projection) = camera_query.into_inner();
49
50 let fspeed = 600.0 * time.delta_secs();
51 let uspeed = fspeed as u32;
52 let window_size = window.resolution.physical_size();
53
54 // Camera movement controls
55 if input.pressed(KeyCode::ArrowUp) {
56 transform.translation.y += fspeed;
57 }
58 if input.pressed(KeyCode::ArrowDown) {
59 transform.translation.y -= fspeed;
60 }
61 if input.pressed(KeyCode::ArrowLeft) {
62 transform.translation.x -= fspeed;
63 }
64 if input.pressed(KeyCode::ArrowRight) {
65 transform.translation.x += fspeed;
66 }
67
68 // Camera zoom controls
69 if let Projection::Orthographic(projection2d) = &mut *projection {
70 if input.pressed(KeyCode::Comma) {
71 projection2d.scale *= powf(4.0f32, time.delta_secs());
72 }
73
74 if input.pressed(KeyCode::Period) {
75 projection2d.scale *= powf(0.25f32, time.delta_secs());
76 }
77 }
78
79 if let Some(viewport) = camera.viewport.as_mut() {
80 // Reset viewport size on window resize
81 if viewport.physical_size.x > window_size.x || viewport.physical_size.y > window_size.y {
82 viewport.physical_size = (window_size.as_vec2() * 0.75).as_uvec2();
83 }
84
85 // Viewport movement controls
86 if input.pressed(KeyCode::KeyW) {
87 viewport.physical_position.y = viewport.physical_position.y.saturating_sub(uspeed);
88 }
89 if input.pressed(KeyCode::KeyS) {
90 viewport.physical_position.y += uspeed;
91 }
92 if input.pressed(KeyCode::KeyA) {
93 viewport.physical_position.x = viewport.physical_position.x.saturating_sub(uspeed);
94 }
95 if input.pressed(KeyCode::KeyD) {
96 viewport.physical_position.x += uspeed;
97 }
98
99 // Bound viewport position so it doesn't go off-screen
100 viewport.physical_position = viewport
101 .physical_position
102 .min(window_size - viewport.physical_size);
103
104 // Viewport size controls
105 if input.pressed(KeyCode::KeyI) {
106 viewport.physical_size.y = viewport.physical_size.y.saturating_sub(uspeed);
107 }
108 if input.pressed(KeyCode::KeyK) {
109 viewport.physical_size.y += uspeed;
110 }
111 if input.pressed(KeyCode::KeyJ) {
112 viewport.physical_size.x = viewport.physical_size.x.saturating_sub(uspeed);
113 }
114 if input.pressed(KeyCode::KeyL) {
115 viewport.physical_size.x += uspeed;
116 }
117
118 // Bound viewport size so it doesn't go off-screen
119 viewport.physical_size = viewport
120 .physical_size
121 .min(window_size - viewport.physical_position)
122 .max(UVec2::new(20, 20));
123 }
124}Sourcepub fn clamp(self, min: UVec2, max: UVec2) -> UVec2
pub fn clamp(self, min: UVec2, max: UVec2) -> UVec2
Component-wise clamping of values, similar to u32::clamp.
Each element in min must be less-or-equal to the corresponding element in max.
Panics
Will panic if min is greater than max when glam_assert is enabled.
Sourcepub fn min_element(self) -> u32
pub fn min_element(self) -> u32
Returns the horizontal minimum of self.
In other words this computes min(x, y, ..).
Sourcepub fn max_element(self) -> u32
pub fn max_element(self) -> u32
Returns the horizontal maximum of self.
In other words this computes max(x, y, ..).
Sourcepub fn min_position(self) -> usize
pub fn min_position(self) -> usize
Returns the index of the first minimum element of self.
Sourcepub fn max_position(self) -> usize
pub fn max_position(self) -> usize
Returns the index of the first maximum element of self.
Sourcepub fn element_sum(self) -> u32
pub fn element_sum(self) -> u32
Returns the sum of all elements of self.
In other words, this computes self.x + self.y + ...
Sourcepub fn element_product(self) -> u32
pub fn element_product(self) -> u32
Returns the product of all elements of self.
In other words, this computes self.x * self.y * ...
Examples found in repository?
26fn setup(mut commands: Commands, assets: Res<AssetServer>) {
27 // We're seeding the PRNG here to make this example deterministic for testing purposes.
28 // This isn't strictly required in practical use unless you need your app to be deterministic.
29 let mut rng = ChaCha8Rng::seed_from_u64(42);
30
31 let chunk_size = UVec2::splat(64);
32 let tile_display_size = UVec2::splat(8);
33 let tile_data: Vec<Option<TileData>> = (0..chunk_size.element_product())
34 .map(|_| rng.random_range(0..5))
35 .map(|i| {
36 if i == 0 {
37 None
38 } else {
39 Some(TileData::from_tileset_index(i - 1))
40 }
41 })
42 .collect();
43
44 commands.spawn((
45 TilemapChunk {
46 chunk_size,
47 tile_display_size,
48 tileset: assets
49 .load_builder()
50 .with_settings(|settings: &mut ImageLoaderSettings| {
51 // The tileset texture is expected to be an array of tile textures, so we tell the
52 // `ImageLoader` that our texture is composed of 4 stacked tile images.
53 settings.array_layout = Some(ImageArrayLayout::RowCount { rows: 4 });
54 })
55 .load("textures/array_texture.png"),
56 ..default()
57 },
58 TilemapChunkTileData(tile_data),
59 UpdateTimer(Timer::from_seconds(0.1, TimerMode::Repeating)),
60 ));
61
62 commands.spawn(Camera2d);
63
64 commands.insert_resource(SeededRng(rng));
65}More examples
17fn setup(mut commands: Commands, assets: Res<AssetServer>) {
18 let chunk_size = UVec2::splat(8);
19 let tile_display_size = UVec2::splat(64);
20
21 // We'll use each possible orientation, one per column
22 let orientation = [
23 TileOrientation::Default,
24 TileOrientation::Rotate90,
25 TileOrientation::Rotate180,
26 TileOrientation::Rotate270,
27 TileOrientation::MirrorH,
28 TileOrientation::MirrorHRotate90,
29 TileOrientation::MirrorHRotate180,
30 TileOrientation::MirrorHRotate270,
31 ];
32
33 // Show different color/alpha on each row
34 let colors = [
35 Color::WHITE,
36 Color::linear_rgb(1.0, 0.0, 0.0),
37 Color::linear_rgb(0.0, 1.0, 0.0),
38 Color::linear_rgb(0.0, 0.0, 1.0),
39 Color::linear_rgba(1.0, 0.0, 0.0, 0.25),
40 Color::linear_rgba(0.0, 1.0, 0.0, 0.25),
41 Color::linear_rgba(0.0, 0.0, 1.0, 0.25),
42 Color::linear_rgba(1.0, 1.0, 1.0, 0.5),
43 ];
44
45 let tile_data = (0..chunk_size.element_product())
46 .map(|i| {
47 let row = i / 8;
48 let col = i % 8;
49 Some(TileData {
50 // Alternate tiles per row
51 tileset_index: (row % 2) as u16,
52 color: colors[row as usize],
53 // Last (top) row is invisible
54 visible: row != 7,
55 orientation: orientation[col as usize],
56 })
57 })
58 .collect();
59
60 commands.spawn((
61 TilemapChunk {
62 chunk_size,
63 tile_display_size,
64 tileset: assets
65 .load_builder()
66 .with_settings(|settings: &mut ImageLoaderSettings| {
67 // The tileset texture is expected to be an array of tile textures, so we tell the
68 // `ImageLoader` that our texture is composed of 2 stacked tile images.
69 settings.array_layout = Some(ImageArrayLayout::RowCount { rows: 2 });
70 })
71 .load("textures/arrow.png"),
72 alpha_mode: AlphaMode2d::Blend,
73 },
74 TilemapChunkTileData(tile_data),
75 ));
76
77 commands.spawn(Camera2d);
78}Sourcepub fn cmpeq(self, rhs: UVec2) -> BVec2
pub fn cmpeq(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a == comparison for each element of
self and rhs.
In other words, this computes [self.x == rhs.x, self.y == rhs.y, ..] for all
elements.
Sourcepub fn cmpne(self, rhs: UVec2) -> BVec2
pub fn cmpne(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a != comparison for each element of
self and rhs.
In other words this computes [self.x != rhs.x, self.y != rhs.y, ..] for all
elements.
Sourcepub fn cmpge(self, rhs: UVec2) -> BVec2
pub fn cmpge(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a >= comparison for each element of
self and rhs.
In other words this computes [self.x >= rhs.x, self.y >= rhs.y, ..] for all
elements.
Sourcepub fn cmpgt(self, rhs: UVec2) -> BVec2
pub fn cmpgt(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a > comparison for each element of
self and rhs.
In other words this computes [self.x > rhs.x, self.y > rhs.y, ..] for all
elements.
Sourcepub fn cmple(self, rhs: UVec2) -> BVec2
pub fn cmple(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a <= comparison for each element of
self and rhs.
In other words this computes [self.x <= rhs.x, self.y <= rhs.y, ..] for all
elements.
Sourcepub fn cmplt(self, rhs: UVec2) -> BVec2
pub fn cmplt(self, rhs: UVec2) -> BVec2
Returns a vector mask containing the result of a < comparison for each element of
self and rhs.
In other words this computes [self.x < rhs.x, self.y < rhs.y, ..] for all
elements.
Sourcepub fn length_squared(self) -> u32
pub fn length_squared(self) -> u32
Computes the squared length of self.
Sourcepub fn manhattan_distance(self, rhs: UVec2) -> u32
pub fn manhattan_distance(self, rhs: UVec2) -> u32
Computes the manhattan distance between two points.
Overflow
This method may overflow if the result is greater than u32::MAX.
See also checked_manhattan_distance.
Sourcepub fn checked_manhattan_distance(self, rhs: UVec2) -> Option<u32>
pub fn checked_manhattan_distance(self, rhs: UVec2) -> Option<u32>
Computes the manhattan distance between two points.
This will returns None if the result is greater than u32::MAX.
Sourcepub fn chebyshev_distance(self, rhs: UVec2) -> u32
pub fn chebyshev_distance(self, rhs: UVec2) -> u32
Computes the chebyshev distance between two points.
Sourcepub fn as_vec2(self) -> Vec2
pub fn as_vec2(self) -> Vec2
Casts all elements of self to f32.
Examples found in repository?
54fn setup(mut commands: Commands, mut images: ResMut<Assets<Image>>) {
55 let mut image = Image::new_target_texture(SIZE.x, SIZE.y, TextureFormat::Rgba32Float, None);
56 image.asset_usage = RenderAssetUsages::RENDER_WORLD;
57 image.texture_descriptor.usage =
58 TextureUsages::COPY_DST | TextureUsages::STORAGE_BINDING | TextureUsages::TEXTURE_BINDING;
59 let image0 = images.add(image.clone());
60 let image1 = images.add(image);
61
62 commands.spawn((
63 Sprite {
64 image: image0.clone(),
65 custom_size: Some(SIZE.as_vec2()),
66 ..default()
67 },
68 Transform::from_scale(Vec3::splat(DISPLAY_FACTOR as f32)),
69 ));
70 commands.spawn(Camera2d);
71
72 commands.insert_resource(GameOfLifeImages {
73 texture_a: image0,
74 texture_b: image1,
75 });
76
77 commands.insert_resource(GameOfLifeUniforms {
78 alive_color: LinearRgba::RED,
79 });
80}More examples
629fn spawn_mip_level_views(
630 commands: &mut Commands,
631 app_status: &AppStatus,
632 app_assets: &AppAssets,
633 windows_query: &Query<&Window, With<PrimaryWindow>>,
634 single_mip_level_materials: &mut Assets<SingleMipLevelMaterial>,
635 image_handle: &Handle<Image>,
636) {
637 let window_size = windows_query.iter().next().unwrap().size();
638
639 // Calculate the placement of the column of mipmap levels.
640 let max_slice_size = app_status.max_mip_slice_size(window_size);
641 let y_origin = app_status.vertical_mip_slice_origin(window_size);
642 let y_spacing = app_status.vertical_mip_slice_spacing(window_size);
643 let x_origin = app_status.horizontal_mip_slice_origin(window_size);
644
645 for (mip_level, mip_size) in MipmapSizeIterator::new(app_status).enumerate() {
646 let y_center = y_origin - y_spacing * mip_level as f32;
647
648 // Size each image to fit its container, preserving aspect ratio.
649 let mut slice_size = mip_size.as_vec2();
650 let ratios = max_slice_size / slice_size;
651 let slice_scale = ratios.x.min(ratios.y).min(1.0);
652 slice_size *= slice_scale;
653
654 // Spawn the image. Use the `SingleMipLevelMaterial` with its custom
655 // shader so that only the mip level in question is displayed.
656 commands.spawn((
657 Mesh2d(app_assets.rectangle.clone()),
658 MeshMaterial2d(single_mip_level_materials.add(SingleMipLevelMaterial {
659 mip_level: mip_level as u32,
660 texture: image_handle.clone(),
661 })),
662 Transform::from_xyz(x_origin, y_center, 0.0).with_scale(slice_size.extend(1.0)),
663 ImageView,
664 ));
665
666 // Display a label to the side.
667 commands.spawn((
668 Text2d::new(format!(
669 "Level {}\n{}×{}",
670 mip_level, mip_size.x, mip_size.y
671 )),
672 app_assets.text_font.clone(),
673 TextLayout::justify(Justify::Center),
674 Text2dShadow::default(),
675 Transform::from_xyz(x_origin - max_slice_size.x * 0.5 - 64.0, y_center, 0.0),
676 ImageView,
677 ));
678 }
679}173fn drive_diegetic_pointer(
174 mut cursor_last: Local<Vec2>,
175 mut raycast: MeshRayCast,
176 rays: Res<RayMap>,
177 cubes: Query<&Mesh3d, With<Cube>>,
178 ui_camera: Query<&RenderTarget, With<Camera2d>>,
179 primary_window: Query<Entity, With<PrimaryWindow>>,
180 windows: Query<(Entity, &Window)>,
181 images: Res<Assets<Image>>,
182 manual_texture_views: Res<ManualTextureViews>,
183 mut window_events: MessageReader<WindowEvent>,
184 mut pointer_inputs: MessageWriter<PointerInput>,
185) -> Result {
186 // Get the size of the texture, so we can convert from dimensionless UV coordinates that span
187 // from 0 to 1, to pixel coordinates.
188 let target = ui_camera
189 .single()?
190 .normalize(primary_window.single().ok())
191 .unwrap();
192 let target_info = target
193 .get_render_target_info(windows, &images, &manual_texture_views)
194 .unwrap();
195 let size = target_info.physical_size.as_vec2();
196
197 // Find raycast hits and update the virtual pointer.
198 let raycast_settings = MeshRayCastSettings {
199 visibility: RayCastVisibility::VisibleInView,
200 filter: &|entity| cubes.contains(entity),
201 early_exit_test: &|_| false,
202 };
203 for (_id, ray) in rays.iter() {
204 for (_cube, hit) in raycast.cast_ray(*ray, &raycast_settings) {
205 let position = size * hit.uv.unwrap();
206 if position != *cursor_last {
207 pointer_inputs.write(PointerInput::new(
208 CUBE_POINTER_ID,
209 Location {
210 target: target.clone(),
211 position,
212 },
213 PointerAction::Move {
214 delta: position - *cursor_last,
215 },
216 ));
217 *cursor_last = position;
218 }
219 }
220 }
221
222 // Pipe pointer button presses to the virtual pointer on the UI texture.
223 for window_event in window_events.read() {
224 if let WindowEvent::MouseButtonInput(input) = window_event {
225 let button = match input.button {
226 MouseButton::Left => PointerButton::Primary,
227 MouseButton::Right => PointerButton::Secondary,
228 MouseButton::Middle => PointerButton::Middle,
229 _ => continue,
230 };
231 let action = match input.state {
232 ButtonState::Pressed => PointerAction::Press(button),
233 ButtonState::Released => PointerAction::Release(button),
234 };
235 pointer_inputs.write(PointerInput::new(
236 CUBE_POINTER_ID,
237 Location {
238 target: target.clone(),
239 position: *cursor_last,
240 },
241 action,
242 ));
243 }
244 }
245
246 Ok(())
247}42fn controls(
43 camera_query: Single<(&mut Camera, &mut Transform, &mut Projection)>,
44 window: Single<&Window>,
45 input: Res<ButtonInput<KeyCode>>,
46 time: Res<Time<Fixed>>,
47) {
48 let (mut camera, mut transform, mut projection) = camera_query.into_inner();
49
50 let fspeed = 600.0 * time.delta_secs();
51 let uspeed = fspeed as u32;
52 let window_size = window.resolution.physical_size();
53
54 // Camera movement controls
55 if input.pressed(KeyCode::ArrowUp) {
56 transform.translation.y += fspeed;
57 }
58 if input.pressed(KeyCode::ArrowDown) {
59 transform.translation.y -= fspeed;
60 }
61 if input.pressed(KeyCode::ArrowLeft) {
62 transform.translation.x -= fspeed;
63 }
64 if input.pressed(KeyCode::ArrowRight) {
65 transform.translation.x += fspeed;
66 }
67
68 // Camera zoom controls
69 if let Projection::Orthographic(projection2d) = &mut *projection {
70 if input.pressed(KeyCode::Comma) {
71 projection2d.scale *= powf(4.0f32, time.delta_secs());
72 }
73
74 if input.pressed(KeyCode::Period) {
75 projection2d.scale *= powf(0.25f32, time.delta_secs());
76 }
77 }
78
79 if let Some(viewport) = camera.viewport.as_mut() {
80 // Reset viewport size on window resize
81 if viewport.physical_size.x > window_size.x || viewport.physical_size.y > window_size.y {
82 viewport.physical_size = (window_size.as_vec2() * 0.75).as_uvec2();
83 }
84
85 // Viewport movement controls
86 if input.pressed(KeyCode::KeyW) {
87 viewport.physical_position.y = viewport.physical_position.y.saturating_sub(uspeed);
88 }
89 if input.pressed(KeyCode::KeyS) {
90 viewport.physical_position.y += uspeed;
91 }
92 if input.pressed(KeyCode::KeyA) {
93 viewport.physical_position.x = viewport.physical_position.x.saturating_sub(uspeed);
94 }
95 if input.pressed(KeyCode::KeyD) {
96 viewport.physical_position.x += uspeed;
97 }
98
99 // Bound viewport position so it doesn't go off-screen
100 viewport.physical_position = viewport
101 .physical_position
102 .min(window_size - viewport.physical_size);
103
104 // Viewport size controls
105 if input.pressed(KeyCode::KeyI) {
106 viewport.physical_size.y = viewport.physical_size.y.saturating_sub(uspeed);
107 }
108 if input.pressed(KeyCode::KeyK) {
109 viewport.physical_size.y += uspeed;
110 }
111 if input.pressed(KeyCode::KeyJ) {
112 viewport.physical_size.x = viewport.physical_size.x.saturating_sub(uspeed);
113 }
114 if input.pressed(KeyCode::KeyL) {
115 viewport.physical_size.x += uspeed;
116 }
117
118 // Bound viewport size so it doesn't go off-screen
119 viewport.physical_size = viewport
120 .physical_size
121 .min(window_size - viewport.physical_position)
122 .max(UVec2::new(20, 20));
123 }
124}
125
126fn setup(
127 mut commands: Commands,
128 mut meshes: ResMut<Assets<Mesh>>,
129 mut materials: ResMut<Assets<ColorMaterial>>,
130 window: Single<&Window>,
131) {
132 let window_size = window.resolution.physical_size().as_vec2();
133
134 // Initialize centered, non-window-filling viewport
135 commands.spawn((
136 Camera2d,
137 Camera {
138 viewport: Some(Viewport {
139 physical_position: (window_size * 0.125).as_uvec2(),
140 physical_size: (window_size * 0.75).as_uvec2(),
141 ..default()
142 }),
143 ..default()
144 },
145 ));
146
147 // Create a minimal UI explaining how to interact with the example
148 commands.spawn((
149 Text::new(
150 "Move the mouse to see the circle follow your cursor.\n\
151 Use the arrow keys to move the camera.\n\
152 Use the comma and period keys to zoom in and out.\n\
153 Use the WASD keys to move the viewport.\n\
154 Use the IJKL keys to resize the viewport.",
155 ),
156 Node {
157 position_type: PositionType::Absolute,
158 top: px(12),
159 left: px(12),
160 ..default()
161 },
162 ));
163
164 // Add mesh to make camera movement visible
165 commands.spawn((
166 Mesh2d(meshes.add(Rectangle::new(40.0, 20.0))),
167 MeshMaterial2d(materials.add(Color::from(GREEN))),
168 ));
169
170 // Add background to visualize viewport bounds
171 commands.spawn((
172 Mesh2d(meshes.add(Rectangle::new(50000.0, 50000.0))),
173 MeshMaterial2d(materials.add(Color::linear_rgb(0.01, 0.01, 0.01))),
174 Transform::from_translation(Vec3::new(0.0, 0.0, -200.0)),
175 ));
176}451 pub fn setup(
452 mut commands: Commands,
453 mut textures: ResMut<Assets<Image>>,
454 mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
455 ) {
456 commands.spawn((Camera2d, DespawnOnExit(super::Scene::TextureAtlasBuilder)));
457
458 for (i, padding) in [UVec2::ZERO, PADDING_SIZE].into_iter().enumerate() {
459 // generate solid red green and blue and yellow images
460 let images = [
461 [255, 0, 0, 255],
462 [0, 255, 0, 255],
463 [0, 0, 255, 255],
464 [255, 255, 0, 255],
465 ]
466 .map(|pixel| {
467 Image::new_fill(
468 Extent3d {
469 width: 28,
470 height: 28,
471 depth_or_array_layers: 1,
472 },
473 TextureDimension::D2,
474 &pixel,
475 TextureFormat::Rgba8UnormSrgb,
476 RenderAssetUsages::MAIN_WORLD | RenderAssetUsages::RENDER_WORLD,
477 )
478 });
479
480 let mut texture_atlas_builder = TextureAtlasBuilder::default();
481 texture_atlas_builder
482 .initial_size(ATLAS_SIZE)
483 .max_size(ATLAS_SIZE)
484 .padding(padding);
485 for image in &images {
486 texture_atlas_builder.add_texture(None, image);
487 }
488
489 let (atlas_layout, _, atlas_texture) = texture_atlas_builder.build().expect(
490 "The images are 28 pixels square, so they should fit with 4 pixels left over",
491 );
492 let atlas_layout = texture_atlases.add(atlas_layout);
493
494 let mut nearest_atlas_image = atlas_texture.clone();
495 nearest_atlas_image.sampler = ImageSampler::nearest();
496
497 let atlas_handle = textures.add(atlas_texture);
498 let nearest_atlas_handle = textures.add(nearest_atlas_image);
499
500 let position = ((2. * i as f32 - 1.) * (0.625 * ATLAS_SIZE.x as f32 * ATLAS_SCALE))
501 .round()
502 * Vec3::X;
503
504 commands.spawn((
505 Sprite {
506 image: nearest_atlas_handle,
507 custom_size: Some(ATLAS_SIZE.as_vec2() * ATLAS_SCALE),
508 ..default()
509 },
510 Anchor::BOTTOM_CENTER,
511 ShowAabbGizmo {
512 color: Some(Color::WHITE),
513 },
514 DespawnOnExit(super::Scene::TextureAtlasBuilder),
515 Transform::from_translation(position),
516 ));
517
518 for (index, anchor) in [
519 Anchor::BOTTOM_RIGHT,
520 Anchor::BOTTOM_LEFT,
521 Anchor::TOP_LEFT,
522 Anchor::TOP_RIGHT,
523 ]
524 .into_iter()
525 .enumerate()
526 {
527 commands.spawn((
528 Sprite {
529 image: atlas_handle.clone(),
530 texture_atlas: Some(TextureAtlas {
531 layout: atlas_layout.clone(),
532 index,
533 }),
534 custom_size: Some(IMAGE_SIZE.as_vec2() * IMAGE_SCALE),
535 ..default()
536 },
537 Transform::from_translation(
538 position
539 + -2.
540 * IMAGE_SCALE
541 * (Vec3::Y * IMAGE_SIZE.y as f32 + anchor.as_vec().extend(0.)),
542 ),
543 anchor,
544 DespawnOnExit(super::Scene::TextureAtlasBuilder),
545 ));
546 }
547 }
548 }Sourcepub fn as_i16vec2(self) -> I16Vec2
pub fn as_i16vec2(self) -> I16Vec2
Casts all elements of self to i16.
Sourcepub fn as_u16vec2(self) -> U16Vec2
pub fn as_u16vec2(self) -> U16Vec2
Casts all elements of self to u16.
Sourcepub fn as_i64vec2(self) -> I64Vec2
pub fn as_i64vec2(self) -> I64Vec2
Casts all elements of self to i64.
Sourcepub fn as_u64vec2(self) -> U64Vec2
pub fn as_u64vec2(self) -> U64Vec2
Casts all elements of self to u64.
Sourcepub fn as_isizevec2(self) -> ISizeVec2
pub fn as_isizevec2(self) -> ISizeVec2
Casts all elements of self to isize.
Sourcepub fn as_usizevec2(self) -> USizeVec2
pub fn as_usizevec2(self) -> USizeVec2
Casts all elements of self to usize.
Sourcepub const fn checked_add(self, rhs: UVec2) -> Option<UVec2>
pub const fn checked_add(self, rhs: UVec2) -> Option<UVec2>
Returns a vector containing the wrapping addition of self and rhs.
In other words this computes Some([self.x + rhs.x, self.y + rhs.y, ..]) but returns None on any overflow.
Sourcepub const fn checked_sub(self, rhs: UVec2) -> Option<UVec2>
pub const fn checked_sub(self, rhs: UVec2) -> Option<UVec2>
Returns a vector containing the wrapping subtraction of self and rhs.
In other words this computes Some([self.x - rhs.x, self.y - rhs.y, ..]) but returns None on any overflow.
Sourcepub const fn checked_mul(self, rhs: UVec2) -> Option<UVec2>
pub const fn checked_mul(self, rhs: UVec2) -> Option<UVec2>
Returns a vector containing the wrapping multiplication of self and rhs.
In other words this computes Some([self.x * rhs.x, self.y * rhs.y, ..]) but returns None on any overflow.
Sourcepub const fn checked_div(self, rhs: UVec2) -> Option<UVec2>
pub const fn checked_div(self, rhs: UVec2) -> Option<UVec2>
Returns a vector containing the wrapping division of self and rhs.
In other words this computes Some([self.x / rhs.x, self.y / rhs.y, ..]) but returns None on any division by zero.
Sourcepub const fn wrapping_add(self, rhs: UVec2) -> UVec2
pub const fn wrapping_add(self, rhs: UVec2) -> UVec2
Returns a vector containing the wrapping addition of self and rhs.
In other words this computes [self.x.wrapping_add(rhs.x), self.y.wrapping_add(rhs.y), ..].
Sourcepub const fn wrapping_sub(self, rhs: UVec2) -> UVec2
pub const fn wrapping_sub(self, rhs: UVec2) -> UVec2
Returns a vector containing the wrapping subtraction of self and rhs.
In other words this computes [self.x.wrapping_sub(rhs.x), self.y.wrapping_sub(rhs.y), ..].
Sourcepub const fn wrapping_mul(self, rhs: UVec2) -> UVec2
pub const fn wrapping_mul(self, rhs: UVec2) -> UVec2
Returns a vector containing the wrapping multiplication of self and rhs.
In other words this computes [self.x.wrapping_mul(rhs.x), self.y.wrapping_mul(rhs.y), ..].
Sourcepub const fn wrapping_div(self, rhs: UVec2) -> UVec2
pub const fn wrapping_div(self, rhs: UVec2) -> UVec2
Returns a vector containing the wrapping division of self and rhs.
In other words this computes [self.x.wrapping_div(rhs.x), self.y.wrapping_div(rhs.y), ..].
Sourcepub const fn saturating_add(self, rhs: UVec2) -> UVec2
pub const fn saturating_add(self, rhs: UVec2) -> UVec2
Returns a vector containing the saturating addition of self and rhs.
In other words this computes [self.x.saturating_add(rhs.x), self.y.saturating_add(rhs.y), ..].
Sourcepub const fn saturating_sub(self, rhs: UVec2) -> UVec2
pub const fn saturating_sub(self, rhs: UVec2) -> UVec2
Returns a vector containing the saturating subtraction of self and rhs.
In other words this computes [self.x.saturating_sub(rhs.x), self.y.saturating_sub(rhs.y), ..].
Sourcepub const fn saturating_mul(self, rhs: UVec2) -> UVec2
pub const fn saturating_mul(self, rhs: UVec2) -> UVec2
Returns a vector containing the saturating multiplication of self and rhs.
In other words this computes [self.x.saturating_mul(rhs.x), self.y.saturating_mul(rhs.y), ..].
Sourcepub const fn saturating_div(self, rhs: UVec2) -> UVec2
pub const fn saturating_div(self, rhs: UVec2) -> UVec2
Returns a vector containing the saturating division of self and rhs.
In other words this computes [self.x.saturating_div(rhs.x), self.y.saturating_div(rhs.y), ..].
Sourcepub const fn checked_add_signed(self, rhs: IVec2) -> Option<UVec2>
pub const fn checked_add_signed(self, rhs: IVec2) -> Option<UVec2>
Returns a vector containing the wrapping addition of self and signed vector rhs.
In other words this computes Some([self.x + rhs.x, self.y + rhs.y, ..]) but returns None on any overflow.
Sourcepub const fn wrapping_add_signed(self, rhs: IVec2) -> UVec2
pub const fn wrapping_add_signed(self, rhs: IVec2) -> UVec2
Returns a vector containing the wrapping addition of self and signed vector rhs.
In other words this computes [self.x.wrapping_add_signed(rhs.x), self.y.wrapping_add_signed(rhs.y), ..].
Sourcepub const fn saturating_add_signed(self, rhs: IVec2) -> UVec2
pub const fn saturating_add_signed(self, rhs: IVec2) -> UVec2
Returns a vector containing the saturating addition of self and signed vector rhs.
In other words this computes [self.x.saturating_add_signed(rhs.x), self.y.saturating_add_signed(rhs.y), ..].