pub struct ViewVisibility(/* private fields */);Expand description
Algorithmically computed indication of whether an entity is visible and should be extracted for rendering.
Not to be confused with Visibility and InheritedVisibility.
For more information, see module level documentation.
If you wish to add a custom visibility system that sets this value, be sure to add it to the
CheckVisibility set.
Implementations
Sourceimpl ViewVisibility
impl ViewVisibility
Sourcepub const HIDDEN: ViewVisibility
pub const HIDDEN: ViewVisibility
An entity that cannot be seen from any views.
Sourcepub const VISIBLE: ViewVisibility
pub const VISIBLE: ViewVisibility
An entity that is visible and was visible last frame.
Entities that have opted out of CPU culling, if not hidden due to
inherited visibility, will have their ViewVisibility set to this
value.
Sourcepub fn get(self) -> bool
pub fn get(self) -> bool
Returns true if the entity is visible in any view.
Otherwise, returns false.
Examples found in repository?
examples/stress_tests/many_cubes.rs (line 574)
563fn print_mesh_count(
564 time: Res<Time>,
565 mut timer: Local<PrintingTimer>,
566 sprites: Query<(&Mesh3d, &ViewVisibility)>,
567) {
568 timer.tick(time.delta());
569
570 if timer.just_finished() {
571 info!(
572 "Meshes: {} - Visible Meshes {}",
573 sprites.iter().len(),
574 sprites.iter().filter(|(_, vis)| vis.get()).count(),
575 );
576 }
577}More examples
examples/math/render_primitives.rs (line 642)
632fn rotate_primitive_2d_meshes(
633 mut primitives_2d: Query<
634 (&mut Transform, &ViewVisibility),
635 (With<PrimitiveData>, With<MeshDim2>),
636 >,
637 time: Res<Time>,
638) {
639 let rotation_2d = Quat::from_mat3(&Mat3::from_angle(time.elapsed_secs()));
640 primitives_2d
641 .iter_mut()
642 .filter(|(_, vis)| vis.get())
643 .for_each(|(mut transform, _)| {
644 transform.rotation = rotation_2d;
645 });
646}
647
648fn rotate_primitive_3d_meshes(
649 mut primitives_3d: Query<
650 (&mut Transform, &ViewVisibility),
651 (With<PrimitiveData>, With<MeshDim3>),
652 >,
653 time: Res<Time>,
654) {
655 let rotation_3d = Quat::from_rotation_arc(
656 Vec3::Z,
657 Vec3::new(
658 ops::sin(time.elapsed_secs()),
659 ops::cos(time.elapsed_secs()),
660 ops::sin(time.elapsed_secs()) * 0.5,
661 )
662 .try_normalize()
663 .unwrap_or(Vec3::Z),
664 );
665 primitives_3d
666 .iter_mut()
667 .filter(|(_, vis)| vis.get())
668 .for_each(|(mut transform, _)| {
669 transform.rotation = rotation_3d;
670 });
671}examples/stress_tests/many_text2d.rs (line 198)
179fn print_counts(
180 time: Res<Time>,
181 mut timer: Local<PrintingTimer>,
182 texts: Query<&ViewVisibility, With<Text2d>>,
183 font_atlas_set: Res<FontAtlasSet>,
184 images: Res<Assets<Image>>,
185) {
186 timer.tick(time.delta());
187 if !timer.just_finished() {
188 return;
189 }
190
191 let num_atlases = font_atlas_set
192 .iter()
193 // Removed this filter for now as the keys no longer include the AssetIds
194 // .filter(|(key, _)| key.0 == font_id)
195 .map(|(_, atlases)| atlases.len())
196 .sum::<usize>();
197
198 let visible_texts = texts.iter().filter(|visibility| visibility.get()).count();
199
200 info!(
201 "Texts: {} Visible: {} Atlases: {} Bytes: {}",
202 texts.iter().count(),
203 visible_texts,
204 num_atlases,
205 font_atlas_set.total_bytes(images.as_ref())
206 );
207}examples/shader_advanced/manual_material.rs (line 254)
242fn extract_image_materials(
243 mut material_instances: ResMut<RenderMaterialInstances>,
244 changed_meshes_query: Extract<
245 Query<
246 (Entity, &ViewVisibility, &ImageMaterial3d),
247 Or<(Changed<ViewVisibility>, Changed<ImageMaterial3d>)>,
248 >,
249 >,
250) {
251 let last_change_tick = material_instances.current_change_tick;
252
253 for (entity, view_visibility, material) in &changed_meshes_query {
254 if view_visibility.get() {
255 material_instances.instances.insert(
256 entity.into(),
257 RenderMaterialInstance {
258 asset_id: material.0.id().untyped(),
259 last_change_tick,
260 },
261 );
262 } else {
263 material_instances
264 .instances
265 .remove(&MainEntity::from(entity));
266 }
267 }
268}examples/2d/mesh2d_manual.rs (line 353)
332pub fn extract_colored_mesh2d(
333 mut commands: Commands,
334 mut previous_len: Local<usize>,
335 // When extracting, you must use `Extract` to mark the `SystemParam`s
336 // which should be taken from the main world.
337 query: Extract<
338 Query<
339 (
340 Entity,
341 RenderEntity,
342 &ViewVisibility,
343 &GlobalTransform,
344 &Mesh2d,
345 ),
346 With<ColoredMesh2d>,
347 >,
348 >,
349 mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
350) {
351 let mut values = Vec::with_capacity(*previous_len);
352 for (entity, render_entity, view_visibility, transform, handle) in &query {
353 if !view_visibility.get() {
354 continue;
355 }
356
357 let transforms = Mesh2dTransforms {
358 world_from_local: transform.affine().into(),
359 flags: MeshFlags::empty().bits(),
360 };
361
362 values.push((render_entity, ColoredMesh2d));
363 render_mesh_instances.insert(
364 entity.into(),
365 RenderMesh2dInstance {
366 mesh_asset_id: handle.0.id(),
367 transforms,
368 material_bind_group_id: Material2dBindGroupId::default(),
369 automatic_batching: false,
370 tag: 0,
371 },
372 );
373 }
374 *previous_len = values.len();
375 commands.try_insert_batch(values);
376}Trait Implementations
Sourceimpl Clone for ViewVisibility
impl Clone for ViewVisibility
Sourcefn clone(&self) -> ViewVisibility
fn clone(&self) -> ViewVisibility
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Component for ViewVisibility
impl Component for ViewVisibility
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
A constant indicating the storage type used for this component.
Sourcetype Mutability = Mutable
type Mutability = Mutable
A marker type to assist Bevy with determining if this component is
mutable, or immutable. Mutable components will have
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Registers required components. Read more
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Called when registering this component, allowing to override clone function (or disable cloning altogether) for this component. Read more
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<ViewVisibility>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<ViewVisibility>>
Returns
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
Maps the entities on this component using the given
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreimpl Copy for ViewVisibility
Sourceimpl Debug for ViewVisibility
impl Debug for ViewVisibility
Sourceimpl Default for ViewVisibility
impl Default for ViewVisibility
Sourcefn default() -> ViewVisibility
fn default() -> ViewVisibility
Returns the “default value” for a type. Read more
impl Eq for ViewVisibility
Sourceimpl FromArg for ViewVisibility
impl FromArg for ViewVisibility
Sourceimpl FromReflect for ViewVisibility
impl FromReflect for ViewVisibility
Sourcefn from_reflect(
reflect: &(dyn PartialReflect + 'static),
) -> Option<ViewVisibility>
fn from_reflect( reflect: &(dyn PartialReflect + 'static), ) -> Option<ViewVisibility>
Constructs a concrete instance of
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Attempts to downcast the given value to
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for ViewVisibility
impl GetOwnership for ViewVisibility
Sourceimpl GetTypeRegistration for ViewVisibility
impl GetTypeRegistration for ViewVisibility
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
Returns the default
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Registers other types needed by this type. Read more
Sourceimpl IntoReturn for ViewVisibility
impl IntoReturn for ViewVisibility
Sourcefn into_return<'into_return>(self) -> Return<'into_return>where
ViewVisibility: 'into_return,
fn into_return<'into_return>(self) -> Return<'into_return>where
ViewVisibility: 'into_return,
Sourceimpl PartialEq for ViewVisibility
impl PartialEq for ViewVisibility
Sourceimpl PartialReflect for ViewVisibility
impl PartialReflect for ViewVisibility
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Returns a zero-sized enumeration of “kinds” of type. Read more
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Returns an immutable enumeration of “kinds” of type. Read more
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Returns a mutable enumeration of “kinds” of type. Read more
Sourcefn reflect_owned(self: Box<ViewVisibility>) -> ReflectOwned
fn reflect_owned(self: Box<ViewVisibility>) -> ReflectOwned
Returns an owned enumeration of “kinds” of type. Read more
Sourcefn try_into_reflect(
self: Box<ViewVisibility>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<ViewVisibility>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Attempts to cast this type to a boxed, fully-reflected value.
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Attempts to cast this type to a fully-reflected value.
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Attempts to cast this type to a mutable, fully-reflected value.
Sourcefn into_partial_reflect(self: Box<ViewVisibility>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<ViewVisibility>) -> Box<dyn PartialReflect>
Casts this type to a boxed, reflected value. Read more
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Casts this type to a reflected value. Read more
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Casts this type to a mutable, reflected value. Read more
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Returns a “partial equality” comparison result. Read more
Sourcefn reflect_partial_cmp(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Returns a “partial comparison” result. Read more
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Debug formatter for the value. Read more
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Attempts to clone
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Applies a reflected value to this value. Read more
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
For a type implementing
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Returns a hash of the value (which includes the type). Read more
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Indicates whether or not this type is a dynamic type. Read more
Sourceimpl Reflect for ViewVisibility
impl Reflect for ViewVisibility
Sourcefn into_any(self: Box<ViewVisibility>) -> Box<dyn Any>
fn into_any(self: Box<ViewVisibility>) -> Box<dyn Any>
Returns the value as a
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
Returns the value as a
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<ViewVisibility>) -> Box<dyn Reflect>
fn into_reflect(self: Box<ViewVisibility>) -> Box<dyn Reflect>
Casts this type to a boxed, fully-reflected value.
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Casts this type to a fully-reflected value.
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Casts this type to a mutable, fully-reflected value.
impl StructuralPartialEq for ViewVisibility
Sourceimpl TupleStruct for ViewVisibility
impl TupleStruct for ViewVisibility
Sourcefn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
Returns a reference to the value of the field with index
index as a
&dyn Reflect.Sourcefn field_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
Returns a mutable reference to the value of the field with index
index
as a &mut dyn Reflect.Sourcefn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
fn iter_fields(&self) -> TupleStructFieldIter<'_> ⓘ
Returns an iterator over the values of the tuple struct’s fields.
Sourcefn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
fn to_dynamic_tuple_struct(&self) -> DynamicTupleStruct
Creates a new
DynamicTupleStruct from this tuple struct.Sourcefn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
fn get_represented_tuple_struct_info(&self) -> Option<&'static TupleStructInfo>
Will return
None if TypeInfo is not available.Sourceimpl TypePath for ViewVisibility
impl TypePath for ViewVisibility
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Returns the fully qualified path of the underlying type. Read more
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Returns a short, pretty-print enabled path to the type. Read more
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>