pub struct DynamicArray { /* private fields */ }Expand description
A fixed-size list of reflected values.
This differs from DynamicList in that the size of the DynamicArray
is constant, whereas a DynamicList can have items added and removed.
This isn’t to say that a DynamicArray is immutable— its items
can be mutated— just that the number of items cannot change.
Implementations
Sourceimpl DynamicArray
impl DynamicArray
Sourcepub fn new(values: Box<[Box<dyn PartialReflect>]>) -> DynamicArray
pub fn new(values: Box<[Box<dyn PartialReflect>]>) -> DynamicArray
Creates a new DynamicArray.
Sourcepub fn set_represented_type(
&mut self,
represented_type: Option<&'static TypeInfo>,
)
pub fn set_represented_type( &mut self, represented_type: Option<&'static TypeInfo>, )
Sets the type to be represented by this DynamicArray.
Panics
Panics if the given type is not a TypeInfo::Array.