Cameras, meshes, textures, shaders, and pipelines.
Use RenderDevice::features,
RenderDevice::limits, and the
WgpuAdapterInfo resource to
get runtime information about the actual adapter, backend, features, and limits.
Modules
Overview
Structs
The simulation
World of the application, stored as a resource.
This resource is only available during RenderStage::Extract and not
during command application of that stage.
See Extract for more details.A Label for the rendering sub-app.
Contains the default Bevy rendering backend based on wgpu.
The Render App World. This is only available as a resource during the Extract step.
Enums
The labels of the default App rendering stages.