pub struct InstanceInputUniformBuffer<BDI>where
BDI: AtomicPod,{ /* private fields */ }bevy_render only.Expand description
Holds the GPU buffer of instance input data, which is the data about each mesh instance that the CPU provides.
BDI is the buffer data input type, which the GPU mesh preprocessing
shader is expected to expand to the full buffer data type.
Implementations
Sourceimpl<BDI> InstanceInputUniformBuffer<BDI>where
BDI: AtomicPod,
impl<BDI> InstanceInputUniformBuffer<BDI>where
BDI: AtomicPod,
Sourcepub fn new() -> InstanceInputUniformBuffer<BDI>
pub fn new() -> InstanceInputUniformBuffer<BDI>
Creates a new, empty buffer.
Sourcepub fn buffer(&self) -> &AtomicSparseBufferVec<BDI>
pub fn buffer(&self) -> &AtomicSparseBufferVec<BDI>
Returns the AtomicSparseBufferVec corresponding to this input
uniform buffer.
Sourcepub fn add(&mut self, element: BDI) -> u32
pub fn add(&mut self, element: BDI) -> u32
Adds a new piece of buffered data to the uniform buffer and returns its index.
Sourcepub fn remove(&mut self, uniform_index: u32)
pub fn remove(&mut self, uniform_index: u32)
Removes a piece of buffered data from the uniform buffer.
This simply marks the data as free.
Sourcepub fn get(&self, uniform_index: u32) -> Option<BDI>
pub fn get(&self, uniform_index: u32) -> Option<BDI>
Returns the piece of buffered data at the given index.
Returns None if the index is out of bounds or the data is removed.
Sourcepub fn get_unchecked(&self, uniform_index: u32) -> BDI
pub fn get_unchecked(&self, uniform_index: u32) -> BDI
Returns the piece of buffered data at the given index. Can return data that has previously been removed.
Panics
if uniform_index is not in bounds of Self::buffer.
Sourcepub fn set(&self, uniform_index: u32, element: BDI)
pub fn set(&self, uniform_index: u32, element: BDI)
Stores a piece of buffered data at the given index.
Panics
if uniform_index is not in bounds of Self::buffer.
pub fn ensure_nonempty(&mut self)
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Returns true if this buffer has no instances or false if it contains any instances.
Sourcepub fn into_buffer(self) -> AtomicSparseBufferVec<BDI>
pub fn into_buffer(self) -> AtomicSparseBufferVec<BDI>
Consumes this InstanceInputUniformBuffer and returns the raw buffer
ready to be uploaded to the GPU.