PhaseBatchedInstanceBuffers

Struct PhaseBatchedInstanceBuffers 

pub struct PhaseBatchedInstanceBuffers<PI, BD>
where PI: PhaseItem, BD: GpuArrayBufferable + Sync + Send + 'static,
{ pub buffers: UntypedPhaseBatchedInstanceBuffers<BD>, /* private fields */ }
The GPU buffers holding the data needed to render batches for a single phase.

These are split out per phase so that we can run the phases in parallel. This is the version of the structure that has a type parameter, which enables Bevy’s scheduler to run the batching operations for the different phases in parallel.

See the documentation for BatchedInstanceBuffers for more information.

Fields

buffers: UntypedPhaseBatchedInstanceBuffers<BD>

The buffers for this phase.