Module camera

Module camera 

Structs

CameraMainPassTextureFormats
Main-pass color TextureFormat keyed by camera render entity.
CameraPlugin
CameraRenderGraph
Configures the render schedule to be run for a given Camera entity.
DirtySpecializations
Stores information about all entities that have changed in such a way as to potentially require their pipelines to be re-specialized.
DirtyWireframeSpecializations
Stores information about all entities that have changed in such a way as to potentially require their wireframe pipelines to be re-specialized.
ExtractedCamera
Describes a Camera in the render world.
MipBias
Camera component specifying a mip bias to apply when sampling from material textures.
PendingQueues
Holds all entities that couldn’t be specialized and/or queued because their materials or other dependent resources hadn’t loaded yet.
SortedCamera
SortedCameras
Cameras sorted by their order field. This is updated in the sort_cameras system.
TemporalJitter
A subpixel offset to jitter a perspective camera’s frustum by.
ViewPendingQueues
Holds all entities that couldn’t be specialized and/or queued because their materials and/or other dependent resources hadn’t loaded yet for a single view.

Enums

DirtySpecializationSystems
A SystemSet that contains all systems that mutate the DirtySpecializations resource and other resources that wrap that type.
MissingRenderTargetInfoError

Traits

NormalizedRenderTargetExt

Functions

camera_system
System in charge of updating a Camera when its window or projection changes.
clear_dirty_specializations
Clears out the DirtySpecializations resource in preparation for a new frame.
clear_dirty_wireframe_specializations
Clears out the DirtyWireframeSpecializations resource in preparation for a new frame.
expire_specializations_for_views
A system that removes views that don’t exist any longer from DirtySpecializations.
expire_wireframe_specializations_for_views
A system that removes views that don’t exist any longer from DirtyWireframeSpecializations.
extract_cameras
sort_cameras