Structs
The defining component for camera entities, storing information about how and what to render
through this camera.
Adds
Camera driver systems for a given projection type.Configures the
RenderGraph name assigned to be run for a given Camera entity.Label for
camera_system<T>, shared accross all T.Holds internally computed
Camera values.A 3D camera projection in which distant objects appear smaller than close objects.
Information about the current
RenderTarget.Enums
A configurable
CameraProjection that can select its projection type at runtime.Traits
Trait to control the projection matrix of a camera.
Functions
System in charge of updating a
Camera when its window or projection changes.