pub enum Readback {
Texture(Handle<Image>),
Buffer {
buffer: Handle<ShaderBuffer>,
start_offset_and_size: Option<(u64, u64)>,
},
}Expand description
A component that registers the wrapped handle for gpu readback, either a texture or a buffer.
Data is read asynchronously and will be triggered on the entity via the ReadbackComplete event
when complete. If this component is not removed, the readback will be attempted every frame
Variants
Implementations
Sourceimpl Readback
impl Readback
Sourcepub fn texture(image: Handle<Image>) -> Readback
pub fn texture(image: Handle<Image>) -> Readback
Create a readback component for a texture using the given handle.
Examples found in repository?
examples/shader/gpu_readback.rs (line 128)
67fn setup(
68 mut commands: Commands,
69 mut images: ResMut<Assets<Image>>,
70 mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72 // Create a storage buffer with some data
73 let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74 let mut buffer = ShaderBuffer::from(buffer);
75 // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76 buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77 let buffer = buffers.add(buffer);
78
79 // Create a storage texture with some data
80 let size = Extent3d {
81 width: BUFFER_LEN as u32,
82 height: 1,
83 ..default()
84 };
85 // We create an uninitialized image since this texture will only be used for getting data out
86 // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87 let mut image = Image::new_uninit(
88 size,
89 TextureDimension::D2,
90 TextureFormat::R32Uint,
91 RenderAssetUsages::RENDER_WORLD,
92 );
93 // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94 // compute shader
95 image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96 let image = images.add(image);
97
98 // Spawn the readback components. For each frame, the data will be read back from the GPU
99 // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100 // to stop reading back the data.
101 commands
102 .spawn(Readback::buffer(buffer.clone()))
103 .observe(|event: On<ReadbackComplete>| {
104 // This matches the type which was used to create the `ShaderBuffer` above,
105 // and is a convenient way to interpret the data.
106 let data: Vec<u32> = event.to_shader_type();
107 info!("Buffer {:?}", data);
108 });
109
110 // It is also possible to read only a range of the buffer.
111 commands
112 .spawn(Readback::buffer_range(
113 buffer.clone(),
114 4 * u32::SHADER_SIZE.get(), // skip the first four elements
115 8 * u32::SHADER_SIZE.get(), // read eight elements
116 ))
117 .observe(|event: On<ReadbackComplete>| {
118 let data: Vec<u32> = event.to_shader_type();
119 info!("Buffer range {:?}", data);
120 });
121
122 // This is just a simple way to pass the buffer handle to the render app for our compute node
123 commands.insert_resource(ReadbackBuffer(buffer));
124
125 // Textures can also be read back from the GPU. Pay careful attention to the format of the
126 // texture, as it will affect how the data is interpreted.
127 commands
128 .spawn(Readback::texture(image.clone()))
129 .observe(|event: On<ReadbackComplete>| {
130 // You probably want to interpret the data as a color rather than a `ShaderType`,
131 // but in this case we know the data is a single channel storage texture, so we can
132 // interpret it as a `Vec<u32>`
133 let data: Vec<u32> = event.to_shader_type();
134 info!("Image {:?}", data);
135 });
136 commands.insert_resource(ReadbackImage(image));
137}Sourcepub fn buffer(buffer: Handle<ShaderBuffer>) -> Readback
pub fn buffer(buffer: Handle<ShaderBuffer>) -> Readback
Create a readback component for a full buffer using the given handle.
Examples found in repository?
examples/shader/gpu_readback.rs (line 102)
67fn setup(
68 mut commands: Commands,
69 mut images: ResMut<Assets<Image>>,
70 mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72 // Create a storage buffer with some data
73 let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74 let mut buffer = ShaderBuffer::from(buffer);
75 // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76 buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77 let buffer = buffers.add(buffer);
78
79 // Create a storage texture with some data
80 let size = Extent3d {
81 width: BUFFER_LEN as u32,
82 height: 1,
83 ..default()
84 };
85 // We create an uninitialized image since this texture will only be used for getting data out
86 // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87 let mut image = Image::new_uninit(
88 size,
89 TextureDimension::D2,
90 TextureFormat::R32Uint,
91 RenderAssetUsages::RENDER_WORLD,
92 );
93 // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94 // compute shader
95 image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96 let image = images.add(image);
97
98 // Spawn the readback components. For each frame, the data will be read back from the GPU
99 // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100 // to stop reading back the data.
101 commands
102 .spawn(Readback::buffer(buffer.clone()))
103 .observe(|event: On<ReadbackComplete>| {
104 // This matches the type which was used to create the `ShaderBuffer` above,
105 // and is a convenient way to interpret the data.
106 let data: Vec<u32> = event.to_shader_type();
107 info!("Buffer {:?}", data);
108 });
109
110 // It is also possible to read only a range of the buffer.
111 commands
112 .spawn(Readback::buffer_range(
113 buffer.clone(),
114 4 * u32::SHADER_SIZE.get(), // skip the first four elements
115 8 * u32::SHADER_SIZE.get(), // read eight elements
116 ))
117 .observe(|event: On<ReadbackComplete>| {
118 let data: Vec<u32> = event.to_shader_type();
119 info!("Buffer range {:?}", data);
120 });
121
122 // This is just a simple way to pass the buffer handle to the render app for our compute node
123 commands.insert_resource(ReadbackBuffer(buffer));
124
125 // Textures can also be read back from the GPU. Pay careful attention to the format of the
126 // texture, as it will affect how the data is interpreted.
127 commands
128 .spawn(Readback::texture(image.clone()))
129 .observe(|event: On<ReadbackComplete>| {
130 // You probably want to interpret the data as a color rather than a `ShaderType`,
131 // but in this case we know the data is a single channel storage texture, so we can
132 // interpret it as a `Vec<u32>`
133 let data: Vec<u32> = event.to_shader_type();
134 info!("Image {:?}", data);
135 });
136 commands.insert_resource(ReadbackImage(image));
137}Sourcepub fn buffer_range(
buffer: Handle<ShaderBuffer>,
start_offset: u64,
size: u64,
) -> Readback
pub fn buffer_range( buffer: Handle<ShaderBuffer>, start_offset: u64, size: u64, ) -> Readback
Create a readback component for a buffer range using the given handle, a start offset in bytes and a number of bytes to read.
Examples found in repository?
examples/shader/gpu_readback.rs (lines 112-116)
67fn setup(
68 mut commands: Commands,
69 mut images: ResMut<Assets<Image>>,
70 mut buffers: ResMut<Assets<ShaderBuffer>>,
71) {
72 // Create a storage buffer with some data
73 let buffer: Vec<u32> = (0..BUFFER_LEN as u32).collect();
74 let mut buffer = ShaderBuffer::from(buffer);
75 // We need to enable the COPY_SRC usage so we can copy the buffer to the cpu
76 buffer.buffer_description.usage |= BufferUsages::COPY_SRC;
77 let buffer = buffers.add(buffer);
78
79 // Create a storage texture with some data
80 let size = Extent3d {
81 width: BUFFER_LEN as u32,
82 height: 1,
83 ..default()
84 };
85 // We create an uninitialized image since this texture will only be used for getting data out
86 // of the compute shader, not getting data in, so there's no reason for it to exist on the CPU
87 let mut image = Image::new_uninit(
88 size,
89 TextureDimension::D2,
90 TextureFormat::R32Uint,
91 RenderAssetUsages::RENDER_WORLD,
92 );
93 // We also need to enable the COPY_SRC, as well as STORAGE_BINDING so we can use it in the
94 // compute shader
95 image.texture_descriptor.usage |= TextureUsages::COPY_SRC | TextureUsages::STORAGE_BINDING;
96 let image = images.add(image);
97
98 // Spawn the readback components. For each frame, the data will be read back from the GPU
99 // asynchronously and trigger the `ReadbackComplete` event on this entity. Despawn the entity
100 // to stop reading back the data.
101 commands
102 .spawn(Readback::buffer(buffer.clone()))
103 .observe(|event: On<ReadbackComplete>| {
104 // This matches the type which was used to create the `ShaderBuffer` above,
105 // and is a convenient way to interpret the data.
106 let data: Vec<u32> = event.to_shader_type();
107 info!("Buffer {:?}", data);
108 });
109
110 // It is also possible to read only a range of the buffer.
111 commands
112 .spawn(Readback::buffer_range(
113 buffer.clone(),
114 4 * u32::SHADER_SIZE.get(), // skip the first four elements
115 8 * u32::SHADER_SIZE.get(), // read eight elements
116 ))
117 .observe(|event: On<ReadbackComplete>| {
118 let data: Vec<u32> = event.to_shader_type();
119 info!("Buffer range {:?}", data);
120 });
121
122 // This is just a simple way to pass the buffer handle to the render app for our compute node
123 commands.insert_resource(ReadbackBuffer(buffer));
124
125 // Textures can also be read back from the GPU. Pay careful attention to the format of the
126 // texture, as it will affect how the data is interpreted.
127 commands
128 .spawn(Readback::texture(image.clone()))
129 .observe(|event: On<ReadbackComplete>| {
130 // You probably want to interpret the data as a color rather than a `ShaderType`,
131 // but in this case we know the data is a single channel storage texture, so we can
132 // interpret it as a `Vec<u32>`
133 let data: Vec<u32> = event.to_shader_type();
134 info!("Image {:?}", data);
135 });
136 commands.insert_resource(ReadbackImage(image));
137}Trait Implementations
Sourceimpl Component for Readback
impl Component for Readback
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
A constant indicating the storage type used for this component.
Sourcetype Mutability = Mutable
type Mutability = Mutable
A marker type to assist Bevy with determining if this component is
mutable, or immutable. Mutable components will have
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Registers required components. Read more
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Called when registering this component, allowing to override clone function (or disable cloning altogether) for this component. Read more
Sourcefn map_entities<M>(this: &mut Readback, mapper: &mut M)where
M: EntityMapper,
fn map_entities<M>(this: &mut Readback, mapper: &mut M)where
M: EntityMapper,
Maps the entities on this component using the given
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<Readback>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<Readback>>
Returns
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourceimpl ExtractComponent for Readback
impl ExtractComponent for Readback
Sourcetype QueryData = &'static Readback
type QueryData = &'static Readback
ECS
ReadOnlyQueryData to fetch the components to extract.Sourcetype QueryFilter = ()
type QueryFilter = ()
Filters the entities with additional constraints.
Sourcetype Out = Readback
type Out = Readback
The output from extraction, i.e.
ExtractComponent::extract_component. Read moreSourcefn extract_component(
item: <<Readback as ExtractComponent>::QueryData as QueryData>::Item<'_, '_>,
) -> Option<<Readback as ExtractComponent>::Out>
fn extract_component( item: <<Readback as ExtractComponent>::QueryData as QueryData>::Item<'_, '_>, ) -> Option<<Readback as ExtractComponent>::Out>
Defines how the component is transferred into the “render world”. Read more
Sourceimpl FromTemplate for Readback
impl FromTemplate for Readback
Sourcetype Template = ReadbackTemplate
type Template = ReadbackTemplate
The
Template for this type.Sourceimpl SyncComponent for Readback
impl SyncComponent for Readback
impl Unpin for Readbackwhere
[()]: for<'a> SpecializeFromTemplate,