ArrayLength

Struct ArrayLength 

pub struct ArrayLength;
Helper type meant to be used together with the ShaderType derive macro

This type should be interpreted as an u32 in the shader

Problem

There are cases where the use of the WGSL function arrayLength() might be inadequate because of its return value

  • being a minimum of 1 due to how minBindingSize is calculated

  • possibly being higher than expected due to padding at the end of a struct or buffer being interpreted as array elements

  • representing the capacity of the array for use cases that require oversized buffers

Solution

Using this type on a field of a struct with the ShaderType derive macro will automatically:

  • on write, write the length of the contained runtime-sized array as an u32 to the buffer

  • on read, read the value as an u32 from the buffer (rep as LEN) and when reading the elements of the contained runtime-sized array a max of LEN elements will be read