pub struct ArrayLength;Expand description
Helper type meant to be used together with the ShaderType derive macro
This type should be interpreted as an u32 in the shader
Problem
There are cases where the use of the WGSL function arrayLength()
might be inadequate because of its return value
-
being a minimum of 1 due to how
minBindingSizeis calculated -
possibly being higher than expected due to padding at the end of a struct or buffer being interpreted as array elements
-
representing the capacity of the array for use cases that require oversized buffers
Solution
Using this type on a field of a struct with the ShaderType derive macro will automatically: