Struct bevy::render::renderer::RenderDevice
pub struct RenderDevice { /* private fields */ }Expand description
This GPU device is responsible for the creation of most rendering and compute resources.
Implementations
impl RenderDevice
impl RenderDevice
pub fn features(&self) -> Features
pub fn features(&self) -> Features
List all Features that may be used with this device.
Functions may panic if you use unsupported features.
pub fn limits(&self) -> Limits
pub fn limits(&self) -> Limits
List all Limits that were requested of this device.
If any of these limits are exceeded, functions may panic.
pub fn create_shader_module(
&self,
desc: ShaderModuleDescriptor<'_>
) -> ShaderModule
pub fn create_shader_module(
&self,
desc: ShaderModuleDescriptor<'_>
) -> ShaderModule
Creates a ShaderModule from either SPIR-V or WGSL source code.
pub fn poll(&self, maintain: Maintain<SubmissionIndex>)
pub fn poll(&self, maintain: Maintain<SubmissionIndex>)
Check for resource cleanups and mapping callbacks.
no-op on the web, device is automatically polled.
pub fn create_command_encoder(
&self,
desc: &CommandEncoderDescriptor<Option<&str>>
) -> CommandEncoder
pub fn create_command_encoder(
&self,
desc: &CommandEncoderDescriptor<Option<&str>>
) -> CommandEncoder
Creates an empty CommandEncoder.
pub fn create_render_bundle_encoder(
&self,
desc: &RenderBundleEncoderDescriptor<'_>
) -> RenderBundleEncoder<'_>
pub fn create_render_bundle_encoder(
&self,
desc: &RenderBundleEncoderDescriptor<'_>
) -> RenderBundleEncoder<'_>
Creates an empty RenderBundleEncoder.
pub fn create_bind_group(&self, desc: &BindGroupDescriptor<'_>) -> BindGroup
pub fn create_bind_group(&self, desc: &BindGroupDescriptor<'_>) -> BindGroup
Creates a new BindGroup.
pub fn create_bind_group_layout(
&self,
desc: &BindGroupLayoutDescriptor<'_>
) -> BindGroupLayout
pub fn create_bind_group_layout(
&self,
desc: &BindGroupLayoutDescriptor<'_>
) -> BindGroupLayout
Creates a BindGroupLayout.
pub fn create_pipeline_layout(
&self,
desc: &PipelineLayoutDescriptor<'_>
) -> PipelineLayout
pub fn create_pipeline_layout(
&self,
desc: &PipelineLayoutDescriptor<'_>
) -> PipelineLayout
Creates a PipelineLayout.
pub fn create_render_pipeline(
&self,
desc: &RenderPipelineDescriptor<'_>
) -> RenderPipeline
pub fn create_render_pipeline(
&self,
desc: &RenderPipelineDescriptor<'_>
) -> RenderPipeline
Creates a RenderPipeline.
pub fn create_compute_pipeline(
&self,
desc: &ComputePipelineDescriptor<'_>
) -> ComputePipeline
pub fn create_compute_pipeline(
&self,
desc: &ComputePipelineDescriptor<'_>
) -> ComputePipeline
Creates a ComputePipeline.
pub fn create_buffer(&self, desc: &BufferDescriptor<Option<&str>>) -> Buffer
pub fn create_buffer(&self, desc: &BufferDescriptor<Option<&str>>) -> Buffer
Creates a Buffer.
pub fn create_buffer_with_data(&self, desc: &BufferInitDescriptor<'_>) -> Buffer
pub fn create_buffer_with_data(&self, desc: &BufferInitDescriptor<'_>) -> Buffer
Creates a Buffer and initializes it with the specified data.
pub fn create_texture_with_data(
&self,
render_queue: &RenderQueue,
desc: &TextureDescriptor<Option<&str>>,
data: &[u8]
) -> Texture
pub fn create_texture_with_data(
&self,
render_queue: &RenderQueue,
desc: &TextureDescriptor<Option<&str>>,
data: &[u8]
) -> Texture
Creates a new Texture and initializes it with the specified data.
desc specifies the general format of the texture.
data is the raw data.
pub fn create_texture(&self, desc: &TextureDescriptor<Option<&str>>) -> Texture
pub fn create_texture(&self, desc: &TextureDescriptor<Option<&str>>) -> Texture
Creates a new Texture.
desc specifies the general format of the texture.
pub fn create_sampler(&self, desc: &SamplerDescriptor<'_>) -> Sampler
pub fn create_sampler(&self, desc: &SamplerDescriptor<'_>) -> Sampler
Creates a new Sampler.
desc specifies the behavior of the sampler.
pub fn configure_surface(&self, surface: &Surface, config: &SurfaceConfiguration)
pub fn configure_surface(&self, surface: &Surface, config: &SurfaceConfiguration)
Initializes Surface for presentation.
Panics
- A old
SurfaceTextureis still alive referencing an old surface. - Texture format requested is unsupported on the surface.
pub fn wgpu_device(&self) -> &Device
pub fn wgpu_device(&self) -> &Device
Returns the wgpu Device.
pub fn map_buffer(
&self,
buffer: &BufferSlice<'_>,
map_mode: MapMode,
callback: impl FnOnce(Result<(), BufferAsyncError>) + Send + 'static
)
pub fn align_copy_bytes_per_row(row_bytes: usize) -> usize
pub fn get_supported_read_only_binding_type(
&self,
buffers_per_shader_stage: u32
) -> BufferBindingType