Structs
Represents the backends that wgpu will use.
Features that are not guaranteed to be supported.
Represents the sets of limits an adapter/device supports.
Provides configuration for renderer initialization. Use
RenderDevice::features,
RenderDevice::limits, and the WgpuAdapterInfo
resource to get runtime information about the actual adapter, backend, features, and limits.
NOTE: Backends::DX12, Backends::METAL, and
Backends::VULKAN are enabled by default for non-web and the best choice
is automatically selected. Web using the webgl feature uses Backends::GL.
NOTE: If you want to use Backends::GL in a native app on Windows, you must
use ANGLE. This is because wgpu requires EGL to
create a GL context without a window and only ANGLE supports that.Enums
Power Preference when choosing a physical adapter.
Configures the priority used when automatically configuring the features/limits of
wgpu.Functions
Get a features/limits priority from the environment variable
WGPU_SETTINGS_PRIO