pub struct WgpuSettings {Show 15 fields
pub device_label: Option<Cow<'static, str>>,
pub backends: Option<Backends>,
pub power_preference: PowerPreference,
pub priority: WgpuSettingsPriority,
pub features: Features,
pub disabled_features: Option<Features>,
pub limits: Limits,
pub constrained_limits: Option<Limits>,
pub dx12_shader_compiler: Dx12Compiler,
pub gles3_minor_version: Gles3MinorVersion,
pub instance_flags: InstanceFlags,
pub memory_hints: MemoryHints,
pub instance_memory_budget_thresholds: MemoryBudgetThresholds,
pub force_fallback_adapter: bool,
pub adapter_name: Option<String>,
}Expand description
Provides configuration for renderer initialization. Use RenderDevice::features,
RenderDevice::limits, and the RenderAdapterInfo
resource to get runtime information about the actual adapter, backend, features, and limits.
NOTE: Backends::DX12, Backends::METAL, and
Backends::VULKAN are enabled by default for non-web and the best choice
is automatically selected. Web using the webgl feature uses Backends::GL.
NOTE: If you want to use Backends::GL in a native app on Windows and/or macOS, you must
use ANGLE and enable the gles feature. This is
because wgpu requires EGL to create a GL context without a window and only ANGLE supports that.
Fields
device_label: Option<Cow<'static, str>>backends: Option<Backends>power_preference: PowerPreferencepriority: WgpuSettingsPriorityfeatures: FeaturesThe features to ensure are enabled regardless of what the adapter/backend supports. Setting these explicitly may cause renderer initialization to fail.
disabled_features: Option<Features>The features to ensure are disabled regardless of what the adapter/backend supports
limits: LimitsThe imposed limits.
constrained_limits: Option<Limits>The constraints on limits allowed regardless of what the adapter/backend supports
dx12_shader_compiler: Dx12CompilerThe shader compiler to use for the DX12 backend.
gles3_minor_version: Gles3MinorVersionAllows you to choose which minor version of GLES3 to use (3.0, 3.1, 3.2, or automatic) This only applies when using ANGLE and the GL backend.
instance_flags: InstanceFlagsThese are for controlling WGPU’s debug information to eg. enable validation and shader debug info in release builds.
memory_hints: MemoryHintsThis hints to the WGPU device about the preferred memory allocation strategy.
instance_memory_budget_thresholds: MemoryBudgetThresholdsThe thresholds for device memory budget.
force_fallback_adapter: boolIf true, will force wgpu to use a software renderer, if available.
adapter_name: Option<String>The name of the adapter to use.