pub struct ViewTarget { /* private fields */ }Implementations
Sourceimpl ViewTarget
impl ViewTarget
pub const TEXTURE_FORMAT_HDR: TextureFormat = TextureFormat::Rgba16Float
Sourcepub fn get_color_attachment(&self) -> RenderPassColorAttachment<'_>
pub fn get_color_attachment(&self) -> RenderPassColorAttachment<'_>
Retrieve this target’s main texture’s color attachment.
Sourcepub fn get_unsampled_color_attachment(&self) -> RenderPassColorAttachment<'_>
pub fn get_unsampled_color_attachment(&self) -> RenderPassColorAttachment<'_>
Retrieve this target’s “unsampled” main texture’s color attachment.
Sourcepub fn main_texture(&self) -> &Texture
pub fn main_texture(&self) -> &Texture
The “main” unsampled texture.
Sourcepub fn main_texture_other(&self) -> &Texture
pub fn main_texture_other(&self) -> &Texture
The other “main” unsampled texture.
In most cases you should use Self::main_texture instead and never this.
The textures will naturally be swapped when Self::post_process_write is called.
A use case for this is to be able to prepare a bind group for all main textures ahead of time.
Sourcepub fn main_texture_view(&self) -> &TextureView
pub fn main_texture_view(&self) -> &TextureView
The “main” unsampled texture.
Sourcepub fn main_texture_other_view(&self) -> &TextureView
pub fn main_texture_other_view(&self) -> &TextureView
The other “main” unsampled texture view.
In most cases you should use Self::main_texture_view instead and never this.
The textures will naturally be swapped when Self::post_process_write is called.
A use case for this is to be able to prepare a bind group for all main textures ahead of time.
Sourcepub fn sampled_main_texture(&self) -> Option<&Texture>
pub fn sampled_main_texture(&self) -> Option<&Texture>
The “main” sampled texture.
Sourcepub fn sampled_main_texture_view(&self) -> Option<&TextureView>
pub fn sampled_main_texture_view(&self) -> Option<&TextureView>
The “main” sampled texture view.
pub fn main_texture_format(&self) -> TextureFormat
Sourcepub fn is_hdr(&self) -> bool
pub fn is_hdr(&self) -> bool
Returns true if and only if the main texture is Self::TEXTURE_FORMAT_HDR
Sourcepub fn out_texture(&self) -> &TextureView
pub fn out_texture(&self) -> &TextureView
The final texture this view will render to.
pub fn out_texture_color_attachment( &self, clear_color: Option<LinearRgba>, ) -> RenderPassColorAttachment<'_>
Sourcepub fn needs_present(&self) -> bool
pub fn needs_present(&self) -> bool
Whether the final texture this view will render to needs to be presented.
Sourcepub fn out_texture_view_format(&self) -> TextureFormat
pub fn out_texture_view_format(&self) -> TextureFormat
The format of the final texture this view will render to
Sourcepub fn post_process_write(&self) -> PostProcessWrite<'_>
pub fn post_process_write(&self) -> PostProcessWrite<'_>
This will start a new “post process write”, which assumes that the caller
will write the PostProcessWrite’s source to the destination.
source is the “current” main texture. This will internally flip this
ViewTarget’s main texture to the destination texture, so the caller
must ensure source is copied to destination, with or without modifications.
Failing to do so will cause the current main texture information to be lost.