pub struct RenderVisibleEntities {
pub classes: IndexMap<TypeId, RenderVisibleEntitiesClass, NoOpHash>,
}Expand description
Stores a list of all entities that are visible from a single view or subview, as well as the change lists.
This component is only placed directly on camera entities. Lights instead
have a RenderShadowMapVisibleEntities component that contains one or
more RenderVisibleEntities components, one for each cascade or cubemap
side.
The crate::camera::extract_cameras and extract_lights systems create
this object, but they don’t populate it. Instead, the
collect_visible_cpu_culled_entities and
collect_gpu_culled_meshes systems are responsible for
updating this component from the lists of entities in
RenderExtractedVisibleEntities and RenderGpuCulledEntities,
respectively.
Fields
classes: IndexMap<TypeId, RenderVisibleEntitiesClass, NoOpHash>Entities visible from this view or subview, sorted by
VisibilityClass.
Implementations
Sourceimpl RenderVisibleEntities
impl RenderVisibleEntities
Sourcepub fn get<QF>(&self) -> Option<&RenderVisibleEntitiesClass>where
QF: 'static,
pub fn get<QF>(&self) -> Option<&RenderVisibleEntitiesClass>where
QF: 'static,
Returns the RenderVisibleEntitiesClass corresponding to the given
VisibilityClass.
Examples found in repository?
379pub fn queue_colored_mesh2d(
380 transparent_draw_functions: Res<DrawFunctions<Transparent2d>>,
381 colored_mesh2d_pipeline: Res<ColoredMesh2dPipeline>,
382 mut pipelines: ResMut<SpecializedRenderPipelines<ColoredMesh2dPipeline>>,
383 pipeline_cache: Res<PipelineCache>,
384 render_meshes: Res<RenderAssets<RenderMesh>>,
385 render_mesh_instances: Res<RenderColoredMesh2dInstances>,
386 mut transparent_render_phases: ResMut<ViewSortedRenderPhases<Transparent2d>>,
387 views: Query<(&RenderVisibleEntities, &ExtractedView, &Msaa)>,
388) {
389 if render_mesh_instances.is_empty() {
390 return;
391 }
392 // Iterate each view (a camera is a view)
393 for (visible_entities, view, msaa) in &views {
394 let Some(transparent_phase) = transparent_render_phases.get_mut(&view.retained_view_entity)
395 else {
396 continue;
397 };
398
399 let draw_colored_mesh2d = transparent_draw_functions.read().id::<DrawColoredMesh2d>();
400
401 let mesh_key = Mesh2dPipelineKey::from_msaa_samples(msaa.samples())
402 | Mesh2dPipelineKey::from_target_format(view.target_format);
403
404 // Queue all entities visible to that view
405 let Some(visible_entities) = visible_entities.get::<Mesh2d>() else {
406 continue;
407 };
408 for (render_entity, visible_entity) in visible_entities.iter_visible() {
409 if let Some(mesh_instance) = render_mesh_instances.get(visible_entity) {
410 let mesh2d_handle = mesh_instance.mesh_asset_id;
411 let mesh2d_transforms = &mesh_instance.transforms;
412 // Get our specialized pipeline
413 let mut mesh2d_key = mesh_key;
414 let Some(mesh) = render_meshes.get(mesh2d_handle) else {
415 continue;
416 };
417 mesh2d_key |= Mesh2dPipelineKey::from_primitive_topology_and_strip_index(
418 mesh.primitive_topology(),
419 mesh.index_format(),
420 );
421
422 let pipeline_id =
423 pipelines.specialize(&pipeline_cache, &colored_mesh2d_pipeline, mesh2d_key);
424
425 let mesh_z = mesh2d_transforms.world_from_local.translation.z;
426 transparent_phase.add_retained(Transparent2d {
427 entity: (*render_entity, *visible_entity),
428 draw_function: draw_colored_mesh2d,
429 pipeline: pipeline_id,
430 // The 2d render items are sorted according to their z value before rendering,
431 // in order to get correct transparency
432 sort_key: FloatOrd(mesh_z),
433 // This material is not batched
434 batch_range: 0..1,
435 extra_index: PhaseItemExtraIndex::None,
436 extracted_index: usize::MAX,
437 indexed: mesh.indexed(),
438 });
439 }
440 }
441 }
442}More examples
227fn queue_custom_phase_item(
228 pipeline_cache: Res<PipelineCache>,
229 mut pipeline: ResMut<CustomPhasePipeline>,
230 mut opaque_render_phases: ResMut<ViewBinnedRenderPhases<Opaque3d>>,
231 opaque_draw_functions: Res<DrawFunctions<Opaque3d>>,
232 views: Query<(&ExtractedView, &RenderVisibleEntities, &Msaa)>,
233 dirty_specializations: Res<DirtySpecializations>,
234 mut pending_custom_phase_item_queues: ResMut<PendingCustomPhaseItemQueues>,
235) {
236 let draw_custom_phase_item = opaque_draw_functions
237 .read()
238 .id::<DrawCustomPhaseItemCommands>();
239
240 // Render phases are per-view, so we need to iterate over all views so that
241 // the entity appears in them. (In this example, we have only one view, but
242 // it's good practice to loop over all views anyway.)
243 for (view, view_visible_entities, msaa) in views.iter() {
244 let Some(opaque_phase) = opaque_render_phases.get_mut(&view.retained_view_entity) else {
245 continue;
246 };
247
248 // Fetch the list of visible entities in the `CustomRenderedEntity`
249 // class. If there are no such entities, then we have no entities to
250 // render, and we're done.
251 let Some(render_visible_mesh_entities) =
252 view_visible_entities.get::<CustomRenderedEntity>()
253 else {
254 continue;
255 };
256
257 let view_pending_custom_phase_item_queues =
258 pending_custom_phase_item_queues.prepare_for_new_frame(view.retained_view_entity);
259
260 // First, remove meshes that need to be respecialized, and those that
261 // were removed, from the bins.
262 for &main_entity in dirty_specializations
263 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
264 {
265 opaque_phase.remove(main_entity);
266 }
267
268 // Find all the custom rendered entities that are visible from this
269 // view.
270 for (render_entity, main_entity) in dirty_specializations.iter_to_queue(
271 view.retained_view_entity,
272 render_visible_mesh_entities,
273 &view_pending_custom_phase_item_queues.prev_frame,
274 ) {
275 // Ordinarily, the [`SpecializedRenderPipeline::Key`] would contain
276 // some per-view settings, such as whether the view is HDR, but for
277 // simplicity's sake we simply hard-code the view's characteristics,
278 // with the exception of number of MSAA samples.
279 let Ok(pipeline_id) = pipeline
280 .variants
281 .specialize(&pipeline_cache, CustomPhaseKey(*msaa))
282 else {
283 continue;
284 };
285
286 // Add the custom render item. We use the
287 // [`BinnedRenderPhaseType::NonMesh`] type to skip the special
288 // handling that Bevy has for meshes (preprocessing, indirect
289 // draws, etc.)
290 //
291 // The asset ID is arbitrary; we simply use [`AssetId::invalid`],
292 // but you can use anything you like. Note that the asset ID need
293 // not be the ID of a [`Mesh`].
294 opaque_phase.add(
295 Opaque3dBatchSetKey {
296 draw_function: draw_custom_phase_item,
297 pipeline: pipeline_id,
298 material_bind_group_index: None,
299 lightmap_slab: None,
300 slabs: MeshSlabs::default(),
301 },
302 Opaque3dBinKey {
303 asset_id: AssetId::<Mesh>::invalid().untyped(),
304 },
305 (*render_entity, *main_entity),
306 InputUniformIndex::default(),
307 BinnedRenderPhaseType::NonMesh,
308 );
309 }
310 }
311}531fn queue_custom_meshes(
532 custom_draw_functions: Res<DrawFunctions<Stencil3d>>,
533 mut pipelines: ResMut<SpecializedMeshPipelines<StencilPipeline>>,
534 pipeline_cache: Res<PipelineCache>,
535 custom_draw_pipeline: Res<StencilPipeline>,
536 render_meshes: Res<RenderAssets<RenderMesh>>,
537 render_mesh_instances: Res<RenderMeshInstances>,
538 maybe_batched_instance_buffers: Option<
539 Res<BatchedInstanceBuffers<MeshUniform, MeshInputUniform>>,
540 >,
541 mut custom_render_phases: ResMut<ViewSortedRenderPhases<Stencil3d>>,
542 mut views: Query<(&ExtractedView, &RenderVisibleEntities)>,
543 view_key_cache: Res<ViewKeyCache>,
544 dirty_specializations: Res<DirtySpecializations>,
545 mut pending_custom_mesh_queues: ResMut<PendingCustomMeshQueues>,
546 has_marker: Query<(), With<DrawStencil>>,
547) {
548 for (view, visible_entities) in &mut views {
549 let Some(custom_phase) = custom_render_phases.get_mut(&view.retained_view_entity) else {
550 continue;
551 };
552 let draw_custom = custom_draw_functions.read().id::<DrawMesh3dStencil>();
553
554 let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
555 continue;
556 };
557
558 // Since our phase can work on any 3d mesh we can reuse the default mesh 3d filter
559 let Some(render_visible_mesh_entities) = visible_entities.get::<Mesh3d>() else {
560 continue;
561 };
562
563 let view_pending_custom_mesh_queues =
564 pending_custom_mesh_queues.prepare_for_new_frame(view.retained_view_entity);
565
566 // First, remove meshes that need to be respecialized, and those that were removed, from the bins.
567 for &main_entity in dirty_specializations
568 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
569 {
570 custom_phase.remove(Entity::PLACEHOLDER, main_entity);
571 }
572
573 for (render_entity, visible_entity) in dirty_specializations.iter_to_queue(
574 view.retained_view_entity,
575 render_visible_mesh_entities,
576 &view_pending_custom_mesh_queues.prev_frame,
577 ) {
578 // We only want meshes with the marker component to be queued to our phase.
579 if has_marker.get(*render_entity).is_err() {
580 continue;
581 }
582 let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*visible_entity)
583 else {
584 // We couldn't fetch the mesh, probably because it hasn't been
585 // loaded yet. Add the entity to the list of pending custom mesh
586 // queues and bail.
587 view_pending_custom_mesh_queues
588 .current_frame
589 .insert((*render_entity, *visible_entity));
590 continue;
591 };
592 let Some(mesh) = render_meshes.get(mesh_instance.mesh_asset_id()) else {
593 continue;
594 };
595
596 // Specialize the key for the current mesh entity
597 // For this example we only specialize based on the mesh topology
598 // but you could have more complex keys and that's where you'd need to create those keys
599 let mut mesh_key = view_key;
600 mesh_key |= MeshPipelineKey::from_primitive_topology_and_strip_index(
601 mesh.primitive_topology(),
602 mesh.index_format(),
603 );
604
605 let pipeline_id = pipelines.specialize(
606 &pipeline_cache,
607 &custom_draw_pipeline,
608 mesh_key,
609 &mesh.layout,
610 );
611 let pipeline_id = match pipeline_id {
612 Ok(id) => id,
613 Err(err) => {
614 error!("{}", err);
615 continue;
616 }
617 };
618 // At this point we have all the data we need to create a phase item and add it to our
619 // phase
620 custom_phase.add_retained(Stencil3d {
621 sorting_info: TransparentSortingInfo3d::Sorted {
622 mesh_center: pbr::get_mesh_instance_world_from_local(
623 *visible_entity,
624 mesh_instance.current_uniform_index,
625 &render_mesh_instances,
626 maybe_batched_instance_buffers.as_deref(),
627 )
628 .transform_point3(
629 render_meshes
630 .get(mesh_instance.mesh_asset_id())
631 .unwrap()
632 .aabb_center,
633 ),
634 depth_bias: 0.0,
635 },
636 distance: FloatOrd(0.0),
637 entity: (Entity::PLACEHOLDER, *visible_entity),
638 pipeline: pipeline_id,
639 draw_function: draw_custom,
640 // Sorted phase items aren't batched
641 batch_range: 0..1,
642 extra_index: PhaseItemExtraIndex::None,
643 indexed: mesh.indexed(),
644 });
645 }
646 }
647}277fn queue_custom_mesh_pipeline(
278 pipeline_cache: Res<PipelineCache>,
279 custom_mesh_pipeline: Res<CustomMeshPipeline>,
280 (mut opaque_render_phases, opaque_draw_functions): (
281 ResMut<ViewBinnedRenderPhases<Opaque3d>>,
282 Res<DrawFunctions<Opaque3d>>,
283 ),
284 mut specialized_mesh_pipelines: ResMut<SpecializedMeshPipelines<CustomMeshPipeline>>,
285 views: Query<(&RenderVisibleEntities, &ExtractedView)>,
286 view_key_cache: Res<ViewKeyCache>,
287 (render_meshes, render_mesh_instances): (
288 Res<RenderAssets<RenderMesh>>,
289 Res<RenderMeshInstances>,
290 ),
291 mut change_tick: Local<Tick>,
292 mesh_allocator: Res<MeshAllocator>,
293 gpu_preprocessing_support: Res<GpuPreprocessingSupport>,
294 dirty_specializations: Res<DirtySpecializations>,
295 mut pending_custom_mesh_queues: ResMut<PendingCustomMeshQueues>,
296) {
297 // Get the id for our custom draw function
298 let draw_function = opaque_draw_functions
299 .read()
300 .id::<DrawSpecializedPipelineCommands>();
301
302 // Render phases are per-view, so we need to iterate over all views so that
303 // the entity appears in them. (In this example, we have only one view, but
304 // it's good practice to loop over all views anyway.)
305 for (view_visible_entities, view) in views.iter() {
306 let Some(opaque_phase) = opaque_render_phases.get_mut(&view.retained_view_entity) else {
307 continue;
308 };
309
310 let Some(&view_key) = view_key_cache.get(&view.retained_view_entity) else {
311 continue;
312 };
313
314 let Some(render_visible_mesh_entities) =
315 view_visible_entities.get::<CustomRenderedEntity>()
316 else {
317 continue;
318 };
319
320 // Initialize the pending queues.
321 let view_pending_custom_mesh_queues =
322 pending_custom_mesh_queues.prepare_for_new_frame(view.retained_view_entity);
323
324 // First, remove meshes that need to be respecialized, and those that were removed, from the bins.
325 for &main_entity in dirty_specializations
326 .iter_to_dequeue(view.retained_view_entity, render_visible_mesh_entities)
327 {
328 opaque_phase.remove(main_entity);
329 }
330
331 // Find all the custom rendered entities that are visible from this
332 // view.
333 for (render_entity, visible_entity) in dirty_specializations.iter_to_queue(
334 view.retained_view_entity,
335 render_visible_mesh_entities,
336 &view_pending_custom_mesh_queues.prev_frame,
337 ) {
338 // Get the mesh instance
339 let Some(mesh_instance) = render_mesh_instances.render_mesh_queue_data(*visible_entity)
340 else {
341 // We couldn't fetch the mesh, probably because it hasn't been
342 // loaded yet. Add the entity to the list of pending custom
343 // meshes and bail.
344 view_pending_custom_mesh_queues
345 .current_frame
346 .insert((*render_entity, *visible_entity));
347 continue;
348 };
349
350 // Get the mesh data
351 let Some(mesh) = render_meshes.get(mesh_instance.mesh_asset_id()) else {
352 continue;
353 };
354
355 let Some(mesh_slabs) = mesh_allocator.mesh_slabs(&mesh_instance.mesh_asset_id()) else {
356 continue;
357 };
358
359 // Specialize the key for the current mesh entity
360 // For this example we only specialize based on the mesh topology
361 // but you could have more complex keys and that's where you'd need to create those keys
362 let mut mesh_key = view_key;
363 mesh_key |= MeshPipelineKey::from_primitive_topology_and_strip_index(
364 mesh.primitive_topology(),
365 mesh.index_format(),
366 );
367
368 // Finally, we can specialize the pipeline based on the key
369 let pipeline_id = specialized_mesh_pipelines
370 .specialize(
371 &pipeline_cache,
372 &custom_mesh_pipeline,
373 mesh_key,
374 &mesh.layout,
375 )
376 // This should never happen with this example, but if your pipeline
377 // specialization can fail you need to handle the error here
378 .expect("Failed to specialize mesh pipeline");
379
380 // Bump the change tick so that Bevy is forced to rebuild the bin.
381 let next_change_tick = change_tick.get() + 1;
382 change_tick.set(next_change_tick);
383
384 // Add the mesh with our specialized pipeline
385 opaque_phase.add(
386 Opaque3dBatchSetKey {
387 draw_function,
388 pipeline: pipeline_id,
389 material_bind_group_index: None,
390 slabs: mesh_slabs,
391 lightmap_slab: None,
392 },
393 // For this example we can use the mesh asset id as the bin key,
394 // but you can use any asset_id as a key
395 Opaque3dBinKey {
396 asset_id: mesh_instance.mesh_asset_id().into(),
397 },
398 (*render_entity, *visible_entity),
399 mesh_instance.current_uniform_index,
400 // This example supports batching and multi draw indirect,
401 // but if your pipeline doesn't support it you can use
402 // `BinnedRenderPhaseType::UnbatchableMesh`
403 BinnedRenderPhaseType::mesh(
404 mesh_instance.should_batch(),
405 &gpu_preprocessing_support,
406 ),
407 );
408 }
409 }
410}Trait Implementations
Sourceimpl Clone for RenderVisibleEntities
impl Clone for RenderVisibleEntities
Sourcefn clone(&self) -> RenderVisibleEntities
fn clone(&self) -> RenderVisibleEntities
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Component for RenderVisibleEntities
impl Component for RenderVisibleEntities
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn relationship_accessor() -> Option<ComponentRelationshipAccessor<RenderVisibleEntities>>
fn relationship_accessor() -> Option<ComponentRelationshipAccessor<RenderVisibleEntities>>
ComponentRelationshipAccessor required for working with relationships in dynamic contexts. Read moreSourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_discard() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceimpl Debug for RenderVisibleEntities
impl Debug for RenderVisibleEntities
Sourceimpl Default for RenderVisibleEntities
impl Default for RenderVisibleEntities
Sourcefn default() -> RenderVisibleEntities
fn default() -> RenderVisibleEntities