pub struct Cylinder {
pub radius: f32,
pub half_height: f32,
}Expand description
A cylinder primitive centered on the origin
Fields
radius: f32The radius of the cylinder
half_height: f32The half height of the cylinder
Implementations
Sourceimpl Cylinder
impl Cylinder
Sourcepub const fn new(radius: f32, height: f32) -> Cylinder
pub const fn new(radius: f32, height: f32) -> Cylinder
Create a new Cylinder from a radius and full height
Examples found in repository?
examples/3d/transmission.rs (line 71)
62fn setup(
63 mut commands: Commands,
64 mut meshes: ResMut<Assets<Mesh>>,
65 mut materials: ResMut<Assets<StandardMaterial>>,
66 asset_server: Res<AssetServer>,
67) {
68 let icosphere_mesh = meshes.add(Sphere::new(0.9).mesh().ico(7).unwrap());
69 let cube_mesh = meshes.add(Cuboid::new(0.7, 0.7, 0.7));
70 let plane_mesh = meshes.add(Plane3d::default().mesh().size(2.0, 2.0));
71 let cylinder_mesh = meshes.add(Cylinder::new(0.5, 2.0).mesh().resolution(50));
72
73 // Cube #1
74 commands.spawn((
75 Mesh3d(cube_mesh.clone()),
76 MeshMaterial3d(materials.add(StandardMaterial::default())),
77 Transform::from_xyz(0.25, 0.5, -2.0).with_rotation(Quat::from_euler(
78 EulerRot::XYZ,
79 1.4,
80 3.7,
81 21.3,
82 )),
83 ExampleControls {
84 color: true,
85 specular_transmission: false,
86 diffuse_transmission: false,
87 },
88 ));
89
90 // Cube #2
91 commands.spawn((
92 Mesh3d(cube_mesh),
93 MeshMaterial3d(materials.add(StandardMaterial::default())),
94 Transform::from_xyz(-0.75, 0.7, -2.0).with_rotation(Quat::from_euler(
95 EulerRot::XYZ,
96 0.4,
97 2.3,
98 4.7,
99 )),
100 ExampleControls {
101 color: true,
102 specular_transmission: false,
103 diffuse_transmission: false,
104 },
105 ));
106
107 // Candle
108 commands.spawn((
109 Mesh3d(cylinder_mesh),
110 MeshMaterial3d(materials.add(StandardMaterial {
111 base_color: Color::srgb(0.9, 0.2, 0.3),
112 diffuse_transmission: 0.7,
113 perceptual_roughness: 0.32,
114 thickness: 0.2,
115 ..default()
116 })),
117 Transform::from_xyz(-1.0, 0.0, 0.0),
118 ExampleControls {
119 color: true,
120 specular_transmission: false,
121 diffuse_transmission: true,
122 },
123 ));
124
125 // Candle Flame
126 let scaled_white = LinearRgba::from(ANTIQUE_WHITE) * 20.;
127 let scaled_orange = LinearRgba::from(ORANGE_RED) * 4.;
128 let emissive = LinearRgba {
129 red: scaled_white.red + scaled_orange.red,
130 green: scaled_white.green + scaled_orange.green,
131 blue: scaled_white.blue + scaled_orange.blue,
132 alpha: 1.0,
133 };
134
135 commands.spawn((
136 Mesh3d(icosphere_mesh.clone()),
137 MeshMaterial3d(materials.add(StandardMaterial {
138 emissive,
139 diffuse_transmission: 1.0,
140 ..default()
141 })),
142 Transform::from_xyz(-1.0, 1.15, 0.0).with_scale(Vec3::new(0.1, 0.2, 0.1)),
143 Flicker,
144 NotShadowCaster,
145 ));
146
147 // Glass Sphere
148 commands.spawn((
149 Mesh3d(icosphere_mesh.clone()),
150 MeshMaterial3d(materials.add(StandardMaterial {
151 base_color: Color::WHITE,
152 specular_transmission: 0.9,
153 diffuse_transmission: 1.0,
154 thickness: 1.8,
155 ior: 1.5,
156 perceptual_roughness: 0.12,
157 ..default()
158 })),
159 Transform::from_xyz(1.0, 0.0, 0.0),
160 ExampleControls {
161 color: true,
162 specular_transmission: true,
163 diffuse_transmission: false,
164 },
165 ));
166
167 // R Sphere
168 commands.spawn((
169 Mesh3d(icosphere_mesh.clone()),
170 MeshMaterial3d(materials.add(StandardMaterial {
171 base_color: RED.into(),
172 specular_transmission: 0.9,
173 diffuse_transmission: 1.0,
174 thickness: 1.8,
175 ior: 1.5,
176 perceptual_roughness: 0.12,
177 ..default()
178 })),
179 Transform::from_xyz(1.0, -0.5, 2.0).with_scale(Vec3::splat(0.5)),
180 ExampleControls {
181 color: true,
182 specular_transmission: true,
183 diffuse_transmission: false,
184 },
185 ));
186
187 // G Sphere
188 commands.spawn((
189 Mesh3d(icosphere_mesh.clone()),
190 MeshMaterial3d(materials.add(StandardMaterial {
191 base_color: LIME.into(),
192 specular_transmission: 0.9,
193 diffuse_transmission: 1.0,
194 thickness: 1.8,
195 ior: 1.5,
196 perceptual_roughness: 0.12,
197 ..default()
198 })),
199 Transform::from_xyz(0.0, -0.5, 2.0).with_scale(Vec3::splat(0.5)),
200 ExampleControls {
201 color: true,
202 specular_transmission: true,
203 diffuse_transmission: false,
204 },
205 ));
206
207 // B Sphere
208 commands.spawn((
209 Mesh3d(icosphere_mesh),
210 MeshMaterial3d(materials.add(StandardMaterial {
211 base_color: BLUE.into(),
212 specular_transmission: 0.9,
213 diffuse_transmission: 1.0,
214 thickness: 1.8,
215 ior: 1.5,
216 perceptual_roughness: 0.12,
217 ..default()
218 })),
219 Transform::from_xyz(-1.0, -0.5, 2.0).with_scale(Vec3::splat(0.5)),
220 ExampleControls {
221 color: true,
222 specular_transmission: true,
223 diffuse_transmission: false,
224 },
225 ));
226
227 // Chessboard Plane
228 let black_material = materials.add(StandardMaterial {
229 base_color: Color::BLACK,
230 reflectance: 0.3,
231 perceptual_roughness: 0.8,
232 ..default()
233 });
234
235 let white_material = materials.add(StandardMaterial {
236 base_color: Color::WHITE,
237 reflectance: 0.3,
238 perceptual_roughness: 0.8,
239 ..default()
240 });
241
242 for x in -3..4 {
243 for z in -3..4 {
244 commands.spawn((
245 Mesh3d(plane_mesh.clone()),
246 MeshMaterial3d(if (x + z) % 2 == 0 {
247 black_material.clone()
248 } else {
249 white_material.clone()
250 }),
251 Transform::from_xyz(x as f32 * 2.0, -1.0, z as f32 * 2.0),
252 ExampleControls {
253 color: true,
254 specular_transmission: false,
255 diffuse_transmission: false,
256 },
257 ));
258 }
259 }
260
261 // Paper
262 commands.spawn((
263 Mesh3d(plane_mesh),
264 MeshMaterial3d(materials.add(StandardMaterial {
265 base_color: Color::WHITE,
266 diffuse_transmission: 0.6,
267 perceptual_roughness: 0.8,
268 reflectance: 1.0,
269 double_sided: true,
270 cull_mode: None,
271 ..default()
272 })),
273 Transform::from_xyz(0.0, 0.5, -3.0)
274 .with_scale(Vec3::new(2.0, 1.0, 1.0))
275 .with_rotation(Quat::from_euler(EulerRot::XYZ, PI / 2.0, 0.0, 0.0)),
276 TransmittedShadowReceiver,
277 ExampleControls {
278 specular_transmission: false,
279 color: false,
280 diffuse_transmission: true,
281 },
282 ));
283
284 // Candle Light
285 commands.spawn((
286 Transform::from_xyz(-1.0, 1.7, 0.0),
287 PointLight {
288 color: Color::from(
289 LinearRgba::from(ANTIQUE_WHITE).mix(&LinearRgba::from(ORANGE_RED), 0.2),
290 ),
291 intensity: 4_000.0,
292 radius: 0.2,
293 range: 5.0,
294 shadow_maps_enabled: true,
295 ..default()
296 },
297 Flicker,
298 ));
299
300 // Camera
301 commands.spawn((
302 Camera3d::default(),
303 ScreenSpaceTransmission::default(),
304 Transform::from_xyz(1.0, 1.8, 7.0).looking_at(Vec3::ZERO, Vec3::Y),
305 ColorGrading {
306 global: ColorGradingGlobal {
307 post_saturation: 1.2,
308 ..default()
309 },
310 ..default()
311 },
312 Tonemapping::TonyMcMapface,
313 Exposure { ev100: 6.0 },
314 #[cfg(any(feature = "webgpu", not(target_arch = "wasm32")))]
315 Msaa::Off,
316 #[cfg(any(feature = "webgpu", not(target_arch = "wasm32")))]
317 TemporalAntiAliasing::default(),
318 EnvironmentMapLight {
319 intensity: 25.0,
320 diffuse_map: asset_server.load("environment_maps/pisa_diffuse_rgb9e5_zstd.ktx2"),
321 specular_map: asset_server.load("environment_maps/pisa_specular_rgb9e5_zstd.ktx2"),
322 ..default()
323 },
324 Bloom::default(),
325 ));
326
327 // Controls Text
328 commands.spawn((
329 Text::default(),
330 Node {
331 position_type: PositionType::Absolute,
332 top: px(12),
333 left: px(12),
334 ..default()
335 },
336 ExampleDisplay,
337 ));
338}Sourcepub const fn lateral_area(&self) -> f32
pub const fn lateral_area(&self) -> f32
Get the surface area of the side of the cylinder, also known as the lateral area