Pools for async, IO, and compute tasks.
Modules
Structs
- A newtype for a task pool for CPU-intensive work that may span across multiple frames
- A newtype for a task pool for CPU-intensive work that must be completed to deliver the next frame
- A newtype for a task pool for IO-intensive work (i.e. tasks that spend very little time in a “woken” state)
- A
TaskPoolscope for running one or more non-'staticfutures. - Wraps
async_executor::Task, a spawned future. - A thread pool for executing tasks. Tasks are futures that are being automatically driven by the pool on threads owned by the pool.
- Used to create a
TaskPool - An executor that can only be ticked on the thread it was instantiated on. But can spawn
Sendtasks from other threads. - Used to tick the
ThreadExecutor. The executor does not make progress unless it is manually ticked on the thread it was created on.
Traits
ParallelIteratorclosely emulates thestd::iter::Iteratorinterface. However, it usesbevy_taskto compute batches in parallel.- Provides functions for mapping read-only slices across a provided
TaskPool. - Provides functions for mapping mutable slices across a provided
TaskPool.
Functions
- Gets the logical CPU core count available to the current process.
- A function used by
bevy_coreto tick the global tasks pools on the main thread. This will run a maximum of 100 local tasks per executor per call to this function.