pub struct GlobalTransform(/* private fields */);Expand description
GlobalTransform is an affine transformation from entity-local coordinates to worldspace coordinates.
You cannot directly mutate GlobalTransform; instead, you change an entity’s transform by manipulating
its Transform, which indirectly causes Bevy to update its GlobalTransform.
- To get the global transform of an entity, you should get its
GlobalTransform. - For transform hierarchies to work correctly, you must have both a
Transformand aGlobalTransform.GlobalTransformis automatically inserted wheneverTransformis inserted.
Transform and GlobalTransform
Transform transforms an entity relative to its parent’s reference frame, or relative to world space coordinates,
if it doesn’t have a ChildOf component.
GlobalTransform is managed by Bevy; it is computed by successively applying the Transform of each ancestor
entity which has a Transform. This is done automatically by Bevy-internal systems in the TransformSystems::Propagate
system set.
This system runs during PostUpdate. If you
update the Transform of an entity in this schedule or after, you will notice a 1 frame lag
before the GlobalTransform is updated.
Examples
Implementations
Sourceimpl GlobalTransform
impl GlobalTransform
Sourcepub const IDENTITY: GlobalTransform
pub const IDENTITY: GlobalTransform
An identity GlobalTransform that maps all points in space to themselves.
Sourcepub fn affine(&self) -> Affine3A
pub fn affine(&self) -> Affine3A
Returns the 3d affine transformation matrix as an Affine3A.
Examples found in repository?
208fn try_plot(
209 event: On<TryPlot>,
210 sprite: Query<(&Sprite, &Anchor, &GlobalTransform), With<SpriteToSave>>,
211 camera: Single<(&Camera, &GlobalTransform)>,
212 texture_atlases: Res<Assets<TextureAtlasLayout>>,
213 draw_color: Res<DrawColor>,
214 mut images: ResMut<Assets<Image>>,
215) {
216 let Ok((sprite, anchor, sprite_transform)) = sprite.get(event.entity) else {
217 return;
218 };
219 let (camera, camera_transform) = camera.into_inner();
220 let Ok(world_position) = camera.viewport_to_world_2d(camera_transform, event.location.position)
221 else {
222 return;
223 };
224 let relative_to_sprite = sprite_transform
225 .affine()
226 .inverse()
227 .transform_point3(world_position.extend(0.0));
228 let Ok(pixel_space) = sprite.compute_pixel_space_point(
229 relative_to_sprite.xy(),
230 *anchor,
231 &images,
232 &texture_atlases,
233 ) else {
234 return;
235 };
236 let pixel_coordinates = pixel_space.floor().as_uvec2();
237 let mut image = images.get_mut(&sprite.image).unwrap();
238 // For an actual drawing app, you'd at least draw a line from the last point, but this is
239 // simpler.
240 image
241 .set_color_at(pixel_coordinates.x, pixel_coordinates.y, draw_color.0)
242 .unwrap();
243}More examples
332pub fn extract_colored_mesh2d(
333 mut commands: Commands,
334 mut previous_len: Local<usize>,
335 // When extracting, you must use `Extract` to mark the `SystemParam`s
336 // which should be taken from the main world.
337 query: Extract<
338 Query<
339 (
340 Entity,
341 RenderEntity,
342 &ViewVisibility,
343 &GlobalTransform,
344 &Mesh2d,
345 ),
346 With<ColoredMesh2d>,
347 >,
348 >,
349 mut render_mesh_instances: ResMut<RenderColoredMesh2dInstances>,
350) {
351 let mut values = Vec::with_capacity(*previous_len);
352 for (entity, render_entity, view_visibility, transform, handle) in &query {
353 if !view_visibility.get() {
354 continue;
355 }
356
357 let transforms = Mesh2dTransforms {
358 world_from_local: transform.affine().into(),
359 flags: MeshFlags::empty().bits(),
360 };
361
362 values.push((render_entity, ColoredMesh2d));
363 render_mesh_instances.insert(
364 entity.into(),
365 RenderMesh2dInstance {
366 mesh_asset_id: handle.0.id(),
367 transforms,
368 material_bind_group_id: Material2dBindGroupId::default(),
369 automatic_batching: false,
370 tag: 0,
371 },
372 );
373 }
374 *previous_len = values.len();
375 commands.try_insert_batch(values);
376}Sourcepub fn compute_transform(&self) -> Transform
pub fn compute_transform(&self) -> Transform
Returns the transformation as a Transform.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Sourcepub fn to_isometry(&self) -> Isometry3d
pub fn to_isometry(&self) -> Isometry3d
Computes a Scale-Rotation-Translation decomposition of the transformation and returns the isometric part as an isometry. Any scaling done by the transformation will be ignored. Note: this is a somewhat costly and lossy conversion.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Sourcepub fn reparented_to(&self, parent: &GlobalTransform) -> Transform
pub fn reparented_to(&self, parent: &GlobalTransform) -> Transform
Returns the Transform self would have if it was a child of an entity
with the parent GlobalTransform.
This is useful if you want to “reparent” an Entity.
Say you have an entity e1 that you want to turn into a child of e2,
but you want e1 to keep the same global transform, even after re-parenting. You would use:
#[derive(Component)]
struct ToReparent {
new_parent: Entity,
}
fn reparent_system(
mut commands: Commands,
mut targets: Query<(&mut Transform, Entity, &GlobalTransform, &ToReparent)>,
transforms: Query<&GlobalTransform>,
) {
for (mut transform, entity, initial, to_reparent) in targets.iter_mut() {
if let Ok(parent_transform) = transforms.get(to_reparent.new_parent) {
*transform = initial.reparented_to(parent_transform);
commands.entity(entity)
.remove::<ToReparent>()
.insert(ChildOf(to_reparent.new_parent));
}
}
}The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Sourcepub fn to_scale_rotation_translation(&self) -> (Vec3, Quat, Vec3)
pub fn to_scale_rotation_translation(&self) -> (Vec3, Quat, Vec3)
Extracts scale, rotation and translation from self.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Examples found in repository?
104fn draw_bounds<Shape: Bounded2d + Send + Sync + 'static>(
105 q: Query<(&DrawBounds<Shape>, &GlobalTransform)>,
106 mut gizmos: Gizmos,
107) {
108 for (shape, transform) in &q {
109 let (_, rotation, translation) = transform.to_scale_rotation_translation();
110 let translation = translation.truncate();
111 let rotation = rotation.to_euler(EulerRot::XYZ).2;
112 let isometry = Isometry2d::new(translation, Rot2::radians(rotation));
113
114 let aabb = shape.0.aabb_2d(isometry);
115 gizmos.rect_2d(aabb.center(), aabb.half_size() * 2.0, RED);
116
117 let bounding_circle = shape.0.bounding_circle(isometry);
118 gizmos.circle_2d(bounding_circle.center, bounding_circle.radius(), BLUE);
119 }
120}Sourcepub fn up(&self) -> Dir3
pub fn up(&self) -> Dir3
Return the local up vector (Y).
Examples found in repository?
13fn draw_cursor(
14 camera_query: Single<(&Camera, &GlobalTransform)>,
15 ground: Single<&GlobalTransform, With<Ground>>,
16 window: Single<&Window>,
17 mut gizmos: Gizmos,
18) {
19 let (camera, camera_transform) = *camera_query;
20
21 if let Some(cursor_position) = window.cursor_position()
22 // Calculate a ray pointing from the camera into the world based on the cursor's position.
23 && let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
24 // Calculate if and where the ray is hitting the ground plane.
25 && let Some(point) = ray.plane_intersection_point(ground.translation(), InfinitePlane3d::new(ground.up()))
26 {
27 // Draw a circle just above the ground plane at that position.
28 gizmos.circle(
29 Isometry3d::new(
30 point + ground.up() * 0.01,
31 Quat::from_rotation_arc(Vec3::Z, ground.up().as_vec3()),
32 ),
33 0.2,
34 Color::WHITE,
35 );
36 }
37}Sourcepub fn translation(&self) -> Vec3
pub fn translation(&self) -> Vec3
Get the translation as a Vec3.
Examples found in repository?
122fn interact_bodies(mut query: Query<(&Mass, &GlobalTransform, &mut Acceleration)>) {
123 let mut iter = query.iter_combinations_mut();
124 while let Some([(Mass(m1), transform1, mut acc1), (Mass(m2), transform2, mut acc2)]) =
125 iter.fetch_next()
126 {
127 let delta = transform2.translation() - transform1.translation();
128 let distance_sq: f32 = delta.length_squared();
129
130 let f = GRAVITY_CONSTANT / distance_sq;
131 let force_unit_mass = delta * f;
132 acc1.0 += force_unit_mass * *m2;
133 acc2.0 -= force_unit_mass * *m1;
134 }
135}More examples
346fn draw_camera_gizmo(cameras: Query<(&Camera, &GlobalTransform)>, mut gizmos: Gizmos) {
347 for (camera, transform) in &cameras {
348 // As above, we use the order as a cheap tag to tell the depth texture
349 // apart from the main texture.
350 if camera.order >= 0 {
351 continue;
352 }
353
354 // Draw a cone representing the camera.
355 gizmos.primitive_3d(
356 &Cone {
357 radius: 1.0,
358 height: 3.0,
359 },
360 Isometry3d::new(
361 transform.translation(),
362 // We have to rotate here because `Cone` primitives are oriented
363 // along +Y and cameras point along +Z.
364 transform.rotation() * Quat::from_rotation_x(FRAC_PI_2),
365 ),
366 LIME,
367 );
368 }
369}13fn draw_cursor(
14 camera_query: Single<(&Camera, &GlobalTransform)>,
15 ground: Single<&GlobalTransform, With<Ground>>,
16 window: Single<&Window>,
17 mut gizmos: Gizmos,
18) {
19 let (camera, camera_transform) = *camera_query;
20
21 if let Some(cursor_position) = window.cursor_position()
22 // Calculate a ray pointing from the camera into the world based on the cursor's position.
23 && let Ok(ray) = camera.viewport_to_world(camera_transform, cursor_position)
24 // Calculate if and where the ray is hitting the ground plane.
25 && let Some(point) = ray.plane_intersection_point(ground.translation(), InfinitePlane3d::new(ground.up()))
26 {
27 // Draw a circle just above the ground plane at that position.
28 gizmos.circle(
29 Isometry3d::new(
30 point + ground.up() * 0.01,
31 Quat::from_rotation_arc(Vec3::Z, ground.up().as_vec3()),
32 ),
33 0.2,
34 Color::WHITE,
35 );
36 }
37}45fn observe_on_step(
46 step: On<Step>,
47 particle: Res<ParticleAssets>,
48 mut commands: Commands,
49 transforms: Query<&GlobalTransform>,
50 mut seeded_rng: ResMut<SeededRng>,
51) -> Result {
52 let translation = transforms.get(step.trigger().target)?.translation();
53 // Spawn a bunch of particles.
54 for _ in 0..14 {
55 let horizontal = seeded_rng.0.random::<Dir2>() * seeded_rng.0.random_range(8.0..12.0);
56 let vertical = seeded_rng.0.random_range(0.0..4.0);
57 let size = seeded_rng.0.random_range(0.2..1.0);
58
59 commands.spawn((
60 Particle {
61 lifetime_timer: Timer::from_seconds(
62 seeded_rng.0.random_range(0.2..0.6),
63 TimerMode::Once,
64 ),
65 size,
66 velocity: Vec3::new(horizontal.x, vertical, horizontal.y) * 10.0,
67 },
68 Mesh3d(particle.mesh.clone()),
69 MeshMaterial3d(particle.material.clone()),
70 Transform {
71 translation,
72 scale: Vec3::splat(size),
73 ..Default::default()
74 },
75 ));
76 }
77 Ok(())
78}330fn observe_on_step(
331 step: On<Step>,
332 particle: Res<ParticleAssets>,
333 mut commands: Commands,
334 transforms: Query<&GlobalTransform>,
335 mut seeded_rng: ResMut<SeededRng>,
336) -> Result {
337 let translation = transforms.get(step.trigger().target)?.translation();
338 // Spawn a bunch of particles.
339 for _ in 0..14 {
340 let horizontal = seeded_rng.0.random::<Dir2>() * seeded_rng.0.random_range(8.0..12.0);
341 let vertical = seeded_rng.0.random_range(0.0..4.0);
342 let size = seeded_rng.0.random_range(0.2..1.0);
343
344 commands.spawn((
345 Particle {
346 lifetime_timer: Timer::from_seconds(
347 seeded_rng.0.random_range(0.2..0.6),
348 TimerMode::Once,
349 ),
350 size,
351 velocity: Vec3::new(horizontal.x, vertical, horizontal.y) * 10.0,
352 },
353 Mesh3d(particle.mesh.clone()),
354 MeshMaterial3d(particle.material.clone()),
355 Transform {
356 translation,
357 scale: Vec3::splat(size),
358 ..Default::default()
359 },
360 ));
361 }
362 Ok(())
363}247fn example_control_system(
248 mut materials: ResMut<Assets<StandardMaterial>>,
249 controllable: Query<(&MeshMaterial3d<StandardMaterial>, &ExampleControls)>,
250 camera: Single<
251 (
252 Entity,
253 &mut Camera,
254 &mut Transform,
255 &GlobalTransform,
256 Has<Hdr>,
257 ),
258 With<Camera3d>,
259 >,
260 mut labels: Query<(&mut Node, &ExampleLabel)>,
261 mut display: Single<&mut Text, With<ExampleDisplay>>,
262 labeled: Query<&GlobalTransform>,
263 mut state: Local<ExampleState>,
264 time: Res<Time>,
265 input: Res<ButtonInput<KeyCode>>,
266 mut commands: Commands,
267) {
268 if input.pressed(KeyCode::ArrowUp) {
269 state.alpha = (state.alpha + time.delta_secs()).min(1.0);
270 } else if input.pressed(KeyCode::ArrowDown) {
271 state.alpha = (state.alpha - time.delta_secs()).max(0.0);
272 }
273
274 if input.just_pressed(KeyCode::Space) {
275 state.unlit = !state.unlit;
276 }
277
278 let randomize_colors = input.just_pressed(KeyCode::KeyC);
279
280 for (material_handle, controls) in &controllable {
281 let mut material = materials.get_mut(material_handle).unwrap();
282
283 if controls.color && randomize_colors {
284 material.base_color = Srgba {
285 red: random(),
286 green: random(),
287 blue: random(),
288 alpha: state.alpha,
289 }
290 .into();
291 } else {
292 material.base_color.set_alpha(state.alpha);
293 }
294
295 if controls.unlit {
296 material.unlit = state.unlit;
297 }
298 }
299
300 let (entity, camera, mut camera_transform, camera_global_transform, hdr) = camera.into_inner();
301
302 if input.just_pressed(KeyCode::KeyH) {
303 if hdr {
304 commands.entity(entity).remove::<Hdr>();
305 } else {
306 commands.entity(entity).insert(Hdr);
307 }
308 }
309
310 let rotation = if input.pressed(KeyCode::ArrowLeft) {
311 time.delta_secs()
312 } else if input.pressed(KeyCode::ArrowRight) {
313 -time.delta_secs()
314 } else {
315 0.0
316 };
317
318 camera_transform.rotate_around(Vec3::ZERO, Quat::from_rotation_y(rotation));
319
320 for (mut node, label) in &mut labels {
321 let world_position = labeled.get(label.entity).unwrap().translation() + Vec3::Y;
322
323 let viewport_position = camera
324 .world_to_viewport(camera_global_transform, world_position)
325 .unwrap();
326
327 node.top = px(viewport_position.y);
328 node.left = px(viewport_position.x);
329 }
330
331 display.0 = format!(
332 " HDR: {}\nAlpha: {:.2}",
333 if hdr { "ON " } else { "OFF" },
334 state.alpha
335 );
336}Sourcepub fn translation_vec3a(&self) -> Vec3A
pub fn translation_vec3a(&self) -> Vec3A
Get the translation as a Vec3A.
Examples found in repository?
357fn draw_gizmos(mut gizmos: Gizmos, spotlight: Query<(&GlobalTransform, &SpotLight, &Visibility)>) {
358 if let Ok((global_transform, spotlight, visibility)) = spotlight.single()
359 && visibility != Visibility::Hidden
360 {
361 gizmos.primitive_3d(
362 &Cone::new(7.0 * spotlight.outer_angle, 7.0),
363 Isometry3d {
364 rotation: global_transform.rotation() * Quat::from_rotation_x(FRAC_PI_2),
365 translation: global_transform.translation_vec3a() * 0.5,
366 },
367 YELLOW,
368 );
369 }
370}More examples
263fn draw_gizmos(mut gizmos: Gizmos, decals: Query<&GlobalTransform, With<ClusteredDecal>>) {
264 for global_transform in &decals {
265 gizmos.primitive_3d(
266 &Cuboid {
267 // Since the clustered decal is a 1×1×1 cube in model space, its
268 // half-size is half of the scaling part of its transform.
269 half_size: global_transform.scale() * 0.5,
270 },
271 Isometry3d {
272 rotation: global_transform.rotation(),
273 translation: global_transform.translation_vec3a(),
274 },
275 GOLD,
276 );
277 }
278}311fn draw_gizmos(
312 mut gizmos: Gizmos,
313 decals: Query<(&GlobalTransform, &Selection), With<ClusteredDecal>>,
314) {
315 for (global_transform, selection) in &decals {
316 let color = match *selection {
317 Selection::Camera => continue,
318 Selection::DecalA => ORANGE_RED,
319 Selection::DecalB => LIME,
320 };
321
322 gizmos.primitive_3d(
323 &Cuboid {
324 // Since the clustered decal is a 1×1×1 cube in model space, its
325 // half-size is half of the scaling part of its transform.
326 half_size: global_transform.scale() * 0.5,
327 },
328 Isometry3d {
329 rotation: global_transform.rotation(),
330 translation: global_transform.translation_vec3a(),
331 },
332 color,
333 );
334 }
335}Sourcepub fn rotation(&self) -> Quat
pub fn rotation(&self) -> Quat
Get the rotation as a Quat.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Warning
This is calculated using to_scale_rotation_translation, meaning that you
should probably use it directly if you also need translation or scale.
Examples found in repository?
357fn draw_gizmos(mut gizmos: Gizmos, spotlight: Query<(&GlobalTransform, &SpotLight, &Visibility)>) {
358 if let Ok((global_transform, spotlight, visibility)) = spotlight.single()
359 && visibility != Visibility::Hidden
360 {
361 gizmos.primitive_3d(
362 &Cone::new(7.0 * spotlight.outer_angle, 7.0),
363 Isometry3d {
364 rotation: global_transform.rotation() * Quat::from_rotation_x(FRAC_PI_2),
365 translation: global_transform.translation_vec3a() * 0.5,
366 },
367 YELLOW,
368 );
369 }
370}More examples
263fn draw_gizmos(mut gizmos: Gizmos, decals: Query<&GlobalTransform, With<ClusteredDecal>>) {
264 for global_transform in &decals {
265 gizmos.primitive_3d(
266 &Cuboid {
267 // Since the clustered decal is a 1×1×1 cube in model space, its
268 // half-size is half of the scaling part of its transform.
269 half_size: global_transform.scale() * 0.5,
270 },
271 Isometry3d {
272 rotation: global_transform.rotation(),
273 translation: global_transform.translation_vec3a(),
274 },
275 GOLD,
276 );
277 }
278}346fn draw_camera_gizmo(cameras: Query<(&Camera, &GlobalTransform)>, mut gizmos: Gizmos) {
347 for (camera, transform) in &cameras {
348 // As above, we use the order as a cheap tag to tell the depth texture
349 // apart from the main texture.
350 if camera.order >= 0 {
351 continue;
352 }
353
354 // Draw a cone representing the camera.
355 gizmos.primitive_3d(
356 &Cone {
357 radius: 1.0,
358 height: 3.0,
359 },
360 Isometry3d::new(
361 transform.translation(),
362 // We have to rotate here because `Cone` primitives are oriented
363 // along +Y and cameras point along +Z.
364 transform.rotation() * Quat::from_rotation_x(FRAC_PI_2),
365 ),
366 LIME,
367 );
368 }
369}311fn draw_gizmos(
312 mut gizmos: Gizmos,
313 decals: Query<(&GlobalTransform, &Selection), With<ClusteredDecal>>,
314) {
315 for (global_transform, selection) in &decals {
316 let color = match *selection {
317 Selection::Camera => continue,
318 Selection::DecalA => ORANGE_RED,
319 Selection::DecalB => LIME,
320 };
321
322 gizmos.primitive_3d(
323 &Cuboid {
324 // Since the clustered decal is a 1×1×1 cube in model space, its
325 // half-size is half of the scaling part of its transform.
326 half_size: global_transform.scale() * 0.5,
327 },
328 Isometry3d {
329 rotation: global_transform.rotation(),
330 translation: global_transform.translation_vec3a(),
331 },
332 color,
333 );
334 }
335}Sourcepub fn scale(&self) -> Vec3
pub fn scale(&self) -> Vec3
Get the scale as a Vec3.
The transform is expected to be non-degenerate and without shearing, or the output will be invalid.
Some of the computations overlap with to_scale_rotation_translation, which means you should use
it instead if you also need rotation.
Examples found in repository?
263fn draw_gizmos(mut gizmos: Gizmos, decals: Query<&GlobalTransform, With<ClusteredDecal>>) {
264 for global_transform in &decals {
265 gizmos.primitive_3d(
266 &Cuboid {
267 // Since the clustered decal is a 1×1×1 cube in model space, its
268 // half-size is half of the scaling part of its transform.
269 half_size: global_transform.scale() * 0.5,
270 },
271 Isometry3d {
272 rotation: global_transform.rotation(),
273 translation: global_transform.translation_vec3a(),
274 },
275 GOLD,
276 );
277 }
278}More examples
311fn draw_gizmos(
312 mut gizmos: Gizmos,
313 decals: Query<(&GlobalTransform, &Selection), With<ClusteredDecal>>,
314) {
315 for (global_transform, selection) in &decals {
316 let color = match *selection {
317 Selection::Camera => continue,
318 Selection::DecalA => ORANGE_RED,
319 Selection::DecalB => LIME,
320 };
321
322 gizmos.primitive_3d(
323 &Cuboid {
324 // Since the clustered decal is a 1×1×1 cube in model space, its
325 // half-size is half of the scaling part of its transform.
326 half_size: global_transform.scale() * 0.5,
327 },
328 Isometry3d {
329 rotation: global_transform.rotation(),
330 translation: global_transform.translation_vec3a(),
331 },
332 color,
333 );
334 }
335}Sourcepub fn radius_vec3a(&self, extents: Vec3A) -> f32
pub fn radius_vec3a(&self, extents: Vec3A) -> f32
Get an upper bound of the radius from the given extents.
Sourcepub fn transform_point(&self, point: Vec3) -> Vec3
pub fn transform_point(&self, point: Vec3) -> Vec3
Transforms the given point from local space to global space, applying shear, scale, rotation and translation.
It can be used like this:
let global_transform = GlobalTransform::from_xyz(1., 2., 3.);
let local_point = Vec3::new(1., 2., 3.);
let global_point = global_transform.transform_point(local_point);
assert_eq!(global_point, Vec3::new(2., 4., 6.));let global_point = Vec3::new(2., 4., 6.);
let global_transform = GlobalTransform::from_xyz(1., 2., 3.);
let local_point = global_transform.affine().inverse().transform_point3(global_point);
assert_eq!(local_point, Vec3::new(1., 2., 3.))To apply shear, scale, and rotation without applying translation, different functions are available:
let global_transform = GlobalTransform::from_xyz(1., 2., 3.);
let local_direction = Vec3::new(1., 2., 3.);
let global_direction = global_transform.affine().transform_vector3(local_direction);
assert_eq!(global_direction, Vec3::new(1., 2., 3.));
let roundtripped_local_direction = global_transform.affine().inverse().transform_vector3(global_direction);
assert_eq!(roundtripped_local_direction, local_direction);Examples found in repository?
528fn create_cubes(
529 image_assets: Res<Assets<Image>>,
530 mut commands: Commands,
531 irradiance_volumes: Query<(&IrradianceVolume, &GlobalTransform)>,
532 voxel_cube_parents: Query<Entity, With<VoxelCubeParent>>,
533 voxel_cubes: Query<Entity, With<VoxelCube>>,
534 example_assets: Res<ExampleAssets>,
535 mut voxel_visualization_material_assets: ResMut<Assets<VoxelVisualizationMaterial>>,
536) {
537 // If voxel cubes have already been spawned, don't do anything.
538 if !voxel_cubes.is_empty() {
539 return;
540 }
541
542 let Some(voxel_cube_parent) = voxel_cube_parents.iter().next() else {
543 return;
544 };
545
546 for (irradiance_volume, global_transform) in irradiance_volumes.iter() {
547 let Some(image) = image_assets.get(&irradiance_volume.voxels) else {
548 continue;
549 };
550
551 let resolution = image.texture_descriptor.size;
552
553 let voxel_cube_material = voxel_visualization_material_assets.add(ExtendedMaterial {
554 base: StandardMaterial::from(Color::from(RED)),
555 extension: VoxelVisualizationExtension {
556 irradiance_volume_info: VoxelVisualizationIrradianceVolumeInfo {
557 world_from_voxel: VOXEL_FROM_WORLD.inverse(),
558 voxel_from_world: VOXEL_FROM_WORLD,
559 resolution: uvec3(
560 resolution.width,
561 resolution.height,
562 resolution.depth_or_array_layers,
563 ),
564 intensity: IRRADIANCE_VOLUME_INTENSITY,
565 },
566 },
567 });
568
569 let scale = vec3(
570 1.0 / resolution.width as f32,
571 1.0 / resolution.height as f32,
572 1.0 / resolution.depth_or_array_layers as f32,
573 );
574
575 // Spawn a cube for each voxel.
576 for z in 0..resolution.depth_or_array_layers {
577 for y in 0..resolution.height {
578 for x in 0..resolution.width {
579 let uvw = (uvec3(x, y, z).as_vec3() + 0.5) * scale - 0.5;
580 let pos = global_transform.transform_point(uvw);
581 let voxel_cube = commands
582 .spawn((
583 Mesh3d(example_assets.voxel_cube.clone()),
584 MeshMaterial3d(voxel_cube_material.clone()),
585 Transform::from_scale(Vec3::splat(VOXEL_CUBE_SCALE))
586 .with_translation(pos),
587 ))
588 .insert(VoxelCube)
589 .insert(NotShadowCaster)
590 .id();
591
592 commands.entity(voxel_cube_parent).add_child(voxel_cube);
593 }
594 }
595 }
596 }
597}Sourcepub fn mul_transform(&self, transform: Transform) -> GlobalTransform
pub fn mul_transform(&self, transform: Transform) -> GlobalTransform
Multiplies self with transform component by component, returning the
resulting GlobalTransform