pub struct Outline {
pub width: Val,
pub offset: Val,
pub color: Color,
}Expand description
The Outline component adds an outline outside the edge of a UI node.
Outlines do not take up space in the layout.
To add an Outline to a ui node you can spawn a (Node, Outline) tuple bundle:
fn setup_ui(mut commands: Commands) {
commands.spawn((
Node {
width: Val::Px(100.),
height: Val::Px(100.),
..Default::default()
},
BackgroundColor(BLUE.into()),
Outline::new(Val::Px(10.), Val::ZERO, RED.into())
));
}Outline components can also be added later to existing UI nodes:
fn outline_hovered_button_system(
mut commands: Commands,
mut node_query: Query<(Entity, &Interaction, Option<&mut Outline>), Changed<Interaction>>,
) {
for (entity, interaction, mut maybe_outline) in node_query.iter_mut() {
let outline_color =
if matches!(*interaction, Interaction::Hovered) {
Color::WHITE
} else {
Color::NONE
};
if let Some(mut outline) = maybe_outline {
outline.color = outline_color;
} else {
commands.entity(entity).insert(Outline::new(Val::Px(10.), Val::ZERO, outline_color));
}
}
}Inserting and removing an Outline component repeatedly will result in table moves, so it is generally preferable to
set Outline::color to Color::NONE to hide an outline.
Fields
width: ValThe width of the outline.
Percentage Val values are resolved based on the width of the outlined Node.
offset: ValThe amount of space between a node’s outline the edge of the node.
Percentage Val values are resolved based on the width of the outlined Node.
color: ColorThe color of the outline.
If you are frequently toggling outlines for a UI node on and off it is recommended to set Color::NONE to hide the outline.
This avoids the table moves that would occur from the repeated insertion and removal of the Outline component.
Implementations
Sourceimpl Outline
impl Outline
Sourcepub const fn new(width: Val, offset: Val, color: Color) -> Outline
pub const fn new(width: Val, offset: Val, color: Color) -> Outline
Create a new outline
Examples found in repository?
18fn setup(
19 mut commands: Commands,
20 asset_server: Res<AssetServer>,
21 mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
22) {
23 // Camera
24 commands.spawn(Camera2d);
25
26 let text_font = TextFont::default();
27
28 let texture_handle = asset_server.load("textures/rpg/chars/gabe/gabe-idle-run.png");
29 let texture_atlas = TextureAtlasLayout::from_grid(UVec2::splat(24), 7, 1, None, None);
30 let texture_atlas_handle = texture_atlases.add(texture_atlas);
31
32 // root node
33 commands
34 .spawn(Node {
35 width: percent(100),
36 height: percent(100),
37 flex_direction: FlexDirection::Column,
38 justify_content: JustifyContent::Center,
39 align_items: AlignItems::Center,
40 row_gap: px(text_font.font_size * 2.),
41 ..default()
42 })
43 .with_children(|parent| {
44 parent.spawn((
45 ImageNode::from_atlas_image(
46 texture_handle,
47 TextureAtlas::from(texture_atlas_handle),
48 ),
49 Node {
50 width: px(256),
51 height: px(256),
52 ..default()
53 },
54 BackgroundColor(ANTIQUE_WHITE.into()),
55 Outline::new(px(8), Val::ZERO, CRIMSON.into()),
56 ));
57 parent
58 .spawn((Text::new("press "), text_font.clone()))
59 .with_child((
60 TextSpan::new("space"),
61 TextColor(YELLOW.into()),
62 text_font.clone(),
63 ))
64 .with_child((TextSpan::new(" to advance frames"), text_font));
65 });
66}More examples
271fn setup_node_rects(commands: &mut Commands) {
272 for (node_rect, node_type) in NODE_RECTS.iter().zip(NODE_TYPES.iter()) {
273 let node_string = match *node_type {
274 NodeType::Clip(ref clip) => clip.text,
275 NodeType::Blend(text) => text,
276 };
277
278 let text = commands
279 .spawn((
280 Text::new(node_string),
281 TextFont {
282 font_size: 16.0,
283 ..default()
284 },
285 TextColor(ANTIQUE_WHITE.into()),
286 TextLayout::new_with_justify(Justify::Center),
287 ))
288 .id();
289
290 let container = {
291 let mut container = commands.spawn((
292 Node {
293 position_type: PositionType::Absolute,
294 bottom: px(node_rect.bottom),
295 left: px(node_rect.left),
296 height: px(node_rect.height),
297 width: px(node_rect.width),
298 align_items: AlignItems::Center,
299 justify_items: JustifyItems::Center,
300 align_content: AlignContent::Center,
301 justify_content: JustifyContent::Center,
302 ..default()
303 },
304 BorderColor::all(WHITE),
305 Outline::new(px(1), Val::ZERO, Color::WHITE),
306 ));
307
308 if let NodeType::Clip(clip) = node_type {
309 container.insert((
310 Interaction::None,
311 RelativeCursorPosition::default(),
312 (*clip).clone(),
313 ));
314 }
315
316 container.id()
317 };
318
319 // Create the background color.
320 if let NodeType::Clip(_) = node_type {
321 let background = commands
322 .spawn((
323 Node {
324 position_type: PositionType::Absolute,
325 top: px(0),
326 left: px(0),
327 height: px(node_rect.height),
328 width: px(node_rect.width),
329 ..default()
330 },
331 BackgroundColor(DARK_GREEN.into()),
332 ))
333 .id();
334
335 commands.entity(container).add_child(background);
336 }
337
338 commands.entity(container).add_child(text);
339 }
340}Trait Implementations
Sourceimpl Component for Outline
impl Component for Outline
Sourceconst STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
const STORAGE_TYPE: StorageType = bevy_ecs::component::StorageType::Table
Sourcetype Mutability = Mutable
type Mutability = Mutable
Component<Mutability = Mutable>,
while immutable components will instead have Component<Mutability = Immutable>. Read moreSourcefn register_required_components(
_requiree: ComponentId,
required_components: &mut RequiredComponentsRegistrator<'_, '_>,
)
fn register_required_components( _requiree: ComponentId, required_components: &mut RequiredComponentsRegistrator<'_, '_>, )
Sourcefn clone_behavior() -> ComponentCloneBehavior
fn clone_behavior() -> ComponentCloneBehavior
Sourcefn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_add() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_insert() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_replace() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_replace() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_remove() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
fn on_despawn() -> Option<for<'w> fn(DeferredWorld<'w>, HookContext)>
Sourcefn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
fn map_entities<E>(_this: &mut Self, _mapper: &mut E)where
E: EntityMapper,
EntityMapper. This is used to remap entities in contexts like scenes and entity cloning.
When deriving Component, this is populated by annotating fields containing entities with #[entities] Read moreSourceimpl<'de> Deserialize<'de> for Outline
impl<'de> Deserialize<'de> for Outline
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<Outline, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<Outline, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl FromReflect for Outline
impl FromReflect for Outline
Sourcefn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Outline>
fn from_reflect(reflect: &(dyn PartialReflect + 'static)) -> Option<Outline>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl GetOwnership for Outline
impl GetOwnership for Outline
Sourceimpl GetTypeRegistration for Outline
impl GetTypeRegistration for Outline
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(registry: &mut TypeRegistry)
fn register_type_dependencies(registry: &mut TypeRegistry)
Sourceimpl IntoReturn for Outline
impl IntoReturn for Outline
Sourceimpl PartialReflect for Outline
impl PartialReflect for Outline
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn try_apply(
&mut self,
value: &(dyn PartialReflect + 'static),
) -> Result<(), ApplyError>
fn try_apply( &mut self, value: &(dyn PartialReflect + 'static), ) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<Outline>) -> ReflectOwned
fn reflect_owned(self: Box<Outline>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<Outline>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<Outline>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
fn try_as_reflect(&self) -> Option<&(dyn Reflect + 'static)>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
fn try_as_reflect_mut(&mut self) -> Option<&mut (dyn Reflect + 'static)>
Sourcefn into_partial_reflect(self: Box<Outline>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<Outline>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
fn as_partial_reflect(&self) -> &(dyn PartialReflect + 'static)
Sourcefn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
fn as_partial_reflect_mut(&mut self) -> &mut (dyn PartialReflect + 'static)
Sourcefn reflect_partial_eq(
&self,
value: &(dyn PartialReflect + 'static),
) -> Option<bool>
fn reflect_partial_eq( &self, value: &(dyn PartialReflect + 'static), ) -> Option<bool>
Sourcefn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
fn debug(&self, f: &mut Formatter<'_>) -> Result<(), Error>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails. Read moreSourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl Reflect for Outline
impl Reflect for Outline
Sourcefn into_any(self: Box<Outline>) -> Box<dyn Any>
fn into_any(self: Box<Outline>) -> Box<dyn Any>
Box<dyn Any>. Read moreSourcefn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
fn as_any_mut(&mut self) -> &mut (dyn Any + 'static)
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<Outline>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Outline>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &(dyn Reflect + 'static)
fn as_reflect(&self) -> &(dyn Reflect + 'static)
Sourcefn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
fn as_reflect_mut(&mut self) -> &mut (dyn Reflect + 'static)
Sourceimpl Serialize for Outline
impl Serialize for Outline
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Sourceimpl Struct for Outline
impl Struct for Outline
Sourcefn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
fn field(&self, name: &str) -> Option<&(dyn PartialReflect + 'static)>
name as a &dyn PartialReflect.Sourcefn field_mut(
&mut self,
name: &str,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_mut( &mut self, name: &str, ) -> Option<&mut (dyn PartialReflect + 'static)>
name as a
&mut dyn PartialReflect.Sourcefn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
fn field_at(&self, index: usize) -> Option<&(dyn PartialReflect + 'static)>
index as a
&dyn PartialReflect.Sourcefn field_at_mut(
&mut self,
index: usize,
) -> Option<&mut (dyn PartialReflect + 'static)>
fn field_at_mut( &mut self, index: usize, ) -> Option<&mut (dyn PartialReflect + 'static)>
index
as a &mut dyn PartialReflect.Sourcefn name_at(&self, index: usize) -> Option<&str>
fn name_at(&self, index: usize) -> Option<&str>
index.Sourcefn iter_fields(&self) -> FieldIter<'_> ⓘ
fn iter_fields(&self) -> FieldIter<'_> ⓘ
Sourcefn to_dynamic_struct(&self) -> DynamicStruct
fn to_dynamic_struct(&self) -> DynamicStruct
DynamicStruct from this struct.Sourcefn get_represented_struct_info(&self) -> Option<&'static StructInfo>
fn get_represented_struct_info(&self) -> Option<&'static StructInfo>
None if TypeInfo is not available.Sourceimpl TypePath for Outline
impl TypePath for Outline
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
impl Copy for Outline
impl StructuralPartialEq for Outline