pub struct Checkbox;bevy_ui_widgets only.Expand description
Headless widget implementation for checkboxes. The Checked component represents the current
state of the checkbox. The widget will emit a ValueChange<bool> event when clicked, or when
the Enter or Space key is pressed while the checkbox is focused.
Add the checkbox_self_update observer watching the entity with this component to automatically add and remove the Checked component.
Toggle switches
The Checkbox component can be used to implement other kinds of toggle widgets. If you
are going to do a toggle switch, you should override the AccessibilityNode component with
the Switch role instead of the Checkbox role.
Note: For information on how widget state is managed and how to respond to state changes, see the [crate-level documentation]. [crate-level documentation]: crate