Enum bevy::window::PresentMode
#[repr(C)]pub enum PresentMode {
AutoVsync,
AutoNoVsync,
Immediate,
Mailbox,
Fifo,
}Expand description
Presentation mode for a Window.
The presentation mode specifies when a frame is presented to the window. The Fifo
option corresponds to a traditional VSync, where the framerate is capped by the
display refresh rate. Both Immediate and Mailbox are low-latency and are not
capped by the refresh rate, but may not be available on all platforms. Tearing
may be observed with Immediate mode, but will not be observed with Mailbox or
Fifo.
AutoVsync or AutoNoVsync will gracefully fallback to Fifo when unavailable.
Immediate or Mailbox will panic if not supported by the platform.
Variants
AutoVsync
Chooses FifoRelaxed -> Fifo based on availability.
Because of the fallback behavior, it is supported everywhere.
AutoNoVsync
Chooses Immediate -> Mailbox -> Fifo (on web) based on availability.
Because of the fallback behavior, it is supported everywhere.
Immediate
The presentation engine does not wait for a vertical blanking period and the request is presented immediately. This is a low-latency presentation mode, but visible tearing may be observed. Not optimal for mobile.
Selecting this variant will panic if not supported, it is preferred to use
PresentMode::AutoNoVsync.
Mailbox
The presentation engine waits for the next vertical blanking period to update the current image, but frames may be submitted without delay. This is a low-latency presentation mode and visible tearing will not be observed. Not optimal for mobile.
Selecting this variant will panic if not supported, it is preferred to use
PresentMode::AutoNoVsync.
Fifo
The presentation engine waits for the next vertical blanking period to update
the current image. The framerate will be capped at the display refresh rate,
corresponding to the VSync. Tearing cannot be observed. Optimal for mobile.