pub enum WorldFilter {
Unset,
Allowlist(HashSet<TypeId>),
Denylist(HashSet<TypeId>),
}bevy_world_serialization only.Expand description
A filter used to control which types can be added to a DynamicWorld.
This world filter can be used more generically to represent a filter for any given type;
however, note that its intended usage with DynamicWorld only considers components and resources.
Adding types that are not a component or resource will have no effect when used with DynamicWorld.
Variants
Unset
Represents an unset filter.
This is the equivalent of an empty Denylist or an Allowlist containing every type—
essentially, all types are permissible.
Allowing a type will convert this filter to an Allowlist.
Similarly, denying a type will convert this filter to a Denylist.
Allowlist(HashSet<TypeId>)
Contains the set of permitted types by their TypeId.
Types not contained within this set should not be allowed to be saved to an associated DynamicWorld.
Denylist(HashSet<TypeId>)
Contains the set of prohibited types by their TypeId.
Types contained within this set should not be allowed to be saved to an associated DynamicWorld.
Implementations
Sourceimpl WorldFilter
impl WorldFilter
Sourcepub fn allow_all() -> WorldFilter
pub fn allow_all() -> WorldFilter
Creates a filter where all types are allowed.
This is the equivalent of creating an empty Denylist.
Sourcepub fn deny_all() -> WorldFilter
pub fn deny_all() -> WorldFilter
Creates a filter where all types are denied.
This is the equivalent of creating an empty Allowlist.
Sourcepub fn allow<T>(self) -> WorldFilterwhere
T: Any,
pub fn allow<T>(self) -> WorldFilterwhere
T: Any,
Sourcepub fn allow_by_id(self, type_id: TypeId) -> WorldFilter
pub fn allow_by_id(self, type_id: TypeId) -> WorldFilter
Sourcepub fn deny<T>(self) -> WorldFilterwhere
T: Any,
pub fn deny<T>(self) -> WorldFilterwhere
T: Any,
Sourcepub fn deny_by_id(self, type_id: TypeId) -> WorldFilter
pub fn deny_by_id(self, type_id: TypeId) -> WorldFilter
Sourcepub fn is_allowed<T>(&self) -> boolwhere
T: Any,
pub fn is_allowed<T>(&self) -> boolwhere
T: Any,
Returns true if the given type, T, is allowed by the filter.
If the filter is Unset, this will always return true.
Sourcepub fn is_allowed_by_id(&self, type_id: TypeId) -> bool
pub fn is_allowed_by_id(&self, type_id: TypeId) -> bool
Returns true if the given type is allowed by the filter.
If the filter is Unset, this will always return true.
Sourcepub fn is_denied<T>(&self) -> boolwhere
T: Any,
pub fn is_denied<T>(&self) -> boolwhere
T: Any,
Returns true if the given type, T, is denied by the filter.
If the filter is Unset, this will always return false.
Sourcepub fn is_denied_by_id(&self, type_id: TypeId) -> bool
pub fn is_denied_by_id(&self, type_id: TypeId) -> bool
Returns true if the given type is denied by the filter.
If the filter is Unset, this will always return false.
Sourcepub fn iter(&self) -> Box<dyn ExactSizeIterator<Item = &TypeId> + '_>
pub fn iter(&self) -> Box<dyn ExactSizeIterator<Item = &TypeId> + '_>
Returns an iterator over the items in the filter.
If the filter is Unset, this will return an empty iterator.